/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.IO;
using System.Text;
using System.Windows.Forms;
using gov.va.med.imaging.exchange.VixInstaller.business;

namespace gov.va.med.imaging.exchange.VixInstaller.ui
{
    public partial class NewInstallWelcomePage : WizardPage
    {
        public NewInstallWelcomePage()
        {
            InitializeComponent();
        }

        public NewInstallWelcomePage(IWizardForm wizForm, int pageIndex)
            : base(wizForm, pageIndex)
        {
            InitializeComponent();
        }

        #region IWizardPage Members
        public override bool IsComplete()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override void Initialize()
        {
            IVixConfigurationParameters config = this.WizardForm.GetVixConfigurationParameters();
            this.InitializeBusinessFacadeDelegates();

            if (config.VixRole == VixRoleType.DicomGateway)
            {
                this.pictureBox1.Image = global::VixInstaller.Properties.Resources.HDIG;
                this.textBoxWelcomeHeader.Text = "Welcome to the HDIG Service Installation Wizard";
            }
            else
            {
                this.pictureBox1.Image = global::VixInstaller.Properties.Resources.VixInstaller;
                this.textBoxWelcomeHeader.Text = "Welcome to the VIX Service Installation Wizard";
            }

            this.WizardForm.EnableBackButton(false);
            this.WizardForm.EnableCancelButton(true);

            string vixconfig = Environment.GetEnvironmentVariable("vixconfig", EnvironmentVariableTarget.Machine);
            if (vixconfig != null)
            {
                vixconfig = vixconfig.Replace("/", @"\");
            }

            string vixcache = Environment.GetEnvironmentVariable("vixcache", EnvironmentVariableTarget.Machine);
            if (vixcache != null)
            {
                vixcache = vixcache.Replace("/", @"\");
            }

            if (config.IsNewVixInstallation() == true && config.VixDeploymentOption == VixDeploymentType.FocClusterNode && vixconfig != null && Directory.Exists(vixconfig) == false)
            {
                this.WizardForm.EnableNextButton(false);
                this.textBoxWelcomeText.Text = "Error: cannot access the VIX configuration directory " + vixconfig + ".\r\n\r\n" + config.VixServerNameProp + " does not own the resource group that contains the shared drive where the VIX configuration folder resides.\r\n\r\nPress cancel to exit.";
            }
            else if (config.IsNewVixInstallation() == true && config.VixDeploymentOption == VixDeploymentType.FocClusterNode && vixcache != null && Directory.Exists(vixcache) == false)
            {
                this.WizardForm.EnableNextButton(false);
                this.textBoxWelcomeText.Text = "Error: cannot access the VIX cache directory " + vixcache + ".\r\n\r\n" + config.VixServerNameProp + " does not own the resource group that contains the shared drive where the VIX cache folder resides.\r\n\r\nPress cancel to exit.";
            }
            else
            {
                this.WizardForm.EnableNextButton(true);
                this.textBoxWelcomeText.Text = VixFacade.Manifest.WelcomeMessage;
            }

        }

        #endregion

    }
}
