/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using gov.va.med.imaging.exchange.VixInstaller.business;
using System.IO;

namespace gov.va.med.imaging.exchange.VixInstaller.ui
{
    public partial class SitesFileDialog : Form
    {
        public String SitesFile
        {
            get { return this.textBoxSitesFile.Text; }
        }

        public SitesFileDialog()
        {
            InitializeComponent();
        }

        private void ClearErrors()
        {
            this.labelErrorInfo.Visible = false;
            this.errorProvider.Clear();
        }

        private void buttonSitesFile_Click(object sender, EventArgs e)
        {
            DialogResult result = this.openSitesFileDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                this.textBoxSitesFile.Text = this.openSitesFileDialog.FileName;
                this.ClearErrors();
            }
        }

        private void SitesFileDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.ClearErrors();

            if (this.DialogResult == DialogResult.OK) // OK button pressed
            {
                if (this.textBoxSitesFile.Text.Length == 0)
                {
                    this.errorProvider.SetError(this.textBoxSitesFile, "You must select the sites file.");
                    this.labelErrorInfo.Visible = true;
                    e.Cancel = true;
                }
                else
                {
                    string vixConfigDir = VixFacade.GetVixConfigurationDirectory();
                    if (vixConfigDir != null)
                    {
                        string sourceFilespec = this.textBoxSitesFile.Text;
                        string targetFilespec = Path.Combine(vixConfigDir, "VhaSites.xml");
                        if (sourceFilespec.ToUpper() == targetFilespec.ToUpper())
                        {
                            this.errorProvider.SetError(this.textBoxSitesFile, "Source filespec cannot equal target filespec.");
                            this.labelErrorInfo.Visible = true;
                            e.Cancel = true;
                        }
                    }
                }
            }
        }
    }
}
