/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using log4net;
using gov.va.med.imaging.exchange.VixInstaller.business;

namespace gov.va.med.imaging.exchange.VixInstaller.ui
{
    public partial class WizardPage : UserControl, IWizardPage
    {
        protected ILog Logger()
        {
            return LogManager.GetLogger(this.GetType().Name);
        }

        protected virtual String Info(String infoMessage)
        {
            return infoMessage;
        }

        /// <summary>
        /// Wrap Application.DoEvents for use as a delegate.
        /// </summary>
        private void AppEvents()
        {
            Application.DoEvents();
        }

        private IWizardForm wizForm;
        private int pageIndex = 0;

        protected WizardPage() // used by the Visual Studio designer
        {
            InitializeComponent();
        }

        protected WizardPage(IWizardForm wizForm, int pageIndex) // user by the VixInstaller
        {
            InitializeComponent();
            this.wizForm = wizForm;
            this.pageIndex = pageIndex;
        }

        /// <summary>
        /// Initializes the delegates that allow the static business facade classes to provide feedback to the
        /// current Wizard Page.
        /// </summary>
        protected void InitializeBusinessFacadeDelegates()
        {
            JavaFacade.InfoDelegate = this.Info; // note Info method is virtual
            VixFacade.InfoDelegate = this.Info;
            VixFacade.AppEventsDelegate = this.AppEvents; // private to this base class
            LaurelBridgeFacade.InfoDelegate = this.Info;
            LaurelBridgeFacade.AppEventsDelegate = this.AppEvents;
            BusinessFacade.InfoDelegate = this.Info;
        }

        #region IWizardPage Members
        public virtual bool IsComplete() { return false; }
        public virtual int GetNextPageIndex() { return this.pageIndex + 1;} // most pages will use
        public virtual void Initialize()
        {
            this.InitializeBusinessFacadeDelegates();
        }
        public void RegisterDevModeChangeHandler()
        {
            this.WizardForm.OnDevModeChange += this.OnDevModeChange;
        }
        public void UnregisterDevModeChangeHandler()
        {
            this.WizardForm.OnDevModeChange -= this.OnDevModeChange;
        }
        #endregion

        #region Form Events
        protected virtual void OnDevModeChange(Object sender, EventArgs e)
        {
            this.Initialize();
        }
        #endregion

        #region properties 
        public int PageIndex
        {
            get { return this.pageIndex; }
        }

        protected IWizardForm WizardForm
        {
            get { return this.wizForm; }
        }

        #endregion
    }
}
