/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Win32;
using System.Diagnostics;


namespace gov.va.med.imaging.exchange.VixInstaller.business
{
    public static class ClusterFacade
    {
        private static readonly string CLUSTER_SERVICE_NAME = "ClusSvc";

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public static bool IsServerClusterNode()
        {
            bool isCluster = false;
            if (BusinessFacade.IsWindowsServer2003() || BusinessFacade.IsWindowsServer2008())
            {
                string key = @"Cluster";
                using (RegistryKey cluster = Registry.LocalMachine.OpenSubKey(key))
                {
                    if (cluster != null)
                    {
                        isCluster = true;
                    }
                }
            }

            return isCluster;
        }


        public static bool IsServerClusterServiceRunning()
        {
            bool isRunning = false;
            System.ServiceProcess.ServiceControllerStatus status = ServiceUtilities.GetLocalServiceStatus(ClusterFacade.CLUSTER_SERVICE_NAME);
            if (status == System.ServiceProcess.ServiceControllerStatus.Running)
            {
                isRunning = true;
            }
            return isRunning;
        }


        /// <summary>
        /// Per Ron Paulin, we dont need any special handling to filter out the QUORUM logical drive. Keeping the code around just in case.
        /// </summary>
        /// <param name="drive">The drive to test.</param>
        /// <returns>true if the passed drive is the QUORUM drive, false otherwise</returns>
        public static bool IsClusterQuorumDrive(DriveInfo drive)
        {
            bool isClusterQuorumDrive = false;
            string key = @"Cluster\Quorum";
            using (RegistryKey quorum = Registry.LocalMachine.OpenSubKey(key, false))
            {
                if (quorum != null)
                {
                    string path = quorum.GetValue("Path").ToString();
                    Debug.Assert(path != null);
                    string pathRoot = Path.GetPathRoot(path).ToUpper();
                    if (drive.Name.ToUpper() == pathRoot)
                    {
                        isClusterQuorumDrive = true;
                    }
                }
            }

            return isClusterQuorumDrive;
        }

    }
}
