/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace gov.va.med.imaging.exchange.VixInstaller.business
{
    public class ProductVersion : IComparable 
    {
        private Int32 productVersionAsInt = 0;
        private string productVersionAsString;

        public ProductVersion(String productVersion)
        {
            this.productVersionAsString = productVersion;
            string[] splits = productVersion.Split('.');
            Debug.Assert(splits.Length == 4);
            foreach (String split in splits)
            {
                this.productVersionAsInt += Int32.Parse(split);
            }
        }

        public override string ToString()
        {
            return this.productVersionAsString;
        }

        public int CompareTo(object obj)
        {
            if (!(obj is ProductVersion))
            {
                throw new ArgumentException(
                   "A ProductVersion object is required for comparison.");
            }
            return this.productVersionAsInt.CompareTo(((ProductVersion)obj).productVersionAsInt);
        }

        // Omitting Equals violates rule: OverrideMethodsOnComparableTypes.
        public override bool Equals(Object obj)
        {
            if (!(obj is ProductVersion))
                return false;
            return (this.CompareTo(obj) == 0);
        }

        // Omitting getHashCode violates rule: OverrideGetHashCodeOnOverridingEquals.
        public override int GetHashCode()
        {
            return this.productVersionAsInt;
        }

        // Omitting any of the following operator overloads 
        // violates rule: OverrideMethodsOnComparableTypes.
        public static bool operator ==(ProductVersion v1, ProductVersion v2)
        {
            return v1.Equals(v2);
        }
        public static bool operator !=(ProductVersion v1, ProductVersion v2)
        {
            return !(v1 == v2);
        }
        public static bool operator <(ProductVersion v1, ProductVersion v2)
        {
            return (v1.CompareTo(v2) < 0);
        }
        public static bool operator >(ProductVersion v1, ProductVersion v2)
        {
            return (v1.CompareTo(v2) > 0);
        }

        // other operators
        public static bool operator >=(ProductVersion v1, ProductVersion v2)
        {
            return (v1.CompareTo(v2) >= 0);
        }
        public static bool operator <=(ProductVersion v1, ProductVersion v2)
        {
            return (v1.CompareTo(v2) <= 0);
        }  


    }
}
