/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Feb 22, 2012
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  ZZZZZZWERFEJ
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.awiv.web;

import gov.va.med.imaging.awiv.client.ui.widgets.AwivConstants;
import gov.va.med.imaging.awiv.configuration.AwivClientConfiguration;
import gov.va.med.imaging.transactioncontext.TransactionContextFactory;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

/**
 * This filter is used for URLs that do not require any authentication.  The only thing this filter does is check if there is a transaction context and if so clear it.
 *  This filter must be used by any service that calls a command because commands leave stuff in transaction context and that must be cleared 
 *  before the operation is complete.
 * 
 * @author ZZZZZZWERFEJ
 *
 */
public class UnauthenticatedFilter
implements Filter
{

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#destroy()
	 */
	@Override
	public void destroy()
	{
		
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	@Override
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) 
	throws IOException, ServletException
	{
		try
		{
			setSslPort(response);
			chain.doFilter(request, response);
		}
		finally
		{
			// if a context was created, then clear it
			if(TransactionContextFactory.isTransactionContextEstablished())
				TransactionContextFactory.get().clear();
		}
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	@Override
	public void init(FilterConfig filterConfig) 
	throws ServletException
	{
			
	}
	
	/**
	 * Set the ssl port in a cookie to use for possible redirects if they are needed
	 * @param response
	 */
	private void setSslPort(ServletResponse response)
	{
		if(response instanceof HttpServletResponse)
		{
			AwivClientConfiguration config = AwivClientConfiguration.getConfiguration();
			HttpServletResponse resp = (HttpServletResponse)response;
			Cookie sslCookie = new Cookie(AwivConstants.sslPortCookieName, config.getSslPort() + "");
			sslCookie.setMaxAge(-1);
			sslCookie.setPath("/Awiv");
			resp.addCookie(sslCookie);
		}
	}
}
