/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.exchange.business.taglib;

import gov.va.med.imaging.exchange.business.taglib.exceptions.MissingRequiredArgumentException;
import java.io.IOException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTag;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.IterationTag;
import org.apache.log4j.Logger;

/**
 * @author zzzzzzbeckec
 *
 */
public abstract class AbstractUrlCollectionTag
extends BodyTagSupport
{
	private static final long serialVersionUID = 1L;
	private Logger logger = Logger.getLogger(this.getClass());
	private String emptyResultMessage = null;
	
	private Collection<URL> urls;
	private Iterator<URL> urlIterator;
	private URL currentUrl;
	
	/**
	 * Either this method XOR getUrlIterator() must be overridden by derived
	 * classes.  By default, this method returns null.  If this method is overridden
	 * then getUrlIterator() will call this method to get the URL collection.  If
	 * getUrlIterator() is overridden then this method will not be called.
	 * 
	 * @return
	 * @throws JspException
	 * @throws MissingRequiredArgumentException
	 */
	protected Collection<URL> getUrls()
	throws JspException
	{
		return null;
	}

	/**
	 * Either this method XOR getUrls() must be overridden by derived classes.
	 * 
	 * @return
	 * @throws JspException
	 */
	protected Iterator<URL> getUrlIterator()
	throws JspException
	{
		urls = getUrls();
		return urls == null ? null : urls.iterator();
	}
	
	/**
	 * The message to show if the site list is empty
	 * @return
	 */
	public String getEmptyResultMessage()
    {
    	return emptyResultMessage;
    }

	public void setEmptyResultMessage(String emptyResultMessage)
    {
    	this.emptyResultMessage = emptyResultMessage;
    }
	
	// ==============================================================================
	// JSP Tag Lifecycle Events
	// ==============================================================================

	/**
	 * Create and expose the current RequestContext. Delegates to
	 * {@link #doStartTagInternal()} for actual work.
	 * 
	 * @see #REQUEST_CONTEXT_PAGE_ATTRIBUTE
	 * @see org.springframework.web.servlet.support.JspAwareRequestContext
	 */
	public final int doStartTag() 
	throws JspException
	{
		urlIterator = getUrlIterator();
		if(urlIterator.hasNext())
		{
			currentUrl = urlIterator.next();
			return BodyTag.EVAL_BODY_INCLUDE;
		}
		else
		{
	    	if(getEmptyResultMessage() != null)
	    		try{pageContext.getOut().write(getEmptyResultMessage());}
	    		catch(IOException ioX){logger.error("Unable to write empty result set message.");}
	    		
			currentUrl = null;
			return BodyTag.SKIP_BODY;
		}
	}

	/**
	 * 
	 * @return
	 */
	public URL getCurrentUrl()
	{
		return currentUrl;
	}
	
	@Override
    public int doAfterBody() 
	throws JspException
    {
		if(urlIterator.hasNext())
		{
			currentUrl = urlIterator.next();
			return IterationTag.EVAL_BODY_AGAIN;
		}
		else
		{
			currentUrl = null;
			return IterationTag.SKIP_BODY;
		}
    }

}
