/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.business.taglib.artifactsource;

import gov.va.med.RoutingTokenImpl;
import gov.va.med.exceptions.RoutingTokenFormatException;
import gov.va.med.imaging.BaseWebFacadeRouter;
import gov.va.med.imaging.artifactsource.ArtifactSource;
import gov.va.med.imaging.artifactsource.ResolvedArtifactSource;
import gov.va.med.imaging.core.FacadeRouterUtility;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.Site;
import gov.va.med.imaging.exchange.business.taglib.exceptions.MissingRequiredArgumentException;

import java.util.Collection;

import javax.servlet.jsp.JspException;

import org.apache.log4j.Logger;

/**
 * 
 * @author ZZZZZZBECKEC
 *
 */
public class TreatingFacilitySiteCollectionTag 
extends AbstractArtifactSourceCollectionTag
{
	private static final long serialVersionUID = 1L;

	private Logger logger = Logger.getLogger(this.getClass());
	private String patientIcn;
	private String siteNumber;

	public String getPatientIcn()
    {
    	return patientIcn;
    }

	public void setPatientIcn(String patientIcn)
    {
    	this.patientIcn = patientIcn;
    }

	public String getSiteNumber()
    {
    	return siteNumber;
    }

	public void setSiteNumber(String siteNumber)
    {
    	this.siteNumber = siteNumber;
    }


	@Override
	protected Collection<ResolvedArtifactSource> getArtifactSources() 
	throws JspException, MissingRequiredArgumentException
	{
		if(getPatientIcn() == null || getPatientIcn().length() < 1)
			throw new MissingRequiredArgumentException("The patient ICN is required but was not provided.");
		
		if(getSiteNumber() == null || getSiteNumber().length() < 1)
			throw new MissingRequiredArgumentException("The site number is required but was not provided.");
		
    	//Router vixCore = ApplicationContextUtil.getRouter(this.pageContext);
    	
    	try 
    	{
        	BaseWebFacadeRouter router;
    		try
    		{
    			router = FacadeRouterUtility.getFacadeRouter(BaseWebFacadeRouter.class);
    		} 
    		catch (Exception x)
    		{
    			logger.error("Exception getting the facade router implementation.", x);
    			throw new JspException(x);
    		}
    		
    		return router.getTreatingSites( 
    			RoutingTokenImpl.createVARadiologySite(getSiteNumber()),
    			getPatientIcn()
    		);
    	}
    	catch(MethodException mX)
    	{
    		logger.error(mX);
    		throw new JspException(mX);
    	}
    	catch(ConnectionException mX)
    	{
    		logger.error(mX);
    		throw new JspException(mX);
    	}
		catch (RoutingTokenFormatException rtfX)
		{
    		logger.error(rtfX);
    		throw new JspException(rtfX);
		}
    }

	
}
