/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/*
 * Originally HttpParameterList.java 
 * created on Nov 22, 2004 @ 5:51:14 PM
 * by Chris Beckey mailto:c.beckey@seetab.com
 *
 */
package gov.va.med.imaging.http;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Iterator;

/**
 * @author Chris Beckey mailto:c.beckey@seetab.com
 * @since Nov 22, 2004 5:51:14 PM
 *
 * A typesfe LinkedList derivation which is guaranteed to contain
 * only instances of HttpParameter
 */
public class HttpParameterList 
extends LinkedList
{
	/**
	 * 
	 */
	public HttpParameterList()
	{
		super();
	}

	/**
	 * @param c
	 */
	public HttpParameterList(Collection c)
	{
		super(c);
	}

	/**
	 * Add an instance of HttpParameter, or throw a ClassCastException
	 * 
	 * @see java.util.List#add(int, java.lang.Object)
	 */
	public void add(int index, Object element)
	{
		super.add(index, (HttpParameter)element);
	}

	/**
	 * Add an instance of HttpParameter, or throw a ClassCastException
	 * 
	 * @see java.util.Collection#add(java.lang.Object)
	 */
	public boolean add(Object o)
	{
		return super.add((HttpParameter)o);
	}

	public boolean collectionIsAddable(Collection c)
	{
		for(Iterator iter = c.iterator(); iter.hasNext();)
			if( ! (iter.next() instanceof HttpParameter) )
				return false;
				
		return true;
	}

	/**
	 * @see java.util.Collection#addAll(java.util.Collection)
	 */
	public boolean addAll(Collection c)
	{
		if(collectionIsAddable(c))
			return super.addAll(c);
		return false;
	}

	/* (non-Javadoc)
	 * @see java.util.List#addAll(int, java.util.Collection)
	 */
	public boolean addAll(int index, Collection c)
	{
		if(collectionIsAddable(c))
			return super.addAll(index, c);
		return false;
	}

	/* (non-Javadoc)
	 * @see java.util.LinkedList#addFirst(java.lang.Object)
	 */
	public void addFirst(Object o)
	{
		super.addFirst((HttpParameter)o);
	}

	/* (non-Javadoc)
	 * @see java.util.LinkedList#addLast(java.lang.Object)
	 */
	public void addLast(Object o)
	{
		super.addLast((HttpParameter)o);
	}
	
	/**
	 * 
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer(64);
			
		for(Iterator iter = iterator();
			iter.hasNext(); )
		{
			if(sb.length() > 0)
				sb.append(";");
			sb.append( ((HttpParameter)iter.next()).toString() );
		}
			
		return sb.toString();
	}
}
