/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.wado;

public enum ImageContentType
{
	dicom( "application", "dicom" ), 
	jpeg( "image", "jpeg" ), 
	jp2( "image", "jp2" ),
	text("text", "plain"),
	anything("*", "*");
	
	
	public static ImageContentType getContentType(String mimetype)
	{
		String[] mimeParts = null;

		if( mimetype != null)
			mimeParts = mimetype.split("/");
		else
			return null;
		
		return getContentType(mimeParts[0], mimeParts[1]);
	}
	
	public static ImageContentType getContentType(String type, String subtype)
	{
		for(ImageContentType ct:ImageContentType.values())
			if(ct.isMatch(type, subtype))
				return ct;
		
		return null;
	}
	
	private String type = null;
	private String subtype = null;
	ImageContentType(String type, String subtype)
	{
		this.type = type;
		this.subtype = subtype;
	}
	
	boolean isMatch(String type, String subtype)
	{
		return this.type.equalsIgnoreCase(type) && this.subtype.equalsIgnoreCase(subtype);
	}
	
	public String toString()
	{
		return this.type + "/" + this.subtype;
	}
	
	public String getType()
	{
		return this.type;
	}
	
	public String getSubType()
	{
	return this.subtype;
	}
}
