/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache;

import gov.va.med.imaging.storage.cache.exceptions.CacheStateException;

/**
 * The interface to be implemented by classes that are interested in lifecycle events
 * of the cache (start and stop). The Cache and Region interface derive from this interface,
 * hence Region and Cache implementations must understand lifecycle events.
 * Cache implementations should forward lifecycle messages to its constituent Regions.
 * 
 * @author ZZZZZZBECKEC
 *
 * @see Region
 * @see Cache
 */
public interface CacheLifecycleListener
{
	/**
	 * Notify the cache instance that it is starting or stopping.
	 * Depending on the cache implementation, it may be critical that it be
	 * notified of start and stop events.
	 * 
	 * @param event either START or STOP
	 * @throws CacheStateException if the listener is not in a state that it may be 
	 * started ot stopped.
	 */
	public void cacheLifecycleEvent(CacheLifecycleEvent event)
	throws CacheStateException;
}
