/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache;

import gov.va.med.imaging.storage.cache.memento.EvictionTimerMemento;


/**
 * 
 * @author ZZZZZZBECKEC
 *
 */
public interface EvictionTimer
{
	/**
	 * Schedule eviction sweeps for age based eviction strategies.  The maximum age 
	 * parameter is used to calculate an interval for sweeps.  The interval MUST be 
	 * less than the maximum age but is otherwise up to the realization. 
	 * 
	 * @param sweepTask
	 * @param maximumAge
	 */
	public void scheduleSweep(EvictionTimerTask sweepTask, long maximumAge);
	
	/**
	 * Schedule an eviction sweep at the specified interval.  This type of
	 * scheduling is intended for eviction strategies that are not age-based
	 * but simply need to run periodically.  This is a simple pass-through to
	 * the Timer method of the same name.
	 */
	public void scheduleSweep(EvictionTimerTask sweepTask, long delay, long interval);

	/**
	 * 
	 */
	public void cancel();
	
	/**
	 * 
	 * @return
	 */
	public abstract EvictionTimerMemento createMemento();
}
