/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache;

import java.util.Iterator;
import java.util.regex.Pattern;

import gov.va.med.imaging.storage.cache.events.GroupLifecycleListener;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;

/**
 * 
 * i.e. A Group may contain other Group instances, which contain other Group instances,
 * which eventually contain Instance.	
 * 
 */
public interface Group
extends MutableNamedObject, GroupAndInstanceAncestor, GroupSet, InstanceSet
{
	// A group name must be 1 to 64 chars, start with a letter and contain letters, numbers, dashes and underscores
	public static Pattern NamePattern = Pattern.compile( "[a-zA-Z][a-zA-Z0-9-_]{0,63}" );
	
	/**
	 * Return some human readable identifier for this group.
	 * This must be unique within the parent group, not necessarily across all groups.
	 * @return
	 */
	@Override
	public String getName();
	
	public java.util.Date getLastAccessed() 
	throws CacheException;

	public long getSize()
	throws CacheException;

	// ===========================================================================
	// Methods that recursively walk down the Region/Group/Instance graph are 
	// defined in GroupAndInstanceAncestor
	// ===========================================================================
	
	// ===========================================================================
	// Methods operating on the child Group of this Group are defined in GroupSet
	// ===========================================================================
	
	// =====================================================================================
	// Methods operating on child Instance are defined in InstanceSet
	// =====================================================================================
	
	// =====================================================================================
	// Eviction Methods
	// =====================================================================================
	public int evaluateAndEvictChildGroups(EvictionJudge<Group> judge)
	throws CacheException;
	
	// ======================================================================================================
	// Listener Management
	// ======================================================================================================
	public abstract void registerListener(GroupLifecycleListener listener);
	public abstract void unregisterListener(GroupLifecycleListener listener);
	
}
