/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache;

import gov.va.med.imaging.storage.cache.exceptions.CacheException;

/**
 * @author ZZZZZZBECKEC
 * 
 * A collection of methods on Group and Instance instances that are
 * generally passed through recursively until the parent of the
 * Group or Instance actually does the operation. 
 *
 * e.g. Region and Group instances pass calls to getInstance() until
 * the owner of the Instance is found, which then calls getChildInstance().
 */
public interface GroupAndInstanceAncestor 
{
	// ============================================================================
	// Group Related Methods
	// ============================================================================
	public Group getOrCreateGroup(String[] group)
	throws CacheException;

	public Group getGroup(String[] group) 
	throws CacheException;

	public void deleteGroup(String[] group, boolean forceDelete) 
	throws CacheException;

	// ============================================================================
	// Instance Related Methods
	// ============================================================================
	public Instance getOrCreateInstance(String[] group, String key)
	throws CacheException;

	public Instance getInstance(String[] group, String key) 
	throws CacheException;

	public void deleteInstance(String[] group, String key, boolean forceDelete) 
	throws CacheException;

}
