/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache;

import gov.va.med.imaging.storage.cache.exceptions.CacheException;

import java.util.Iterator;

/**
 * Implemented by a MutableNamedObject that is a Set of Group (Group and Region).
 * @see Group
 * @see Region
 * @author ZZZZZZBECKEC
 *
 * NOTE: this interface defines methods that generally follow the Set methods though the method
 * naming is different.  This is because InstanceSet and GroupSet may both be implemented by a 
 * single class and the naming would get confused otherwise.
 * 
 * NOTE: the word "delete" is used to mean specifically remove from a Collection.  The word
 * "remove" is used to mean specifically remove a persistent copy of an Instance or Group.
 */
public interface GroupSet 
{
	public Iterator<? extends Group> getGroups() 
	throws CacheException;

	public Group getOrCreateChildGroup(String group)
	throws CacheException;

	public Group getChildGroup(String group) 
	throws CacheException;
	
	public void deleteChildGroup(Group childGroup, boolean forceDelete)
	throws CacheException;

	public void deleteAllChildGroups(boolean forceDelete)
	throws CacheException;

	// ============================================================================
	// Eviction Related Methods
	// ============================================================================
	/**
	 * @param judge
	 * @return
	 * @throws CacheException
	 */
	public int evaluateAndEvictChildGroups(EvictionJudge<Group> judge)
	throws CacheException;
}
