/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache;

import gov.va.med.imaging.storage.cache.exceptions.CacheException;
import gov.va.med.imaging.storage.cache.exceptions.PersistenceIOException;
import gov.va.med.imaging.storage.cache.memento.ByteChannelFactoryMemento;

/**
 * The interface for the byte channel factory.  The factory is responsible for creating
 * readable and writable byte channels and then monitoring their usage.  If the channels are
 * not used for a defined period then the factory must notify the registered listener.
 *  
 * @author ZZZZZZBECKEC
 *
 */
public interface InstanceByteChannelFactory<T>
{
	// ========================================================================================================================================
	// Business Methods
	// ========================================================================================================================================
	
	/**
	 * 
	 * @param instanceIdentifier
	 * @param timeoutListener
	 * @return
	 * @throws PersistenceIOException
	 * @throws CacheException
	 */
	public abstract InstanceReadableByteChannel getInstanceReadableByteChannel(T instanceIdentifier, InstanceByteChannelListener timeoutListener) 
	throws PersistenceIOException, CacheException;

	/**
	 * 
	 * @param instanceIdentifier
	 * @param timeoutListener
	 * @return
	 * @throws PersistenceIOException
	 * @throws CacheException
	 */
	public abstract InstanceWritableByteChannel getInstanceWritableByteChannel(T instanceIdentifier, InstanceByteChannelListener timeoutListener) 
	throws PersistenceIOException, CacheException;

	// ========================================================================================================================================
	// Behavioral Modification Methods
	// ========================================================================================================================================
	public abstract long getMaxChannelOpenDuration();
	public abstract void setMaxChannelOpenDuration(long max);
	
	public abstract long getSweepTime();
	public abstract void setSweepTime(long sweep);
	
	/**
	 * If traceChannelInstantiation is set then the factory will record
	 * the stack trace when a channel is instantiated and report the stack
	 * trace when the channel is closed due to a timeout. 
	 * 
	 * @return
	 */
	public abstract boolean isTraceChannelInstantiation();
	public void setTraceChannelInstantiation(boolean traceChannelInstantiation);
	
	// ========================================================================================================================================
	// Statistics Gathering Methods
	// ========================================================================================================================================
	public abstract int getCurrentlyOpenReadableByteChannels();
	public abstract int getCurrentlyOpenWritableByteChannels();

	// ========================================================================================================================================
	// State persistence and restoration methods
	// ========================================================================================================================================
	public abstract ByteChannelFactoryMemento createMemento();
	public abstract void restoreMemento(ByteChannelFactoryMemento memento);
	
}
