/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache;


public interface InstanceByteChannelListener
{
	/**
	 * Notification that a writable byte channel has been closed.  This is 
	 * the 'happy' path.  The listeners may allow read channels to open
	 * on the source (file).
	 * Note that this event will NOT be sent to the listeners after a timeout signal
	 * has been sent.  The responsibility for closing the channel relies on the
	 * listener(s).
	 * 
	 * @param writable
	 */
	public abstract void writeChannelClose(InstanceWritableByteChannel writable);
	
	/**
	 * Notification that a readable byte channel has been closed.  This is 
	 * the 'happy' path.
	 * Note that this event will NOT be sent to the listeners after a timeout signal
	 * has been sent.  The responsibility for closing the channel relies on the
	 * listener(s).
	 * 
	 * @param readable
	 */
	public abstract void readChannelClose(InstanceReadableByteChannel readable);

	/**
	 * Notification that a writable byte channel has been open without activity for longer
	 * than the allowable time.  This is a one time notification, the channel is closed
	 * when this message is received.  This message will be received AFTER a writeChannelClose
	 * message on the same channel has been received.
	 * 
	 * @param writable
	 */
	public abstract void writeChannelIdleTimeout(InstanceWritableByteChannel writable);
	
	/**
	 * Notification that a readable byte channel has been open without activity for longer
	 * than the allowable time.  This is a one time notification, the channel is closed
	 * when this message is received.  This message will be received AFTER a readChannelClose
	 * message on the same channel has been received.
	 * 
	 * @param readable
	 */
	public abstract void readChannelIdleTimeout(InstanceReadableByteChannel readable);

}
