/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.exceptions;

import gov.va.med.imaging.storage.cache.memento.CacheMemento;

/**
 * @author ZZZZZZBECKEC
 *
 * An exception thrown during cache initialization, these exceptions should be considered fatal, the
 * cache will not be in an operable state after throwing this exception.
 * The CacheInitializationException indicates a failure in initializing the cache itself, regions and 
 * eviction strategies should throw RegionInitializationException and EvictionStrtegyInitializationException
 * respectively, each of which are derivations of CacheException.
 */
public class CacheInitializationException 
extends InitializationException
{
	private static final long serialVersionUID = 8881701735937618207L;

	/**
	 * 
	 */
	public CacheInitializationException()
	{
	}

	/**
	 * @param message
	 * @param cause
	 */
	public CacheInitializationException(String message, Throwable cause)
	{
		super(message, cause);
	}

	/**
	 * @param message
	 */
	public CacheInitializationException(String message)
	{
		super(message);
	}

	/**
	 * @param cause
	 */
	public CacheInitializationException(Throwable cause)
	{
		super(cause);
	}

	/**
	 * Throw an exception using this constructor when the given memento type is not one that the 
	 * specific derivation expects.
	 * 
	 * @param actualMementoClass
	 * @param expectedMementoClass
	 */
	public CacheInitializationException(Class<? extends CacheMemento> actualMementoClass, Class<? extends CacheMemento> expectedMementoClass)
	{
		this("Unknown memento class '" + actualMementoClass.getName() + "', expected memento class '" + expectedMementoClass.getName() + "'.");
	}
}
