/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.exceptions;

import gov.va.med.imaging.storage.cache.memento.EvictionStrategyMemento;

/**
 * @author ZZZZZZBECKEC
 *
 * An exception thrown during cache initialization.  If this exception occurs during
 * the startup sequence then it should be considered fatal, the cache will not be in an 
 * operable state after throwing this exception.  If this exception occurs when creating or
 * adding new eviction strategies using the management interface then the eviction strategy
 * is not usable, will not be persisted, but the cache should keep running.
 * 
 * Any method that takes a memento to initialize an object may throw an InitializationException
 * if the memento is the wrong type.
 */
public class EvictionStrategyInitializationException 
extends CacheException
{
	private static final long serialVersionUID = -4735250274450620302L;

	/**
	 * 
	 */
	public EvictionStrategyInitializationException()
	{
	}

	/**
	 * @param message
	 * @param cause
	 */
	public EvictionStrategyInitializationException(String message, Throwable cause)
	{
		super(message, cause);
	}

	/**
	 * @param message
	 */
	public EvictionStrategyInitializationException(String message)
	{
		super(message);
	}

	/**
	 * @param cause
	 */
	public EvictionStrategyInitializationException(Throwable cause)
	{
		super(cause);
	}

	/**
	 * Throw an exception using this constructor when the given memento type is not one that the 
	 * specific derivation expects.
	 * 
	 * @param actualMementoClass
	 * @param expectedMementoClass
	 */
	public EvictionStrategyInitializationException(Class<? extends EvictionStrategyMemento> actualMementoClass, Class<? extends EvictionStrategyMemento> expectedMementoClass)
	{
		this("Unknown memento class '" + actualMementoClass.getName() + "', expected memento class '" + expectedMementoClass.getName() + "'.");
	}
}
