/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache.exceptions;

/**
 * An instance exists in the cache but the persistent image is inaccessible.  The operation that resulted in this exception 
 * may be retried, though the existence of the persistent instance image may not be relied on.
 * This error will only occur in really odd circumstances, like when the eviction thread has deleted
 * an Instance after another thread has a reference to it or when a writing thread calls error() and removes the
 * persistent image.
 * 
 * @see gov.va.med.imaging.storage.cache.exceptions.InstanceInaccessibleException 
 * 
 * @author ZZZZZZBECKEC
 *
 */
public class InstanceUnavailableException 
extends TransientCacheException
{
	private static final long serialVersionUID = 3239977796584018148L;

	public final static String defaultRetryMessage =
		"The operation that resulted in this exception may be retried, though the existence of the persistent instance image may not be relied on.";
	
	public final static String defaultCauseMessage =
		"This error should only occur in really odd circumstances, like when the eviction thread has deleted \n" +
		"an Instance after another thread has a reference to it or when a writing thread calls error() and removes the \n" +
		"persistent image.";

	public final static String defaultMessage = 
		"An instance exists in the cache but the persistent image is unavailable.\n" + defaultRetryMessage + "\n" + defaultCauseMessage;
	
	public final static String defaultFileDoesNotExistMessage = 
		"An instance exists in the cache but the persistent image (file) does not exist.\n" + defaultRetryMessage + "\n" + defaultCauseMessage;
	
	public final static String defaultInstanceInvalidMessage = 
		"An instance exists in the cache but it has been marked invalid, probably due to an error while writing.\n" + defaultRetryMessage + "\n" + defaultCauseMessage;
	
	public InstanceUnavailableException()
	{
		super(defaultMessage);
	}

	public InstanceUnavailableException(String message, Throwable cause)
	{
		super(message, cause);
	}

	public InstanceUnavailableException(String instanceIdentifier)
	{
		super("Instance '" + instanceIdentifier + "'-" + defaultMessage);
	}

	public InstanceUnavailableException(Throwable cause)
	{
		super(defaultMessage, cause);
	}

	/**
	 * Using this constructor allows the creator to indicate if the
	 * image exists in persistent storage or not.
	 * 
	 * @param persistentImageExists 
	 * - if true then the cause is an instance marked invalid
	 * - if false then the cause is an image file does not exist
	 */
	public InstanceUnavailableException(boolean persistentImageExists)
	{
		super(persistentImageExists ? defaultInstanceInvalidMessage : defaultFileDoesNotExistMessage);
	}
	
	public InstanceUnavailableException(String instanceIdentifier, boolean persistentImageExists)
	{
		super( "Instance '" + instanceIdentifier + "'-" + 
			(persistentImageExists ? defaultInstanceInvalidMessage : defaultFileDoesNotExistMessage) 
		);
	}
}
