/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.impl;

import java.util.List;

import gov.va.med.imaging.storage.cache.Group;
import gov.va.med.imaging.storage.cache.Region;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;

/**
 * @author ZZZZZZBECKEC
 * A simple value object that contains references to a Group
 * and all of its ancestors.
 * The path field DOES NOT include the Group to which the GroupPath applies.
 * The last entry in the path is the immediate ancestor (the parent) of the target Group.
 */
public class GroupPath
{
	private Region region;
	private List<Group> path;
	private Group group;
	
	public GroupPath(Region region, List<Group> path, Group group)
	{
		super();
		this.region = region;
		this.path = path;
		this.group = group;
	}

	/**
	 * @return the group
	 */
	public Group getGroup()
	{
		return this.group;
	}

	/**
	 * @return the path
	 */
	public List<Group> getPath()
	{
		return this.path;
	}
	
	/**
	 * Return the names of the Group instances in the path as a String array, which
	 * is the way that the Cache usually deals with paths.
	 * @return
	 */
	public String[] getPathName()
	{
		String[] pathName = new String[path.size()];
		
		int index=0;
		for(Group group : path)
			pathName[index++] = group.getName();
		
		return pathName;
	}

	/**
	 * @return the region
	 */
	public Region getRegion()
	{
		return this.region;
	}
	
	/**
	 * Determines if the path from Region, through the path to the Group is valid.
	 * this method uses the Group.getName() to determine existence of the Group instances.
	 * 
	 * @return
	 */
	public boolean validate()
	{
		try
		{
			Group parent = getRegion().getGroup(getPathName());
			parent.getChildGroup(group.getName());
		} 
		catch (CacheException x)
		{
			return false;
		}
		
		return true;
	}
}
