/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.impl;

import java.util.logging.Logger;

import sun.security.action.GetLongAction;

import gov.va.med.imaging.storage.cache.Group;
import gov.va.med.imaging.storage.cache.GroupAndInstanceAncestor;
import gov.va.med.imaging.storage.cache.GroupSet;
import gov.va.med.imaging.storage.cache.Instance;
import gov.va.med.imaging.storage.cache.InstanceSet;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;

/**
 * @author ZZZZZZBECKEC
 *
 * This class adds recursive group name handling for instances as well as groups.
 * Any class that can be a parent of a set of Group and Instance instances should derive from this
 * class.
*/
public abstract class RecursiveGroupAndInstanceParent 
extends RecursiveGroupParent
implements GroupAndInstanceAncestor, GroupSet, InstanceSet
{
	/**
	 * 
	 */
	public RecursiveGroupAndInstanceParent()
	{
	}

	// ========================================================================================================
	// Abstract methods bind this class to a persistence mechanism
	// ========================================================================================================
	
	/**
	 * Get or create an Instance.  The groupName is an ordered array if the
	 * instances ancestor groups, starting from the progeny of this
	 * group.
	 */
	@Override
	public Instance getOrCreateInstance(String[] groupName, String key) 
	throws CacheException
	{
		return getOrCreateInstance(groupName, key, true); 
	}

	/**
	 * Get an Instance.  The groupName is an ordered array if the
	 * instances ancestor groups, starting from the progeny of this
	 * group.
	 */
	@Override
	public Instance getInstance(String[] groupName, String key) 
	throws CacheException
	{
		return getOrCreateInstance(groupName, key, false); 
	}
	
	/**
	 * 
	 * @param groupName
	 * @param key
	 * @param allowCreate
	 * @return
	 * @throws CacheException
	 */
	@Override
	public Instance getOrCreateInstance(String[] groupName, String key, boolean allowCreate) 
	throws CacheException
	{
		if(groupName == null || groupName.length == 0)
			return allowCreate ?
				getOrCreateChildInstance(key):
				getChildInstance(key);

		return super.getOrCreateInstance(groupName, key, allowCreate);
	}
	
	/**
	 * Delete an Instance.  The groupName is an ordered array of the
	 * instances ancestor groups, starting from the progeny of this
	 * group.
	 * NOTE: this is a recursive function.  The group name specifies a path
	 * through an object graph of Group instances.  This method follows that path,
	 * removing the group name that it currently resides on. 
	 */
	public void deleteInstance(String[] groupName, String key, boolean forceDelete) 
	throws CacheException
	{
		if(groupName == null || groupName.length == 0)
		{
			deleteChildInstance(key, forceDelete);
			return;
		}
		
		super.deleteInstance(groupName, key, forceDelete);
	}
}
