/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache.impl.eviction;

import gov.va.med.imaging.storage.cache.EvictionTimerTask;

/**
 * An EvictionTimerTask realization that just calls to the given eviction strategy at scheduled intervals.
 */
class SweepTask
extends EvictionTimerTask
{
	private long lastRunDate;

	/**
	 * 
	 */
	private PeriodicSweepEvictionStrategy periodicSweepStrategy; 
	SweepTask(PeriodicSweepEvictionStrategy periodicSweepStrategy)
	{
		this.periodicSweepStrategy = periodicSweepStrategy;
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.imaging.storage.cache.EvictionTimerTask#getEvictionStrategyName()
	 */
	@Override
	public String getEvictionStrategyName()
	{
		return getEvictionStrategy().getName();
	}

	public PeriodicSweepEvictionStrategy getEvictionStrategy()
	{
		return this.periodicSweepStrategy;
	}
	
	public long getLastRunDate()
	{
		return this.lastRunDate;
	}
	
	protected void setLastRunDate(long lastRunDate)
	{
		this.lastRunDate = lastRunDate;
	}
	
	/* (non-Javadoc)
	 * @see java.util.TimerTask#run()
	 */
	@Override
	public void run()
	{
		setLastRunDate(System.currentTimeMillis());
		getEvictionStrategy().sweep();
	}

	@Override
	public boolean cancel()
	{
		getEvictionStrategy().cancelSweeps();
		return super.cancel();
	}

}
