/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.impl;

import gov.va.med.imaging.storage.cache.EvictionJudge;
import gov.va.med.imaging.storage.cache.Group;
import gov.va.med.imaging.storage.cache.InstanceByteChannelFactory;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;

import java.lang.ref.SoftReference;
import java.util.ConcurrentModificationException;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

/**
 * @author ZZZZZZBECKEC
 * 
 * An abstract class that represents a Set of Group instances in a cache implementation
 * that persistent stores cache data.  Both Group and Region implementations have
 * sets of Group instances.  This class makes the management of those instances easier.
 *
 * Known Derivations:
 * @see gov.va.med.imaging.storage.cache.impl.filesystem.FileSystemGroupSet
 */
public abstract class PersistentGroupSet
extends PersistentSet<Group>
{
	private static final long serialVersionUID = 1L;
	private Logger log = Logger.getLogger(this.getClass());

	// ============================================================================================================================================
	// Constructors
	// ============================================================================================================================================
	protected PersistentGroupSet(
		InstanceByteChannelFactory byteChannelFactory,
		int secondsReadWaitsForWriteCompletion,
		boolean setModificationTimeOnRead)
	{
		super(byteChannelFactory, secondsReadWaitsForWriteCompletion, setModificationTimeOnRead);
	}
	
	// =============================================================================================================================
	// The eviction group as determined by the given eviction judge
	// =============================================================================================================================
	/**
	 * Return the Set of Group instances that are evictable according to the
	 * given EvictionJudge.
	 * NOTE: this is not a recursive method.  The descendant groups, that may be
	 * evictable, are not included in this list.
	 * 
	 * @param judge
	 * @return
	 */
	public Set<? extends Group> internalEvictableGroups(EvictionJudge<Group> judge)
	{
		Set<Group> evictableGroups = new HashSet<Group>();
		
		for(SoftReference<? extends Group> childGroupRef : this)
		{
			Group childGroup = childGroupRef.get();
			
			// the child group may no longer be referenced
			if(childGroup != null)
			{
				try
				{
					if( judge.isEvictable(childGroup) )
					{
						log.info("Queueing group '" + childGroup.getName() + " for eviction.");
						evictableGroups.add(childGroup);
					}
				} 
				catch (CacheException e)
				{
					log.error("CacheException evaluating eviction criteria for group '" + childGroup.toString() + "', which may have to be manually deleted.", e);
				}
			}
		}
		
		return evictableGroups;
	}
}
