/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.impl;

import gov.va.med.imaging.storage.cache.Instance;
import gov.va.med.imaging.storage.cache.InstanceByteChannelFactory;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;

import java.lang.ref.SoftReference;
import java.util.Set;

import org.apache.log4j.Logger;

/**
 * @author ZZZZZZBECKEC
 *
 * An abstract class that represents a Set of Instance instances in a cache implementation
 * that persistent stores cache data.  Group implementations have
 * sets of Instance instances.  
 * This class makes the management of those instances easier.
 *
 * Known Derivations:
 * @see gov.va.med.imaging.storage.cache.impl.filesystem.FileSystemInstanceSet
 * 
 * Derivations must implement the following methods:
 * 
 * protected abstract Instance getOrCreate(String name, boolean create) 
 * throws CacheException;
 * 
 * protected abstract void synchronizeChildren()
 * throws CacheException;
 */
public abstract class PersistentInstanceSet 
extends PersistentSet<Instance>
{
	private Logger log = Logger.getLogger(this.getClass());
	
	protected PersistentInstanceSet(
		InstanceByteChannelFactory byteChannelFactory, 
		int secondsReadWaitsForWriteCompletion, 
		boolean setModificationTimeOnRead)
	{
		super(byteChannelFactory, secondsReadWaitsForWriteCompletion, setModificationTimeOnRead);
	}

	protected Logger getLogger(){return this.log;}
	
	// =========================================================================================================
	// Abstract overrides of methods in the parent templated abstract class, PersistentSet.
	// Having these just makes the derived classes a bit more clear in their derivation.
	// These methods are not strictly necessary as they just override the templated methods in
	// the superclass.
	// =========================================================================================================
	
	@Override
	protected abstract Instance getOrCreate(String name, boolean create) 
	throws CacheException;
	
	@Override
	protected abstract void internalSynchronizeChildren()
	throws CacheException;
}
