/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.impl;

import gov.va.med.imaging.storage.cache.exceptions.CacheException;
import gov.va.med.imaging.storage.cache.exceptions.UnknownEvictionStrategyException;

/**
 * The MBean interface definition.  In general, since the PersistentRegion is an abstract class,
 * this interface is not direclty referenced but is derived from for Region implementations that derive from 
 * PersistentRegion.
 * 
 * @author ZZZZZZBECKEC
 *
 */
public interface PersistentRegionMBean
{
	/**
	 * Get the name of the Region.
	 * 
	 * @return
	 */
	public abstract String getName();

	/**
	 * Getting and setting the eviction strategy is done by name rather than by 
	 * reference to be consistent with setting parameters from the memento.
	 */
	public abstract String[] getEvictionStrategyNames();

	public abstract void setEvictionStrategyNames(String[] evictionStrategyNames) throws UnknownEvictionStrategyException;

	/**
	 * 
	 * @return
	 */
	public abstract int getSecondsReadWaitsForWriteCompletion();

	public abstract void setSecondsReadWaitsForWriteCompletion(int secondsReadWaitsForWriteCompletion);

	/**
	 * 
	 * @return
	 */
	public abstract boolean isSetModificationTimeOnRead();

	public abstract void setSetModificationTimeOnRead(boolean setModificationTimeOnRead);

	/**
	 * The default handling of the initialized flag simply sets and return the
	 * Boolean value.  Derived classes that need to do initialization should
	 * override these methods.
	 * Note that the initialized flag is a latch, once it becomes true it stays true.
	 */
	public abstract Boolean isInitialized();

	public abstract void setInitialized(Boolean initialized) throws CacheException;

}
