/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.impl;

import java.util.List;

import gov.va.med.imaging.storage.cache.Group;
import gov.va.med.imaging.storage.cache.Region;
import gov.va.med.imaging.storage.cache.util.GroupVisitor;

/**
 * A class that simply visits every leaf node in a cache region and
 * sums the sizes.
 * For normal, synchronous use, call the static createAndRun() method.
 * To run asynch, creat an instance of the class manually and run it on
 * it own thread.  Get the collectiveSize when it is complete.
 * 
 * @author zzzzzzbeckec
 *
 */
public class RegionSizeCalculator 
extends GroupVisitor
{
	public static long createAndRun(
			Region region)
	{
		RegionSizeCalculator calculator = new RegionSizeCalculator(region);
		calculator.run();
		
		return calculator.getCollectiveSize();
	}
	
	private long collectiveSize = 0L;
	
	/**
	 * @param region
	 * @param includeBranchNodes
	 * @param includeLeafNodes
	 */
	public RegionSizeCalculator(Region region)
	{
		super(region, false, true);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.storage.cache.util.GroupVisitor#groupVisit(gov.va.med.imaging.storage.cache.Region, java.util.List, gov.va.med.imaging.storage.cache.Group)
	 */
	@Override
	public void groupVisit(Region region, List<Group> path, Group group)
	throws Exception
	{
		collectiveSize += group.getSize();
	}

	/**
	 * 
	 * @return
	 */
	public long getCollectiveSize()
	{
		return this.collectiveSize;
	}
}
