/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.storage.cache.impl.config;

import gov.va.med.imaging.storage.cache.impl.eviction.LastAccessedEvictionStrategyMemento;
import gov.va.med.imaging.storage.cache.impl.memento.PersistentRegionMemento;
import gov.va.med.imaging.storage.cache.memento.CacheConfigurationMemento;
import gov.va.med.imaging.storage.cache.memento.EvictionStrategyMemento;
import gov.va.med.imaging.storage.cache.memento.RegionMemento;

import java.beans.XMLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * @author ZZZZZZBECKEC
 *
 */
public class TestWithEvictionConfigurationStrategy 
{
	
	/* (non-Javadoc)
	 * @see gov.va.med.imaging.storage.cache.CacheConfigurationStrategy#getEvictionStrategyMementoes()
	 */
	private List<? extends EvictionStrategyMemento> getEvictionStrategyMementoes()
	{
		List<EvictionStrategyMemento> evictionStrategies = new ArrayList<EvictionStrategyMemento>();
		
		LastAccessedEvictionStrategyMemento memento = null;
		
		memento = new LastAccessedEvictionStrategyMemento();
		memento.setName("ten-second-lifespan");
		memento.setMaximumTimeSinceLastAccess(10L * 1000L);
		memento.setInitialized(true);
		evictionStrategies.add(memento);

		memento = new LastAccessedEvictionStrategyMemento();
		memento.setName("one-minute-lifespan");
		memento.setMaximumTimeSinceLastAccess(60L * 1000L);
		memento.setInitialized(true);
		evictionStrategies.add(memento);
		
		return evictionStrategies;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.storage.cache.CacheConfigurationStrategy#getRegionMementoes()
	 */
	private List<? extends RegionMemento> getRegionMementoes()
	{
		List<RegionMemento> regions = new ArrayList<RegionMemento>();
		
		RegionMemento memento = null;
		
		memento = new PersistentRegionMemento();
		memento.setName("test-image");
		memento.setEvictionStrategyNames(new String[]{"one-minute-lifespan"});
		regions.add(memento);
		
		memento = new PersistentRegionMemento();
		memento.setName("test-metadata");
		memento.setEvictionStrategyNames(new String[]{"ten-second-lifespan"});
		regions.add(memento);
		
		return regions;
	}

	public CacheConfigurationMemento getMemento()
	{
		CacheConfigurationMemento memento = new CacheConfigurationMemento(getEvictionStrategyMementoes(), getRegionMementoes());
		
		return memento;
	}
	
	public static void main(String[] argv)
	{
		TestWithEvictionConfigurationStrategy strategy = new TestWithEvictionConfigurationStrategy();
		
		CacheConfigurationMemento memento = strategy.getMemento();
		
		XMLEncoder encoder = new XMLEncoder(System.out);
	
		encoder.writeObject(memento);
		
		encoder.flush();
		encoder.close();
	}
}
