/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.storage.cache.interactive;


import java.util.concurrent.ArrayBlockingQueue;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;
import gov.va.med.imaging.storage.cache.impl.CacheManagerImpl;
import gov.va.med.interactive.Command;
import gov.va.med.interactive.CommandFactory;
import gov.va.med.interactive.CommandController;
import gov.va.med.interactive.CommandProcessor;
import javax.management.MBeanException;

/**
 * This is a class that will start a cache instance and allow it to be managed
 * through JMX.  It takes the place of CacheFactory (in the production, application 
 * server resident) code.  It also allows for interactive or command-line driven 
 * management through the CacheManagerImpl interface.
 * 
 * This class creates a command queue and then starts two threads:
 * 1.) A command line processor that first process the command line args and then
 * goes into an interactive mode to handle user commands.  This thread pushes commands
 * onto the queue.
 * 2.) A command processor that polls commands from the queue and processes them. 
 * 
 * @author ZZZZZZBECKEC
 *
 */
public class InteractiveCacheContainer
extends CommandController<CacheManagerImpl>
{
	/**
	 * 
	 * @param managedObject
	 * @param commandFactory
	 */
	public InteractiveCacheContainer(
		CacheManagerImpl managedObject, 
		CommandFactory<CacheManagerImpl> commandFactory)
	{
		super(managedObject, commandFactory);
	}
	
	
	/**
	 * This main() method will start a cache instance and wait for user input.
	 * It is intended for development time to test managability.
	 * 
	 * @param argv
	 */
	public static void main(String[] argv)
	{
		CommandController.initializeLogging();
		
		CacheManagerImpl cacheManager;
		try
		{
			cacheManager = CacheManagerImpl.getSingleton();
		}
		catch (MBeanException x1)
		{
			x1.printStackTrace();
			return;
		}
		catch (CacheException x1)
		{
			x1.printStackTrace();
			return;
		}
		try
		{
			InteractiveCacheCommandFactoryImpl commandFactory = InteractiveCacheCommandFactoryImpl.getSingleton();
			InteractiveCacheContainer icc = new InteractiveCacheContainer(cacheManager, commandFactory);
			
			icc.getCommandSource().pushCommands(argv);
			icc.run();
		}
		catch (Exception x)
		{
			x.printStackTrace();
			return;
		}
	}
	

}
