/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core;

import gov.va.med.imaging.exchange.business.ImageFormatQuality;
import gov.va.med.imaging.exchange.enums.ImageFormat;
import gov.va.med.imaging.exchange.enums.ImageQuality;

/**
 * On object to represent the result from receiving an image from the cache.
 * Contains the result message string and the number of bytes returned
 * 
 * @author ZZZZZZWERFEJ
 * 
 */
public class StreamImageFromCacheResponse
{
	private final String result;
	private final int bytesReturnedFromDataSource;
	private final ImageFormat imageFormat;
	private final ImageQuality imageQuality;

	/**
	 * @param result
	 * @param bytesReturnedFromDataSource
	 * @param imageFormat
	 * @param imageQuality
	 */
	public StreamImageFromCacheResponse(String result,
			int bytesReturnedFromDataSource, ImageFormat imageFormat,
			ImageQuality imageQuality)
	{
		super();
		this.result = result;
		this.bytesReturnedFromDataSource = bytesReturnedFromDataSource;
		this.imageFormat = imageFormat;
		this.imageQuality = imageQuality;
	}

	public StreamImageFromCacheResponse(String result,
			int bytesReturnedFromDataSource,
			ImageFormatQuality imageFormatQuality)
	{
		super();
		this.result = result;
		this.bytesReturnedFromDataSource = bytesReturnedFromDataSource;
		this.imageFormat = imageFormatQuality.getImageFormat();
		this.imageQuality = imageFormatQuality.getImageQuality();
	}

	/**
	 * @return the result
	 */
	public String getResult()
	{
		return this.result;
	}

	/**
	 * @return the bytesReturnedFromDataSource
	 */
	public int getBytesReturnedFromDataSource()
	{
		return this.bytesReturnedFromDataSource;
	}

	/**
	 * @return the imageFormat
	 */
	public ImageFormat getImageFormat()
	{
		return this.imageFormat;
	}

	/**
	 * @return the imageQuality
	 */
	public ImageQuality getImageQuality()
	{
		return this.imageQuality;
	}
}
