/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core.router;

import java.util.Date;

/**
 * The base abstract class for any command processors that may be retried asynchronously.
 * As long as the getRemainingRetries() returns a positive number a command that has
 * failed can request that it be put back on the queue for another attempt.
 * 
 * @author ZZZZZZBECKEC
 *
 * @param <R> - the result type of the asynchronous process
 */
public abstract class AbstractRetryableCommandImpl<R> 
extends AbstractCommandImpl<R>
{
	// 
	private static final long serialVersionUID = -7429202447266747607L;
	private final int maximumRetries;
	private final long retryDelay;
	private int remainingRetries;
	
	public AbstractRetryableCommandImpl(
        int maximumRetries,
        long retryDelay)
    {
	    super();
	    this.maximumRetries = maximumRetries;
	    this.remainingRetries = maximumRetries;
	    this.retryDelay = retryDelay;
    }

	public void updateAccessibilityDateForRetry()
	{
		this.accessibilityDate = new Date(System.currentTimeMillis() + getRetryDelay());
	}
	
	public int getRemainingRetries()
	{
		return this.remainingRetries;
	}
	
	public int getMaximumRetries()
    {
    	return maximumRetries;
    }

	/**
	 * Get the delay before the next execution attempt on a retry.
	 * @return
	 */
	public long getRetryDelay()
    {
    	return retryDelay;
    }

	@Override
    protected void commandProcessingComplete()
    {
		--this.remainingRetries;
    }
}
