/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Mar 18, 2011
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  zzzzzzwerfej
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core.router.commands;

import java.util.List;

import gov.va.med.RoutingToken;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.core.router.AbstractDataSourceCommandImpl;
import gov.va.med.imaging.datasource.UserDataSourceSpi;
import gov.va.med.imaging.exchange.business.Division;

/**
 * @author zzzzzzwerfej
 *
 */
public class GetDivisionListCommandImpl
extends AbstractDataSourceCommandImpl<List<Division>, UserDataSourceSpi>
{
	private static final long serialVersionUID = 927797832639285649L;

	private final String accessCode;
	private final RoutingToken routingToken;
	
	private static final String SPI_METHOD_NAME = "getDivisionList";
	
	public GetDivisionListCommandImpl(String accessCode, RoutingToken routingToken)
	{
		this.accessCode = accessCode;
		this.routingToken = routingToken;
	}

	@Override
	protected List<Division> getCommandResult(UserDataSourceSpi spi)
	throws ConnectionException, MethodException
	{
		return spi.getDivisionList(getAccessCode(), getRoutingToken());
	}

	@Override
	public RoutingToken getRoutingToken()
	{
		return routingToken;
	}

	@Override
	protected String getSiteNumber()
	{
		return getRoutingToken().getRepositoryUniqueId();
	}

	@Override
	protected Class<UserDataSourceSpi> getSpiClass()
	{
		return UserDataSourceSpi.class;
	}

	@Override
	protected String getSpiMethodName()
	{
		return SPI_METHOD_NAME;
	}

	@Override
	protected Object[] getSpiMethodParameters()
	{
		return new Object[]{getAccessCode(), getRoutingToken()};
	}

	@Override
	protected Class<?>[] getSpiMethodParameterTypes()
	{
		return new Class<?>[]{String.class, RoutingToken.class};
	}

	public String getAccessCode() {
		return accessCode;
	}
}
