/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core.router.commands;

import gov.va.med.RoutingToken;
import gov.va.med.imaging.BeanUtils;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.core.router.AbstractDataSourceCommandImpl;
import gov.va.med.imaging.datasource.VeinsDataSourceSpi;
import gov.va.med.imaging.veins.ErrorTypeNotificationConfiguration;

public class PostErrorTypeNotificationConfigurationCommandImpl extends
		AbstractDataSourceCommandImpl<Boolean, VeinsDataSourceSpi> {
	private static final long serialVersionUID = 4759272349502375615L;
	private static final String SPI_METHOD_NAME = "updateErrorTypeNotificationConfiguration";
	private final ErrorTypeNotificationConfiguration config;
	private final RoutingToken routingToken;

	public PostErrorTypeNotificationConfigurationCommandImpl(RoutingToken routingToken,
			ErrorTypeNotificationConfiguration config) {
		super();
		this.routingToken = routingToken;
		this.config = config;
	}

	@Override
	public boolean equals(Object obj) {
		return BeanUtils.equals(this, obj);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.va.med.imaging.core.router.AbstractDataSourceCommandImpl#getCommandResult(gov.va.med.imaging.datasource.VersionableDataSourceSpi)
	 */
	@Override
	protected Boolean getCommandResult(VeinsDataSourceSpi spi)
			throws ConnectionException, MethodException {
		spi.updateErrorTypeNotificationConfiguration(getConfig());
		return true;
	}

	public ErrorTypeNotificationConfiguration getConfig() {
		return config;
	}

	public RoutingToken getRoutingToken() {
		return routingToken;
	}

	/**
	 * @return the siteNumber
	 */
	public String getSiteNumber() {
		return getRoutingToken().getRepositoryUniqueId();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.va.med.imaging.core.router.AbstractDataSourceCommandImpl#getSpiClass()
	 */
	@Override
	protected Class<VeinsDataSourceSpi> getSpiClass() {
		return VeinsDataSourceSpi.class;
	}

	@Override
	protected String getSpiMethodName() {
		return SPI_METHOD_NAME;
	}

	@Override
	protected Object[] getSpiMethodParameters() {
		return new Object[] { getConfig() };
	}

	@Override
	protected Class<?>[] getSpiMethodParameterTypes() {
		return new Class<?>[] { ErrorTypeNotificationConfiguration.class };
	}

	@Override
	public int hashCode() {
		return BeanUtils.hashCode(this);
	}

	@Override
	protected String parameterToString() {
		return getConfig().toString();
	}

	@Override
	public String toString() {
		return BeanUtils.toString(this);
	}
}
