/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core.router.commands;

import gov.va.med.RoutingToken;
import gov.va.med.imaging.core.annotations.routerfacade.RouterCommandExecution;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.core.router.AbstractCommandImpl;
import gov.va.med.imaging.core.router.facade.InternalContext;
import gov.va.med.imaging.core.router.facade.InternalRouter;
import gov.va.med.imaging.exchange.business.DurableQueueMessage;

import java.util.List;

import org.apache.log4j.Logger;

@RouterCommandExecution(asynchronous = true, distributable = false)
public class PostMoveAllDurableQueueMessagesCommandImpl extends
		AbstractCommandImpl<Boolean> {

	private static final long serialVersionUID = 9472325650382L;

	private Logger logger = Logger
			.getLogger(PostMoveAllDurableQueueMessagesCommandImpl.class);
	private final int sourceQueueId;
	private final int targetQueueId;
	private final String messageGroupId;
	private final RoutingToken routingToken;

	public PostMoveAllDurableQueueMessagesCommandImpl(
			RoutingToken routingToken, int sourceQueueId, int targetQueueId) {
		super();
		this.routingToken = routingToken;
		this.sourceQueueId = sourceQueueId;
		this.targetQueueId = targetQueueId;
		this.messageGroupId = null;
	}

	public PostMoveAllDurableQueueMessagesCommandImpl(
			RoutingToken routingToken, int sourceQueueId, int targetQueueId,
			String messageGroupId) {
		super();
		this.routingToken = routingToken;
		this.sourceQueueId = sourceQueueId;
		this.targetQueueId = targetQueueId;
		this.messageGroupId = messageGroupId;
	}

	@Override
	public Boolean callSynchronouslyInTransactionContext()
			throws MethodException, ConnectionException {
		InternalRouter internalRouter = InternalContext.getRouter();
		List<DurableQueueMessage> messages = internalRouter
				.getDurableQueueMessages(routingToken, sourceQueueId,
						messageGroupId, 0, 0);
		for (DurableQueueMessage message : messages) {
			internalRouter.postMoveDurableQueueMessage(routingToken, message
					.getId(), targetQueueId);
		}
		return true;
	}

	@Override
	public boolean equals(Object obj) {
		return false;
	}

	public RoutingToken getRoutingToken() {
		return routingToken;
	}

	@Override
	protected String parameterToString() {
		return Integer.toString(sourceQueueId) + ", "
				+ Integer.toString(targetQueueId);
	}
}
