/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core.router.commands.worklist;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.core.interfaces.exceptions.SecurityCredentialsExpiredException;
import gov.va.med.imaging.datasource.WorkListDataSourceSpi;
import gov.va.med.imaging.exchange.business.WorkItem;

public class PutWorkItemCommandImpl 
extends AbstractWorkListDataSourceCommandImpl<Boolean>
{

	private static final long serialVersionUID = 1L;

	private static final String SPI_METHOD_NAME = "updateWorkItem";
	
	private final int workItemId;
	private final String expectedStatus; 
	private final String newStatus; 
	private final String newMessage; 
	private final String updatingUser; 
	private final String updatingApplication; 
	
	@Override
	protected Class<?>[] getSpiMethodParameterTypes() {
		return new Class<?>[]{int.class, String.class, String.class, String.class, String.class, String.class};
	}

	@Override
	protected Object[] getSpiMethodParameters() {
		return new Object[]{getWorkItemId(), getExpectedStatus(), getNewStatus(), getNewMessage(), getUpdatingApplication(), getUpdatingApplication()} ;
	}
	
	public PutWorkItemCommandImpl(int workItemId, String expectedStatus,String newStatus, String newMessage, String updatingUser, String updatingApplication) {
		this.workItemId = workItemId;
		this.expectedStatus = expectedStatus;
		this.newStatus = newStatus;
		this.newMessage = newMessage;
		this.updatingUser = updatingUser;
		this.updatingApplication = updatingApplication;
	}

	@Override
	protected String parameterToString()
	{
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.core.router.AbstractDataSourceCommandImpl#getCommandResult(gov.va.med.imaging.datasource.VersionableDataSourceSpi)
	 */
	@Override
	protected Boolean getCommandResult(WorkListDataSourceSpi spi) 
	throws ConnectionException, MethodException, SecurityCredentialsExpiredException 
	{
		return spi.updateWorkItem(getWorkItemId(), getExpectedStatus(), getNewStatus(), getNewMessage(), getUpdatingUser(), getUpdatingApplication());
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.core.router.AbstractDataSourceCommandImpl#getSpiMethodName()
	 */
	@Override
	protected String getSpiMethodName() {
		return SPI_METHOD_NAME;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	public int getWorkItemId() {
		return workItemId;
	}

	public String getExpectedStatus() {
		return expectedStatus;
	}
	
	public String getNewStatus() {
		return newStatus;
	}

	public String getNewMessage() {
		return newMessage;
	}

	public String getUpdatingUser() {
		return updatingUser;
	}

	public String getUpdatingApplication() {
		return updatingApplication;
	}

}
