/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.core.annotations.routerfacade;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Annotation that indicates to the router whether a command can be executed asynchronously
 * and whether it may be safely executed on a remote node (in a cluster).
 * By default, commands will not be executed asynchronously and will always be
 * executed on the node that created them.  Commands that may be executed asynchronously
 * and may be executed on another node must be explicitly marked so. 
 * 
 * @author zzzzzzbeckec
 *
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface RouterCommandExecution
{
	/**
	 * Set to true if this command may be executed asynchronously.  The command should not
	 * depend on its listeners getting called to free system resources (i.e. no streams).
	 *
	 * @return
	 */
	public boolean asynchronous() default false;

	/**
	 * Set to true if this command is safe to run on any node in the cluster.
	 * Set to false and this command must execute on the node on which it was created.
	 * 
	 * @return
	 */
	public boolean distributable() default false;
}
