/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core;

import java.util.regex.Pattern;

/**
 * ENUM of all permissible action verbs, with synonyms.
 */
public enum Action
{
	GET(new String[]{ "[gG]et", "[rR]ead" }), 
	POST(new String[]{ "[pP]ost", "[cC]reate", "[sS]tore" }), 
	PUT(new String[]{ "[pP]ut", "[uU]pdate" }), 
	DELETE(new String[]{ "[dD]elete" }),
	HEAD(new String[]{ "[hH]ead" }),
	TRACE(new String[]{ "[tT]race" }),
	OPTIONS(new String[]{ "[oO]ptions" }),
	PREFETCH(new String[]{ "[pP]refetch" }),
	PROCESS(new String[]{ "[pP]rocess" }),
	ENQUEUE(new String[]{ "[eE]nqueue" }),
	DEQUEUE(new String[]{ "[dD]equeue" });

	private String[] expression;

	Action(String[] expression)
	{
		this.expression = expression;
	}

	public String[] getExpression()
	{
		return expression;
	}

	public String getNominalExpression()
	{
		return this.expression[0];
	}

	// for purposes of lookup the synonyms are case-insensitive
	public static Action getAction(String synonym)
	{
		if (synonym == null)
			return null;

		for (Action action : Action.values())
			for (String actionSynonym : action.getExpression())
				if(Pattern.matches(actionSynonym, synonym))
					return action;
		return null;
	}
}
