/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core;

import java.util.regex.Pattern;

/**
 * ENUM of all permissible prepositions with synonyms.
 */
public enum Preposition
{
	BY(new String[] { "[bB]y" }), 
	LIKE(new String[] { "[lL]ike" });

	private String[] expression;

	Preposition(String[] expression)
	{
		this.expression = expression;
	}

	public String[] getExpression()
	{
		return expression;
	}

	public String getNominalExpression()
	{
		return this.expression[0];
	}

	// for purposes of lookup the synonyms are case-insensitive
	public static Preposition getPreposition(String synonym)
	{
		if (synonym == null)
			return null;

		synonym = synonym.toLowerCase();
		for (Preposition preposition : Preposition.values())
			for (String prepositionSynonym : preposition.getExpression())
				if(Pattern.matches(prepositionSynonym, synonym))
					return preposition;
		return null;
	}

	/**
	 * @return
	 */
	public Object toStringFirstCharUpperCase()
	{
		String result = this.name().toLowerCase();
		result = Character.toUpperCase(result.charAt(0))
				+ result.substring(1);
		return result;
	}
}
