/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.core;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;

import org.apache.log4j.Logger;

import gov.va.med.imaging.core.interfaces.FacadeRouter;

/**
 * Utility class to provide access to router facades without having to know the
 * router facade implementation name.
 * 
 * @author zzzzzzbeckec
 *
 */
public class FacadeRouterUtility
{
	private FacadeRouterUtility(){}		// prevent creation
	
	private static Map<Class<?>, Object> routers = new java.util.HashMap<Class<?>, Object>();
	
	@SuppressWarnings("unchecked")
	public static <R extends FacadeRouter> R getFacadeRouter(Class<R> interfaceClass) 
	throws ClassNotFoundException, InstantiationException, IllegalAccessException, IllegalArgumentException, InvocationTargetException
	{
		synchronized(routers)
		{
			FacadeRouter cachedRouter = (FacadeRouter) routers.get(interfaceClass);
			if(cachedRouter != null)
				return (R)cachedRouter;
			
			String interfaceName = interfaceClass.getSimpleName();
			String packageName = interfaceClass.getPackage().getName();
			String implementationName = interfaceName + "Impl";
			String implementationClassName = packageName + "." + implementationName;
	
			// load the implementation class through the interface's class loader
			// else it probably will not find it
			Class<R> implementationClass = (Class<R>) Class.forName(implementationClassName, true, interfaceClass.getClassLoader());
			
			try
			{
				Method singletonAccessor = implementationClass.getMethod("getSingleton", (Class<?>[])null);
				
				// return the singleton accessor result (should be the singleton router facade
				R facadeRouter = (R)singletonAccessor.invoke(null, (Object[])null);
				if(facadeRouter == null)
					return null;
				
				routers.put(interfaceClass, facadeRouter);
				return facadeRouter;
			} 
			catch (SecurityException x)
			{
				Logger.getLogger(FacadeRouterUtility.class).error("The facade router implementation '" + implementationClassName + "' singleton accessor method is inaccessible.");
				throw x;
			} 
			catch (NoSuchMethodException x)
			{
				Logger.getLogger(FacadeRouterUtility.class).error("The implementation for facade router '" + implementationClassName + "' has no singleton accessor method, i.e. public static getSingleton()");
				R facadeRouter = (R)implementationClass.newInstance();
				if(facadeRouter == null)
					return null;
				
				routers.put(interfaceClass, facadeRouter);
				return facadeRouter;
			}
		}
	}
}
