/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core.interfaces;

import gov.va.med.imaging.core.interfaces.exceptions.ProcedureFilterTermsException;

import java.util.AbstractList;

public interface IProcedureFilterTerms
{
	//-----------------------------------
	// properties
	//-----------------------------------
	
	/**
	 * @return a collection of procedure filter terms
	 */
	public abstract AbstractList<String> getFilterTerms();
	
	/**
	 * Reload the internal collection of filter terms.
	 * @param filterTerms - a collection of filter terms to initialize from. 
	 */
	public abstract void setFilterTerms(AbstractList<String> filterTerms);
	
	/**
	 * Add a single filter term to the internal filter term collection. 
	 * @param filterTerm - the filter term to add
	 */
	public abstract void setFilterTerm(String filterTerm);
	
	//-----------------------------------
	// persistence methods
	//-----------------------------------
	/**
	 * @throws ProcedureFilterTermsException
	 * @return true if the filter terms were successfully loaded from file
	 */
	public abstract boolean loadProcedureFilterTermsFromFile() throws ProcedureFilterTermsException;
	
	/**
	 * @throws ProcedureFilterTermsException
	 * @return true if the filter terms were successfully saved to file
	 */
	public abstract boolean saveProcedureFilterTermsToFile() throws ProcedureFilterTermsException;

}	
