/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.core.interfaces;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.core.interfaces.router.Command;
import gov.va.med.imaging.datasource.DataSourceProvider;
import gov.va.med.imaging.datasource.SiteResolutionDataSourceSpi;

/**
 * This interface is the definition of the VIX core API.  All facade projects operate through 
 * this interface exclusively.
 * 
 * All of the methods within this interface behave as follows:
 * 1.) The return value of a successful request is a populated value object.
 * 2.) If the request did not find the requested object(s) then the return value is an empty
 * Collection or a null if the return type is not a Collection
 * 3.) A method may throw only MethodException or one of its derivatives.
 * 
 * @author ZZZZZZBECKEC
 *
 */
public interface Router
{
	/**
	 * 
	 * @param command
	 */
    public abstract <T extends Object> void doAsynchronously(Command<T> command);

	/**
	 * 
	 * @param command
	 * @return
	 * @throws Exception 
	 */
    public abstract <T extends Object> T doSynchronously(Command<T> command)
	throws MethodException, ConnectionException;
	
	/**
	 * @return
	 */
	public boolean isFailoverOnMethodException();

	/**
	 * @return
	 */
	public boolean isCachingEnabled();

	/**
	 * Make the application configuration available for commands
	 * @return
	 */
	public IAppConfiguration getAppConfiguration();
	
	/**
	 * 
	 * @return
	 */
	public DataSourceProvider getProvider();

	/**
	 * 
	 * @return
	 */
	public SiteResolutionDataSourceSpi getSiteResolver();

}
