/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.core.interfaces.router;

import java.lang.reflect.Method;

import gov.va.med.RoutingToken;
import gov.va.med.imaging.artifactsource.ResolvedArtifactSource;
import gov.va.med.imaging.core.interfaces.Router;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.datasource.*;
import gov.va.med.imaging.exchange.business.ResolvedSite;

/**
 * This interface defines the context available to a Command.
 * THe CommandFactory implementation MUST provide an implementation of
 * this interface to each command that it creates.
 * 
 * @author zzzzzzbeckec
 *
 */
public interface CommandContext
{

	/**
	 * @return the router
	 */
	public abstract Router getRouter();

	/**
	 * 
	 * @return
	 */
	public abstract DataSourceProvider getProvider();

	/**
	 * @return the siteResolver
	 */
	public abstract SiteResolutionDataSourceSpi getSiteResolver();
	
	/**
	 * @return the commandFactory
	 */
	public abstract CommandFactory getCommandFactory();
	
	/**
	 * @return the transactionLoggerService
	 */
	public abstract TransactionLoggerDataSourceSpi getTransactionLoggerService();
	
	/**
	 * @return - true if caching is enabled and commands should attempt to cache instances
	 */
	public abstract boolean isCachingEnabled();
	
	/**
	 * This will return a ResolvedSite instance representing the local site that
	 * this VIX is configured to represent as configured in the AppConfiguration 
	 * or null if this VIX does not represent a Site.
	 * 
	 * @return
	 */
	public abstract ResolvedSite getLocalSite();
	
	/**
	 * @param siteNumber
	 * @return
	 * @throws MethodException
	 */
	//public ResolvedSite getSite(String siteNumber) 
	//throws MethodException;
	
	/**
	 * This method DOES NOT respect the routing override services and MUST NOT
	 * be used except for site resolution expressed externally. Use:
	 * getArtifactSource(RoutingToken, spi, method, parameters)
	 * for normal site resolution within a Command.
	 * 
	 * @param siteNumber
	 * @return
	 * @throws MethodException
	 */
	public ResolvedArtifactSource getResolvedArtifactSource(RoutingToken routingToken) 
	throws MethodException;
	
	/**
	 * @param spi
	 * @param method
	 * @param parameters
	 * @return
	 * @throws MethodException 
	 */
	//public ResolvedSite getSite(
	//	String siteNumber, 
	//	Class<? extends VersionableDataSourceSpi> spi, 
	//	Method method, 
	//	Object[] parameters) 
	//throws MethodException;

	/**
	 * This method should be used by the Command implementations to resolve the RoutingToken to 
	 * a ResolvedArtifactSource.  This method will make the calls to the routingOverrideServices if
	 * they are in place.
	 * 
	 * @param routingToken
	 * @param spi
	 * @param method
	 * @param parameters
	 * @return
	 * @throws MethodException
	 */
	public ResolvedArtifactSource getResolvedArtifactSource(
		RoutingToken routingToken, 
		Class<? extends VersionableDataSourceSpi> spi, 
		Method method, 
		Object[] parameters) 
	throws MethodException;

}
