/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.core.interfaces.router.exceptions;

import gov.va.med.imaging.core.interfaces.exceptions.MethodException;

/**
 * @author zzzzzzbeckec
 *
 */
public class IncompatibleRoutingTokenException
extends MethodException
{
	private static final long serialVersionUID = 1L;
	private Class<?> clazz;
	private String msg;
	
	private static final String NULL_TOKEN_MESSAGE = "The routing token was null and this command/SPI must have a non-null value.";
	private static final String NULL_HOME_TOKEN_MESSAGE = "The routing token included a null for the home community ID.  All routing tokens must have non-null home community and repository values.";
	private static final String NULL_REPOSITORY_TOKEN_MESSAGE = "The routing token included a null for the repository ID.  All routing tokens must have non-null home community and repository values.";
	private static final String WILDCARD_HOME_TOKEN_MESSAGE = "The routing token included a wildcard for the home community ID and this command/SPI must have a singular value.";
	private static final String WILDCARD_REPOSITORY_TOKEN_MESSAGE = "The routing token included a wildcard for the repository community ID and this command/SPI must have a singular value.";
	
	/**
	 * 
	 */
	private IncompatibleRoutingTokenException(Class<?> clazz, String msg)
	{
		this.clazz = clazz;
		this.msg = msg;
	}

	/**
	 * @param class1
	 * @return
	 */
	public static IncompatibleRoutingTokenException createInvalidNull(Class<?> clazz)
	{
		return new IncompatibleRoutingTokenException(clazz, NULL_TOKEN_MESSAGE);
	}

	/**
	 * @param class1
	 * @return
	 */
	public static IncompatibleRoutingTokenException createInvalidHomeCommunityWildcard(Class<?> clazz)
	{
		return new IncompatibleRoutingTokenException(clazz, WILDCARD_HOME_TOKEN_MESSAGE);
	}

	/**
	 * @param class1
	 * @return
	 */
	public static IncompatibleRoutingTokenException createInvalidRepositoryWildcard(Class<?> clazz)
	{
		return new IncompatibleRoutingTokenException(clazz, WILDCARD_REPOSITORY_TOKEN_MESSAGE);
	}

}
