/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.datasource;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.core.interfaces.exceptions.ParentREFDeletedMethodException;
import gov.va.med.imaging.datasource.annotations.SPI;
import gov.va.med.imaging.exchange.business.dicom.DGWEmailInfo;
import gov.va.med.imaging.exchange.business.dicom.DicomCorrectInfo;
import gov.va.med.imaging.exchange.business.dicom.InstanceFile;
import gov.va.med.imaging.exchange.business.dicom.InstrumentConfig;
import gov.va.med.imaging.exchange.business.dicom.ModalityConfig;
import gov.va.med.imaging.exchange.business.dicom.PatientRef;
import gov.va.med.imaging.exchange.business.dicom.PatientStudyInfo;
import gov.va.med.imaging.exchange.business.dicom.PatientStudyLookupResults;
import gov.va.med.imaging.exchange.business.dicom.ProcedureRef;
import gov.va.med.imaging.exchange.business.dicom.SOPInstance;
import gov.va.med.imaging.exchange.business.dicom.Series;
import gov.va.med.imaging.exchange.business.dicom.Study;
import gov.va.med.imaging.exchange.business.dicom.UIDActionConfig;
import gov.va.med.imaging.exchange.business.dicom.UIDCheckInfo;
import gov.va.med.imaging.exchange.business.dicom.UIDCheckResult;

import java.util.List;

/**
 * This class defines the Service Provider Interface (SPI) for the DicomDataSource class. 
 * All the abstract methods in this class must be implemented by each 
 * data source service provider who wishes to supply the implementation of a 
 * DicomDataSource for a particular datasource type.
 * 
 * @since 1.0
 * @author zzzzzzlouthj
 *
 */
@SPI(description="The service provider interface for DICOM storage")
public interface DicomStorageDataSourceSpi
extends VersionableDataSourceSpi
{

	// Configuration data retrieval
	List<InstrumentConfig> getDgwInstrumentList(String hostName) throws MethodException, ConnectionException;
	List<ModalityConfig> getDgwModalityList(String hostName) throws MethodException, ConnectionException;
//	List<SourceAESecurityConfig> getSourceAESecurityMatrix() throws MethodException, ConnectionException;
	DGWEmailInfo getDgwEmailInfo(String hostName) throws MethodException, ConnectionException;
	List<UIDActionConfig> getDgwUIDActionTable(String type, String subType, String action) throws MethodException, ConnectionException;
	
	// Patient/Study lookup
	PatientStudyLookupResults getPatientStudyLookupResults(PatientStudyInfo patientStudyInfo) throws MethodException, ConnectionException;

	// UID checking methods
    UIDCheckResult getStudyUIDCheckResult(UIDCheckInfo uidCheckInfo) throws MethodException, ConnectionException;
    UIDCheckResult getSeriesUIDCheckResult(UIDCheckInfo uidCheckInfo) throws MethodException, ConnectionException;
    UIDCheckResult getSOPInstanceUIDCheckResult(UIDCheckInfo uidCheckInfo) throws MethodException, ConnectionException;

    // Object storage and retrieval methods
    PatientRef getOrCreatePatientRef(PatientRef patientRef) throws MethodException, ConnectionException;
	ProcedureRef getOrCreateProcedureRef(PatientRef patientRef, ProcedureRef procedureRef) throws MethodException, ConnectionException;
	Study getOrCreateStudy(PatientRef patientRef, ProcedureRef procedureRef, Study study) throws MethodException, ConnectionException;
	Series getOrCreateSeries(Study study, Series series, Integer iodValidationStatus) throws MethodException, ConnectionException, ParentREFDeletedMethodException;
	SOPInstance createSOPInstance(Series series, SOPInstance sopInstance) throws MethodException, ConnectionException, ParentREFDeletedMethodException;
	InstanceFile createInstanceFile(SOPInstance sopInstance, InstanceFile instanceFile) throws MethodException, ConnectionException;
	Series getTIUPointer(Series series) throws MethodException, ConnectionException;
	Boolean deleteStudyAndSeriesCache() throws MethodException;	

    // DICOM Correct
//	DicomCorrectInfo postDicomCorrectEntry(DicomCorrectInfo dicomCorrectInfo) throws MethodException, ConnectionException;
//    List<DicomCorrectEntry> getDicomCorrectEntryList(DicomCorrectEntry dicomCorrectEntry) throws MethodException, ConnectionException;
//    Boolean deleteDicomCorrectEntry(DicomCorrectEntry dicomCorrectEntry) throws MethodException, ConnectionException;
	Integer getDicomCorrectCount(DicomCorrectInfo dicomCorrectInfo) throws MethodException, ConnectionException;
}
