/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.datasource;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.datasource.annotations.SPI;
import gov.va.med.imaging.exchange.business.DurableQueue;
import gov.va.med.imaging.exchange.business.DurableQueueMessage;

import java.util.List;

/**
 * This class defines the Service Provider Interface (SPI) for the
 * DurableQueueDataSource. All the abstract methods in this class must be
 * implemented by each data source service provider who wishes to supply the
 * implementation of a DurableQueueDataSource for a particular datasource type.
 * 
 * @since 1.0
 * @author ZZZZZZGRAVER
 * 
 */
@SPI(description="The service provider interface for persistent queues.")
public interface DurableQueueDataSourceSpi 
extends VersionableDataSourceSpi 
{
	DurableQueueMessage dequeue(int queueId) throws MethodException,
			ConnectionException;

	DurableQueueMessage dequeue(int queueId, String messageGroupId)
			throws MethodException, ConnectionException;

	DurableQueueMessage enqueue(DurableQueueMessage message)
			throws MethodException, ConnectionException;

	List<DurableQueue> getAll() throws MethodException, ConnectionException;

	DurableQueue getByName(String name) throws MethodException,
			ConnectionException;

	int getMessageCount(int queueId) throws MethodException,
			ConnectionException;

	int getMessageCount(int queueId, String messageGroupId)
			throws MethodException, ConnectionException;

	List<DurableQueueMessage> getMessages(int queueId, int startIndex,
			int numRecords) throws MethodException, ConnectionException;

	List<DurableQueueMessage> getMessages(int queueId, String messageGroupId,
			int previousRecordId, int numRecords) throws MethodException,
			ConnectionException;

	void moveMessage(int messageId, int targetQueueId) throws MethodException,
			ConnectionException;

	DurableQueueMessage peek(int queueId) throws MethodException,
			ConnectionException;

	DurableQueueMessage peek(int queueId, String messageGroupId)
			throws MethodException, ConnectionException;

	DurableQueue updateQueue(DurableQueue queue) throws MethodException,
			ConnectionException;
}
