/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.datasource;

import gov.va.med.imaging.core.annotations.routerfacade.FacadeRouterMethod;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.datasource.annotations.SPI;
import gov.va.med.imaging.exchange.business.WorkItem;
import gov.va.med.imaging.exchange.business.WorkItemFilter;
import gov.va.med.imaging.exchange.business.WorkItemTag;
import gov.va.med.imaging.exchange.business.WorkItemTags;

import java.util.List;

/**
 * This class defines the Service Provider Interface (SPI) for the DicomImporterDataSource class. 
 * All the abstract methods in this class must be implemented by each 
 * data source service provider who wishes to supply the implementation of a 
 * DicomDataSource for a particular datasource type.
 * 
 * @since 1.0
 * @author zzzzzzlouthj
 *
 */
@SPI(description="The service provider interface for DICOM storage")
public interface WorkListDataSourceSpi 
extends VersionableDataSourceSpi
{
	/**
	 * Creates a new work item
	 * @param workItem
	 * @return
	 * @throws MethodException
	 * @throws ConnectionException
	 */
	public abstract WorkItem createWorkItem(WorkItem workItem)
	throws MethodException, ConnectionException;

	/**
	 * Gets the list of work items that match the provided filter
	 * @param filter
	 * @return
	 * @throws MethodException
	 * @throws ConnectionException
	 */
	public abstract List<WorkItem> getWorkItemList(WorkItemFilter filter) 
	throws MethodException, ConnectionException;

	/**
	 * Updates a work item
	 * @param workItemId
	 * @param expectedStatus
	 * @param newStatus
	 * @param newMessage
	 * @param updatingUser
	 * @param updatingApplication
	 * @return
	 * @throws MethodException
	 * @throws ConnectionException
	 */
	public abstract boolean updateWorkItem(
			int workItemId, 
			String expectedStatus, 
			String newStatus, 
			String newMessage, 
			String updatingUser, 
			String updatingApplication) 
	throws MethodException, ConnectionException;
	
	/**
	 * Adds work item tags to an existing work item, as long as the work item is still
	 * in one of the allowed statuses
	 * @param workItemId
	 * @param allowedStatuses
	 * @param newTags
	 * @param updatedBy
	 * @return
	 * @throws MethodException
	 * @throws ConnectionException
	 */
	public abstract boolean postWorkItemTags(
			int workItemId, 
			List<String> allowedStatuses, 
			List<WorkItemTag> newTags, 
			String updatingUser, 
			String updatingApplication) 
	throws MethodException, ConnectionException;
	
	/**
	 * Deletes a work item by id
	 * @param id
	 * @return
	 * @throws MethodException
	 * @throws ConnectionException
	 */
	public abstract boolean deleteWorkItem(int id) 
	throws MethodException, ConnectionException;
	
	/**
	 * Gets a work item by ID and transitions it to the new status, but only if it
	 * is still in the expected status. Otherwise, throws an exception.
	 * @param id
	 * @param expectedStatus
	 * @param newStatus
	 * @param updatingUser
	 * @param updatingApplication
	 * @return
	 * @throws MethodException
	 * @throws ConnectionException
	 */
	public abstract WorkItem getAndTransitionWorkItem(
			int id, 
			String expectedStatus, 
			String newStatus, 
			String updatingUser, 
			String updatingApplication)
	throws MethodException, ConnectionException;

	/**
	 * Gets the oldest work item of a given type in the specified status (if any exist), and 
	 * updates it to the new status. 
	 * @param type
	 * @param expectedStatus
	 * @param newStatus
	 * @param updatingUser
	 * @param updatingApplication
	 * @return
	 * @throws MethodException
	 * @throws ConnectionException
	 */
	public abstract WorkItem getAndTransitionNextWorkItem(
			String type, 
			String expectedStatus, 
			String newStatus, 
			String updatingUser, 
			String updatingApplication,
			String placeId)
	throws MethodException, ConnectionException;

}
