/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.exchange;

import gov.va.med.MediaType;
import gov.va.med.imaging.channels.ChecksumValue;
import java.nio.channels.ByteChannel;

/**
 * @author zzzzzzbeckec
 * 
 * Defines an interface for business objects that include
 * large binary content that is stored externally from the object.
 * This includes all of the Image and Document types, where the 
 * metadata is included in the object and the content is stored separately.
 *  
 */
public abstract class AttachedContent
{
	private String storageKey;
	private MediaType mediaType;
	private long contentLength;
	private ChecksumValue checksumValue;
	private String compressionType;
	
	/**
	 * 
	 */
	protected AttachedContent()
	{
		
	}
	
    /**
	 * @param storageKey
	 * @param mediaType
	 * @param contentLength
	 * @param checksumValue
	 * @param compressionType
	 */
	protected AttachedContent(String storageKey, MediaType mediaType,
			long contentLength, ChecksumValue checksumValue,
			String compressionType)
	{
		super();
		this.storageKey = storageKey;
		this.mediaType = mediaType;
		this.contentLength = contentLength;
		this.checksumValue = checksumValue;
		this.compressionType = compressionType;
	}

	public String getStorageKey()
    {
    	return storageKey;
    }
    
	public MediaType getMediaType()
	{
		return mediaType;
	}
	
	public ByteChannel getContent()
	{
		return null;
	}
	
	public long getContentLength()
	{
		return contentLength;
	}
	
	public ChecksumValue getChecksumValue()
	{
		return checksumValue;
	}

	public String getCompressionType()
	{
		return compressionType;
	}
	
}
