/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.business.dicom;

import gov.va.med.imaging.exchange.business.PersistentEntity;

import java.util.ArrayList;
import java.util.List;

public class PatientStudyLookupResults implements PersistentEntity 
{
	private List<String> errorMessages = new ArrayList<String>();
	
	private int id;
	private PatientStudyInfo patientStudyInfo;
	
	/**
	 * This is a universal constructor for good and bad cases.
	 * It is possible to have patient info, and an error!
	 * 
	 * @param patientStudyInfo
	 * @param errorMessages
	 */
	public PatientStudyLookupResults(PatientStudyInfo patientStudyInfo, List<String> errorMessages)
	{
		this.patientStudyInfo = patientStudyInfo;
		if (errorMessages != null)
		{
			this.errorMessages = errorMessages;
		}
	}

	/**
	 * Returns the fully-populated patientStudyInfo object if lookup was successful. Returns
	 * null if lookup failed for any reason (i.e. isOK()==false).
	 * 
	 * @return
	 */
	public PatientStudyInfo getPatienStudyInfo()
	{
		return patientStudyInfo;
	}

	/**
	 * Returns true if no errors were found during patient/study lookup. False, otherwise.
	 * @return
	 */
	public boolean isOk()
	{
		return (errorMessages.size() == 0);
	}

	/**
	 * Returns the list of error messages received during patient/study lookup. If lookup was successful,
	 * the list of errors will be empty.
	 * 
	 * @return
	 */
	public List<String> getErrorMessages() {
		return errorMessages;
	}
	
    public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}

	

}

