/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.business.dicom;

import gov.va.med.imaging.exchange.business.PersistentEntity;

public class UIDCheckResult implements PersistentEntity 
{
	private static final String NoDuplicateFound = "0";
	private static final String DuplicateFound = "1";
	private static final String SOPInstanceResend = "2";
	private static final String FatalError = "-1"; // Fatal error
	private static final String FatalError2 = "-3"; // fatal UID gen error
	private static final String IllegalUID = "3"; // was "-2" before

	private int id;
	private String originalUID;
	private String correctedUID;
	private String resultCode;

	private UIDCheckResult() {}
	private UIDCheckResult(String resultCode) 
	{
		this.resultCode = resultCode;
	}
	
	private UIDCheckResult(String originalUID, String correctedUID, String resultCode) 
	{
		this.originalUID = originalUID;
		this.correctedUID = correctedUID;
		this.resultCode = resultCode;
	}
	
	public static UIDCheckResult createResultFatalError()
	{
		return new UIDCheckResult(FatalError);
	}
	
	public static UIDCheckResult createResultDuplicateFound(String originalUID, String correctedUID)
	{
		return new UIDCheckResult(originalUID, correctedUID, DuplicateFound);
	}

	public static UIDCheckResult createResultIllegalUIDFound(String originalUID, String correctedUID)
	{
		return new UIDCheckResult(originalUID, correctedUID, IllegalUID);
	}

	public static UIDCheckResult createResultNoDuplicateFound(String originalUID)
	{
		return new UIDCheckResult(originalUID, originalUID, NoDuplicateFound);
	}
	
	public static UIDCheckResult createResultSOPInstanceResendFound(String originalUID)
	{
		return new UIDCheckResult(originalUID, originalUID, SOPInstanceResend);
	}
	
    public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public boolean isDuplicateUID()
	{
		return !originalUID.equals(correctedUID);
	}

	public boolean isIllegalUID()
	{
		return resultCode.equals(IllegalUID);
	}

	public boolean isSOPInstanceResend()
	{
		return resultCode.equals(SOPInstanceResend);
	}

	public boolean isFatalError()
	{
		return (resultCode.equals(FatalError) || resultCode.equals(FatalError2));
	}

	public String getOriginalUID()
	{
		return originalUID;
	}

	public String getCorrectedUID()
	{
		return correctedUID;
	}
	
	public String getResultCode()
	{
		return resultCode;
	}
	
}

