/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.business.dicom;

public class UIDCheckResults
{
	private UIDCheckResult studyResult;
	private UIDCheckResult seriesResult;
	private UIDCheckResult sopInstanceResult;
	
	public UIDCheckResults(UIDCheckResult studyResult, UIDCheckResult seriesResult, UIDCheckResult sopInstanceResult)
	{
		this.studyResult = studyResult;
		this.seriesResult = seriesResult;
		this.sopInstanceResult = sopInstanceResult;
	}
	
	public boolean isDuplicateStudyUID()
	{
		return studyResult.isDuplicateUID();
	}

	public boolean isDuplicateSeriesUID()
	{
		return seriesResult.isDuplicateUID();
	}

	public boolean isDuplicateInstanceUID()
	{
		return sopInstanceResult.isDuplicateUID();
	}

	public boolean anyDuplicatesExist()
	{
		return isDuplicateStudyUID() || isDuplicateSeriesUID() || isDuplicateInstanceUID();
	}

	public boolean anyIllegalsExist()
	{
		return isIllegalStudyUID() || isIllegalSeriesUID() || isIllegalInstanceUID();
	}
		
	public UIDCheckResult getStudyResult()
	{
		return studyResult;
	}

	public UIDCheckResult getSeriesResult()
	{
		return seriesResult;
	}

	public UIDCheckResult getSOPInstanceResult()
	{
		return sopInstanceResult;
	}
	
	public boolean isSOPInstanceResend()
	{
		return this.sopInstanceResult.isSOPInstanceResend();
	}

	public boolean isIllegalStudyUID()
	{
		return this.studyResult.isIllegalUID();
	}
	public boolean isIllegalSeriesUID()
	{
		return this.seriesResult.isIllegalUID();
	}
	public boolean isIllegalInstanceUID()
	{
		return this.sopInstanceResult.isIllegalUID();
	}

}

