/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.business.dicom.importer;

import gov.va.med.imaging.exchange.business.Patient;
import gov.va.med.imaging.exchange.business.dicom.OriginIndex;
import gov.va.med.imaging.exchange.business.dicom.UIDActionConfig;
import gov.va.med.imaging.xstream.FieldUpperCaseMapper;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.basic.DateConverter;
import com.thoughtworks.xstream.mapper.MapperWrapper;

public class ImporterUtils
{
    public static XStream getXStream() 
    {
    	XStream xstream = new XStream() {
            protected MapperWrapper wrapMapper(MapperWrapper next) {
                return new FieldUpperCaseMapper(next);
            }
    	};

    	// Add all the standard aliases
    	xstream.alias("ImporterWorkItem", ImporterWorkItem.class);
    	xstream.alias("ImporterWorkItemDetails", ImporterWorkItemDetails.class);
    	xstream.alias("ImporterWorkItemDetailsReference", ImporterWorkItemDetailsReference.class);
    	xstream.alias("ImporterWorkListFilter", ImporterWorkItemFilter.class);
    	xstream.alias("Order", Order.class);
    	xstream.alias("OrderingLocation", OrderingLocation.class);
    	xstream.alias("OrderingProvider", OrderingProvider.class);
    	xstream.alias("Patient", Patient.class);
    	xstream.alias("Procedure", Procedure.class);
    	xstream.alias("ProcedureModifier", ProcedureModifier.class);
    	xstream.alias("Reconciliation", Reconciliation.class);
    	xstream.alias("Series", Series.class);
    	xstream.alias("SopInstance", SopInstance.class);
    	xstream.alias("Study", Study.class);
    	xstream.alias("ReportParameters", ReportParameters.class);
    	xstream.alias("Report", Report.class);
    	xstream.alias("OriginIndex", OriginIndex.class);
    	xstream.alias("UIDActionConfig", UIDActionConfig.class);
    	
    	
    	String[] acceptableDateFormats = new String[]
    	{
    		"MM-dd-yyyy",
    		"yyyyMMdd"
    	};
    	
    	DateConverter dateConverter = new DateConverter("MM/dd/yyyy", acceptableDateFormats);
    	xstream.registerConverter(dateConverter);
    	return xstream;

    }
}
