/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.datasource;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.ResolvedSite;
import gov.va.med.imaging.exchange.business.Study;
import gov.va.med.imaging.exchange.business.StudyFilter;
import gov.va.med.imaging.exchange.enums.StudyLoadLevel;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;
import junit.framework.TestCase;

/**
 * @author ZZZZZZBECKEC
 *
 */
public abstract class TestStudyGraphDataSourceService 
extends TestCase
{
	/**
	 * 
	 * @return
	 */
	protected abstract StudyGraphDataSourceSpi getStudyGraphDataSource();
	protected abstract List<GetPatientStudiesParameters> getGetPatientStudiesParametersList();
	protected abstract List<SortedSet<Study>> getGetPatientStudiesResultsList();
	protected abstract ResolvedSite getResolvedSite();

	public void testGetPatientStudies(boolean compareResults) 
	{
		StudyGraphDataSourceSpi service = getStudyGraphDataSource();
		if(service == null)
			fail("No service available to test.");
		
		if(getGetPatientStudiesResultsList() == null && getGetPatientStudiesParametersList() == null && compareResults)
			return;
		
		if(getGetPatientStudiesResultsList() == null && getGetPatientStudiesParametersList() != null && compareResults)
			fail("Results list was NOT provided, while parameter list was provided.");
		
		if(getGetPatientStudiesResultsList() != null && getGetPatientStudiesParametersList() == null && compareResults)
			fail("Results list was provided, while parameter list was NOT provided.");
		
		if(getGetPatientStudiesResultsList().size() != getGetPatientStudiesParametersList().size() && compareResults)
			fail("Results list and parameter list are different sizes.");
		
		Iterator<SortedSet<Study>> expectedResultIter = getGetPatientStudiesResultsList().iterator();
		for( GetPatientStudiesParameters parameters : getGetPatientStudiesParametersList() )
		{
			SortedSet<Study> expectedResult = expectedResultIter.next();
			StudyFilter filter = parameters.getFilter();
			String patientIcn = parameters.getPatientIcn();
			SortedSet<Study> result;
			
			try
            {
	            result = service.getPatientStudies(getResolvedSite().getSite().createRoutingToken(), 
	            		patientIcn, filter, StudyLoadLevel.FULL).getArtifacts();
	            if(compareResults)
	            	assertEquals(expectedResult, result);
            } 
			catch (MethodException e)
            {
	            e.printStackTrace();
	            fail(e.getMessage());
            } 
			catch (ConnectionException e)
            {
	            e.printStackTrace();
	            fail(e.getMessage());
            }
		}
	}

	/**
	 * A method to run the getPatientStudy() method with the parameters from 
	 * the getGetPatientStudiesParametersList() method and write the results.
	 * A derived class should store the results somewhere an provide them, once
	 * validated, to a regular test run.
	 * 
	 * @param parameters
	 * @param resultsStream
	 */
	protected List<SortedSet<Study>> getResults()
	{
		StudyGraphDataSourceSpi service = getStudyGraphDataSource();
		if(service == null)
		{
			System.err.println("No service available to produce results from.");
			return null;
		}
		
		if(getGetPatientStudiesParametersList() == null)
			return null;
		
		List<SortedSet<Study>> results = new ArrayList<SortedSet<Study>>();
		
		for( GetPatientStudiesParameters parameters : getGetPatientStudiesParametersList() )
		{
			StudyFilter filter = parameters.getFilter();
			String patientIcn = parameters.getPatientIcn();
			SortedSet<Study> result;
			
			try
            {
	            result = service.getPatientStudies(getResolvedSite().getSite().createRoutingToken(),
	            		patientIcn, filter, StudyLoadLevel.FULL).getArtifacts();
	            results.add(result);
            } 
			catch (MethodException e)
            {
	            e.printStackTrace();
	            fail(e.getMessage());
            } 
			catch (ConnectionException e)
            {
	            e.printStackTrace();
	            fail(e.getMessage());
            }
		}
		
		return results;
	}
}
