/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.datasource;

import java.net.URL;

import gov.va.med.imaging.artifactsource.ResolvedArtifactSource;
import gov.va.med.imaging.core.interfaces.exceptions.SecurityException;

/**
 * @author zzzzzzbeckec
 *
 */
public abstract class AbstractVersionableDataSource
implements VersionableDataSourceSpi
{
	private final ResolvedArtifactSource resolvedArtifactSource;
	private String protocol;
	private Object configuration;
	
	/**
	 * All versionable data sources should include a constructor and/or a static factory
	 * method taking a ResolvedArtifactSource and a protocol.
	 * 
	 * @param resolvedArtifactSource
	 * @param protocol
	 */
	public AbstractVersionableDataSource(ResolvedArtifactSource resolvedArtifactSource, String protocol)
	{
		this.resolvedArtifactSource = resolvedArtifactSource;
		this.protocol = protocol;
	}

	/**
	 * Get the ResolvedArtifactSource that this data source was created with.
	 * 
	 * @return
	 */
	public ResolvedArtifactSource getResolvedArtifactSource()
	{
		return this.resolvedArtifactSource;
	}

	/**
	 * Get the protocol that this data source was created with.
	 * 
	 * @return
	 */
	public String getProtocol()
	{
		return this.protocol;
	}

	/**
	 * Get the metadata URL that the ResolvedArtifactSource includes that uses the
	 * protocol.
	 * 
	 * @return
	 */
	public URL getMetadataUrl()
	{
		return resolvedArtifactSource.getMetadataUrl(getProtocol());		
	}

	/**
	 * Get the artifact URL that the ResolvedArtifactSource includes that uses the
	 * protocol.
	 * 
	 * @return
	 */
	public URL getArtifactUrl()
	{
		return resolvedArtifactSource.getArtifactUrl(getProtocol());		
	}
	
	/**
	 * Default implementation returns false always.
	 * 
	 * @see gov.va.med.imaging.datasource.VersionableDataSourceSpi#isVersionCompatible()
	 */
	@Override
	public boolean isVersionCompatible() throws SecurityException
	{
		return false;
	}

	/**
	 * 
	 */
	public void setConfiguration(Object configuration)
	{
		this.configuration = configuration;
	}

	protected Object getConfiguration()
	{
		return this.configuration;
	}
}
