/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.dicom.importer.rest.resources;

import java.util.List;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.dicom.importer.commands.order.GetOrderingLocationListCommand;
import gov.va.med.imaging.dicom.importer.commands.order.GetOrderingProviderListCommand;
import gov.va.med.imaging.dicom.importer.commands.order.GetProcedureListCommand;
import gov.va.med.imaging.dicom.importer.commands.order.GetProcedureModifierListCommand;
import gov.va.med.imaging.exchange.business.dicom.importer.ImporterUtils;
import gov.va.med.imaging.exchange.business.dicom.importer.Order;
import gov.va.med.imaging.exchange.business.dicom.importer.OrderFilter;
import gov.va.med.imaging.exchange.business.dicom.importer.Procedure;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

import com.thoughtworks.xstream.XStream;

@Path("/order")
public class OrderResource extends BaseImporterResource
{
    // and implement the following GET method 
    @GET 
    @Path("/getOrdersForPatient")
    @Produces("application/xml")
    public String getOrdersForPatient(
    		@QueryParam("patientDfn") String patientDfn,
    		@QueryParam("siteId") String siteId) 
    throws MethodException, ConnectionException 
    {

    	OrderFilter orderFilter = new OrderFilter();
    	orderFilter.setDfn(patientDfn);
    	orderFilter.setSiteId(siteId);
    	
    	List<Order> orders = getRouter().getOrderListForPatient(orderFilter);
    	XStream xstream = ImporterUtils.getXStream();
    	xstream.alias("ArrayOfOrder", List.class);
        return xstream.toXML(orders);

    }

    @GET 
    @Path("/getOrderingProviderList")
    @Produces("application/xml")
    public String getOrderingProviderList(@QueryParam("siteId") String siteId, @QueryParam("searchString") String searchString) 
    throws MethodException, ConnectionException 
    {
    	GetOrderingProviderListCommand command = new GetOrderingProviderListCommand(siteId, searchString, "1.0");
    	return command.execute();
    }

    @GET 
    @Path("/getOrderingLocationList")
    @Produces("application/xml")
    public String getOrderingLocationList(@QueryParam("siteId") String siteId) 
    throws MethodException, ConnectionException 
    {
    	GetOrderingLocationListCommand command = new GetOrderingLocationListCommand(siteId, "1.0");
    	return command.execute();
    }

    @GET 
    @Path("/getProcedureList")
    @Produces("application/xml")
    public String getProcedureList(@QueryParam("siteId") String siteId, @QueryParam("applyFilter") String applyFilter) 
    throws MethodException, ConnectionException, Exception 
    {
    	GetProcedureListCommand command = new GetProcedureListCommand(siteId, applyFilter, "1.0");
    	
    	return command.execute();
    }

    @GET 
    @Path("/getProcedureModifierList")
    @Produces("application/xml")
    public String getProcedureModifierList(@QueryParam("siteId") String siteId) 
    throws MethodException, ConnectionException 
    {
    	GetProcedureModifierListCommand command = new GetProcedureModifierListCommand(siteId, "1.0");
    	return command.execute();
    }
}
