/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.ihe.request;

import java.util.Date;
import gov.va.med.imaging.ihe.FindDocumentStoredQueryParameterNames;
import gov.va.med.imaging.ihe.TransactionType;
import gov.va.med.imaging.ihe.exceptions.ParameterFormatException;
import gov.va.med.imaging.ihe.request.filter.FilterTerm;

/**
 * @author zzzzzzbeckec
 *
 */
public class CrossGatewayQueryRequest
extends FindDocumentStoredQueryRequest
{
	/**
	 * @param transactionType
	 * @param queryUUID
	 * @param homeCommunityId 
	 * @param maxResults
	 * @param returnComposedObject
	 * @param parameters
	 */
	public CrossGatewayQueryRequest(
		long maxResults, 
		boolean returnComposedObject,
		StoredQueryParameterList parameters)
	{
		super(TransactionType.CrossGatewayQuery, maxResults, returnComposedObject, parameters);
	}

	/**
	 * Must be a single value.
	 * 
	 * @return
	 * @throws ParameterFormatException
	 */
	public String getPatientId()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsString( FindDocumentStoredQueryParameterNames.patientId.toString() ); 
	}
	
	public String[] getClassCode()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterRawValue( FindDocumentStoredQueryParameterNames.classCode.toString() )[0]; 
	}

	public FilterTerm getPracticeSettingCode()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterFilterTerm( FindDocumentStoredQueryParameterNames.practiceSettingCode.toString() ); 
	}
	
	public Date getCreationTimeFrom()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsDate( FindDocumentStoredQueryParameterNames.creationTimeFrom.toString() ); 
	}
	public Date getCreationTimeTo()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsDate( FindDocumentStoredQueryParameterNames.creationTimeTo.toString() ); 
	}
	public Date getServiceStartTimeFrom()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsDate( FindDocumentStoredQueryParameterNames.serviceStartTimeFrom.toString() ); 
	}
	public Date getServiceStartTimeTo()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsDate( FindDocumentStoredQueryParameterNames.serviceStartTimeTo.toString() ); 
	}
	public Date getServiceStopTimeFrom()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsDate( FindDocumentStoredQueryParameterNames.serviceStopTimeFrom.toString() ); 
	}
	public Date getServiceStopTimeTo()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsDate( FindDocumentStoredQueryParameterNames.serviceStopTimeTo.toString() ); 
	}
	
	public FilterTerm getHealthcareFacilityTypeCode()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterFilterTerm(FindDocumentStoredQueryParameterNames.healthcareFacilityTypeCode.toString() ); 
	}
	
	public FilterTerm getEventCodeList()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterFilterTerm( FindDocumentStoredQueryParameterNames.eventCodeList.toString() ); 
	}
	
	public FilterTerm getConfidentialityCodeList()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterFilterTerm( FindDocumentStoredQueryParameterNames.confidentialityCodeList.toString() ); 
	}
	
	public String getAuthor()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsString( FindDocumentStoredQueryParameterNames.author.toString() ); 
	}
	
	public String getFormatCode()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsString( FindDocumentStoredQueryParameterNames.formatCode.toString() ); 
	}
	
	public String getEntryStatus()
	throws ParameterFormatException
	{ 
		return getParameters().getParameterAsString( FindDocumentStoredQueryParameterNames.entryStatus.toString() ); 
	}
	
}
