/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.terminology.properties;

import gov.va.med.imaging.terminology.ClassifiedValue;
import gov.va.med.imaging.terminology.SchemeTranslationSPI;
import java.io.IOException;
import java.io.InputStream;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;
import org.apache.log4j.Logger;

/**
 *
 */
abstract class PropertyFileTranslationService
extends Properties
implements SchemeTranslationSPI
{
	private Logger logger = Logger.getLogger(PropertyFileSchemeTranslationProvider.class);
	
	/**
	 * 
	 * @throws InstantiationException
	 */
	PropertyFileTranslationService() 
	throws InstantiationException
	{
		try
		{
			InputStream inStream = getXmlInputStream();
			if(inStream != null)
				this.loadFromXML(inStream);
			else
			{
				inStream = getInputStream();
				if(inStream != null)
					this.load(inStream);
				else
				{
					String msg = "Derived class '" + this.getClass().getSimpleName() + "' does not return an input stream for initialization.";
					this.logger.error(msg);
					throw new java.lang.InstantiationException(msg);
				}
			}
		}
		catch (InvalidPropertiesFormatException x)
		{
			String msg = "Derived class '" + this.getClass().getSimpleName() + "' references a properties file in an invalid format.";
			this.logger.error(msg);
			throw new java.lang.InstantiationException(msg);
		}
		catch (IOException x)
		{
			String msg = "Derived class '" + this.getClass().getSimpleName() + "' references a properties file that is inaccessible.";
			this.logger.error(msg);
			throw new java.lang.InstantiationException(msg);
		}
	}
	
	protected abstract InputStream getInputStream();
	protected abstract InputStream getXmlInputStream();

	@Override
	public ClassifiedValue[] translate(String sourceCode)
	{
		String rawTranslatedValue = this.getProperty(sourceCode);
		logger.debug("Translating '" + sourceCode + "' => '" + rawTranslatedValue + "'.");
		
		if(rawTranslatedValue == null)
			return new ClassifiedValue[]{new ClassifiedValue(getSourceCodingScheme(), sourceCode)};
		else
		{
			// if the mapping is one to many, then the values will be comma delimited
			String[] translatedValues = rawTranslatedValue.split(",");
			ClassifiedValue[] result = new ClassifiedValue[translatedValues.length];
			
			for(int index=0; index<translatedValues.length; ++index)
				result[index] = 
					new ClassifiedValue(getDestinationCodingScheme(), translatedValues[index]);
			
			return result;
		}
	}
}
