/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.ihe.request.filter;

import java.util.Iterator;
import junit.framework.TestCase;

/**
 * @author zzzzzzbeckec
 *
 */
public class TestFilterTerm
extends TestCase
{
	public void testSimpleTerm() 
	throws InvalidTermValueFormatException
	{
		FilterTerm t = new SimpleTerm("'a'");
		assertTrue(t.matches("a"));
		
		t = new SimpleTerm("'_'");
		assertTrue(t.matches("a"));
		assertTrue(t.matches("5"));
		assertTrue(t.matches("A"));
		assertFalse(t.matches("abcdef"));
		
		t = new SimpleTerm("'%'");
		assertTrue(t.matches("a"));
		assertTrue(t.matches("5"));
		assertTrue(t.matches("A"));
		assertTrue(t.matches("abcdef"));
	}
	
	public void testDisjunctionTerm() 
	throws InvalidTermValueFormatException
	{
		FilterTerm leftTerm = new SimpleTerm("'a'");
		FilterTerm rightTerm = new SimpleTerm("'b'");
		FilterTerm t = new DisjunctionTerm(leftTerm, rightTerm);
		assertFalse(t.matches("a"));

		leftTerm = new SimpleTerm("'a'");
		rightTerm = new SimpleTerm("'_'");
		t = new DisjunctionTerm(leftTerm, rightTerm);
		assertTrue(t.matches("a"));
		
		leftTerm = new SimpleTerm("'abc_e'");
		rightTerm = new SimpleTerm("'___d_'");
		t = new DisjunctionTerm(leftTerm, rightTerm);
		assertTrue(t.matches("abcde"));
	}
	
	public void testConjunctionTerm()
	throws InvalidTermValueFormatException
	{
		FilterTerm leftTerm = new SimpleTerm("'a'");
		FilterTerm rightTerm = new SimpleTerm("'b'");
		FilterTerm t = new ConjunctionTerm(leftTerm, rightTerm);
		assertTrue(t.matches("a"));
		assertTrue(t.matches("b"));
		assertFalse(t.matches("c"));
		assertFalse(t.matches("ab"));

		leftTerm = new SimpleTerm("'a'");
		rightTerm = new SimpleTerm("'b'");
		t = new ConjunctionTerm(leftTerm, rightTerm);
		assertTrue(t.matches("a"));
		
		leftTerm = new SimpleTerm("'abc_e'");
		rightTerm = new SimpleTerm("'___d_'");
		t = new ConjunctionTerm(leftTerm, rightTerm);
		assertTrue(t.matches("abcde"));
	}
	
	public void testComplexTerm()
	throws InvalidTermValueFormatException
	{
		// represent ("ab%" OR "xyz") and '__cd'
		// "abcd" should match
		FilterTerm t = 
			new DisjunctionTerm(
				new ConjunctionTerm(new SimpleTerm("'ab%'"), new SimpleTerm("'xyz'")),
				new SimpleTerm("'__cd'")
			);
		assertTrue(t.matches("abcd"));
		assertFalse(t.matches("xyz"));
		assertFalse(t.matches("abcdef"));
		
		// represent ("ab%" AND "__cd") OR 'xyz'
		// "abcd" and "xyz" should match
		t = 
			new ConjunctionTerm(
				new DisjunctionTerm(new SimpleTerm("'ab%'"), new SimpleTerm("'__cd'")),
				new SimpleTerm("'xyz'")
			);
		assertTrue(t.matches("abcd"));
		assertTrue(t.matches("xyz"));
		assertFalse(t.matches("abcdef"));
	}
	
	public void testIteration()
	throws InvalidTermValueFormatException
	{
		FilterTerm t = 
			new DisjunctionTerm(
				new ConjunctionTerm(new SimpleTerm("'ab%'"), new SimpleTerm("'xyz'")),
				new SimpleTerm("'__cd'")
			);

		for( Iterator<SimpleTerm> stIter = t.getSimpleTermIterator(); stIter.hasNext();)
			System.out.println(stIter.next().toString());
	}
}
