/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.terminology;

import gov.va.med.imaging.ihe.exceptions.TranslationException;
import gov.va.med.imaging.terminology.ClassifiedValue;
import gov.va.med.imaging.terminology.CodingScheme;
import junit.framework.TestCase;

/**
 * @author zzzzzzbeckec
 *
 */
public class TestSchemeTranslationServiceFactory
extends TestCase
{
	@Override
	protected void setUp() throws Exception
	{
		super.setUp();
	}

	public void testClassifiedValueTranslation() 
	throws TranslationException
	{
		SchemeTranslationServiceFactory factory = SchemeTranslationServiceFactory.getFactory();
		SchemeTranslationSPI schemeTranslator = null;
		ClassifiedValue[] translatedValue = null;
		
		// translate a value that is mapped, the result should be in the destination coding scheme
		schemeTranslator = factory.getSchemeTranslator(CodingScheme.VADOCUMENTCLASS, CodingScheme.MHS);
		assertNotNull(schemeTranslator);
		translatedValue = schemeTranslator.translate("INSURANCE FORM");
		assertNotNull(translatedValue);
		assertEquals( CodingScheme.MHS, translatedValue[0].getCodingScheme() );
		
		// translate a value that is NOT mapped, the result should be in the source coding scheme
		schemeTranslator = factory.getSchemeTranslator(CodingScheme.VADOCUMENTCLASS, CodingScheme.LOINC);
		assertNotNull(schemeTranslator);
		translatedValue = schemeTranslator.translate("6");
		assertEquals( CodingScheme.VADOCUMENTCLASS, translatedValue[0].getCodingScheme() );
		
		// translate a value that is mapped, the result should be in the destination coding scheme
		schemeTranslator = factory.getSchemeTranslator(CodingScheme.VADOCUMENTCLASS, CodingScheme.LOINC);
		assertNotNull(schemeTranslator);
		translatedValue = schemeTranslator.translate("PATIENT RIGHTS AND RESPONSIBILITIES");
		assertNotNull(translatedValue);
		assertEquals( CodingScheme.LOINC, translatedValue[0].getCodingScheme() );
		
		// translate a value that is NOT mapped, the result should be in the source coding scheme
		schemeTranslator = factory.getSchemeTranslator(CodingScheme.VADOCUMENTCLASS, CodingScheme.MHS);
		assertNotNull(schemeTranslator);
		translatedValue = schemeTranslator.translate("Yata Yata Yata");
		assertEquals( CodingScheme.VADOCUMENTCLASS, translatedValue[0].getCodingScheme() );
	}
}
