/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Layout;

/**
 * A cheap little appender that is convenient for debugging.
 * Each time it is started it will create a new file based on the 
 * time.  Other than that, it is just a derivation of FileAppender.
 * This appender makes no attempt to clean up old files so its use in
 * a production system is strongly discouraged.
 * 
 * @author zzzzzzbeckec
 *
 */
public class DebugFileAppender
extends FileAppender
implements org.apache.log4j.Appender
{
	private static String fileExtension = ".log";
	private static String dateFormatSpecification = "ddMMyyyy-hhmm"; 
	private static DateFormat df = new SimpleDateFormat(dateFormatSpecification);
	
	private static String generateFilename(String filenameRoot)
	{
		return filenameRoot == null ? "appender" : filenameRoot + df.format(new Date()) + fileExtension;
	}
	
	/**
	 * 
	 */
	public DebugFileAppender()
	{
	}

	/**
	 * @param layout
	 * @param filename
	 * @throws IOException
	 */
	public DebugFileAppender(Layout layout, String filenameRoot) 
	throws IOException
	{
		super(layout, generateFilename(filenameRoot));
	}

	/**
	 * @param layout
	 * @param filename
	 * @param append
	 * @throws IOException
	 */
	public DebugFileAppender(Layout layout, String filenameRoot, boolean append)
			throws IOException
	{
		super(layout, generateFilename(filenameRoot), append);
	}

	/**
	 * @param layout
	 * @param filename
	 * @param append
	 * @param bufferedIO
	 * @param bufferSize
	 * @throws IOException
	 */
	public DebugFileAppender(Layout layout, String filenameRoot, boolean append,
			boolean bufferedIO, int bufferSize) throws IOException
	{
		super(layout, generateFilename(filenameRoot), append, bufferedIO, bufferSize);
	}

	public void setFilenameRoot(String filenameRoot)
	{
		setFile(generateFilename(filenameRoot));
	}
}
