/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med;

import java.net.MalformedURLException;
import java.net.URL;

/**
 * An enumeration of the protocols that are currently known.
 * There is no guarantee that because a schema is listed here that the
 * connection handlers are installed.
 * 
 * @author zzzzzzbeckec
 *
 */
public enum ProtocolSchema
{
	HTTP("http"),
	VFTP("vftp"),
	CDTP("cdtp"),
	EXCHANGE("exchange"),
	VISTA("vista"),
	VISTAIMAGING("vistaimaging"),
	XCA("xca");
	
	private final String schema;
	
	ProtocolSchema(String schema)
	{
		this.schema = schema;
	}

	@Override
	public String toString()
	{
		return schema;
	}
	
	/**
	 * Returns true if the connection handler is installed for the given
	 * schema.
	 * NOTE: this will work with the currently known list of schema but may
	 * not work generically and will definitely not work on a non-IP based protocol.
	 * 
	 * @param schema
	 * @return
	 */
	public boolean isConnectionHandlerInstalled()
	{
		try
		{
			URL url = new URL(this.toString(), "127.0.0.1", "");
			return url != null;
		}
		catch (MalformedURLException x)
		{
			return false;
		}
	} 
}
