/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.asynchproxy;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Proxy;

/**
 * 
 * @author ZZZZZZBECKEC
 *
 * @param <I>
 */
public class AsynchProxyFactory<I>
{
	public AsynchProxyFactory()
	{
		
	}
	
	/**
	 * Creates a dynamic proxy which queues the call and immediately returns from every call.
	 * From an Executor, the proxy invocation will take place asynchronously, the result
	 * is made available from the results queue.
	 * 
	 * @param interfaceClass
	 * @param target
	 * @return
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 */
	public I createAsynchProxy(Class<I> interfaceClass, I target, AsynchProxyListener listener) 
	throws IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException
	{
		InvocationHandler handler = new AsynchInvocationHandler<I>(target, null, listener);
		Class<?> proxyClass = Proxy.getProxyClass(interfaceClass.getClassLoader(), new Class[]{ interfaceClass });
		Constructor<I> proxyConstructor = (Constructor<I>)proxyClass.getConstructor(new Class[]{ InvocationHandler.class });
		I proxy = (I) proxyConstructor.newInstance(new Object[]{ handler });

		return proxy;
	}
}
