/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Oct 20, 2008
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  ZZZZZZWERFEJ
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging;

import gov.va.med.*;
import gov.va.med.imaging.exceptions.URNFormatException;
import java.io.Serializable;
import java.util.regex.Pattern;

/**
 * Abstract Imaging URN that ImageURN and StudyURN extend. This is useful
 * for when the identifier from Display can be either a study URN or an 
 * image URN.
 * 
 * @author ZZZZZZWERFEJ
 *
 */
public abstract class AbstractImagingURN 
extends URN
implements Serializable, PatientArtifactIdentifier
{
	private static final long serialVersionUID = -3268947193110412282L;
	
	// /x5e is the carat '^' character
	// allow RFC2141 escaped octet sequences (e.g. %5e)
	//  	"([A-Za-z0-9\\x2e_]*|%[0-9a-fA-f]{2,2})*"
	// these patterns are valid once an identifier has been converted into
	// RFC2141 compliant form
	protected final static String SITEID_REGEX = "((?:[A-Za-z0-9\\x2e_]+)|(?:%[0-9a-fA-F]{2}))+";
	protected final static String GROUPID_REGEX = "((?:[A-Za-z0-9\\x2e_]+)|(?:%[0-9a-fA-F]{2}))+";		// either study or document set
	protected final static String PATIENTID_REGEX = "((?:[A-Za-z0-9\\x2e_]+)|(?:%[0-9a-fA-F]{2}))+";
	protected final static String IMAGEID_REGEX = "((?:[A-Za-z0-9\\x2e_]+)|(?:%[0-9a-fA-F]{2}))+";
	protected final static String MODALITY_REGEX = "((?:[A-Za-z0-9\\x2e_]+)|(?:%[0-9a-fA-F]{2}))+";
	
	public static Pattern siteIdPattern = Pattern.compile(SITEID_REGEX);
	public static Pattern groupIdPattern = Pattern.compile(GROUPID_REGEX);	// either study or document set
	public static Pattern studyUrnPatientIdPattern = Pattern.compile(PATIENTID_REGEX);
	public static Pattern patientIcnPattern = Pattern.compile(PATIENTID_REGEX);
	public static Pattern patientIdPattern = Pattern.compile(PATIENTID_REGEX);
	public static Pattern imageIdPattern = Pattern.compile(IMAGEID_REGEX);
	public static Pattern modalityPattern = Pattern.compile(MODALITY_REGEX);

	public static final String DOD_REPOSITORY_ID = "200"; 
	
	/**
	 * 
	 * @param namespaceIdentifier
	 * @throws URNFormatException
	 */
	protected AbstractImagingURN(NamespaceIdentifier namespaceIdentifier) 
	throws URNFormatException
	{
		super(namespaceIdentifier);
		assert namespaceIdentifier != null : "Namespace identifier cannot be a null value when building " + this.getClass().getSimpleName();
	}
	
	/**
	 * 
	 * @param namespaceIdentifier
	 * @param namespaceSpecificString
	 * @param additionalIdentifiers
	 * @throws URNFormatException
	 */
	protected AbstractImagingURN(
		NamespaceIdentifier namespaceIdentifier, 
		String namespaceSpecificString, 
		String... additionalIdentifiers) 
	throws URNFormatException
	{
		super(namespaceIdentifier, namespaceSpecificString, additionalIdentifiers);
	}
	
	/**
	 * 
	 * @param urnComponents
	 * @throws URNFormatException
	 */
	protected AbstractImagingURN(URNComponents urnComponents, SERIALIZATION_FORMAT serializationFormat) 
	throws URNFormatException
	{
		super(urnComponents, serializationFormat);
	}
	
	public abstract String getOriginatingSiteId();

	public boolean isOriginVA()
	{
		return ! DOD_REPOSITORY_ID.equals(getOriginatingSiteId());
	}
	
	public boolean isOriginDOD()
	{
		return DOD_REPOSITORY_ID.equals(getOriginatingSiteId());
	}
	
	@Override
	public String getPatientIdentifier(){return getPatientId();}
	
	public abstract String getPatientId();

	/**
	 * 
	 */
	@Override
	public abstract AbstractImagingURN clone()
	throws CloneNotSupportedException;
	
	/**
	 * 
	 * @return
	 */
	public abstract String getImagingIdentifier();
	
	/**
	 * 
	 */
	@Override
	public String toRoutingTokenString() 
	{
		return getHomeCommunityId() + "," + getRepositoryUniqueId();
	}
}
