/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging;

/**
 * This is a bit of a kludge because checksums got thrown in late in the project.
 * What happens is that the cache calculates the checksums when the Instance is written.
 * When a caller reads something from the cache, the read may or may not succeed depending
 * on whether the Instance existed in the cache.  Even if it does exist it may not be completely
 * written so the checksum won't be available.
 * 
 * What this interface does is define a callback so that when the web application gets an image,
 * regardless of whether it is cached or not when the request is made, the notification method
 * in here will be sent the valid checksum when it is available but before the content has been
 * streamed.  This way the web application can write the headers (with the checksum) before writing
 * the image.
 * 
 * @author ZZZZZZBECKEC
 *
 */
public interface InstanceChecksumNotification
{
	public void instanceChecksum(String checksumValue);
}
