/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging;

import java.io.IOException;
import java.io.OutputStream;
import java.util.zip.Checksum;

/**
 * A Class that extends OutputStream and drops the bytes written to it to the bit bucket.
 * 
 * @author ZZZZZZBECKEC
 *
 */
public class NullOutputStream 
extends OutputStream
{
	private Checksum checksum;
	private long bytesWritten;
	
	/**
	 * 
	 */
	public NullOutputStream()
	{
	}

	/**
	 * Construct a NullOutputStream with a checksum, which is updated on
	 * each write()
	 * 
	 * @param checksum
	 */
	public NullOutputStream(Checksum checksum)
	{
		this.checksum = checksum;
		bytesWritten = 0;
	}
	
	/**
	 * @see java.io.OutputStream#write(int)
	 */
	@Override
	public void write(int b) 
	throws IOException
	{
		if(checksum != null)
			checksum.update(b);
		++bytesWritten;
	}

	@Override
	public void write(byte[] b, int off, int len) 
	throws IOException
	{
		if(checksum != null)
			checksum.update(b, off, len);
		
		bytesWritten += len;
	}

	@Override
	public void write(byte[] b) 
	throws IOException
	{
		if(checksum != null)
			checksum.update(b, 0, b.length);
		
		bytesWritten += b.length;
	}
	
	public long getBytesWritten()
	{
		return bytesWritten;
	}
}
