/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Proxy;
import java.util.List;
import java.util.Map;

/**
 * This class is a group of static methods that provides similar functionality to that which the 
 * java.util.Collections class provides for synchronized sets, lists and maps.  This class provides 
 * Set, List and Map instances that synchronize using ReadWriteLock implementations so that 
 * multiple read operations may occur simultaneously.  Write operations must be executed serially
 * with respect to both read and write operations.
 * This class creates proxy instances that provide the correct locking and then delegate to the 
 * underlying Set, List or Map instance.
 * 
 * 
 * @author zzzzzzbeckec
 * @since 1.0
 * @version 1.0
 */
public class ReadWriteLockCollections
{
	private static Class<?>[] listInterfaces = new Class[]{ReadWriteLockList.class};
	
	/**
	 * 
	 * @param target
	 * @return
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 */
	@SuppressWarnings("unchecked")
	public static <T extends Object> ReadWriteLockList<T> readWriteLockList(List<T> target) 
	throws IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException
	{
		InvocationHandler handler = new ListReadWriteLockProxyHandler<T>(target);
		
	    Class<?> proxyClass = Proxy.getProxyClass(
	      ReadWriteLockCollections.class.getClassLoader(), 
	      listInterfaces
	    );
	    
	    Constructor<?> proxyConstructor = proxyClass.getConstructor(new Class[] { InvocationHandler.class });
	    ReadWriteLockList<T> lockList = (ReadWriteLockList<T>) proxyConstructor.newInstance( new Object[] { handler } );
	     
	    return lockList;
	}

	private static Class<?>[] mapInterfaces = new Class[]{ReadWriteLockMap.class};
	
	/**
	 * 
	 * @param target
	 * @return
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 */
	@SuppressWarnings("unchecked")
	public static <K extends Object, V extends Object> ReadWriteLockMap<K, V> readWriteLockMap(Map<K, V> target) 
	throws IllegalArgumentException, SecurityException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException
	{
		InvocationHandler handler = new MapReadWriteLockProxyHandler<K,V>(target);
		
	    Class<?> proxyClass = Proxy.getProxyClass(
	      ReadWriteLockCollections.class.getClassLoader(), 
	      mapInterfaces 
	    );
	    
	    Constructor<?> proxyConstructor = proxyClass.getConstructor(new Class[] { InvocationHandler.class }); 
	    ReadWriteLockMap<K,V> lockMap = (ReadWriteLockMap<K,V>) proxyConstructor.newInstance(new Object[] { handler });
	     
	    return lockMap;
	}
}
