/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.channels;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ReadableByteChannel;
import java.util.zip.Checksum;

/**
 * A ReadableByteChannel that calculates a Checksum while it is being read from.
 * This class iterates over a read-only buffer over the destination buffer, as such its
 * performance should be evaluated before using in a production situation. 
 * 
 * @author ZZZZZZBECKEC
 *
 */
public class CheckedReadableByteChannel 
implements ReadableByteChannel
{
	private ReadableByteChannel readableChannel;
	private Checksum checksum;
	
	public CheckedReadableByteChannel(ReadableByteChannel readableChannel, Checksum checksum)
	{
		this.readableChannel = readableChannel;
		this.checksum = checksum;
	}
	
	/* (non-Javadoc)
	 * @see java.nio.channels.ReadableByteChannel#read(java.nio.ByteBuffer)
	 */
	public int read(ByteBuffer dst) 
	throws IOException
	{
		int bytesRead = readableChannel.read(dst);
		
		ByteBuffer piggy = dst.asReadOnlyBuffer();
		piggy.flip();
		while( piggy.position() < piggy.limit() )
			checksum.update(piggy.get());
		
		return bytesRead;
	}

	/* (non-Javadoc)
	 * @see java.nio.channels.Channel#close()
	 */
	public void close() 
	throws IOException
	{
		readableChannel.close();
	}

	/* (non-Javadoc)
	 * @see java.nio.channels.Channel#isOpen()
	 */
	public boolean isOpen()
	{
		return readableChannel.isOpen();
	}

	public Checksum getChecksum()
	{
		return this.checksum;
	}
}
