/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.channels;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.WritableByteChannel;
import java.util.zip.Checksum;

/**
 * A WritableByteChannel that calculates a Checksum while it is being written to.
 * This class iterates over a read-only buffer over the source buffer, as such its
 * performance should be evaluated before using in a production situation. 
 * 
 * @author ZZZZZZBECKEC
 *
 */
public class CheckedWritableByteChannel 
implements WritableByteChannel
{
	private WritableByteChannel writableChannel;
	private Checksum checksum;
	
	public CheckedWritableByteChannel(WritableByteChannel writableChannel, Checksum checksum)
	{
		this.writableChannel = writableChannel;
		this.checksum = checksum;
	}
	
	/* (non-Javadoc)
	 * @see java.nio.channels.WritableByteChannel#write(java.nio.ByteBuffer)
	 */
	public int write(ByteBuffer src) 
	throws IOException
	{
		ByteBuffer piggy = src.asReadOnlyBuffer();
		while( piggy.position() < piggy.limit() )
			checksum.update(piggy.get());
		
		return writableChannel.write(src);
	}

	/* (non-Javadoc)
	 * @see java.nio.channels.Channel#close()
	 */
	public void close() throws IOException
	{
		writableChannel.close();
	}

	/* (non-Javadoc)
	 * @see java.nio.channels.Channel#isOpen()
	 */
	public boolean isOpen()
	{
		return writableChannel.isOpen();
	}

	public Checksum getChecksum()
	{
		return this.checksum;
	}
}
