/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gov.va.med.imaging.notifications;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import gov.va.med.imaging.notifications.email.NotificationEmailProtocol;
import gov.va.med.server.ServerAdapterImpl;
import gov.va.med.server.ServerLifecycleEvent;
import gov.va.med.server.ServerLifecycleListener;

/**
 *
 * @author Jon Louthian
 */
public class NotificationFacade
implements ServerLifecycleListener
{
	private static final Logger logger = Logger.getLogger(NotificationFacade.class);
    private static NotificationConfiguration config = NotificationConfiguration.getConfiguration();
    static List<NotificationProtocol> protocols;

    static
    {
        protocols = new ArrayList<NotificationProtocol>();
        protocols.add(new NotificationEmailProtocol());
    }
    
	public NotificationFacade()
	{
		ServerAdapterImpl.getSingleton().addServerLifecycleListener(this);
	}

    public static void sendGenericNotification(String subject, String message)
    {
        send(Notification.getGenericNotification(subject, message));
    }

    public static void sendVixStartupNotification()
    {
        send(Notification.getVixStartupNotification());
    }

    public static void sendNotification(NotificationTypes notificationType, String subject, String message)
    {
    	Notification notification = Notification.createNotification(notificationType, subject, message);
    	send(notification);
    }

    private static void send(Notification notification)
    {
        if (config.isNotificationEnabled())
        {
        	logNotification(notification);
            for (NotificationProtocol protocol : protocols)
            {
                protocol.send(notification);
            }
        }
        else
        {
        	logger.info("A notification of type " + notification.getNotificationType().name() 
        			+ " was requested, but the notification subsystem is currently disabled.");
        }
    }

	@Override
	public void serverLifecycleEvent(ServerLifecycleEvent event)
	{
		if (event.getEventType().equals(ServerLifecycleEvent.EventType.AFTER_START))
		{
			// Send startup notification
			sendVixStartupNotification();
		}
		
	}
	
	private static void logNotification(Notification notification)
	{
		StringBuilder message = new StringBuilder("Notification of type " + notification.getNotificationType().name() + " received. ");
		message.append("Subject: " + notification.getSubject() + ".");
		logger.info(message);
	}

}
