/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.net;

import java.net.MalformedURLException;
import java.net.URL;

/**
 * A collection of static utility methods that are used to
 * manipulate URL instances. 
 * 
 * @author zzzzzzbeckec
 *
 */
public class URLUtility
{
	/**
	 * 
	 * @param url
	 * @param protocol
	 * @return
	 * @throws MalformedURLException
	 */
	public static URL changeProtocol(URL url, String protocol)
	throws MalformedURLException
	{
		return build(protocol, url.getUserInfo(), url.getHost(), url.getPort(), url.getPath(), url.getQuery(), url.getRef());
	}
	
	public static URL removeUserInfo(URL url)
	throws MalformedURLException
	{
		return changeUserInfo(url, null);
	}
	
	public static URL changeUserInfo(URL url, String userInfo)
	throws MalformedURLException
	{
		return build(url.getProtocol(), userInfo, url.getHost(), url.getPort(), url.getPath(), url.getQuery(), url.getRef());
	}
	
	public static URL changeHost(URL url, String host)
	throws MalformedURLException
	{
		return build(url.getProtocol(), url.getUserInfo(), host, url.getPort(), url.getPath(), url.getQuery(), url.getRef());
	}
	
	public static URL changePort(URL url, int port)
	throws MalformedURLException
	{
		return build(url.getProtocol(), url.getUserInfo(), url.getHost(), port, url.getPath(), url.getQuery(), url.getRef());
	}
	
	public static URL changePath(URL url, String path)
	throws MalformedURLException
	{
		return build(url.getProtocol(), url.getUserInfo(), url.getHost(), url.getPort(), path, url.getQuery(), url.getRef());
	}
	
	public static URL changeQuery(URL url, String query)
	throws MalformedURLException
	{
		return build(url.getProtocol(), url.getUserInfo(), url.getHost(), url.getPort(), url.getPath(), query, url.getRef());
	}
	
	public static URL changeRef(URL url, String ref)
	throws MalformedURLException
	{
		return build(url.getProtocol(), url.getUserInfo(), url.getHost(), url.getPort(), url.getPath(), url.getQuery(), ref);
	}
	
	public static URL build(
		String protocol, 
		String userInfo, 
		String host, 
		int port, 
		String path, 
		String query, 
		String ref) 
	throws MalformedURLException
	{
		StringBuilder sb = new StringBuilder();
		
		if(protocol != null)
		{
			sb.append(protocol);
			sb.append(':');
		}
		
		if(host != null)
		{
			sb.append("//");
			if(userInfo != null && host != null)
			{
				sb.append(userInfo);
				sb.append('@');
			}
			sb.append(host);
			
			if(port > 0)
			{
				sb.append(':');
				sb.append(Integer.toString(port));
			}
		}		
		
		if(path != null)
			sb.append(path);
		
		if(query != null)
		{
			sb.append('?');
			sb.append(query);
		}
		
		if(ref != null)
		{
			sb.append('#');
			sb.append(ref);
		}
		
		return new URL(sb.toString());
	}
	
	public static String buildUserInfo(String uid, String pwd)
	{
		return uid == null ? null :
			pwd == null ? uid : 
				(uid + ":" + pwd);
	}
	
	public static String parseUidFromUserInfo(String userInfo)
	{
		if(userInfo == null || userInfo.length() == 0)
			return null;
		String[] userInfoElements = userInfo.split(":");
		return userInfoElements[0];
	}
	
	public static String parsePwdFromUserInfo(String userInfo)
	{
		if(userInfo == null || userInfo.length() == 0)
			return null;
		String[] userInfoElements = userInfo.split(":");
		return userInfoElements.length > 1 && userInfoElements[1].length() > 0 ? userInfoElements[1] : null;
	}
}
