/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med;

import gov.va.med.imaging.exceptions.OIDFormatException;
import junit.framework.TestCase;

/**
 * @author zzzzzzbeckec
 *
 */
public class OIDTest
extends TestCase
{
	public void testValidOIDCreation() 
	throws OIDFormatException
	{
		OID oid;
		
		oid = OID.create("1");
		assertNotNull(oid);
		assertEquals("1", oid.toString());
		
		oid = OID.create("1.2");
		assertNotNull(oid);
		assertEquals("1.2", oid.toString());
		
		oid = OID.create("1.2.3.4");
		assertNotNull(oid);
		assertEquals("1.2.3.4", oid.toString());
	}
	
	public void testInvalidOIDCreation() 
	{
		OID oid;
		
		try
		{
			oid = OID.create("01");
			fail("Invalid OID format was accepted");
		}
		catch (OIDFormatException x){}
		try
		{
			oid = OID.create("1.02");
			fail("Invalid OID format was accepted");
		}
		catch (OIDFormatException x){}
		try
		{
			oid = OID.create("1.2.a");
			fail("Invalid OID format was accepted");
		}
		catch (OIDFormatException x){}
	}
	
	public void testEquality() 
	throws OIDFormatException
	{
		OID oid;
		OID noid;
		
		oid = OID.create("1");
		noid = OID.create("1");
		assertEquals(oid, noid);
		
		oid = OID.create("1.2");
		noid = OID.create("1.2");
		assertEquals(oid, noid);
		
		oid = OID.create("1.2.3.4");
		noid = OID.create("1.2.3.4");
		assertEquals(oid, noid);
		
		oid = OID.create("1");
		noid = OID.create("2");
		assertTrue(! oid.equals(noid) );
		
		oid = OID.create("1");
		noid = OID.create("1.2");
		assertTrue(! oid.equals(noid) );
		
	}
	
	public void testCompare() 
	throws OIDFormatException
	{
		OID oid;
		OID noid;
		
		oid = OID.create("1");
		noid = OID.create("1");
		assertEquals(0, oid.compareTo(noid));
		
		oid = OID.create("1");
		noid = OID.create("2");
		assertTrue(oid.compareTo(noid) < 0);
		
		oid = OID.create("1");
		noid = OID.create("1.2");
		assertTrue(oid.compareTo(noid) < 0);
		
		oid = OID.create("2");
		noid = OID.create("1");
		assertTrue(oid.compareTo(noid) > 0);
		
		oid = OID.create("1.2");
		noid = OID.create("1");
		assertTrue(oid.compareTo(noid) > 0);
	}	
	
	public void testAncestry() 
	throws OIDFormatException
	{
		OID oid;
		OID noid;
		
		oid = OID.create("1");
		noid = OID.create("1");
		assertFalse(oid.isAncestorOf(noid));
		
		oid = OID.create("1");
		noid = OID.create("2");
		assertFalse(oid.isAncestorOf(noid));
		
		oid = OID.create("1");
		noid = OID.create("1.2");
		assertTrue(oid.isAncestorOf(noid));
		
		oid = OID.create("1");
		noid = OID.create("1.2.3.4");
		assertTrue(oid.isAncestorOf(noid));
		
	}
}
