/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging;

import java.lang.reflect.Method;

import junit.framework.TestCase;

public class StackTraceAnalyzerTest 
extends TestCase
{
	StackTraceElement[] stackTrace = null;
	StackTraceAnalyzer analyzer = null;
	
	protected void setUp() throws Exception
	{
		super.setUp();
		TraceTestClassOne one = new TraceTestClassOne();
		
		stackTrace = one.makeStackTrace();
		analyzer = new StackTraceAnalyzer(stackTrace);
		System.out.println( analyzer.toString() );
	}

	protected void tearDown() throws Exception
	{
		super.tearDown();
	}

	public void testIsClassInStack()
	{
		assertTrue( analyzer.isClassInStack(TraceTestClassOne.class) );
		assertTrue( analyzer.isClassInStack(TraceTestClassTwo.class) );
	}

	public void testIsMethodNameInStackMethod() 
	throws SecurityException, NoSuchMethodException
	{
		Method subjectMethodOne = TraceTestClassOne.class.getMethod("makeStackTrace", (Class[])null);
		Method subjectMethodTwo = TraceTestClassOne.class.getMethod("makeStackTrace", (Class[])null);
		
		assertTrue( analyzer.isMethodNameInStack(subjectMethodOne) );
		assertTrue( analyzer.isMethodNameInStack(subjectMethodTwo) );
	}

	public void testIsMethodNameInStackStringString()
	{
		assertTrue( analyzer.isMethodNameInStack(TraceTestClassOne.class.getName(), "makeStackTrace") );
		assertTrue( analyzer.isMethodNameInStack(TraceTestClassTwo.class.getName(), "makeStackTrace") );
	}

	public void testGetFirstElementInPackage()
	{
		StackTraceElement element = analyzer.getFirstElementInPackage("gov.va.med.imaging");
		assertNull(element);
	}

	public void testGetFirstElementNotInPackage()
	{
		StackTraceElement element = analyzer.getFirstElementNotInPackage("gov.va.med.imaging");
		assertNotNull(element);
	}

	public void testGetFirstElementInPackageHierarchy()
	{
		StackTraceElement element = analyzer.getFirstElementInPackageHierarchy("gov.va");
		assertNull(element);
	}

	public void testGetFirstElementNotInPackageHierarchy()
	{
		StackTraceElement element = analyzer.getFirstElementNotInPackageHierarchy("gov.va");
		assertNotNull(element);
	}

	class TraceTestClassOne
	{
		private TraceTestClassTwo another = null;
		TraceTestClassOne()
		{
			another = new TraceTestClassTwo();
		}
		
		public StackTraceElement[] makeStackTrace()
		{
			return another.makeStackTrace();
		}
	}

	class TraceTestClassTwo
	{
		TraceTestClassTwo()
		{
			
		}
		
		public StackTraceElement[] makeStackTrace()
		{
			return Thread.currentThread().getStackTrace();
		}
		
	}
}


