/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.channels;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.util.zip.Adler32;

import junit.framework.TestCase;

/**
 * @author ZZZZZZBECKEC
 *
 */
public class CheckedByteChannelTest extends TestCase
{

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() 
	throws Exception
	{
		super.setUp();
	}

	public ReadableByteChannel getTestDataAsReadableByteChannel()
	{
		return Channels.newChannel(
			getClass().getClassLoader().getResourceAsStream("edvard_munch.gif")
		);
	}
	
	public WritableByteChannel getDrain() 
	throws IOException
	{
		File tempFile = File.createTempFile("sink", ".tmp");
		FileOutputStream outStream = new FileOutputStream(tempFile);
		
		tempFile.deleteOnExit();
		
		return outStream.getChannel();
	}

	public void testCheckedByteChannels() 
	throws IOException
	{
		ByteBuffer buffy = ByteBuffer.allocate(1024);
		CheckedReadableByteChannel readChannel = new CheckedReadableByteChannel(getTestDataAsReadableByteChannel(), new Adler32());
		CheckedWritableByteChannel writeChannel = new CheckedWritableByteChannel(getDrain(), new Adler32());
		while( readChannel.read(buffy) >= 0 )
		{
			buffy.flip();
			writeChannel.write(buffy);
			buffy.clear();
		}
		
		assertTrue(readChannel.getChecksum().getValue() != 0);
		assertTrue(writeChannel.getChecksum().getValue() != 0);
		
		assertEquals( readChannel.getChecksum().getValue(), writeChannel.getChecksum().getValue() );
		
		readChannel.close();
		writeChannel.close();
	}
	
	protected void tearDown() 
	throws Exception
	{
		super.tearDown();
	}
	
}
