/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.dicom.common.stats;


import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;

public class DicomServiceStatsTests extends TestCase{

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	public void testPortStatus(){
		DicomServiceStats.getInstance().setCurrentPortStatus(10, DicomServiceStats.UP);
		DicomServiceStats.getInstance().setCurrentPortStatus(11, DicomServiceStats.UP);
		DicomServiceStats.getInstance().setCurrentPortStatus(13, DicomServiceStats.DOWN);
		
		ListeningPortStatistics port1 = DicomServiceStats.getInstance().getFromPortList(10);
		assertEquals(DicomServiceStats.UP, port1.getCurrentStatus());

		ListeningPortStatistics port2 = DicomServiceStats.getInstance().getFromPortList(11);
		assertEquals(DicomServiceStats.UP, port2.getCurrentStatus());

		ListeningPortStatistics port3 = DicomServiceStats.getInstance().getFromPortList(13);
		assertEquals(DicomServiceStats.DOWN, port3.getCurrentStatus());
	}
	
	public void testInboundAssociations(){
		DicomServiceStats.getInstance().incrementInboundAssociationAcceptCount("Strawberry", "127.00.0.1");
		DicomServiceStats.getInstance().incrementInboundAssociationAcceptCount("Chocolate", "127.00.0.1");
		DicomServiceStats.getInstance().incrementInboundAssociationAcceptCount("Vanilla", "127.00.0.1");
		
		DicomServiceStats.getInstance().incrementInboundAssociationRejectCount("Vanilla", "127.00.0.1");
		DicomServiceStats.getInstance().incrementInboundAssociationRejectCount("Vanilla", "127.00.0.1");
		DicomServiceStats.getInstance().incrementInboundAssociationRejectCount("Strawberry", "127.00.0.1");
	
		
		InboundDicomAssociationStatistics strawberry = DicomServiceStats.getInstance().getFromInboundAssociationList("Strawberry", "127.00.0.1");
		assertEquals(1, strawberry.getTotalAcceptedAssociations());
		assertEquals(1, strawberry.getTotalRejectedAssociations());

		InboundDicomAssociationStatistics chocolate = DicomServiceStats.getInstance().getFromInboundAssociationList("Chocolate", "127.00.0.1");
		assertEquals(1, chocolate.getTotalAcceptedAssociations());
		assertEquals(0, chocolate.getTotalRejectedAssociations());

		InboundDicomAssociationStatistics vanilla = DicomServiceStats.getInstance().getFromInboundAssociationList("Vanilla", "127.00.0.1");
		assertEquals(1, vanilla.getTotalAcceptedAssociations());
		assertEquals(2, vanilla.getTotalRejectedAssociations());
	}
	
	public void testInboundDimseMessages(){
		DicomServiceStats.getInstance().incrementInboundDimseMessageAcceptCount("Strawberry", DicomServiceStats.CSTORE);
		DicomServiceStats.getInstance().incrementInboundDimseMessageAcceptCount("Strawberry", DicomServiceStats.CSTORE);
		DicomServiceStats.getInstance().incrementInboundDimseMessageRejectCount("Vanilla", DicomServiceStats.CFIND);
		
		InboundDicomDimseStatistics strawberry = DicomServiceStats.getInstance().getFromDimseList("Strawberry", DicomServiceStats.CSTORE);
		assertEquals(2, strawberry.getTotalProcessedDimseMessages());
		assertEquals(0, strawberry.getTotalRejectedDimseMessages());

		InboundDicomDimseStatistics vanilla = DicomServiceStats.getInstance().getFromDimseList("Vanilla", DicomServiceStats.CFIND);
		assertEquals(1, vanilla.getTotalRejectedDimseMessages());
		assertEquals(0, vanilla.getTotalProcessedDimseMessages());
	}
	
	public void testInboundObjects(){
		DicomServiceStats.getInstance().incrementInboundObjectProcessedCount("Chocolate", "Veterans Affairs", "Party Cake", "Computed Radiography");
		DicomServiceStats.getInstance().incrementInboundObjectProcessedCount("Chocolate", "Veterans Affairs", "Party Cake", "Computed Radiography");
		DicomServiceStats.getInstance().incrementInboundObjectProcessedCount("Vanilla", "Veterans Affairs", "Fudge Ripple", "Computed Tomography");
		DicomServiceStats.getInstance().incrementInboundObjectProcessedCount("Strawberry", "Veterans Affairs", "Fudge Ripple", "Magnetic Resonance");
		
		DicomServiceStats.getInstance().incrementInboundObjectRejectedCount("Strawberry", "Veterans Affairs", "Fudge Ripple", "Magnetic Resonance");
		
		DicomServiceStats.getInstance().incrementInboundDuplicateObjectCount("Vanilla", "Veterans Affairs", "Fudge Ripple");
		DicomServiceStats.getInstance().incrementInboundDuplicateObjectCount("Vanilla", "Veterans Affairs", "Fudge Ripple");
		DicomServiceStats.getInstance().incrementInboundDuplicateObjectCount("Vanilla", "Veterans Affairs", "Fudge Ripple");
		DicomServiceStats.getInstance().incrementInboundDuplicateObjectCount("Strawberry", "Veterans Affairs", "Cookie Dough");
		
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToLegacyGWCount("Vanilla");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToLegacyGWCount("Vanilla");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToLegacyGWCount("Vanilla");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToLegacyGWCount("Strawberry");
		
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToHDIGDataStructureCount("Vanilla");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToHDIGDataStructureCount("Vanilla");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToHDIGDataStructureCount("Strawberry");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToHDIGDataStructureCount("Chocolate");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToHDIGDataStructureCount("Chocolate");
		DicomServiceStats.getInstance().incrementInboundObjectsPassedToHDIGDataStructureCount("Chocolate");
			
		DicomServiceStats.getInstance().incrementDuplicateInstanceUIDsCount("Veterans Affairs", "Cookie Dough");
		DicomServiceStats.getInstance().incrementDuplicateInstanceUIDsCount("Veterans Affairs", "Cookie Dough");
		DicomServiceStats.getInstance().incrementDuplicateInstanceUIDsCount("Veterans Affairs ", "Fudge Ripple ");
				
		DicomServiceStats.getInstance().incrementIODViolationCount("Veterans Affairs", "Neopolitan");
		DicomServiceStats.getInstance().incrementIODViolationCount("Veterans Affairs", "Neopolitan");
		
		InboundDicomObjectStatistics chocolate = DicomServiceStats.getInstance().getFromStorageSCPList("Chocolate");
		assertEquals(2, chocolate.getTotalObjectsProcessed());
		assertEquals(0, chocolate.getTotalObjectsRejected());
		assertEquals(0, chocolate.getTotalObjectsPassedToLegacyGW());
		assertEquals(3, chocolate.getTotalObjectsPassedToHDIGDataStructure());
		assertEquals(0, chocolate.getTotalDuplicateObjects());

		InboundDicomObjectStatistics strawberry = DicomServiceStats.getInstance().getFromStorageSCPList("Strawberry");
		assertEquals(1, strawberry.getTotalObjectsProcessed());
		assertEquals(1, strawberry.getTotalObjectsRejected());
		assertEquals(1, strawberry.getTotalObjectsPassedToLegacyGW());
		assertEquals(1, strawberry.getTotalObjectsPassedToHDIGDataStructure());
		assertEquals(1, strawberry.getTotalDuplicateObjects());

		InboundDicomObjectStatistics vanilla = DicomServiceStats.getInstance().getFromStorageSCPList("Vanilla");
		assertEquals(1, vanilla.getTotalObjectsProcessed());
		assertEquals(0, vanilla.getTotalObjectsRejected());
		assertEquals(3, vanilla.getTotalObjectsPassedToLegacyGW());
		assertEquals(2, vanilla.getTotalObjectsPassedToHDIGDataStructure());
		assertEquals(3, vanilla.getTotalDuplicateObjects());

		InboundModalityDeviceStatistics cookie = DicomServiceStats.getInstance().getFromModalityDeviceList("Veterans Affairs", "Cookie Dough");
		assertEquals(2, cookie.getTotalDuplicateInstanceUIDs());
		assertEquals(0, cookie.getTotalDicomObjectsProcessed());
		assertEquals(0, cookie.getTotalDicomObjectsRejected());
		assertEquals(0, cookie.getTotalDicomObjectsWithIODViolations());
		assertEquals(1, cookie.getTotalDuplicateObjects());

		InboundModalityDeviceStatistics cake = DicomServiceStats.getInstance().getFromModalityDeviceList("Veterans Affairs", "Party Cake");
		assertEquals(0, cake.getTotalDuplicateInstanceUIDs());
		assertEquals(2, cake.getTotalDicomObjectsProcessed());
		assertEquals(0, cake.getTotalDicomObjectsRejected());
		assertEquals(0, cake.getTotalDicomObjectsWithIODViolations());
		assertEquals(0, cake.getTotalDuplicateObjects());

		InboundModalityDeviceStatistics ripple = DicomServiceStats.getInstance().getFromModalityDeviceList("Veterans Affairs", "Fudge Ripple");
		assertEquals(1, ripple.getTotalDuplicateInstanceUIDs());
		assertEquals(2, ripple.getTotalDicomObjectsProcessed());
		assertEquals(1, ripple.getTotalDicomObjectsRejected());
		assertEquals(0, ripple.getTotalDicomObjectsWithIODViolations());
		assertEquals(3, ripple.getTotalDuplicateObjects());

		InboundModalityDeviceStatistics neo = DicomServiceStats.getInstance().getFromModalityDeviceList("Veterans Affairs", "Neopolitan");
		assertEquals(0, neo.getTotalDuplicateInstanceUIDs());
		assertEquals(0, neo.getTotalDicomObjectsProcessed());
		assertEquals(0, neo.getTotalDicomObjectsRejected());
		assertEquals(2, neo.getTotalDicomObjectsWithIODViolations());
		assertEquals(0, neo.getTotalDuplicateObjects());
		
		InboundSOPClassStatistics cr = DicomServiceStats.getInstance().getFromSOPClassList("Computed Radiography");
		assertEquals(2,cr.getTotalDicomObjectsProcessed());
		assertEquals(0,cr.getTotalDicomObjectsRejected());
		
		InboundSOPClassStatistics ct = DicomServiceStats.getInstance().getFromSOPClassList("Computed Tomography");
		assertEquals(1,ct.getTotalDicomObjectsProcessed());
		assertEquals(0,ct.getTotalDicomObjectsRejected());

		InboundSOPClassStatistics mr = DicomServiceStats.getInstance().getFromSOPClassList("Magnetic Resonance");
		assertEquals(1,mr.getTotalDicomObjectsProcessed());
		assertEquals(1,mr.getTotalDicomObjectsRejected());

	}
	
	public void testOutboundAssociations(){
		DicomServiceStats.getInstance().incrementOutboundAssociationAcceptCount("Chocolate");
		DicomServiceStats.getInstance().incrementOutboundAssociationAcceptCount("Chocolate");
		DicomServiceStats.getInstance().incrementOutboundAssociationRejectCount("Chocolate");
		DicomServiceStats.getInstance().incrementOutboundAssociationAcceptCount("Chocolate");
		
		OutboundDicomAssociationStatistics chocolate = DicomServiceStats.getInstance().getFromOutboundAssociationList("Chocolate");
		assertEquals(3, chocolate.getTotalAcceptedAssociations());
		assertEquals(1, chocolate.getTotalRejectedAssociations());		
	}
	
	public void testOutboundObjects(){
		DicomServiceStats.getInstance().incrementOutboundObjectTransmittedCount("Chocolate");
		DicomServiceStats.getInstance().incrementOutboundObjectTransmittedCount("Chocolate");
		DicomServiceStats.getInstance().incrementOutboundObjectTransmittedCount("Chocolate");
		DicomServiceStats.getInstance().incrementOutboundObjectRejectedCount("Chocolate");
		
		OutboundDicomObjectStatistics chocolate = DicomServiceStats.getInstance().getFromStorageSCUList("Chocolate");
		assertEquals(3,chocolate.getTotalObjectsTransmittedToSCP());
		assertEquals(1, chocolate.getTotalObjectsRejectedBySCP());		
	}
	
	public void testDicomServiceStats(){
		DicomServiceStats.getInstance().setTotalRADObjectsToCorrectForHDIG(25);
		DicomServiceStats.getInstance().setTotalCONObjectsToCorrectForHDIG(50);
		assertEquals(25, DicomServiceStats.getInstance().getTotalRADObjectsToCorrectForHDIG());
		assertEquals(50, DicomServiceStats.getInstance().getTotalCONObjectsToCorrectForHDIG());
		
		DicomServiceStats.getInstance().setTotalRADObjectsToCorrectForHDIG(0);
		DicomServiceStats.getInstance().setTotalCONObjectsToCorrectForHDIG(0);
		assertEquals(0, DicomServiceStats.getInstance().getTotalRADObjectsToCorrectForHDIG());
		assertEquals(0, DicomServiceStats.getInstance().getTotalCONObjectsToCorrectForHDIG());
	}
}
