/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Jul 28, 2009
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  zzzzzzwerfej
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.proxy.services;

import gov.va.med.imaging.datasource.ImageAccessLoggingSpi;
import gov.va.med.imaging.datasource.UserDataSourceSpi;
import gov.va.med.imaging.proxy.ids.IDSOperation;

/**
 * This enumeration defines the types of service operations available to a proxy
 * 
 * @author zzzzzzwerfej
 *
 */
public enum ProxyServiceType 
{
	
	image("Standard image retrieval using a URN", IDSOperation.IDS_OPERATION_IMAGE), 
	metadata("Any form of metadata retrieval usually pointing to a web service endpoint", IDSOperation.IDS_OPERATION_METADATA), 
	photo("Retrieval of a photo Id, the interface for this method may be different than an image retrieval", IDSOperation.IDS_OPERATION_PHOTO), 
	text("Retrieval of an image TXT file, the interface for this method may be different than an image retrieval", IDSOperation.IDS_OPERATION_TEXT),
	examImage("VistARad exam image retrieval using a URN", IDSOperation.IDS_OPERATION_EXAM_IMAGE),
	examImageText("VistARad exam image text file retrieval using a URN", IDSOperation.IDS_OPERATION_EXAM_IMAGE_TEXT),
	vistaRadMetadata("Metadata retrieval for VistARad methods", IDSOperation.IDS_OPERATION_VISTARAD_METADATA),
	document("Document retrieval", IDSOperation.IDS_OPERATION_DOCUMENT),
	// JMW 9/10/2012 going forward Federation functionality will be split based on the SPI so the SPI name will be in IDS
	user("User Methods", UserDataSourceSpi.class.getSimpleName()),
	imageAccessLogging("Image Access Logging Methods", ImageAccessLoggingSpi.class.getSimpleName());
	
	final String description;
	final String idsOperationType;
	
	ProxyServiceType(String description, String idsOperationType)
	{
		this.description = description;		
		this.idsOperationType = idsOperationType;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @return the idsOperationType
	 */
	public String getIdsOperationType() {
		return idsOperationType;
	}
	
	/**
	 * Retrieve the ProxyServiceType from a given IDS operation name
	 * @param idsOperationType
	 * @return the specified ProxyServiceType or null if none matches
	 */
	public static ProxyServiceType getProxyServiceTypeFromIDSOperationType(String idsOperationType)
	{
		for(ProxyServiceType serviceType : ProxyServiceType.values())
		{
			if(idsOperationType.equals(serviceType.getIdsOperationType()))
			{
				return serviceType;
			}
		}
		return null;
	}
	
	/**
	 * Retrieve the ProxyServiceType from a given IDS operation name
	 * @param idsOperation
	 * @return the specified ProxyServiceType or null if none matches
	 */
	public static ProxyServiceType getProxyServiceTypeFromIDSOperation(IDSOperation idsOperation)
	{
		return getProxyServiceTypeFromIDSOperationType(idsOperation.getOperationType());
	}
}
