/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.proxy.services;

import java.net.URL;
import gov.va.med.imaging.artifactsource.ResolvedArtifactSource;

/**
 * Wraps a ResolvedArtifactSource along with a type and a protocol into
 * something that looks like a ProxyService. 
 * 
 * @author zzzzzzbeckec
 *
 */
public class ResolvedArtifactSourceProxyService
extends AbstractProxyService
{
	private final ResolvedArtifactSource resolvedArtifactSource;
	private final String protocol;
	private final ProxyServiceType type;
	
	/**
	 * 
	 */
	public ResolvedArtifactSourceProxyService(
		ResolvedArtifactSource resolvedArtifactSource, 
		String protocol,
		ProxyServiceType type)
	{
		assert resolvedArtifactSource != null;
		assert protocol != null;
		
		this.resolvedArtifactSource = resolvedArtifactSource;
		this.protocol = protocol;
		this.type = type;
	}
	
	private URL getURL()
	{
		if(ProxyServiceType.metadata == getProxyServiceType())
			getResolvedArtifactSource().getMetadataUrl(getProtocol());
		else if(ProxyServiceType.image == getProxyServiceType())
			getResolvedArtifactSource().getArtifactUrl(getProtocol());
		
		return null;
	}
	
	public ResolvedArtifactSource getResolvedArtifactSource()
	{
		return this.resolvedArtifactSource;
	}
	
	public String getProtocol()
	{
		return this.protocol;
	}

	@Override
	public String getApplicationPath()
	{
		String path = getURL().getPath();
		if(path == null)
			return null;
		
		String[] pathElements = path.split("/");
		return pathElements[0];
	}

	@Override
	public String getOperationPath()
	{
		String path = getURL().getPath();
		if(path == null)
			return null;

		int firstSlash = path.indexOf('/');
		return firstSlash >= 0 ? path.substring(firstSlash) : null;
	}

	@Override
	public String getConnectionURL()
	{
		return getURL().toExternalForm();
	}

	@Override
	public Object getCredentials()
	{
		String userInfo = getURL().getUserInfo();
		if(userInfo == null)
			return null;
		String[] userInfoElements = userInfo.split(":");
		return userInfoElements.length >= 2 ? userInfoElements[1] : null;
	}

	@Override
	public String getHost()
	{
		return getURL().getHost();
	}

	@Override
	public int getPort()
	{
		return getURL().getPort();
	}

	@Override
	public ProxyServiceType getProxyServiceType()
	{
		return type;
	}

	@Override
	public String getUid()
	{
		String userInfo = getURL().getUserInfo();
		if(userInfo == null)
			return null;
		String[] userInfoElements = userInfo.split(":");
		return userInfoElements[0];
	}

}
