/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.exchange.storage.cache;

import java.io.Serializable;
import gov.va.med.GlobalArtifactIdentifier;
import gov.va.med.imaging.exchange.business.Study;
import gov.va.med.imaging.exchange.business.documents.Document;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;

/**
 * The interface to get/retrieve cache objects.
 * The cache is "realized" because it is mapped to one or more Cache
 * regions that have associated storage and eviction rules.
 * 
 * @author zzzzzzbeckec
 *
 */
public interface RealizedCache
{
	/**
	 * This method will use the Study's global artifact identifier to
	 * build the cache groups.
	 * 
	 * @param study
	 * @throws CacheException
	 */
	public void createStudy(Study study) 
	throws CacheException;
	
	/**
	 * Find the Study instance in the cache, using the global artifact identifier (the
	 * study URN).
	 * 
	 * @param gai
	 * @return
	 * @throws CacheException
	 */
	public Study getStudy(GlobalArtifactIdentifier gai) 
	throws CacheException;
	
	/**
	 * @param homeCommunityId
	 * @param siteNumber
	 * @param documentUrn
	 * @param document
	 */
	public void createDocumentMetadata(GlobalArtifactIdentifier gaid, Document document)
	throws CacheException;

	/**
	 * @param homeCommunityId
	 * @param siteNumber
	 * @param documentId
	 * @return
	 * @throws CacheException
	 */
	public Document getDocumentMetadata(GlobalArtifactIdentifier gaid) 
	throws CacheException;

	/**
	 * @param homeCommunityId
	 * @param siteNumber
	 * @param documentId
	 * @return
	 * @throws CacheException
	 */
	public ImmutableInstance getDocumentContent(GlobalArtifactIdentifier gaid)
	throws CacheException;

	/**
	 * @param homeCommunityId
	 * @param siteNumber
	 * @param documentId
	 * @param inStream
	 * @throws CacheException
	 */
	public ImmutableInstance createDocumentContent( GlobalArtifactIdentifier gaid )
	throws CacheException;

	/**
	 * 
	 * @param <T>
	 * @param gai
	 * @param object
	 * @throws CacheException
	 */
	public <T extends Serializable> void create(GlobalArtifactIdentifier gai, T object) 
	throws CacheException;

	/**
	 * 
	 * @param <T>
	 * @param expectedClass
	 * @param gai
	 * @return
	 * @throws CacheException
	 */
	public <T extends Serializable> T get(Class<T> expectedClass, GlobalArtifactIdentifier gai) 
	throws CacheException;
}
