/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.storage.cache;

import gov.va.med.GlobalArtifactIdentifier;
import gov.va.med.RoutingToken;
import gov.va.med.imaging.exchange.business.Study;
import gov.va.med.imaging.exchange.business.vistarad.ExamSite;
import gov.va.med.imaging.exchange.business.vistarad.PatientEnterpriseExams;
import gov.va.med.imaging.storage.cache.Cache;
import gov.va.med.imaging.storage.cache.Instance;
import gov.va.med.imaging.storage.cache.exceptions.CacheException;
import org.apache.log4j.Logger;

/**
 * 
 * 
 */
public class VASourcedCacheDecorator 
extends AbstractCacheDecorator
implements VASourcedCache
{
	private Logger logger = Logger.getLogger(this.getClass());
	private Cache wrappedCache;
	private String metadataRegionName = null;
	private String imageRegionName = null;

	public VASourcedCacheDecorator(
		Cache wrappedCache,
		String metadataRegionName, 
		String imageRegionName)
	{
		if(wrappedCache == null)
			throw new IllegalArgumentException(getClass().getSimpleName() + " was passed a null wrapped cache parameter.");
		if(metadataRegionName == null)
			throw new IllegalArgumentException(getClass().getSimpleName() + " was passed a null metadata region name parameter.");
		if(imageRegionName == null)
			throw new IllegalArgumentException(getClass().getSimpleName() + " was passed a null image region name parameter.");
		
		this.wrappedCache = wrappedCache;
		this.metadataRegionName = metadataRegionName;
		this.imageRegionName = imageRegionName;
		logger.info("DODSourcedCacheDecorator <ctor> backed with cache '" + wrappedCache.getName() + "'.");
	}

	@Override
	protected Cache getWrappedCache()
	{
		return wrappedCache;
	}
	
	@Override
	protected String getImageRegionName()
	{
		return this.imageRegionName;
	}

	@Override
	protected String getMetadataRegionName()
	{
		return this.metadataRegionName;
	}

	public ImmutableInstance createImage(GlobalArtifactIdentifier gaid, String quality, String mimeType) 
	throws CacheException
	{
		String[] groups = AbstractCacheDecorator.createInternalInstanceGroupKeys(gaid, true);
		String instanceKey = AbstractCacheDecorator.createInternalImageKey(gaid, quality, mimeType);
		
		return new ImmutableInstance( createImage(groups, instanceKey) );
	}

	public ImmutableInstance getImage(GlobalArtifactIdentifier gaid, String quality, String mimeType) 
	throws CacheException
	{
		String[] groups = AbstractCacheDecorator.createInternalInstanceGroupKeys(gaid, true);
		String instanceKey = AbstractCacheDecorator.createInternalImageKey(gaid, quality, mimeType);
		
		Instance instance = getImage(groups, instanceKey);
		if(instance == null)
			return null;
		return new ImmutableInstance(instance);
	}

	@Override
	public void createStudy(Study study) 
	throws CacheException
	{
		GlobalArtifactIdentifier gaid = study.getGlobalArtifactIdentifier();
		String[] groups = AbstractCacheDecorator.createInternalInstanceGroupKeys(gaid, true);
		String groupKey = AbstractCacheDecorator.createStudyKey(gaid);
		
		createStudy(groups, groupKey, study);
	}

	public Study getStudy(GlobalArtifactIdentifier gaid) 
	throws CacheException
	{
		String[] groups = AbstractCacheDecorator.createInternalInstanceGroupKeys(gaid, true);
		String groupKey = AbstractCacheDecorator.createStudyKey(gaid);
		
		return getStudy(groups, groupKey);
	}

	private String createPatientEnterpriseExamsKey(String patientICN)
	{
		return "patientEnterpriseExams.xml";
	}
	
	private String createExamSiteKey()
	{
		return "examSite.xml";
	}
	
	private String createPatientPhotoIdKey(String siteNumber)
	{
		return siteNumber + "_photo";
	}

	private String[] createPatientEnterpriseExamsGroupName(String patientICN)
	{
		return new String[] {patientICN};
	}
	
	private String [] createExamSiteGroupName(RoutingToken routingToken, String patientIcn)
	{
		return new String[] {routingToken.getHomeCommunityId(), routingToken.getRepositoryUniqueId(), patientIcn};
	}
	
	private String [] createPatientPhotoIdGroupName(String patientIcn)
	{
		return new String[] {patientIcn};
	}


	public void createPatientEnterpriseExams(PatientEnterpriseExams patientEnterpriseExams)
	throws CacheException
	{
		String[] groups = createPatientEnterpriseExamsGroupName(patientEnterpriseExams.getPatientIcn());
		String groupKey = createPatientEnterpriseExamsKey(patientEnterpriseExams.getPatientIcn());
		
		createPatientEnterpriseExams(groups, groupKey, patientEnterpriseExams);
	}

	public PatientEnterpriseExams getPatientEnterpriseExams(String patientIcn)
	throws CacheException
	{
		String[] groups = createPatientEnterpriseExamsGroupName(patientIcn);
		String groupKey = createPatientEnterpriseExamsKey(patientIcn);
		
		return getPatientEnterpriseExams(groups, groupKey);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.exchange.storage.cache.VASourcedCache#createExamSite(gov.va.med.imaging.exchange.business.vistarad.ExamSite)
	 */
	@Override
	public void createExamSite(RoutingToken routingToken, String patientIcn, ExamSite examSite)
	throws CacheException 
	{
		String[] groups = createExamSiteGroupName(routingToken, patientIcn);
		String groupKey = createExamSiteKey();
		
		createExamSite(groups, groupKey, examSite);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.exchange.storage.cache.VASourcedCache#getExamSite(java.lang.String, java.lang.String)
	 */
	@Override
	public ExamSite getExamSite(RoutingToken routingToken, String patientIcn)
	throws CacheException 
	{
		String[] groups = createExamSiteGroupName(routingToken, patientIcn);
		String groupKey = createExamSiteKey();
		
		return getExamSite(groups, groupKey);
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.exchange.storage.cache.VASourcedCache#createPatientPhotoId(java.lang.String, java.lang.String)
	 */
	@Override
	public ImmutableInstance createPatientPhotoId(String siteNumber,
			String patientIcn) 
	throws CacheException 
	{
		String[] groups = createPatientPhotoIdGroupName(patientIcn);
		String instanceKey = createPatientPhotoIdKey(siteNumber);
		
		return new ImmutableInstance( createImage(groups, instanceKey) );
	}

	/* (non-Javadoc)
	 * @see gov.va.med.imaging.exchange.storage.cache.VASourcedCache#getPatientPhotoId(java.lang.String, java.lang.String)
	 */
	@Override
	public ImmutableInstance getPatientPhotoId(String siteNumber,
			String patientIcn) 
	throws CacheException 
	{
		String[] groups = createPatientPhotoIdGroupName(patientIcn);
		String instanceKey = createPatientPhotoIdKey(siteNumber);
		
		Instance instance = getImage(groups, instanceKey);
		if(instance == null)
			return null;
		return new ImmutableInstance(instance);
	}
}
