/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.router.commands.documents;

import gov.va.med.imaging.DocumentURN;
import gov.va.med.imaging.channels.WriteChannelSiphoningInputStream;
import gov.va.med.imaging.exchange.FileTypeIdentifierStream;
import gov.va.med.imaging.exchange.business.documents.DocumentRetrieveResult;
import gov.va.med.imaging.exchange.enums.VistaImageType;
import gov.va.med.imaging.storage.cache.InstanceWritableByteChannel;
import org.apache.log4j.Logger;

/**
 * A derivation of a DocumentRetrieveResult that will write to a
 * document write channel (usually to a cache instance) as the enclosed
 * stream is read from.
 * 
 * @author zzzzzzbeckec
 *
 */
public class CachingDocumentRetrieveResult
extends DocumentRetrieveResult
{

	/**
	 * @param res
	 * @param documentWriteChannel 
	 * @return
	 */
	public static DocumentRetrieveResult create(DocumentRetrieveResult res, InstanceWritableByteChannel documentWriteChannel)
	{
		WriteChannelSiphoningInputStream siphonedInputStream = 
			new WriteChannelSiphoningInputStream(res.getDocumentStream(), documentWriteChannel);
		
		FileTypeIdentifierStream ftis = new FileTypeIdentifierStream(siphonedInputStream);
		DocumentRetrieveResult siphoned = 
			new DocumentRetrieveResult( res.getDocumentIdentifier(), ftis, res.getVistaImageType() );
		
		return siphoned;
	}

	/**
	 * @param documentUrn
	 * @param documentStream
	 * @param vistaImageType
	 */
	private CachingDocumentRetrieveResult(DocumentURN documentUrn, FileTypeIdentifierStream documentStream, VistaImageType vistaImageType)
	{
		super(documentUrn, documentStream, vistaImageType);
	}
	
	/**
	 * @return
	 */
	public Logger getLogger()
	{
		return Logger.getLogger(this.getClass());
	}

}
