/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.tomcat.vistarealm;


import java.util.HashMap;

import org.apache.catalina.Container;
import org.apache.catalina.ServerFactory;
import org.apache.catalina.Service;
import org.apache.catalina.core.StandardServer;
import org.apache.log4j.Logger;

public class RealmAuthentication
{
    private HashMap<String, VistaAccessVerifyRealm> realms = null;
    
    private static final Logger logger = Logger.getLogger (RealmAuthentication.class);

    public RealmAuthentication()
    {
    	// TODO: Fix this! 
    	// For now, log in to the Realm
		realms = new HashMap<String, VistaAccessVerifyRealm>();
    	StandardServer server = (StandardServer)ServerFactory.getServer();
    	for (Service service : server.findServices())
    	{
    		addVistaRealmsToList(service);
    	}
    }

	public void authenticate(String siteId, String accessCode, String verifyCode)
	{
    	// Try to log in to the configured site...
    	realms.get(siteId).authenticate(accessCode, verifyCode);
	}
	
	private void addVistaRealmsToList(Service service)
	{
		recurseContainers(service.getContainer());
	}

	private void recurseContainers(Container container)
	{
		try
		{
			Container[] childContainers = container.findChildren();
	
			for (Container childContainer : container.findChildren())
			{
				recurseContainers(childContainer);
			}
	
			if (container instanceof org.apache.catalina.core.StandardHost ||
				container instanceof org.apache.catalina.core.StandardEngine ||
				container instanceof org.apache.catalina.core.StandardService)
				return;
			
			if (container.getRealm() instanceof VistaAccessVerifyRealm)
			{
				VistaAccessVerifyRealm realm = (VistaAccessVerifyRealm) container.getRealm();
				if (!realms.containsKey(realm.getSiteNumber()))
				{
					realms.put(realm.getSiteNumber(), realm);
				}
			}
		}
		catch (Throwable t)
		{
			logger.error(t.getMessage(), t);
		}
	}
}
