/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.tomcat.vistarealm.config;

import java.util.Collection;
import java.util.Iterator;

/**
 * This class encapsulates the collection of permissible command line options.
 * It provides some utility functions to find a command line option given an 
 * argument name.
 * 
 * @author ZZZZZZBECKEC
 *
 */
public class CommandLineOptions
{
	private final Collection<? extends CommandLineOption> permissableParameters;
	
	public CommandLineOptions(Collection<? extends CommandLineOption> permissableParameters)
	{
		super();
		this.permissableParameters = permissableParameters;
	}

	public CommandLineOption valueOf(String value)
	{
		return valueOf(value, false);
	}
	
	public CommandLineOption valueOfIgnoreCase(String value)
	{
		return valueOf(value, true);
	}

	public CommandLineOption valueOf(String value, boolean ignoreCase)
	{
		for( Iterator<? extends CommandLineOption> iter=permissableParameters.iterator(); iter.hasNext(); )
		{
			CommandLineOption clo = iter.next();
			
			if( ignoreCase ? clo.getName().equalsIgnoreCase(value) : clo.getName().equals(value) )
				return clo;
		}
		
		return null;
	}
	
	public String getUselessMessage()
	{
		StringBuilder sb = new StringBuilder();
		
		for( Iterator<? extends CommandLineOption> iter=permissableParameters.iterator(); iter.hasNext(); )
		{
			CommandLineOption clo = iter.next();
			
			if(sb.length() > 0)
				sb.append("\n");
				
			if(! clo.isRequired())
				sb.append("[");
			sb.append(clo.getName());
			if( clo.getType() != null )		// a flag has a type of null
				sb.append("=<" + clo.getType().toString() + ">");
			if(! clo.isRequired())
				sb.append("]");
		}
		return sb.toString();
	}

	public Iterator<? extends CommandLineOption> iterator()
	{
		return permissableParameters.iterator();
	}	

}
