/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.tomcat.vistarealm;

import java.util.concurrent.CountDownLatch;
import junit.framework.TestCase;

/**
 * @author zzzzzzbeckec
 *
 */
public class TestRealmDelegationContext
extends TestCase
{
	private final static int THREADS = 100;
	private final static int ITERATIONS = 100;
	
	private CountDownLatch count = new CountDownLatch(THREADS);
	
	public void testThreadedUsage()
	{
		for(int threadIndex=0; threadIndex < THREADS; ++threadIndex)
		{
			WorkerThread wt = new WorkerThread("TEST" + threadIndex);
			wt.setDaemon(false);
			wt.start();
		}
		try{count.await();}catch (InterruptedException x){x.printStackTrace(); fail(x.getMessage());}
	}

	class WorkerThread 
	extends Thread
	{
		/**
		 * @param name
		 */
		public WorkerThread(String name)
		{
			super(name);
		}

		@Override
		public void run()
		{
			System.out.println("Thread '" + this.getName() + "' starting.");
			for(int index=0; index < ITERATIONS; ++index)
			{
				assertNotNull(RealmDelegationContext.getRealmDelegationProperties());
				String value = this.getName() + Integer.toHexString(index);
				RealmDelegationContext.getRealmDelegationProperties().put( this.getName(), value );
				try{Thread.sleep(10);}catch(InterruptedException iX){iX.printStackTrace();fail(iX.getMessage());}
				assertEquals(value, RealmDelegationContext.getRealmDelegationProperties().get(this.getName()) );
				RealmDelegationContext.unsetRealmDelegationContext();
				assertNull(RealmDelegationContext.getRealmDelegationProperties().get(this.getName()));
			}
			count.countDown();
			System.out.println("Thread '" + this.getName() + "' complete.");
		}
	}
}
