/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med;

import gov.va.med.GenericDataGenerator.Mode;
import gov.va.med.NetworkSimulatingInputStream.DELAY_MODE;
import gov.va.med.NetworkSimulatingInputStream.EXCEPTION_MODE;
import gov.va.med.ReferenceMap.ReferenceKey;
import gov.va.med.imaging.exchange.ImageAccessLogEvent;
import gov.va.med.imaging.exchange.business.PassthroughParameter;
import gov.va.med.imaging.exchange.business.PassthroughParameterType;
import gov.va.med.imaging.exchange.enums.PatientSensitivityLevel;

/**
 * @author zzzzzzbeckec
 *
 */
public class PassthroughDataGenerator
extends VistaDataGenerator
{
	public static final PatientSensitivityLevel DEFAULT_PATIENT_SENSITIVITY = PatientSensitivityLevel.NO_ACTION_REQUIRED;
	/**
	 * @param mode
	 */
	public PassthroughDataGenerator(DataGenerationConfiguration configuraton)
	{
		super(configuraton);
	}

	/**
	 * @return
	 */
	public PassthroughParameter createPassthroughParameter(
		InstancePopulation instancePopulation, 
		AggregationPopulation aggregation,
		CompositionPopulation composition,
		ReferenceMap references)
	{
		PassthroughParameter result;
		
		if(instancePopulation == InstancePopulation.NULL)
		{
			result = new PassthroughParameter();
			references.putReference(ReferenceKey.PASSTHROUGH_PARAMETER, result);
		}
		else
		{		
			result = new PassthroughParameter();
			references.putReference(ReferenceKey.PASSTHROUGH_PARAMETER, result);
				
			if(instancePopulation == InstancePopulation.DEFAULT)
				return result;
			
			populateChildren(result, instancePopulation, aggregation, composition, references);
			populateComponentInstances(ImageAccessLogEvent.class, result, instancePopulation, aggregation, composition, references);
		}
		
		return result;
	}
	
	public PassthroughParameterType createPassthroughParameterType(
		InstancePopulation instancePopulation, 
		AggregationPopulation aggregation,
		CompositionPopulation composition,
		ReferenceMap references)
	{
		PassthroughParameterType result = 
			getConfiguration().getMode() == Mode.DEFAULT ? PassthroughParameterType.literal :
			this.selectEnum(PassthroughParameterType.class);
		
		return result;
		
	}
	
	/**
	 * 
	 * @param argv
	 */
	public static void main(String[] argv)
	{
		PassthroughDataGenerator generator = new PassthroughDataGenerator(
			new DataGenerationConfiguration(Mode.RANDOMIZE, EXCEPTION_MODE.RELIABLE, DELAY_MODE.NONE)
		);
		
		PassthroughParameter passthrough = generator.createPassthroughParameter(
			InstancePopulation.FULL, 
			AggregationPopulation.MANY,
			CompositionPopulation.MANY,
			ReferenceMap.createRoot()
		);
		System.out.println("Passthrough Parameter");
		System.out.println(passthrough.toString());
	}
}
