/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.patient.rest.resources;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.PatientIdentifier;
import gov.va.med.imaging.exchange.enums.PatientIdentifierType;
import gov.va.med.imaging.patient.commands.GetPatientSensitivityLevelCommand;
import gov.va.med.imaging.patient.commands.LogSensitivePatientAccessCommand;
import gov.va.med.imaging.patient.commands.PatientSearchCommand;

import java.io.IOException;
import java.io.InputStream;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;

/**
 * @author zzzzzzlouthj
 *
 */
@Path("/patient")
public class PatientResource {

    @GET
    @Path("/getPatientList")
    @Consumes("text/plain")
    public String getPatientList(@QueryParam("searchCriteria") String searchCriteria) 
    	throws MethodException, ConnectionException
	{
    	PatientSearchCommand command = new PatientSearchCommand(searchCriteria, "1.0");
    	return command.execute();
    }
    
    @GET
    @Path("/getPatientSensitivityLevelByDfn")
    @Consumes("text/plain")
    public String getPatientSensitivityLevelByDfn(@QueryParam("patientDfn") String patientDfn) 
    	throws MethodException, ConnectionException
	{
    	GetPatientSensitivityLevelCommand command = new GetPatientSensitivityLevelCommand(
    			new PatientIdentifier(patientDfn, PatientIdentifierType.dfn), 
    			"1.0");
    	
    	return command.execute();
    }
    
    
    @GET
    @Path("/getPatientSensitivityLevelByIcn")
    @Consumes("text/plain")
    public String getPatientSensitivityLevelByIcn(@QueryParam("patientIcn") String patientIcn) 
    	throws MethodException, ConnectionException
	{
    	GetPatientSensitivityLevelCommand command = new GetPatientSensitivityLevelCommand(
    			new PatientIdentifier(patientIcn, PatientIdentifierType.icn), 
    			"1.0");
    
    	return command.execute();
    }
    
    @GET
    @Path("/logSensitivePatientAccessByDfn")
    @Consumes("text/plain")
    public String logSensitivePatientAccessByDfn(@QueryParam("patientDfn") String patientDfn) 
    	throws MethodException, ConnectionException
	{
    	LogSensitivePatientAccessCommand command = new LogSensitivePatientAccessCommand(
    			new PatientIdentifier(patientDfn, PatientIdentifierType.dfn), 
    			"1.0");
    	
    	command.execute();
    	
    	return "";
    }
    
    
    @GET
    @Path("/logSensitivePatientAccessByIcn")
    @Consumes("text/plain")
    public String logSensitivePatientAccessByIcn(@QueryParam("patientIcn") String patientIcn) 
    	throws MethodException, ConnectionException
	{
    	PatientIdentifier patientIdentifier = new PatientIdentifier(patientIcn, PatientIdentifierType.icn);
    	LogSensitivePatientAccessCommand command = new LogSensitivePatientAccessCommand(
    			new PatientIdentifier(patientIcn, PatientIdentifierType.icn), 
    			"1.0");

    	command.execute();
    	return "";
    }
    
}
