/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.siteservice;

import gov.va.med.OID;
import gov.va.med.imaging.artifactsource.ArtifactSource;
import gov.va.med.siteservice.siteprotocol.ProtocolPreference;

/**
 * An internal implementation of ProtocolPreferenceFactory
 * to allow overriding of the preferred preferences by request.
 * This class is used in testing and shouldn't be touched in
 * production. 
 */
class ConstantProtocolPreferenceFactory
implements SiteProtocolPreferenceFactory
{
	private static final long serialVersionUID = 1L;
	private final String[] preferredProtocols;
	
	public ConstantProtocolPreferenceFactory(String[] preferredProtocols)
	{
		this.preferredProtocols = preferredProtocols;
	}
	
	@Override
    public String[] getPreferredProtocols(String siteNumber)
    {
        return preferredProtocols;
    }

	@Override
    public boolean isSiteAlien(String siteNumber)
    {
        return false;
    }

	@Override
    public boolean isSiteLocal(String siteNumber)
    {
        return false;
    }

	@Override
	public boolean isSiteEnabled(String siteNumber)
	{
		return true;
	}

	@Override
	public String[] getPreferredProtocols(ArtifactSource artifactSource)
	{
        return preferredProtocols;
	}

	@Override
	public boolean add(ProtocolPreference ppm)
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.siteservice.SiteProtocolPreferenceFactory#clear()
	 */
	@Override
	public void clear()
	{
	}

	/* (non-Javadoc)
	 * @see gov.va.med.siteservice.SiteProtocolPreferenceFactory#getPreferredProtocols(gov.va.med.OID, java.lang.String)
	 */
	@Override
	public String[] getPreferredProtocols(OID homeCommunityId, String repositoryId)
	{
        return preferredProtocols;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.siteservice.SiteProtocolPreferenceFactory#remove(gov.va.med.OID, java.lang.String)
	 */
	@Override
	public void remove(OID homeCommunityId, String repositoryId)
	{
	}
	
}
