/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.siteservice;

import gov.va.med.RoutingToken;
import gov.va.med.RoutingTokenImpl;
import gov.va.med.imaging.artifactsource.ArtifactSource;
import gov.va.med.imaging.artifactsource.ArtifactSourceImpl;
import java.io.Serializable;
import java.util.Iterator;

/**
 * @author zzzzzzbeckec
 *
 */
public interface ExternalArtifactSources
extends Iterable<RoutingToken>, Serializable
{
	/**
	 * Add a routing map from a routing token to an artifact source.
	 * Note that to resolve the RoutingToken to a ResolvedArtifactSource 
	 * (i.e. so the resolve() call works) still requires that the 
	 * ArtifactSource be mapped to one or more InterfaceURLs.
	 * 
	 * @param routingToken
	 * @param artifactSource
	 */
	public abstract void add(RoutingToken routingToken, ArtifactSourceImpl artifactSource);

	/**
	 * @param routingToken
	 * @param indirectionRoutingToken
	 */
	void addIndirection(RoutingToken routingToken, RoutingTokenImpl indirectionRoutingToken);
	
	/**
	 * Delete the mappings from the RoutingTokens to the artifact source 
	 * and from the ArtifactSource to all InterfaceURLs associated to it.
	 * 
	 * @param routingToken
	 */
	public abstract void remove(RoutingToken routingToken);
	
	public abstract void remove(String homeCommunityId, String repositoryId);
	
	/**
	 * 
	 * @param routingToken
	 * @return
	 */
	public abstract ArtifactSourceLookupResult getEquivalent(RoutingToken routingToken);

	/**
	 * 
	 * @param routingToken
	 * @return
	 */
	public abstract ArtifactSourceLookupResult getIncluding(RoutingToken routingToken);
	
	/**
	 * 
	 * @return
	 */
	public abstract Iterator<RoutingToken> iterator();

	/**
	 * 
	 */
	public abstract void clear();

}
