/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.siteservice.interactive.commands;

import gov.va.med.interactive.Command;
import gov.va.med.interactive.CommandParametersDescription;
import gov.va.med.interactive.CommandProcessor;
import gov.va.med.siteservice.ProtocolServerConfiguration;
import gov.va.med.siteservice.SiteResolutionProviderConfiguration;

/**
 * @author zzzzzzbeckec
 *
 */
public class AddProtocolCommand
extends Command<SiteResolutionProviderConfiguration>
{
	private static final CommandParametersDescription<?>[] commandParametersDescription = 
		new CommandParametersDescription[]
		{
			new CommandParametersDescription<String>("protocol", String.class, true), 
			new CommandParametersDescription<String>("application", String.class, true), 
			new CommandParametersDescription<String>("metadataPath", String.class, true), 
			new CommandParametersDescription<String>("imagePath", String.class, true), 
			new CommandParametersDescription<Boolean>("vista", Boolean.class, false), 
		};
	public static CommandParametersDescription<?>[] getCommandParametersDescription()
	{
		return commandParametersDescription;
	}
	
	/**
	 * @param commandParameterValues
	 */
	public AddProtocolCommand(String[] commandParameterValues)
	{
		super(commandParameterValues);
	}

	/**
	 * @see gov.va.med.interactive.Command#processCommand(gov.va.med.interactive.CommandProcessor, java.lang.Object)
	 */
	@Override
	public void processCommand(CommandProcessor<SiteResolutionProviderConfiguration> processor, SiteResolutionProviderConfiguration config)
	throws Exception
	{
		String protocol = this.getParameterValue("protocol", String.class);
		String application = this.getParameterValue("application", String.class);
		String metadataPath = this.getParameterValue("metadataPath", String.class);
		String imagePath = this.getParameterValue("imagePath", String.class);
		Boolean vista = this.getParameterValue("vista", Boolean.class);
		
		ProtocolServerConfiguration protocolConfiguration = new ProtocolServerConfiguration(
			vista == null ? false : vista.booleanValue(),
			application,
			metadataPath,
			imagePath
		);
		
		config.addProtocolConfiguration(protocol, protocolConfiguration );
	}
}
