/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.siteservice.interactive.commands;

import gov.va.med.OID;
import gov.va.med.interactive.Command;
import gov.va.med.interactive.CommandParametersDescription;
import gov.va.med.interactive.CommandProcessor;
import gov.va.med.siteservice.SiteResolutionProviderConfiguration;
import gov.va.med.siteservice.siteprotocol.ProtocolPreference;

/**
 * @author zzzzzzbeckec
 *
 */
public class AddProtocolPreferenceCommand
extends Command<SiteResolutionProviderConfiguration>
{
	private static final CommandParametersDescription<?>[] commandParametersDescription = 
		new CommandParametersDescription[]
		{
			new CommandParametersDescription<String>("homeCommunityId", String.class, true), 
			new CommandParametersDescription<String>("repositoryId", String.class, true), 
			new CommandParametersDescription<String[]>("protocols", String[].class, true),
			new CommandParametersDescription<Boolean>("local", Boolean.class, false), 
			new CommandParametersDescription<Boolean>("alien", Boolean.class, false),
			new CommandParametersDescription<Boolean>("enabled", Boolean.class, false), 
		};
	public static CommandParametersDescription<?>[] getCommandParametersDescription()
	{
		return commandParametersDescription;
	}
	
	/**
	 * @param commandParameterValues
	 */
	public AddProtocolPreferenceCommand(String[] commandParameterValues)
	{
		super(commandParameterValues);
	}


	/**
	 * @see gov.va.med.interactive.Command#processCommand(gov.va.med.interactive.CommandProcessor, java.lang.Object)
	 */
	@Override
	public void processCommand(CommandProcessor<SiteResolutionProviderConfiguration> processor, SiteResolutionProviderConfiguration config)
	throws Exception
	{
		String homeCommunityId = this.getParameterValue("homeCommunityId", String.class);
		OID homeCommunityOid = OID.create(homeCommunityId);
		String repositoryId = this.getParameterValue("repositoryId", String.class);
		String[] protocols = this.getParameterValue("protocols", String[].class);
		Boolean local = this.getParameterValue("local", Boolean.class);
		Boolean alien = this.getParameterValue("alien", Boolean.class);
		Boolean enabled = this.getParameterValue("enabled", Boolean.class);
		
		ProtocolPreference ppm = ProtocolPreference.create(
			homeCommunityOid, repositoryId, 
			local != null ? local.booleanValue() : false, alien != null ? alien.booleanValue() : false,
			enabled != null ? enabled.booleanValue() : true,
			protocols);
		
		config.getSiteProtocolPreferenceFactory().add(ppm);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		StringBuilder sb = new StringBuilder();
		
		sb.append( "AddProtocolPreferenceCommand [" );
		for(String arg : getCommandParameterValues())
		{
			sb.append(arg);
			sb.append(',');
		}
		sb.append("]");
		
		return sb.toString();
	}
	
}
