/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.siteservice.interactive.commands;

import gov.va.med.imaging.datasource.Provider;
import gov.va.med.interactive.Command;
import gov.va.med.interactive.CommandParametersDescription;
import gov.va.med.interactive.CommandProcessor;
import gov.va.med.siteservice.SiteResolutionProvider;
import gov.va.med.siteservice.SiteResolutionProviderConfiguration;
import gov.va.med.siteservice.SiteServiceConfiguration;

/**
 * @author zzzzzzbeckec
 *
 */
public class ClearCommand
extends Command<SiteResolutionProviderConfiguration>
{
	private static final CommandParametersDescription<?>[] commandParametersDescription = 
		new CommandParametersDescription[]
		{
			new CommandParametersDescription<Boolean>("siteService", Boolean.class, false), 
			new CommandParametersDescription<Boolean>("protocol", Boolean.class, false), 
			new CommandParametersDescription<Boolean>("protocolPreference", Boolean.class, false), 
			new CommandParametersDescription<Boolean>("repository", Boolean.class, false), 
		};
	public static CommandParametersDescription<?>[] getCommandParametersDescription()
	{
		return commandParametersDescription;
	}
	
	/**
	 * @param commandParameterValues
	 */
	public ClearCommand(String[] commandParameterValues)
	{
		super(commandParameterValues);
	}

	/**
	 * @see gov.va.med.interactive.Command#processCommand(gov.va.med.interactive.CommandProcessor, java.lang.Object)
	 */
	@Override
	public void processCommand(CommandProcessor<SiteResolutionProviderConfiguration> processor, SiteResolutionProviderConfiguration config)
	throws Exception
	{
		Boolean siteService = this.getParameterValue("siteService", Boolean.class);
		Boolean protocol = this.getParameterValue("protocol", Boolean.class);
		Boolean protocolPreference = this.getParameterValue("protocolPreference", Boolean.class);
		Boolean repository = this.getParameterValue("repository", Boolean.class);
		
		if(siteService.booleanValue())
			config.setSiteServiceConfiguration(
				SiteServiceConfiguration.createDefault(
					SiteResolutionProvider.getProviderConfiguration().getConfigurationDirectory()
				)
			);
		if(protocol.booleanValue() && config.getProtocolConfiguration() != null)
			config.getProtocolConfiguration().clear();
		if(protocolPreference.booleanValue() && config.getSiteProtocolPreferenceFactory() != null)
			config.getSiteProtocolPreferenceFactory().clear();
		if(repository.booleanValue() && config.getExternalArtifactSources() != null)
			config.getExternalArtifactSources().clear();
		
	}
}
