/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Mar 5, 2012
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  ZZZZZZWERFEJ
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.siteservice.rest.translator;

import java.util.List;
import java.util.Map;

import gov.va.med.imaging.exchange.business.Region;
import gov.va.med.imaging.exchange.business.Site;
import gov.va.med.imaging.exchange.business.SiteConnection;
import gov.va.med.imaging.exchange.siteservice.rest.types.SiteServiceSiteConnectionType;
import gov.va.med.imaging.exchange.siteservice.rest.types.SiteServiceSiteConnectionsType;
import gov.va.med.imaging.exchange.siteservice.rest.types.SiteServiceSiteType;
import gov.va.med.imaging.exchange.siteservice.rest.types.SiteServiceSitesType;
import gov.va.med.imaging.exchange.siteservice.rest.types.SiteServiceVisnType;
import gov.va.med.imaging.exchange.siteservice.rest.types.SiteServiceVisnsType;

/**
 * @author ZZZZZZWERFEJ
 *
 */
public class SiteServiceRestTranslator
{
	
	public static SiteServiceSiteType translate(Site site)
	{
		if(site == null)
			return null;
		
		SiteServiceSiteType result = new SiteServiceSiteType();
		
		result.setVisnNumber(site.getRegionId());
		result.setSiteAbbr(site.getSiteAbbr());
		result.setSiteName(site.getSiteName());
		result.setSiteNumber(site.getSiteNumber());
		result.setSitePatientLookupable(site.isSitePatientLookupable());
		result.setSiteUserAuthenticatable(site.isSiteUserAuthenticatable());
		
		result.setSiteConnections(translate(site.getSiteConnections()));		
		
		return result;
	}
	
	private static SiteServiceSiteConnectionsType translate(Map<String, SiteConnection> siteConnections)
	{
		if(siteConnections == null)
			return null;
		SiteServiceSiteConnectionType [] result = 
			new SiteServiceSiteConnectionType[siteConnections.size()];
		int i = 0;
		for(SiteConnection siteConnection : siteConnections.values())
		{
			result[i] = new SiteServiceSiteConnectionType(siteConnection.getProtocol(), siteConnection.getServer(), siteConnection.getPort());
			i++;
		}		
		
		return new SiteServiceSiteConnectionsType(result);
	}
	
	public static SiteServiceVisnsType translate(List<Region> regions)
	{
		SiteServiceVisnType [] visns = translateRegions(regions);
		return new SiteServiceVisnsType(visns);
	}
	
	
	private static SiteServiceVisnType [] translateRegions(List<Region> regions)
	{
		if(regions == null)
			return null;
		SiteServiceVisnType [] result = new SiteServiceVisnType[regions.size()];
		for(int i = 0; i < regions.size(); i++)
		{
			result[i] = translate(regions.get(i));
		}
		return result;
	}
	
	public static SiteServiceVisnType translate(Region region)
	{
		if(region == null)
			return null;
		
		SiteServiceVisnType result = new SiteServiceVisnType();
		
		result.setVisnName(region.getRegionName());
		result.setVisnNumber(region.getRegionNumber());
		result.setSites(translateSites(region.getSites()));		
		
		return result;
	}
	
	public static SiteServiceSitesType translateSites(List<Site> sites)
	{
		return new SiteServiceSitesType(translateSitesToArray(sites));
	}
	
	private static SiteServiceSiteType[] translateSitesToArray(List<Site> sites)
	{
		if(sites == null)
			return null;
		SiteServiceSiteType []result = new SiteServiceSiteType[sites.size()];
		for(int i = 0; i < sites.size(); i++)
		{
			result[i] = translate(sites.get(i));
		}
		return result;
	}

}
