/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.transactioncontext;

import java.io.*;

import gov.va.med.imaging.tomcat.vistarealm.VistaRealmPrincipal;

/**
 * @author ZZZZZZBECKEC
 * 
 * A wrapper around a byte array, with a fancy name.
 */
public class TransactionContextMemento
{
	private byte[] buffer;
	
	static TransactionContextMemento create(VistaRealmPrincipal principal)
	throws IOException
	{
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(out);
		
		// writeObject iterates through any collection instances in the principal
		// this means that any modifications to the principal while the serialization
		// is taking place will result in a ConcurrentModificationException
		oos.writeObject(principal);
		
		TransactionContextMemento memento = new TransactionContextMemento(out.toByteArray());
		oos.close();
		
		return memento;
	}
	
	static VistaRealmPrincipal create(TransactionContextMemento memento)
	throws IOException, ClassNotFoundException
	{
		ByteArrayInputStream in = new ByteArrayInputStream( memento.getBuffer() );
		ObjectInputStream ois = new ObjectInputStream(in);
		VistaRealmPrincipal principal = (VistaRealmPrincipal)ois.readObject();
		ois.close();
		
		return principal;
	}
	
	TransactionContextMemento(byte[] buffer)
	{
		this.buffer = buffer;
	}
	
	/**
	 * @return the buffer
	 */
	byte[] getBuffer()
	{
		return this.buffer;
	}
}
