/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.url.vista.dates;

public final class Precision implements Comparable {

	public static final int YEAR_INT_VALUE = 0;
	public static final int MONTH_INT_VALUE = 1;
	public static final int DATE_INT_VALUE = 2;
	public static final int HOUR_INT_VALUE = 3;
	public static final int MINUTE_INT_VALUE = 4;
	public static final int SECOND_INT_VALUE = 5;
	public static final int MILLISECOND_INT_VALUE = 6;

	public static final Precision YEAR = new Precision("year", YEAR_INT_VALUE);
	public static final Precision MONTH = new Precision("month", MONTH_INT_VALUE);
	public static final Precision DATE = new Precision("date", DATE_INT_VALUE);
	public static final Precision HOUR = new Precision("hour", HOUR_INT_VALUE);
	public static final Precision MINUTE = new Precision("minute", MINUTE_INT_VALUE);
	public static final Precision SECOND = new Precision("second", SECOND_INT_VALUE);
	public static final Precision MILLISECOND = new Precision("millisecond", MILLISECOND_INT_VALUE);

	private String value;
	private int intValue;

	private Precision(String value, int intValue) {
		this.value = value;
		this.intValue = intValue;
	}

	public boolean equals(Object obj) {
		Precision p = (Precision) obj;
		return this.intValue == p.intValue;
	}

	public int hashCode() {
		return value.hashCode();
	}

	public String toString() {
		return value;
	}

	public int compareTo(Object o) {
		Precision p = (Precision) o;
		if (this.intValue == p.intValue)
			return 0;
		else if (this.intValue < p.intValue)
			return -1;
		else
			return 1;
	}

	public boolean lessThan(Precision p) {
		return compareTo(p) < 0;
	}

	public boolean greaterThan(Precision p) {
		return compareTo(p) > 0;
	}

	public static Precision lesser(Precision p1, Precision p2) {
		if (p2.lessThan(p1))
			return p2;
		else
			return p1;
	}

	public static Precision greater(Precision p1, Precision p2) {
		if (p2.greaterThan(p1))
			return p2;
		else
			return p1;
	}
}
