/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.url.vista;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import junit.framework.TestCase;

public class TestVistaConnection 
extends AbstractVistaConnectionTest
{
	private VistaConnection vistaConnection;
	
	@Override
    protected void setUp() 
	throws Exception
    {
	    super.setUp();
	    
	    URL vistaUrl = new URL("vista://localhost:9300");
	    vistaUrl = new URL("vista://1.2.3.4:9200"); // station 200
	    vistaConnection = new VistaConnection(vistaUrl);
    }

	
	private VistaConnection getVistaConnection()
    {
    	return vistaConnection;
    }
	
	public void testConnect()
	{
		try
        {
			boolean capri = false;
			long startTime = System.currentTimeMillis();
			logger.info("start time: " + startTime);
			getVistaConnection().connect();
			System.out.println("Connect took '" + (System.currentTimeMillis() - startTime) + "' ms.");
	        logger.info("Connect complete");
	       
	        VistaQuery vq = null;
	        if(capri)
	        {
	        	
	        	vq = new VistaQuery("XUS SIGNON SETUP");
	        	//vq.addParameter(VistaQuery.LITERAL, "2");
	        	//vq.addParameter(VistaQuery.LITERAL, "2");
	        	getVistaConnection().call(vq);
	        	
	        	String param = "-31^DVBA_" + 
				"^SSN1" + 
				"^IMAGPROVIDERONETWOSIX,ONETWOSIX" + 
				"^Salt Lake City" + 
				"^660" +  
				"^126" + 
				"^No Phone";
	        	
	        	
	        	vq = new VistaQuery("XUS SIGNON SETUP");
	        	vq.addParameter(VistaQuery.LITERAL, param);
	        	getVistaConnection().call(vq);
	        	
	        }
	        else
	        {	        		        	
	        	vq = new VistaQuery("XUS SIGNON SETUP");
	        	//vq.addParameter(VistaQuery.LITERAL, "");
	        	//vq.addParameter(VistaQuery.LITERAL, "2");
	        	getVistaConnection().call(vq);
	        	
	        	
	        	vq.clear();
		        vq.setRpcName("XUS AV CODE");
		        //vq.addEncryptedParameter(VistaQuery.LITERAL, "username5;password6");
		        vq.addEncryptedParameter(VistaQuery.LITERAL, "username3;password5");
		        //vq.addEncryptedParameter(VistaQuery.LITERAL, "vix1234;washington.456");
		        String response = getVistaConnection().call(vq);
		        logger.info("'" + vq.getRpcName() + "' result: " + response);
	        }	        	       
	        
	        vq.clear();
	        long endTime = System.currentTimeMillis();
	        /*
	        vq.setRpcName("XWB CREATE CONTEXT");
	        vq.addEncryptedParameter(VistaQuery.LITERAL, "MAG WINDOWS");
	        String response = getVistaConnection().call(vq);
	        logger.info("'" + vq.getRpcName() + "' result: " + response);
	        
	        
	        vq.clear();
	        vq.setRpcName("MAGG PAT INFO");
	        vq.addParameter(VistaQuery.LITERAL, "711^^0");
	        response = getVistaConnection().call(vq);
	        logger.info("'" + vq.getRpcName() + "' result: " + response);	        	       	        
	        */
	        System.out.println("Full login took '" + (endTime - startTime) + "' ms.");
	        
	        
        } 
		catch (IOException e)
        {
	        e.printStackTrace();
	        fail(e.getMessage());
        }
		catch(Exception e)
		{
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// we're not testing disconnect so don't broadcast errors
        getVistaConnection().disconnectImmediately();
	}

	/*
	public void testDisconnect()
	{
		try
        {
	        getVistaConnection().connect();
        } 
		catch (IOException e)
        {
	        e.printStackTrace();
	        TestCase.fail(e.getMessage());
        }
	    assertTrue( getVistaConnection().isConnected() );
		
	    getVistaConnection().disconnect();
	    assertFalse( getVistaConnection().isConnected() );
	}*/
	
	/**
	 * Just repeatedly connect and disconnect
	 */
	/*
	public void testRepetitiveConnectDisconnect()
	{
		for(int n=0; n < 10; ++n)
		{
			try
	        {
		        getVistaConnection().connect();
	        } 
			catch (IOException e)
	        {
		        e.printStackTrace();
		        TestCase.fail(e.getMessage());
	        }
			
		    getVistaConnection().disconnect();
		}
		
	}
	
	public void testGetInputStream()
	{
		try
        {
	        getVistaConnection().connect();
	        InputStream inStream = getVistaConnection().getInputStream();
	        assertNotNull("Input stream is null and should not be.", inStream);
	        
	        getVistaConnection().disconnect();
        } 
		catch (IOException e)
        {
	        e.printStackTrace();
	        TestCase.fail(e.getMessage());
        }
	}

	public void testGetOutputStream()
	{
		try
        {
	        getVistaConnection().connect();
	        OutputStream outStream = getVistaConnection().getOutputStream();
	        assertNotNull("Output stream is null and should not be.", outStream);
	        
	        getVistaConnection().disconnect();
        } 
		catch (IOException e)
        {
	        e.printStackTrace();
	        TestCase.fail(e.getMessage());
        }
	}*/

	@Override
    protected void tearDown() 
	throws Exception
    {
	    super.tearDown();
    }
}
