/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.protocol.vista;

import gov.va.med.imaging.url.vista.StringUtils;

public class DicomTranslatorUtility
{
	public static final char CRCHAR = '\r';
	public static final char NEW_LINECHAR = '\n';
	public static final char TABCHAR = '\t';
	public static final char FORM_FEEDCHAR = '\f';
	public static final char BACKCHAR = '\b';
	public static final char DOUBLE_QUOTECHAR = '\"';
	public static final char SINGLE_QUOTECHAR = '\'';
	public static final char SLASHCHAR = '\\';
	
	public static String[] createResultsArray(String returnValue)
	{
		return createResultsArrayStrippingHeaderLines(returnValue, 0);
	}
	
	public static String[] createResultsArrayStrippingHeaderLines(String returnValue, int numHeaderLines)
	{
		// Split the output from the RPC into a string array, and strip off the first two
		// header lines.
		String[] allLines = StringUtils.Split(returnValue, StringUtils.NEW_LINE);
		String[] dataLines = new String[allLines.length - numHeaderLines];
		System.arraycopy(allLines, numHeaderLines, dataLines, 0, dataLines.length);
		
		// trim whitespace, including CR and/or LF characters
		if(dataLines != null)
		{
			for(int index=0; index<dataLines.length; ++index)
			{
				dataLines[index] = dataLines[index].trim();
			}
		}		
		return dataLines;
	}

	public static String displayEncodedChars(String encodedString)
	{
		StringBuffer decodedString = new StringBuffer();
		char[] chars = encodedString.toCharArray();

		for (int x = 0; x < chars.length; x++)
		{
			if (chars[x] == TABCHAR)
			{
				decodedString.append("\\t");
			}
			else if (chars[x] == CRCHAR)
			{
				decodedString.append("\\r");
			}
			else if (chars[x] == NEW_LINECHAR)
			{
				decodedString.append("\\n");
			}
			else if (chars[x] == FORM_FEEDCHAR)
			{
				decodedString.append("\\f");
			}
			else if (chars[x] == BACKCHAR)
			{
				decodedString.append("\\b");
			}
			else if (chars[x] == DOUBLE_QUOTECHAR)
			{
				decodedString.append("\\\"");
			}
			else if (chars[x] == SINGLE_QUOTECHAR)
			{
				decodedString.append("\\'");
			}
			else if (chars[x] == SLASHCHAR)
			{
				decodedString.append("\\\\");
			}
			else
			{
				decodedString.append(chars[x]);
			}
		}
		return decodedString.toString();
	}

    /**
     * Method intVal Calculates integer value based on first characters in a
     * string: "123abcde" yields 123
     *
     * @param str
     *            input string
     * @return integer value of start of string
     */
    public static int intVal (String str)
    {
      int charpos = 0;
      int ready = 0;
      char letter;
      int value = 0;
      int neg = 1;

      while ((ready < 1) && (charpos < str.length ()))
      {
        letter = str.charAt (charpos++);
        switch (letter)
        {
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
          value = value * 10 + letter - '0';
          break;
        case '-':
          neg = -neg;
          break;
        default:
          ready = 1;
          break;
        }
      }
      return value * neg;
    }


}
