/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.vistaimagingdatasource;

import gov.va.med.GlobalArtifactIdentifier;
import gov.va.med.GlobalArtifactIdentifierFactory;
import gov.va.med.exceptions.GlobalArtifactIdentifierFormatException;
import gov.va.med.imaging.DocumentURN;
import gov.va.med.imaging.ImageURN;
import gov.va.med.imaging.artifactsource.ResolvedArtifactSource;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.datasource.DocumentDataSourceSpi;
import gov.va.med.imaging.exceptions.URNFormatException;
import gov.va.med.imaging.exchange.business.ImageFormatQuality;
import gov.va.med.imaging.exchange.business.ImageFormatQualityList;
import gov.va.med.imaging.exchange.business.ImageStreamResponse;
import gov.va.med.imaging.exchange.enums.ImageFormat;
import gov.va.med.imaging.exchange.enums.ImageQuality;

/**
 * This is an implementation of the DocumentDataSourceSpi with VistA as the backing
 * data storage.  This class is a derivation of the StudyGraphDataSourceSpi over VistA
 * with additional function to translate into Document semantics.
 * 
 * @author zzzzzzbeckec
 *
 */
public class VistaImagingDocumentDataSourceService
extends VistaImageDataSourceService
implements DocumentDataSourceSpi
{
 

	/**
	 * @param resolvedArtifactSource
	 * @param protocol
	 */
	public VistaImagingDocumentDataSourceService(ResolvedArtifactSource resolvedArtifactSource, String protocol)
	{
		super(resolvedArtifactSource, protocol);
	}

	@Override
	public ImageStreamResponse getDocument(DocumentURN documentUrn)
	throws MethodException, ConnectionException
	{
		ImageURN imageUrn;
		try
		{
			imageUrn = ImageURN.create(documentUrn);
		}
		catch (URNFormatException x)
		{
			throw new MethodException("Unexpected exception converting document URN '" + documentUrn.toString() + "' to image URN.", x);
		}
		ImageFormatQualityList docFormat = new ImageFormatQualityList();
		docFormat.add(new ImageFormatQuality(ImageFormat.ORIGINAL, ImageQuality.DIAGNOSTICUNCOMPRESSED));
		return this.getImage(imageUrn, docFormat);
	}
	
	@Override
	public ImageStreamResponse getDocument(GlobalArtifactIdentifier gai)
	throws MethodException, ConnectionException
	{
		ImageURN imageUrn = null;
		try
		{
			imageUrn = GlobalArtifactIdentifierFactory.create(
				gai.getHomeCommunityId(), 
				gai.getRepositoryUniqueId(), 
				gai.getDocumentUniqueId(), 
				DocumentURN.class);
			//imageUrn = URNFactory.create(documentId, ImageURN.class);
		}
		//catch (URNFormatException x)
		//{
		//	throw new MethodException(
		//			"Unexpected exception converting document URN '" + 
		//			(imageUrn == null ? "<null>" : imageUrn.toString()) + 
		//			"' to image URN.", x);
		//}
		catch (GlobalArtifactIdentifierFormatException x)
		{
			throw new MethodException(
				"Unexpected exception building image URN from '" + 
				gai.toString() + "'.", x);
		}
		ImageFormatQualityList docFormat = new ImageFormatQualityList();
		docFormat.add(new ImageFormatQuality(ImageFormat.ORIGINAL, ImageQuality.DIAGNOSTICUNCOMPRESSED));
		return this.getImage(imageUrn, docFormat);
	}
}
