/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Nov, 2009
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  zzzzzzlouthj
  Description: DICOM Study cache manager. Maintains the cache of study instances
  			   and expires old studies after 15 minutes. 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.vistaimagingdatasource.dicom.importer;

import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.Patient;
import gov.va.med.imaging.exchange.business.Patient.PatientSex;
import gov.va.med.imaging.exchange.business.dicom.importer.Order;
import gov.va.med.imaging.exchange.business.dicom.importer.OrderingLocation;
import gov.va.med.imaging.exchange.business.dicom.importer.OrderingProvider;
import gov.va.med.imaging.exchange.business.dicom.importer.Procedure;
import gov.va.med.imaging.exchange.business.dicom.importer.ProcedureModifier;
import gov.va.med.imaging.exchange.business.dicom.importer.Report;
import gov.va.med.imaging.exchange.business.dicom.importer.ReportParameters;
import gov.va.med.imaging.exchange.business.dicom.importer.Study;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class MockVistaImagingDicomImporterDataSourceService 
{

	public List<Patient> getPatientsBySearchCriteria(String searchCriteria)  throws MethodException, ConnectionException 
	{
		List<Patient> patients = new ArrayList<Patient>();
		
		patients.add(createPatient(searchCriteria, 1));
		patients.add(createPatient(searchCriteria, 2));
		patients.add(createPatient(searchCriteria, 3));
		patients.add(createPatient(searchCriteria, 4));
		patients.add(createPatient(searchCriteria, 5));

		return patients;

	}
	
    private Patient createPatient(String searchCriteria, int i)
    {
        String ctr = Integer.toString(i);

        Patient patient = Patient.create(
        		searchCriteria + "_Patient_" + i, 
        		ctr + ctr + ctr + "-" + ctr + ctr + "-" + ctr + ctr + ctr + ctr, 
        		"Veteran", 
        		PatientSex.Male, 
        		new Date("01/02/1950"),
        		ctr + ctr + ctr + "-" + ctr + ctr + "-" + ctr + ctr + ctr + ctr, 
        		"dfn_" + ctr,
        		false);
        
        return patient;
    }

	public Study getStudyImportStatus(Study study) throws MethodException, ConnectionException 
	{
		if (study.getIdInMediaBundle() % 3 == 0)
		{
			study.setImportStatus("Partial");
		}
		return study;
	}

	public List<Order> getOrderListForPatient(String patientId, String siteId) throws MethodException, ConnectionException 
	{
        List<Order> orders = new ArrayList<Order>();
        
        Order order = new Order();
        order.setId(1);
        order.setLocation("SLC/TD-RAD");
        order.setSpecialty("RAD");
        order.setOrderDate("10/9/2010");
        order.setAccessionNumber("111222333444");
        order.setExamStatus("Pending");
        order.setDescription("DX Hand");
     	orders.add(order);
     
        order = new Order();
        order.setId(2);
        order.setLocation("SLC/TD-RAD");
        order.setSpecialty("RAD");
        order.setOrderDate("10/11/2010");
        order.setAccessionNumber("222333444555");
        order.setExamStatus("Pending");
        order.setDescription("CR Chest");
     	orders.add(order);
     
        order = new Order();
        order.setId(3);
        order.setLocation("SLC");
        order.setSpecialty("CON");
        order.setOrderDate("10/12/2010");
        order.setAccessionNumber("333444555666");
        order.setExamStatus("Pending");
        order.setDescription("CONSULTATION");
     	orders.add(order);

     	return orders;
	}

	public List<OrderingLocation> getOrderingLocationList() throws MethodException, ConnectionException 
	{
		List<OrderingLocation> orderingLocations = new ArrayList<OrderingLocation>();
		orderingLocations.add(new OrderingLocation(1, "Location 1"));
		orderingLocations.add(new OrderingLocation(2, "Location 2"));
		orderingLocations.add(new OrderingLocation(3, "Location 3"));
		orderingLocations.add(new OrderingLocation(4, "Location 4"));
		orderingLocations.add(new OrderingLocation(5, "Location 5"));
		return orderingLocations;
	}

	public List<OrderingProvider> getOrderingProviderList() throws MethodException, ConnectionException 
	{
		List<OrderingProvider> orderingProviders = new ArrayList<OrderingProvider>();
		orderingProviders.add(new OrderingProvider(1, "Ordering Provider 1"));
		orderingProviders.add(new OrderingProvider(2, "Ordering Provider 2"));
		orderingProviders.add(new OrderingProvider(3, "Ordering Provider 3"));
		orderingProviders.add(new OrderingProvider(4, "Ordering Provider 4"));
		orderingProviders.add(new OrderingProvider(5, "Ordering Provider 5"));
		return orderingProviders;
	}

	public List<Procedure> getProcedureList() throws MethodException, ConnectionException 
	{
		List<Procedure> procedures = new ArrayList<Procedure>();
		
		Procedure procedure = new Procedure("Procedure 1", 1, 1, 1);
		procedure.getProcedureModifiers().add(new ProcedureModifier(0,1,""));
		procedure.getProcedureModifiers().add(new ProcedureModifier(1,1,"P1, M1"));
		procedure.getProcedureModifiers().add(new ProcedureModifier(2,1,"P1, M2"));
		procedures.add(procedure);
		
		procedure = new Procedure("Procedure 2", 2, 2, 1);
		procedure.getProcedureModifiers().add(new ProcedureModifier(0,2,""));
		procedure.getProcedureModifiers().add(new ProcedureModifier(1,2,"P2, M1"));
		procedure.getProcedureModifiers().add(new ProcedureModifier(2,2,"P2, M2"));
		procedures.add(procedure);

		procedure = new Procedure("Procedure 3", 3, 3, 1);
		procedure.getProcedureModifiers().add(new ProcedureModifier(0,3,""));
		procedure.getProcedureModifiers().add(new ProcedureModifier(1,3,"P3, M1"));
		procedure.getProcedureModifiers().add(new ProcedureModifier(2,3,"P3, M2"));
		procedures.add(procedure);

		procedure = new Procedure("Procedure 4", 4, 4, 1);
		procedures.add(procedure);

		return procedures;
	}
	
	public Report getImporterReport(ReportParameters reportParameters)
	{
		Report report = new Report();
		report.setReportText("blah from the server...");
		return report;
	}


}
