/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.vistaimagingdatasource.dicom.storage;

import gov.va.med.imaging.exchange.business.dicom.InstanceFile;
import gov.va.med.imaging.exchange.business.dicom.PatientRef;
import gov.va.med.imaging.exchange.business.dicom.PatientStudyInfo;
import gov.va.med.imaging.exchange.business.dicom.PatientStudyLookupResults;
import gov.va.med.imaging.exchange.business.dicom.ProcedureRef;
import gov.va.med.imaging.exchange.business.dicom.SOPInstance;
import gov.va.med.imaging.exchange.business.dicom.Series;
import gov.va.med.imaging.exchange.business.dicom.Study;
import gov.va.med.imaging.exchange.business.dicom.UIDCheckResult;
import gov.va.med.imaging.protocol.vista.DicomTranslatorUtility;
import gov.va.med.imaging.url.vista.StringUtils;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

public class VistaImagingDicomStorageUtility
{
	private static Logger logger = Logger.getLogger(VistaImagingDicomStorageUtility.class);


	public static UIDCheckResult translateUIDCheckResults(String returnValue, String originalUID, String fieldSeparator)
	{
		UIDCheckResult result = null;

		if (returnValue == null) 
		{
			result = UIDCheckResult.createResultFatalError();
		}
		else
		{
			returnValue = returnValue.trim();

			if (returnValue.equals("0"))
			{
				// normal case
				result = UIDCheckResult.createResultNoDuplicateFound(originalUID);
			}
			else if (returnValue.startsWith("1"))
			{
				// Duplicate found: Parse out the new value
				String cleanUID = StringUtils.Split(returnValue, fieldSeparator)[2];
				result = UIDCheckResult.createResultDuplicateFound(originalUID, cleanUID);
			}
			else if (returnValue.startsWith("-2") )
			{
				// illegal UID found: Parse out the new value
				String cleanUID = StringUtils.Split(returnValue, fieldSeparator)[2];
				result = UIDCheckResult.createResultIllegalUIDFound(originalUID, cleanUID);
			}
			else if (returnValue.startsWith("2"))
			{
				result = UIDCheckResult.createResultSOPInstanceResendFound(originalUID);
			}
			else // -1 (fatal Error) received
			{
				result = UIDCheckResult.createResultFatalError();
			}
		}
		
		return result;
	}


	

}
