/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.xstream;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.thoughtworks.xstream.converters.ConversionException;
import com.thoughtworks.xstream.converters.SingleValueConverter;

public class VistaDateConverter implements SingleValueConverter {
	private static final String VISTA_DATE_FORMAT = "yyyyMMdd.hhmmss";

	public boolean canConvert(Class clazz) {
		return Date.class.isAssignableFrom(clazz);
	}

	public static Date parseDate(String value) {
		DateFormat formatter = new SimpleDateFormat(VISTA_DATE_FORMAT);
		Date date = null;
		if (value != null && !value.equals("")) {
			try {
				double dateTimeNumber = Double.parseDouble(value);
				DecimalFormat paddingFormatter = new DecimalFormat(
						"00000000.000000");
				value = paddingFormatter.format(dateTimeNumber);
				date = formatter.parse(value);
			} catch (ParseException e) {
				String msg = "can't parse \"" + value + "\" into a Date";
				throw new ConversionException(msg, e);
			}
		}
		return date;
	}

	public static String dateToString(Date date) {
		DateFormat formatter = new SimpleDateFormat(VISTA_DATE_FORMAT);
		return formatter.format(date);
	}

	public Object fromString(String value) {
		return parseDate(value);
	}

	public String toString(Object obj) {
		return dateToString((Date) obj);
	}
}
