/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.vistadatasource;

import gov.va.med.RoutingToken;
import gov.va.med.imaging.artifactsource.ResolvedArtifactSource;
import gov.va.med.imaging.business.LocalizedSiteTestImpl;
import gov.va.med.imaging.core.interfaces.LocalizedSite;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.datasource.DataSourceProvider;
import gov.va.med.imaging.datasource.Provider;
import gov.va.med.imaging.datasource.StudyGraphDataSourceSpi;
import gov.va.med.imaging.exchange.business.SiteImpl;
import gov.va.med.imaging.exchange.business.StudyFilter;
import gov.va.med.imaging.exchange.enums.StudyLoadLevel;
import gov.va.med.imaging.protocol.vista.AbstractVistaConnectionTest;

import java.net.MalformedURLException;
import java.net.URL;

public class StudyGraphTest 
extends AbstractVistaConnectionTest
{
	protected StudyGraphDataSourceSpi studyGraphDataSource;
	protected ResolvedArtifactSource source;
	

	@Override
    protected void setUp() throws Exception
    {
	    super.setUp();
	    
		System.out.println("java.protocol.handler.pkgs is '" + System.getProperty("java.protocol.handler.pkgs") + "'.");

		source = getTestResolvedSite();
		
		studyGraphDataSource = getProvider().createStudyGraphDataSource(source, "vistaimaging");
    }

	public StudyGraphDataSourceSpi getStudyGraphDataSource()
    {
    	return studyGraphDataSource;
    }
	
	public RoutingToken getRoutingToken()
	{
		return source.getArtifactSource().createRoutingToken();
	}

	public void testAvailability() 
	throws MalformedURLException
	{
		assertNotNull( getStudyGraphDataSource() );
	}
	
	private final static int PATIENT_ICN_INDEX = 0;
	private final static int PATIENT_CHECKSUM_INDEX = 1;
	
	private String[][] testPatientIcns = 
	{
		{ "1006184063", "088473" },
		{ "1008861107", "475740" },
		{ "1006167324", "385420" },
		{ "1006170647", "052871" },
		{ "1006170580", "294705" },
		{ "1006151329", "503966" },
		{ "1006152719", "948936" },
		{ "1006147276", "569483" },
		{ "1006147126", "079083" },
		{ "1006145121", "631417" }    			
	};
	
	public void testGetPatientStudiesStringFilter()
	{
		StudyFilter filter = new StudyFilter();
		
		for(String[] testPatientIcn : testPatientIcns)
			testGetPatientStudies(testPatientIcn[PATIENT_ICN_INDEX], filter);
	}	
	
	public void testGetPatientStudies(String patientIcn, StudyFilter filter)
	{
		try
        {
	        getStudyGraphDataSource().getPatientStudies(getRoutingToken(), patientIcn, filter, StudyLoadLevel.FULL);
        } 
		catch (UnsupportedOperationException e)
        {
	        e.printStackTrace();
	        fail(e.getMessage());
        } 
		catch (MethodException e)
        {
	        e.printStackTrace();
	        fail(e.getMessage());
        } 
		catch (ConnectionException e)
        {
	        e.printStackTrace();
	        fail(e.getMessage());
        }
	}
}
