/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.vistaimagingdatasource;

import org.junit.Assert;
import org.junit.Test;

import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.DurableQueue;
import gov.va.med.imaging.exchange.business.DurableQueueMessage;

import java.util.List;

public class DurableQueueDAOTests{
	@Test
	public void testTranslateFindAll(){
		StringBuilder builder = new StringBuilder();
		builder.append("0^^4\r\n");
		builder.append("IEN^NAME^QUEUE TYPE^ACTIVE^NUM RETRIES^RETRY DELAY IN SECONDS^TRIGGER DELAY IN SECONDS\r\n");
		builder.append("3^ASYNC_ICON_QUEUE^I^1^3^1200^1200\r\n");
		builder.append("2^Async Storage Request Error Queue^E^1^5^1200^1200\r\n");
		builder.append("1^Async Storage Request Queue^S^1^5^1200^1200\r\n");
		builder.append("4^E-Mail Queue^M^1^5^1200^1200");
		DurableQueueDAO dao = new DurableQueueDAO(null);
		List<DurableQueue> queues = dao.translateFindAll(builder.toString());
		Assert.assertTrue(queues.size() == 4);
		for (int i=0; i < queues.size(); i++){
			Assert.assertTrue(queues.get(i).getId() > 0);
			Assert.assertTrue(queues.get(i).getName() != null);
			Assert.assertTrue(queues.get(i).getName().length() > 0);
			Assert.assertTrue(queues.get(i).getType() != null);
			Assert.assertTrue(queues.get(i).getType().length() > 0);
			Assert.assertTrue(queues.get(i).getIsActive());
			Assert.assertTrue(queues.get(i).getRetryDelayInSeconds() == 1200);
			Assert.assertTrue(queues.get(i).getTriggerDelayInSeconds() == 1200);
		}
	}
	
/*	
	@Test
	public void testTranslateDurableQueueMessage(){
		StringBuilder builder = new StringBuilder();
		builder.append("0^^5\r\n");
		builder.append("<QUEUEMESSAGE\r\n");
		builder.append("PK=\"5\"\r\n");
		builder.append("QUEUE=\"4\"\r\n");
		builder.append("PRIORITY=\"50\"\r\n");
		builder.append("ENQUEUEDDATETIME=\"20120206.133321\"\r\n");
		builder.append("EARLIESTDELIVERYDATETIME=\"\"\r\n");
		builder.append("EXPIRATIONDATETIME=\"\"\r\n");
		builder.append("MESSAGEGROUPID=\"host\"\r\n");
		builder.append("MESSAGE=\"email3^DICOM Storage SCP Replaced DUP/Illegal UID^P116_SCU -&gt; Warning=Replaced: DUP StdIUID; DUP SerIUID; DUP SOPIUID;=1.2.840.113754.1.4.660.6879793.8749.1.20612.361^0\" >\r\n");
		builder.append("</QUEUEMESSAGE >");
		DurableQueueDAO dao = new DurableQueueDAO(null);
		try {
			DurableQueueMessage message = dao.translateDurableQueueMessage(builder.toString());
			Assert.assertEquals(5, message.getId());
			Assert.assertEquals(4, message.getQueueId());
			Assert.assertEquals(50, message.getPriority());
			//Assert.assertEquals(20120206.133321, message.getExpirationDateTime());
			Assert.assertEquals("", message.getMinDeliveryDateTime());
			Assert.assertEquals("host", message.getMessageGroupId());
			Assert.assertEquals("email3^DICOM Storage SCP Replaced DUP/Illegal UID^P116_SCU ->; Warning=Replaced: DUP StdIUID; DUP SerIUID; DUP SOPIUID;=1.2.840.113754.1.4.660.6879793.8749.1.20612.361^0", message.getMessage());
		} catch (MethodException e) {
			e.printStackTrace();
			Assert.fail();
		}
	}
*/	
/*	
	@Test
	public void testTranslateDurableQueueMessage() throws Exception{
		StringBuilder builder = new StringBuilder();
		builder.append("0^^2\r\n");
		builder.append("This is a test message.");
		DurableQueueDAO dao = new DurableQueueDAO(null);
		DurableQueueMessage qmessage = dao.translateDurableQueueMessage(builder.toString());
		Assert.assertTrue(qmessage.getId() == 2);
		Assert.assertTrue(qmessage.getMessage().length() > 0);
	}
*/
}
