/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Nov, 2009
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  zzzzzzlouthj
  Description: DICOM Study cache manager. Maintains the cache of study instances
  			   and expires old studies after 15 minutes. 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */

package gov.va.med.imaging.vistaimagingdatasource.worklist;

import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.WorkItemCounts;
import gov.va.med.imaging.exchange.business.storage.Provider;
import gov.va.med.imaging.exchange.business.storage.exceptions.CreationException;
import gov.va.med.imaging.exchange.business.storage.exceptions.RetrievalException;
import gov.va.med.imaging.exchange.business.storage.exceptions.UpdateException;

import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

public class WorkItemDAOTests
{
	private WorkItemDAO dao = new WorkItemDAO();

	@Test
	public void testTranslateGetWorkItemCounts() throws MethodException
	{

		StringBuilder builder = new StringBuilder();
		builder.append("0`4\r\n");
		builder.append("SUBTYPE`STATUS`COUNT\r\n");
		builder.append("DicomCorrect`New`78\r\n");
		builder.append("DicomCorrect`Staged`7\r\n");
		builder.append("DirectImport`FailedImport`1\r\n");
		builder.append("DirectImport`New`4");

		WorkItemCounts counts = dao.translateGetWorkItemCounts(builder.toString());

		Assert.assertEquals(78, counts.getCountForSubtypeAndStatus("DicomCorrect", "New"));
		Assert.assertEquals(7, counts.getCountForSubtypeAndStatus("DicomCorrect", "Staged"));
		Assert.assertEquals(1, counts.getCountForSubtypeAndStatus("DirectImport", "FailedImport"));
		Assert.assertEquals(4, counts.getCountForSubtypeAndStatus("DirectImport", "New"));
		Assert.assertEquals(0, counts.getCountForSubtypeAndStatus("DirectImport", "ReadyForImport"));
		Assert.assertEquals(0, counts.getCountForSubtypeAndStatus("NetworkImport", "New"));

	}

}