/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.cache;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * @author ZZZZZZBECKEC
 *
 */
public class CacheVO
extends AbstractNamedVO
implements Serializable
{
	private static final long	serialVersionUID	= 1L;
	
	private CacheMetadata metadata = null;
	private SortedSet<RegionVO> regions = new TreeSet<RegionVO>();

	public CacheVO(){}		// required for GWT IsSerializable
	
	public CacheVO(String name, CacheMetadata metadata)
	{
		super(name);
		this.metadata = metadata;
	}

	@Override
	public CacheItemPath getPath()
	{
		return new CacheItemPath(getName());
	}

	public CacheMetadata getMetadata() {
		return metadata;
	}

	public void setMetadata(CacheMetadata metadata) {
		this.metadata = metadata;
	}

	public boolean add(RegionVO region){return regions.add(region);}
	public boolean addAll(Collection<RegionVO> regionCollection){return regions.addAll(regionCollection);}
	
	@Override
	public boolean removeChild(AbstractNamedVO child){return remove((RegionVO)child);}
	
	public boolean remove(RegionVO region){return regions.remove(region);}
	public boolean removeAll(Collection<RegionVO> regionCollection){return regions.removeAll(regionCollection);}
	
	public Iterator<RegionVO> iterator(){return regions.iterator();}
	
	public int getRegionCount(){return this.regions.size();}
	
	public SortedSet<RegionVO> getRegions()
	{
		return this.regions;
	}
	
	@Override
	public void merge(AbstractNamedVO other)
	throws MergeException
	{
		if(other instanceof CacheVO)
		{
			super.merge(other);
			
			CacheVO otherCache = (CacheVO)other;
			
			if(this.getMetadata() == null && otherCache.getMetadata() != null)
				this.setMetadata(otherCache.getMetadata());
			
			mergeCollections(this, this.regions, otherCache.regions);
		}
		else
			throw new MergeException("CacheVO is unable to merge with '" + other.toString() + "'.");
	}

	@Override
	public int getChildCount(){return getRegionCount();}
	
	@Override
	public AbstractNamedVO childWithName(String name)
	{
		return this.searchChildCollection(getRegions(), name);
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public SortedSet<RegionVO> getChildren()
	{
		return getRegions();
	}
}
