/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.cache;

import java.io.Serializable;
import java.util.SortedSet;

/**
 * @author ZZZZZZBECKEC
 *
 */
public class InstanceVO
extends AbstractNamedVO
implements Serializable
{
	private static final long	serialVersionUID	= 1L;
	private String semanticTypeName;
	private CacheInstanceMetadata metadata = null;
	private CacheItemPath path = null;
	
	public InstanceVO(){}
	
	public InstanceVO(String name, CacheInstanceMetadata metadata, String semanticTypeName)
	{
		super(name);
		this.metadata = metadata;
		this.semanticTypeName = semanticTypeName;
	}
	
	public InstanceVO(String name, CacheInstanceMetadata metadata, String semanticTypeName, CacheItemPath path)
	{
		super(name);
		this.metadata = metadata;
		this.semanticTypeName = semanticTypeName;
		this.path = path;
	}

	public CacheInstanceMetadata getMetadata() {
		return metadata;
	}

	public void setMetadata(CacheInstanceMetadata metadata) {
		this.metadata = metadata;
	}

	/**
	 * Indicates a semantic to be applied to this group, the specifics are dependent on the
	 * individual caches, the UI will use this as a CSS style name.
	 * @return
	 */
	public String getSemanticTypeName() {return semanticTypeName;}
	public void setSemanticTypeName(String semanticTypeName) {this.semanticTypeName = semanticTypeName;}
	
	@Override
	public void merge(AbstractNamedVO other) 
	throws MergeException
	{
		if(other instanceof InstanceVO)
		{
			if(this.getMetadata() == null && ((InstanceVO)other).getMetadata() != null)
				this.setMetadata( ((InstanceVO)other).getMetadata() );
			
			super.merge(other);
		}
		else
			throw new MergeException("InstanceVO is unable to merge with '" + other.toString() + "'.");
	}

	
	@Override
	public CacheItemPath getPath()
	{
		if (path != null)
			return path;
		else if( getParent() != null )
		{
			CacheItemPath parentPath = getParent().getPath();
			path = parentPath.createChildPath(this.getName(), false);
			return path;
		}
		else
		{
			logger.severe( "InstanceVO.getParent() returns null." );
			return null;
		}
	}
	
	@Override
	public int getChildCount(){return 0;}
	
	@Override
	public AbstractNamedVO childWithName(String name)
	{
		return null;
	}
	
	@Override
	public boolean removeChild(AbstractNamedVO child){return false;}
	
	@Override
	public SortedSet<AbstractNamedVO> getChildren()
	{
		return null;
	}
}
