/**
 * Package: MAG - VistA Imaging
 * WARNING: Per VHA Directive 2004-038, this routine should not be modified.
 * @date Feb 9, 2011
 * Site Name:  Washington OI Field Office, Silver Spring, MD
 * @author zzzzzzbeckec
 * @version 1.0
 *
 * ----------------------------------------------------------------
 * Property of the US Government.
 * No permission to copy or redistribute this software is given.
 * Use of unreleased versions of this software requires the user
 * to execute a written test agreement with the VistA Imaging
 * Development Office of the Department of Veterans Affairs,
 * telephone (301) 734-0100.
 * 
 * The Food and Drug Administration classifies this software as
 * a Class II medical device.  As such, it may not be changed
 * in any way.  Modifications to this software may result in an
 * adulterated medical device under 21CFR820, the use of which
 * is considered to be a violation of US Federal Statutes.
 * ----------------------------------------------------------------
 */

package gov.va.med.imaging.exchange;

import gov.va.med.GlobalArtifactIdentifier;
import gov.va.med.GlobalArtifactIdentifierImpl;
import gov.va.med.RoutingToken;
import gov.va.med.RoutingTokenImpl;
import gov.va.med.exceptions.RoutingTokenFormatException;
import gov.va.med.imaging.BaseWebFacadeRouter;
import gov.va.med.imaging.GUID;
import gov.va.med.imaging.core.FacadeRouterUtility;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exceptions.URNFormatException;
import gov.va.med.imaging.exchange.business.TransactionContextLogEntrySnapshot;
import gov.va.med.imaging.exchange.business.documents.DocumentRetrieveResult;
import gov.va.med.imaging.transactioncontext.TransactionContext;
import gov.va.med.imaging.transactioncontext.TransactionContextFactory;
import java.io.IOException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author zzzzzzbeckec
 *
 */
public class DocumentDataAccessServlet
extends HttpServlet
{
	private static final long serialVersionUID = 1L;
	private final VixGuiWebAppRouter router = VixGuiWebAppRouterImpl.getSingleton();
	private String redirectDisplay;

	/**
	 * @see javax.servlet.GenericServlet#init(javax.servlet.ServletConfig)
	 */
	@Override
	public void init(ServletConfig config) throws ServletException
	{
		super.init(config);
		this.redirectDisplay = this.getInitParameter("redirectDisplay");
		if(this.redirectDisplay == null)
			throw new ServletException("The 'redirectDisplay' initialization parameter must be provided to " + getClass().getSimpleName() + " servlet."); 
	}
	
	/**
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException
	{
		// the document identifier components are in the URL
		String pathInfo = req.getPathInfo();
		String[] pathInfoElements = pathInfo != null ? pathInfo.split("/") : null;
		
		String homeId = pathInfoElements != null && pathInfoElements.length > 1 ? pathInfoElements[1] : null; 
		String repositoryId = pathInfoElements != null && pathInfoElements.length > 2 ? pathInfoElements[2] : null; 
		String documentId = pathInfoElements != null && pathInfoElements.length > 3 ? pathInfoElements[3] : null; 

		TransactionContext tc = TransactionContextFactory.get();
		tc.setTransactionId( (new GUID()).toString() );
		tc.setSsn("123-45-6789");
		tc.setFullName("Worthington J Testuser III");
		tc.setPurposeOfUse("Routine Medical Care");
		tc.setUserDivision("VistA Imaging Development");
		
		try
		{
			GlobalArtifactIdentifier documentIdentifier = 
				GlobalArtifactIdentifierImpl.createFromGlobalArtifactIdentifiers(homeId, repositoryId, documentId);
			
			DocumentRetrieveResult document = router.getDocument(documentIdentifier);
			
			req.setAttribute("documentRetrieveResult", document);
			req.getRequestDispatcher(redirectDisplay).forward(req, resp);
		}
		catch (MethodException x){throw new ServletException(x);}
		catch (ConnectionException x){throw new ServletException(x);}
		catch (URNFormatException x){throw new ServletException(x);}
		finally
		{
			try 
			{
				BaseWebFacadeRouter router = FacadeRouterUtility.getFacadeRouter(BaseWebFacadeRouter.class);
				router.postTransactionLogEntryImmediate(new TransactionContextLogEntrySnapshot(tc));
			} 
			catch (Exception xAny) 
			{
				System.err.println("postTransactionLogEntryImmediate Failed: " + xAny.getMessage());
			}
		}
	}

}
