/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange;

import gov.va.med.GlobalArtifactIdentifier;
import gov.va.med.imaging.AbstractImagingURN;
import gov.va.med.imaging.ImageURN;
import gov.va.med.imaging.access.TransactionLogWriter;
import gov.va.med.imaging.core.annotations.routerfacade.FacadeRouterInterface;
import gov.va.med.imaging.core.annotations.routerfacade.FacadeRouterInterfaceCommandTester;
import gov.va.med.imaging.core.annotations.routerfacade.FacadeRouterMethod;
import gov.va.med.imaging.core.interfaces.ImageMetadataNotification;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.ImageNotFoundException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.ImageFormatQualityList;

import java.io.IOException;
import java.io.OutputStream;

/**
 * 
 * @author zzzzzzbeckec
 *
 */
@FacadeRouterInterface(extendsClassName="gov.va.med.imaging.BaseWebFacadeRouterImpl")
@FacadeRouterInterfaceCommandTester
public interface VixGuiWebAppRouter
extends gov.va.med.imaging.BaseWebFacadeRouter
{
	
	@FacadeRouterMethod(asynchronous=false, commandClassName="PostImageAccessEventCommand")
	public abstract void logImageAccessEventImmediate(ImageAccessLogEvent event) 
	throws MethodException, ConnectionException;
	
	/**
	 * Retrieves image information about an image in the form of a newline terminated string
	 * @param imageUrn
	 * @return - image information or a null if the image is not found
	 * @throws MethodException
	 */
	@FacadeRouterMethod(asynchronous=false)
	public abstract String getImageInformation(AbstractImagingURN imagingUrn)
	throws MethodException, ConnectionException;

	/**
	 * 
	 * @param imageUrn
	 * @return
	 * @throws MethodException
	 */
	@FacadeRouterMethod(asynchronous=false)
	public abstract String getImageSystemGlobalNode(AbstractImagingURN imagingUrn)
	throws MethodException, ConnectionException;

	/**
	 * 
	 * @param imageURN
	 * @param flags
	 * @return
	 * @throws IOException
	 * @throws ImageNotFoundException
	 * @throws MethodException
	 */
	@FacadeRouterMethod(asynchronous=false)
	public abstract String getImageDevFields(AbstractImagingURN imagingUrn, String flags)
	throws MethodException, ConnectionException;
	
	/**
	 * Given an ImageURN (a globally unique identifier of an image) return the
	 * number of bytes written to the output stream. The metadata callback is called
	 * before writing to the stream
	 * 
	 * @param imageUrn
	 * @param requestedFormatQuality
	 * @param outStream
	 * @param metadataCallback
	 * @return
	 * @throws IOException
	 * @throws MethodException
	 */
	@FacadeRouterMethod(asynchronous=false, commandClassName="GetInstanceByImageUrnCommand")
	public abstract Long getInstanceByImageURN(
			ImageURN imageUrn, 
			ImageFormatQualityList requestedFormatQuality, 
			OutputStream outStream,
			ImageMetadataNotification metadataCallback)
	throws MethodException, ConnectionException;
	
	/**
	 * 
	 * Given an ImageURN (a globally unique identifier of an image) return the
	 * number of bytes written to the output stream. The metadata callback is called
	 * before writing to the stream. If logAccess is true and the image is returned then
	 * retrieving the image is logged as an access in the image access log (not transaction log). 
	 * 
	 * @param imageUrn
	 * @param requestedFormatQuality
	 * @param outStream
	 * @param metadataCallback
	 * @param logAccess If true and the image is returned, an event is logged in the image access log
	 * @return
	 * @throws IOException
	 * @throws MethodException
	 */
	/*
	@FacadeRouterMethod(asynchronous=false, commandClassName="GetInstanceByImageUrnCommand")
	public abstract Long getInstanceByImageURN(
			ImageURN imageUrn, 
			ImageFormatQualityList requestedFormatQuality, 
			OutputStream outStream,
			ImageMetadataNotification metadataCallback,
			boolean logAccess)
	throws MethodException, ConnectionException;
	*/
	
	@FacadeRouterMethod(asynchronous=false, commandClassName="ProcessSiteServiceCacheRefreshCommand")
	public abstract void refreshSiteServiceCache()
	throws MethodException, ConnectionException;
	
	@FacadeRouterMethod(asynchronous=false, commandClassName="GetImageCachedStatusCommand")
	public abstract Boolean getImageCachedStatus(ImageURN imageUrn, ImageFormatQualityList imageFormatQualityList)
	throws MethodException, ConnectionException;

	@FacadeRouterMethod(asynchronous=false, commandClassName="GetDocumentCachedStatusCommand")
	public abstract Boolean getDocumentCachedStatus(GlobalArtifactIdentifier documentIdentifier)
	throws MethodException, ConnectionException;
	
	@FacadeRouterMethod(asynchronous = false, commandClassName = "GetTransactionLogEntriesCommand")
	public abstract void getTransactionLogEntries(
			TransactionLogWriter transactionLogWriter,			
			String fieldName,
			String fieldValue) 
	throws MethodException, ConnectionException;
	
	@FacadeRouterMethod(asynchronous = false, commandClassName = "GetTransactionLogEntriesByTransactionIdCommand")
	public abstract void getTransactionLogEntriesByTransactionId(
			TransactionLogWriter transactionLogWriter,			
			String transactionId) 
	throws MethodException, ConnectionException;
}
