/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.exchange.business.taglib.study;

import gov.va.med.RoutingToken;
import gov.va.med.RoutingTokenImpl;
import gov.va.med.WellKnownOID;
import gov.va.med.exceptions.RoutingTokenFormatException;
import gov.va.med.imaging.BaseWebFacadeRouter;
import gov.va.med.imaging.core.FacadeRouterUtility;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.business.DocumentFilter;
import gov.va.med.imaging.exchange.business.documents.DocumentSet;
import gov.va.med.imaging.exchange.business.taglib.document.AbstractDocumentSetCollectionTag;
import gov.va.med.imaging.exchange.business.taglib.studyfilter.StudyFilterTag;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Collection;
import java.util.Date;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TryCatchFinally;
import org.apache.log4j.Logger;

/**
 * @author zzzzzzbeckec
 *
 */
public class AWIVDocumentSetListTag
extends AbstractDocumentSetCollectionTag
implements TryCatchFinally
{
	private static final long serialVersionUID = 1L;
	private Collection<DocumentSet> documentSetCollection = null;
	private Logger logger = Logger.getLogger(this.getClass());
	private String patientId;
	private String homeCommunityId;
	private String repositoryUniqueId;
	private String creationTimeFrom;
	private String creationTimeTo;
	private String classCode;

	/**
     * @return the patientId
     */
    public String getPatientId()
    {
    	return patientId;
    }
    public void setPatientId(String patientId)
    {
    	this.patientId = patientId;
    }
	
    public String getHomeCommunityId()
	{
		return this.homeCommunityId;
	}
	public void setHomeCommunityId(String homeCommunityId)
	{
		this.homeCommunityId = homeCommunityId;
	}
	
	public String getRepositoryUniqueId()
	{
		return this.repositoryUniqueId;
	}
	public void setRepositoryUniqueId(String repositoryUniqueId)
	{
		this.repositoryUniqueId = repositoryUniqueId;
	}
	
	public void setCreationTimeFrom(String creationTimeFrom)
	{
		this.creationTimeFrom = creationTimeFrom;
	}
	public String getCreationTimeFrom()
    {
    	return creationTimeFrom;
    }
    
    public void setCreationTimeTo(String creationTimeTo)
	{
		this.creationTimeTo = creationTimeTo;
	}
	public String getCreationTimeTo()
    {
    	return creationTimeTo;
    }
    
    public void setClassCode(String classCode)
	{
		this.classCode = classCode;
	}
	public String getClassCode()
    {
    	return classCode;
    }
    
	/**
	 * @see gov.va.med.imaging.exchange.business.taglib.AbstractBusinessObjectCollectionTag#getCollection()
	 */
	@Override
	protected Collection<DocumentSet> getCollection() 
	throws JspException
	{
    	BaseWebFacadeRouter router;
		try
		{
			router = FacadeRouterUtility.getFacadeRouter(BaseWebFacadeRouter.class);
		} 
		catch (Exception x)
		{
			String msg = "Error getting facade router, unable to continue.";
			logger.error(msg, x);
			throw new JspException(msg, x);
		} 
    	
    	DateFormat df = StudyFilterTag.getDateFormat();
    	Date timeFrom = new Date( Date.UTC(1900, 1, 1, 0, 0, 0) );
    	Date timeTo = new Date( Date.UTC(2100, 1, 1, 0, 0, 0) );
    	
    	try
		{
			if(getCreationTimeFrom() != null)
				timeFrom = df.parse(getCreationTimeFrom());
			if(getCreationTimeTo() != null)
				timeTo = df.parse(getCreationTimeTo());
			
			DocumentFilter documentFilter = 
				new DocumentFilter( getPatientId(), timeFrom, timeTo, getClassCode() );
			
			RoutingToken routingToken = RoutingTokenImpl.create(
				(getHomeCommunityId() == null ? WellKnownOID.HAIMS_DOCUMENT.getCanonicalValue().toString() : getHomeCommunityId()), 
				(getRepositoryUniqueId() == null ? RoutingTokenImpl.ROUTING_WILDCARD : getRepositoryUniqueId()) 
			);
			
			documentSetCollection = router.getDocumentSetList( routingToken, documentFilter );
		}
		catch (ParseException x)
		{
			throw new JspException(x);
		}
		catch (MethodException x)
		{
			throw new JspException(x);
		}
		catch (ConnectionException x)
		{
			throw new JspException(x);
		}
		catch (RoutingTokenFormatException x)
		{
			throw new JspException(x);
		}
    	
		return documentSetCollection;
	}

	/**
     * @see javax.servlet.jsp.tagext.TryCatchFinally#doCatch(java.lang.Throwable)
     */
    @Override
    public void doCatch(Throwable arg0) 
    throws Throwable
    {
    	logger.error(arg0.getMessage());
    }
	/**
     * @see javax.servlet.jsp.tagext.TryCatchFinally#doFinally()
     */
    @Override
    public void doFinally()
    {
	    
    }
	
}
