/**
 * Package: MAG - VistA Imaging
 * WARNING: Per VHA Directive 2004-038, this routine should not be modified.
 * Date Created: Apr 10, 2008
 * Site Name:  Washington OI Field Office, Silver Spring, MD
 * @author ZZZZZZBECKEC
 * @version 1.0
 *
 * ----------------------------------------------------------------
 * Property of the US Government.
 * No permission to copy or redistribute this software is given.
 * Use of unreleased versions of this software requires the user
 * to execute a written test agreement with the VistA Imaging
 * Development Office of the Department of Veterans Affairs,
 * telephone (301) 734-0100.
 * 
 * The Food and Drug Administration classifies this software as
 * a Class II medical device.  As such, it may not be changed
 * in any way.  Modifications to this software may result in an
 * adulterated medical device under 21CFR820, the use of which
 * is considered to be a violation of US Federal Statutes.
 * ----------------------------------------------------------------
 */
package gov.va.med.imaging.exchange.business.taglib.study;

import gov.va.med.GlobalArtifactIdentifier;
import gov.va.med.GlobalArtifactIdentifierFactory;
import gov.va.med.RoutingTokenImpl;
import gov.va.med.exceptions.RoutingTokenFormatException;
import gov.va.med.imaging.core.FacadeRouterUtility;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exchange.VixGuiWebAppRouter;
import gov.va.med.imaging.exchange.business.Study;
import gov.va.med.imaging.SiteNumber;
import gov.va.med.imaging.exchange.business.StudyFilter;
import gov.va.med.imaging.exchange.business.taglib.studyfilter.StudyFilterTag;
import gov.va.med.imaging.transactioncontext.TransactionContext;
import gov.va.med.imaging.transactioncontext.TransactionContextFactory;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TryCatchFinally;

import org.apache.log4j.Logger;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.servlet.support.JspAwareRequestContext;
import org.springframework.web.servlet.support.RequestContext;

/**
 * @author ZZZZZZBECKEC
 *
 */
public class FederationStudyListTag 
extends AbstractStudyListTag
implements TryCatchFinally
{
	private static final long serialVersionUID = 1L;
	
	private String siteNumber;
	private String patientId;
	private String fromDate;
	private String toDate;
	private GlobalArtifactIdentifier studyId;
	private String targetSiteNumber;

	private Logger logger = Logger.getLogger(this.getClass());

	private RequestContext requestContext;
	
	// ==============================================================================
	// Properties that may be set from the JSP
	// ==============================================================================
	
	/**
     * @return the siteNumber
     */
    public String getSiteNumber()
    {
    	return siteNumber;
    }
    public void setSiteNumber(String siteNumber)
    {
    	this.siteNumber = siteNumber;
    }
	/**
     * @return the patientId
     */
    public String getPatientId()
    {
    	return patientId;
    }
    public void setPatientId(String patientId)
    {
    	this.patientId = patientId;
    }
    
	public String getFromDate()
    {
    	return fromDate;
    }
	public void setFromDate(String fromDate)
    {
    	this.fromDate = fromDate;
    }
	
	public String getToDate()
    {
    	return toDate;
    }
	public void setToDate(String toDate)
    {
    	this.toDate = toDate;
    }
	
	public String getStudyId()
    {
    	return studyId.toString();
    }
	public void setStudyId(String studyId) 
	throws JspException
    {
    	try
		{
			this.studyId = GlobalArtifactIdentifierFactory.create(studyId);
		}
		catch (Throwable x)
		{
			throw new JspException(x);
		}
    }
	
	public String getTargetSiteNumber()
	{
		return targetSiteNumber;
	}
	
	public void setTargetSiteNumber(String targetSiteNumber)
	{
		this.targetSiteNumber = targetSiteNumber;
	}

	// ==============================================================================
	// JSP Tag Lifecycle Events
	// ==============================================================================

	/**
	 * @see gov.va.med.imaging.exchange.business.taglib.study.AbstractStudyListTag#getStudyList()
	 */
	@Override
	protected List<Study> getCollection() 
	throws JspException
	{
		List<Study> studyList;
		
		if( getSiteNumber() == null || getSiteNumber().isEmpty() || 
			getPatientId() == null || getPatientId().isEmpty() )
			return null;
		
		TransactionContext transactionContext = TransactionContextFactory.get();
		
		String targetSiteNumber = getTargetSiteNumber();
		if(targetSiteNumber == null || targetSiteNumber.length() < 1)
			targetSiteNumber = transactionContext.getRealm();		// default to our local site number
		try
		{
			transactionContext.setOverrideRoutingToken(RoutingTokenImpl.createVARadiologySite(targetSiteNumber));
		}
		catch (RoutingTokenFormatException rtfX)
		{
			String msg = "Error overriding routing token '" + rtfX.getMessage() + "'.";
			logger.error(msg, rtfX);
			throw new JspException(msg, rtfX);
		}
		transactionContext.setOverrideProtocol("vftp");
		
		this.requestContext = getRequestContext();
		
    	WebApplicationContext webApplicationContext = 
    		getRequestContext().getWebApplicationContext();
    	
    	VixGuiWebAppRouter router;
		try
		{
			router = FacadeRouterUtility.getFacadeRouter(VixGuiWebAppRouter.class);
		} 
		catch (Exception x)
		{
    		logger.error(x);
    		studyList = null;
    		throw new JspException(x);
		} 
    	DateFormat df = StudyFilterTag.getDateFormat();
    	
    	Date from = new Date(0L);
    	if(getFromDate() != null && getFromDate().length() > 0)
    		try{from = df.parse(getFromDate());}
    		catch(ParseException pX){from=new Date(0L); logger.error(pX);}
    		
    	Date to = new Date(Long.MAX_VALUE); 
    	if(getToDate() != null && getToDate().length() > 0)
    		try{to = df.parse(getToDate());}
    		catch(ParseException pX){to=new Date(Long.MAX_VALUE); logger.error(pX);}

    	StudyFilter filter = new StudyFilter( from, to, studyId );
    	
    	try 
    	{
    		studyList = router.getPatientStudyList(
    			RoutingTokenImpl.createVARadiologySite(getSiteNumber()), 
    			getPatientId(), 
    			filter);
    	}
    	catch(Exception mX)
    	{
    		logger.error(mX);
    		studyList = null;
    		throw new JspException(mX);
    	}
    	
	    return studyList;
	}
	
	/**
     * @see javax.servlet.jsp.tagext.TryCatchFinally#doCatch(java.lang.Throwable)
     */
    @Override
    public void doCatch(Throwable arg0) 
    throws Throwable
    {
    	logger.error(arg0.getMessage());
    }
	/**
     * @see javax.servlet.jsp.tagext.TryCatchFinally#doFinally()
     */
    @Override
    public void doFinally()
    {
	    
    }
	
}