/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: Mar 31, 2011
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:  ZZZZZZWERFEJ
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
package gov.va.med.imaging.exchange.rest;

import java.lang.reflect.InvocationTargetException;

import gov.va.med.GlobalArtifactIdentifier;
import gov.va.med.GlobalArtifactIdentifierFactory;
import gov.va.med.URNFactory;
import gov.va.med.exceptions.GlobalArtifactIdentifierFormatException;
import gov.va.med.imaging.ImageURN;
import gov.va.med.imaging.core.FacadeRouterUtility;
import gov.va.med.imaging.core.interfaces.exceptions.ConnectionException;
import gov.va.med.imaging.core.interfaces.exceptions.MethodException;
import gov.va.med.imaging.exceptions.URNFormatException;
import gov.va.med.imaging.exchange.VixGuiWebAppRouter;
import gov.va.med.imaging.exchange.business.ImageFormatQualityList;
import gov.va.med.imaging.rest.types.RestBooleanReturnType;

import javax.ws.rs.GET;
import javax.ws.rs.HeaderParam;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

/**
 * @author ZZZZZZWERFEJ
 *
 */
@Path("cache")
public class CacheRestService
{
	@GET
	@Path("images/{imageUrn}")
	@Produces(MediaType.APPLICATION_XML)
	public RestBooleanReturnType isImageCached(
		@PathParam("imageUrn") String imageUrn,
		//@HeaderParam("accept") String accept)
		@HeaderParam("xxx-imageFormatQualityList") String imageFormatQualityListString)
	throws MethodException, ConnectionException
	{
		//accept=[application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5],
		ImageFormatQualityList imageFormatQualityList = 
			ImageFormatQualityList.createListFromAcceptString(imageFormatQualityListString);
		try
		{
			ImageURN urn = URNFactory.create(imageUrn, ImageURN.class);
			VixGuiWebAppRouter router = getRouter();
			boolean result = router.getImageCachedStatus(urn, imageFormatQualityList);
			return new RestBooleanReturnType(result);
		}
		catch(URNFormatException urnfX)
		{
			throw new MethodException(urnfX);
		}
	}
	
	@SuppressWarnings("unchecked")
	@GET
	@Path("documents/{gai}")
	@Produces(MediaType.APPLICATION_XML)
	public RestBooleanReturnType isDocumentCached(
			@PathParam("gai") String gai)
	throws MethodException, ConnectionException
	{
		try
		{
			VixGuiWebAppRouter router = getRouter();
			GlobalArtifactIdentifier g = 
				GlobalArtifactIdentifierFactory.create(gai, 
						GlobalArtifactIdentifier.class);
			boolean result = router.getDocumentCachedStatus(g);
			return new RestBooleanReturnType(result);
		}
		catch(GlobalArtifactIdentifierFormatException gaifX)
		{
			throw new MethodException(gaifX);
		}
	}
	
	private VixGuiWebAppRouter getRouter()
	throws MethodException
	{
		try
		{
			VixGuiWebAppRouter router = 
				FacadeRouterUtility.getFacadeRouter (VixGuiWebAppRouter.class);
			return router;
		}
		catch(InvocationTargetException itX)
		{
			throw new MethodException(itX);
		}
		catch(IllegalAccessException iaX)
		{
			throw new MethodException(iaX);
		}
		catch(InstantiationException iX)
		{
			throw new MethodException(iX);
		}
		catch(ClassNotFoundException cnfX)
		{
			throw new MethodException(cnfX);
		}		
	}

}
