/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.ihe.xca.datasource.interactive;

import gov.va.med.imaging.datasource.Provider;
import gov.va.med.imaging.ihe.xca.datasource.configuration.XCADataSourceConfiguration;
import gov.va.med.interactive.Command;
import gov.va.med.interactive.CommandListCommandFactory;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * 
 * @author zzzzzzbeckec
 *
 */
public class XCADatasourceConfigurationCommandFactory
extends CommandListCommandFactory<XCADataSourceConfiguration>
{
	private Provider provider;
	
	/**
	 * @param provider 
	 * @throws IOException
	 * @throws ClassNotFoundException
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public XCADatasourceConfigurationCommandFactory(Provider provider) 
	throws IOException, ClassNotFoundException, SecurityException, NoSuchMethodException, 
		IllegalArgumentException, IllegalAccessException, InvocationTargetException
	{
		super();
		this.provider = provider;
	}

	/**
	 * @return the provider
	 */
	public Provider getProvider()
	{
		return this.provider;
	}

	/**
	 * @see gov.va.med.interactive.CommandListCommandFactory#getCommandPackageNames()
	 */
	@Override
	public String[] getCommandPackageNames()
	{
		return new String[]
		{
			"gov.va.med.interactive.commands",						// standard commands
			this.getClass().getPackage().getName() + ".commands"	// application specific commands
		};
	}

	/* (non-Javadoc)
	 * @see gov.va.med.interactive.CommandListCommandFactory#createCommand(java.lang.String, java.lang.String[])
	 */
	@Override
	public Command<XCADataSourceConfiguration> createCommand(String commandText, String[] commandArgs)
	throws InstantiationException, IllegalAccessException, ClassCastException, SecurityException,
		NoSuchMethodException, IllegalArgumentException, InvocationTargetException
	{
		Command<XCADataSourceConfiguration> command = super.createCommand(commandText, commandArgs);
		try
		{
			Method setProviderMethod = command.getClass().getMethod("setProvider", new Class<?>[]{Provider.class});
			setProviderMethod.invoke(command, new Object[]{getProvider()});
		}
		catch(Exception x){}
		return command;
	}

	/**
	 * @see gov.va.med.interactive.CommandListCommandFactory#createCommand(java.lang.String)
	 */
	@Override
	public Command<XCADataSourceConfiguration> createCommand(String commandText) 
	throws InstantiationException, IllegalAccessException, ClassCastException, SecurityException, 
		NoSuchMethodException, IllegalArgumentException, InvocationTargetException
	{
		Command<XCADataSourceConfiguration> command = super.createCommand(commandText);
		
		return command;
	}
	
	
}
