/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.ihe.xca.datasource.test;

import gov.va.med.ProtocolHandlerUtility;
import gov.va.med.imaging.datasource.DataSourceProvider;
import gov.va.med.imaging.datasource.Provider;
import gov.va.med.imaging.exchange.business.ResolvedSite;
import gov.va.med.imaging.exchange.business.ResolvedSiteImpl;
import gov.va.med.imaging.exchange.business.Site;
import gov.va.med.imaging.exchange.business.SiteImpl;
import gov.va.med.imaging.tomcat.vistarealm.VistaRealmPrincipal.AuthenticationCredentialsType;
import gov.va.med.imaging.transactioncontext.ClientPrincipal;
import gov.va.med.imaging.transactioncontext.TransactionContextFactory;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 * @author zzzzzzbeckec
 *
 */
public abstract class AbstractXCADataSourceTest
{
	private static List<URL> metadataUrls = null;
	private static List<URL> artifactUrls = null;
	
	protected static synchronized List<URL> getMetadataUrls()
	throws MalformedURLException
	{
		if(metadataUrls == null)
		{
			metadataUrls = new ArrayList<URL>();
			metadataUrls.add( new URL("vista://locahost:9300") );
			metadataUrls.add( new URL("vistaimaging://localhost:9300") );
			metadataUrls.add( new URL("vftp://localhost:8080") );
			metadataUrls.add( new URL("xca://localhost:1963/XCARespondingGateway/services/RespondingGateway_Service") );
		}
		return metadataUrls;
	}

	protected static synchronized List<URL> getArtifactUrls() 
	throws MalformedURLException
	{
		if(artifactUrls == null)
		{
			artifactUrls = new ArrayList<URL>();
			artifactUrls.add( new URL("vista://locahost:9300") );
			artifactUrls.add( new URL("vistaimaging://localhost:9300") );
			artifactUrls.add( new URL("vftp://localhost:8080") );
			artifactUrls.add( new URL("xca://localhost:1963/XCARespondingGateway/services/RespondingGateway_Service") );
		}
		return artifactUrls;
	}

	/**
	 * 
	 */
	protected static void initializeConnectionHandlers()
	{
		ProtocolHandlerUtility.initialize(true);
	}
	
	/**
	 * 
	 */
	protected static void initializeTransactionContext()
	{
		List<String> roles = new ArrayList<String>();
		ClientPrincipal principal = new ClientPrincipal(
				"660", true, AuthenticationCredentialsType.Password, 
				"username3", "password5", 
				"126", "IMAGPROVIDERONETWOSIX,ONETWOSIX", "SSN1", "660", "Salt lake City", 
				roles, 
				new HashMap<String, Object>()
		);
		TransactionContextFactory.createClientTransactionContext(principal);
	}
	
	protected static void initializeLogging()
	{
		Logger.getRootLogger().setLevel(Level.ALL);
	}
	
	private static DataSourceProvider provider;
	protected static synchronized DataSourceProvider getProvider()
	{
		if(provider == null)
			provider = new Provider();
		
		return provider;
	}

	protected static Site getVATestSite() 
	throws MalformedURLException
	{
		return new SiteImpl("660", "Salt Lake City", "SLC", "localhost", 9300, "localhost", 8080, "");
	}
	
	protected static Site getDoDTestSite() 
	throws MalformedURLException
	{
		return new SiteImpl("200", "DoD", "DOD", null, 0, "computer2", 8080, "");
	}
	
	private static ResolvedSiteImpl resolvedVASite = null;
	protected static synchronized ResolvedSite getTestVAResolvedSite() 
	throws MalformedURLException
	{
		if(resolvedVASite == null)
		{
			resolvedVASite = ResolvedSiteImpl.create(
				getVATestSite(), 
				true, 
				false, 
				true,
				getMetadataUrls(), 
				getArtifactUrls());
		}		
		return resolvedVASite;
	}
		
	private static ResolvedSiteImpl resolvedDoDSite = null;
	protected static synchronized ResolvedSite getTestDoDResolvedSite() 
	throws MalformedURLException
	{
		if(resolvedDoDSite == null)
		{
			resolvedDoDSite = ResolvedSiteImpl.create(
				getDoDTestSite(), 
				true, 
				false,
				true,
				getMetadataUrls(), 
				getArtifactUrls());
		}		
		return resolvedDoDSite;
	}
}
