/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
/**
 * 
 */
package gov.va.med.imaging.ihe.xca.query;

import gov.va.med.URNFactory;
import gov.va.med.imaging.exceptions.URNFormatException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import org.apache.log4j.Logger;

/**
 * @author zzzzzzbeckec
 *
 */
public class Configuration
{
	private static final String BUNDLE_NAME = "gov.va.med.imaging.ihe.xca.configuration"; //$NON-NLS-1$
	private static final ResourceBundle RESOURCE_BUNDLE = ResourceBundle.getBundle(BUNDLE_NAME);
	private static final Logger logger = Logger.getLogger(Configuration.class);

	private Configuration()
	{
	}

	public static String getString(String key, String defaultValue)
	{
		try
		{
			return RESOURCE_BUNDLE.getString(key);
		}
		catch (MissingResourceException e)
		{
			logger.warn("Missing configuration key '" + key + "', using default value " + defaultValue);
			return defaultValue;
		}
	}
	
	/**
	 * 
	 * @param key
	 * @param defaultValue
	 * @return
	 */
	public static int getInt(String key, int defaultValue)
	{
		try
		{
			return Integer.parseInt( RESOURCE_BUNDLE.getString(key) );
		}
		catch (MissingResourceException mrX)
		{
			logger.warn("Missing configuration key '" + key + "', using default value " + defaultValue);
			return defaultValue;
		}
		catch (NumberFormatException nfX)
		{
			logger.warn("Invalid format configuration key '" + key + "', using default value " + defaultValue);
			return defaultValue;
		}
	}
	
	/**
	 * 
	 * @param key
	 * @param defaultValue
	 * @return
	 */
	public static float getFloat(String key, float defaultValue)
	{
		try
		{
			return Float.parseFloat( RESOURCE_BUNDLE.getString(key) );
		}
		catch (MissingResourceException mrX)
		{
			logger.warn("Missing configuration key '" + key + "', using default value " + defaultValue);
			return defaultValue;
		}
		catch (NumberFormatException nfX)
		{
			logger.warn("Invalid format configuration key '" + key + "', using default value " + defaultValue);
			return defaultValue;
		}
	}
}
