/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using java.util.zip;
using System.IO;

namespace ChecksumGenerator.Business
{
    public class Adler32Checksum
    {
        public static string CalculateChecksum(string filename)
        {

            byte[] buffer = FileToByteArray(filename);
            SByte[] sbuffer = new SByte[buffer.Length]; // checksum buffer - must be signed to make Java happy
            Adler32 adler32 = new Adler32();
            for (int i = 0; i < buffer.Length; i++) sbuffer[i] = (sbyte)buffer[i];
            adler32.update(sbuffer, 0, sbuffer.Length);
            return "{Adler32}" + ((Decimal)adler32.getValue()).ToString();
        }

        public static ChecksumGroup CalculateChecksums(DirectoryInfo dir, string version, LibraryType libraryType, 
            VisaType visaType, OsArchitecture osArchitecture)
        {
            ChecksumGroup group = new ChecksumGroup(version, libraryType, visaType, osArchitecture);
            foreach (FileInfo file in dir.GetFiles())
            {
                string checksum = CalculateChecksum(file.FullName);
                group.Checksums.Add(file.Name, new FileDetails(file.Name, checksum, file.Length));
            }
            return group;
        }

        private static byte[] FileToByteArray(string fileName)
        {
            byte[] buff = null;
            FileStream fs = new FileStream(fileName,
                                           FileMode.Open,
                                           FileAccess.Read);
            BinaryReader br = new BinaryReader(fs);
            long numBytes = new FileInfo(fileName).Length;
            buff = br.ReadBytes((int)numBytes);
            return buff;
        }
    }
}
