/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ChecksumGenerator.Business;
using System.IO;


namespace ChecksumGenerator
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            txtLibDirectory.Text = @"C:\Program Files\Apache Software Foundation\Tomcat 6.0\lib";
            cboLibraryType.Items.Add(LibraryType.tomcatLib);
            cboLibraryType.Items.Add(LibraryType.jreLibExt);

            string catalinaHome =  Environment.GetEnvironmentVariable("CATALINA_HOME");
            if (catalinaHome != null)
                txtLibDirectory.Text = catalinaHome + @"\lib";

            cboLibraryType.SelectedItem = LibraryType.tomcatLib;
            
            FileInfo file = new FileInfo(Application.ExecutablePath);
            string checksumsDirectory = file.Directory.FullName + "\\checksums";
            txtOutputDirectory.Text = checksumsDirectory;
            if (!Directory.Exists(checksumsDirectory))
            {
                Directory.CreateDirectory(checksumsDirectory);
            }

            cboVisaType.Items.Add(VisaType.cvix);
            cboVisaType.Items.Add(VisaType.vix);
            cboVisaType.Items.Add(VisaType.hdig);

            OsArchitecture osArchitecture = OsArchitecture.x86;
            if (OSUtility.Is64BitOperatingSystem())
                osArchitecture = OsArchitecture.x64;

            txtOSArchitecture.Text = osArchitecture + "";

        }

        private void btnGenerateChecksums_Click(object sender, EventArgs e)
        {
            OsArchitecture osArchitecture = OsArchitecture.x86;
            if (OSUtility.Is64BitOperatingSystem())
                osArchitecture = OsArchitecture.x64;

            string filename = GenerateFilename(osArchitecture);
            LibraryType libraryType = (LibraryType)cboLibraryType.SelectedItem;
            VisaType visaType = (VisaType)cboVisaType.SelectedItem;
            ChecksumGroup group = Adler32Checksum.CalculateChecksums(new DirectoryInfo(txtLibDirectory.Text),
                txtVersion.Text, libraryType, visaType, osArchitecture);
            group.Save(filename);
            LogMsg("Done");
        }

        private void LogMsg(string msg)
        {
            txtLog.AppendText(msg + Environment.NewLine);
        }

        private string GenerateFilename(OsArchitecture osArchitecture)
        {
            LibraryType libraryType = (LibraryType)cboLibraryType.SelectedItem;
            VisaType visaType = (VisaType)cboVisaType.SelectedItem;
            string filename = txtVersion.Text + "_" + libraryType + "_" + visaType + "_" + osArchitecture + ".xml";
            return Path.Combine(txtOutputDirectory.Text, filename);
        }

        private void btnOpenChecksumsDirectory_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(txtOutputDirectory.Text);
        }
    }
}
