/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DailyTransactionLogAnalyzerEngine;
using System.Configuration;
using System.IO;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using DailyTransactionLogAnalyzer.json;
using DailyTransactionLogAnalyzerEngine.reports;

namespace DailyTransactionLogAnalyzer
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            txtTransactionLogsDirectory.Text = ConfigurationManager.AppSettings["TransactionLogsDirectory"];
            txtAnalysisDirectory.Text = ConfigurationManager.AppSettings["MonthlyAnalysisDirectory"];
            DateTime today = DateTime.Now;
            //dtMonth.Value = today.AddMonths(-1); // set to previous month
            dtFrom.Value = new DateTime(today.Year, today.Month, today.Day).AddMonths(-1) ; // doing it this way sets the time to midnight
            dtTo.Value = new DateTime(today.Year, today.Month, today.Day).AddMonths(-1); // doing it this way sets the time to midnight
            txtProcessSites.Text = "688";
            txtProcessSites.Text = "2001.1,2001.2,2001.3,2001.4";
            txtProcessSites.Text = "688,660";
            dtFrom.Value = new DateTime(2013, 5, 1);
            dtTo.Value = new DateTime(2013, 5, 11);
            dtFrom.Value = new DateTime(2013, 4, 1);
            dtFrom.Value = new DateTime(2013, 1, 1);
            dtTo.Value = new DateTime(2013, 7, 1);
            rbMonth.Checked = true;
            //rbDay.Checked = true;
            SetCalendarFormat();
            lblReportDescription.Text = "";
            //cboExcelFunctions.Items.AddRange(ExcelOutputOptions.GetExcelOptions());
            cboExcelFunctions.Items.AddRange(ReportHelper.GetAllReports());
            txtAnalysisServer.Text = ConfigurationManager.AppSettings["AnalysisServerUrl"];
        }

        private void btnRunAnalysis_Click(object sender, EventArgs e)
        {
            try
            {

                Cursor.Current = Cursors.WaitCursor;
                string logsDirectory = txtTransactionLogsDirectory.Text;// @"C:\Users\zzzzzzwerfej\VIX\Transaction Log";
                DailyHelper helper = new DailyHelper(logsDirectory, LogMsg);
                //string siteNumber = txtProcessSites.Text;// "688";// "565";
                //DateTime date = new DateTime(2013, 4, 9);
                DateTime date = dtFrom.Value;
                DateTime startTime = DateTime.Now;

                List<string> sites = GetSelectedSites();
                string outputDirectory = txtAnalysisDirectory.Text;

                if (rbMonth.Checked)
                {
                    DateTime startDate = dtFrom.Value;
                    DateTime endDate = dtTo.Value;
                    date = startDate;
                    while (date.CompareTo(endDate) < 1)
                    {
                        LogMsg("Processing Month [" + date.ToString("MMMM yyyy") + "]");
                        int i = 1;
                        int count = sites.Count;
                        foreach (string siteNumber in sites)
                        {
                            tssProgress.Text = i + " of " + count;
                            LogMsg("Processing Site [" + siteNumber + "]");
                            DailyTransactions dailyTransactions = helper.AnalyzeMonthlyTransactions(siteNumber, date.Year, date.Month);
                            if (dailyTransactions != null)
                            {
                                dailyTransactions.SaveMonthToXml(outputDirectory, siteNumber, date.Year, date.Month);
                                dailyTransactions = null;
                            }
                            i++;
                        }
                        date = date.AddMonths(1);
                    }                    
                }
                else if(rbDay.Checked)
                {
                    DateTime startDate = dtFrom.Value;
                    DateTime endDate = dtTo.Value;//.AddDays(1);
                    
                    date = startDate;
                    while (date.CompareTo(endDate) < 1)
                    {
                        LogMsg("Processing date '" + date.ToString() + "'.");
                        int i = 1;
                        int count = sites.Count;
                        foreach (string siteNumber in sites)
                        {
                            tssProgress.Text = i + " of " + count;
                            LogMsg("Processing Site [" + siteNumber + "]");
                            DailyTransactions dailyTransactions = helper.AnalyzeDailyTransactions(siteNumber, date);
                            if (dailyTransactions != null)
                            {
                                dailyTransactions.SaveDateToXml(outputDirectory, siteNumber, date);
                            }
                            i++;
                        }
                        date = date.AddDays(1);
                    }                    
                }
                else if (rbYear.Checked)
                {
                    DateTime startDate = dtFrom.Value;
                    DateTime endDate = dtTo.Value;
                    date = startDate;
                    while (date.CompareTo(endDate) < 1)
                    {
                        int year = date.Year;
                        int i = 1;
                        int count = sites.Count;
                        foreach (string siteNumber in sites)
                        {
                            tssProgress.Text = i + " of " + count;
                            LogMsg("Processing Site [" + siteNumber + "]");

                            List<DailyTransactions> siteYearTransactions = new List<DailyTransactions>();

                            for (int month = 1; month <= 12; month++)
                            {
                                DailyTransactions dailyTransactions =
                                    DailyTransactions.LoadFromXml(outputDirectory, siteNumber, year, month);
                                if (dailyTransactions != null)
                                    siteYearTransactions.Add(dailyTransactions);
                            }
                            LogMsg("Merging all month transactions into year");
                            DailyTransactions yearTransactions =
                                DailyTransactions.MergeIntoSingleYear(siteYearTransactions, siteNumber, year);
                            yearTransactions.SaveYearToXml(outputDirectory, siteNumber, year);
                            yearTransactions = null;
                            siteYearTransactions = null;
                            i++;
                        }
                        date = date.AddYears(1);
                    }
                }
                DateTime endTime = DateTime.Now;
                TimeSpan ts = endTime.Subtract(startTime);
                LogMsg("Done");
                LogMsg("Analyzing sites took '" + ts.ToString(@"hh\:mm\:ss") + "'.");
                //richTextBox1.Text = "Analyzing directory took" + ts.ToString() + Environment.NewLine + dailyTransactions.ToString();

            }
            catch (Exception ex)
            {
                LogMsg(ex.Message + Environment.NewLine + Environment.NewLine + ex.StackTrace);
                //MessageBox.Show(ex.Message + Environment.NewLine + Environment.NewLine + ex.StackTrace);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void LogMsg(string msg)
        {
            txtLog.AppendText(msg + Environment.NewLine);
            txtLog.SelectionStart = txtLog.Text.Length;
            txtLog.ScrollToCaret();
            Application.DoEvents();
        }

        private List<string> GetSelectedSites()
        {
            List<string> sites = new List<string>();
            string[] siteStrings = txtProcessSites.Text.Split(',');
            foreach (string site in siteStrings)
            {
                if (site.Length > 0)
                    sites.Add(site.Trim());
            }
            return sites;
        }

        private void button3_Click_1(object sender, EventArgs e)
        {
            /*
            string logsDirectory = txtTransactionLogsDirectory.Text;// @"C:\Users\zzzzzzwerfej\VIX\Transaction Log";
            DailyHelper helper = new DailyHelper(logsDirectory, LogMsg);
            helper.AnalyzeDailyTransactions("565", new DateTime(2013, 4, 30));
             * */
            DailyTransactions dailyTransactions = DailyTransactions.LoadFromXml(@"C:\Users\zzzzzzwerfej\VIX\Monthly Analysis\688\2013\2013-04.xml", DailyTransactionsType.month);
            long count = 0L;
            foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
            {
                foreach (FromSiteCounts fsc in fc.SiteCounts.Values)
                {
                    count += fsc.TotalTransactions;
                }
            }
            LogMsg("Count [" + count + "]");
        }

        private void rbMonth_CheckedChanged(object sender, EventArgs e)
        {
            SetCalendarFormat();
        }

        private void SetCalendarFormat()
        {
            if (rbMonth.Checked)
            {
                dtFrom.CustomFormat = "MMMM yyyy";
                dtTo.CustomFormat = "MMMM yyyy";
                dtTo.Visible = true;
                lblTo.Visible = true;
            }
            else if(rbDay.Checked)
            {
                dtTo.CustomFormat = "MMMM dd, yyyy";
                dtFrom.CustomFormat = "MMMM dd, yyyy";
                dtTo.Visible = true;
                lblTo.Visible = true;
            }
            else if (rbYear.Checked)
            {
                dtFrom.CustomFormat = "yyyy";
                dtTo.CustomFormat = "yyyy";
                dtTo.Visible = true;
                lblTo.Visible = true;
            }
        }

        private void rbDay_CheckedChanged(object sender, EventArgs e)
        {
            SetCalendarFormat();
        }

        private void btnCVIX_Click(object sender, EventArgs e)
        {
            txtProcessSites.Enabled = true;
            List<Site> cvixSites = SiteHelper.GetCvixSites();
            StringBuilder siteNumbers = new StringBuilder();
            string prefix = "";
            foreach (Site site in cvixSites)
            {
                siteNumbers.Append(prefix);
                siteNumbers.Append(site.SiteId);
                prefix = ",";
            }
            txtProcessSites.Text = siteNumbers.ToString();
        }
        
        private void btnVaSites_Click(object sender, EventArgs e)
        {
            txtProcessSites.Enabled = true;
            List<Site> cvixSites = SiteHelper.GetVASites();
            StringBuilder siteNumbers = new StringBuilder();
            string prefix = "";
            foreach (Site site in cvixSites)
            {
                siteNumbers.Append(prefix);
                siteNumbers.Append(site.SiteId);
                prefix = ",";
            }
            txtProcessSites.Text = siteNumbers.ToString();
        }

        private void cboExcelFunctions_SelectedIndexChanged(object sender, EventArgs e)
        {
            ReportGenerator outputOption = (ReportGenerator)cboExcelFunctions.SelectedItem;
            lblReportDescription.Text = outputOption.Description;
        }

        private void btnGenerateReport_Click(object sender, EventArgs e)
        {           
            ReportGenerator report = (ReportGenerator)cboExcelFunctions.SelectedItem;
            if (report == null)
                return;
            string outputDirectory = txtAnalysisDirectory.Text;
            List<string> sites = GetSelectedSites();
            DateTime startDate = dtFrom.Value;
            DateTime endDate = dtTo.Value;
            DateTime startTime = DateTime.Now;

            ReportDateType reportDateType = ReportDateType.month;

            if (rbMonth.Checked)
            {
                reportDateType = ReportDateType.month;
            }
            else if (rbDay.Checked)
            {
                reportDateType = ReportDateType.day;
            }
            else if (rbYear.Checked)
            {
                reportDateType = ReportDateType.year;
            }

            try
            {
                report.GenerateReport(LogMsg, outputDirectory, sites, startDate, endDate, reportDateType);
            }
            catch (Exception ex)
            {
                LogMsg("Error generating report, " + ex.Message);
            }

            DateTime endTime = DateTime.Now;
            TimeSpan ts = endTime.Subtract(startTime);
            LogMsg("Done");
            LogMsg("Generating report(s) took '" + ts.ToString(@"hh\:mm\:ss") + "'.");
        }

        private void rbYear_CheckedChanged(object sender, EventArgs e)
        {
            SetCalendarFormat();
        }

        private void button8_Click(object sender, EventArgs e)
        {
            Dictionary<string, long> count = SiteHelper.CountSiteCoordinates();
            foreach (string siteNumber in count.Keys)
            {
                long c = count[siteNumber];
                if (c > 1)
                    LogMsg(siteNumber + ": " + c);
            }
        }

        private void button9_Click(object sender, EventArgs e)
        {
            SiteHelper.ReduceSiteCoordinates();
        }

        private void button10_Click(object sender, EventArgs e)
        {
            Dictionary<string, SiteCoordinates> siteCoordinatesDictionary = SiteHelper.GetSiteCoordinates();
            List<Site> sites = SiteHelper.GetAllSites();
            foreach (Site site in sites)
            {
                if (!siteCoordinatesDictionary.ContainsKey(site.SiteId))
                {
                    LogMsg("Site [" + site.SiteId + "] missing coordinates");
                }
            }
            LogMsg("Done");
        }

        /// <summary>
        /// Invert y coordinates of exported json data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button11_Click(object sender, EventArgs e)
        {
            LogMsg("Not used anymore");
            /*
            StreamReader reader = null;
            StreamWriter writer = null;
            try
            {
                reader = new StreamReader(@"C:\Users\zzzzzzwerfej\Desktop\Export\network\data.json");
                writer = new StreamWriter(@"C:\Users\zzzzzzwerfej\Desktop\Export\network\data.new.json", false);
                string line = reader.ReadLine();
                while (line != null)
                {
                    line = line.Trim();
                    if (line.StartsWith("\"y\": -"))
                    {
                        line = line.Replace("\"y\": -", "\"y\": ");
                        line = line.Replace("--", "-");
                    }
                    else if (line.StartsWith("\"y\": "))
                    {
                        line = line.Replace("\"y\": ", "\"y\": -");
                    }                    
                    
                    writer.Write(line);
                    line = reader.ReadLine();
                }

            }
            catch (Exception ex)
            {
                LogMsg(ex.Message);
            }
            finally
            {
                if (reader != null)
                    reader.Close();
                if (writer != null)
                    writer.Close();
            }*/
        }

        private void button12_Click(object sender, EventArgs e)
        {
            
            StreamReader reader = null;
            StreamWriter writer = null;
            try
            {
                reader = new StreamReader(@"C:\Users\zzzzzzwerfej\Desktop\2011\network\data.json");
                writer = new StreamWriter(@"C:\Users\zzzzzzwerfej\Desktop\2011\network\data.new.json", false);

                string json = reader.ReadToEnd();

                Map map = JsonConvert.DeserializeObject<Map>(json);
                LogMsg("Edges: " + map.edges.Count);
                LogMsg("Edges: " + map.nodes.Count);

                foreach (Node node in map.nodes)
                {
                    double y = node.y;
                    y = y * -1;
                    node.y = y; // invert y value
                }

                string newJson = JsonConvert.SerializeObject(map);
                writer.Write(newJson);
                LogMsg("Done");
            }
            catch (Exception ex)
            {
                LogMsg(ex.Message);
            }
            finally
            {
                if (reader != null)
                    reader.Close();
                if (writer != null)
                    writer.Close();
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void btnDownloadAnalysis_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                AnalysisDownloader downloader = new AnalysisDownloader(LogMsg, txtAnalysisServer.Text);
                string outputDirectory = txtAnalysisDirectory.Text;
                 List<string> sites = GetSelectedSites();
                DailyTransactionsType dailyTransactionsType = DailyTransactionsType.month;
                if (rbDay.Checked)
                    dailyTransactionsType = DailyTransactionsType.day;
                if (rbYear.Checked)
                    dailyTransactionsType = DailyTransactionsType.year;
                downloader.DownloadAnalysis(outputDirectory, dtFrom.Value, dtTo.Value, dailyTransactionsType, sites);
            }
            catch (Exception ex)
            {
                LogMsg(ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
                LogMsg("Done");
            }
        }

        private void btnConvertJson_Click(object sender, EventArgs e)
        {
            string filename = txtInputJsonFilename.Text;
            string tempFilename = filename + "_temp";
            StreamReader reader = null;
            StreamWriter writer = null;
            try
            {
                

                reader = new StreamReader(filename);
                writer = new StreamWriter(tempFilename, false);

                string json = reader.ReadToEnd();

                Map map = JsonConvert.DeserializeObject<Map>(json);
                LogMsg("Edges: " + map.edges.Count);
                LogMsg("Edges: " + map.nodes.Count);

                foreach (Node node in map.nodes)
                {
                    double y = node.y;
                    y = y * -1;
                    node.y = y; // invert y value
                }

                string newJson = JsonConvert.SerializeObject(map);
                writer.Write(newJson);

                

                LogMsg("Done");
            }
            catch (Exception ex)
            {
                LogMsg(ex.Message);
            }
            finally
            {
                if (reader != null)
                    reader.Close();
                if (writer != null)
                    writer.Close();
            }
            try
            {
                File.Move(filename, filename + ".old");
                File.Move(tempFilename, filename);
            }
            catch (Exception ex)
            {
                LogMsg(ex.Message);
            }
        }

        private void btnEarliestStart_Click(object sender, EventArgs e)
        {
            dtFrom.Value = new DateTime(2011, 1, 1);
        }

        private void btnSaveSettings_Click(object sender, EventArgs e)
        {
            ConfigurationManager.AppSettings["TransactionLogsDirectory"] = txtTransactionLogsDirectory.Text;
            Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);

            //make changes
            config.AppSettings.Settings["TransactionLogsDirectory"].Value = txtTransactionLogsDirectory.Text;
            config.AppSettings.Settings["MonthlyAnalysisDirectory"].Value = txtAnalysisDirectory.Text;
            config.AppSettings.Settings["AnalysisServerUrl"].Value = txtAnalysisServer.Text;

            //save to apply changes
            config.Save(ConfigurationSaveMode.Modified);
            ConfigurationManager.RefreshSection("appSettings");
        }


        private void btnAll_Click(object sender, EventArgs e)
        {
            txtProcessSites.Enabled = true;
            List<Site> cvixSites = SiteHelper.GetCvixSites();
            List<Site> vaSites = SiteHelper.GetVASites();
            StringBuilder siteNumbers = new StringBuilder();
            string prefix = "";
            foreach (Site site in cvixSites)
            {
                siteNumbers.Append(prefix);
                siteNumbers.Append(site.SiteId);
                prefix = ",";
            }
            foreach (Site site in vaSites)
            {
                siteNumbers.Append(prefix);
                siteNumbers.Append(site.SiteId);
                prefix = ",";
            }
            txtProcessSites.Text = siteNumbers.ToString();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "Daily Transaction Log Analyzer\nVersion: " + Application.ProductVersion, 
                "About", 
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

    }
}
