/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace DailyTransactionLogAnalyzerEngine
{
    public class AnalysisRollup
    {
        public static void OutputDailyRollup(string monthlyAnalysisDirectory, DailyTransactions dailyTransactions, DateTime date)
        {
            string filename = GetDailyRollupFilename(monthlyAnalysisDirectory, date);
            OutputRollup(filename, dailyTransactions);
        }

        public static void OutputMonthlyRollup(string monthlyAnalysisDirectory, DailyTransactions dailyTransactions,
            int year, int month)
        {
            string filename = GetMonthlyRollupFilename(monthlyAnalysisDirectory, year, month);
            OutputRollup(filename, dailyTransactions);
            /*
            StreamWriter writer = null;
            List<string> siteIds = null;
            if (File.Exists(filename))
            {
                siteIds = LoadSiteIdsFromAnalysisFile(filename);
                writer = new StreamWriter(filename, true);
            }
            else
            {
                writer = new StreamWriter(filename);
                siteIds = OutputCsvHeader(writer);
            }

            Dictionary<string, long> siteCounts = new Dictionary<string, long>();
            
            foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
            {
                foreach (FromSiteCounts fsc in fc.SiteCounts.Values)
                {
                    string sn = CleanSiteNumber(fsc.SiteNumber);
                    if (siteCounts.ContainsKey(sn))
                    {
                        siteCounts[sn] += fsc.TotalTransactions;
                    }
                    else
                    {
                        siteCounts.Add(sn, fsc.TotalTransactions);
                    }
                }
            }

            StringBuilder line = new StringBuilder();
            string prefix = "";
            foreach (string siteId in siteIds)
            {
                line.Append(prefix);

                if (siteCounts.ContainsKey(siteId))
                {
                    line.Append(siteCounts[siteId]);
                }                


                prefix = ",";
            }
            writer.WriteLine(line.ToString());

            line = new StringBuilder();
            prefix = "";
            foreach (string siteId in siteCounts.Keys)
            {
                line.Append(prefix);
                line.Append(siteId);

                prefix = ",";
            }
            writer.WriteLine(line.ToString());

            writer.Flush();
            writer.Close();
            */
        }

        private static void OutputRollup(string filename, DailyTransactions dailyTransactions)
        {
            FileInfo f = new FileInfo(filename);
            if (!f.Directory.Exists)
            {
                Directory.CreateDirectory(f.Directory.FullName);
            }
            StreamWriter writer = null;
            List<string> siteIds = null;
            if (File.Exists(filename))
            {
                siteIds = LoadSiteIdsFromAnalysisFile(filename);
                writer = new StreamWriter(filename, true);
            }
            else
            {
                writer = new StreamWriter(filename);
                siteIds = OutputCsvHeader(writer);
            }

            Dictionary<string, long> siteCounts = new Dictionary<string, long>();

            foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
            {
                foreach (FromSiteCounts fsc in fc.SiteCounts.Values)
                {
                    string sn = CleanSiteNumber(fsc.SiteNumber);
                    if (siteCounts.ContainsKey(sn))
                    {
                        siteCounts[sn] += fsc.TotalTransactions;
                    }
                    else
                    {
                        siteCounts.Add(sn, fsc.TotalTransactions);
                    }
                }
            }

            StringBuilder line = new StringBuilder();
            line.Append(dailyTransactions.SiteNumber);
            line.Append(",");
            foreach (string siteId in siteIds)
            {
                line.Append(",");

                if (siteCounts.ContainsKey(siteId))
                {
                    line.Append(siteCounts[siteId]);
                }
            }
            writer.WriteLine(line.ToString());

            /*
            line = new StringBuilder();
            prefix = "";
            foreach (string siteId in siteCounts.Keys)
            {
                line.Append(prefix);
                line.Append(siteId);

                prefix = ",";
            }
            writer.WriteLine(line.ToString());
            */
            writer.Flush();
            writer.Close();
        }

        private static string CleanSiteNumber(string siteNumber)
        {
            if (siteNumber.StartsWith("1.3.6.1.4.1.3768,"))
            {
                return siteNumber.Substring(17);
            }
            else if (siteNumber.StartsWith("2.16.840.1.113883.3.42.10012.100001.207,"))
            {
                return siteNumber.Substring(40);
            }
            return siteNumber;
        }

        private static List<string> LoadSiteIdsFromAnalysisFile(string filename)
        {
            List<string> siteIds = new List<string>();

            StreamReader reader = new StreamReader(filename);
            string headerLine = reader.ReadLine();
            string[] ids = headerLine.Split(',');
            //foreach (string id in ids)
            // skip the first one because it is a place holder for the site number the data came from
            for(int i= 1; i < ids.Length; i++)
            {
                siteIds.Add(ids[i]);
            }
            reader.Close();

            return siteIds;
        }

        private static List<string> OutputCsvHeader(StreamWriter writer)
        {
            List<Site> sites = SiteHelper.GetAllSites();
            StringBuilder sb = new StringBuilder();
            //sb.Append(siteNumber);
            //string prefix = "";
            List<string> siteIds = new List<string>();
            foreach (Site site in sites)
            {

                siteIds.Add(site.SiteId);
                //sb.Append(prefix);
                sb.Append(",");
                sb.Append(site.SiteId);
                //prefix = ",";                    
            }
            writer.WriteLine(sb.ToString());
            return siteIds;
        }

        private static string GetMonthlyRollupFilename(string monthlyAnalysisDirectory,
            int year, int month)
        {
            //return monthlyAnalysisDirectory + "\\" + year + "-" + month.ToString("D2") + ".csv";
            return monthlyAnalysisDirectory + "\\Rollup\\" + year + "\\" + month.ToString("D2") + "\\" + year + "-" + month.ToString("D2") + ".csv";
        }

        private static string GetDailyRollupFilename(string monthlyAnalysisDirectory, DateTime date)
        {
            return monthlyAnalysisDirectory + "\\Rollup\\" + date.Year+ "\\" + date.Month.ToString("D2") + "\\" + date.ToString("yyyy-MM-dd") + ".csv";
        }
    }
}
