/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VixLogAnalyzerEngine;

namespace DailyTransactionLogAnalyzerEngine
{
    public class DailyHelper
    {

        public string LogsDirectory { get; private set; }
        public event LogMsgDelegate OnLogMsgEvent = null;

        public DailyHelper(string logsDirectory, LogMsgDelegate onLogMsgEvent)
        {
            this.LogsDirectory = logsDirectory;
            this.OnLogMsgEvent = onLogMsgEvent;
            VixLogAnalyzerEngine.TransactionLogEntryDownloader.OnLogMsgEvent += LogMsg;
        }

        public DailyTransactions AnalyzeDailyTransactions(string siteNumber, DateTime date)
        {
            List<TransactionLogEntry> entries = GetTransactionLogEntries(siteNumber, date);
            DailyTransactions dailyTransactions = new DailyTransactions(siteNumber, date, DailyTransactionsType.day, 
                DateTime.Now, DailyTransactions.CurrentVersion);
            dailyTransactions.AnalyzeTransactions(entries);
            /*
            TransactionLogEntry entry = entries[0];
            object val = entry.getPropertyFromParentOrChild("DataSourceBytesReceived", true);
            */
            return dailyTransactions;
        }

        /// <summary>
        /// Count all of the stats for an entire month for a site and generate the results
        /// </summary>
        /// <param name="siteNumber"></param>
        /// <param name="year"></param>
        /// <param name="month"></param>
        /// <returns></returns>
        public DailyTransactions AnalyzeMonthlyTransactions(string siteNumber, int year, int month)
        {
            DateTime startDate = new DateTime(year, month, 1);
            DateTime endOfMonth = new DateTime(startDate.Year, startDate.Month, 1).AddMonths(1).AddDays(-1);
            DailyTransactions dailyTransactions = new DailyTransactions(siteNumber, startDate,
                DailyTransactionsType.month, DateTime.Now, DailyTransactions.CurrentVersion);
            for (int i = startDate.Day; i <= endOfMonth.Day; i++)
            {
                DateTime date = new DateTime(year, month, i);
                LogMsg("Getting entries for site '" + siteNumber + "' from '" + date.ToString("MMMM dd, yyyy"));
                List<TransactionLogEntry> entries = GetTransactionLogEntries(siteNumber, date);
                if(entries != null)
                    dailyTransactions.AnalyzeTransactions(entries);
                entries = null;
                GC.Collect();
            }

            return dailyTransactions;
        }

        private List<TransactionLogEntry> GetTransactionLogEntries(string siteNumber, DateTime date)
        {
            List<TransactionLogEntry> allEntries =
                VixLogAnalyzerEngine.TransactionLogEntryDownloader.GetLogEntriesFromDirectory(LogsDirectory, siteNumber, date);
            if (allEntries == null)
                return null;
            return TransactionLogEntryDownloader.GroupTransactions(allEntries, false);
        }

        private void LogMsg(string msg)
        {
            if (OnLogMsgEvent != null)
                OnLogMsgEvent(msg);
        }
    }
}
