/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DailyTransactionLogAnalyzerEngine
{
    public delegate void LogMsgDelegate(string msg);

    public class DailyTransactionsHelper
    {
        public static double ConvertToKbSec(long bytes, long ms)
        {
            if (ms <= 0)
                return 0f;

            if (ms < 1000)
            {
                ms = ms * 1000;
                bytes = bytes * 1000;
            }
            if (bytes < 1024)
            {
                bytes = bytes * 1024;
                ms = ms * 1024;
            }

            long sec = ms / 1000;
            long kb = bytes / 1024;
            return (double)kb / (double)sec;
        }

        public static SiteIdentificationSource GetSiteIdentificationSource(string facadeName)
        {
            if (facadeName == "Federation WebApp")
                return SiteIdentificationSource.requestingSite;
            return SiteIdentificationSource.respondingSite;
        }

        public static double convertBytesToMb(long bytes)
        {
            double kb = (double)bytes / 1024F;
            double mb = kb / 1024F;
            return mb;
        }

        /// <summary>
        /// Formats the bytes to the highest value - so KB, MB, GB, TB in string format
        /// </summary>
        /// <param name="bytes"></param>
        /// <returns></returns>
        public static string formatBytes(long bytes)
        {
            if (bytes < 1024)
                return bytes + " bytes";
            double kb = (double)bytes / 1024F;
            if (kb < 1024.0)
                return kb.ToString("N2") + " KB";
            double mb = kb / 1024F;
            if (mb < 1024.0)
                return mb.ToString("N2") + " MB";
            double gb = mb / 1024F;
            if (gb < 1024.0)
                return gb.ToString("N2") + " GB";
            double tb = gb / 1024F;
            return tb.ToString("N2") + "TB";
        }
    }
}
