/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;

namespace DailyTransactionLogAnalyzerEngine
{
    public class PowerPointHelper
    {
        public static void CreateSiteRelationsTitleSlide(PowerPoint.Presentation presentation, string dateDescription)
        {
            CreateTitleSlide(presentation, "VistA Imaging Exchange (VIX) Statistics for " + dateDescription);
        }

        public static void CreateTitleSlide(PowerPoint.Presentation presentation, string title)
        {
            PowerPoint.Slide slide = null;
            PowerPoint.Shape titleShape = null;
            PowerPoint.Shape subTitleShape = null;
            try
            {
                // make this first
                slide = presentation.Slides.Add(1, PowerPoint.PpSlideLayout.ppLayoutTitle);

                titleShape = slide.Shapes[1];
                titleShape.TextFrame.TextRange.Text = title;

                subTitleShape = slide.Shapes[2];
                subTitleShape.TextFrame.TextRange.Text = "Generated " + DateTime.Now.ToString();
            }
            finally
            {
                System.Runtime.InteropServices.Marshal.ReleaseComObject(slide);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(titleShape);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(subTitleShape);
            }
        }

        public static void CreateContentSlide(PowerPoint.Presentation presentation, int slideIndex, string title,
            string content, bool bulletedContent)
        {
            CreateContentSlide(presentation, slideIndex, title, content, bulletedContent, 32);
        }

        public static void CreateContentSlide(PowerPoint.Presentation presentation, int slideIndex, string title, 
            string content, bool bulletedContent, int fontSize)
        {
            PowerPoint.Slide slide = null;
            PowerPoint.Shape contentShape = null;
            PowerPoint.Shape titleShape = null;
            try
            {
                slide = presentation.Slides.Add(slideIndex, PowerPoint.PpSlideLayout.ppLayoutText);
                
                titleShape = slide.Shapes[1];
                titleShape.TextFrame.TextRange.Text = title;

                contentShape = slide.Shapes[2];
                //contentShape.TextFrame2.AutoSize = MsoAutoSize.msoAutoSizeTextToFitShape;
                contentShape.TextFrame.TextRange.Font.Size = fontSize;
                
                contentShape.TextFrame.TextRange.Text = content;
                if (bulletedContent)
                {
                    //contentShape.TextFrame.TextRange.ParagraphFormat.Bullet.Type = PowerPoint.PpBulletType.ppBulletMixed;

                    foreach (TextRange2 tr in contentShape.TextFrame2.TextRange.Paragraphs)
                    {
                        int tabCount = GetTabStartCount(tr.Text);
                        if (tabCount > 0)
                        {
                            tr.Text = tr.Text.Trim();
                            tr.ParagraphFormat.IndentLevel = (1 + tabCount);// 2;
                            tr.ParagraphFormat.FirstLineIndent = 10;
                            tr.ParagraphFormat.LeftIndent = (tabCount * 40);
                        }
                        /*
                        if (tr.Text.StartsWith("\t"))
                        {
                            tr.Text = tr.Text.Trim();
                            tr.ParagraphFormat.IndentLevel = 2;
                            tr.ParagraphFormat.FirstLineIndent = 10;
                            tr.ParagraphFormat.LeftIndent = 40;
                        }*/
                    }
                }
                else
                {
                    contentShape.TextFrame.TextRange.ParagraphFormat.Bullet.Type = PowerPoint.PpBulletType.ppBulletNone;
                }
                //contentShape.TextFrame2.WordWrap = MsoTriState.msoTrue;
                
                //contentShape.TextFrame.
                //contentShape.TextFrame.AutoSize = PowerPoint.PpAutoSize.ppAutoSizeShapeToFitText;
                

            }
            finally
            {
                System.Runtime.InteropServices.Marshal.ReleaseComObject(slide);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(titleShape);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(contentShape);
            }
        }

        private static int GetTabStartCount(string text)
        {
            int count = 0;

            string str = text;
            while (str.StartsWith("\t"))
            {
                count++;
                str = str.Substring(1);
            }

            return count;
        }
    }
}
