/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;

namespace DailyTransactionLogAnalyzerEngine
{
    public class SiteHelper
    {
        public static string GetSiteName(string siteId)
        {
            /*
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load("VhaSites.xml");

            XmlNode visnTableNode = xmlDoc.ChildNodes[1];


            XmlNodeList visnNodes = xmlDoc.SelectNodes("/VhaVisnTable/VhaVisn");
            foreach (XmlNode visnNode in visnNodes)
            {
                XmlNode siteNode = visnNode.SelectSingleNode("VhaSite[@ID='" + siteId + "']");
                if (siteNode != null)
                {
                    return siteNode.Attributes["name"].Value;
                }
            }

            return "";
             * */
            Site site = GetSite(siteId);
            if (site == null)
                return "";
            return site.Name;
        }

        public static Site GetSite(string siteId)
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load("VhaSites.xml");

            XmlNode visnTableNode = xmlDoc.ChildNodes[1];

            XmlNodeList visnNodes = xmlDoc.SelectNodes("/VhaVisnTable/VhaVisn");
            foreach (XmlNode visnNode in visnNodes)
            {
                string visnId = visnNode.Attributes["ID"].Value;
                XmlNode siteNode = visnNode.SelectSingleNode("VhaSite[@ID='" + siteId + "']");
                if (siteNode != null)
                {
                    string id = siteNode.Attributes["ID"].Value;
                    string name = siteNode.Attributes["name"].Value;
                    return new Site(id, name, visnId);
                }
            }

            return null;
        }

        public static List<Site> GetCvixSites()
        {
            List<Site> allSites = GetAllSites();
            List<Site> cvixSites = new List<Site>();
            foreach (Site site in allSites)
            {
                if(IsSiteCvix(site.SiteId))
                    cvixSites.Add(site);
            }
            return cvixSites;
        }

        /// <summary>
        /// Get VA Sites (not including CVIX)
        /// </summary>
        /// <returns></returns>
        public static List<Site> GetVASites()
        {
            List<Site> allSites = GetAllSites();
            List<Site> cvixSites = new List<Site>();
            foreach (Site site in allSites)
            {
                if (IsSiteVix(site.SiteId))
                    cvixSites.Add(site);
                /*
                if (site.SiteId.StartsWith("200"))
                    continue;
                if (site.SiteId.StartsWith("CDS"))
                    continue;
                if (site.SiteId == "100")
                    continue;
                cvixSites.Add(site);
                 * */
            }
            return cvixSites;
        }

        public static bool IsSiteCvix(string siteId)
        {
            if (siteId.StartsWith("2001."))
                return true;
            return false;
        }

        public static bool IsSiteVix(string siteId)
        {
            if (siteId.StartsWith("200"))
                return false;
            if (siteId.StartsWith("CDS"))
                return false;
            if (siteId.StartsWith("100"))
                return false;
            return true;
        }

        public static List<Site> GetAllSites()
        {
            List<Site> sites = new List<Site>();
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load("VhaSites.xml");

            //XmlNode siteNode = xmlDoc.SelectSingleNode("VhaVisnTable/VhaVisn/VhaSite[@ID='" + siteId + "']");

            XmlNode visnTableNode = xmlDoc.ChildNodes[1];


            XmlNodeList visnNodes = xmlDoc.SelectNodes("/VhaVisnTable/VhaVisn");
            foreach (XmlNode visnNode in visnNodes)
            {
                string visnId = visnNode.Attributes["ID"].Value;
                XmlNodeList siteNodes = visnNode.SelectNodes("VhaSite");
                foreach (XmlNode siteNode in siteNodes)
                {
                    string id = siteNode.Attributes["ID"].Value;
                    string name = siteNode.Attributes["name"].Value;
                    sites.Add(new Site(id, name, visnId));
                }
            }
            return sites;
        }

        public static string CleanSiteNumber(string siteNumber)
        {
            if (siteNumber.StartsWith("1.3.6.1.4.1.3768,"))
            {
                return siteNumber.Substring(17);
            }
            else if (siteNumber.StartsWith("2.16.840.1.113883.3.42.10012.100001.207,"))
            {
                return siteNumber.Substring(40);
            }
            return siteNumber;
        }

        public static SiteCoordinates GetSiteCoordinates(string siteId)
        {
            Dictionary<string, SiteCoordinates> siteCoordinates = GetSiteCoordinates();
            if (siteCoordinates.ContainsKey(siteId))
            {
                return siteCoordinates[siteId];
            }
            return null;
        }

        public static void ReduceSiteCoordinates()
        {
            List<Site> sites = GetAllSites();
            Dictionary<string, Site> sitesDictionary = new Dictionary<string, Site>();
            foreach (Site site in sites)
            {
                sitesDictionary.Add(site.SiteId, site);
            }

            StreamReader reader = null;
            StreamWriter writer = null;
            try
            {
                reader = new StreamReader("VHA Geocode-reduced.csv");
                writer = new StreamWriter(@"C:\Users\zzzzzzwerfej\Documents\Visual Studio 2010\Projects\DailyTransactionLogAnalyzer\DailyTransactionLogAnalyzerEngine\VHA Geocode-VASitesOnly.csv", false);
                string line = reader.ReadLine();
                line = reader.ReadLine();
                while (line != null)
                {
                    string[] pieces = line.Split(',');
                    if (pieces.Length > 1)
                    {
                        string siteId = pieces[0];
                        if (sitesDictionary.ContainsKey(siteId))
                        {
                            writer.WriteLine(line);
                        }
                    }
                    line = reader.ReadLine();
                }
                writer.Flush();
            }
            finally
            {
                if (reader != null)
                    reader.Close();
                if (writer != null)
                    writer.Close();
            }

        }

        public static Dictionary<string, long> CountSiteCoordinates()
        {
            Dictionary<string, long> siteCoordinates = new Dictionary<string, long>();

            StreamReader reader = null;
            try
            {
                reader = new StreamReader("VHA Geocode.csv");
                string line = reader.ReadLine();
                // skip header line
                line = reader.ReadLine();
                while (line != null)
                {

                    string[] pieces = line.Split(',');
                    if (pieces.Length > 2)
                    {
                        string siteNumber = pieces[0];
                        string latitude = pieces[1];
                        string longitude = pieces[2];
                        if (siteNumber.Length > 0 && latitude.Length > 0 && longitude.Length > 0)
                        {
                            
                            if (siteCoordinates.ContainsKey(siteNumber))
                            {
                                siteCoordinates[siteNumber]++;
                            }
                            else
                            {
                                siteCoordinates.Add(siteNumber, 1);
                            }
                             

                        }
                    }
                    line = reader.ReadLine();
                }
            }
            finally
            {
                if (reader != null)
                    reader.Close();
            }

            return siteCoordinates;
        }

        public static Dictionary<string, SiteCoordinates> GetSiteCoordinates()
        {
            Dictionary<string, SiteCoordinates> siteCoordinates = new Dictionary<string, SiteCoordinates>();

            StreamReader reader = null;
            try
            {
                reader = new StreamReader("VHA Geocode.csv");
                string line = reader.ReadLine();
                // skip header line
                line = reader.ReadLine();
                while (line != null)
                {
                    
                    string[] pieces = line.Split(',');
                    if (pieces.Length > 2)
                    {
                        string siteNumber = pieces[0];
                        string latitude = pieces[1];
                        string longitude = pieces[2];
                        if (siteNumber.Length > 0 && latitude.Length > 0 && longitude.Length > 0)
                        {
                            /*
                            if (siteCoordinates.ContainsKey(siteNumber))
                            {
                                siteCoordinates[siteNumber] = new SiteCoordinates(siteNumber, latitude, longitude);
                            }
                            else
                            {
                                siteCoordinates.Add(siteNumber, new SiteCoordinates(siteNumber, latitude, longitude));
                            }
                             * */
                            if (!siteCoordinates.ContainsKey(siteNumber))
                            {
                                siteCoordinates.Add(siteNumber, new SiteCoordinates(siteNumber, latitude, longitude));
                            }
                             
                        }
                    }
                    line = reader.ReadLine();
                }
            }
            finally
            {
                if (reader != null)
                    reader.Close();
            }

            return siteCoordinates;
        }
    }

    public class Site
    {
        public string SiteId { get; private set; }
        public string Name { get; private set; }
        public string VisnId { get; private set; }

        public Site(string siteId, string name, string visnId)
        {
            this.SiteId = siteId;
            this.Name = name;
            this.VisnId = visnId;
        }

        public override string ToString()
        {
            return this.Name + " [" + SiteId + "]";
        }

        public string ToString(bool simple)
        {
            if (simple)
            {
                string val = this.Name + "-" + SiteId;                
                if (val.Length >= 31)
                {
                    val = this.Name;
                }
                if (val.Length >= 31)
                {
                    val = val.Substring(0, 30);
                }
                val = val.Replace('\\', ' ');
                val = val.Replace('/', ' ');
                return val;
            }
            else
                return ToString();            
        }
    }

    public class SiteCoordinates
    {
        public string SiteId { get; private set; }
        public string Latitude { get; private set; }
        public string Longitude { get; private set; }

        public SiteCoordinates(string siteId, string latitude, string longitude)
        {
            this.SiteId = siteId;
            this.Latitude = latitude;
            this.Longitude = longitude;
        }
    }
}
