/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Excel = Microsoft.Office.Interop.Excel;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;

namespace DailyTransactionLogAnalyzerEngine.reports
{
    public abstract class AbstractExcelReport : ReportGenerator
    {


        protected void ConvertBytesValuesToMB(Excel.Worksheet sheet)
        {
            // don't do column 1 - it has facade names
            for (int i = sheet.UsedRange.Columns.Count; i > 1; i--)
            {
                // skip row 1 - it contains site numbers and is never empty
                for (int j = 2; j <= sheet.UsedRange.Rows.Count; j++)
                {
                    Excel.Range r = sheet.Cells[j, i];
                    string val = r.Value2 + "";
                    if (val != "")
                    {
                        long bytes = long.Parse(val);
                        double mb = DailyTransactionsHelper.convertBytesToMb(bytes);

                        sheet.Cells[j, i] = mb;// newVal;
                    }
                }
            }
        }

        protected Excel.ChartObject CreateWorksheetChart(Excel.Worksheet sheet, string dateDescription, Site site,
            bool stacked, ChartType chartType)
        {
            return CreateWorksheetChart(sheet, dateDescription, site, stacked, chartType, false);
        }

        protected Excel.ChartObject CreateWorksheetChart(Excel.Worksheet sheet, string dateDescription, Site site,
            bool stacked, ChartType chartType, bool lineGraph)
        {
            return CreateWorksheetChart(sheet, dateDescription, site, stacked, chartType, lineGraph, true);
        }

        protected Excel.ChartObject CreateWorksheetChart(Excel.Worksheet sheet, string dateDescription, Site site,
            bool stacked, ChartType chartType, bool lineGraph, bool top5Chart)
        {
            return CreateWorksheetChart(sheet, dateDescription, site, stacked, chartType, lineGraph, top5Chart, null);
        }

        protected Excel.ChartObject CreateWorksheetChart(Excel.Worksheet sheet, string dateDescription, Site site,
            bool stacked, ChartType chartType, bool lineGraph, bool top5Chart, string sheetName)
        {
            string rawName = sheetName;            
            if (string.IsNullOrEmpty(rawName))
            {
                rawName = sheet.Name;
            }
            string name = rawName;

            Excel.ChartObjects charts = sheet.ChartObjects(Type.Missing);
            Excel.ChartObject chart = null;
            for (int j = 1; j <= charts.Count; j++)
            {
                Excel.ChartObject co = (Excel.ChartObject)charts.Item(j);
                if (co.Name == name)
                {
                    chart = co;
                    break;
                }
            }

            if (chart == null)
            {
                chart = charts.Add(20, 80, 1000, 500);
            }

            Excel.Chart chartPage = chart.Chart;
            chart.Name = name;
            Excel.Range chartRange = sheet.UsedRange;

            chartPage.HasTitle = true;
            string title = "";
            if (site != null)
            {
                title = site.ToString();
                if (top5Chart)
                {
                    title = title + " Top 5";
                }
                title += " - " + dateDescription;
            }
            else
            {
                if (top5Chart)
                {
                    title = "Top 5 ";
                }
                title += name + " - " + dateDescription;
            }
            switch (chartType)
            {
                case ChartType.transactions:
                    title += " Requests";
                    break;
                case ChartType.bytes:
                    title += " MB Transferred";
                    break;
                case ChartType.facadeBytes:
                    title += " MB To Client";
                    break;
                case ChartType.imageCounts:
                    title += " Successful Image Responses";
                    break;

            }
            chartPage.ChartTitle.Text = title;

            chartPage.SetSourceData(chartRange, Type.Missing);
            chartPage.DisplayBlanksAs = Excel.XlDisplayBlanksAs.xlNotPlotted;
            if (lineGraph)
            {
                chartPage.ChartType = Excel.XlChartType.xlLineMarkers;
            }
            else
            {
                if (stacked)
                {
                    chartPage.ChartType = Excel.XlChartType.xlColumnStacked;
                }
                else
                {
                    chartPage.ChartType = Excel.XlChartType.xlColumnClustered;
                }
            }

            string numberFormat = "";

            switch (chartType)
            {
                case ChartType.bytes:
                case ChartType.decimalCounts:
                case ChartType.facadeBytes:
                    numberFormat = "###,###,##0.00";
                    break;   
                default:
                    numberFormat = "###,###,##0";
                    break;
            }

            Excel.Axis yAxis = chartPage.Axes(Excel.XlAxisType.xlValue, Excel.XlAxisGroup.xlPrimary);
            yAxis.HasTitle = true;
            yAxis.TickLabels.NumberFormat = numberFormat;
            switch (chartType)
            {
                case ChartType.transactions:
                    yAxis.AxisTitle.Text = "Number of Requests";
                    break;
                case ChartType.bytes:                
                    yAxis.AxisTitle.Text = "MB Transferred";
                    break;
                case ChartType.facadeBytes:
                    yAxis.AxisTitle.Text = "MB To Client";
                    break;
                case ChartType.imageCounts:
                    yAxis.AxisTitle.Text = "Successful Image Responses";
                    break;
                default:
                    yAxis.AxisTitle.Text = rawName;
                    break;
            }

            Excel.Axis xAxis = (Excel.Axis)chartPage.Axes(Excel.XlAxisType.xlCategory, Excel.XlAxisGroup.xlPrimary);
            xAxis.HasTitle = true;
            if (lineGraph || !top5Chart)
            {
                xAxis.AxisTitle.Text = "Date";
            }
            else
            {
                xAxis.AxisTitle.Text = "Sites";
            }
            xAxis.TickLabels.Orientation = Excel.XlTickLabelOrientation.xlTickLabelOrientationUpward;

            // applies to all series
            //if (!stacked)
            //{
                chartPage.ApplyDataLabels(Excel.XlDataLabelsType.xlDataLabelsShowLabel, false, false, false, false, false, true, false);
                Excel.SeriesCollection seriesCollection = chartPage.SeriesCollection(Type.Missing);
                foreach (Excel.Series series in seriesCollection)
                {
                    //series.Format.Line.ForeColor.RGB = (int)Microsoft.Office.Interop.Excel.XlRgbColor.rgbOrange;
                    Excel.DataLabels dataLabels = series.DataLabels(Type.Missing);
                    foreach (Excel.DataLabel dataLabel in dataLabels)
                    //for (int j = 0; j < dataLabels.Count; j++)
                    {
                        //Excel.DataLabel dataLabel = dataLabels.Item(j);
                        dataLabel.NumberFormat = numberFormat;                        
                    }
                }
            //}


            // this didn't work - trying to assign labels to a specific series
            //Excel.SeriesCollection chartSeries = chartPage.SeriesCollection();
            //Excel.Series series = chartSeries.Item(chartSeries.Count - 1);
            //series.ApplyDataLabels(Excel.XlDataLabelsType.xlDataLabelsShowLabel, false, false, false, false, true, false, false);

            return chart;
        }

        protected int OutputChartToPpt(PowerPoint.Presentation presentation, Excel.ChartObject chart, string description)
        {
            return OutputChartToPpt(presentation, chart, -1, description);
        }

        protected int OutputChartToPpt(PowerPoint.Presentation presentation, Excel.ChartObject chart, int slideIndex, string description)
        {
            if (slideIndex <= 0)
            {
                slideIndex = presentation.Slides.Count + 1;
            }
            PowerPoint.Slide slide = presentation.Slides.Add(slideIndex, PowerPoint.PpSlideLayout.ppLayoutBlank);

            chart.CopyPicture(Excel.XlPictureAppearance.xlScreen, Excel.XlCopyPictureFormat.xlPicture);
            slide.Shapes.PasteSpecial(PowerPoint.PpPasteDataType.ppPasteDefault);

            foreach (PowerPoint.Shape shape in slide.Shapes)
            {
                if (shape.Type == MsoShapeType.msoPicture)
                {
                    shape.Top = 87.84976F;
                    shape.Left = 33.98417F;
                    shape.Height = 422.7964F;
                    shape.Width = 646.5262F;
                }
            }

            if (description != null && description.Length > 0)
            {
                PowerPoint.Shape textShape =
                    slide.Shapes.AddTextbox(MsoTextOrientation.msoTextOrientationHorizontal, 33.98417F, 415.64616F, 646.5262F, 100.0F);
                textShape.TextFrame.TextRange.Text = description;
            }
            return slide.SlideIndex;
        }
        protected void CreateTotalsSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts,
            int slideIndex)
        {
            CreateTotalsSlide(presentation, totalFacadeCounts, slideIndex, false);
        }


        protected void CreateTotalsSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts,
            int slideIndex, bool includeClientBytes)
        {
            CreateTotalsSlide(presentation, totalFacadeCounts, slideIndex, includeClientBytes, null, null);
        }

        protected void CreateTotalsSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts,
            int slideIndex, bool includeClientBytes, DateTime? startDate, DateTime? endDate)
        {
            CreateTotalsSlide(presentation, totalFacadeCounts, slideIndex, includeClientBytes, startDate, endDate, 0);
        }

        protected void CreateTotalsSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts,
            int slideIndex, bool includeClientBytes, DateTime? startDate, DateTime? endDate, int specifiedFontSize)
        {
            CreateTotalsSlide(presentation, totalFacadeCounts, slideIndex, includeClientBytes, true, startDate, endDate, specifiedFontSize);
        }

        protected void CreateTotalsSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts,
            int slideIndex, bool includeClientBytes, bool includeRequestCount, DateTime? startDate, DateTime? endDate, int specifiedFontSize)
        {
            //CreateTotalsSlide(presentation, facadeTotalBytes, facadeTotalTransactions, 3);

            StringBuilder requestsContent = new StringBuilder();
            requestsContent.Append("Total Requests\r");
            StringBuilder bytesContent = new StringBuilder();
            bytesContent.Append("Total Bytes\r");
            StringBuilder imagesConent = new StringBuilder();
            imagesConent.Append("Successful Image Responses\r");
            StringBuilder clientBytesContent = new StringBuilder();
            clientBytesContent.Append("Total Client Bytes\r");

            foreach (string facadeName in totalFacadeCounts.Keys)
            {
                if (includeRequestCount)
                {
                    long requests = totalFacadeCounts[facadeName].TotalRequests;
                    requestsContent.Append("\t" + facadeName + ": " + requests.ToString("N0") + "\r");
                }

                long bytes = totalFacadeCounts[facadeName].TotalBytes;
                bytesContent.Append("\t" + facadeName + ": " + DailyTransactionsHelper.formatBytes(bytes) + "\r");

                long images = totalFacadeCounts[facadeName].SuccessfulImageResponses;
                imagesConent.Append("\t" + facadeName + ": " + images.ToString("N0") + "\r");

                if (includeClientBytes)
                {
                    long clientBytes = totalFacadeCounts[facadeName].ClientBytes;
                    if (clientBytes > 0)
                    {
                        clientBytesContent.Append("\t" + facadeName + ": " + DailyTransactionsHelper.formatBytes(clientBytes) + "\r");
                    }
                }
            }

            StringBuilder content = new StringBuilder();
            string dateDescription = null;
            if (startDate.HasValue && endDate.HasValue)
            {
                dateDescription = "From " + startDate.Value.ToString("MMMM yyyy") + " to " + endDate.Value.ToString("MMMM yyyy") + "\r";
            }
            if (dateDescription != null)
                content.Append(dateDescription);
            if(includeRequestCount)
                content.Append(requestsContent.ToString());
            content.Append(bytesContent.ToString());
            content.Append(imagesConent.ToString());
            if (includeClientBytes)
                content.Append(clientBytesContent.ToString());

            int fontSize = 22;
            if (specifiedFontSize > 0)
            {
                fontSize = specifiedFontSize;
            }
            else
            {
                if (includeClientBytes)
                    fontSize = 16;
            }

            PowerPointHelper.CreateContentSlide(presentation, slideIndex, "Totals", content.ToString(), true, fontSize);
        }

        protected void CreateConnectionsTotalsSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts,
            int slideIndex, DateTime? startDate, DateTime? endDate, int specifiedFontSize)
        {
            StringBuilder bseLoginsContent = new StringBuilder();
            bseLoginsContent.Append("BSE Logins\r");
            StringBuilder capriLoginsContent = new StringBuilder();
            capriLoginsContent.Append("CAPRI Logins\r");

            Dictionary<string, StringBuilder> protocolContent = new Dictionary<string, StringBuilder>();

            foreach (string facadeName in totalFacadeCounts.Keys)
            {
                FacadeTotals ft = totalFacadeCounts[facadeName];

                long bse = ft.BseLogins;
                bseLoginsContent.Append("\t" + facadeName + ": " + bse.ToString("N0") + "\r");

                long capri = ft.CapriLogins;
                capriLoginsContent.Append("\t" + facadeName + ": " + capri.ToString("N0") + "\r");

                foreach (string protocolName in ft.ProtocolCounts.Keys)
                {
                    if (!protocolContent.ContainsKey(protocolName))
                    {
                        StringBuilder protocolStringBuilder = new StringBuilder();
                        protocolStringBuilder.Append(protocolName + " Protocol\r");
                        protocolContent.Add(protocolName, protocolStringBuilder);
                    }
                    protocolContent[protocolName].Append("\t" + facadeName + ": " + ft.ProtocolCounts[protocolName].Count.ToString("N0") + "\r");
                }
            }

            StringBuilder content = new StringBuilder();
            string dateDescription = null;
            if (startDate.HasValue && endDate.HasValue)
            {
                dateDescription = "From " + startDate.Value.ToString("MMMM yyyy") + " to " + endDate.Value.ToString("MMMM yyyy") + "\r";
            }
            if (dateDescription != null)
                content.Append(dateDescription);
            content.Append(bseLoginsContent.ToString());
            content.Append(capriLoginsContent.ToString());
            foreach (StringBuilder protocolStringBuilders in protocolContent.Values)
            {
                content.Append(protocolStringBuilders.ToString());
            }
            
            int fontSize = 22;
            if (specifiedFontSize > 0)
            {
                fontSize = specifiedFontSize;
            }

            PowerPointHelper.CreateContentSlide(presentation, slideIndex, "Totals", content.ToString(), true, fontSize);
        }

        protected void CreateTotalsSlide(PowerPoint.Presentation presentation, Dictionary<string, long> facadeTotalBytes, Dictionary<string, long> facadeTotalTransactions, int slideIndex)
        {
            StringBuilder content = new StringBuilder();
            //long count = 0;
            foreach (string facadeName in facadeTotalTransactions.Keys)
            {
                long trans = facadeTotalTransactions[facadeName];
                //count += trans;
                content.Append(facadeName + ": " + trans.ToString("N0") + "\r");
            }
            //content.Append("Total Requests: " + count.ToString("N0") + "\r");
            //count = 0;
            foreach (string facadeName in facadeTotalBytes.Keys)
            {
                long bytes = facadeTotalBytes[facadeName];
                //count += bytes;
                content.Append(facadeName + ": " + DailyTransactionsHelper.formatBytes(bytes) + "\r");// DailyTransactionsHelper.convertBytesToMb(bytes).ToString("N2") + " MB \r");
            }
            //content.Append("Total Transferred: " + convertBytesToMb(count).ToString("N2") + " MB \r");

            PowerPointHelper.CreateContentSlide(presentation, slideIndex, "Totals", content.ToString(), true);
        }

        protected string GetMonthlyRollupFilename(string monthlyAnalysisDirectory,
            int year, int month)
        {
            return monthlyAnalysisDirectory + "\\Rollup\\" + year + "\\" + month.ToString("D2") + "\\" + year + "-" + month.ToString("D2") + ".xlsx";
        }

        protected void updateSiteHeaderColumns(Excel.Worksheet sheet, Dictionary<string, Site> siteDictionary)
        {
            int row = 1;
            for (int i = 2; i <= sheet.UsedRange.Columns.Count; i++)
            {
                Excel.Range r = sheet.Cells[row, i];
                string val = r.Value2 + "";
                if (siteDictionary.ContainsKey(val))
                {
                    sheet.Cells[row, i] = siteDictionary[val].ToString();//  .Name + "[" + siteDictionary[val].SiteId + "]";
                }
            }
        }

        /// <summary>
        /// Updates the header columns to show site names in addition to site numbers
        /// </summary>
        /// <param name="workbook"></param>
        protected void updateSiteHeaderColumns(Excel.Workbook workbook)
        {
            Dictionary<string, Site> siteDictionary = GetSitesDictionary();
            foreach (Excel.Worksheet sheet in workbook.Sheets)
            {
                updateSiteHeaderColumns(sheet, siteDictionary);
            }
        }

        protected List<SiteDailyTransactionsList> SortByName(Dictionary<string, List<DailyTransactions>> siteDailyTransactionsDictionary, Dictionary<string, Site> sites)
        {
            List<SiteDailyTransactionsList> siteDailyTransactions = new List<SiteDailyTransactionsList>();
            foreach (string siteNumber in siteDailyTransactionsDictionary.Keys)
            {
                string siteName = "";
                if (sites.ContainsKey(siteNumber))
                {
                    siteName = sites[siteNumber].Name;
                }
                siteDailyTransactions.Add(new SiteDailyTransactionsList(siteNumber, siteName, siteDailyTransactionsDictionary[siteNumber]));
            }
            siteDailyTransactions.Sort(new SiteDailyTransactionsListComparer());
            return siteDailyTransactions;
        }

        /// <summary>
        /// Create a chart and slide to show each month how many sites reported data
        /// </summary>
        /// <param name="workbook"></param>
        /// <param name="pptPres"></param>
        /// <param name="monthReceviedDataCount"></param>
        /// <param name="slideIndex"></param>
        protected void OutputResponseCountChartAndSlide(Excel.Workbook workbook, PowerPoint.Presentation pptPres, Dictionary<DateTime, int> monthReceviedDataCount,
            int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            sheet.Name = "Active Sites Count";
            DateTime? sDate = null;
            DateTime? eDate = null;

            int col = 2;
            sheet.Cells[2, 1] = "Number of Active Sites";

            List<DateCount> dateCounts = new List<DateCount>();
            foreach (DateTime dt in monthReceviedDataCount.Keys)
            {
                dateCounts.Add(new DateCount(dt, monthReceviedDataCount[dt]));
            }
            dateCounts.Sort(new DateCountComparer());

            foreach (DateCount dateCount in dateCounts)
            {
                DateTime dt = dateCount.Date;
                sheet.Cells[1, col] = dt.ToString("MMMM, yyyy");
                if (!sDate.HasValue)
                {
                    sDate = dt;
                }
                eDate = dt;
                int count = dateCount.Count;
                sheet.Cells[2, col] = count;
                col++;
            }


            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, false, ChartType.counts, true, false);
            string description = dtDescription + ", number of sites with at least one relevant request";
            OutputChartToPpt(pptPres, chart, slideIndex, description);
        }

        protected double calculateDailyAverage(long value, DateTime month)
        {
            int daysInMonth = getDaysInMonth(month);
            double result = (double)value / (double)daysInMonth;
            return result;
        }

        protected int getDaysInMonth(DateTime month)
        {
            DateTime dt = new DateTime(month.Year, month.Month, 1); // first day of the month
            DateTime lastDay = dt.AddMonths(1).AddDays(-1); // move to the last day of the month
            return lastDay.Day;
        }
    }

    public class FacadeTotals : LoginTotals
    {
        public long TotalRequests { get; set; }
        public long SuccessfulImageResponses { get; set; }
        public long TotalBytes { get; set; }
        /// <summary>
        /// Bytes sent to the client (for client facades only)
        /// </summary>
        public long ClientBytes { get; set; }
        public Dictionary<string, ProtocolCount> ProtocolCounts { get; private set; }

        public FacadeTotals()
            : base()
        {
            TotalRequests = 0L;
            SuccessfulImageResponses = 0L;
            TotalBytes = 0L;
            ClientBytes = 0L;
            ProtocolCounts = new Dictionary<string, ProtocolCount>();
        }
    }

    public class ProtocolCount
    {
        public string Name { get; private set; }
        public long Count { get; set; }

        public ProtocolCount(string name)
        {
            this.Name = name;
            this.Count = 0L;
        }
    }

    public class LoginTotals
    {
        public long BseLogins { get; set; }
        public long CapriLogins { get; set; }

        public LoginTotals()
        {
            BseLogins = 0L;
            CapriLogins = 0L;
        }
    }

    public class FacadeTotalsByMonth
    {
        public Dictionary<DateTime, FacadeTotals> MonthlyTotals { get; set; }

        public FacadeTotalsByMonth()
        {
            MonthlyTotals = new Dictionary<DateTime, FacadeTotals>();
        }
    }

    public enum ChartType
    {
        transactions, bytes, counts, decimalCounts, imageCounts, facadeBytes, integerCounts
    }

    public class SiteDailyTansactions
    {
        public string SiteName { get; private set; }
        public DailyTransactions DailyTransactions { get; private set; }

        public SiteDailyTansactions(string siteName, DailyTransactions dailyTransactions)
        {
            this.SiteName = siteName;
            this.DailyTransactions = dailyTransactions;
        }
    }

    public class SiteDailyTansactionsComparer : IComparer<SiteDailyTansactions>
    {
        public int Compare(SiteDailyTansactions x, SiteDailyTansactions y)
        {
            //return (-1) * (x.SiteName.CompareTo(y.SiteName));
            return (x.SiteName.CompareTo(y.SiteName));
        }
    }

    public class SiteCount
    {
        public string SiteId { get; private set; }
        public long Count { get; set; }

        public SiteCount(string siteId, long count)
        {
            this.SiteId = siteId;
            this.Count = count;
        }
    }

    public class SiteCountComparer : IComparer<SiteCount>
    {
        public int Compare(SiteCount x, SiteCount y)
        {
            // sort highest to lowest
            return (-1) * (x.Count.CompareTo(y.Count));
        }
    }

    public class HighValuedCount
    {
        public bool HighValue { get; private set; }
        public long Count { get; private set; }

        public HighValuedCount(long count, bool highValue)
        {
            this.HighValue = highValue;
            this.Count = count;
        }
    }

    public enum AverageIncrement
    {
        day, month
    }
}
