/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
//using Excel = Microsoft.Office.Interop.Excel;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;
using System.IO;
using Excel = Microsoft.Office.Interop.Excel;

namespace DailyTransactionLogAnalyzerEngine.reports
{
    public class AveragesReport : AbstractExcelReport
    {
        public AveragesReport()
        {
            this.Name = "Averages Report";
            this.Description = "Creates slides that list the averages for month and daily usage of the VIX";
            this.ExcelOutputGroupType = ExcelOutputGroupType.allDates;
        }

        protected override void GenerateReport(string analysisDirectory, Dictionary<string, List<DailyTransactions>> sitesDailyTransactionsDictionary,
            ReportDateType reportDateType, DateTime startDate, DateTime endDate)
        {
            switch (reportDateType)
            {
                case ReportDateType.month:
                    LogMsg("Generating average report for " + OutputDate(startDate, reportDateType) + " to " + OutputDate(endDate, reportDateType) + ".");
                    OutputAverages(analysisDirectory, sitesDailyTransactionsDictionary, startDate, endDate);
                    break;
                case ReportDateType.year:
                    throw new NotImplementedException();
                default:
                    throw new NotImplementedException();
            }  
        }

        protected override ReportSiteType GetReportSiteSupportedType()
        {
            return ReportSiteType.vix;
        }

        protected override void GenerateReport(string analysisDirectory, List<DailyTransactions> dailyTransactionsList, 
            ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        protected override void GenerateReport(string analysisDirectory, DailyTransactions dailyTransactions,
            ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        private void OutputAverages(string monthlyAnalysisDirectory, Dictionary<string, List<DailyTransactions>> sitesDailyTransactionsDictionary, DateTime startDate, DateTime endDate)
        {
            string filename = monthlyAnalysisDirectory + "\\Rollup\\Averages.pdf";
            FileInfo f = new FileInfo(filename);
            if (!f.Directory.Exists)
            {
                f.Directory.Create();
            }

            if (File.Exists(filename))
                File.Delete(filename);
            string xlsxFilename = filename + ".xlsx";
            if (File.Exists(xlsxFilename))
                File.Delete(xlsxFilename);
            string pptFilename = filename + ".pptx";
            if (File.Exists(pptFilename))
                File.Delete(pptFilename);

            Excel.Application oXL = new Excel.Application();
            oXL.Visible = false;

            PowerPoint.Application ppt = new PowerPoint.Application();

            PowerPoint.Presentation pptPres = ppt.Presentations.Add(MsoTriState.msoFalse);

            Excel.Workbook workbook = null;
            try
            {
                workbook = oXL.Workbooks.Add(Type.Missing);
                // delete all existing sheets except first (Excel creates 3 by default, must keep at least 1)                
                for (int i = workbook.Sheets.Count; i > 1; i--)
                {
                    Excel.Worksheet sheet = workbook.Sheets[i];
                    sheet.Delete();
                }

                List<string> siteIds = new List<string>();
                long totalRequests = 0L;

                Dictionary<string, Site> sitesDictionary = GetSitesDictionary();
                Dictionary<string, FacadeTotalsByMonth> facadeMonthTotals = new Dictionary<string, FacadeTotalsByMonth>(); // totals for each facade by month

                Dictionary<string, FacadeTotals> facadeTotals = new Dictionary<string, FacadeTotals>(); // totals for each facade
                Dictionary<DateTime, int> monthReceviedDataCount = new Dictionary<DateTime, int>(); // number of sites that had data each month

                List<SiteDailyTransactionsList> sitesDailyTransactionsList = SortByName(sitesDailyTransactionsDictionary, sitesDictionary);

                foreach (SiteDailyTransactionsList siteDailyTransactionList in sitesDailyTransactionsList)// string siteNumber in sitesDailyTransactionsDictionary.Keys)
                {
                    string siteNumber = siteDailyTransactionList.SiteNumber;
                    Site thisSite = SiteHelper.GetSite(siteNumber);
                    LogMsg("Calculating totals for site '" + thisSite.Name + "'");
                    
                    List<DailyTransactions> dailyTransactionsList = siteDailyTransactionList.DailyTransactionsList;
                    
                    // each DailyTransaction represents a month of data
                    foreach (DailyTransactions dailyTransactions in dailyTransactionsList)
                    {
                        totalRequests += dailyTransactions.TransactionCounts.TotalParentTransactions;
                        if (dailyTransactions.TransactionCounts.TotalParentTransactions > 0)
                        {
                            if (monthReceviedDataCount.ContainsKey(dailyTransactions.Date))
                            {
                                monthReceviedDataCount[dailyTransactions.Date]++;
                            }
                            else
                            {
                                monthReceviedDataCount.Add(dailyTransactions.Date, 1);
                            }
                        }
                        foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
                        {

                            if (!facadeTotals.ContainsKey(fc.FacadeName))
                            {
                                facadeTotals.Add(fc.FacadeName, new FacadeTotals());
                            }
                            facadeTotals[fc.FacadeName].TotalRequests += fc.TotalTransactions;
                            facadeTotals[fc.FacadeName].TotalBytes += fc.RelevantBytes;
                            facadeTotals[fc.FacadeName].SuccessfulImageResponses += fc.SuccessfulImageResponses;
                            if (fc.IsClientFacingFacade)
                                facadeTotals[fc.FacadeName].ClientBytes += fc.SuccessfulImageBytes;

                            
                            if (!facadeMonthTotals.ContainsKey(fc.FacadeName))
                                facadeMonthTotals.Add(fc.FacadeName, new FacadeTotalsByMonth());

                            if (!facadeMonthTotals[fc.FacadeName].MonthlyTotals.ContainsKey(dailyTransactions.Date))                            
                                facadeMonthTotals[fc.FacadeName].MonthlyTotals.Add(dailyTransactions.Date, new FacadeTotals());
                            
                            facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].TotalRequests += fc.TotalTransactions;
                            facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].TotalBytes += fc.RelevantBytes;
                            facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].SuccessfulImageResponses += fc.SuccessfulImageResponses;
                            if (fc.IsClientFacingFacade)
                                facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].ClientBytes += fc.SuccessfulImageBytes;
                             
                        }

                    }// for loop for each month

                    // done with calculations for current site
                } // loop for each site

                LogMsg("Generating title and totals charts");
                PowerPointHelper.CreateTitleSlide(pptPres, "Averages"); // slide 1
                CreateAveragesPresentationExplanationSlide(pptPres);

                OutputTotalRequestsSlide(pptPres, totalRequests, 3, startDate, endDate);
                CreateTotalsSlide(pptPres, facadeTotals, 4, true, startDate, endDate);
                CreateAveragesSlide(pptPres, facadeTotals, 5, startDate, endDate, AverageIncrement.month);
                CreateAveragesSlide(pptPres, facadeTotals, 6, startDate, endDate, AverageIncrement.day);
                
                OutputResponseCountChartAndSlide(workbook, pptPres, monthReceviedDataCount, 7);
                OutputDailyAveragesChartAndSlide(workbook, pptPres, facadeMonthTotals, ChartType.transactions, 8);
                OutputDailyAveragesChartAndSlide(workbook, pptPres, facadeMonthTotals, ChartType.bytes, 9);
                OutputDailyAveragesChartAndSlide(workbook, pptPres, facadeMonthTotals, ChartType.imageCounts, 10);
                OutputDailyAveragesChartAndSlide(workbook, pptPres, facadeMonthTotals, ChartType.facadeBytes, 11);

                workbook.SaveAs(xlsxFilename);

                pptPres.SaveAs(pptFilename);
                pptPres.SaveAs(filename, PowerPoint.PpSaveAsFileType.ppSaveAsPDF, MsoTriState.msoFalse);
            }
            finally
            {
                if (workbook != null)
                    workbook.Close(false);
                if (pptPres != null)
                    pptPres.Close();
                oXL.Quit();
                ppt.Quit();
                System.Runtime.InteropServices.Marshal.ReleaseComObject(workbook);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(oXL);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(ppt);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(pptPres);

                oXL = null;
                GC.Collect();
            }
        }

        private void OutputTotalRequestsSlide(PowerPoint.Presentation pptPres, long totalRequests, int slideIndex,
            DateTime startDate, DateTime endDate)
        {
            StringBuilder description = new StringBuilder();

            int months = 0;
            DateTime date = startDate;
            while (date.CompareTo(endDate) < 1)
            {
                months++;
                date = date.AddMonths(1);
            }

            // set to the last day of the month
            endDate = endDate.AddMonths(1).AddDays(-1);

            TimeSpan span = endDate.Subtract(startDate);
            int days = (int)span.TotalDays + 1; // add one to count the last day of the month

            string dtDescription = startDate.ToString("MMMM yyyy") + " to " + endDate.ToString("MMMM yyyy");
            description.AppendLine(dtDescription);
            description.AppendLine("Total Requests (all facades): " + totalRequests.ToString("N0"));

            double monthAverage = (double)totalRequests / (double)months;
            double dayAverage = (double)totalRequests / (double)days;

            description.AppendLine("Average Requests Per Month: " + monthAverage.ToString("N2"));
            description.AppendLine("Average Requests Per Day: " + dayAverage.ToString("N2"));

            PowerPointHelper.CreateContentSlide(pptPres, slideIndex, "All VIX Requests", description.ToString(), true, 22);
        }

        private void OutputDailyAveragesChartAndSlide(Excel.Workbook workbook, PowerPoint.Presentation pptPres, Dictionary<string, FacadeTotalsByMonth> facadeMonthTotals,
            ChartType chartType, int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            string chartDescription = "";
            switch (chartType)
            {
                case ChartType.bytes:
                    sheet.Name = "Avg MB Per Day";
                    chartDescription = "Average MB per day transfered (across the WAN) during each month";
                    break;
                case ChartType.transactions:
                    sheet.Name = "Avg Requests Per Day";
                    chartDescription = "Average requests per day during each month";
                    break;
                case ChartType.imageCounts:
                    sheet.Name = "Avg Successful Images Per Day";
                    chartDescription = "Average successful image responses per day during each month";
                    break;
                case ChartType.facadeBytes:
                    sheet.Name  = "Avg Facade MB Per Day";
                    chartDescription = "Average MB per day transfered to clients during each month";
                    break;
            }
            bool firstTime = true;
            DateTime? sDate = null;
            DateTime? eDate = null;
            foreach (string facadeName in facadeMonthTotals.Keys)
            {
                FacadeTotalsByMonth facadeMonthlyTotals = facadeMonthTotals[facadeName];
                if (firstTime)
                {
                    int i = 2;
                    foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
                    {
                        sheet.Cells[1, i] = dt.ToString("MMMM, yyyy");
                        i++;
                        if (!sDate.HasValue)
                        {
                            sDate = dt;
                        }
                        eDate = dt;
                    }
                    firstTime = false;
                }
                int row = sheet.UsedRange.Rows.Count + 1;
                int col = 2;
                //sheet.Cells[row, 1] = facadeName;
                bool facadeNameSet = false;
                //bool facadeHasValue = false;
                foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
                {
                    FacadeTotals facadeTotals = facadeMonthlyTotals.MonthlyTotals[dt];

                    long value = 0L;
                    switch (chartType)
                    {
                        case ChartType.bytes:
                            value = facadeTotals.TotalBytes;
                            break;
                        case ChartType.transactions:
                            value = facadeTotals.TotalRequests;
                            break;
                        case ChartType.imageCounts:
                            value = facadeTotals.SuccessfulImageResponses;
                            break;
                        case ChartType.facadeBytes:
                            value = facadeTotals.ClientBytes;
                            break;
                    }
                    if (value > 0)
                    {
                        //   facadeHasValue = true;
                        if (!facadeNameSet)
                        {
                            sheet.Cells[row, 1] = facadeName;
                            facadeNameSet = true;                            
                        }
                        double avgValue = calculateDailyAverage(value, dt);
                        if (chartType == ChartType.bytes || chartType == ChartType.facadeBytes)
                            avgValue = (double)(long)avgValue; // convert to long and then back to double

                        sheet.Cells[row, col] = avgValue;
                        col++;
                    }
                     
                    
                }
                /*
                if(!facadeHasValue)
                {
                    Excel.Range deleteRange = sheet.Cells[row, Type.Missing];
                    deleteRange.EntireRow.Delete(Excel.XlDeleteShiftDirection.xlShiftUp);
                }*/
            }
            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            if (chartType == ChartType.bytes || chartType == ChartType.facadeBytes)
            {
                ConvertBytesValuesToMB(sheet);
            }
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");// +Environment.NewLine + "Average per day during each month";
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, false, chartType, true, false);
            dtDescription += Environment.NewLine + chartDescription;// "Average per day during each month";
            OutputChartToPpt(pptPres, chart, slideIndex, dtDescription);
        }        

        protected void CreateAveragesSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts, 
            int slideIndex, DateTime startDate, DateTime endDate, AverageIncrement increment)
        {
            string incrementDescription = "Month";
            if (increment == AverageIncrement.day)
                incrementDescription = "Day";
            
            StringBuilder requestsContent = new StringBuilder();
            requestsContent.Append("Average Requests Per " + incrementDescription + "\r");
            StringBuilder bytesContent = new StringBuilder();
            bytesContent.Append("Average Bytes Per " + incrementDescription + "\r");
            StringBuilder imagesConent = new StringBuilder();
            imagesConent.Append("Average Successful Image Responses Per " + incrementDescription + "\r");
            StringBuilder clientByesContent = new StringBuilder();
            clientByesContent.Append("Average Client Bytes Per " + incrementDescription + "\r");

            int months = 0;
            DateTime date = startDate;
            while (date.CompareTo(endDate) < 1)
            {
                months++;
                date = date.AddMonths(1);
            }

            // set to the last day of the month
            endDate = endDate.AddMonths(1).AddDays(-1);

            TimeSpan span = endDate.Subtract(startDate);
            int days = (int)span.TotalDays + 1; // add one to count the last day of the month

            int incrementValue = months;
            if (increment == AverageIncrement.day)
                incrementValue = days;

            foreach (string facadeName in totalFacadeCounts.Keys)
            {
                long requests = totalFacadeCounts[facadeName].TotalRequests;
                double avg = (double)requests / (double)incrementValue;
                requestsContent.Append("\t" + facadeName + ": " + avg.ToString("N2") + "\r");

                long bytes = totalFacadeCounts[facadeName].TotalBytes;
                bytesContent.Append("\t" + facadeName + ": " + DailyTransactionsHelper.formatBytes((bytes / incrementValue)) + "\r");

                long images = totalFacadeCounts[facadeName].SuccessfulImageResponses;
                avg = (double)images / (double)incrementValue;
                imagesConent.Append("\t" + facadeName + ": " + avg.ToString("N2") + "\r");

                long clientBytes = totalFacadeCounts[facadeName].ClientBytes;
                if (clientBytes > 0)
                {
                    clientByesContent.Append("\t" + facadeName + ": " + DailyTransactionsHelper.formatBytes((clientBytes / incrementValue)) + "\r");
                }
            }

            StringBuilder content = new StringBuilder();
            string dateDescription = "From " + startDate.ToString("MMMM yyyy") + " to " + endDate.ToString("MMMM yyyy");
            switch (increment)
            {
                case AverageIncrement.month:
                    dateDescription += " (" + months + " Months)";
                    break;
                case AverageIncrement.day:
                    dateDescription += " (" + days + " Days)";
                    break;
            }
            content.Append(dateDescription + "\r");
            content.Append(requestsContent.ToString());
            content.Append(bytesContent.ToString());
            content.Append(imagesConent.ToString());
            content.Append(clientByesContent.ToString());

            string title = "Monthly Averages";
            if(increment == AverageIncrement.day)
                title = "Daily Averages";
            PowerPointHelper.CreateContentSlide(presentation, slideIndex, title, content.ToString(), true, 16);
        }

        private void CreateAveragesPresentationExplanationSlide(PowerPoint.Presentation presentation)
        {
            string title = "Statistics Explanation";
            StringBuilder description = new StringBuilder();
            description.AppendLine("These statistics are based on the transactions collected from the VIX servers");
            description.AppendLine("Transactions are small actions by the VIX. A request is initiated by a client. These stats are based on requests which might generate multiple transactions");
            description.AppendLine("Federation requests are requests from another VIX or the CVIX");
            description.AppendLine("Clinical Display requests are requests made by Clinical Display");
            description.AppendLine("VistARad requests are requests made by VistARad");
            description.AppendLine("This report contains the sum total of requests, image requests and bytes and then shows their average per month and per day");
            description.AppendLine("Client bytes are the number of bytes sent to the client (cached and uncached, without compression for VistARad)");
            PowerPointHelper.CreateContentSlide(presentation, 2, title, description.ToString(), true, 22);
        }

    }    
}
