/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Excel = Microsoft.Office.Interop.Excel;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;

namespace DailyTransactionLogAnalyzerEngine.reports
{
    /// <summary>
    /// 
    /// </summary>
    public class CachedReport : AbstractExcelReport
    {
        public CachedReport()
        {
            this.Name = "Cached Report";
            this.Description = "A report is generated that lists how effective the VIX cache is";
            this.ExcelOutputGroupType = ExcelOutputGroupType.allDates;
        }


        protected override void GenerateReport(string analysisDirectory, Dictionary<string, List<DailyTransactions>> sitesDailyTransactionsDictionary, ReportDateType reportDateType, 
            DateTime startDate, DateTime endDate)
        {
            switch (reportDateType)
            {
                case ReportDateType.month:
                    LogMsg("Generating cache report for " + OutputDate(startDate, reportDateType) + " to " + OutputDate(endDate, reportDateType) + ".");
                    OutputMonthlyGrowth(analysisDirectory, sitesDailyTransactionsDictionary, startDate, endDate);
                    break;
                case ReportDateType.year:
                    throw new NotImplementedException();
                default:
                    throw new NotImplementedException();
            }   
        }

        protected override ReportSiteType GetReportSiteSupportedType()
        {
            return ReportSiteType.vix;
        }

        protected override void GenerateReport(string analysisDirectory, List<DailyTransactions> dailyTransactionsList, 
            ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        protected override void GenerateReport(string analysisDirectory, DailyTransactions dailyTransactions, ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        private void OutputMonthlyGrowth(string monthlyAnalysisDirectory, Dictionary<string, 
            List<DailyTransactions>> sitesDailyTransactionsDictionary, DateTime reportStartDate, DateTime reportEndDate)
        {
            string filename = monthlyAnalysisDirectory + "\\Rollup\\CacheUsage.pdf";
            FileInfo f = new FileInfo(filename);
            if (!f.Directory.Exists)
            {
                f.Directory.Create();
            }

            if (File.Exists(filename))
                File.Delete(filename);
            string xlsxFilename = filename + ".xlsx";
            if (File.Exists(xlsxFilename))
                File.Delete(xlsxFilename);
            string pptFilename = filename + ".pptx";
            if (File.Exists(pptFilename))
                File.Delete(pptFilename);

            Excel.Application oXL = new Excel.Application();
            oXL.Visible = false;

            PowerPoint.Application ppt = new PowerPoint.Application();

            PowerPoint.Presentation pptPres = ppt.Presentations.Add(MsoTriState.msoFalse);

            Excel.Workbook workbook = null;
            try
            {
                List<string> siteIds = new List<string>();

                workbook = oXL.Workbooks.Add(Type.Missing);
                // delete all existing sheets except first (Excel creates 3 by default, must keep at least 1)                
                for (int i = workbook.Sheets.Count; i > 1; i--)
                {
                    Excel.Worksheet sheet = workbook.Sheets[i];
                    sheet.Delete();
                }
                Dictionary<string, Site> sitesDictionary = GetSitesDictionary();
                /*
                Dictionary<string, FacadeMonthlyTotals> facadeMonthRequestTotals = new Dictionary<string, FacadeMonthlyTotals>();
                Dictionary<string, FacadeMonthlyTotals> facadeMonthBytesTotals = new Dictionary<string, FacadeMonthlyTotals>();

                Dictionary<string, long> facadeRequestTotals = new Dictionary<string, long>(); // total requests for each facade
                Dictionary<string, long> facadeBytesTotals = new Dictionary<string, long>(); // total requests for each facade
                */

                Dictionary<DateTime, int> monthReceviedDataCount = new Dictionary<DateTime, int>(); // number of sites that had data each month

                CacheCounts cacheCounts = new CacheCounts();
                Dictionary<DateTime, CacheCounts> monthTotalCacheCounts = new Dictionary<DateTime, CacheCounts>();

                List<SiteDailyTransactionsList> sitesDailyTransactionsList = SortByName(sitesDailyTransactionsDictionary, sitesDictionary);

                foreach (SiteDailyTransactionsList siteDailyTransactionList in sitesDailyTransactionsList)// string siteNumber in sitesDailyTransactionsDictionary.Keys)
                {
                    string siteNumber = siteDailyTransactionList.SiteNumber;
                    Site thisSite = SiteHelper.GetSite(siteNumber);
                    LogMsg("Calculating totals for site '" + thisSite.Name + "'");
                    Excel.Worksheet requestsSheet = workbook.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing);
                    requestsSheet.Name = thisSite.ToString(true);
                    int count = workbook.Sheets.Count;
                    Excel.Worksheet bytesSheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[count], Type.Missing, Type.Missing);
                    bytesSheet.Name = thisSite.SiteId + "_bytes";
                    List<DailyTransactions> dailyTransactionsList = siteDailyTransactionList.DailyTransactionsList;

                    CacheCounts siteCacheCounts = new CacheCounts();

                    //Dictionary<string, int> facadeRequestRow = new Dictionary<string, int>();
                    //Dictionary<string, int> facadeBytesRow = new Dictionary<string, int>();

                    requestsSheet.Cells[2, 1] = "Uncached Image Responses";
                    requestsSheet.Cells[3, 1] = "Cached Image Responses";

                    bytesSheet.Cells[2, 1] = "Uncached Image Bytes";
                    bytesSheet.Cells[3, 1] = "Cached Image Bytes";

                    DateTime? startDate = null;
                    DateTime? endDate = null;

                    // each DailyTransaction represents a month of data
                    foreach (DailyTransactions dailyTransactions in dailyTransactionsList)
                    {
                        if (!startDate.HasValue)
                        {
                            startDate = dailyTransactions.Date;
                        }
                        endDate = dailyTransactions.Date; // always move to the current date
                        int requestsCol = requestsSheet.UsedRange.Columns.Count + 1;
                        int bytesCol = bytesSheet.UsedRange.Columns.Count + 1;

                        requestsSheet.Cells[1, requestsCol] = dailyTransactions.Date.ToString("MMMM, yyyy");
                        bytesSheet.Cells[1, bytesCol] = dailyTransactions.Date.ToString("MMMM, yyyy");

                        Excel.Range r = requestsSheet.Cells[1, Type.Missing];
                        r.NumberFormat = "[$-409]mmmm, yyyy;@";
                        r = bytesSheet.Cells[1, Type.Missing];
                        r.NumberFormat = "[$-409]mmmm, yyyy;@";

                        if (dailyTransactions.TransactionCounts.TotalParentTransactions > 0)
                        {
                            if (monthReceviedDataCount.ContainsKey(dailyTransactions.Date))
                            {
                                monthReceviedDataCount[dailyTransactions.Date]++;
                            }
                            else
                            {
                                monthReceviedDataCount.Add(dailyTransactions.Date, 1);
                            }
                        }

                        foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
                        {
                            // only supporting clinical display for this 
                            if (fc.FacadeName != "Clinical Display WebApp")
                                continue;
                            
                            //fc.ImageCacheResponses
                            //fc.SuccessfulImageResponses

                            long uncachedResponses = (fc.SuccessfulImageResponses - fc.ImageCacheResponses);
                            // successful image bytes is the total bytes returned so subtract cached bytes returned to get uncached bytes
                            long uncachedBytes = (fc.SuccessfulImageBytes - fc.CachedBytesReturned);
                            
                            //fc.RelevantBytes
                            //fc.CachedBytesReturned

                            requestsSheet.Cells[2, requestsCol] = uncachedResponses;
                            requestsSheet.Cells[3, requestsCol] = fc.ImageCacheResponses;

                            bytesSheet.Cells[2, bytesCol] = uncachedBytes;
                            bytesSheet.Cells[3, bytesCol] = fc.CachedBytesReturned;

                            cacheCounts.CachedImageBytes += fc.CachedBytesReturned;
                            cacheCounts.CachedImageResponse += fc.ImageCacheResponses;
                            cacheCounts.UncachedImageBytes += uncachedBytes;
                            cacheCounts.UncachedImageResponses += uncachedResponses;

                            siteCacheCounts.CachedImageBytes += fc.CachedBytesReturned;
                            siteCacheCounts.CachedImageResponse += fc.ImageCacheResponses;
                            siteCacheCounts.UncachedImageBytes += uncachedBytes;
                            siteCacheCounts.UncachedImageResponses += uncachedResponses;

                            if (!monthTotalCacheCounts.ContainsKey(dailyTransactions.Date))
                            {
                                monthTotalCacheCounts.Add(dailyTransactions.Date, new CacheCounts());
                            }
                            monthTotalCacheCounts[dailyTransactions.Date].CachedImageBytes += fc.CachedBytesReturned;
                            monthTotalCacheCounts[dailyTransactions.Date].CachedImageResponse += fc.ImageCacheResponses;
                            monthTotalCacheCounts[dailyTransactions.Date].UncachedImageBytes += uncachedBytes;
                            monthTotalCacheCounts[dailyTransactions.Date].UncachedImageResponses += uncachedResponses;
                        }

                    }// for loop for each month

                    // done with current site
                    ConvertBytesValuesToMB(bytesSheet);

                    string dateDescription = "From " + startDate.Value.ToString("MMMM yyyy") + " to " + endDate.Value.ToString("MMMM yyyy");
                    string description = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteCacheCounts, ChartType.transactions);
                    Excel.ChartObject requestsChart = CreateWorksheetChart(requestsSheet, dateDescription, thisSite, true, ChartType.transactions, false, false);
                    OutputChartToPpt(pptPres, requestsChart, description);

                    description = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteCacheCounts, ChartType.bytes);
                    Excel.ChartObject bytesChart = CreateWorksheetChart(bytesSheet, dateDescription, thisSite, true, ChartType.bytes, false, false);
                    OutputChartToPpt(pptPres, bytesChart, description);

                } // loop for each site


                LogMsg("Generating title and totals charts");
                PowerPointHelper.CreateTitleSlide(pptPres, "Site Cache Usage"); // slide 1
                CreateCachePresentationExplanationSlide(pptPres);

                CreateTotalsSlide(pptPres, cacheCounts, 3, reportStartDate, reportEndDate);
                OutputResponseCountChartAndSlide(workbook, pptPres, monthReceviedDataCount, 4);
                OutputTotalsChartAndSlide(workbook, pptPres, monthTotalCacheCounts, ChartType.transactions, 5);
                OutputTotalsChartAndSlide(workbook, pptPres, monthTotalCacheCounts, ChartType.bytes, 6);
                OutputCachePercentChartAndSlide(workbook, pptPres, monthTotalCacheCounts, ChartType.transactions, 7);
                OutputCachePercentChartAndSlide(workbook, pptPres, monthTotalCacheCounts, ChartType.bytes, 8);
    
                workbook.SaveAs(xlsxFilename);
                pptPres.SaveAs(pptFilename);
                pptPres.SaveAs(filename, PowerPoint.PpSaveAsFileType.ppSaveAsPDF, MsoTriState.msoFalse);
            }
            finally
            {
                if (workbook != null)
                    workbook.Close(false);
                if (pptPres != null)
                    pptPres.Close();
                oXL.Quit();
                ppt.Quit();
                System.Runtime.InteropServices.Marshal.ReleaseComObject(workbook);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(oXL);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(ppt);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(pptPres);

                oXL = null;
                GC.Collect();
            }
        }

        private string CreateDescriptionForIndividualSite(CacheCounts cacheCounts, ChartType chartType)
        {
            StringBuilder sb = new StringBuilder();
            
            switch (chartType)
            {
                case ChartType.bytes:
                    sb.AppendLine("Cached Image Bytes to Client: " + DailyTransactionsHelper.formatBytes(cacheCounts.CachedImageBytes));
                    sb.AppendLine("Uncached Image Bytes to Client: " + DailyTransactionsHelper.formatBytes(cacheCounts.UncachedImageBytes));
                    sb.AppendLine("Percent of Image Bytes From Cache to Client: " + CalculatePercent(cacheCounts.CachedImageBytes, (cacheCounts.CachedImageBytes + cacheCounts.UncachedImageBytes)));
                    break;
                case ChartType.transactions:
                    sb.AppendLine("Cached Image Responses: " + cacheCounts.CachedImageResponse);
                    sb.AppendLine("Uncached Image Responses: " + cacheCounts.UncachedImageResponses);
                    sb.AppendLine("Percent Image Bytes From Cache: " + CalculatePercent(cacheCounts.CachedImageResponse, (cacheCounts.CachedImageResponse + cacheCounts.UncachedImageResponses)));
                    break;
            }
            
            return sb.ToString();
        }

        private string CalculatePercent(long value, long total)
        {
            double percent = (double)value / (double)total;
            percent *= 100.0f;
            return percent.ToString("N2");
        }

        private void OutputTotalsChartAndSlide(Excel.Workbook workbook, PowerPoint.Presentation pptPres, Dictionary<DateTime, CacheCounts> monthTotalCacheCounts,
            ChartType chartType, int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            if (chartType == ChartType.bytes)
            {
                sheet.Name = "Total Image Bytes";
                sheet.Cells[2, 1] = "Uncached Image Bytes";
                sheet.Cells[3, 1] = "Cached Image Bytes";
            }
            else
            {
                sheet.Name = "Total Image Responses";
                sheet.Cells[2, 1] = "Uncached Image Responses";
                sheet.Cells[3, 1] = "Cached Image Responses";
            }
            DateTime? sDate = null;
            DateTime? eDate = null;

            int col = 2;
            
            foreach (DateTime dt in monthTotalCacheCounts.Keys)
            {
                CacheCounts cacheCounts = monthTotalCacheCounts[dt];
                sheet.Cells[1, col] = dt.ToString("MMMM, yyyy");
                
                if (!sDate.HasValue)
                {
                    sDate = dt;
                }
                eDate = dt;

                if (chartType == ChartType.bytes)
                {
                    sheet.Cells[2, col] = cacheCounts.UncachedImageBytes;
                    sheet.Cells[3, col] = cacheCounts.CachedImageBytes;
                }
                else
                {
                    sheet.Cells[2, col] = cacheCounts.UncachedImageResponses;
                    sheet.Cells[3, col] = cacheCounts.CachedImageResponse ;
                }
                col++;
            }


            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            if (chartType == ChartType.bytes)
            {
                ConvertBytesValuesToMB(sheet);
            }
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, true, chartType, false, false);
            OutputChartToPpt(pptPres, chart, slideIndex, dtDescription);
        }

        private void OutputCachePercentChartAndSlide(Excel.Workbook workbook, PowerPoint.Presentation pptPres, 
            Dictionary<DateTime, CacheCounts> monthTotalCacheCounts, ChartType chartType,
            int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            if (chartType == ChartType.bytes)
            {
                sheet.Name = "Cached Image Bytes Percent";
            }
            else if(chartType == ChartType.transactions)
            {
                sheet.Name = "Cached Image Responses Percent";
            }
            DateTime? sDate = null;
            DateTime? eDate = null;

            int col = 2;
            sheet.Cells[2, 1] = "Percent From Cache";
            foreach (DateTime dt in monthTotalCacheCounts.Keys)
            {
                CacheCounts cacheCounts = monthTotalCacheCounts[dt];
                sheet.Cells[1, col] = dt.ToString("MMMM, yyyy");

                if (!sDate.HasValue)
                {
                    sDate = dt;
                }
                eDate = dt;

                if (chartType == ChartType.bytes)
                {
                    long totalBytes = cacheCounts.UncachedImageBytes + cacheCounts.CachedImageBytes;
                    double percent = (double)cacheCounts.CachedImageBytes / (double)totalBytes;
                    percent *= 100.0f;
                    sheet.Cells[2, col] = percent.ToString("N2");
                }
                else if(chartType == ChartType.transactions)
                {
                    long totalRequests = cacheCounts.UncachedImageResponses + cacheCounts.CachedImageResponse;
                    double percent = (double)cacheCounts.CachedImageResponse / (double)totalRequests;
                    percent *= 100.0f;
                    sheet.Cells[2, col] = percent.ToString("N2");
                }
                col++;
            }

            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");

            Excel.ChartObject chart = null;
            string description = null;
            if (chartType == ChartType.bytes)
            {                
                chart = CreateWorksheetChart(sheet, dtDescription, null, false, ChartType.decimalCounts, true, false);
                description = dtDescription + ", Percent of bytes to Clinical Display from the cache";
            }
            else if (chartType == ChartType.transactions)
            {
                chart = CreateWorksheetChart(sheet, dtDescription, null, false, ChartType.decimalCounts, true, false);
                description = dtDescription + ", Percent of image responses to Clinical Display from the cache";
            }
            OutputChartToPpt(pptPres, chart, slideIndex, description);
        }

        protected void CreateTotalsSlide(PowerPoint.Presentation presentation, CacheCounts cacheCounts, int slideIndex, DateTime startDate, DateTime endDate)
        {
            StringBuilder content = new StringBuilder();

            content.Append("From " + startDate.ToString("MMMM yyyy") + " to " + endDate.ToString("MMMM yyyy") + "\r");
            content.Append("Uncached Image Responses: " + cacheCounts.UncachedImageResponses.ToString("N0") + "\r");
            content.Append("Cached Image Responses: " + cacheCounts.CachedImageResponse.ToString("N0") + "\r");

            long totalCacheResponses = cacheCounts.UncachedImageResponses + cacheCounts.CachedImageResponse;
            double percent = (double)cacheCounts.CachedImageResponse / (double)totalCacheResponses;
            percent *= 100.0f;
            content.Append("Percent Image Responses From Cache: " + percent.ToString("N2") + "\r");

            content.Append("Uncached Image Bytes Returned: " + DailyTransactionsHelper.formatBytes(cacheCounts.UncachedImageBytes) + "\r");
            content.Append("Cached Image Bytes Returned: " + DailyTransactionsHelper.formatBytes(cacheCounts.CachedImageBytes) + "\r");
            long totalBytes = cacheCounts.UncachedImageBytes + cacheCounts.CachedImageBytes;
            percent = (double)cacheCounts.CachedImageBytes / (double)totalBytes;
            percent *= 100.0f;
            content.Append("Percent Image Bytes From Cache: " + percent.ToString("N2") + "\r");

            PowerPointHelper.CreateContentSlide(presentation, slideIndex, "Clinical Display Image Totals", content.ToString(), true);
        }

        private void CreateCachePresentationExplanationSlide(PowerPoint.Presentation presentation)
        {
            string title = "Statistics Explanation";
            StringBuilder description = new StringBuilder();
            description.AppendLine("These statistics are based on the transactions collected from the VIX servers");            
            description.AppendLine("This report describes the usage of the VIX cache");
            description.AppendLine("Included in this report is information from the Clinical Display Web App only");
            description.AppendLine("This report only includes image requests, all other requests (such as metadata) are ignored");
            description.AppendLine("The charts show the month by month usage for each site");
            description.AppendLine("As more sites deployed the VIX the amount of data over the WAN decreases because of increased use of compression");
            PowerPointHelper.CreateContentSlide(presentation, 2, title, description.ToString(), true, 22);
        }


        public class CacheCounts
        {
            public long CachedImageResponse { get; set; }
            public long UncachedImageResponses { get; set; }
            public long CachedImageBytes { get; set; }
            public long UncachedImageBytes { get; set; }

            public CacheCounts()
            {
                CachedImageBytes = 0L;
                UncachedImageBytes = 0L;
                CachedImageResponse = 0L;
                UncachedImageResponses = 0L;
            }
        }
    }
}
