/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;
using Excel = Microsoft.Office.Interop.Excel;

namespace DailyTransactionLogAnalyzerEngine.reports
{
    public class ConnectionReport : AbstractExcelReport
    {
        public ConnectionReport()
        {
            this.Name = "Connections Report";
            this.Description = "Creates slides that list the connection information for month and daily usage of the VIX";
            this.ExcelOutputGroupType = ExcelOutputGroupType.allDates;
        }

        protected override void GenerateReport(string analysisDirectory, Dictionary<string, List<DailyTransactions>> sitesDailyTransactionsDictionary,
            ReportDateType reportDateType, DateTime startDate, DateTime endDate)
        {
            switch (reportDateType)
            {
                case ReportDateType.month:
                    LogMsg("Generating connections report for " + OutputDate(startDate, reportDateType) + " to " + OutputDate(endDate, reportDateType) + ".");
                    OutputConnections(analysisDirectory, sitesDailyTransactionsDictionary, startDate, endDate);
                    break;
                case ReportDateType.year:
                    throw new NotImplementedException();
                default:
                    throw new NotImplementedException();
            }  
        }

        protected override ReportSiteType GetReportSiteSupportedType()
        {
            return ReportSiteType.vix_and_cvix ;
        }

        protected override void GenerateReport(string analysisDirectory, List<DailyTransactions> dailyTransactionsList, 
            ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        protected override void GenerateReport(string analysisDirectory, DailyTransactions dailyTransactions,
            ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        private void OutputConnections(string monthlyAnalysisDirectory, Dictionary<string, List<DailyTransactions>> sitesDailyTransactionsDictionary, DateTime startDate, DateTime endDate)
        {
            string filename = monthlyAnalysisDirectory + "\\Rollup\\Connections.pdf";
            FileInfo f = new FileInfo(filename);
            if (!f.Directory.Exists)
            {
                f.Directory.Create();
            }

            if (File.Exists(filename))
                File.Delete(filename);
            string xlsxFilename = filename + ".xlsx";
            if (File.Exists(xlsxFilename))
                File.Delete(xlsxFilename);
            string pptFilename = filename + ".pptx";
            if (File.Exists(pptFilename))
                File.Delete(pptFilename);

            Excel.Application oXL = new Excel.Application();
            oXL.Visible = false;

            PowerPoint.Application ppt = new PowerPoint.Application();
            PowerPoint.Presentation pptPres = ppt.Presentations.Add(MsoTriState.msoFalse);

            Excel.Workbook workbook = null;
            try
            {
                workbook = oXL.Workbooks.Add(Type.Missing);
                // delete all existing sheets except first (Excel creates 3 by default, must keep at least 1)                
                for (int i = workbook.Sheets.Count; i > 1; i--)
                {
                    Excel.Worksheet sheet = workbook.Sheets[i];
                    sheet.Delete();
                }

                List<string> siteIds = new List<string>();
                long totalRequests = 0L;

                Dictionary<string, Site> sitesDictionary = GetSitesDictionary();
                Dictionary<string, FacadeTotalsByMonth> facadeMonthTotals = new Dictionary<string, FacadeTotalsByMonth>(); // totals for each facade by month

                Dictionary<string, FacadeTotals> facadeTotals = new Dictionary<string, FacadeTotals>(); // totals for each facade
                Dictionary<DateTime, int> monthReceviedDataCount = new Dictionary<DateTime, int>(); // number of sites that had data each month

                List<SiteDailyTransactionsList> sitesDailyTransactionsList = SortByName(sitesDailyTransactionsDictionary, sitesDictionary);

                Dictionary<RemoteLoginSites, RemoteLoginSites> allRemoteLoginCounts = new Dictionary<RemoteLoginSites, RemoteLoginSites>();
                Dictionary<RemoteLoginSites, RemoteLoginSites> lastMonthRemoteLoginCounts = new Dictionary<RemoteLoginSites, RemoteLoginSites>();

                foreach (SiteDailyTransactionsList siteDailyTransactionList in sitesDailyTransactionsList)// string siteNumber in sitesDailyTransactionsDictionary.Keys)
                {
                    string siteNumber = siteDailyTransactionList.SiteNumber;
                    Site thisSite = SiteHelper.GetSite(siteNumber);
                    LogMsg("Calculating totals for site '" + thisSite.Name + "'");

                    List<DailyTransactions> dailyTransactionsList = siteDailyTransactionList.DailyTransactionsList;

                    // each DailyTransaction represents a month of data
                    foreach (DailyTransactions dailyTransactions in dailyTransactionsList)
                    {
                        totalRequests += dailyTransactions.TransactionCounts.TotalParentTransactions;
                        if (dailyTransactions.TransactionCounts.TotalParentTransactions > 0)
                        {
                            if (monthReceviedDataCount.ContainsKey(dailyTransactions.Date))
                            {
                                monthReceviedDataCount[dailyTransactions.Date]++;
                            }
                            else
                            {
                                monthReceviedDataCount.Add(dailyTransactions.Date, 1);
                            }
                        }
                        foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
                        {

                            if (!facadeTotals.ContainsKey(fc.FacadeName))
                            {
                                facadeTotals.Add(fc.FacadeName, new FacadeTotals());
                            }
                            facadeTotals[fc.FacadeName].BseLogins += fc.BseLogins;
                            facadeTotals[fc.FacadeName].CapriLogins += fc.CapriLogins;

                            if (!facadeMonthTotals.ContainsKey(fc.FacadeName))
                                facadeMonthTotals.Add(fc.FacadeName, new FacadeTotalsByMonth());

                            if (!facadeMonthTotals[fc.FacadeName].MonthlyTotals.ContainsKey(dailyTransactions.Date))
                                facadeMonthTotals[fc.FacadeName].MonthlyTotals.Add(dailyTransactions.Date, new FacadeTotals());

                            facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].BseLogins += fc.BseLogins;
                            facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].CapriLogins += fc.CapriLogins;

                            foreach (FromSiteCounts fsc in fc.SiteCounts.Values)
                            {
                                RemoteLoginSites methodCount = null;
                                string sn = SiteHelper.CleanSiteNumber(fsc.SiteNumber);
                                if (!sitesDictionary.ContainsKey(sn))
                                {
                                    continue;
                                }
                                Site fromSite = sitesDictionary[sn];
                                if (fc.SiteIdentificationSource == SiteIdentificationSource.requestingSite)
                                {
                                    // federation only, fsc.SiteNumber is the site the user came from (authentication site)
                                    methodCount = new RemoteLoginSites(fromSite, thisSite);
                                }
                                else
                                {
                                    // client (vistarad, clinical display), fsc.SiteNumber is the site getting data from (remote site)
                                    methodCount = new RemoteLoginSites(thisSite, fromSite);
                                }
                                if (!allRemoteLoginCounts.ContainsKey(methodCount))
                                {
                                    allRemoteLoginCounts.Add(methodCount, methodCount);// new LoginTotals());
                                }

                                allRemoteLoginCounts[methodCount].LoginTotals.BseLogins += fsc.BseLogins;
                                allRemoteLoginCounts[methodCount].LoginTotals.CapriLogins += fsc.CapriLogins;

                                // count protocol usage for each facade
                                foreach (string protocol in fsc.ProtocolCounts.Keys)
                                {
                                    if (!facadeTotals[fc.FacadeName].ProtocolCounts.ContainsKey(protocol))
                                    {
                                        facadeTotals[fc.FacadeName].ProtocolCounts.Add(protocol, new ProtocolCount(protocol));
                                    }
                                    facadeTotals[fc.FacadeName].ProtocolCounts[protocol].Count += fsc.ProtocolCounts[protocol];
                                    if (!facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].ProtocolCounts.ContainsKey(protocol))
                                    {
                                        facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].ProtocolCounts.Add(protocol, new ProtocolCount(protocol));
                                    }
                                    facadeMonthTotals[fc.FacadeName].MonthlyTotals[dailyTransactions.Date].ProtocolCounts[protocol].Count += fsc.ProtocolCounts[protocol];
                                }

                                if (dailyTransactions.Date == endDate)
                                {
                                    if (fc.SiteIdentificationSource == SiteIdentificationSource.requestingSite)
                                    {
                                        // federation only, fsc.SiteNumber is the site the user came from (authentication site)
                                        methodCount = new RemoteLoginSites(fromSite, thisSite);
                                    }
                                    else
                                    {
                                        // client (vistarad, clinical display), fsc.SiteNumber is the site getting data from (remote site)
                                        methodCount = new RemoteLoginSites(thisSite, fromSite);
                                    }
                                    if (!lastMonthRemoteLoginCounts.ContainsKey(methodCount))
                                    {
                                        lastMonthRemoteLoginCounts.Add(methodCount, methodCount);
                                    }
                                    lastMonthRemoteLoginCounts[methodCount].LoginTotals.BseLogins += fsc.BseLogins;
                                    lastMonthRemoteLoginCounts[methodCount].LoginTotals.CapriLogins += fsc.CapriLogins;
                                }
                            }

                        }

                    }// for loop for each month

                    // done with calculations for current site
                } // loop for each site

                LogMsg("Generating title and totals charts");
                PowerPointHelper.CreateTitleSlide(pptPres, "Connections"); // slide 1
                CreateConnectionsPresentationExplanationSlide(pptPres);

                CreateConnectionsTotalsSlide(pptPres, facadeTotals, 3, startDate, endDate, 12);
                CreateAveragesSlide(pptPres, facadeTotals, 4, startDate, endDate, AverageIncrement.month);
                CreateAveragesSlide(pptPres, facadeTotals, 5, startDate, endDate, AverageIncrement.day);

                OutputResponseCountChartAndSlide(workbook, pptPres, monthReceviedDataCount, 6);

                int slideIndex = 7;

                foreach (string facadeName in facadeMonthTotals.Keys)
                {
                    OutputMonthlyConnectionUsageChartAndSlidePerFacade(workbook, pptPres, facadeName, facadeMonthTotals[facadeName], slideIndex);
                    slideIndex++;
                    OutputMonthlyProtocolUsageChartAndSlidePerFacade(workbook, pptPres, facadeName, facadeMonthTotals[facadeName], slideIndex);
                    slideIndex++;
                }
                OutputMonthlyConnectionUsageChartAndSlideAllFacades(workbook, pptPres, facadeMonthTotals, slideIndex);
                OutputRemoteSiteLoginMethodsSlide(pptPres, allRemoteLoginCounts, slideIndex + 1, startDate, endDate);
                OutputRemoteSiteLoginMethodsSlide(pptPres, lastMonthRemoteLoginCounts, slideIndex + 2, null, endDate);

                workbook.SaveAs(xlsxFilename);

                pptPres.SaveAs(pptFilename);
                pptPres.SaveAs(filename, PowerPoint.PpSaveAsFileType.ppSaveAsPDF, MsoTriState.msoFalse);
            }
            finally
            {
                if (workbook != null)
                    workbook.Close(false);
                if (pptPres != null)
                    pptPres.Close();
                oXL.Quit();
                ppt.Quit();
                System.Runtime.InteropServices.Marshal.ReleaseComObject(workbook);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(oXL);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(ppt);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(pptPres);

                oXL = null;
                GC.Collect();
            }
        }

        private void OutputRemoteSiteLoginMethodsSlide(PowerPoint.Presentation pptPres, 
            Dictionary<RemoteLoginSites, RemoteLoginSites> remoteLoginCounts, int slideIndex, 
            DateTime? startDate, DateTime endDate)
        {
            List<RemoteLoginSites> loginSites = new List<RemoteLoginSites>();
            loginSites.AddRange(remoteLoginCounts.Values);
            loginSites.Sort(new RemoteLoginSitesComparer(false));

            long totalCapri = 0L;
            foreach (RemoteLoginSites rls in loginSites)
            {
                totalCapri += rls.LoginTotals.CapriLogins;
            }
            
            string title = "Most CAPRI Logins";
            StringBuilder content = new StringBuilder();
            if (!startDate.HasValue)
            {
                title = "Most Recent CAPRI Logins";
                content.AppendLine(endDate.ToString("MMMM yyyy") + " (" + totalCapri.ToString("N0") + " total CAPRI logins)");
            }
            else
            {
                content.AppendLine("From " + startDate.Value.ToString("MMMM yyyy") + " to " + endDate.ToString("MMMM yyyy") + " (" + totalCapri.ToString("N0") + " total CAPRI logins)");
            }            

            int max = 15;
            if (loginSites.Count < max)
                max = loginSites.Count;

            int end = loginSites.Count - max;

            for (int i = loginSites.Count -1; i >= end; i--)
            {
                RemoteLoginSites rls = loginSites[i];
                content.AppendLine("Authentication Site (" + rls.AuthenticationSite + ") to site (" + rls.RemoteSite + "), CAPRI usage: " + rls.LoginTotals.CapriLogins);
            }            

            PowerPointHelper.CreateContentSlide(pptPres, slideIndex, title, content.ToString(), true, 15);
        }

        private void OutputMonthlyConnectionUsageChartAndSlideAllFacades(Excel.Workbook workbook, PowerPoint.Presentation pptPres, Dictionary<string, FacadeTotalsByMonth> facadeMonthTotals,
            int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            string sheetName = "All Facades Connection Types";
            string chartDescription = "Connection Types during each month for all facades" + Environment.NewLine;

            bool firstTime = true;
            DateTime? sDate = null;
            DateTime? eDate = null;

            Dictionary<DateTime, FacadeTotals> dateCounts = new Dictionary<DateTime, FacadeTotals>();
            string prefix = "";

            foreach (string facadeName in facadeMonthTotals.Keys)
            {
                chartDescription += prefix + facadeName;
                FacadeTotalsByMonth facadeMonthlyTotals = facadeMonthTotals[facadeName];

                if (firstTime)
                {
                    int i = 2;
                    foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
                    {
                        sheet.Cells[1, i] = dt.ToString("MMMM, yyyy");
                        i++;
                        if (!sDate.HasValue)
                        {
                            sDate = dt;
                        }
                        eDate = dt;
                    }
                    firstTime = false;
                    sheet.Cells[2, 1] = "BSE Logins";
                    sheet.Cells[3, 1] = "CAPRI Logins";
                }
               
                foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
                {
                    FacadeTotals facadeTotals = facadeMonthlyTotals.MonthlyTotals[dt];
                    if (!dateCounts.ContainsKey(dt))
                    {
                        dateCounts.Add(dt, new FacadeTotals());
                    }
                    dateCounts[dt].BseLogins += facadeTotals.BseLogins;
                    dateCounts[dt].CapriLogins += facadeTotals.CapriLogins;
                }
                prefix = ", ";
            }
            int col = 2;
            foreach (DateTime dt in dateCounts.Keys)
            {
                FacadeTotals facadeTotals = dateCounts[dt];
                sheet.Cells[2, col] = facadeTotals.BseLogins;
                sheet.Cells[3, col] = facadeTotals.CapriLogins;

                col++;
            }

            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, false, ChartType.integerCounts, true, false, sheetName);
            dtDescription += Environment.NewLine + chartDescription;// "Average per day during each month";
            OutputChartToPpt(pptPres, chart, slideIndex, dtDescription);
        }

        private void OutputMonthlyProtocolUsageChartAndSlidePerFacade(Excel.Workbook workbook, PowerPoint.Presentation pptPres, string facadeName, FacadeTotalsByMonth facadeMonthlyTotals,
            int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            string sheetName = facadeName + " Protocol Types";
            string chartDescription = "Protocols used during each month";

            bool firstTime = true;
            DateTime? sDate = null;
            DateTime? eDate = null;

            Dictionary<string, int> protocolRow = new Dictionary<string, int>();
            int highestRow = 1;

            if (firstTime)
            {
                int i = 2;
                foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
                {
                    sheet.Cells[1, i] = dt.ToString("MMMM, yyyy");
                    i++;
                    if (!sDate.HasValue)
                    {
                        sDate = dt;
                    }
                    eDate = dt;
                }
                firstTime = false;
            }
            int col = 2;
            foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
            {
                FacadeTotals facadeTotals = facadeMonthlyTotals.MonthlyTotals[dt];
                foreach (string protocol in facadeTotals.ProtocolCounts.Keys)
                {
                    int row = highestRow + 1;
                    if (!protocolRow.ContainsKey(protocol))
                    {
                        highestRow = row;
                        sheet.Cells[row, 1] = protocol;
                        protocolRow.Add(protocol, row);
                    }
                    else
                    {
                        row = protocolRow[protocol];
                    }
                    sheet.Cells[row, col] = facadeTotals.ProtocolCounts[protocol].Count;
                }

                col++;
            }

            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, false, ChartType.integerCounts, true, false, sheetName);
            dtDescription += Environment.NewLine + chartDescription;// "Average per day during each month";
            OutputChartToPpt(pptPres, chart, slideIndex, dtDescription);
        }

        private void OutputMonthlyConnectionUsageChartAndSlidePerFacade(Excel.Workbook workbook, PowerPoint.Presentation pptPres, string facadeName, FacadeTotalsByMonth facadeMonthlyTotals,
            int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            string sheetName = facadeName + " Connection Types";
            string chartDescription = "Connection Types during each month";
           
            bool firstTime = true;
            DateTime? sDate = null;
            DateTime? eDate = null;

                if (firstTime)
                {
                    int i = 2;
                    foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
                    {
                        sheet.Cells[1, i] = dt.ToString("MMMM, yyyy");
                        i++;
                        if (!sDate.HasValue)
                        {
                            sDate = dt;
                        }
                        eDate = dt;
                    }
                    firstTime = false;
                    sheet.Cells[2, 1] = "BSE Logins";
                    sheet.Cells[3, 1] = "CAPRI Logins";
                }
                int col = 2;
                foreach (DateTime dt in facadeMonthlyTotals.MonthlyTotals.Keys)
                {
                    FacadeTotals facadeTotals = facadeMonthlyTotals.MonthlyTotals[dt];

                    sheet.Cells[2, col] = facadeTotals.BseLogins;
                    sheet.Cells[3, col] = facadeTotals.CapriLogins;

                    col++;
                }
            
            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, false, ChartType.integerCounts, true, false, sheetName);
            dtDescription += Environment.NewLine + chartDescription;// "Average per day during each month";
            OutputChartToPpt(pptPres, chart, slideIndex, dtDescription);
        }

        protected void CreateAveragesSlide(PowerPoint.Presentation presentation, Dictionary<string, FacadeTotals> totalFacadeCounts,
            int slideIndex, DateTime startDate, DateTime endDate, AverageIncrement increment)
        {
            string incrementDescription = "Month";
            if (increment == AverageIncrement.day)
                incrementDescription = "Day";

            StringBuilder bseContent = new StringBuilder();
            bseContent.Append("Average BSE Logins Per " + incrementDescription + "\r");
            StringBuilder capriContent = new StringBuilder();
            capriContent.Append("Average CAPRI Logins Per " + incrementDescription + "\r");            

            int months = 0;
            DateTime date = startDate;
            while (date.CompareTo(endDate) < 1)
            {
                months++;
                date = date.AddMonths(1);
            }

            // set to the last day of the month
            endDate = endDate.AddMonths(1).AddDays(-1);

            TimeSpan span = endDate.Subtract(startDate);
            int days = (int)span.TotalDays + 1; // add one to count the last day of the month

            int incrementValue = months;
            if (increment == AverageIncrement.day)
                incrementValue = days;

            foreach (string facadeName in totalFacadeCounts.Keys)
            {
                long bse = totalFacadeCounts[facadeName].BseLogins;
                double avg = (double)bse / (double)incrementValue;
                bseContent.Append("\t" + facadeName + ": " + avg.ToString("N2") + "\r");

                long capri = totalFacadeCounts[facadeName].CapriLogins;
                avg = (double)capri / (double)incrementValue;
                capriContent.Append("\t" + facadeName + ": " + avg.ToString("N2") + "\r");     
            }

            StringBuilder content = new StringBuilder();
            string dateDescription = "From " + startDate.ToString("MMMM yyyy") + " to " + endDate.ToString("MMMM yyyy");
            switch (increment)
            {
                case AverageIncrement.month:
                    dateDescription += " (" + months + " Months)";
                    break;
                case AverageIncrement.day:
                    dateDescription += " (" + days + " Days)";
                    break;
            }
            content.Append(dateDescription + "\r");
            content.Append(bseContent.ToString());
            content.Append(capriContent.ToString());

            string title = "Monthly Averages";
            if (increment == AverageIncrement.day)
                title = "Daily Averages";
            PowerPointHelper.CreateContentSlide(presentation, slideIndex, title, content.ToString(), true, 16);
        }

        private void CreateConnectionsPresentationExplanationSlide(PowerPoint.Presentation presentation)
        {
            string title = "Statistics Explanation";
            StringBuilder description = new StringBuilder();
            description.AppendLine("These statistics are based on the transactions collected from the VIX servers");
            description.AppendLine("Transactions are small actions by the VIX. A request is initiated by a client. These stats are based on requests which might generate multiple transactions");
            description.AppendLine("Federation requests are requests from another VIX or the CVIX");
            description.AppendLine("Clinical Display requests are requests made by Clinical Display");
            description.AppendLine("VistARad requests are requests made by VistARad");
            description.AppendLine("This report contains the types of connections made to VistA by each facade.");
            description.AppendLine("BSE connections should be used primarily but CAPRI is used as a backup if BSE fails.");
            PowerPointHelper.CreateContentSlide(presentation, 2, title, description.ToString(), true, 22);
        }
    }

    public class RemoteLoginSites
    {
        public Site AuthenticationSite { get; private set; }
        public Site RemoteSite { get; private set; }

        public LoginTotals LoginTotals { get; private set; }

        public RemoteLoginSites(Site authenticationSite, Site remoteSite)
        {
            this.AuthenticationSite = authenticationSite;
            this.RemoteSite = remoteSite;
            this.LoginTotals = new LoginTotals();
        }

        public override bool Equals(object obj)
        {
            if (obj is RemoteLoginSites)
            {
                RemoteLoginSites that = (RemoteLoginSites)obj;
                return (this.RemoteSite.SiteId == that.RemoteSite.SiteId && this.AuthenticationSite.SiteId == that.AuthenticationSite.SiteId);
            }
            return false;            
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class RemoteLoginSitesComparer : IComparer<RemoteLoginSites>
    {
        public bool SortBse { get; private set; }

        public RemoteLoginSitesComparer(bool sortBse)
        {
            this.SortBse = sortBse;
        }

        public int Compare(RemoteLoginSites x, RemoteLoginSites y)
        {
            if (SortBse)
            {
                return x.LoginTotals.BseLogins.CompareTo(y.LoginTotals.BseLogins);
            }
            else
            {
                return x.LoginTotals.CapriLogins.CompareTo(y.LoginTotals.CapriLogins);
            }
        }
    }

}
