/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Excel = Microsoft.Office.Interop.Excel;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;

namespace DailyTransactionLogAnalyzerEngine.reports
{
    public class CvixReport : AbstractExcelReport
    {

        public CvixReport()
        {
            this.Name = "CVIX Report";
            this.Description = "Combine the statistics for the CVIX nodes into a single report";
            this.ExcelOutputGroupType = ExcelOutputGroupType.allDates;
        }

        protected override void GenerateReport(string analysisDirectory, Dictionary<string, List<DailyTransactions>> sitesDailyTransactionsDictionary, 
            ReportDateType reportDateType, DateTime startDate, DateTime endDate)
        {
            switch (reportDateType)
            {
                case ReportDateType.month:
                    LogMsg("Generating CVIX report for " + OutputDate(startDate, reportDateType) + " to " + OutputDate(endDate, reportDateType) + ".");
                    OutputMonthlyGrowth(analysisDirectory, sitesDailyTransactionsDictionary, startDate, endDate);
                    break;
                case ReportDateType.year:
                    throw new NotImplementedException();
                default:
                    throw new NotImplementedException();
            }   
        }

        protected override ReportSiteType GetReportSiteSupportedType()
        {
            return ReportSiteType.cvix;
        }

        protected override void GenerateReport(string analysisDirectory, List<DailyTransactions> dailyTransactionsList, 
            ReportDateType reportDateType, DateTime date)
        {                     
            throw new NotImplementedException();
        }

        protected override void GenerateReport(string analysisDirectory, DailyTransactions dailyTransactions, ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        private void OutputMonthlyGrowth(string monthlyAnalysisDirectory, Dictionary<string,
            List<DailyTransactions>> sitesDailyTransactionsDictionary, DateTime reportStartDate, DateTime reportEndDate)
        {
            string filename = monthlyAnalysisDirectory + "\\Rollup\\CVIX.pdf";
            FileInfo f = new FileInfo(filename);
            if (!f.Directory.Exists)
            {
                f.Directory.Create();
            }

            if (File.Exists(filename))
                File.Delete(filename);
            string xlsxFilename = filename + ".xlsx";
            if (File.Exists(xlsxFilename))
                File.Delete(xlsxFilename);
            string pptFilename = filename + ".pptx";
            if (File.Exists(pptFilename))
                File.Delete(pptFilename);

            Excel.Application oXL = new Excel.Application();
            oXL.Visible = false;

            PowerPoint.Application ppt = new PowerPoint.Application();

            PowerPoint.Presentation pptPres = ppt.Presentations.Add(MsoTriState.msoFalse);

            Excel.Workbook workbook = null;
            try
            {
                List<string> siteIds = new List<string>();

                workbook = oXL.Workbooks.Add(Type.Missing);
                // delete all existing sheets except first (Excel creates 3 by default, must keep at least 1)                
                for (int i = workbook.Sheets.Count; i > 1; i--)
                {
                    Excel.Worksheet sheet = workbook.Sheets[i];
                    sheet.Delete();
                }
                Dictionary<string, Site> sitesDictionary = GetSitesDictionary();

                Dictionary<string, FacadeTotalsByMonth> facadeMonthTotals = new Dictionary<string, FacadeTotalsByMonth>(); // totals for each facade by month
                Dictionary<string, FacadeTotalsByMonth> awivTotals = new Dictionary<string, FacadeTotalsByMonth>(); // AWIV only so there will only be one facade - just easier to use this object for outputting totals
                //FacadeTotalsByMonth exchangeTotals = new FacadeTotalsByMonth();
                //FacadeTotalsByMonth awivTotals = new FacadeTotalsByMonth();
                Dictionary<string, FacadeTotals> awivSiteRequestTotals = new Dictionary<string, FacadeTotals>();
                Dictionary<string, FacadeTotals> federationDoDSiteRequestTotals = new Dictionary<string, FacadeTotals>();

                Dictionary<string, FacadeTotals> facadeTotals = new Dictionary<string, FacadeTotals>(); // totals for each facade
                long totalRequests = 0L;

                List<SiteDailyTransactionsList> sitesDailyTransactionsList = SortByName(sitesDailyTransactionsDictionary, sitesDictionary);

                foreach (SiteDailyTransactionsList siteDailyTransactionList in sitesDailyTransactionsList)// string siteNumber in sitesDailyTransactionsDictionary.Keys)
                {
                    string siteNumber = siteDailyTransactionList.SiteNumber;
                    Site thisSite = SiteHelper.GetSite(siteNumber);
                    LogMsg("Calculating totals for site '" + thisSite.Name + "'");
                           
                    List<DailyTransactions> dailyTransactionsList = siteDailyTransactionList.DailyTransactionsList;

                    FacadeTotals siteTotals = new FacadeTotals();
                    Dictionary<string, FacadeTotals> siteFacadeTotals = new Dictionary<string, FacadeTotals>();


                    // each DailyTransaction represents a month of data
                    foreach (DailyTransactions dailyTransactions in dailyTransactionsList)
                    {
                        totalRequests += dailyTransactions.TransactionCounts.TotalParentTransactions;
                        foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
                        {
                            if (fc.DisplayName == "AWIV WebApp Responses")
                            {
                                if (!awivTotals.ContainsKey(fc.DisplayName))
                                {
                                    awivTotals.Add(fc.DisplayName, new FacadeTotalsByMonth());
                                }

                                if (!awivTotals[fc.DisplayName].MonthlyTotals.ContainsKey(dailyTransactions.Date))
                                {
                                    awivTotals[fc.DisplayName].MonthlyTotals.Add(dailyTransactions.Date, new FacadeTotals());
                                }
                                awivTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].ClientBytes += fc.SuccessfulImageBytes;
                                awivTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].SuccessfulImageResponses += fc.SuccessfulImageResponses;
                                awivTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].TotalBytes += fc.RelevantBytes;
                                awivTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].TotalRequests += fc.TotalTransactions;

                                if (!facadeTotals.ContainsKey(fc.DisplayName))
                                {
                                    facadeTotals.Add(fc.DisplayName, new FacadeTotals());
                                }
                                facadeTotals[fc.DisplayName].ClientBytes += fc.SuccessfulImageBytes;
                                facadeTotals[fc.DisplayName].SuccessfulImageResponses += fc.SuccessfulImageResponses;
                                facadeTotals[fc.DisplayName].TotalBytes += fc.RelevantBytes;
                                facadeTotals[fc.DisplayName].TotalRequests += fc.TotalTransactions;                                
                            }
                            else if (fc.DisplayName == "AWIV WebApp Requests")
                            {
                                foreach (FromSiteCounts fsc in fc.SiteCounts.Values)
                                {
                                    if (!awivSiteRequestTotals.ContainsKey(fsc.SiteNumber))
                                    {
                                        awivSiteRequestTotals.Add(fsc.SiteNumber, new FacadeTotals());
                                    }
                                    awivSiteRequestTotals[fsc.SiteNumber].TotalBytes += fsc.SuccessfulImageBytes; // same as client bytes
                                    awivSiteRequestTotals[fsc.SiteNumber].SuccessfulImageResponses += fsc.SuccessfulImageResponses;
                                    awivSiteRequestTotals[fsc.SiteNumber].TotalRequests += fsc.TotalTransactions;
                                }
                            }
                            else if (fc.FacadeName == "Exchange WebApp")
                            {
                                if (!facadeMonthTotals.ContainsKey(fc.DisplayName))
                                {
                                    facadeMonthTotals.Add(fc.DisplayName, new FacadeTotalsByMonth());
                                }
                                if (!facadeMonthTotals[fc.DisplayName].MonthlyTotals.ContainsKey(dailyTransactions.Date))
                                {
                                    facadeMonthTotals[fc.DisplayName].MonthlyTotals.Add(dailyTransactions.Date, new FacadeTotals());
                                }
                                facadeMonthTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].ClientBytes += fc.SuccessfulImageBytes;
                                facadeMonthTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].SuccessfulImageResponses += fc.SuccessfulImageResponses;
                                facadeMonthTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].TotalBytes += fc.RelevantBytes;
                                facadeMonthTotals[fc.DisplayName].MonthlyTotals[dailyTransactions.Date].TotalRequests += fc.TotalTransactions;

                                if (!facadeTotals.ContainsKey(fc.DisplayName))
                                {
                                    facadeTotals.Add(fc.DisplayName, new FacadeTotals());
                                }
                                facadeTotals[fc.DisplayName].ClientBytes += fc.SuccessfulImageBytes;
                                facadeTotals[fc.DisplayName].SuccessfulImageResponses += fc.SuccessfulImageResponses;
                                facadeTotals[fc.DisplayName].TotalBytes += fc.RelevantBytes;
                                facadeTotals[fc.DisplayName].TotalRequests += fc.TotalTransactions;

                            }
                            else if (fc.FacadeName == "Federation WebApp")
                            {
                                foreach (FromSiteCounts fsc in fc.SiteCounts.Values)
                                {
                                    foreach (ImageRequest ir in fsc.ImageRequests.Values)
                                    {
                                        string facadeName = "DoD Images - " + ir.ImageSource;
                                        if (!facadeMonthTotals.ContainsKey(facadeName))
                                        {
                                            facadeMonthTotals.Add(facadeName, new FacadeTotalsByMonth());
                                        }
                                        if (!facadeMonthTotals[facadeName].MonthlyTotals.ContainsKey(dailyTransactions.Date))
                                        {
                                            facadeMonthTotals[facadeName].MonthlyTotals.Add(dailyTransactions.Date, new FacadeTotals());
                                        }
                                        facadeMonthTotals[facadeName].MonthlyTotals[dailyTransactions.Date].SuccessfulImageResponses += ir.SuccessfulResponses;
                                        facadeMonthTotals[facadeName].MonthlyTotals[dailyTransactions.Date].TotalBytes += ir.BytesReturned;
                                        facadeMonthTotals[facadeName].MonthlyTotals[dailyTransactions.Date].TotalRequests += ir.RequestCount;

                                        if (!facadeTotals.ContainsKey(facadeName))
                                        {
                                            facadeTotals.Add(facadeName, new FacadeTotals());
                                        }
                                        //facadeTotals[facadeName].ClientBytes += fc.SuccessfulImageBytes;
                                        facadeTotals[facadeName].SuccessfulImageResponses += ir.SuccessfulResponses;
                                        facadeTotals[facadeName].TotalBytes += ir.BytesReturned;
                                        facadeTotals[facadeName].TotalRequests += ir.RequestCount;
                                    }
                                    if (!federationDoDSiteRequestTotals.ContainsKey(fsc.SiteNumber))
                                    {
                                        federationDoDSiteRequestTotals.Add(fsc.SiteNumber, new FacadeTotals());
                                    }
                                    federationDoDSiteRequestTotals[fsc.SiteNumber].TotalBytes += fsc.SuccessfulImageBytes; // bytes returned to the client - should be same as data source in this case since no conversion (except VRad), this is WAN from CVIX to VIX value
                                    federationDoDSiteRequestTotals[fsc.SiteNumber].SuccessfulImageResponses += fsc.SuccessfulImageResponses;
                                    federationDoDSiteRequestTotals[fsc.SiteNumber].TotalRequests += fsc.TotalTransactions;
                                }
                            }
                        }

                    }// for loop for each month

                    // done with calculations for current site
                    /*
                    ConvertBytesValuesToMB(bytesSheet);

                    string dateDescription = "From " + reportStartDate.ToString("MMMM yyyy") + " to " + reportEndDate.ToString("MMMM yyyy");

                    string chartDescription = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteFacadeTotals, ChartType.transactions) + "Total Requests: " + siteTotals.TotalRequests.ToString("N0");
                    Excel.ChartObject requestsChart = CreateWorksheetChart(requestsSheet, dateDescription, thisSite, false, ChartType.transactions, true, false);
                    OutputChartToPpt(pptPres, requestsChart, chartDescription);

                    chartDescription = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteFacadeTotals, ChartType.bytes) + "Total Transferred: " + DailyTransactionsHelper.formatBytes(siteTotals.TotalBytes);
                    Excel.ChartObject bytesChart = CreateWorksheetChart(bytesSheet, dateDescription, thisSite, false, ChartType.bytes, true, false);
                    OutputChartToPpt(pptPres, bytesChart, chartDescription);

                    chartDescription = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteFacadeTotals, ChartType.imageCounts) + "Total Images: " + siteTotals.SuccessfulImageResponses.ToString("N0");
                    Excel.ChartObject imagesChart = CreateWorksheetChart(imagesSheet, dateDescription, thisSite, false, ChartType.imageCounts, true, false);
                    OutputChartToPpt(pptPres, imagesChart, chartDescription);
                     * */
                } // loop for each site

                LogMsg("Generating title and totals charts");
                PowerPointHelper.CreateTitleSlide(pptPres, "CVIX"); // slide 1
                CreateCVIXPresentationExplanationSlide(pptPres);

                OutputTotalRequestsSlide(pptPres, totalRequests, 3, reportStartDate, reportEndDate);
                CreateTotalsSlide(pptPres, facadeTotals, 4, true, false, reportStartDate, reportEndDate, 14);// facadeBytesTotals, facadeRequestTotals, 3);
                //OutputTotalsChartAndSlide(workbook, pptPres, facadeMonthRequestTotals, ChartType.transactions, 5);
                OutputTotalsChartAndSlide(workbook, pptPres, awivTotals, ChartType.bytes, 5, reportStartDate, reportEndDate);
                OutputTotalsChartAndSlide(workbook, pptPres, awivTotals, ChartType.imageCounts, 6, reportStartDate, reportEndDate);
                OutputBusiestAwivSites(pptPres, awivSiteRequestTotals, sitesDictionary, ChartType.bytes, 7, reportStartDate, reportEndDate);
                OutputBusiestAwivSites(pptPres, awivSiteRequestTotals, sitesDictionary, ChartType.imageCounts, 8, reportStartDate, reportEndDate);
                OutputVaSitesRequestingMostDoDData(pptPres, federationDoDSiteRequestTotals, sitesDictionary, ChartType.bytes, 9, reportStartDate, reportEndDate);
                OutputVaSitesRequestingMostDoDData(pptPres, federationDoDSiteRequestTotals, sitesDictionary, ChartType.imageCounts, 10, reportStartDate, reportEndDate);
                OutputTotalsChartAndSlide(workbook, pptPres, facadeMonthTotals, ChartType.bytes, 11, reportStartDate, reportEndDate);
                OutputTotalsChartAndSlide(workbook, pptPres, facadeMonthTotals, ChartType.imageCounts, 12, reportStartDate, reportEndDate);


                workbook.SaveAs(xlsxFilename);
                pptPres.SaveAs(pptFilename);
                pptPres.SaveAs(filename, PowerPoint.PpSaveAsFileType.ppSaveAsPDF, MsoTriState.msoFalse);
            }
            finally
            {
                if (workbook != null)
                    workbook.Close(false);
                if (pptPres != null)
                    pptPres.Close();
                oXL.Quit();
                ppt.Quit();
                System.Runtime.InteropServices.Marshal.ReleaseComObject(workbook);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(oXL);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(ppt);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(pptPres);

                oXL = null;
                GC.Collect();
            }
        }

        private void OutputTotalRequestsSlide(PowerPoint.Presentation pptPres, long totalRequests, int slideIndex, DateTime startDate, DateTime endDate)
        {
            StringBuilder description = new StringBuilder();

            int months = 0;
            DateTime date = startDate;
            while (date.CompareTo(endDate) < 1)
            {
                months++;
                date = date.AddMonths(1);
            }

            // set to the last day of the month
            endDate = endDate.AddMonths(1).AddDays(-1);

            TimeSpan span = endDate.Subtract(startDate);
            int days = (int)span.TotalDays + 1; // add one to count the last day of the month

            string dtDescription = startDate.ToString("MMMM yyyy") + " to " + endDate.ToString("MMMM yyyy");
            description.AppendLine(dtDescription);
            description.AppendLine("Total Requests (all facades): " + totalRequests.ToString("N0"));

            double monthAverage = (double)totalRequests / (double)months;
            double dayAverage = (double)totalRequests / (double)days;

            description.AppendLine("Average Requests Per Month: " + monthAverage.ToString("N2"));
            description.AppendLine("Average Requests Per Day: " + dayAverage.ToString("N2"));

            PowerPointHelper.CreateContentSlide(pptPres, slideIndex, "All CVIX Requests", description.ToString(), true, 22);
        }

        private void OutputVaSitesRequestingMostDoDData(PowerPoint.Presentation pptPres, Dictionary<string,
            FacadeTotals> federationDoDSiteRequestTotals, Dictionary<string, Site> sitesDictionary,
            ChartType chartType, int slideIndex, DateTime startDate, DateTime endDate)
        {
            List<SiteCount> siteCounts = new List<SiteCount>();
            StringBuilder content = new StringBuilder();
            string title = "";
            switch (chartType)
            {
                case ChartType.imageCounts:
                    title = "DoD Images Sent to VA";
                    content.AppendLine("VA Sites that requested the most images from the DoD");
                    foreach (string siteNumber in federationDoDSiteRequestTotals.Keys)
                    {
                        FacadeTotals ft = federationDoDSiteRequestTotals[siteNumber];
                        siteCounts.Add(new SiteCount(siteNumber, ft.SuccessfulImageResponses));
                    }
                    break;
                case ChartType.bytes:
                    title = "DoD Image Bytes Sent to VA";
                    content.AppendLine("VA Sites that requested the most image bytes from the DoD");
                    foreach (string siteNumber in federationDoDSiteRequestTotals.Keys)
                    {
                        FacadeTotals ft = federationDoDSiteRequestTotals[siteNumber];
                        siteCounts.Add(new SiteCount(siteNumber, ft.TotalBytes));
                    }
                    break;
            }

            siteCounts.Sort(new SiteCountComparer());
            int max = 10;
            if (siteCounts.Count < max)
                max = siteCounts.Count;
            for (int i = 0; i < max; i++)
            {
                SiteCount sc = siteCounts[i];
                string name = "";
                if (sitesDictionary.ContainsKey(sc.SiteId))
                {
                    Site site = sitesDictionary[sc.SiteId];
                    name = site.ToString();
                }
                else
                {
                    name = sc.SiteId;
                }
                if (chartType == ChartType.bytes)
                {
                    content.AppendLine(name + ": " + DailyTransactionsHelper.formatBytes(sc.Count));
                }
                else
                {
                    content.AppendLine(name + ": " + sc.Count.ToString("N0"));
                }
            }

            PowerPointHelper.CreateContentSlide(pptPres, slideIndex, title, content.ToString(), true, 18);
        }

        private void OutputBusiestAwivSites(PowerPoint.Presentation pptPres, Dictionary<string,
            FacadeTotals> awivSiteRequestTotals, Dictionary<string, Site> sitesDictionary, 
            ChartType chartType, int slideIndex, DateTime startDate, DateTime endDate)
        {
            List<SiteCount> siteCounts = new List<SiteCount>();
            StringBuilder content = new StringBuilder();
            string title = "";
            switch (chartType)
            {
                case ChartType.imageCounts:
                    title = "AWIV Image Requests";
                    content.AppendLine("Sites that requested the most images through the AWIV");
                    foreach (string siteNumber in awivSiteRequestTotals.Keys)
                    {
                        FacadeTotals ft = awivSiteRequestTotals[siteNumber];
                        siteCounts.Add(new SiteCount(siteNumber, ft.SuccessfulImageResponses));
                    }
                    break;
                case ChartType.bytes:
                    title = "AWIV Image Bytes";
                    content.AppendLine("Sites that requested the most image bytes through the AWIV");
                    foreach (string siteNumber in awivSiteRequestTotals.Keys)
                    {
                        FacadeTotals ft = awivSiteRequestTotals[siteNumber];
                        siteCounts.Add(new SiteCount(siteNumber, ft.TotalBytes));
                    }
                    break;
            }

            siteCounts.Sort(new SiteCountComparer());
            int max = 10;
            if (siteCounts.Count < max)
                max = siteCounts.Count;
            for (int i = 0; i < max; i++)
            {
                SiteCount sc = siteCounts[i];
                string name = "";
                if (sitesDictionary.ContainsKey(sc.SiteId))
                {
                    Site site = sitesDictionary[sc.SiteId];
                    name = site.ToString();
                }
                else
                {
                    name = sc.SiteId;
                }
                if (chartType == ChartType.bytes)
                {
                    content.AppendLine(name + ": " + DailyTransactionsHelper.formatBytes(sc.Count));
                }
                else
                {
                    content.AppendLine(name + ": " + sc.Count.ToString("N0"));
                }
            }

            PowerPointHelper.CreateContentSlide(pptPres, slideIndex, title, content.ToString(), true, 18);
        }

        private void OutputTotalsChartAndSlide(Excel.Workbook workbook, PowerPoint.Presentation pptPres, Dictionary<string, FacadeTotalsByMonth> facadeTotals,
            ChartType chartType, int slideIndex, DateTime startDate, DateTime endDate)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            string sheetName = "";
            switch (chartType)
            {
                case ChartType.bytes:
                    sheetName = "Total Bytes";
                    break;
                case ChartType.transactions:
                    sheetName = "Total Requests";
                    break;
                case ChartType.imageCounts:
                    sheetName = "Total Successful Images";
                    break;
            }
            int i = 2;
            DateTime date = startDate;
            while (date.CompareTo(endDate) <= 0)
            {
                sheet.Cells[1, i] = date.ToString("MMMM, yyyy");
                i++;
                date = date.AddMonths(1);
            }
            foreach (string facadeName in facadeTotals.Keys)
            {
                FacadeTotalsByMonth ftbm = facadeTotals[facadeName];
                
                int row = sheet.UsedRange.Rows.Count + 1;
                int col = 2;
                sheet.Cells[row, 1] = facadeName;

                date = startDate;
                while (date.CompareTo(endDate) <= 0)
                {
                    if (ftbm.MonthlyTotals.ContainsKey(date))
                    {
                        long value = 0L;
                        switch (chartType)
                        {
                            case ChartType.bytes:
                                value = ftbm.MonthlyTotals[date].TotalBytes;
                                break;
                            case ChartType.facadeBytes:
                                value = ftbm.MonthlyTotals[date].ClientBytes;
                                break;
                            case ChartType.imageCounts:
                                value = ftbm.MonthlyTotals[date].SuccessfulImageResponses;
                                break;
                            case ChartType.transactions:
                                value = ftbm.MonthlyTotals[date].TotalRequests;
                                break;
                        }
                        sheet.Cells[row, col] = value;
                    }
                    col++;
                    date = date.AddMonths(1);
                }
            }

            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            if (chartType == ChartType.bytes)
            {
                ConvertBytesValuesToMB(sheet);
            }
            string dtDescription = startDate.ToString("MMMM yyyy") + " to " + endDate.ToString("MMMM yyyy");
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, false, chartType, true, false, sheetName);
            OutputChartToPpt(pptPres, chart, slideIndex, dtDescription);
        }        

        private void CreateCVIXPresentationExplanationSlide(PowerPoint.Presentation presentation)
        {
            string title = "Statistics Explanation";
            StringBuilder description = new StringBuilder();
            description.AppendLine("These statistics are based on the transactions collected from the VIX servers");
            description.AppendLine("Transactions are small actions by the VIX. A request is initiated by a client. These stats are based on requests which might generate multiple transactions");
            description.AppendLine("AWIV requests are requests from the AWIV component");
            description.AppendLine("Exchange requests are requests made by the DoD for VA radiology images");
            description.AppendLine("pacsi are radiology images requested by the VA from the DoD PACSi systems");
            description.AppendLine("haims_radiology are radiology images requested by the VA from the DoD HAIMS systems");
            description.AppendLine("haims_artifact are artifacts images requested by the VA from the DoD HAIMS systems");
            description.AppendLine("Client bytes are the number of bytes sent to the client (cached and uncached, without compression for VistARad)");
            PowerPointHelper.CreateContentSlide(presentation, 2, title, description.ToString(), true, 19);
        }
    }
}
