/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Excel = Microsoft.Office.Interop.Excel;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;

namespace DailyTransactionLogAnalyzerEngine.reports
{
    /// <summary>
    /// This generates a report that has a slide showing the number of requests and bytes per month over a period of time
    /// </summary>
    public class SiteGrowthReport : AbstractExcelReport
    {
        public SiteGrowthReport()
        {
            this.Name = "Site Growth Report";
            this.Description = "For each site create a line chart showing counts for each facade";
            this.ExcelOutputGroupType = ExcelOutputGroupType.allDates;
        }

        protected override void GenerateReport(string analysisDirectory, Dictionary<string, List<DailyTransactions>> sitesDailyTransactionsDictionary, 
            ReportDateType reportDateType, DateTime startDate, DateTime endDate)
        {
            switch (reportDateType)
            {
                case ReportDateType.month:
                    LogMsg("Generating growth report for " + OutputDate(startDate, reportDateType) + " to " + OutputDate(endDate, reportDateType) + ".");
                    OutputMonthlyGrowth(analysisDirectory, sitesDailyTransactionsDictionary, startDate, endDate);
                    break;
                case ReportDateType.year:
                    throw new NotImplementedException();
                default:
                    throw new NotImplementedException();
            }   
        }

        protected override void GenerateReport(string analysisDirectory, List<DailyTransactions> dailyTransactionsList, 
            ReportDateType reportDateType, DateTime date)
        {                     
            throw new NotImplementedException();
        }

        protected override void GenerateReport(string analysisDirectory, DailyTransactions dailyTransactions, ReportDateType reportDateType, DateTime date)
        {
            throw new NotImplementedException();
        }

        private void OutputMonthlyGrowth(string monthlyAnalysisDirectory, Dictionary<string, 
            List<DailyTransactions>> sitesDailyTransactionsDictionary, DateTime reportStartDate, DateTime reportEndDate)
        {
            string filename = monthlyAnalysisDirectory + "\\Rollup\\SiteGrowth.pdf";
            FileInfo f = new FileInfo(filename);
            if (!f.Directory.Exists)
            {
                f.Directory.Create();
            }

            if (File.Exists(filename))
                File.Delete(filename);
            string xlsxFilename = filename + ".xlsx";
            if (File.Exists(xlsxFilename))
                File.Delete(xlsxFilename);
            string pptFilename = filename + ".pptx";
            if (File.Exists(pptFilename))
                File.Delete(pptFilename);

            Excel.Application oXL = new Excel.Application();
            oXL.Visible = false;

            PowerPoint.Application ppt = new PowerPoint.Application();

            PowerPoint.Presentation pptPres = ppt.Presentations.Add(MsoTriState.msoFalse);

            Excel.Workbook workbook = null;
            try
            {
                List<string> siteIds = new List<string>();

                workbook = oXL.Workbooks.Add(Type.Missing);
                // delete all existing sheets except first (Excel creates 3 by default, must keep at least 1)                
                for (int i = workbook.Sheets.Count; i > 1; i--)
                {
                    Excel.Worksheet sheet = workbook.Sheets[i];
                    sheet.Delete();
                }
                Dictionary<string, Site> sitesDictionary = GetSitesDictionary();
                Dictionary<string, FacadeMonthlyTotals> facadeMonthRequestTotals = new Dictionary<string, FacadeMonthlyTotals>();
                Dictionary<string, FacadeMonthlyTotals> facadeMonthBytesTotals = new Dictionary<string, FacadeMonthlyTotals>();
                Dictionary<string, FacadeMonthlyTotals> facadeMonthImageResponseTotals = new Dictionary<string, FacadeMonthlyTotals>();

                //Dictionary<string, long> facadeRequestTotals = new Dictionary<string, long>(); // total requests for each facade
                //Dictionary<string, long> facadeBytesTotals = new Dictionary<string, long>(); // total requests for each facade
                Dictionary<string, FacadeTotals> facadeTotals = new Dictionary<string, FacadeTotals>();

                Dictionary<DateTime, int> monthReceviedDataCount = new Dictionary<DateTime, int>(); // number of sites that had data each month

                List<SiteDailyTransactionsList> sitesDailyTransactionsList = SortByName(sitesDailyTransactionsDictionary, sitesDictionary);

                foreach (SiteDailyTransactionsList siteDailyTransactionList in sitesDailyTransactionsList)// string siteNumber in sitesDailyTransactionsDictionary.Keys)
                {
                    string siteNumber = siteDailyTransactionList.SiteNumber;
                    Site thisSite = SiteHelper.GetSite(siteNumber);
                    LogMsg("Calculating totals for site '" + thisSite.Name + "'");
                    Excel.Worksheet requestsSheet = workbook.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing);
                    requestsSheet.Name = thisSite.ToString(true);
                    
                    int count = workbook.Sheets.Count;
                    Excel.Worksheet bytesSheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[count], Type.Missing, Type.Missing);
                    bytesSheet.Name = thisSite.SiteId + "_bytes";
                    
                    count = workbook.Sheets.Count;
                    Excel.Worksheet imagesSheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[count], Type.Missing, Type.Missing);
                    imagesSheet.Name = thisSite.SiteId + "_images";

                    List<DailyTransactions> dailyTransactionsList = siteDailyTransactionList.DailyTransactionsList;

                    Dictionary<string, int> facadeRequestRow = new Dictionary<string, int>();
                    Dictionary<string, int> facadeBytesRow = new Dictionary<string, int>();
                    Dictionary<string, int> facadeImagesRow = new Dictionary<string, int>();

                    FacadeTotals siteTotals = new FacadeTotals();
                    Dictionary<string, FacadeTotals> siteFacadeTotals = new Dictionary<string, FacadeTotals>();

                    DateTime? startDate = null;
                    DateTime? endDate = null;

                    // each DailyTransaction represents a month of data
                    foreach (DailyTransactions dailyTransactions in dailyTransactionsList)
                    {
                        if (!startDate.HasValue)
                        {
                            startDate = dailyTransactions.Date;
                        }
                        endDate = dailyTransactions.Date; // always move to the current date
                        int requestsCol = requestsSheet.UsedRange.Columns.Count + 1;
                        int bytesCol = bytesSheet.UsedRange.Columns.Count + 1;
                        int imagesCol = imagesSheet.UsedRange.Columns.Count + 1;

                        requestsSheet.Cells[1, requestsCol] = dailyTransactions.Date.ToString("MMMM, yyyy");
                        bytesSheet.Cells[1, bytesCol] = dailyTransactions.Date.ToString("MMMM, yyyy");
                        imagesSheet.Cells[1, imagesCol] = dailyTransactions.Date.ToString("MMMM, yyyy");

                        Excel.Range r = requestsSheet.Cells[1, Type.Missing];
                        r.NumberFormat = "[$-409]mmmm, yyyy;@";
                        r = bytesSheet.Cells[1, Type.Missing];
                        r.NumberFormat = "[$-409]mmmm, yyyy;@";
                        r = imagesSheet.Cells[1, Type.Missing];
                        r.NumberFormat = "[$-409]mmmm, yyyy;@";

                        if (dailyTransactions.TransactionCounts.TotalParentTransactions > 0)
                        {
                            if (monthReceviedDataCount.ContainsKey(dailyTransactions.Date))
                            {
                                monthReceviedDataCount[dailyTransactions.Date]++;
                            }
                            else
                            {
                                monthReceviedDataCount.Add(dailyTransactions.Date, 1);
                            }
                        }
                        foreach (FacadeCounts fc in dailyTransactions.FacadeCounts.Values)
                        {
                            int requestsRow = -1;
                            if (facadeRequestRow.ContainsKey(fc.FacadeName))
                            {
                                requestsRow = facadeRequestRow[fc.FacadeName];
                            }
                            else
                            {
                                requestsRow = facadeRequestRow.Count + 2;
                                facadeRequestRow.Add(fc.FacadeName, requestsRow);
                                requestsSheet.Cells[requestsRow, 1] = fc.FacadeName;
                            }
                            int bytesRow = -1;
                            if (facadeBytesRow.ContainsKey(fc.FacadeName))
                            {
                                bytesRow = facadeBytesRow[fc.FacadeName];
                            }
                            else
                            {
                                bytesRow = facadeBytesRow.Count + 2;
                                facadeBytesRow.Add(fc.FacadeName, bytesRow);
                                bytesSheet.Cells[bytesRow, 1] = fc.FacadeName;
                            }
                            int imagesRow = -1;
                            if (facadeImagesRow.ContainsKey(fc.FacadeName))
                            {
                                imagesRow = facadeImagesRow[fc.FacadeName];
                            }
                            else
                            {
                                imagesRow = facadeImagesRow.Count + 2;
                                facadeImagesRow.Add(fc.FacadeName, imagesRow);
                                imagesSheet.Cells[imagesRow, 1] = fc.FacadeName;
                            }
                            requestsSheet.Cells[requestsRow, requestsCol] = fc.TotalTransactions;
                            bytesSheet.Cells[bytesRow, bytesCol] = fc.RelevantBytes;
                            imagesSheet.Cells[imagesRow, imagesCol] = fc.SuccessfulImageResponses; 

                            if (!facadeTotals.ContainsKey(fc.FacadeName))
                            {
                                facadeTotals.Add(fc.FacadeName, new FacadeTotals());
                            }
                            facadeTotals[fc.FacadeName].TotalRequests += fc.TotalTransactions;
                            facadeTotals[fc.FacadeName].TotalBytes += fc.RelevantBytes;
                            facadeTotals[fc.FacadeName].SuccessfulImageResponses += fc.SuccessfulImageResponses;
                            if (fc.IsClientFacingFacade)
                                facadeTotals[fc.FacadeName].ClientBytes += fc.SuccessfulImageBytes;

                            if (!siteFacadeTotals.ContainsKey(fc.FacadeName))
                            {
                                siteFacadeTotals.Add(fc.FacadeName, new FacadeTotals());
                            }
                            siteFacadeTotals[fc.FacadeName].TotalRequests += fc.TotalTransactions;
                            siteFacadeTotals[fc.FacadeName].TotalBytes += fc.RelevantBytes;
                            siteFacadeTotals[fc.FacadeName].SuccessfulImageResponses += fc.SuccessfulImageResponses;

                            siteTotals.TotalRequests += fc.TotalTransactions;
                            siteTotals.TotalBytes += fc.RelevantBytes;
                            siteTotals.SuccessfulImageResponses += fc.SuccessfulImageResponses;

                            /*
                            if (facadeRequestTotals.ContainsKey(fc.FacadeName))
                            {
                                facadeRequestTotals[fc.FacadeName] += fc.TotalTransactions;
                            }
                            else
                            {
                                facadeRequestTotals.Add(fc.FacadeName, fc.TotalTransactions);
                            }
                            if (facadeBytesTotals.ContainsKey(fc.FacadeName))
                            {
                                facadeBytesTotals[fc.FacadeName] += fc.RelevantBytes;
                            }
                            else
                            {
                                facadeBytesTotals.Add(fc.FacadeName, fc.RelevantBytes);
                            }*/

                            if (facadeMonthBytesTotals.ContainsKey(fc.FacadeName))
                            {
                                FacadeMonthlyTotals fmt = facadeMonthBytesTotals[fc.FacadeName];
                                if (fmt.MonthCounts.ContainsKey(dailyTransactions.Date))
                                {
                                    fmt.MonthCounts[dailyTransactions.Date] += fc.RelevantBytes;
                                }
                                else
                                {
                                    fmt.MonthCounts.Add(dailyTransactions.Date, fc.RelevantBytes);
                                }
                            }
                            else
                            {
                                FacadeMonthlyTotals fmt = new FacadeMonthlyTotals(fc.FacadeName);
                                fmt.MonthCounts.Add(dailyTransactions.Date, fc.RelevantBytes);
                                facadeMonthBytesTotals.Add(fc.FacadeName, fmt);
                            }
                            if (facadeMonthRequestTotals.ContainsKey(fc.FacadeName))
                            {
                                FacadeMonthlyTotals fmt = facadeMonthRequestTotals[fc.FacadeName];
                                if (fmt.MonthCounts.ContainsKey(dailyTransactions.Date))
                                {
                                    fmt.MonthCounts[dailyTransactions.Date] += fc.TotalTransactions;
                                }
                                else
                                {
                                    fmt.MonthCounts.Add(dailyTransactions.Date, fc.TotalTransactions);
                                }
                            }
                            else
                            {
                                FacadeMonthlyTotals fmt = new FacadeMonthlyTotals(fc.FacadeName);
                                fmt.MonthCounts.Add(dailyTransactions.Date, fc.TotalTransactions);
                                facadeMonthRequestTotals.Add(fc.FacadeName, fmt);
                            }

                            if (facadeMonthImageResponseTotals.ContainsKey(fc.FacadeName))
                            {
                                FacadeMonthlyTotals fmt = facadeMonthImageResponseTotals[fc.FacadeName];
                                if (fmt.MonthCounts.ContainsKey(dailyTransactions.Date))
                                {
                                    fmt.MonthCounts[dailyTransactions.Date] += fc.SuccessfulImageResponses;
                                }
                                else
                                {
                                    fmt.MonthCounts.Add(dailyTransactions.Date, fc.SuccessfulImageResponses);
                                }
                            }
                            else
                            {
                                FacadeMonthlyTotals fmt = new FacadeMonthlyTotals(fc.FacadeName);
                                fmt.MonthCounts.Add(dailyTransactions.Date, fc.SuccessfulImageResponses);
                                facadeMonthImageResponseTotals.Add(fc.FacadeName, fmt);
                            }
                        }

                    }// for loop for each month

                    // done with calculations for current site
                    ConvertBytesValuesToMB(bytesSheet);

                    string dateDescription = "From " + startDate.Value.ToString("MMMM yyyy") + " to " + endDate.Value.ToString("MMMM yyyy");

                    string chartDescription = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteFacadeTotals, ChartType.transactions) + "Total Requests: " + siteTotals.TotalRequests.ToString("N0");
                    Excel.ChartObject requestsChart = CreateWorksheetChart(requestsSheet, dateDescription, thisSite, false, ChartType.transactions, true, false);
                    OutputChartToPpt(pptPres, requestsChart, chartDescription);

                    chartDescription = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteFacadeTotals, ChartType.bytes) + "Total Transferred: " + DailyTransactionsHelper.formatBytes(siteTotals.TotalBytes);
                    Excel.ChartObject bytesChart = CreateWorksheetChart(bytesSheet, dateDescription, thisSite, false, ChartType.bytes, true, false);
                    OutputChartToPpt(pptPres, bytesChart, chartDescription);

                    chartDescription = dateDescription + Environment.NewLine + CreateDescriptionForIndividualSite(siteFacadeTotals, ChartType.imageCounts) + "Total Images: " + siteTotals.SuccessfulImageResponses.ToString("N0");
                    Excel.ChartObject imagesChart = CreateWorksheetChart(imagesSheet, dateDescription, thisSite, false, ChartType.imageCounts, true, false);
                    OutputChartToPpt(pptPres, imagesChart, chartDescription);
                } // loop for each site
                
                LogMsg("Generating title and totals charts");
                PowerPointHelper.CreateTitleSlide(pptPres, "Site Growth"); // slide 1
                CreateGrowthPresentationExplanationSlide(pptPres);

                CreateTotalsSlide(pptPres, facadeTotals, 3, false, reportStartDate, reportEndDate, 18);// facadeBytesTotals, facadeRequestTotals, 3);
                OutputResponseCountChartAndSlide(workbook, pptPres, monthReceviedDataCount, 4);
                OutputTotalsChartAndSlide(workbook, pptPres, facadeMonthRequestTotals, ChartType.transactions, 5);
                OutputTotalsChartAndSlide(workbook, pptPres, facadeMonthBytesTotals, ChartType.bytes, 6);
                OutputTotalsChartAndSlide(workbook, pptPres, facadeMonthImageResponseTotals, ChartType.imageCounts, 7);

                //CreateTotalsSlide(pptPres, totalBytesCount, totalTransactionsCount); // slide 3

                // create a sheet/chart that shows the total transactions for all sites for each facade
                //CreateTotalsSheetAndChart(workbook, siteIds, facadeSiteTransactionCounts, ChartType.transactions, dateDescription, pptPres, sitesDictionary, 4);

                // create a sheet/chart that shows the total bytes for all sites for each facade
                //CreateTotalsSheetAndChart(workbook, siteIds, facadeSiteByteCounts, ChartType.bytes, dateDescription, pptPres, sitesDictionary, 5);


                workbook.SaveAs(xlsxFilename);
                pptPres.SaveAs(pptFilename);
                pptPres.SaveAs(filename, PowerPoint.PpSaveAsFileType.ppSaveAsPDF, MsoTriState.msoFalse);
            }
            finally
            {
                if (workbook != null)
                    workbook.Close(false);
                if (pptPres != null)
                    pptPres.Close();
                oXL.Quit();
                ppt.Quit();
                System.Runtime.InteropServices.Marshal.ReleaseComObject(workbook);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(oXL);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(ppt);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(pptPres);

                oXL = null;
                GC.Collect();
            }
        }

        private string CreateDescriptionForIndividualSite(Dictionary<string, FacadeTotals> siteFacadeTotals, ChartType chartType)
        {
            StringBuilder sb = new StringBuilder();
            foreach (string facadeName in siteFacadeTotals.Keys)
            {             
                switch (chartType)
                {
                    case ChartType.bytes:
                        sb.AppendLine(facadeName + ": " + DailyTransactionsHelper.formatBytes(siteFacadeTotals[facadeName].TotalBytes));
                        break;
                    case ChartType.transactions:
                        sb.AppendLine(facadeName + ": " + siteFacadeTotals[facadeName].TotalRequests.ToString("N0"));
                        break;
                    case ChartType.imageCounts:
                        sb.AppendLine(facadeName + ": " + siteFacadeTotals[facadeName].SuccessfulImageResponses.ToString("N0"));
                        break;
                }
                
            }
            return sb.ToString();
        }     

        private void OutputTotalsChartAndSlide(Excel.Workbook workbook, PowerPoint.Presentation pptPres, Dictionary<string, FacadeMonthlyTotals> facadeTotals, 
            ChartType chartType, int slideIndex)
        {
            Excel.Worksheet sheet = workbook.Sheets.Add(Type.Missing, workbook.Sheets[workbook.Sheets.Count], Type.Missing, Type.Missing);
            switch (chartType)
            {
                case ChartType.bytes:
                    sheet.Name = "Total Bytes";
                    break;
                case ChartType.transactions:
                    sheet.Name = "Total Requests";
                    break;
                case ChartType.imageCounts:
                    sheet.Name = "Total Successful Images";
                    break;
            }
            bool firstTime = true;
            DateTime? sDate = null;
            DateTime? eDate = null;
            foreach (FacadeMonthlyTotals fmt in facadeTotals.Values)
            {
                if (firstTime)
                {
                    int i = 2;
                    foreach (DateTime dt in fmt.MonthCounts.Keys)
                    {
                        sheet.Cells[1, i] = dt.ToString("MMMM, yyyy");
                        i++;
                        if (!sDate.HasValue)
                        {
                            sDate = dt;
                        }
                        eDate = dt;
                    }
                    firstTime = false;
                }
                int row = sheet.UsedRange.Rows.Count + 1;
                int col = 2;
                sheet.Cells[row, 1] = fmt.FacadeName;
                foreach (DateTime dt in fmt.MonthCounts.Keys)
                {
                    long value = fmt.MonthCounts[dt];
                    sheet.Cells[row, col] = value;
                    col++;
                }
            }
            Excel.Range tbsr = sheet.Cells[1, Type.Missing];
            tbsr.NumberFormat = "[$-409]mmmm, yyyy;@";
            if (chartType == ChartType.bytes)
            {
                ConvertBytesValuesToMB(sheet);
            }
            string dtDescription = sDate.Value.ToString("MMMM yyyy") + " to " + eDate.Value.ToString("MMMM yyyy");
            Excel.ChartObject chart = CreateWorksheetChart(sheet, dtDescription, null, false, chartType, true, false);
            OutputChartToPpt(pptPres, chart, slideIndex, dtDescription);
        }        

        private void CreateGrowthPresentationExplanationSlide(PowerPoint.Presentation presentation)
        {
            string title = "Statistics Explanation";
            StringBuilder description = new StringBuilder();
            description.AppendLine("These statistics are based on the transactions collected from the VIX servers");
            description.AppendLine("Transactions are small actions by the VIX. A request is initiated by a client. These stats are based on requests which might generate multiple transactions");
            description.AppendLine("Federation requests are requests from another VIX or the CVIX");
            description.AppendLine("Clinical Display requests are requests made by Clinical Display");
            description.AppendLine("VistARad requests are requests made by VistARad");
            description.AppendLine("Image requests for cached data are not included in the bytes calculations");
            description.AppendLine("The charts show the month by month usage for each site");
            PowerPointHelper.CreateContentSlide(presentation, 2, title, description.ToString(), true, 22);
        }

        protected override ReportSiteType GetReportSiteSupportedType()
        {
            return ReportSiteType.vix;
        }
    }

    public class FacadeMonthlyTotals
    {
        public string FacadeName { get; private set; }
        public Dictionary<DateTime, long> MonthCounts { get; private set; }

        public FacadeMonthlyTotals(string facadeName)
        {
            this.FacadeName = facadeName;
            this.MonthCounts = new Dictionary<DateTime, long>();
        }
    }

    public class DateCount
    {
        public DateTime Date { get; private set; }
        public int Count { get; private set; }

        public DateCount(DateTime date, int count)
        {
            this.Date = date;
            this.Count = count;
        }
    }

    public class DateCountComparer : IComparer<DateCount>
    {

        public int Compare(DateCount x, DateCount y)
        {
            return x.Date.CompareTo(y.Date);
        }
    }
}
