/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using VixLogAnalyzerEngine;
using TransactionEntryTracer;

namespace TransactionLogDownloader
{
    public partial class TransactionLogViewer : UserControl
    {
        public TransactionLogViewer()
        {
            InitializeComponent();
        }

        public void DisplayTransactionLogEntries(string transactionId, DateTime fromDate, 
            DateTime toDate, VASite site, List<TransactionLogEntry> entries)
        {
            string dateFormat = "MM/dd/yyyy";
            lblTransactionID.Text = transactionId + " from " + site.ToString();
            lblDateRange.Text = fromDate.ToString(dateFormat) + " to " + toDate.ToString(dateFormat);
            foreach (TransactionLogEntry entry in entries)
            {
                DisplayEntry(new SiteTransactionLogEntry(entry, site));
            }
        }

        public void DisplayTransactionLogEntries(string transactionId, DateTime date,
            VASite site, List<TransactionLogEntry> entries)
        {
            lblTransactionID.Text = transactionId;
            string dateFormat = "MM/dd/yyyy";
            lblDateRange.Text = date.ToString(dateFormat);
            foreach (TransactionLogEntry entry in entries)
            {
                DisplayEntry(new SiteTransactionLogEntry(entry, site));
            }
        }

        public void Clear()
        {
            dgvEntries.Rows.Clear();
            dgvEntries.Columns.Clear();
            lblDateRange.Text = "";
            lblTransactionID.Text = "";
        }

        private void DisplayEntry(SiteTransactionLogEntry siteTransactionLogEntry)
        {
            DataGridViewRow row = new DataGridViewRow();

            if (dgvEntries.Rows.Count <= 0)
            {
                // create columns
                dgvEntries.Columns.Add(createColumn("Site"));
                foreach (string key in siteTransactionLogEntry.TransactionLogEntry.Values.Keys)
                {
                    dgvEntries.Columns.Add(createColumn(key));
                }
            }
            //dgvEntries.Rows.Add(row);
            row.CreateCells(dgvEntries);
            row.Cells[0].Value = siteTransactionLogEntry.VASite.SiteName;
            row.Tag = siteTransactionLogEntry;

            int i = 1;
            foreach (string key in siteTransactionLogEntry.TransactionLogEntry.Values.Keys)
            {
                row.Cells[i].Value = siteTransactionLogEntry.TransactionLogEntry.Values[key];
                i++;
            }
            dgvEntries.Rows.Add(row);
        }

        private DataGridViewColumn createColumn(string key)
        {
            DataGridViewColumn col = new DataGridViewColumn();
            DataGridViewCell cell = new DataGridViewTextBoxCell();
            col.CellTemplate = cell;
            col.Name = key;
            col.HeaderText = key;
            return col;
        }

        private void displayTransactionDetailsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SiteTransactionLogEntry siteTransactionLogEntry = GetSelectedLogEntry();
            if (siteTransactionLogEntry != null)
            {
                TransactionLogUrlHelper.LaunchTransactionViewer(siteTransactionLogEntry);
            }
        }

        private SiteTransactionLogEntry GetSelectedLogEntry()
        {
            if (dgvEntries.SelectedRows[0].Tag != null)
            {
                SiteTransactionLogEntry siteTransactionLogEntry = (SiteTransactionLogEntry)dgvEntries.SelectedRows[0].Tag;
                return siteTransactionLogEntry;
            }
            return null;
        }

        private void copyTransactionToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SiteTransactionLogEntry siteTransactionLogEntry = GetSelectedLogEntry();
            if (siteTransactionLogEntry != null)
            {
                StringBuilder sb = new StringBuilder();

                Dictionary<string, string> values = siteTransactionLogEntry.TransactionLogEntry.Values;
                foreach (string key in values.Keys)
                {
                    sb.AppendLine(key + "=" + values[key]);
                }

                Clipboard.SetText(sb.ToString());
                
            }
        }
    }
}
