/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using VixLogAnalyzerEngine;
using TransactionLogStatisticsEngine.commands;

namespace TransactionLogStatistics
{
    public partial class TransactionLogEntriesViewer : UserControl
    {
        public TransactionLogEntriesViewer()
        {
            InitializeComponent();
        }

        public void DisplayTransactionEntries(TransactionLogProcessingCommand command)
        {
            foreach (TransactionLogEntry entry in command.MatchingEntries)
            {
                TreeNode node = new TreeNode(entry.TransactionId);
                node.Tag = entry;
                treEntries.Nodes.Add(node);
                if (entry.IsError())
                {
                    node.ForeColor = Color.Red;
                }
                AddChildNodes(entry, node);
            }
            TransactionLogStatisticsEngine.commands.TransactionLogEntriesStatistics stats = command.calculateStatistics();
            txtStats.Text = stats.ToString();
        }

        private void AddChildNodes(TransactionLogEntry entry, TreeNode parentNode)
        {
            if (entry.ChildEntries != null)
            {
                foreach (TransactionLogEntry child in entry.ChildEntries)
                {
                    TreeNode node = new TreeNode(child.TransactionId);
                    parentNode.Nodes.Add(node);
                    node.ForeColor = parentNode.ForeColor;
                    node.Tag = child;
                    AddChildNodes(child, node);
                    
                }
            }
        }

        private void treEntries_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node.Tag != null)
            {
                TransactionLogEntry entry = (TransactionLogEntry)e.Node.Tag;
                txtDetails.Text = entry.ToString();
            }
        }
    }
}
