/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TransactionLogStatisticsEngine;
using TransactionLogStatisticsEngine.commands;
using VixLogAnalyzerEngine;
using System.Configuration;
using System.Reflection;
using System.IO;
using System.Xml;

namespace TransactionLogStatistics
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private string CommandsDirectory { get; set; }

        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                TransactionLogStatisticsEngine.TransactionLogStatisticsHelper engine =
                    new TransactionLogStatisticsEngine.TransactionLogStatisticsHelper(txtLogsDirectory.Text, txtStatisticsDirectory.Text);

                HaimsRadiologyImageRetrievesCommand haimsCommand = new HaimsRadiologyImageRetrievesCommand();
                PacsiRadiologyImageRetrievesCommand pacsiCommand = new PacsiRadiologyImageRetrievesCommand();
                NcatImageRetrievesCommand ncatCommand = new NcatImageRetrievesCommand();
                HaimsArtifactsImageRetrievesCommand haimsArtifactsCommand = new HaimsArtifactsImageRetrievesCommand();
                //RadiologyImageRetrieveCommand radiologyCommand = new RadiologyImageRetrieveCommand();
                List<TransactionLogProcessingCommand> commands = new List<TransactionLogProcessingCommand>();
                /*
                commands.Add(haimsCommand);
                commands.Add(pacsiCommand);
                commands.Add(ncatCommand);
                commands.Add(haimsArtifactsCommand);
                //commands.Add(radiologyCommand);
                 * */
                commands.Add(new XcaVaQueries());
                commands.Add(new XcaVaRetrieves());
                commands.Add(new ExchangeVaImageRetrieves());

                List<string> sites = new List<string>();

                string[] siteArray = txtSites.Text.Split(",".ToCharArray());
                foreach (string s in siteArray)
                {
                    sites.Add(s);
                }
                /*
                sites.Add("2001.1");
                sites.Add("2001.2");
                sites.Add("2001.3");
                 * */
                //sites.Add("test1");
                //DateTime date = new DateTime(2012, 3, 16);
                DateTime date = dtDate.Value;
                engine.searchLogs(sites, date, commands);

                foreach (TransactionLogProcessingCommand cmd in commands)
                {
                    TabPage tp = new TabPage(cmd.GetType().Name);
                    TransactionLogEntriesViewer viewer = new TransactionLogEntriesViewer();
                    viewer.Parent = tp;
                    viewer.Dock = DockStyle.Fill;
                    tcResults.TabPages.Add(tp);
                    viewer.DisplayTransactionEntries(cmd);
                }

                //engine.OutputStatistics(date, commands);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void btnProcessDateRange_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                TransactionLogStatisticsEngine.TransactionLogStatisticsHelper engine =
                    new TransactionLogStatisticsEngine.TransactionLogStatisticsHelper(txtLogsDirectory.Text, txtStatisticsDirectory.Text);

                HaimsRadiologyImageRetrievesCommand haimsCommand = new HaimsRadiologyImageRetrievesCommand();
                PacsiRadiologyImageRetrievesCommand pacsiCommand = new PacsiRadiologyImageRetrievesCommand();
                NcatImageRetrievesCommand ncatCommand = new NcatImageRetrievesCommand();
                HaimsArtifactsImageRetrievesCommand haimsArtifactsCommand = new HaimsArtifactsImageRetrievesCommand();
                //RadiologyImageRetrieveCommand radiologyCommand = new RadiologyImageRetrieveCommand();
                ExchangeVaImageRetrieves exchangeCommand = new ExchangeVaImageRetrieves();
                List<TransactionLogProcessingCommand> commands = new List<TransactionLogProcessingCommand>();

                commands.Add(haimsCommand);
                commands.Add(pacsiCommand);
                commands.Add(ncatCommand);
                commands.Add(haimsArtifactsCommand);
                commands.Add(new XcaVaRetrieves());
                commands.Add(exchangeCommand);
                //commands.Add(radiologyCommand);

                List<string> sites = new List<string>();
                string[] siteArray = txtDateRangeSites.Text.Split(",".ToCharArray());
                foreach (string s in siteArray)
                {
                    sites.Add(s);
                }
                /*
                sites.Add("2001.1");
                sites.Add("2001.2");
                sites.Add("2001.3");
                 * */

                DateTime currentDate = dtpStartDate.Value.Date;
                DateTime endDate = dtpEndDate.Value.Date;

                while (currentDate <= endDate)
                {
                    engine.searchLogs(sites, currentDate, commands);
                    engine.OutputStatistics(currentDate, commands);
                    currentDate = currentDate.AddDays(1);
                    foreach (TransactionLogProcessingCommand cmd in commands)
                    {
                        cmd.ClearMatchingEntries();
                    }

                }
                MessageBox.Show("Done");

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }



        private void frmMain_Load(object sender, EventArgs e)
        {
            dtpStartDate.Value = DateTime.Today.AddDays(-2);// new DateTime(2012, 11, 1);
            dtpEndDate.Value = DateTime.Today.AddDays(-2); //new DateTime(2012, 11, 1);

            //dtpStartDate.Value = new DateTime(2012, 11, 1);
            //dtpEndDate.Value = new DateTime(2012, 11, 3);

            dtDate.Value = new DateTime(2012, 11, 1);
            //txtSites.Text = "2001.1,2001.2,2001.3";
            //txtSites.Text = "565";
            txtStatisticsDirectory.Text = ConfigurationManager.AppSettings["StatisticsDirectory"];
            txtLogsDirectory.Text = ConfigurationManager.AppSettings["LogsDirectory"];
            //txtDateRangeSites.Text = ConfigurationManager.AppSettings["DateRangeSites"];

            string dir = Path.GetDirectoryName(Application.ExecutablePath);
            CommandsDirectory = dir + "\\commands";

            try
            {
                if (!Directory.Exists(CommandsDirectory))
                {
                    Directory.CreateDirectory(CommandsDirectory);
                }

                List<TransactionLogProcessingCommand> commands =
                    TransactionLogStatisticsHelper.GetCommands(CommandsDirectory);
                foreach (TransactionLogProcessingCommand command in commands)
                {
                    clbCommands.Items.Add(command);
                }

                txtCommandDescription.Text = "My new command";
                txtCommandName.Text = "New Command Name";
                txtCommandEvaluation.Text = "entry.QueryType.Contains(\"Federation WebApp V4 getPatientArtifacts\")";

                Type t = typeof(ParentTransactionLogEntry);
                PropertyInfo[] properties = t.GetProperties();
                foreach (PropertyInfo prop in properties)
                {
                    if (prop.CanRead)
                    {
                        txtTransactionEntryFields.AppendText(prop.Name + Environment.NewLine);
                    }
                }

            }
            catch (Exception ex)
            {
                LogMsg(ex.Message);
                MessageBox.Show(ex.Message);
            }

            cboCommandOutputType.Items.Add(GenericCommandOutput.List);
            cboCommandOutputType.Items.Add(GenericCommandOutput.Image);
            cboCommandOutputType.Items.Add(GenericCommandOutput.Count);

            cboCommandOutputType.SelectedItem = GenericCommandOutput.Count;
            txtParseFilename.Text = @"C:\Users\zzzzzzwerfej\Desktop\Year_2013.xml";
        }

        private void btnRunSpecificCommands_Click(object sender, EventArgs e)
        {
            frmRunCommands runCommands = new frmRunCommands();
            runCommands.DisplayRunCommands(txtLogsDirectory.Text);
        }

        private void btnDateRangeCommands_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;

                TransactionLogStatisticsEngine.TransactionLogStatisticsHelper engine =
                    new TransactionLogStatisticsEngine.TransactionLogStatisticsHelper(txtLogsDirectory.Text, txtStatisticsDirectory.Text);
                engine.OnLogMessageEvent += LogMsg;

                List<TransactionLogProcessingCommand> commands = new List<TransactionLogProcessingCommand>();
                foreach (object obj in clbCommands.CheckedItems)
                {
                    TransactionLogProcessingCommand cmd = (TransactionLogProcessingCommand)obj;
                    commands.Add(cmd);
                }

                List<string> sites = new List<string>();

                if (chkAllVixSites.Checked)
                {
                    SiteService.SiteServiceHelper helper = new SiteService.SiteServiceHelper();
                    List<VISACommon.VaSite> vixSites = SiteService.SiteServiceHelper.getVaSites("http://siteserver.vista.domain.ext/VistaWebSvcs/ImagingExchangeSiteService.asmx",
                        false);

                    foreach (VISACommon.VaSite site in vixSites)
                    {
                        if (!site.SiteNumber.StartsWith("200") && 
                            (!site.SiteNumber.StartsWith("100")))
                        {
                            sites.Add(site.SiteNumber);
                        }
                    }
                }
                else
                {
                    string[] siteArray = txtDateRangeSites.Text.Split(",".ToCharArray());
                    foreach (string s in siteArray)
                    {
                        sites.Add(s);
                    }
                }

                foreach (string s in sites)
                {
                    LogMsg("Searching site [" + s + "]");
                }

                DateTime currentDate = dtpStartDate.Value.Date;
                DateTime endDate = dtpEndDate.Value.Date;

                while (currentDate <= endDate)
                {
                    LogMsg("Searching date '" + currentDate.ToString() + "'");
                    engine.searchLogs(sites, currentDate, commands);
                    engine.OutputStatistics(currentDate, commands);
                    currentDate = currentDate.AddDays(1);
                    foreach (TransactionLogProcessingCommand cmd in commands)
                    {
                        cmd.ClearMatchingEntries();
                    }
                }
                LogMsg("Done");
                //MessageBox.Show("Done");

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void LogMsg(string msg)
        {
            txtLog.AppendText(msg + Environment.NewLine);
            this.txtLog.SelectionStart = txtLog.Text.Length;
            this.txtLog.ScrollToCaret();
            Application.DoEvents();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                //TransactionLogProcessingCommand command = GenericCommandHelper.CreateCommand();
                //clbCommands.Items.Add(command);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnCreateCommand_Click(object sender, EventArgs e)
        {
            try
            {
                GenericCommand genericCommand = new GenericCommand(txtCommandName.Text, txtCommandDescription.Text);
                genericCommand.Evaluation = txtCommandEvaluation.Text;
                genericCommand.OutputType = (GenericCommandOutput)cboCommandOutputType.SelectedItem;
                TransactionLogProcessingCommand command = GenericCommandHelper.CreateCommand(genericCommand);
                clbCommands.Items.Add(command);
                LogMsg("Command '" + command.GetName() + "' created");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnSaveCommand_Click(object sender, EventArgs e)
        {
            try
            {
                GenericCommand genericCommand = new GenericCommand(txtCommandName.Text, txtCommandDescription.Text);
                genericCommand.Evaluation = txtCommandEvaluation.Text;
                genericCommand.OutputType = (GenericCommandOutput)cboCommandOutputType.SelectedItem;
                TransactionLogProcessingCommand command = GenericCommandHelper.CreateCommand(genericCommand);
                string  filename = CommandsDirectory + "\\" + DateTime.Now.Ticks + ".xml";
                genericCommand.Save(filename);
                LogMsg("Command '" + command.GetName() + "' saved to file '" + filename + "'");

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnCountUnique_Click(object sender, EventArgs e)
        {
            try
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(@"C:\Users\zzzzzzwerfej\Desktop\Year_2013 - Copy.xml");
                XmlNodeList entryNodes = xmlDoc.SelectNodes("VixStatistics/Entry");
                Dictionary<string, int> users = new Dictionary<string, int>();
                //Dictionary<string, int> requestingSites = new Dictionary<string, int>();
                foreach (XmlNode entryNode in entryNodes)
                {
                    string user = entryNode.Attributes["user"].Value;
                    string requestingSite = entryNode.Attributes["requestingSite"].Value;
                    string userSite = user + "@" + requestingSite;
                    if (!users.ContainsKey(userSite))
                    {
                        users.Add(userSite, 1);
                    }
                }
                LogMsg("Users:");
                foreach (string user in users.Keys)
                {
                    LogMsg(user);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnParseResults_Click(object sender, EventArgs e)
        {
            try
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(txtParseFilename.Text);
                XmlNodeList entryNodes = xmlDoc.SelectNodes("VixStatistics/Dates/Date/Source/Entry");
                Dictionary<string, UserCount> users = new Dictionary<string, UserCount>();
                //Dictionary<string, int> requestingSites = new Dictionary<string, int>();
                foreach (XmlNode entryNode in entryNodes)
                {
                    string user = entryNode.Attributes["user"].Value;
                    string requestingSite = entryNode.Attributes["requestingSite"].Value;
                    string userSite = user + "@" + requestingSite;
                    if (!users.ContainsKey(userSite))
                    {
                        UserCount count = new UserCount(userSite);
                        count.Count = 1;
                        users.Add(userSite, count);
                    }
                    else
                    {
                        UserCount count = users[userSite];
                        count.Count++;
                    }
                }
                LogMsg("Users:");
                List<UserCount> allUsers = new List<UserCount>();
                allUsers.AddRange(users.Values);
                allUsers.Sort(new UserCountComparer());
                foreach (UserCount uc in allUsers)
                {
                    LogMsg(uc.Username + " [" + uc.Count + "]");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnCVIX_Click(object sender, EventArgs e)
        {
            txtDateRangeSites.Text = "2001.1,2001.2,2001.3,2001.4,2001.5,2001.6";
        }

        private void btnSaveSettings_Click(object sender, EventArgs e)
        {
            Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);

            //make changes
            config.AppSettings.Settings["StatisticsDirectory"].Value = txtStatisticsDirectory.Text;
            config.AppSettings.Settings["LogsDirectory"].Value = txtLogsDirectory.Text;
         
            //save to apply changes
            config.Save(ConfigurationSaveMode.Modified);
            ConfigurationManager.RefreshSection("appSettings");
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "Transaction Log Statistics\nVersion: " + Application.ProductVersion,
                "About",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

    }

    public class UserCount
    {
        public string Username { get; private set; }
        public int Count { get; set; }

        public UserCount(string username)
        {
            this.Username = username;
            this.Count = 0;
        }
    }

    public class UserCountComparer : IComparer<UserCount>
    {
        public int Compare(UserCount x, UserCount y)
        {
            return x.Count.CompareTo(y.Count);
        }
    }
}
