/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using TransactionLogStatisticsEngine.commands;
using System.IO;
using System.Reflection;

namespace TransactionLogStatisticsConsole
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                //Settings settings = new Settings
                string path = Path.GetDirectoryName(
                     Assembly.GetAssembly(typeof(Program)).CodeBase);
                path = path + @"\settings.xml";
                Uri uri = new Uri(path);
                //FileInfo file = new FileInfo(uri.LocalPath);

                Settings settings = new Settings(uri.LocalPath);

                string statisticsDirectory = ConfigurationManager.AppSettings["StatisticsDirectory"];
                string logsDirectory = ConfigurationManager.AppSettings["LogsDirectory"];
                string sitesList = ConfigurationManager.AppSettings["DateRangeSites"];

                TransactionLogStatisticsEngine.TransactionLogStatisticsHelper engine =
                    new TransactionLogStatisticsEngine.TransactionLogStatisticsHelper(logsDirectory, statisticsDirectory);

                HaimsRadiologyImageRetrievesCommand haimsCommand = new HaimsRadiologyImageRetrievesCommand();
                PacsiRadiologyImageRetrievesCommand pacsiCommand = new PacsiRadiologyImageRetrievesCommand();
                NcatImageRetrievesCommand ncatCommand = new NcatImageRetrievesCommand();
                HaimsArtifactsImageRetrievesCommand haimsArtifactsCommand = new HaimsArtifactsImageRetrievesCommand();
                //RadiologyImageRetrieveCommand radiologyCommand = new RadiologyImageRetrieveCommand();
                ExchangeVaImageRetrieves exchangeCommand = new ExchangeVaImageRetrieves();
                List<TransactionLogProcessingCommand> commands = new List<TransactionLogProcessingCommand>();

                commands.Add(haimsCommand);
                commands.Add(pacsiCommand);
                commands.Add(ncatCommand);
                commands.Add(haimsArtifactsCommand);
                commands.Add(new XcaVaRetrieves());
                commands.Add(exchangeCommand);
                //commands.Add(radiologyCommand);

                List<string> sites = new List<string>();
                string[] siteArray = sitesList.Split(",".ToCharArray());
                foreach (string s in siteArray)
                {
                    sites.Add(s);
                }
                DateTime today = DateTime.Now;

                // processing should not happen for the current month, so the end date should be the last date of the previous month
                DateTime endDate = new DateTime(today.Year, today.Month, 1); // the first day of the current month
                endDate = endDate.AddDays(-1); // one less than the first day is the last day of the previous month

                DateTime currentDate = new DateTime(settings.LastProcessedYear, 
                    settings.LastProcessedMonth, 1); // first day of last processed month
                currentDate = currentDate.AddMonths(1);
                Console.WriteLine("Starting processing with '" + currentDate.ToString() + "'.");
                Console.WriteLine("Ending processing with '" + endDate.ToString() + "'.");
             
                while (currentDate <= endDate)
                {
                    engine.searchLogs(sites, currentDate, commands);
                    engine.OutputStatistics(currentDate, commands);
                    currentDate = currentDate.AddDays(1);
                    foreach (TransactionLogProcessingCommand cmd in commands)
                    {
                        cmd.ClearMatchingEntries();
                    }

                }
                settings.LastProcessedYear = endDate.Year;
                settings.LastProcessedMonth = endDate.Month;
                settings.Save();
                Console.WriteLine("Done");

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message + "\n\n" + ex.StackTrace);
            }
        }
    }
}
