/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;

namespace TransactionLogStatisticsConsole
{
    public class Settings
    {
        public string Filename { get; set; }
        public int LastProcessedMonth { get; set; }
        public int LastProcessedYear { get; set; }

        public Settings(String filename)
        {
            this.Filename = filename;
            LoadOrCreate();
        }

        private void LoadOrCreate()
        {
            if (File.Exists(Filename))
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(Filename);
                XmlNode settingsNode = xmlDoc.SelectSingleNode("TransactionLogStatisticsConsole/Settings");
                LastProcessedMonth = int.Parse(settingsNode.Attributes["Month"].Value);
                LastProcessedYear = int.Parse(settingsNode.Attributes["Year"].Value);
            }
            else
            {
                LastProcessedMonth = 1;
                LastProcessedYear = 2012;
                XmlDocument xmlDoc = new XmlDocument();
                XmlNode rootNode = xmlDoc.CreateElement("TransactionLogStatisticsConsole");
                XmlNode settingsNode = xmlDoc.CreateElement("Settings");
                xmlDoc.AppendChild(rootNode);
                rootNode.AppendChild(settingsNode);
                XmlAttribute attr = xmlDoc.CreateAttribute("Month");
                attr.Value = LastProcessedMonth + "";
                settingsNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Year");
                attr.Value = LastProcessedYear + "";
                settingsNode.Attributes.Append(attr);
                xmlDoc.Save(Filename);
            }
        }

        public void Save()
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(Filename);
            XmlNode settingsNode = xmlDoc.SelectSingleNode("TransactionLogStatisticsConsole/Settings");
            settingsNode.Attributes["Month"].Value = LastProcessedMonth + "";
            settingsNode.Attributes["Year"].Value = LastProcessedYear + "";
            xmlDoc.Save(Filename);
        }
    }
}
