/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VixLogAnalyzerEngine;
using System.Xml;

namespace TransactionLogStatisticsEngine.commands
{
    /// <summary>
    /// The intention of this was to count how many images are in studies by modality (so the average CT has X images, the average MR has Y images, etc). Unfortunately the StudyURN doesn't contain the modality so there is no way to determine this
    /// </summary>
    public class CountStudySizes : TransactionLogProcessingCommand
    {

        public override bool isEntryMatching(VixLogAnalyzerEngine.TransactionLogEntry entry)
        {
            return (entry.CommandClassName == "GetStudyCommandImpl");
        }

        public override string GetName()
        {
            return "Counted Studies";
        }

        public override string GetDescription()
        {
            return "Counts the number of studies and how many images in each study";
        }

        public override bool includeParentlessChildCommands()
        {
            return true;
        }

        public override TransactionLogEntriesStatistics calculateStatistics()
        {
            CountStudiesTransactionLogStatistics result = new CountStudiesTransactionLogStatistics();

            foreach (TransactionLogEntry entry in MatchingEntries)
            {
                //entry.Values["Items (Size) Returned"]
                if (entry.Values.ContainsKey("Items (Size) Returned"))
                {
                    string modality = entry.Urn;// entry.Modality;
                    string value = entry.Values["Items (Size) Returned"];
                    if (value != null && value.Length > 0)
                    {
                        int count = int.Parse(value);
                        StudyCount studyCount = null;
                        if (result.Counts.ContainsKey(modality))
                        {
                            studyCount = result.Counts[modality];
                        }
                        else
                        {
                            studyCount = new StudyCount();
                            result.Counts.Add(modality, studyCount);
                        }
                        studyCount.NumStudies++;
                        studyCount.NumImages += count;
                    }
                }
            }

            return result;
        }
    }

    public class StudyCount
    {
        public int NumStudies { get; set; }
        public int NumImages { get; set; }

        public StudyCount()
        {
            NumStudies = 0;
            NumImages = 0;
        }
    }

    public class CountStudiesTransactionLogStatistics : TransactionLogEntriesStatistics
    {
        public Dictionary<string, StudyCount> Counts = new Dictionary<string, StudyCount>();


        public XmlNode toXml(XmlDocument xmlDoc, string sourceName)
        {
            StringBuilder sb = new StringBuilder();
            foreach (string modality in Counts.Keys)
            {
                sb.AppendLine(modality);
            }

            throw new NotImplementedException();
        }

        public void updateXmlStats(XmlNode xmlNode)
        {
            throw new NotImplementedException();
        }
    }
}
