/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VixLogAnalyzerEngine;

namespace TransactionLogStatisticsEngine.commands
{
    public class DailyStatsCommand : TransactionLogProcessingCommand
    {
        public DailyStatsCommand()
            : base()
        {
        }

        public override bool isEntryMatching(VixLogAnalyzerEngine.TransactionLogEntry entry)
        {
            return true;
        }

        public override string GetName()
        {
            return "Dails Stats";
        }

        public override string GetDescription()
        {
            return "All Transactions";
        }

        public override TransactionLogEntriesStatistics calculateStatistics()
        {
            GenericFieldsStatistics stats = new GenericFieldsStatistics();
            foreach (TransactionLogEntry entry in MatchingEntries)
            {
                //stats.IncrementValue(new GenericFieldDescription("VistA Login Method", entry.VistaLoginMethod, false));
                //stats.IncrementValue(new GenericFieldDescription("Exception Class Name", entry.ExceptionClassName, false));
                //stats.IncrementValue(new GenericFieldDescription("Responding Site", entry.RespondingSite, false));
                //stats.IncrementValue(new GenericFieldDescription("Requesting Site", entry.RequestingSite, false));

                string respondingSite = entry.RespondingSite;
                if (respondingSite.StartsWith("1.3.6.1.4.1.3768,"))
                {
                    respondingSite = respondingSite.Substring(17);
                }

                if (entry.QueryType.StartsWith("Clinical Display WebApp"))
                {
                    /*
                    if (entry.QueryType.Contains("image transfer"))
                    {
                        stats.IncrementValue(new GenericFieldDescription("Query Type", "Clinical Display Image Requests"));
                        stats.AddValue(new GenericFieldDescription("Query Type", "Clinical Display Bytes"),  (long)entry.getPropertyFromParentOrChild("FacadeBytesReturned"));
                        stats.IncrementValue(new GenericFieldDescription("Clinical Display Image Responses From Site", entry.RespondingSite, false));
                    }
                    else
                    {
                        stats.IncrementValue(new GenericFieldDescription("Query Type", "Clinical Display Metadata Requests"));
                        stats.IncrementValue(new GenericFieldDescription("Clinical Display Metadata Responses From Site", entry.RespondingSite,false));
                    }*/

                    if (respondingSite.Length > 0)
                    {
                        // got data from a site
                        //stats.IncrementValue(new GenericFieldDescription(entry.RespondingSite, "Responses"));

                        if (entry.QueryType.Contains("image transfer"))
                        {
                            stats.IncrementValue(new GenericFieldDescription(respondingSite, "Clinical Display Images Received"));
                            stats.AddValue(new GenericFieldDescription(respondingSite, "Clinical Display Bytes Received"), (long)entry.getPropertyFromParentOrChild("FacadeBytesReturned", false));
                        }
                        else
                        {
                            stats.IncrementValue(new GenericFieldDescription(respondingSite, "Clinical Display Metadata Responses"));
                        }
                    }
                }
                else if (entry.QueryType.StartsWith("VistARad WebApp"))
                {


                    if (respondingSite.Length > 0)
                    {
                        // got data from a site
                        //stats.IncrementValue(new GenericFieldDescription(entry.RespondingSite, "Responses"));

                        if (entry.QueryType.Contains("image transfer"))
                        {
                            stats.IncrementValue(new GenericFieldDescription(respondingSite, "VistARad Images Received"));
                            stats.AddValue(new GenericFieldDescription(respondingSite, "VistARad Bytes Received"), (long)entry.getPropertyFromParentOrChild("FacadeBytesReturned", false));
                        }
                        else
                        {
                            stats.IncrementValue(new GenericFieldDescription(respondingSite, "VistARad Metadata Responses"));
                        }
                    }
                }
                else if (entry.QueryType.StartsWith("Federation WebApp"))
                {
                    if (entry.RequestingSite != null && entry.RequestingSite.Length > 0)
                    {
                        if (entry.QueryType.Contains("image transfer"))
                        {
                            stats.IncrementValue(new GenericFieldDescription(entry.RequestingSite, "Federation Images Sent"));
                            stats.AddValue(new GenericFieldDescription(entry.RequestingSite, "Federation Bytes Sent"), (long)entry.getPropertyFromParentOrChild("FacadeBytesReturned", false));
                        }
                        else
                        {
                            stats.IncrementValue(new GenericFieldDescription(entry.RequestingSite, "Federation Metadata Responses Sent"));
                        }
                    }
                }
            }

            return stats;
        }
    }
}
