/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Xml;
using System.IO;

namespace TransactionLogStatisticsEngine.commands
{
    public class GenericCommandHelper
    {
        public static TransactionLogProcessingCommand CreateCommand(GenericCommand genericCommand)
        {

            CSharpCodeProvider c = new CSharpCodeProvider();

            ICodeCompiler icc = c.CreateCompiler();
            CompilerParameters cp = new CompilerParameters();

            cp.ReferencedAssemblies.Add("system.dll");
            cp.ReferencedAssemblies.Add("system.core.dll");
            cp.ReferencedAssemblies.Add("system.xml.dll");
            cp.ReferencedAssemblies.Add("system.data.dll");
            cp.ReferencedAssemblies.Add("system.windows.forms.dll");
            cp.ReferencedAssemblies.Add("System.Xml.Linq.dll");
            cp.ReferencedAssemblies.Add("VixLogAnalyzerEngine.dll");
            cp.ReferencedAssemblies.Add("TransactionLogStatisticsEngine.dll");
            //cp.ReferencedAssemblies.Add("system.drawing.dll");

            cp.CompilerOptions = "/t:library";
            cp.GenerateInMemory = true;

            StringBuilder sb = new StringBuilder("");
            sb.Append("using System;\n");
            sb.Append("using System.Text;\n");
            sb.Append("using System.Linq;\n");
            sb.Append("using System.Collections.Generic;\n");
            sb.Append("using VixLogAnalyzerEngine;\n");
            sb.Append("using TransactionLogStatisticsEngine.commands;\n");
            //sb.Append("using System.Drawing;\n");

            string className = "GenericCommand_" + DateTime.Now.Ticks;
            string classNamespace = "TransactionLogStatisticsEngine.commands";

            sb.Append("namespace " + classNamespace + " { \n");
            sb.Append("public class " + className + " : TransactionLogProcessingCommand { \n");

            sb.Append("public " + className + "() : base() {}\n");

            sb.Append("public override bool isEntryMatching(VixLogAnalyzerEngine.TransactionLogEntry entry)\n");
            sb.Append("{\n");
            sb.Append("return (" + genericCommand.Evaluation + ");\n");
            //sb.Append("return true;\n");
            sb.Append("}\n");


            sb.Append("public override string GetName()\n");
            sb.Append("{\n");
            sb.Append("return \"" + genericCommand.Name + "\";\n");
            sb.Append("}\n");

            sb.Append("public override string GetDescription()\n");
            sb.Append("{\n");
            sb.Append("return \"" + genericCommand.Description + "\";\n");
            sb.Append("}\n");

            if (genericCommand.OutputType == GenericCommandOutput.Count)
            {
                sb.Append("public override TransactionLogEntriesStatistics calculateStatistics()\n");
                sb.Append("{\n");
                sb.Append("GenericOccurenceEntriesStatistics stats = new GenericOccurenceEntriesStatistics();\n");
                sb.Append("stats.Count = MatchingEntries.Count;\n");
                sb.Append("return stats;\n");
                sb.Append("}\n");
            }
            else if (genericCommand.OutputType == GenericCommandOutput.List)
            {
                //
                sb.Append("public override TransactionLogEntriesStatistics calculateStatistics()\n");
                sb.Append("{\n");
                sb.Append("TransactionEntryListStatistics stats = new TransactionEntryListStatistics();\n");
                sb.Append("stats.MatchingEntries = MatchingEntries;\n");
                sb.Append("return stats;\n");
                sb.Append("}\n");
            }

            sb.Append("}\n");
            sb.Append("}\n");

            CompilerResults cr = icc.CompileAssemblyFromSource(cp, sb.ToString());
            if (cr.Errors.Count > 0)
            {
                throw new Exception(cr.Errors[0].ErrorText + Environment.NewLine + sb.ToString());
            }

            System.Reflection.Assembly a = cr.CompiledAssembly;
            object o = a.CreateInstance(classNamespace + "." + className);

            return (TransactionLogProcessingCommand)o;
        }
    }

    public class GenericCommand
    {
        public string Name { get; private set; }
        public string Description { get; private set; }

        public string Evaluation { get; set; }
        public GenericCommandOutput OutputType { get; set; }

        public GenericCommand(string name, string description)
        {
            this.Name = name;
            this.Description = description;
            this.OutputType = GenericCommandOutput.Count;
        }


        public void Save(string filename)
        {
            XmlDocument xmlDoc = new XmlDocument();
            XmlNode rootNode = xmlDoc.CreateElement("TransactionLogStatistics");
            xmlDoc.AppendChild(rootNode);

            XmlNode genericCommandNode = xmlDoc.CreateElement("GenericCommand");
            rootNode.AppendChild(genericCommandNode);
            XmlAttribute attr = xmlDoc.CreateAttribute("Name");
            attr.Value = this.Name;
            genericCommandNode.Attributes.Append(attr);
            attr = xmlDoc.CreateAttribute("Description");
            attr.Value = this.Description;
            genericCommandNode.Attributes.Append(attr);
            attr = xmlDoc.CreateAttribute("OutputType");
            attr.Value = this.OutputType.ToString();
            genericCommandNode.Attributes.Append(attr);


            genericCommandNode.AppendChild(xmlDoc.CreateTextNode(Evaluation));

            xmlDoc.Save(filename);
        }

        public static GenericCommand Load(string filename)
        {
            GenericCommand genericCommand = null;
            if (File.Exists(filename))
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(filename);

                XmlNode rootNode = xmlDoc.SelectSingleNode("TransactionLogStatistics/GenericCommand");
                string name = rootNode.Attributes["Name"].Value;
                string description = rootNode.Attributes["Description"].Value;
                string evaluation = rootNode.InnerText;

                GenericCommandOutput outputType = GenericCommandOutput.Count;
                if (rootNode.Attributes["ImageCommand"] != null)
                {
                    bool imageCommand = bool.Parse(rootNode.Attributes["ImageCommand"].Value);
                    if(imageCommand)
                        outputType = GenericCommandOutput.Image;

                }
                // this property overrides ImageCommand
                if (rootNode.Attributes["OutputType"] != null)
                {
                    switch (rootNode.Attributes["OutputType"].Value)
                    {
                        case "Count":
                            outputType = GenericCommandOutput.Count;
                            break;
                        case "Image":
                            outputType = GenericCommandOutput.Image;
                            break;
                        case "List":
                            outputType = GenericCommandOutput.List;
                            break;
                    }
                }
                


                genericCommand = new GenericCommand(name, description);
                genericCommand.Evaluation = evaluation;
                genericCommand.OutputType = outputType;
            }
            return genericCommand;
        }
    }

    public enum GenericCommandOutput
    {
        /// <summary>
        /// Output the number of transactions that match
        /// </summary>
        Count, 
        /// <summary>
        /// 
        /// </summary>
        Image, 
        /// <summary>
        /// List each transaction that matches
        /// </summary>
        List
    }
}
