/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VixLogAnalyzerEngine;

namespace TransactionLogStatisticsEngine.commands
{
    /// <summary>
    /// This command finds all transactions that have the missing patient ID error message from the PACSi.  This is a very specific test searching for a specific purpose
    /// 
    /// </summary>
    public class MissingPatientIdException : TransactionLogProcessingCommand
    {
        public MissingPatientIdException()
            : base()
        {
        }

        public override bool isEntryMatching(VixLogAnalyzerEngine.TransactionLogEntry entry)
        {
            return (entry.ErrorMessage.Contains("System.Web.Services.Protocols.SoapException: Invalid arguments, missing patientId    at Cps.PacsIntegrator.WebAdapters"));
        }

        public override string GetName()
        {
            return "VA Retrieve PACSi Study Missing Patient ID";
        }

        public override string GetDescription()
        {
            return "VA metadata retrieve of PACSi data missing the patient ID";
        }

        public override TransactionLogEntriesStatistics calculateStatistics()
        {
            GenericOccurenceEntriesStatistics stats = new GenericOccurenceEntriesStatistics();
            stats.Count = MatchingEntries.Count;
            return stats;
        }
    }
}
