/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using VixLogAnalyzerEngine;

namespace TransactionLogStatisticsEngine.commands
{
    public interface TransactionLogEntriesStatistics 
    {
        XmlNode toXml(XmlDocument xmlDoc, string sourceName);

        void updateXmlStats(XmlNode xmlNode);
    }

    public class TransactionEntryListStatistics : TransactionLogEntriesStatistics
    {
        public List<TransactionLogEntry> MatchingEntries { get; set; }

        public TransactionEntryListStatistics()
        {
            MatchingEntries = new List<TransactionLogEntry>();
        }

        public XmlNode toXml(XmlDocument xmlDoc, string sourceName)
        {
            XmlNode node = xmlDoc.CreateElement("Source");

            XmlAttribute attr = xmlDoc.CreateAttribute("name");
            attr.Value = sourceName;
            node.Attributes.Append(attr);

            foreach (TransactionLogEntry entry in MatchingEntries)
            {
                XmlNode entryNode = xmlDoc.CreateElement("Entry");
                node.AppendChild(entryNode);

                attr = xmlDoc.CreateAttribute("queryType");
                attr.Value = entry.QueryType;
                entryNode.Attributes.Append(attr);

                attr = xmlDoc.CreateAttribute("transactionId");
                attr.Value = entry.TransactionId;
                entryNode.Attributes.Append(attr);

                attr = xmlDoc.CreateAttribute("patientIcn");
                attr.Value = entry.PatientIcn;
                entryNode.Attributes.Append(attr);

                attr = xmlDoc.CreateAttribute("user");
                attr.Value = entry.User;
                entryNode.Attributes.Append(attr);

                attr = xmlDoc.CreateAttribute("requestingSite");
                attr.Value = entry.RequestingSite;
                entryNode.Attributes.Append(attr);
            }

            return node;
        }


        public void updateXmlStats(XmlNode xmlNode)
        {
            // xmlNode is the source node
            /*
            foreach (GenericFieldDescription key in Stats.Keys)
            {

                XmlNode statsNameNode = findOrCreateStatNameNode(xmlNode, key.FieldName);
                XmlNode valueNode = statsNameNode.SelectSingleNode("Value[@name='" + key.Value + "']");
                if (valueNode == null)
                {
                    statsNameNode.AppendChild(createStatValueNode(statsNameNode, key.Value, Stats[key]));
                }
                else
                {
                    long curValue = long.Parse(valueNode.Attributes["count"].Value);
                    valueNode.Attributes["count"].Value = (curValue + Stats[key]) + "";
                }
            }*/
        }
    }

    public class GenericFieldsStatistics : TransactionLogEntriesStatistics
    {
        public Dictionary<GenericFieldDescription, long> Stats;

        public GenericFieldsStatistics()
        {
            Stats = new Dictionary<GenericFieldDescription, long>(new GenericFieldDescriptionEqualityComparer());
        }

        public void AddValue(GenericFieldDescription name, long count)
        {
            if (string.IsNullOrEmpty(name.Value) && (!name.IncludeEmptyValues))
            {
                // omit empty value
            }
            else
            {
                if (Stats.ContainsKey(name))
                {
                    Stats[name] += count;
                }
                else
                {
                    Stats.Add(name, count);
                }
            }
        }

        public void IncrementValue(GenericFieldDescription name)
        {
            AddValue(name, 1);
        }

        public XmlNode toXml(XmlDocument xmlDoc, string sourceName)
        {
            XmlNode node = xmlDoc.CreateElement("Source");

            XmlAttribute attr = xmlDoc.CreateAttribute("name");
            attr.Value = sourceName;
            node.Attributes.Append(attr);

            foreach (GenericFieldDescription key in Stats.Keys)            
            {

                XmlNode statNode = findOrCreateStatNameNode(node, key.FieldName);
                statNode.AppendChild(createStatValueNode(statNode, key.Value, Stats[key]));
                //node.AppendChild(createStatsNode(xmlDoc, key, Stats[key]));
            }

            return node;
        }

        private XmlNode findOrCreateStatNameNode(XmlNode sourceNode, string statName)
        {
            XmlNode statNode = sourceNode.SelectSingleNode("Stat[@name='" + statName + "']");
            if (statNode == null)
            {
                statNode = sourceNode.OwnerDocument.CreateElement("Stat");
                XmlAttribute attr = sourceNode.OwnerDocument.CreateAttribute("name");
                attr.Value = statName;
                statNode.Attributes.Append(attr);
                sourceNode.AppendChild(statNode);
            }
            return statNode;
        }

        /*
        private XmlNode findStatValueNode(XmlNode statNode, string valueName)
        {
            return statNode.SelectSingleNode("Stat[@value='" + valueName + "']");
        }*/

        private XmlNode createStatValueNode(XmlNode statNode, string valueName, long count)
        {
            XmlNode statValueNode = statNode.OwnerDocument.CreateElement("Value");
            XmlAttribute attr = statNode.OwnerDocument.CreateAttribute("name");
            attr.Value = valueName;
            statValueNode.Attributes.Append(attr);
            attr = statNode.OwnerDocument.CreateAttribute("count");
            attr.Value = count + "";
            statValueNode.Attributes.Append(attr);
            return statValueNode;
        }

        /*
        private XmlNode createStatsNode(XmlDocument xmlDoc, GenericFieldDescription fieldDescription, long count)
        {

            XmlNode statNode = xmlDoc.CreateElement("Stat");
            XmlAttribute attr = xmlDoc.CreateAttribute("name");
            attr.Value = fieldDescription.FieldName;
            statNode.Attributes.Append(attr);
            attr = xmlDoc.CreateAttribute("value");
            attr.Value = fieldDescription.Value;
            statNode.Attributes.Append(attr);
            attr = xmlDoc.CreateAttribute("count");
            attr.Value = count + "";
            statNode.Attributes.Append(attr);
            return statNode;
        }*/

        public void updateXmlStats(XmlNode xmlNode)
        {
            // xmlNode is the source node
            foreach (GenericFieldDescription key in Stats.Keys)
            {

                XmlNode statsNameNode = findOrCreateStatNameNode(xmlNode, key.FieldName);
                XmlNode valueNode = statsNameNode.SelectSingleNode("Value[@name='" + key.Value + "']");
                if (valueNode == null)
                {
                    statsNameNode.AppendChild(createStatValueNode(statsNameNode, key.Value, Stats[key]));
                }
                else
                {
                    long curValue = long.Parse(valueNode.Attributes["count"].Value);
                    valueNode.Attributes["count"].Value = (curValue + Stats[key]) + "";
                }



                /*
                XmlNode curNode = xmlNode.SelectSingleNode("Stat[@name='" + key.FieldName + "' and @value='" + key.Value + "']");
                if (curNode == null)
                {
                    xmlNode.AppendChild(createStatsNode(xmlNode.OwnerDocument, key, Stats[key]));
                }
                else
                {
                    long curValue = long.Parse(curNode.Attributes["count"].Value);
                    curNode.Attributes["count"].Value = (curValue + Stats[key]) + "";
                }*/
            }
        }
    }

    public class GenericFieldDescription //: IComparer<GenericFieldDescription>
    {
        public string FieldName { get; private set ; }
        public string Value { get; private set; }
        public bool IncludeEmptyValues { get; set; }

        public GenericFieldDescription(string name, string value)
            : this(name, value, true)
        {
        }

        public GenericFieldDescription(string name, string value, bool includeEmptyValues)
        {
            this.FieldName = name;
            this.Value = value;
            this.IncludeEmptyValues = includeEmptyValues;
        }

        public override bool Equals(object obj)
        {
            if (obj is GenericFieldDescription)
            {
                GenericFieldDescription that = (GenericFieldDescription)obj;
                return (this.FieldName == that.FieldName) && (this.Value == that.Value);
            }
            return false;
        }

        public override int GetHashCode()
        {
            return FieldName.GetHashCode() ^ Value.GetHashCode();
        }
    }

    public class GenericFieldDescriptionEqualityComparer : IEqualityComparer<GenericFieldDescription>
    {
        public bool Equals(GenericFieldDescription x, GenericFieldDescription y)
        {
            return (x.FieldName == y.FieldName) && (x.Value == y.Value);
        }

        public int GetHashCode(GenericFieldDescription obj)
        {
            return obj.GetHashCode();
        }
    }

    public class GenericOccurenceEntriesStatistics : TransactionLogEntriesStatistics
    {

        public long Count { get; set; }

        public GenericOccurenceEntriesStatistics()
        {
            Count = 0L;
        }

        public XmlNode toXml(XmlDocument xmlDoc, string sourceName)
        {
            XmlNode node = xmlDoc.CreateElement("Source");

            XmlAttribute attr = xmlDoc.CreateAttribute("name");
            attr.Value = sourceName;
            node.Attributes.Append(attr);

            node.AppendChild(createCountNode(xmlDoc));

            return node;
        }

        private XmlNode createCountNode(XmlDocument xmlDoc)
        {
            XmlNode countNode = xmlDoc.CreateElement("Count");
            XmlAttribute attr = xmlDoc.CreateAttribute("value");
            attr.Value = Count + "";
            countNode.Attributes.Append(attr);
            return countNode;
        }

        public void updateXmlStats(XmlNode xmlNode)
        {
            XmlNode countNode = xmlNode.SelectSingleNode("Count");
            if (countNode == null)
            {
                xmlNode.AppendChild(createCountNode(xmlNode.OwnerDocument));
            }
            else
            {
                countNode.Attributes["value"].Value = long.Parse(countNode.Attributes["value"].Value) + Count + "";
            }

        }
    }

    public class TransactionLogImageEntriesStatistics : TransactionLogEntriesStatistics
    {
        public int ParentTransactions { get; set; }
        public ImageTypeStatistics ThumbnailRequests { get; set; }
        public ImageTypeStatistics ReferenceRequests { get; set; }
        public ImageTypeStatistics DiagnosticRequests { get; set; }
        public ImageTypeStatistics DiagnosticUncompressedRequests { get; set; }
        public ImageTypeStatistics ArtifactRequests { get; set; }

        public TransactionLogImageEntriesStatistics()
        {
            ThumbnailRequests = new ImageTypeStatistics();
            ReferenceRequests = new ImageTypeStatistics();
            DiagnosticRequests = new ImageTypeStatistics();
            ArtifactRequests = new ImageTypeStatistics();
            DiagnosticUncompressedRequests = new ImageTypeStatistics();
            ParentTransactions = 0;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("Transactions: " + ParentTransactions);
            sb.AppendLine("Thumbnail Requests: " + ThumbnailRequests);
            sb.AppendLine("Reference Requests: " + ReferenceRequests);
            sb.AppendLine("Diagnostic Requests: " + DiagnosticRequests);
            sb.AppendLine("Artifact Requests: " + ArtifactRequests);
            return sb.ToString();
        }

        public XmlNode toXml(XmlDocument xmlDoc, string sourceName)
        {
            XmlNode node = xmlDoc.CreateElement("Source");

            XmlAttribute attr = xmlDoc.CreateAttribute("name");
            attr.Value = sourceName;
            node.Attributes.Append(attr);

            node.AppendChild(ThumbnailRequests.toXml(xmlDoc, "Thumbnail"));
            node.AppendChild(ReferenceRequests.toXml(xmlDoc, "Reference"));
            node.AppendChild(DiagnosticRequests.toXml(xmlDoc, "Diagnostic"));
            node.AppendChild(DiagnosticUncompressedRequests.toXml(xmlDoc, "DiagnosticUncompressed"));
            node.AppendChild(ArtifactRequests.toXml(xmlDoc, "Artifacts"));

            return node;
        }

        public void updateXmlStats(XmlNode xmlNode)
        {
            ThumbnailRequests.updateXmlStats(xmlNode, "Thumbnail");
            ReferenceRequests.updateXmlStats(xmlNode, "Reference");
            DiagnosticRequests.updateXmlStats(xmlNode, "Diagnostic");
            DiagnosticUncompressedRequests.updateXmlStats(xmlNode, "DiagnosticUncompressed");
            ArtifactRequests.updateXmlStats(xmlNode, "Artifacts");
        }
    }

    public class ImageTypeStatistics
    {
        public long Requests { get; set; }
        public long Successful { get; set; }
        public long Failed { get; set; }
        public long BytesTransfered { get; set; }

        public ImageTypeStatistics()
        {
            Requests = Successful = Failed = 0;
            BytesTransfered = 0;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("Requests [" + Requests + "], Successful [" + Successful + "], Failed [" + Failed + "], BytesTransfered [" + BytesTransfered + "]");
            return sb.ToString();
        }

        public void updateXmlStats(XmlNode node, string requestType)
        {
            XmlNode requestNode = GetRequestTypeNode(node, requestType);// node.SelectSingleNode("Request[type='" + requestType + "']");
            if (requestNode == null)
            {
                node.AppendChild(toXml(node.OwnerDocument, requestType));
            }
            else
            {
                requestNode.Attributes["requests"].Value = long.Parse(requestNode.Attributes["requests"].Value) + Requests + "";
                requestNode.Attributes["successful"].Value = long.Parse(requestNode.Attributes["successful"].Value) + Successful + "";
                requestNode.Attributes["failed"].Value = long.Parse(requestNode.Attributes["failed"].Value) + Failed + "";
                requestNode.Attributes["bytesTransfered"].Value = long.Parse(requestNode.Attributes["bytesTransfered"].Value) + BytesTransfered + "";
            }
        }

        private XmlNode GetRequestTypeNode(XmlNode sourceNode, string requestType)
        {
            XmlNodeList requestNodes = sourceNode.SelectNodes("Request");
            foreach (XmlNode requestNode in requestNodes)
            {
                if (requestNode.Attributes["type"].Value == requestType)
                    return requestNode;
            }
            return null;
        }

        public XmlNode toXml(XmlDocument xmlDoc, string requestType)
        {
            XmlNode node = xmlDoc.CreateElement("Request");
            XmlAttribute attr = xmlDoc.CreateAttribute("type");
            attr.Value = requestType;
            node.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("requests");
            attr.Value = Requests.ToString();
            node.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("successful");
            attr.Value = Successful.ToString();
            node.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("failed");
            attr.Value = Failed.ToString();
            node.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("bytesTransfered");
            attr.Value = BytesTransfered.ToString();
            node.Attributes.Append(attr);

            return node;
        }
    }
}
