/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VISACommon;

namespace SiteService
{
    public class SiteServiceHelper
    {
        private static ExchangeSiteService.ImagingExchangeSiteService exchangeSiteService =
            new ExchangeSiteService.ImagingExchangeSiteService();

        public static List<VaSite> getVaSites(string siteServiceUrl, bool onlyIncludeVixSites)
        {
            exchangeSiteService.Url = siteServiceUrl;
            List<VaSite> result = new List<VaSite>();

            ExchangeSiteService.ImagingExchangeSiteTO[] sites = exchangeSiteService.getImagingExchangeSites();
            foreach (ExchangeSiteService.ImagingExchangeSiteTO site in sites)
            {
                VaSite r = translate(site);
                if (onlyIncludeVixSites)
                {
                    if (r.HasVix)
                        result.Add(r);
                }
                else
                {
                    result.Add(r);
                }
            }
            return result;
        }

        private static VaSite translate(ExchangeSiteService.ImagingExchangeSiteTO site)
        {
            return new VaSite(site.siteName, site.siteNumber, site.siteAbbr, 
                site.regionID, site.vistaServer, site.vistaPort, site.acceleratorServer, site.acceleratorPort);
        }

        public static List<VaSite> GetSites(string siteServiceUrl, string[] siteNumbers)
        {
            exchangeSiteService.Url = siteServiceUrl;
            List<VaSite> result = new List<VaSite>();

            string sitesQuery = "";
            string prefix = "";
            foreach (string siteNumber in siteNumbers)
            {
                sitesQuery = sitesQuery + prefix + siteNumber;
                prefix = "^";
            }

            ExchangeSiteService.ImagingExchangeSiteTO[] sites = exchangeSiteService.getSites(sitesQuery);
            foreach (ExchangeSiteService.ImagingExchangeSiteTO site in sites)
            {
                VaSite r = translate(site);
                result.Add(r);
            }
            return result;
        }

        public static VaSite GetSite(string siteServiceUrl, string siteNumber)
        {
            exchangeSiteService.Url = siteServiceUrl;
            ExchangeSiteService.ImagingExchangeSiteTO site = exchangeSiteService.getSite(siteNumber);

            VaSite result = translate(site);
            return result;
        }

    }
}
