/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;

namespace VISAHealthMonitor
{
    public class SelectedItemChangedBehavior
    {
        public static DependencyProperty SelectedItemChangedProperty =
            DependencyProperty.RegisterAttached("SelectedItemChanged",
            typeof(ICommand),
            typeof(SelectedItemChangedBehavior),
            new UIPropertyMetadata(SelectedItemChangedBehavior.SelectedItemChangedFired));

        public static void SetSelectedItemChanged(DependencyObject target, ICommand value)
        {
            target.SetValue(SelectedItemChangedBehavior.SelectedItemChangedProperty, value);
        }

        private static void SelectedItemChangedFired(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            TreeView control = target as TreeView;
            if (control != null)
            {
                if ((e.NewValue != null) && (e.OldValue == null))
                {
                    control.SelectedItemChanged += SelectedItemChanged;
                }
                else if ((e.NewValue == null) && (e.OldValue != null))
                {
                    control.SelectedItemChanged -= SelectedItemChanged;
                }
            }
        }

        private static void SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            TreeView control = sender as TreeView;
            ICommand command = (ICommand)control.GetValue(SelectedItemChangedBehavior.SelectedItemChangedProperty);
            object[] arguments = new object[] { control.SelectedItem};
            command.Execute(arguments);
        }
    }

    public class MouseDoubleClickBehavior
    {
        public static DependencyProperty MouseDoubleClickProperty =
            DependencyProperty.RegisterAttached("MouseDoubleClick",
            typeof(ICommand),
            typeof(MouseDoubleClickBehavior),
            new UIPropertyMetadata(MouseDoubleClickBehavior.MouseDoubleClickFired));

        public static void SetMouseDoubleClick(DependencyObject target, ICommand value)
        {
            target.SetValue(MouseDoubleClickBehavior.MouseDoubleClickProperty, value);
        }

        private static void MouseDoubleClickFired(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            Control control = target as Control;
            if (control != null)
            {
                if ((e.NewValue != null) && (e.OldValue == null))
                {
                    control.MouseDoubleClick += MouseDoubleClick;
                }
                else if ((e.NewValue == null) && (e.OldValue != null))
                {
                    control.MouseDoubleClick -= MouseDoubleClick;
                }
            }
        }

        private static void MouseDoubleClick(object sender, RoutedEventArgs e)
        {
            Control control = sender as Control;
            ICommand command = (ICommand)control.GetValue(MouseDoubleClickBehavior.MouseDoubleClickProperty);
            object[] arguments = new object[] { };
            command.Execute(arguments);
        }
    }
}
