/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using GalaSoft.MvvmLight;
using System.Collections.ObjectModel;
using VISACommon;
using SiteService;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Messaging;
using VISAHealthMonitorCommon.messages;
using VISAHealthMonitorCommon;
using System.Windows.Input;

namespace VISAHealthMonitor.Client.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class VisaSourceTreeViewModel : ViewModelBase
    {

        public ObservableCollection<VisaHealthSource> VisaSources { get; set; }
        public VisaSource SelectedTreeItem { get; set; }
        public RelayCommand TreeMouseDoubleClickCommand { get; set; }
        public RelayCommand<object> TreeSelectedItemChangedCommand { get; set; }
        private object selectedItem = null;


        /// <summary>
        /// Initializes a new instance of the VisaSourceTreeViewModel class.
        /// </summary>
        public VisaSourceTreeViewModel()
        {
            TreeMouseDoubleClickCommand = new RelayCommand(() => TreeMouseDoubleClick());
            TreeSelectedItemChangedCommand = new RelayCommand<object>((i) => TreeSelectedItemChanged(i));

            if (IsInDesignMode)
            {
                VisaSources = new ObservableCollection<VisaHealthSource>();
                // Code runs in Blend --> create design time data.
            }
            else
            {

                Messenger.Default.Register<ReloadSourcesMessage>(this, action => LoadVixSites(action));
                Messenger.Default.Register<VisaHealthUpdatedMessage>(this, action => ReceiveUpdatedVisaHealthMessage(action));
                //VisaSources = new ObservableCollection<VisaSource>(SiteServiceHelper.getVaSites(url, true));
                // Code runs "for real"
            }
        }

        private void LoadVixSites(ReloadSourcesMessage msg)
        {            
            /*
            string url = "http://siteserver.vista.domain.ext/VistaWebSvcs/ImagingExchangeSiteService.asmx";
            //url = "http://zzzzzzimmixcvix/VistaWebSvcs/ImagingExchangeSiteService.asmx";
            System.Collections.Generic.List<VaSite> sites = SiteServiceHelper.getVaSites(url, true);
            */
            VisaSources = new ObservableCollection<VisaHealthSource>();
            foreach (VaSite site in VixSourceHolder.getSingleton().VisaSources)
            {
                VisaHealth health = VisaHealthManager.GetVisaHealth(site);
                VisaSources.Add(new VisaHealthSource(site, health));
            }            
        }

        public void TreeSelectedItemChanged(object item)
        {
            this.selectedItem = item;
            VisaHealthSource source = getSelectedVisaHealthSource();
            if (source != null)
            {
                Messenger.Default.Send<DisplayVisaHealthMessage>(new DisplayVisaHealthMessage(source.VisaSource));
            }
        }

        private VisaHealthSource getSelectedVisaHealthSource()
        {
            if (this.selectedItem != null)
            {
                VisaHealthSource source = null;
                if (selectedItem.GetType().IsArray)
                {
                    object[] selectedItems = (object[])selectedItem;
                    source = (VisaHealthSource)selectedItems[0];
                }
                else
                {
                    source = (VisaHealthSource)this.selectedItem;
                }
                return source;
            }
            return null;
        }

        private void TreeMouseDoubleClick()
        {
            VisaHealthSource source = getSelectedVisaHealthSource();
            if (source != null)
            {
                Messenger.Default.Send<UpdateAndDisplayVisaHealthMessage>(new UpdateAndDisplayVisaHealthMessage(source.VisaSource));
            }
        }

        private void ReceiveUpdatedVisaHealthMessage(VisaHealthUpdatedMessage msg)
        {
            
            //VisaSources.
            //base.RaisePropertyChanged("VisaSources");
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}
