/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VISAChecksums;
using VISACommon;
using System.Xml;

namespace VISAHealthMonitorCommon.checksums
{
    public class VisaChecksumRetriever
    {
        public static List<FileDetails> DownloadChecksums(VisaSource visaSource, LibraryType libraryType)
        {
            List<FileDetails> result = new List<FileDetails>();

            string url = GetUrl(visaSource, libraryType);

            XmlDocument xmlDoc = IOUtilities.MakeXmlWebRequest(url);

            XmlNodeList propertyNodes = xmlDoc.SelectNodes("/VisaConfiguration/Property");
            foreach (XmlNode propertyNode in propertyNodes)
            {
                string name = propertyNode.Attributes["name"].Value;
                long size = long.Parse(propertyNode.Attributes["size"].Value);
                string checksum = null;
                if (propertyNode.Attributes["checksum"] != null)
                {
                    checksum = propertyNode.Attributes["checksum"].Value;
                }
                result.Add(new FileDetails(name, checksum, size));
            }
            
            return result;
        }

        /// <summary>
        /// http://server:port/VixServerHealthWebApp/secure/VisaConfigurationServlet?checksum=true
        /// </summary>
        /// <param name="visaSource"></param>
        /// <param name="method"></param>
        /// <returns></returns>
        private static string GetUrl(VisaSource visaSource, LibraryType libraryType)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("http://");
            sb.Append(visaSource.VisaHost);
            sb.Append(":");
            sb.Append(visaSource.VisaPort);
            sb.Append("/VixServerHealthWebApp/secure/VisaConfigurationServlet?checksum=true&configurationType=" + libraryType);
            return sb.ToString();
        }
    }
}
