/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Runtime.CompilerServices;

namespace VISAHealthMonitorCommon.wiki
{
    public class VixCurrentVersionHelper
    {

        private static string currentVersion = null;
        private static DateTime? cachedDate = null;

        /// <summary>
        /// This method caches the current version and will use that cached version if it is available and has not expired (value from yesterday).
        /// </summary>
        /// <param name="wikiConfiguration"></param>
        /// <param name="timeout"></param>
        /// <returns></returns>
        [MethodImpl(MethodImplOptions.Synchronized)]
        public static string GetCachedCurrentReleasedVersion(WikiConfiguration wikiConfiguration, int timeout)
        {
            if (currentVersion == null || IsExpired())
            {
                currentVersion = GetCurrentReleasedVixVersion(wikiConfiguration, timeout);
                cachedDate = DateTime.Now;
            }
            return currentVersion;
        }

        private static bool IsExpired()
        {
            if (cachedDate.HasValue)
            {
                DateTime dt = cachedDate.Value;
                bool isYesterday = DateTime.Today - dt.Date == TimeSpan.FromDays(1);
                return isYesterday;
            }
            return true;
        }

        /// <summary>
        /// Always gets the currently released version from the VIX Wiki page
        /// </summary>
        /// <param name="wikiConfiguration"></param>
        /// <param name="timeout"></param>
        /// <returns></returns>
        public static string GetCurrentReleasedVixVersion(WikiConfiguration wikiConfiguration, int timeout)
        {
            string vixCurrentVersionWikiPage = wikiConfiguration.WikiRootUrl + "?page=CurrentVIXVersion&&skin=raw";
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(vixCurrentVersionWikiPage);
            request.Timeout = timeout * 1000;
            HttpWebResponse response = (HttpWebResponse)request.GetResponse();
            StreamReader reader = new StreamReader(IOUtilities.getStreamFromResponse(response));

            string line = reader.ReadLine();
            return line;
        }
    }
}
