/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;

namespace VISAHealthMonitorCommonControls.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class InputDialogViewModel : ViewModelBase
    {
        public string Input { get; set; }
        public RelayCommand OKCommand { get; set; }
        public bool? DialogResult { get; set; }
        public RelayCommand WindowLoadedCommand { get; set; }

        /// <summary>
        /// Initializes a new instance of the InputDialogViewModel class.
        /// </summary>
        public InputDialogViewModel()
        {
            this.DialogResult = null;
            if (IsInDesignMode)
            {
                Input = "Test Input";
            }
            OKCommand = new RelayCommand(() => OK());
            WindowLoadedCommand = new RelayCommand(() => WindowLoaded());
            ////if (IsInDesignMode)
            ////{
            ////    // Code runs in Blend --> create design time data.
            ////}
            ////else
            ////{
            ////    // Code runs "for real": Connect to service, etc...
            ////}
        }

        public void OK()
        {
            this.DialogResult = true;
            RaisePropertyChanged("DialogResult");
        }

        private void WindowLoaded()
        {
            // necessary to reset this value each time the Window is loaded so that the value will change when Save or Close is clicked. The ViewModel is static so it persists after
            // the form is closed so the previous DialogResult value persists as well - it needs to be changed here so it can change later
            this.DialogResult = null;
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}
