/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using GalaSoft.MvvmLight;
using VISACommon;
using GalaSoft.MvvmLight.Command;
using System.Windows;
using VISAHealthMonitorCommon.jmx;
using System;
using System.Windows.Input;

namespace VISAHealthMonitorCommonControls.ViewModel
{
    public delegate void CloseJmxWindowDelegate();

    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class JmxViewModel : ViewModelBase
    {
        public VisaSource VisaSource { get;  set; }
        public event CloseJmxWindowDelegate OnCloseWindowEvent;
        public string Title { get; private set; }
        public string SiteName { get; private set; }
        public RelayCommand GetValueCommand { get; private set; }
        public RelayCommand CloseCommand { get; set; }
        public string StatusMessage { get; set; }
        public Visibility ErrorIconVisibility { get; set; }
        public string ObjectName { get; set; }
        public string Attribute { get; set; }
        public string Output { get; set; }
        public Cursor Cursor { get; set; }

        /// <summary>
        /// Initializes a new instance of the JmxViewModel class.
        /// </summary>
        public JmxViewModel()
        {
            if (IsInDesignMode)
            {
                // Code runs in Blend --> create design time data.
                Title = "JMX";
            }
            else
            {
                // Code runs "for real": Connect to service, etc...
            }
            GetValueCommand = new RelayCommand(() => GetValue());
            CloseCommand = new RelayCommand(() => CloseWindow());
            ClearStatusMessage();
        }

        private void GetValue()
        {
            try
            {
                this.Cursor = Cursors.Wait;
                Output = "";
                Output = JmxUtility.GetJmxBeanValue(VisaSource, ObjectName, Attribute);
            }
            catch (Exception ex)
            {
                SetStatusMessage(ex.Message, true);
            }
            finally
            {
                this.Cursor = Cursors.Arrow;
            }
        }

        private void CloseWindow()
        {
            if (OnCloseWindowEvent != null)
                OnCloseWindowEvent();
        }

        public void Initialize(VisaSource visaSource)
        {
            this.VisaSource = visaSource;
            VaSite site = (VaSite)VisaSource;
            this.SiteName = site.DisplayName;
            this.Title = site.DisplayName + "- JMX";            
        }

        private void ClearStatusMessage()
        {
            SetStatusMessage("", false);
        }

        private void SetStatusMessage(string msg, bool error)
        {
            if (error)
            {
                ErrorIconVisibility = Visibility.Visible;
            }
            else
            {
                ErrorIconVisibility = Visibility.Collapsed;
            }
            StatusMessage = msg;
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}
