/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Messaging;
using VISAHealthMonitorCommon.messages;
using System.Windows;

namespace VISAHealthMonitorCommonControls.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class StatusMessageViewModel : ViewModelBase
    {
        public string Message { get; set; }
        public string AsyncStatus { get; set; }
        public string IntervalTestStatus { get; set; }
        public Visibility ErrorIconVisibility { get; private set; }

        private bool isTestRunning = false;

        /// <summary>
        /// Initializes a new instance of the StatusMessageViewModel class.
        /// </summary>
        public StatusMessageViewModel()
        {
            if (IsInDesignMode)
            {
                // Code runs in Blend --> create design time data.
                Message = "Status Message";
                AsyncStatus = "AsyncStatus";
                IntervalTestStatus = "Interval Test Status";
            }
            else
            {
                Messenger.Default.Register<StatusMessage>(this, msg => ReceiveStatusMessage(msg));
                Messenger.Default.Register<AsyncHealthRefreshUpdateMessage>(this, msg => ReceieveAsyncHealthRefreshUpdateMessage(msg));
                Messenger.Default.Register<StartStopIntervalTestMessage>(this, msg => ReceiveStartStopIntervalTestMessage(msg));
                // Code runs "for real": Connect to service, etc...
            }
            ErrorIconVisibility = Visibility.Collapsed;
        }

        private void EvaluateTestRunning()
        {
            IntervalTestStatus = (isTestRunning == true ? "Running" : "Stopped");
        }

        private void ReceiveStartStopIntervalTestMessage(StartStopIntervalTestMessage msg)
        {
            isTestRunning = !isTestRunning;
            EvaluateTestRunning();
        }

        protected void ReceiveStatusMessage(StatusMessage msg)
        {
            Message = msg.Message;
            ErrorIconVisibility = (msg.Error == true ? Visibility.Visible : Visibility.Collapsed);
        }

        private void ReceieveAsyncHealthRefreshUpdateMessage(AsyncHealthRefreshUpdateMessage msg)
        {
            if (msg.IsComplete)
            {
                AsyncStatus = "Done";
            }
            else
            {
                AsyncStatus = msg.Completed + " of " + msg.Total;
            }
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}
