/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VISACommon;
using SiteService;
using GalaSoft.MvvmLight.Messaging;
using VISAHealthMonitorCommon.messages;
using System.Windows.Input;
using VixHealthMonitorCommon;
using VISAHealthMonitorCommon;

namespace VixHealthMonitor
{
    public class VixSourceHolder
    {
        private List<VisaSource> visaSources = new List<VisaSource>();

        public List<VisaSource> VisaSources
        {
            get { return visaSources; }
        }

        public VisaSource GetVisaSource(string id)
        {
            foreach (VisaSource visaSource in VisaSources)
            {
                if (visaSource.ID == id)
                    return visaSource;
            }
            return null;
        }

        private static VixSourceHolder singleton = null;
        public static VixSourceHolder getSingleton()
        {
            if (singleton == null)
            {
                singleton = new VixSourceHolder();
            }
            return singleton;
        }

        public VixSourceHolder()
        {
        }

        public void Refresh()
        {
            Messenger.Default.Send<CursorChangeMessage>(new CursorChangeMessage(Cursors.Wait));
            visaSources.Clear();
            // JMW 2/11/2013 - clear all sources to make sure get health from new site if sites updated
            VisaHealthManager.ClearAllSources();
            string url = "http://siteserver.vista.domain.ext/VistaWebSvcs/ImagingExchangeSiteService.asmx";

            VixHealthMonitorConfiguration configuration = VixHealthMonitorConfiguration.GetVixHealthMonitorConfiguration();
            url = configuration.SiteServiceUrl;

            try
            {
                System.Collections.Generic.List<VaSite> sites = SiteServiceHelper.getVaSites(url, true);
                
                foreach (VaSite testSite in configuration.TestSites)
                {
                    sites.Add(testSite);
                }
                sites.Sort(new VaSiteComparer());
                visaSources.AddRange(sites);

                Messenger.Default.Send<StatusMessage>(new StatusMessage("Loaded " + sites.Count + " sites from Site Service"));
            }
            catch (Exception ex)
            {
                Messenger.Default.Send<StatusMessage>(new StatusMessage("Error loading from site service, " + ex.Message, true));
            }
            
            Messenger.Default.Send<CursorChangeMessage>(new CursorChangeMessage(Cursors.Arrow));
            Messenger.Default.Send<ReloadSourcesMessage>(new ReloadSourcesMessage());
        }


    }
}
