/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VISAHealthMonitorCommon.formattedvalues;

namespace VixHealthMonitorCommon
{
    public class ThreadProcessingTime : IComparable<ThreadProcessingTime>
    {
        public string ThreadName { get; private set; }
        public FormattedTime ProcessingTime { get; private set; }


        public ThreadProcessingTime(string threadName, long processingTime)
        {
            this.ThreadName = threadName;
            this.ProcessingTime = new FormattedTime(processingTime, true);
        }

        public int CompareTo(ThreadProcessingTime that)
        {
            return this.ProcessingTime.Ticks.CompareTo(that.ProcessingTime.Ticks);
        }
    }

    public class ThreadProcessingTimeComparer : IComparer<ThreadProcessingTime>
    {

        private bool mbSortAscending;

        public ThreadProcessingTimeComparer(bool sortAscending)
        {
            this.mbSortAscending = sortAscending;
        }

        public int Compare(ThreadProcessingTime x, ThreadProcessingTime y)
        {
            int c = x.ProcessingTime.Ticks.CompareTo(y.ProcessingTime.Ticks);
            if (c == 0)
                return c;
            if (!mbSortAscending)
                return -1 * c;
            return c;
        }
    }
}
