/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VISACommon;

namespace VixHealthMonitorCommon.monitoredproperty
{
    public class MonitoredProperty
    {
        public VisaSource VisaSource { get; private set; }
        public string Name { get; private set; }
        public SortedList<DateTime, MonitoredPropertyHistory> MonitoredPropertyHistory { get; private set; }
//        public string Value { get; private set; }
//        public DateTime DateUpdated { get; private set; }

        public MonitoredProperty(VisaSource visaSource, string name)
        {
            this.Name = name;
            this.VisaSource = visaSource;
            MonitoredPropertyHistory = new SortedList<DateTime, MonitoredPropertyHistory>();
        }

        public MonitoredPropertyHistory MostRecentHistory
        {
            get
            {
                if (this.MonitoredPropertyHistory.Count > 0)
                {
                    // the newest one is at the end
                    return this.MonitoredPropertyHistory.Values[this.MonitoredPropertyHistory.Count - 1];
                }
                return null;
            }
        }

        public bool IsValueDifferentFromMostRecentValue(string value)
        {
            MonitoredPropertyHistory mostRecentValue = MostRecentHistory;
            if (mostRecentValue == null)
                return true;

            return (mostRecentValue.Value != value);
        }
    }

    public class MonitoredPropertyHistory
    {
        public string Value { get; private set; }
        public DateTime DateUpdated { get; private set; }

        public MonitoredPropertyHistory(string value, DateTime dateUpdated)
        {
            this.Value = value;
            this.DateUpdated = dateUpdated;
        }
    }

    public class MonitoredPropertyComparer : IComparer<MonitoredProperty>
    {

        public int Compare(MonitoredProperty x, MonitoredProperty y)
        {
            MonitoredPropertyHistory mphX = x.MostRecentHistory;
            MonitoredPropertyHistory mphY = y.MostRecentHistory;
            if (mphX == null)
            {
                if (mphY == null)
                    return 0;
                return -1;
            }
            else
            {
                if (mphY == null)
                    return 1;
                return mphX.DateUpdated.CompareTo(mphY.DateUpdated);
            }
        }
    }
}
