/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using VixAKCommon.gov.va.med.imaging.vix;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using System.Diagnostics;

namespace VixAK
{
    public partial class frmAddAction : Form
    {
        public frmAddAction()
        {
            InitializeComponent();
        }

        public delegate void AddAction(BaseAction action);

        public event AddAction OnAddActionEvent;
        private BaseAction curAction = null;

        private void frmAddAction_Load(object sender, EventArgs e)
        {

        }

        public void ShowAddAction()
        {
            /*
            cboAction.Items.Add(new ExchangeTestAction());
            cboAction.Items.Add(new ClearCacheAction());
            cboAction.Items.Add(new ClearTomcatLogsAction());
            cboAction.Items.Add(new ClinicalDisplayTestAction());
            cboAction.Items.Add(new FileSwapAction());
            cboAction.Items.Add(new ClinicalDisplayImageAction());
            cboAction.Items.Add(new ClinicalDisplayLogImageAccessAction());
            cboAction.Items.Add(new ExchangeSiteServiceAction());
            cboAction.Items.Add(new SiteServiceTestAction());
            cboAction.Items.Add(new ClinicalDisplayGetImageInformationAction());
            cboAction.Items.Add(new FileCopyAction());
            cboAction.Items.Add(new FolderCopyAction());
             */

            List<BaseAction> actions = VixAKCommon.gov.va.med.imaging.vix.VixAKCommonUtils.GetPublicActions();
            foreach (BaseAction action in actions)
            {
                cboAction.Items.Add(action);
            }

            cboAction.Sorted = true;

            // need to clone the baseaction because we will be setting the selected
            // number of patients and stuff, don't want to reduce the total possible patients
            // for another action, so we need to clone the action/create a new one
            // can't just create a new one without knowing what type it is...
            // maybe select the action in this form, then know what type it is, or just look 
            // at type of figure it out

            this.ShowDialog();
        }

        private void AddPropertiesToList(Dictionary<string, object> properties)
        {
            lstProperties.Items.Clear();
            foreach (string key in properties.Keys)
            {
                ListViewItem item = new ListViewItem();
                item.Text = key;

                object value = properties[key];
                item.Tag = value;
                item.SubItems.Add(value.ToString());
                lstProperties.Items.Add(item);
            }
            foreach (ColumnHeader header in lstProperties.Columns)
            {
                header.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
            }
        }

        private void cboAction_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboAction.SelectedItem is BaseAction)
            {
                BaseAction ba = (BaseAction)cboAction.SelectedItem;
                curAction = VixAKCommon.gov.va.med.imaging.vix.VixAKCommonUtils.CreateNewActionInstance(ba);
                AddPropertiesToList(curAction.NecessaryProperties);
                txtTestDescription.Text = "";
            }
        }

        private void lstProperties_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstProperties.SelectedItems.Count > 0)
            {
                ListViewItem item = lstProperties.SelectedItems[0];
                if (item.Tag is string)
                {
                    txtPropertyValue.Text = item.Tag.ToString();
                    txtPropertyValue.Visible = true;
                    chkPropertyValue.Visible = false;
                    btnUnselectAll.Visible = false;
                    btnSelectAll.Visible = false;
                }
                else if (item.Tag is ActionCustomOption)
                {
                    txtPropertyValue.Visible = false;
                    chkPropertyValue.Visible = true;
                    btnUnselectAll.Visible = true;
                    btnSelectAll.Visible = true;
                    chkPropertyValue.Items.Clear();
                    ActionCustomOption option = (ActionCustomOption)item.Tag;
                    foreach (object opt in option.Options)
                    {
                        chkPropertyValue.Items.Add(opt,true);
                    }
                }
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnAddAction_Click(object sender, EventArgs e)
        {
            Debug.Assert(curAction != null, "Current action cannot be null!");
            if (OnAddActionEvent != null)
            {
                if (cboAction.SelectedItem is BaseAction)
                {
                    //BaseAction ba = (BaseAction)cboAction.SelectedItem;
                    //BaseAction newAction = VixCommon.CreateNewActionInstance(ba);
                    foreach (ListViewItem item in lstProperties.Items)
                    {
                        if (curAction.NecessaryProperties.ContainsKey(item.Text))
                        {
                            curAction.NecessaryProperties[item.Text] = item.Tag;
                        }
                    }
                    curAction.TestDescription = txtTestDescription.Text;
                    OnAddActionEvent(curAction);
                    toolStripStatusLabel1.Text = "Added action [" + curAction.GetActionName() + "]";
                    ClearFields();
                }
            }
        }

        private void ClearFields()
        {
            chkPropertyValue.Visible = false;
            txtPropertyValue.Visible = true;
            btnUnselectAll.Visible = false;
            btnSelectAll.Visible = false;
            txtPropertyValue.Text = "";
            chkPropertyValue.Items.Clear();
            lstProperties.Items.Clear();
            cboAction.SelectedItem = null;
            curAction = null;
            txtTestDescription.Text = "";
        }

        private void btnSetProperty_Click(object sender, EventArgs e)
        {
            if (lstProperties.SelectedItems.Count > 0)
            {
                ListViewItem item = lstProperties.SelectedItems[0];
                if (item.Tag is string)
                {
                    item.Tag = txtPropertyValue.Text;
                    curAction.NecessaryProperties[item.Text] = txtPropertyValue.Text;
                    item.SubItems[1].Text = txtPropertyValue.Text;
                }
                else if (item.Tag is ActionCustomOption)
                {
                    ActionCustomOption option = (ActionCustomOption)item.Tag;
                    option.Options.Clear();
                    foreach (object obj in chkPropertyValue.CheckedItems)
                    {
                        option.Options.Add((BaseCustomOption)obj);
                    }
                    curAction.NecessaryProperties[item.Text] = option;
                    item.SubItems[1].Text = option.ToString();
                    
                }
            }
        }

        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            SetItemsChecked(true);
        }

        private void SetItemsChecked(bool check)
        {
            for (int i = 0; i < chkPropertyValue.Items.Count; i++)
            {
                chkPropertyValue.SetItemChecked(i, check);
            }
        }

        private void btnUnselectAll_Click(object sender, EventArgs e)
        {
            SetItemsChecked(false);
        }

        private void cxOptionsMenu_Opening(object sender, CancelEventArgs e)
        {
            bool enabled = false;
            if (lstProperties.SelectedItems.Count > 0)
            {
                if (lstProperties.SelectedItems[0].Tag is string)
                {
                    enabled = true;
                }
            }
            clearValueToolStripMenuItem.Enabled = enabled;
        }

        private void clearValueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (lstProperties.SelectedItems.Count > 0)
            {
                foreach (ListViewItem item in lstProperties.SelectedItems)
                {
                    if (item.Tag is string)
                    {
                        item.Tag = "";
                        curAction.NecessaryProperties[item.Text] = "";
                        item.SubItems[1].Text = "";
                    }
                }
            }
        }

        private void developerModeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < cboAction.Items.Count; i++)
            {
                BaseAction action = (BaseAction) cboAction.Items[i];
                if (action is FakeTestAction)
                {
                    return;
                }
            }
            cboAction.Items.Add(new FakeTestAction());
            SaveValuesAsDefaultToolStripMenuItem.Visible = true;
            toolStripSeparator1.Visible = true;
        }

        private void SaveValuesAsDefaultToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (curAction != null)
            {
                foreach (ListViewItem item in lstProperties.Items)
                {
                    if (curAction.NecessaryProperties.ContainsKey(item.Text))
                    {
                        if (item.Tag is string)
                        {
                            curAction.SetDefaultValue(item.Text, item.Tag.ToString());
                        }
                    }
                    curAction.SaveDefaultValues();
                }
            }
        }

        private void chkPropertyValue_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (lstProperties.SelectedItems.Count > 0)
            {
                ListViewItem item = lstProperties.SelectedItems[0];
                if (item.Tag is string)
                {
                    // do nothing - should not happen
                }
                else if (item.Tag is ActionCustomOption)
                {
                    ActionCustomOption option = (ActionCustomOption)item.Tag;

                    if (!option.MultipleSelect)
                    {
                        if (e.NewValue == CheckState.Checked)
                        {
                            for (int i = 0; i < chkPropertyValue.Items.Count; i++)
                            {
                                if (i != e.Index)
                                {
                                    chkPropertyValue.SetItemChecked(i, false);
                                }
                            }
                        }
                    }
                }
            }
        }

        private void pasteFromClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (lstProperties.SelectedItems.Count > 0)
            {
                foreach (ListViewItem item in lstProperties.SelectedItems)
                {
                    if (item.Tag is string)
                    {
                        item.Tag = Clipboard.GetText();
                        curAction.NecessaryProperties[item.Text] = Clipboard.GetText();
                        item.SubItems[1].Text = Clipboard.GetText();
                    }
                }
            }
        }

        private void lstProperties_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (lstProperties.SelectedItems.Count > 0)
            {
                ListViewItem item = lstProperties.SelectedItems[0];
                if (item.Tag is string)
                {
                    VixCommon.frmInputDialog inputDialog = new VixCommon.frmInputDialog();
                    if (inputDialog.ShowInputDialog("Set Value", "Set value for field '" + item.Text + "'", item.Tag.ToString()) == DialogResult.OK)
                    {
                        item.Tag = inputDialog.Value;
                        curAction.NecessaryProperties[item.Text] = inputDialog.Value;
                        item.SubItems[1].Text = inputDialog.Value;
                    }
                }
            }
        }
    }
}
