/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using VixAKCommon.gov.va.med.imaging.vix;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using System.Collections;
using log4net;
using VixAK.Properties;

namespace VixAK
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        protected ILog Logger()
        {
            return LogManager.GetLogger(this.GetType().Name);
        }

        private TreeNode testNode = new TreeNode("Test");
        private PictureBox kickerPicture = new PictureBox();
        private int actionCount;
        private int curActionNumber;
        private BaseAction curAction = null;
        private string mstrConfigFilename = null;

        private const string FileFilter = "XML Files (*.xml)|*.xml";
        
        private void frmMain_Load(object sender, EventArgs e)
        {
            kickerPicture.Visible = false;
            this.Controls.Add(kickerPicture);
            treActions.Nodes.Add(testNode);
            Logger().Info(LogMsg("VixAK version '" + Application.ProductVersion + "' started"));
            /*
            //System.Configuration.SettingsProperty prop = Settings.Default.Properties["frmMainPosition"];
            System.Configuration.SettingsPropertyValue value = Settings.Default.PropertyValues["frmMainPosition"];
            if (   Settings.Default.PropertyValues[ "frmMainPosition"] != null)
            {                
                Point location = Settings.Default.frmMainPosition;
                if (location.X < 0)
                    location.X = 0;
                if (location.Y < 0)
                    location.Y = 0;
                this.Location = location;
                
            }
            if (Settings.Default.PropertyValues["frmMainSize"] != null)
            {
                Size size = Settings.Default.frmMainSize;
                this.Size = size;
            }
            splitContainer1.SplitterDistance = Settings.Default.frmMainHorizontalBar;
             */ 
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            /*
            Settings.Default.frmMainPosition = this.Location;
            Settings.Default.frmMainSize = this.Size;
            Settings.Default.frmMainHorizontalBar = splitContainer1.SplitterDistance;
            Settings.Default.Save();
             */ 
        }

        private void btnAddAction_Click(object sender, EventArgs e)
        {
            frmAddAction addAction = new frmAddAction();
            addAction.OnAddActionEvent += OnAddAction;
            addAction.ShowAddAction();
        }

        private void OnAddAction(BaseAction action)
        {
            action.OnLogMessageEvent += LogMsg;
            action.OnSetProgressBarMaxEvent += SetMaxProgress;
            action.OnIncrementProgressEvent += IncrementProgress;
            action.OnActionCompleteEvent += ActionComplete;
            action.OnDoApplicationEventsEvent += DoApplicationEvents;
            AddActionToTree(action);
            mstrConfigFilename = null;
        }

        private void AddActionToTree(BaseAction action)
        {
            TreeNode node = new TreeNode();
            testNode.Nodes.Add(node);
            node.Text = action.GetActionName();
            node.Tag = action;
            SetNodeColor(node, action);
            foreach (string key in action.NecessaryProperties.Keys)
            {
                TreeNode propertyNode = new TreeNode();
                node.Nodes.Add(propertyNode);
                propertyNode.Text = key + " - " + action.NecessaryProperties[key].ToString();
            }
            testNode.Expand();
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void mnuFileOptionActions_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "";
            openFileDialog1.Filter = FileFilter;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                LoadActions(openFileDialog1.FileName);
                mstrConfigFilename = openFileDialog1.FileName;
            }
        }

        private void DoApplicationEvents()
        {
            if (this.InvokeRequired)
            {
                DoApplicationEventsDelegate d = new DoApplicationEventsDelegate(this.IncrementProgress);
                this.Invoke(d, new object[] { });
            }
            else
            {
                Application.DoEvents();
            }
        }

        private void IncrementProgress()
        {
            if (this.InvokeRequired)
            {
                IncrementProgressDelegate d = new IncrementProgressDelegate(this.IncrementProgress);
                this.Invoke(d, new object[] { });
            }
            else
            {
                if (toolStripProgressBar1.Value < toolStripProgressBar1.Maximum)
                {
                    toolStripProgressBar1.PerformStep();
                    toolStripProgressLabel.Text = toolStripProgressBar1.Value + " of " + toolStripProgressBar1.Maximum;
                    Application.DoEvents();
                }
            }
        }

        private void SetMaxProgress(int max)
        {
            if (InvokeRequired)
            {
                SetProgressBarMax d = new SetProgressBarMax(this.SetMaxProgress);
                this.Invoke(d, new object[] { max });
            }
            else
            {
                toolStripProgressBar1.Maximum = max;
                toolStripProgressBar1.Value = toolStripProgressBar1.Minimum;
                toolStripProgressLabel.Text = "";
                Application.DoEvents();
            }
        }

        private string LogMsg(string msg)
        {
            if (InvokeRequired)
            {
                LogMessageDelegate d = new LogMessageDelegate(this.LogMsg);
                this.Invoke(d, new object[] { msg });                
            }
            else
            {
                //toolStripStatusLabel1.Text = msg;
                txtOutput.AppendText(msg + Environment.NewLine);
                txtOutput.ScrollToCaret();
                Application.DoEvents();
            }
            return msg;
        }

        private void LoadActions(string filename)
        {
            testNode.Nodes.Clear();
            List<BaseAction> actions = VixAKCommon.gov.va.med.imaging.vix.VixAKCommonUtils.LoadActionsFromXml(filename);
            foreach (BaseAction action in actions)
            {
                OnAddAction(action);
            }
            toolStripStatusLabel1.Text = "Loaded actions from file '" + filename + "'";
            this.Text = "VixAK - " + filename;
        }

        private void mnuFileSaveActions_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Filter = FileFilter;
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                SaveActions(saveFileDialog1.FileName);
            }
        }

        private void SaveActions(string filename)
        {
            List<BaseAction> actions = new List<BaseAction>();
            foreach (TreeNode node in testNode.Nodes)
            {
                if (node.Tag is BaseAction)
                {
                    BaseAction ba = (BaseAction)node.Tag;
                    actions.Add(ba);
                }
            }
            VixAKCommon.gov.va.med.imaging.vix.VixAKCommonUtils.SaveActionsAsXml(filename, actions);
            toolStripStatusLabel1.Text = "Saved actions to file '" + filename + "'";
        }

        private void btnRunActions_Click(object sender, EventArgs e)
        {
            RunActions();
        }

        private List<BaseAction> runningActions = null;

        private void RunActions()
        {
            Cursor.Current = Cursors.WaitCursor;
            SetButtonsEnabled(false);
            if (mstrConfigFilename != null)
            {
                Logger().Info("Running tests loaded from config file [" + mstrConfigFilename + "]");
            }
            
            //try
            //{
                
                List<BaseAction> actions = new List<BaseAction>();
                foreach (TreeNode node in testNode.Nodes)
                {
                    if (node.Tag is BaseAction)
                    {
                        BaseAction ba = (BaseAction)node.Tag;
                        if (ba.Enabled)
                        {
                            actions.Add(ba);
                        }
                    }
                }
                if (ValidateActionProperties(actions))
                {
                    if (runningActions == null)
                        runningActions = new List<BaseAction>();
                    else
                        runningActions.Clear();
                    Logger().Info(LogMsg("Running [" + actions.Count + "] actions"));
                    runningActions.AddRange(actions);
                    actionCount = runningActions.Count;
                    curActionNumber = 0;
                    curAction = null;
                    RunNextAction();
                    /*
                    foreach (BaseAction action in actions)
                    {
                        if (!RunAction(action))
                        {
                            Logger().Error(LogMsg("Action did not complete correctly, stopping running actions"));
                            break;
                        }
                    }
                    Logger().Info(LogMsg("Actions completed"));
                    toolStripProgressBar1.Value = toolStripProgressBar1.Minimum;
                     */
                }
                else
                {
                    SetButtonsEnabled(true);
                }
            /*    
            }
            finally
            {
                SetButtonsEnabled(true);
                Cursor.Current = Cursors.Default;
            }
         */ 
        }

        private void ActionComplete(BaseAction action, ActionCompletedResult actionCompleteResult)
        {
            if (InvokeRequired)
            {
                ActionCompleteDelegate d = new ActionCompleteDelegate(this.ActionComplete);
                this.Invoke(d, new object[] { action, actionCompleteResult });  
            }
            else
            {
                if ((actionCompleteResult == ActionCompletedResult.success) || (actionCompleteResult == ActionCompletedResult.resultsFailed))
                    RunNextAction();
                else
                {
                    Logger().Error(LogMsg("The previous action failed to complete correctly, halting processing"));
                    toolStripProgressBar1.Value = toolStripProgressBar1.Minimum;
                    SetButtonsEnabled(true);
                    Cursor.Current = Cursors.Default;
                    toolStripCurActionCountLabel.Text = "";
                    curActionNumber = 0;
                    actionCount = 0;
                }
            }
        }

        private bool ValidateActionProperties(List<BaseAction> actions)
        {
            foreach (BaseAction action in actions)
            {
                if (!action.ValidateNecessaryProperties())
                {
                    Logger().Info(LogMsg("Action [" + action.GetActionName() + "] does not have valid properties set"));
                    return false;
                }
            }
            return true;
        }

        private void RunNextAction()
        {
            if ((runningActions != null) && (runningActions.Count > 0))
            {
                BaseAction action = runningActions[0];
                runningActions.Remove(action);
                RunAction(action);
            }
            else
            {
                Logger().Info(LogMsg("Actions completed"));
                toolStripProgressBar1.Value = toolStripProgressBar1.Minimum;
                SetButtonsEnabled(true);
                toolStripCurActionCountLabel.Text = "";
                curActionNumber = 0;
                actionCount = 0;
                Cursor.Current = Cursors.Default;
            }
        }

        private void RunAction(BaseAction action)
        {
            curAction = action;
            curActionNumber++;
            toolStripCurActionCountLabel.Text = curActionNumber + " of " + actionCount;
            toolStripStatusLabel1.Text = "Running action '" + action.GetActionName() + "'";
            action.runAction();
        }

        private void btnRemoveAction_Click(object sender, EventArgs e)
        {
            TreeNode node = treActions.SelectedNode;
            if (node != null)
            {
                if (node.Tag is BaseAction)
                {
                    treActions.Nodes.Remove(node);
                }
                else if ((node.Parent != null) && (node.Parent.Tag is BaseAction))
                {
                    treActions.Nodes.Remove(node.Parent);
                }
            }
        }

        private void btnMoveUp_Click(object sender, EventArgs e)
        {
            if (treActions.SelectedNode != null)
            {
                TreeNode node = treActions.SelectedNode;
                if (node.Tag is BaseAction)
                {
                    if (node.PrevNode != null)
                    {
                        int prevIndex = node.PrevNode.Index;
                        testNode.Nodes.Remove(node);
                        testNode.Nodes.Insert(prevIndex, node);
                    }
                }
            }
        }

        private void btnMoveDown_Click(object sender, EventArgs e)
        {
            if (treActions.SelectedNode != null)
            {
                TreeNode node = treActions.SelectedNode;
                if (node.Tag is BaseAction)
                {
                    if (node.NextNode != null)
                    {
                        int nextIndex = node.NextNode.Index;
                        testNode.Nodes.Remove(node);
                        testNode.Nodes.Insert(nextIndex, node);
                    }
                }
            }
        }

        private void treActions_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            treActions.SelectedNode = e.Node;
        }

        private void cxActionTree_Opening(object sender, CancelEventArgs e)
        {
            if (treActions.SelectedNode != null)
            {
                if (treActions.SelectedNode.Tag is BaseAction)
                {
                    cxActionTreeEnabled.Enabled = true ;
                    BaseAction action = (BaseAction)treActions.SelectedNode.Tag;
                    cxActionTreeEnabled.Checked = action.Enabled;
                }
                else
                {
                    cxActionTreeEnabled.Enabled = false;
                }
            }
            else
            {
                cxActionTreeEnabled.Enabled = false;
            }
        }

        private void cxActionTreeEnabled_Click(object sender, EventArgs e)
        {
            if (treActions.SelectedNode != null)
            {
                if (treActions.SelectedNode.Tag is BaseAction)
                {
                    BaseAction action = (BaseAction)treActions.SelectedNode.Tag;
                    action.Enabled = !action.Enabled;
                    SetNodeColor(treActions.SelectedNode, action);
                }                
            }
        }

        private void SetNodeColor(TreeNode node, BaseAction action)
        {
            Color selectedColor = Color.Black;
            if (!action.Enabled)
            {
                selectedColor= Color.Gray;
            }
            node.ForeColor = selectedColor;
            foreach (TreeNode childNode in node.Nodes)
            {
                childNode.ForeColor = selectedColor;
            }
        }

        private void SetButtonsEnabled(bool enabled)
        {
            btnAddAction.Enabled = enabled;
            btnRemoveAction.Enabled = enabled;
            btnRunActions.Enabled = enabled;
            btnMoveDown.Enabled = enabled;
            btnMoveUp.Enabled = enabled;
            btnStopActions.Enabled = (!enabled);
        }

        private void kickerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (kickerPicture.Visible)
            {
                kickerPicture.Visible = false;
            }
            else
            {
                //string filename = VixCommon.getApplicationDir(Application.ExecutablePath) + "\\images\\ak.PNG";
                string filename = VixAKCommon.gov.va.med.imaging.vix.VixAKCommonUtils.getApplicationDir(Application.ExecutablePath) + "\\images\\ak_animated.gif";
                Image img = Bitmap.FromFile(filename);                
                kickerPicture.Image = img;
                kickerPicture.Show();
                kickerPicture.BringToFront();
                kickerPicture.Left = 0;
                kickerPicture.Top = 0;
                kickerPicture.Width = this.Width;
                kickerPicture.Height = this.Height;
                kickerPicture.SizeMode = PictureBoxSizeMode.StretchImage;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                VixAKCommon.gov.va.med.imaging.vix.tester.actions.CompareVixResultsWithLogAction compareAction = new CompareVixResultsWithLogAction();
                compareAction.OnLogMessageEvent += LogMsg;
                //compareAction.VixTestResultFilename = @"C:\vixtests\va\2008-12-23_cached- not-threaded,1 repeats(1)\testResults.csv";
                compareAction.VixTestResultFilename = @"C:\vixtests\va\2008-12-23_cached- not-threaded,1 repeats\testResults.csv";
                compareAction.DownloadDate = new DateTime(2008, 12, 23);
                compareAction.ResultsCompareOutputFilename = @"C:\temp\compare.csv";
                compareAction.VixServer = "";
                compareAction.runAction();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }

        }

        private void btnStopActions_Click(object sender, EventArgs e)
        {
            Logger().Info(LogMsg("User pressed the Stop Actions button"));
            if (curAction != null)
            {
                curAction.stopAction();
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("VistA Imaging Exchange Application Kruncher (ViX AK) version ");
            sb.Append(Application.ProductVersion);
            //sb.AppendLine(Application.CompanyName);
            MessageBox.Show(sb.ToString(), "About ViXAK", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            System.IO.StreamReader reader = null;
            try
            {
                //reader = new System.IO.StreamReader(@"C:\VixCache\va-image-region\660\GY2TSMA\GY2TSMI-THUMBNAIL-imagejpeg.jpg");
                //reader = new System.IO.StreamReader(@"C:\Documents and Settings\zzzzzzwerfej\Desktop\GY2TSMI-REFERENCE-applicationdicomimagej2k.dcm");
                //reader = new System.IO.StreamReader(@"C:\VixCache\va-image-region\660\GY2TSMA\GY2TSMI-REFERENCE-applicationdicomimagej2k");
                reader = new System.IO.StreamReader(@"C:\Documents and Settings\zzzzzzwerfej\Desktop\GY2TSMI-REFERENCE-applicationdicomimagej2k_clu5.dcm");
                byte[] buffer = VixAKCommon.gov.va.med.imaging.vix.VixAKCommonUtils.readStreamIntoBuffer(reader.BaseStream);
                string checksum = VixAKCommon.gov.va.med.imaging.vix.VixAKCommonUtils.CalculateAdler32(buffer);
                MessageBox.Show(checksum);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                if (reader != null)
                    reader.Close();
            }
        }
    }
}
