/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixAKCommon.gov.va.med.imaging
{
    public class DocumentSet : Study, IComparable<DocumentSet>
    {
        private string mstrHomeCommunityId;
        private string mstrRepositoryId;

        public string HomeCommunityId
        {
            get { return mstrHomeCommunityId; }
            set { mstrHomeCommunityId = value; }
        } 

        public string RepositoryId
        {
            get { return mstrRepositoryId; }
            set { mstrRepositoryId = value; }
        } 


        public DocumentSet(string homeCommunityId, string repositoryId, string documentId) : base(documentId)
        {
            mstrHomeCommunityId = homeCommunityId;
            mstrRepositoryId = repositoryId;
        }

        public DocumentSet()
        {
        }

        public override bool Equals(object obj)
        {
            if (obj is DocumentSet)
            {
                DocumentSet that = (DocumentSet)obj;
                return (this.mstrRepositoryId == that.mstrRepositoryId) &&
                    (this.mstrHomeCommunityId == that.mstrHomeCommunityId) &&
                    (this.StudyId == that.StudyId);
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// This sort only puts the data into a consistent order, not a logical order - this is not sorted by date.
        /// The purpose is so if two lists of DocumentSets are compared they can be found equivalent even if the order
        /// was originally different
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public int CompareTo(DocumentSet other)
        {
            int i = this.mstrHomeCommunityId.CompareTo(other.mstrHomeCommunityId);
            if (i != 0)
                return i;
            i = this.mstrRepositoryId.CompareTo(other.mstrRepositoryId);
            if (i != 0)
                return i;
            return this.StudyId.CompareTo(other.StudyId);

        }
    }

    /// <summary>
    /// This sort only puts the data into a consistent order, not a logical order - this is not sorted by date.
    /// The purpose is so if two lists of DocumentSets are compared they can be found equivalent even if the order
    /// was originally different
    /// </summary>
    public class DocumentSetComparer : IComparer<Study>
    {
        
        public int Compare(Study studyX, Study studyY)
        {
            DocumentSet x = null;
            DocumentSet y = null;
            if (studyX is DocumentSet)
            {
                x = (DocumentSet)studyX;
            }
            if (studyY is DocumentSet)
            {
                y = (DocumentSet)studyY;
            }
            if (x == null)
                return -1;
            if (y == null)
                return 1;
            int i = x.HomeCommunityId.CompareTo(y.HomeCommunityId);
            if (i != 0)
                return i;
            i = x.RepositoryId.CompareTo(y.RepositoryId);
            if (i != 0)
                return i;
            return x.StudyId.CompareTo(y.StudyId);
        }

        
    }
}
