/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixAKCommon.gov.va.med.imaging
{
    public class Exam : SavableObject
    {
        private string mstrSiteNumber;
        private string mstrPatientIcn;
        private string mstrExamId;

        private string mstrRadiologyReport;

        public string RadiologyReport
        {
            get { return mstrRadiologyReport; }
            set { mstrRadiologyReport = value; }
        }
        private string mstrRequisitionReport;

        public string RequisitionReport
        {
            get { return mstrRequisitionReport; }
            set { mstrRequisitionReport = value; }
        }

        private List<ExamImage> images = new List<ExamImage>();

        public List<ExamImage> Images
        {
            get { return images; }
            set { images = value; }
        } 

        public string SiteNumber
        {
            get { return mstrSiteNumber; }
            set { mstrSiteNumber = value; }
        }        

        public string PatientIcn
        {
            get { return mstrPatientIcn; }
            set { mstrPatientIcn = value; }
        }        

        public string ExamId
        {
            get { return mstrExamId; }
            set { mstrExamId = value; }
        }

        public Exam()
        {
            mstrSiteNumber = null;
            mstrPatientIcn = null;
            mstrExamId = null;
            mstrRequisitionReport = null;
            mstrRadiologyReport = null;
        }

        public Exam(string siteNumber, string patientIcn, string examId)
        {
            mstrSiteNumber = siteNumber;
            mstrPatientIcn = patientIcn;
            mstrExamId = examId;
            mstrRequisitionReport = null;
            mstrRadiologyReport = null;
        }

        public override bool Equals(object obj)
        {
            if (obj is Exam)
            {
                Exam that = (Exam)obj;
                if (this.mstrExamId != that.mstrExamId)
                    return false;
                if (this.mstrPatientIcn != that.mstrPatientIcn)
                    return false;
                /*
                 // not comparing report - it includes a time stamp so it never equals properly
                if (this.mstrRadiologyReport != that.mstrRadiologyReport)
                    return false;
                if (this.mstrRequisitionReport != that.mstrRequisitionReport)
                    return false;
                 * */
                if (this.mstrSiteNumber != that.mstrSiteNumber)
                    return false;
                if (this.images.Count != that.images.Count)
                    return false;
                for (int i = 0; i < this.images.Count; i++)
                {
                    ExamImage ei = this.images[i];
                    ExamImage t = that.images[i];
                    if (!ei.Equals(t))
                    {
                        return false;
                    }
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class ExamSavableList : SavableObject
    {
        private List<Exam> mExams = new List<Exam>();

        public List<Exam> Exams
        {
            get { return mExams; }
            set { mExams = value; }
        }

        public override bool Equals(object obj)
        {
            if (obj is ExamSavableList)
            {
                ExamSavableList that = (ExamSavableList)obj;
                if (this.mExams.Count != that.mExams.Count)
                    return false;
                for (int i = 0; i < this.mExams.Count; i++)
                {
                    Exam e = this.mExams[i];
                    Exam t = that.mExams[i];
                    if (!e.Equals(t))
                        return false;
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        public ExamSavableList()
        {
        }

        public ExamSavableList(List<Exam> exams)
        {
            mExams.AddRange(exams);
        }
    }
}
