/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using System.Xml;

namespace VixAKCommon.gov.va.med.imaging
{
    public class ImagingPatient : BaseCustomOption
    {
        private readonly string mstrPatientIcn;
        private string mstrPatientName;
        private List<string> siteNumbers = new List<string>();
        private bool mbActive;

        public bool Active
        {
            get { return mbActive; }
            set { mbActive = value; }
        }

        public List<string> SiteNumbers
        {
            get { return siteNumbers; }
            set { siteNumbers = value; }
        }

        public string PatientIcn
        {
            get { return mstrPatientIcn; }
        }

        public string PatientName
        {
            get { return mstrPatientName; }
            set { mstrPatientName = value; }
        }

        public ImagingPatient(string patientIcn, string patientName, string [] siteNumbers)
        {
            mstrPatientIcn = patientIcn;
            mstrPatientName = patientName;
            if (siteNumbers != null)
            {
                this.siteNumbers.AddRange(siteNumbers);
            }
            mbActive = true;
        }

        public ImagingPatient(string patientIcn, string patientName) : this(patientIcn, patientName, null)
        {
            
        }

        public ImagingPatient(string patientIcn)
            : this(patientIcn, "")
        {
        }

        public override string ToString()
        {
            if (mstrPatientName.Length > 0)
                return mstrPatientName;
            return mstrPatientIcn;
        }

        public string ToString(bool verbose)
        {
            if (verbose)
            {
                if (mstrPatientName.Length > 0)
                {
                    return mstrPatientName + " (" + mstrPatientIcn + ")";
                }
                return mstrPatientIcn;
            }
            return ToString();
        }

        public static string IMAGING_PATIENT_NODE_NAME = "ImagingPatient";
        public static string IMAGING_PATIENT_SITES_NODE_NAME = "Sites";
        public static string IMAGING_PATIENT_SITE_NODE_NAME = "Site";

        public override System.Xml.XmlNode toXml(XmlDocument xmlDoc)
        {
            XmlNode node = xmlDoc.CreateElement(IMAGING_PATIENT_NODE_NAME);

            XmlAttribute attr = xmlDoc.CreateAttribute("PatientName");
            attr.Value = mstrPatientName;
            node.Attributes.Append(attr);
            attr = xmlDoc.CreateAttribute("PatientICN");
            attr.Value = mstrPatientIcn;
            node.Attributes.Append(attr);
            attr = xmlDoc.CreateAttribute("active");
            attr.Value = mbActive.ToString();
            node.Attributes.Append(attr);

            XmlNode sitesNode = xmlDoc.CreateElement(IMAGING_PATIENT_SITES_NODE_NAME);
            node.AppendChild(sitesNode);

            foreach (string siteNumber in siteNumbers)
            {
                XmlNode siteNode = xmlDoc.CreateElement(IMAGING_PATIENT_SITE_NODE_NAME);
                sitesNode.AppendChild(siteNode);
                attr = xmlDoc.CreateAttribute("SiteNumber");
                attr.Value = siteNumber;
                siteNode.Attributes.Append(attr);
            }

            return node;
        }
    }
}
