/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixAKCommon.gov.va.med.imaging
{
    public class Instance : SavableObject
    {
        private string mstrInstanceId;
        private string mstrGloballyUniqueId;       
        private int mintImageType;
        private string mstrDicomUid;

        public string GloballyUniqueId
        {
            get 
            {
                if (string.IsNullOrEmpty(mstrGloballyUniqueId))
                    return mstrInstanceId;
                return mstrGloballyUniqueId;
            }
            set
            {
                mstrGloballyUniqueId = value;
            }
        } 

        public string DicomUid
        {
            get { return mstrDicomUid; }
            set { mstrDicomUid = value; }
        }

        public string InstanceId
        {
            get { return mstrInstanceId; }
            set { mstrInstanceId = value; }
        }

        public int ImageType
        {
            get { return mintImageType; }
            set { mintImageType = value; }
        }

        public Instance()
        {
        }

        public Instance(string id) : this(id, null)
        {
            
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="id"></param>
        /// <param name="globallyUniqueId">This is a globally unique identifier used when the ID field is not 
        /// globally unique (such as in XCA where it must be paired with repository ID and home community ID
        /// </param>
        public Instance(string id, string globallyUniqueId)
        {
            mstrInstanceId = id;
            mstrGloballyUniqueId = globallyUniqueId;
        }

        public override bool Equals(object obj)
        {
            if (obj is Instance)
            {
                Instance that = (Instance)obj;
                return (this.mstrInstanceId == that.mstrInstanceId) &&
                    //(this.mstrGloballyUniqueId == that.mstrGloballyUniqueId) &&
                    // compare the GloballyUniqueId from the property to ensure the correct comparison
                    (this.GloballyUniqueId == that.GloballyUniqueId) &&
                    (this.mstrDicomUid == that.mstrDicomUid) &&
                    (this.mintImageType == that.mintImageType);
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}
