/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixAKCommon.gov.va.med.imaging
{
    public class Site : SavableObject
    {
        private string mstrSiteNumber;
        private string mstrSiteName;
        private string mstrRegionNumber; 
        private string mstrSiteAbbr;
        private string mstrVistaServer;
        private int mintVistaPort;
        private string mstrVixServer;
        private int mintVixPort;

        #region "Site Properties"

        public string SiteNumber
        {
            get { return mstrSiteNumber; }
            set { mstrSiteNumber = value; }
        }

        public string SiteName
        {
            get { return mstrSiteName; }
            set { mstrSiteName = value; }
        }

        public string RegionNumber
        {
            get { return mstrRegionNumber; }
            set { mstrRegionNumber = value; }
        }

        public string SiteAbbr
        {
            get { return mstrSiteAbbr; }
            set { mstrSiteAbbr = value; }
        }
        
        public string VistaServer
        {
            get { return mstrVistaServer; }
            set { mstrVistaServer = value; }
        }

        public int VistaPort
        {
            get { return mintVistaPort; }
            set { mintVistaPort = value; }
        }

        public string VixServer
        {
            get { return mstrVixServer; }
            set { mstrVixServer = value; }
        }

        public int VixPort
        {
            get { return mintVixPort; }
            set { mintVixPort = value; }
        }

        #endregion

        public Site()
        {
        }


        public Site(string siteNumber, string siteName, string regionNumber, string siteAbbr, 
            string vistaServer, int vistaPort, string vixServer, int vixPort)
        {
            mstrSiteAbbr = siteAbbr;
            mstrSiteName = siteName;
            mstrSiteNumber = siteNumber;
            mstrVistaServer = vistaServer;
            mstrVixServer = vixServer;
            mstrRegionNumber = regionNumber;
            mintVistaPort = vistaPort;
            mintVixPort = vixPort;
        }

        public override string ToString()
        {
            return ToString(false);
        }

        public string ToString(bool verbose)
        {
            if (verbose)
            {
                StringBuilder sb = new StringBuilder();
                sb.Append(mstrSiteName);
                sb.Append(" [");
                sb.Append(mstrSiteNumber);
                sb.Append("] Region [");
                sb.Append(mstrRegionNumber);
                sb.Append("] Vista (");
                sb.Append(mstrVistaServer);
                sb.Append(",");
                sb.Append(mintVistaPort);
                sb.Append(") Vix (");
                sb.Append(mstrVixServer);
                sb.Append(",");
                sb.Append(mintVixPort);
                sb.Append(")");
                return sb.ToString();
            }
            else
            {
                return mstrSiteName;
            }
        }

        public override bool Equals(object obj)
        {
            if (obj is Site)
            {
                Site that = (Site)obj;
                return (mstrRegionNumber == that.mstrRegionNumber) &&
                    (mstrSiteAbbr == that.mstrSiteAbbr) &&
                    (mstrSiteName == that.mstrSiteName) &&
                    (mstrSiteNumber == that.mstrSiteNumber) &&
                    (mstrVistaServer == that.mstrVistaServer) &&
                    (mstrVixServer == that.mstrVixServer) &&
                    (mintVistaPort == that.mintVistaPort) &&
                    (mintVixPort == that.mintVixPort);
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class SiteArray : SavableObject
    {
        private List<Site> mSites = new List<Site>();

        public List<Site> Sites
        {
            get { return mSites; }
            set { mSites = value; }
        } 

        public SiteArray(List<Site> sites)
        {
            mSites.AddRange(sites);
        }

        public SiteArray()
        {
        }

        public override bool Equals(object obj)
        {
            if (obj is SiteArray)
            {
                SiteArray that = (SiteArray)obj;
                if (that.Sites.Count != this.Sites.Count)
                    return false;
                for (int i = 0; i < mSites.Count; i++)
                {
                    Site s = mSites[i];
                    Site t = that.Sites[i];
                    if (!s.Equals(t))
                        return false;
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}
