/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixAKCommon.gov.va.med.imaging
{
    [System.Xml.Serialization.XmlInclude(typeof(DocumentSet))]
    public class Study : SavableObject
    {

        private string mstrStudyId;
        private string mstrStudyDescription;
        private List<Series> series = new List<Series>();

        public List<Series> Series
        {
            get { return series; }
            set { series = value; }
        }

        public string StudyDescription
        {
            get { return mstrStudyDescription; }
            set { mstrStudyDescription = value; }
        }

        public string StudyId
        {
            get { return mstrStudyId; }
            set { mstrStudyId = value; }
        }

        public Study(string id)
        {
            mstrStudyId = id;
        }

        public Study()
        {
        }

        public override bool Equals(object obj)
        {
            if (obj is Study)
            {
                Study that = (Study)obj;
                if (this.mstrStudyId != that.mstrStudyId)
                    return false;
                if (this.series.Count != that.series.Count)
                    return false;
                for (int i = 0; i < this.series.Count; i++)
                {
                    Series s = this.series[i];
                    Series t = that.series[i];
                    if (!s.Equals(t))
                        return false;
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class StudySavableList : SavableObject
    {
        private List<Study> studies = new List<Study>();

        public List<Study> Studies
        {
            get { return studies; }
            set { studies = value; }
        }

        public StudySavableList()
        {

        }

        public StudySavableList(List<Study> studies)
        {
            this.studies.AddRange(studies);
        }

        public override bool Equals(object obj)
        {
            if (obj is StudySavableList)
            {
                StudySavableList that = (StudySavableList)obj;
                if (this.studies.Count != that.studies.Count)
                    return false;
                for (int i = 0; i < studies.Count; i++)
                {
                    Study s = this.studies[i];
                    Study t = that.studies[i];
                    if (!s.Equals(t))
                        return false;
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class ClinicalDisplayStudyList : StudySavableList
    {
        private string mstrErrorMessage;
        private string mstrErrorCode;

        public string ErrorMessage
        {
            get { return mstrErrorMessage; }
            set { mstrErrorMessage = value; }
        }        

        public string ErrorCode
        {
            get { return mstrErrorCode; }
            set { mstrErrorCode = value; }
        }

        public ClinicalDisplayStudyList()
        {
            mstrErrorCode = null;
            mstrErrorMessage = null;
        }

        public ClinicalDisplayStudyList(List<Study> studies) : base(studies)
        {
            mstrErrorCode = null;
            mstrErrorMessage = null;
        }

        public ClinicalDisplayStudyList(string errorCode, string errorMessage)
        {
            mstrErrorMessage = errorMessage;
            mstrErrorCode = errorCode;
        }

        public override bool Equals(object obj)
        {
            if (obj is ClinicalDisplayStudyList)
            {
                ClinicalDisplayStudyList that = (ClinicalDisplayStudyList)obj;
                if (this.mstrErrorCode != that.mstrErrorCode)
                    return false;
                if (this.mstrErrorMessage != that.mstrErrorMessage)
                    return false;
                return base.Equals(obj);
            }
            return false;

            //return base.Equals(obj);
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}
