/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixAKCommon.gov.va.med.imaging.vix
{
    public class ImageFormat
    {
        private readonly string mstrMimeType;
        private readonly string mstrEnclosedMimeType;

        public string MimeType
        {
            get { return mstrMimeType; }
        }

        public string EnclosedMimeType
        {
            get { return mstrEnclosedMimeType; }
        }

        public ImageFormat(string mimeType, string enclosedMimeType)
        {
            this.mstrMimeType = mimeType;
            this.mstrEnclosedMimeType = enclosedMimeType;
        }

        public ImageFormat(string mimeType)
            : this(mimeType, "")
        {
        }

        public override string ToString()
        {
            return mstrMimeType;
        }

        public string ToString(bool includeEnclosedType)
        {
            if ((includeEnclosedType) && (mstrEnclosedMimeType.Length > 0))
            {
                return this.ToString() + "/" + mstrEnclosedMimeType;
            }
            else
            {
                return this.ToString();
            }
        }

        public override bool Equals(object obj)
        {
            if (obj is ImageQuality)
            {
                ImageFormat that = (ImageFormat)obj;
                return (this.mstrMimeType == that.mstrMimeType) && (this.mstrEnclosedMimeType == that.mstrEnclosedMimeType);
            }
            else
            {
                return base.Equals(obj);
            }
        }

        public override int GetHashCode()
        {
            return base.GetHashCode() ^ mstrMimeType.GetHashCode() ^ mstrEnclosedMimeType.GetHashCode();
        }

        public static bool operator ==(ImageFormat a, ImageFormat b)
        {
            // If both are null, or both are same instance, return true.
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            // If one is null, but not both, return false.
            if (((object)a == null) || ((object)b == null))
            {
                return false;
            }
            return (a.mstrMimeType == b.mstrMimeType) && (a.mstrEnclosedMimeType == b.mstrEnclosedMimeType);
        }

        public static bool operator !=(ImageFormat a, ImageFormat b)
        {
            return !(a == b);
        }

        public static ImageFormat JPG = new ImageFormat("image/jpeg");
        public static ImageFormat J2K = new ImageFormat("image/j2k");
        public static ImageFormat DICOM = new ImageFormat("application/dicom");
        public static ImageFormat DICOMJ2K = new ImageFormat("application/dicom", "image/j2k");
        public static ImageFormat TGA = new ImageFormat("image/x-targa");
        public static ImageFormat PDF = new ImageFormat("application/pdf");
        public static ImageFormat TIFF = new ImageFormat("image/tiff");
        public static ImageFormat BMP = new ImageFormat("image/bmp");
        public static ImageFormat HTML = new ImageFormat("text/html");
        
    }
}
