/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using System.Xml;

namespace VixAKCommon.gov.va.med.imaging.vix
{
    public class ImageQuality : BaseCustomOption
    {

        private int mintImageQualityValue;
        private string mstrImageQualityName;

        public int ImageQualityValue
        {
            get { return mintImageQualityValue; }
            set { mintImageQualityValue = value; }
        }        

        public string ImageQualityName
        {
            get { return mstrImageQualityName; }
            set { mstrImageQualityName = value; }
        }

        public ImageQuality(int imageQualityValue, string imageQualityName)
        {
            mintImageQualityValue = imageQualityValue;
            mstrImageQualityName = imageQualityName;
        }

        public override string ToString()
        {
            return mstrImageQualityName.ToUpper();
        }

        public string ToString(string d)
        {
            return mintImageQualityValue.ToString(d);
        }

        public static string IMAGE_QUALITY_NODE_NAME = "ImageQuality";

        public override System.Xml.XmlNode toXml(System.Xml.XmlDocument xmlDoc)
        {
            XmlNode node = xmlDoc.CreateElement(IMAGE_QUALITY_NODE_NAME);

            XmlAttribute attr = xmlDoc.CreateAttribute("QualityName");
            attr.Value = mstrImageQualityName;
            node.Attributes.Append(attr);
            attr = xmlDoc.CreateAttribute("QualityValue");
            attr.Value = mintImageQualityValue.ToString();
            node.Attributes.Append(attr);

            return node;
        }

        public override bool Equals(object obj)
        {
            if (obj is ImageQuality)
            {
                ImageQuality that = (ImageQuality)obj;
                return this.mintImageQualityValue == that.mintImageQualityValue;
            }
            else
            {
                return base.Equals(obj);
            }
        }

        public override int GetHashCode()
        {
            return base.GetHashCode() ^ mintImageQualityValue;
        }

        public static bool operator ==(ImageQuality a, ImageQuality b)
        {
            // If both are null, or both are same instance, return true.
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            // If one is null, but not both, return false.
            if (((object)a == null) || ((object)b == null))
            {
                return false;
            }
            return a.mintImageQualityValue == b.mintImageQualityValue;
        }

        public static bool operator !=(ImageQuality a, ImageQuality b)
        {
            return !(a == b);
        }

        public static ImageQuality THUMBNAIL = new ImageQuality(20, "THUMBNAIL");
        public static ImageQuality REFERENCE = new ImageQuality(70, "REFERENCE");
        public static ImageQuality DIAGNOSTIC = new ImageQuality(90, "DIAGNOSTIC");

        public static List<ImageQuality> GetImageQualities()
        {
            List<ImageQuality> qualities = new List<ImageQuality>();
            qualities.Add(THUMBNAIL);
            qualities.Add(REFERENCE);
            qualities.Add(DIAGNOSTIC);
            return qualities;
        }
    }
}
