/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class AWIVServiceCommon
    {
    }

    public class AWIVImageRequestInput
    {
        private readonly Study study;
        private readonly Instance instance;
        private readonly ImageQuality imageQuality;
        private readonly string transactionId;
        private readonly string expectedChecksum;

        private string mstrRequestUrl;
        private string mstrImageFormatSubTypeRequestString;
        private string mstrRequestedImageFormat;

        public string RequestedImageFormat
        {
            get { return mstrRequestedImageFormat; }
            set { mstrRequestedImageFormat = value; }
        }  

        public string ImageFormatSubTypeRequestString
        {
            get { return mstrImageFormatSubTypeRequestString; }
            set { mstrImageFormatSubTypeRequestString = value; }
        }        

        public Study Study
        {
            get { return study; }
        } 

        public ImageQuality ImageQuality
        {
          get { return imageQuality; }  
        } 

        public string RequestUrl
        {
            get { return mstrRequestUrl; }
            set { mstrRequestUrl = value; }
        }

        public string ExpectedChecksum
        {
            get { return expectedChecksum; }
        }


        public Instance Instance
        {
            get { return instance; }
        }

        public string TransactionId
        {
            get { return transactionId; }
        }

        public AWIVImageRequestInput(Study study, Instance instance, ImageQuality imageQuality, string transactionId, string expectedChecksum)
        {
            this.study = study;
            this.instance = instance;
            this.imageQuality = imageQuality;
            this.transactionId = transactionId;
            this.expectedChecksum = expectedChecksum;
        }

    }

    public class AWIVDownloadResult
    {
        private readonly Dictionary<string, string> mResponseHeaders =
            new Dictionary<string, string>();

        private readonly string mstrReturnedImageQuality;
        private readonly string mstrReturnedImageFormat;
        private readonly string mstrReturnedImageChecksum;

        public string ReturnedImageQuality
        {
            get { return mstrReturnedImageQuality; }
        }

        public string ReturnedImageFormat
        {
            get { return mstrReturnedImageFormat; }
        }

        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
        }

        public Dictionary<string, string> ResponseHeaders
        {
            get { return mResponseHeaders; }
        }

        public string HeaderChecksum
        {
            get
            {
                if (mResponseHeaders.ContainsKey("xxx-checksum"))
                {
                    return mResponseHeaders["xxx-checksum"];
                }
                return "";
            }
        }

        /// <summary>
        /// Return the name of the machine (server) that returned the image (from HTTP response header)
        /// </summary>
        public string HeaderMachineName
        {
            get
            {
                if (mResponseHeaders.ContainsKey("xxx-machineName"))
                {
                    return mResponseHeaders["xxx-machineName"];
                }
                return "";
            }
        }

        public AWIVDownloadResult(string returnedImageQuality, string returnedImageFormat, string returnedImageChecksum,
            WebHeaderCollection responseHeaders)
        {
            mstrReturnedImageChecksum = returnedImageChecksum;
            mstrReturnedImageFormat = returnedImageFormat;
            mstrReturnedImageQuality = returnedImageQuality;
            foreach (string key in responseHeaders.AllKeys)
            {
                mResponseHeaders.Add(key, responseHeaders[key]);
            }
        }
    }
}
