/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester;
using VixAKCommon.gov.va.med.imaging.vix;
using VixAKCommon.gov.va.med.imaging.vix.exceptions;
using System.Net;
using System.IO;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class ClinicalDisplayServiceImpl_v6 : AbstractBaseVixInterfaceService, 
        VixInterface, ClinicalDisplayInterface
    {
        private readonly string mstrMetdataUrl;
        private readonly string mstrImageUrl;
        private readonly string mstrUsername = "";
        private readonly string mstrPassword = "";
        private readonly bool mbAllowJ2kCompression;
        private readonly bool mbIncludeImageFormatsSubTypes;
        //private string mstrImageFormatSubTypeRequestString;
        //private string mstrImageFormat = "";
        //private string mstrImageQuality = "";
        //private string mstrRequestImageFormat = "";
        private readonly VixTestUser mTestUser;
        //private string mstrReturnedImageChecksum = "";

        

        /*
        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
        }

        public string ImageFormatSubTypeRequestString
        {
            get { return mstrImageFormatSubTypeRequestString; }
        }*/

        public bool IncludeImageFormatsSubTypes
        {
            get { return mbIncludeImageFormatsSubTypes; }
        }

        public string MetadataUrl
        {
            get { return mstrMetdataUrl; }
        }

        public string ImageUrl
        {
            get { return mstrImageUrl; }
        }

        /*
        public string ReturnedImageFormat
        {
            get { return mstrImageFormat; }
        }

        public string ReturnedImageQuality
        {
            get { return mstrImageQuality; }
        }

        public string RequestedImageFormat
        {
            get { return mstrRequestImageFormat; }
        }*/

        private readonly ClinicalDisplayService_v6.ImageMetadataClinicalDisplayService metadataService =
            new VixAKCommon.ClinicalDisplayService_v6.ImageMetadataClinicalDisplayService();

        public ClinicalDisplayServiceImpl_v6(string metadataUrl, string imageUrl, bool allowJ2kCompression,
            bool includeImageSubTypes, VixTestUser testUser, LogMessageDelegate OnLogMessage)
            : base(OnLogMessage)
        {
            mTestUser = testUser;
            mstrMetdataUrl = metadataUrl;
            mstrImageUrl = imageUrl;
            mbAllowJ2kCompression = allowJ2kCompression;
            mbIncludeImageFormatsSubTypes = includeImageSubTypes;

            if (!string.IsNullOrEmpty(mstrMetdataUrl))
            {
                metadataService.Url = mstrMetdataUrl;
            }
            metadataService.Credentials = new NetworkCredential(mstrUsername, mstrPassword);
            metadataService.Timeout = 500000; // 500 sec            
        }

        private ClinicalDisplayService_v6.UserCredentialsType GetUserCredentials()
        {
            ClinicalDisplayService_v6.UserCredentialsType credentials = new ClinicalDisplayService_v6.UserCredentialsType();        
            credentials.duz = mTestUser.Duz;
            credentials.fullname = mTestUser.Fullname;
            credentials.securityToken = mTestUser.SecurityToken;
            credentials.siteName = mTestUser.SiteName;
            credentials.siteNumber = mTestUser.SiteNumber;
            credentials.ssn = mTestUser.Ssn;
            return credentials;
        }

        private ClinicalDisplayService_v6.UserCredentialsType translateVixUser(VixTestUser user)
        {
            ClinicalDisplayService_v6.UserCredentialsType credentials = new VixAKCommon.ClinicalDisplayService_v6.UserCredentialsType();
            credentials.duz = user.Duz;
            credentials.fullname = user.Fullname;
            credentials.siteName = user.SiteName;
            credentials.siteNumber = user.SiteNumber;
            credentials.ssn = user.Ssn;
            return credentials;
        }

        public string GetImageInformation(string imageId, string transactionId)
        {
            return GetImageInformation(imageId, transactionId, null);
        }

        public string GetImageInformation(string imageId, string transactionId, VixTestUser testUser)
        {
            ClinicalDisplayService_v6.UserCredentialsType credentials = null;// GetUserCredentials();
            if (testUser == null)
            {
                credentials = GetUserCredentials();
            }
            else
            {
                credentials = translateVixUser(testUser);
            }
            return metadataService.getImageInformation(imageId, transactionId, credentials, false);
        }

        public string GetStudyReport(string studyId, string transactionId)
        {
            return metadataService.getStudyReport(transactionId, translateVixUser(mTestUser), studyId);
        }

        public bool LogImageAccess(string imageId, string patientIcn, string transactionId)
        {
            return LogImageAccess(imageId, patientIcn, transactionId, null);
        }

        public bool LogImageAccess(string imageId, string patientIcn, string transactionId, VixTestUser testUser)
        {
            ClinicalDisplayService_v6.UserCredentialsType credentials = null;
            if (testUser != null)
            {
                credentials = translateVixUser(testUser);
            }
            else
            {
                credentials = GetUserCredentials();
            }
            ClinicalDisplayService_v6.ImageAccessLogEventType logEvent = new ClinicalDisplayService_v6.ImageAccessLogEventType();
            logEvent.credentials = credentials;
            logEvent.eventType = ClinicalDisplayService_v6.ImageAccessLogEventTypeEventType.IMAGE_ACCESS;
            logEvent.id = imageId;           
            logEvent.patientIcn = patientIcn;
            logEvent.reasoncode = "";
            logEvent.reasondescription = "";            
            return metadataService.postImageAccessEvent(transactionId, logEvent);
        }


        public List<Study> GetPatientStudies(string siteNumber, string patientIcn, string transactionId)
        {
            ClinicalDisplayService_v6.FilterType filter = new ClinicalDisplayService_v6.FilterType();
            ClinicalDisplayService_v6.UserCredentialsType credentials = GetUserCredentials();
            filter.@class = null;
            filter.@event = null;
            filter.fromdate = null;

            ClinicalDisplayService_v6.ShallowStudiesType studiesType = metadataService.getPatientShallowStudyList(transactionId,
                siteNumber, patientIcn, filter, credentials, "1", true);

            if ((studiesType.error != null) && (studiesType.error.errorMessage.Length > 0))
            {
                throw new Exception(studiesType.error.errorMessage);
            }

            List<Study> result = ServiceTranslator.TranslateStudies(studiesType.studies);

            foreach (Study study in result)
            {
                transactionId = Guid.NewGuid().ToString();
                ClinicalDisplayService_v6.FatImageType[] images = metadataService.getStudyImageList(transactionId, study.StudyId, credentials, false);
                study.Series.Add(ServiceTranslator.TranslateImagesToSeries(images, study.StudyId));
            }

            return result;
        }

        public ClinicalDisplayStudyList GetPatientStudies(ClinicalDisplayStudiesRequest request, string transactionId)
        {
            ClinicalDisplayService_v6.FilterType filter = new ClinicalDisplayService_v6.FilterType();
            ClinicalDisplayService_v6.UserCredentialsType credentials = GetUserCredentials();
            filter.@class = null;
            filter.@event = null;
            filter.fromdate = request.FromDate;
            filter.todate = request.ToDate;

            LogMsg("Getting studies for patient with transaction ID '" + transactionId + "'");
            ClinicalDisplayService_v6.ShallowStudiesType studiesType = 
                metadataService.getPatientShallowStudyList(transactionId,
                request.SiteNumber, request.PatientIcn, filter, credentials, request.AuthorizedSensitivityLevel, request.IncludeArtifacts);

            if ((studiesType.error != null) && (studiesType.error.errorMessage.Length > 0))
            {
                return new ClinicalDisplayStudyList(studiesType.error.errorCode, studiesType.error.errorMessage);
            }

            List<Study> result = ServiceTranslator.TranslateStudies(studiesType.studies);

            foreach (Study study in result)
            {
                transactionId = Guid.NewGuid().ToString();
                ClinicalDisplayService_v6.FatImageType[] images = metadataService.getStudyImageList(transactionId, study.StudyId, credentials, false);
                study.Series.Add(ServiceTranslator.TranslateImagesToSeries(images, study.StudyId));
            }

            ClinicalDisplayStudyList clinicalDisplayStudyList = new ClinicalDisplayStudyList(result);
            return clinicalDisplayStudyList;
        }

        public DownloadResult DownloadTextFile(ImageRequestInput imageRequestInput)
        {
            return DownloadImage(imageRequestInput, true);
        }

        public DownloadResult DownloadImage(ImageRequestInput imageRequestInput)
        {
            return DownloadImage(imageRequestInput, false);
        }

        private DownloadResult DownloadImage(ImageRequestInput imageRequestInput, bool textFile)
        {
            string url = createImageUrl(imageRequestInput.Instance, imageRequestInput.ImageQuality, textFile);
            imageRequestInput.RequestUrl = url;
            string requestImageFormat = GetContentType(imageRequestInput.Instance.ImageType, imageRequestInput.ImageQuality, false);
            imageRequestInput.RequestedImageFormat = requestImageFormat;
            WebRequest request = WebRequest.Create(url);
            request.Timeout = 500000; // 500 seconds
            request.Credentials = new NetworkCredential(mstrUsername, mstrPassword);
            ClinicalDisplayService_v6.UserCredentialsType userCredentials = GetUserCredentials();
            request.Headers.Add("xxx-fullname", userCredentials.fullname);
            request.Headers.Add("xxx-sitename", userCredentials.siteName);
            request.Headers.Add("xxx-sitenumber", userCredentials.siteNumber);
            request.Headers.Add("xxx-ssn", userCredentials.ssn);
            request.Headers.Add("xxx-duz", userCredentials.duz);
            request.Headers.Add("xxx-transaction-id", imageRequestInput.TransactionId);
            string imageFormatSubTypeRequestString = "";
            if (mbIncludeImageFormatsSubTypes && !textFile)
            {
                imageFormatSubTypeRequestString = GetContentType(imageRequestInput.Instance.ImageType, imageRequestInput.ImageQuality, true);
                request.Headers.Add("xxx-contentTypeWithSubType", imageFormatSubTypeRequestString);
            }
            imageRequestInput.ImageFormatSubTypeRequestString = imageFormatSubTypeRequestString;

            WebResponse response = null;
            try
            {
                response = request.GetResponse();
            }
            catch (System.Net.WebException webEx)
            {
                if (webEx.Status != System.Net.WebExceptionStatus.ProtocolError)
                {
                    LogMsg(webEx.InnerException + "\n\n" + webEx.StackTrace);
                }
                System.Net.HttpWebResponse resp = (System.Net.HttpWebResponse)webEx.Response;
                if (resp.StatusCode == HttpStatusCode.NotFound)
                    throw new vix.exceptions.ImageNotFoundException(webEx.Message);
                else if (resp.StatusCode == HttpStatusCode.InternalServerError)
                    throw new vix.exceptions.InternalServerErrorException(webEx.Message);
                else
                    throw webEx;
            }
            if (response == null)
                throw new Exception("Got null response from server for url '" + url + "' from transaction [" + imageRequestInput.TransactionId + "].");
            Stream stream = response.GetResponseStream();

            byte[] buffer = VixAKCommonUtils.readStreamIntoBuffer(stream);

            string calculatedAlder32 = VixAKCommonUtils.CalculateAdler32(buffer);
            string headerAlder32 = response.Headers["xxx-checksum"];

            if (calculatedAlder32 != headerAlder32)
            {
                if (imageRequestInput.FailIfChecksumFromVixDoesNotMatch)
                {
                    throw new ChecksumFailedException("Calculated checksum [" + calculatedAlder32 + "] does not match checksum [" + headerAlder32 + "] from VIX", headerAlder32, calculatedAlder32);
                }
                else
                {
                    LogMsg("Calculated checksum [" + calculatedAlder32 + "] does not match checksum [" + headerAlder32 + "] from VIX, continuing");
                }
            }

            string format = response.Headers["xxx-image-format"];
            string returnedImageFormat = "";
            string returnedImageQuality = "";
            if (format != null)
                returnedImageFormat = format;
            string quality = response.Headers["xxx-image-quality"];
            if (quality != null)
                returnedImageQuality = quality;
            return new ClinicalDisplayDownloadResult(returnedImageQuality, returnedImageFormat, calculatedAlder32, 
                response.Headers, imageFormatSubTypeRequestString, buffer);
        }

        public string ExecutePassthroughMethod(string transactionId, VixTestUser testUser, string siteNumber, string methodName,
            List<PassthroughParameter> parameters)
        {
            ClinicalDisplayService_v6.UserCredentialsType credentials = null;// GetUserCredentials();
            if (testUser == null)
            {
                credentials = GetUserCredentials();
            }
            else
            {
                credentials = translateVixUser(testUser);
            }

            ClinicalDisplayService_v6.RemoteMethodParameterType[] parameterTypes = 
                ServiceTranslator.translate(parameters);
    
            return metadataService.remoteMethodPassthrough(transactionId, credentials, 
                siteNumber, methodName, parameterTypes);
        }

        public string GetImageDevFields(string transactionId, VixTestUser testUser, string imageId, string flags)
        {
            ClinicalDisplayService_v6.UserCredentialsType credentials = null;// GetUserCredentials();
            if (testUser == null)
            {
                credentials = GetUserCredentials();
            }
            else
            {
                credentials = translateVixUser(testUser);
            }
            return metadataService.getImageDevFields(imageId, flags, transactionId, credentials);
        }

        public string GetImageGlobalNodes(string transactionId, VixTestUser testUser, string imageId)
        {
            ClinicalDisplayService_v6.UserCredentialsType credentials = null;// GetUserCredentials();
            if (testUser == null)
            {
                credentials = GetUserCredentials();
            }
            else
            {
                credentials = translateVixUser(testUser);
            }
            return metadataService.getImageSystemGlobalNode(imageId, transactionId, credentials);
        }

        private string createImageUrl(Instance instace, ImageQuality imageQuality, bool textFile)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(mstrImageUrl);
            sb.Append("?imageURN=");
            sb.Append(instace.InstanceId);
            sb.Append("&imageQuality=" + imageQuality.ToString("d"));
            sb.Append("&contentType=" + GetContentType(instace.ImageType, imageQuality, false));
            if (textFile)
                sb.Append("&textFile=true");
            return sb.ToString();
        }

        private string GetContentType(int imageType, ImageQuality imageQuality, bool includeImageSubTypes)
        {
            string contentType = "";
            string delimiter = ",";
            bool isThumbnail = (imageQuality == ImageQuality.THUMBNAIL);
            switch (imageType)
            {
                case 1: // JPEG
                case 9: // bwmed
                case 17: // color scan
                case 18: // patient photo
                case 19: // XRAY JPEG
                // document images
                case 15: // TIF
                    contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter + 
                        ImageFormat.TIFF.ToString(includeImageSubTypes) + delimiter + 
                        ImageFormat.BMP.ToString(includeImageSubTypes);
                    break;
                case 21: // motion video
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        contentType = "video/x-msvideo";
                    }
                    break;
                case 101: // html
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        contentType = "text/html";
                    }
                    break;
                case 102: // Word document
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        contentType = "application/msword";
                    }
                    break;
                case 103: // text (ASCII text)
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        contentType = "text/plain";
                    }
                    break;
                case 104: // PDF
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        contentType = "application/pdf";
                    }
                    break;
                case 105: // RTF
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        contentType = "text/rtf";
                    }
                    break;
                case 106: // Audio
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        contentType = "audio/x-wav,audio/mpeg3";
                    }
                    break;
                case 3: // XRAY
                case 100: // DICOM
                    if (isThumbnail)
                    {
                        contentType = ImageFormat.JPG.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.TGA.ToString(includeImageSubTypes) + delimiter +
                            ImageFormat.BMP.ToString(includeImageSubTypes);
                    }
                    else
                    {
                        if (mbAllowJ2kCompression)
                        {
                            contentType = ImageFormat.DICOMJ2K.ToString(includeImageSubTypes) + delimiter +
                                ImageFormat.J2K.ToString(includeImageSubTypes) + delimiter + 
                                ImageFormat.DICOM.ToString(includeImageSubTypes) + delimiter + 
                                ImageFormat.TGA.ToString(includeImageSubTypes);
                        }
                        else
                        {
                            contentType = ImageFormat.DICOM.ToString(includeImageSubTypes) + delimiter +
                                ImageFormat.TGA.ToString(includeImageSubTypes);
                        }
                    }
                    break;
                default:
                    System.Diagnostics.Debug.Assert(false, "Unknown image quality type in GetContentType [" + imageQuality + "]");
                    break;
            }
            if (contentType.Length > 0)
            {
                contentType += ",*/*";
            }
            else
            {
                contentType = "*/*";
            }

            return contentType;
            
        }
    }
}
