/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using VixAKCommon.gov.va.med.imaging.vix.tester;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class ExchangeServiceImpl : AbstractBaseVixInterfaceService, VixInterface
    {
        private readonly string mstrMetdataUrl;
        private readonly string mstrImageUrl;
        private readonly string mstrUsername;
        private readonly string mstrPassword;
        //private string mstrReturnedImageChecksum = "";

        /*
        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
        }*/

        public string MetadataUrl
        {
            get { return mstrMetdataUrl; }
        }

        public string ImageUrl
        {
            get { return mstrImageUrl; }
        }

        /*
        public string ReturnedImageFormat
        {
            get { return ""; }
        }

        public string ReturnedImageQuality
        {
            get { return ""; }
        }

        public string RequestedImageFormat
        {
            get { return ""; }
        }*/

        private readonly ExchangeService.ImageMetadataService metadataService = new VixAKCommon.ExchangeService.ImageMetadataService();

        public ExchangeServiceImpl(string metadataUrl, string imageUrl, string username, string password, LogMessageDelegate OnLogMessage)
            : base(OnLogMessage)
        {
            mstrMetdataUrl = metadataUrl;
            mstrImageUrl = imageUrl;
            mstrUsername = username;
            mstrPassword = password;
            if (!string.IsNullOrEmpty(mstrMetdataUrl))
            {
                metadataService.Url = mstrMetdataUrl;
            }
            metadataService.Credentials = new NetworkCredential(mstrUsername, mstrPassword);
            metadataService.Timeout = 500000; // 500 sec
        }

        public string GetStudyReport(string studyId, string transactionId)
        {
            return "";
        }


        public List<Study> GetPatientStudies(string siteNumber, string patientIcn, string transactionId)
        {
            ExchangeService.RequestorType requestor = new ExchangeService.RequestorType();
            requestor.facilityid = "HA.OSD.MIL/HP0067-PI01";
            requestor.facilityname = "Walter Reed AMC";
            requestor.purposeofuse = ExchangeService.RequestorTypePurposeofuse.Routinehealthcare;
            requestor.ssn = "222-22-2222";
            requestor.username = "Colonel Mustard";
            ExchangeService.FilterType filter = new ExchangeService.FilterType();
            filter.fromdate = null;
            filter.todate = null;
            filter.studyid = null;
            ExchangeService.StudyType [] studies = metadataService.getPatientStudyList(siteNumber, requestor, filter, patientIcn, transactionId);

            return ServiceTranslator.TranslateStudies(studies);
        }

        public DownloadResult DownloadImage(ImageRequestInput imageRequestInput)
        {
            string url = createImageUrl(imageRequestInput.Study, imageRequestInput.Instance, imageRequestInput.ImageQuality);
            imageRequestInput.RequestUrl = url;
            WebRequest request = WebRequest.Create(url);
            request.Timeout = 500000; // 500 seconds
            request.Credentials = new NetworkCredential(mstrUsername, mstrPassword);
            request.Headers.Add("xxx-fullname", "Colonel Mustard");
            request.Headers.Add("xxx-sitename", "Walter Reed AMC");
            request.Headers.Add("xxx-sitenumber", "HA.OSD.MIL/HP0067-PI01");
            request.Headers.Add("xxx-ssn", "222-22-2222");
            request.Headers.Add("xxx-transaction-id", imageRequestInput.TransactionId);
            WebResponse response = null;
            try
            {
                response = request.GetResponse();
            }
            catch (System.Net.WebException webEx)
            {
                if (webEx.Status != System.Net.WebExceptionStatus.ProtocolError)
                {
                    LogMsg(webEx.InnerException + "\n\n" + webEx.StackTrace);
                }
                System.Net.HttpWebResponse resp = (System.Net.HttpWebResponse)webEx.Response;
                if (resp.StatusCode == HttpStatusCode.NotFound)
                    throw new vix.exceptions.ImageNotFoundException(webEx.Message);
                else if (resp.StatusCode == HttpStatusCode.InternalServerError)
                    throw new vix.exceptions.InternalServerErrorException(webEx.Message);
                else
                    throw webEx;
            }
            if (response == null)
                throw new Exception("Got null response from server for url '" + url + "' from transaction [" + imageRequestInput.TransactionId + "].");
            Stream stream = response.GetResponseStream();

            byte[] buffer = VixAKCommonUtils.readStreamIntoBuffer(stream);


            string calculatedAlder32 = VixAKCommonUtils.CalculateAdler32(buffer);
            string headerAlder32 = response.Headers["xxx-checksum"];

            if (calculatedAlder32 != headerAlder32)
            {
                //TryOutputImage(imageRequestInput.ChecksumOptions, buffer, imageRequestInput.TransactionId);
                throw new Exception("Calculated checksum [" + calculatedAlder32 + "] does not match checksum [" + headerAlder32 + "] from ViX");
            }
            /*
            if (imageRequestInput.ChecksumOptions != null)
            {
                if (!string.IsNullOrEmpty(imageRequestInput.ChecksumOptions.ExpectedChecksum))
                {
                    if (imageRequestInput.ChecksumOptions.ExpectedChecksum != calculatedAlder32)
                    {
                        TryOutputImage(imageRequestInput.ChecksumOptions, buffer, imageRequestInput.TransactionId);
                        throw new Exception("Calculated checksum [" + calculatedAlder32 + "] does not match expected checksum [" + imageRequestInput.ChecksumOptions.ExpectedChecksum + "]");
                    }
                }
            }*/
            return new DownloadResult("", "", calculatedAlder32, response.Headers, buffer);
        }

        private string createImageUrl(Study study, Instance instace, ImageQuality imageQuality)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(mstrImageUrl);
            sb.Append("?imageURN=");
            sb.Append(instace.InstanceId);
            sb.Append("&imageQuality=" + imageQuality.ToString("d"));
            sb.Append("&contentType=" + GetContentType(instace.ImageType, imageQuality));
            return sb.ToString();
        }

        private string GetContentType(int imageType, ImageQuality imageQuality)
        {
            if (imageQuality == ImageQuality.THUMBNAIL)
            {
                return "image/jpeg";
            }
            else if (imageQuality == ImageQuality.REFERENCE)
            {
                return "application/dicom";
            }
            else if (imageQuality == ImageQuality.DIAGNOSTIC)
            {
                return "application/dicom";
            }
            else
            {
                System.Diagnostics.Debug.Assert(false, "Unknown image quality type in GetContentType [" + imageQuality + "]");

                return "";
            }
        }

        
    }
}
