/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class ExchangeSiteServiceImpl: SiteServiceBaseInterface
    {

        private readonly ExchangeSiteService.ImagingExchangeSiteService siteService =
            new VixAKCommon.ExchangeSiteService.ImagingExchangeSiteService();

        private readonly string mstrSiteServiceUrl;

        public string SiteServiceUrl
        {
            get { return mstrSiteServiceUrl; }
        }

        public ExchangeSiteServiceImpl(string siteServiceUrl)
        {
            mstrSiteServiceUrl = siteServiceUrl;
            siteService.Url = siteServiceUrl;
        }

        public Site GetSite(string siteNumber)
        {
            ExchangeSiteService.ImagingExchangeSiteTO siteTo = siteService.getSite(siteNumber);
            return translateSite(siteTo);
        }

        public List<Site> GetSites(string siteNumbers)
        {
            List<Site> sites = new List<Site>();
            ExchangeSiteService.ImagingExchangeSiteTO[] siteTos = siteService.getSites(siteNumbers);
            if (siteTos != null)
            {
                foreach (ExchangeSiteService.ImagingExchangeSiteTO siteTo in siteTos)
                {
                    sites.Add(translateSite(siteTo));
                }
            }
            return sites;
        }

        public List<Site> GetAllSites()
        {
            List<Site> sites = new List<Site>();
            ExchangeSiteService.ImagingExchangeSiteTO[] siteTos = siteService.getImagingExchangeSites();
            if (siteTos != null)
            {
                foreach (ExchangeSiteService.ImagingExchangeSiteTO siteTo in siteTos)
                {
                    sites.Add(translateSite(siteTo));
                }
            }
            return sites;
        }

        private Site translateSite(ExchangeSiteService.ImagingExchangeSiteTO siteTo)
        {
            if (siteTo == null)
                return null;
            Site site = new Site(siteTo.siteNumber, siteTo.siteName, siteTo.regionID, 
                siteTo.siteAbbr, siteTo.vistaServer, siteTo.vistaPort, 
                siteTo.acceleratorServer, siteTo.acceleratorPort);
            return site;
        }

    }
}
