/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class ServiceTranslator
    {
        public static ExamSite TranslateExamSite(VistaRadService_v1.ExamSite examSite, string patientIcn)
        {
            ExamSite result = new ExamSite(examSite.siteNumber, patientIcn);
            foreach (VistaRadService_v1.ShallowExamType exam in examSite.exam)
            {
                result.Exams.Add(TranslateExam(exam));
            }
            return result;
        }

        public static VistaRadService_v1.RemoteMethodParameterType[] translateVistaRad(List<PassthroughParameter> parameters)
        {
            VistaRadService_v1.RemoteMethodParameterType[] result = new VixAKCommon.VistaRadService_v1.RemoteMethodParameterType[parameters.Count];
            for (int i = 0; i < parameters.Count; i++)
            {
                PassthroughParameter parameter = parameters[i];
                result[i] = new VixAKCommon.VistaRadService_v1.RemoteMethodParameterType();
                result[i].parameterIndex = parameter.ParameterIndex;
                result[i].parameterValue = new VixAKCommon.VistaRadService_v1.RemoteMethodParameterValueType();
                result[i].parameterValue.multipleValue = parameter.ParameterValue.MultipleValueField;
                result[i].parameterValue.value = parameter.ParameterValue.ValueField;
                switch (parameter.ParameterType)
                {
                    case PassthroughParameterType.list:
                        result[i].parameterType = VixAKCommon.VistaRadService_v1.RemoteMethodParameterTypeType.LIST;
                        break;
                    case PassthroughParameterType.literal:
                        result[i].parameterType = VixAKCommon.VistaRadService_v1.RemoteMethodParameterTypeType.LITERAL;
                        break;
                    case PassthroughParameterType.reference:
                        result[i].parameterType = VixAKCommon.VistaRadService_v1.RemoteMethodParameterTypeType.REFERENCE;
                        break;
                }
            }
            return result;
        }

        public static ClinicalDisplayService_v6.RemoteMethodParameterType[] translate(List<PassthroughParameter> parameters)
        {
            ClinicalDisplayService_v6.RemoteMethodParameterType[] result = new VixAKCommon.ClinicalDisplayService_v6.RemoteMethodParameterType[parameters.Count];
            for (int i = 0; i < parameters.Count; i++)
            {
                PassthroughParameter parameter = parameters[i];
                result[i] = new VixAKCommon.ClinicalDisplayService_v6.RemoteMethodParameterType();
                result[i].parameterIndex = parameter.ParameterIndex;
                result[i].parameterValue = new VixAKCommon.ClinicalDisplayService_v6.RemoteMethodParameterValueType();
                result[i].parameterValue.multipleValue = parameter.ParameterValue.MultipleValueField;
                result[i].parameterValue.value = parameter.ParameterValue.ValueField;
                switch (parameter.ParameterType)
                {
                    case PassthroughParameterType.list:
                        result[i].parameterType = VixAKCommon.ClinicalDisplayService_v6.RemoteMethodParameterTypeType.LIST;
                        break;
                    case PassthroughParameterType.literal:
                        result[i].parameterType = VixAKCommon.ClinicalDisplayService_v6.RemoteMethodParameterTypeType.LITERAL;
                        break;
                    case PassthroughParameterType.reference:
                        result[i].parameterType = VixAKCommon.ClinicalDisplayService_v6.RemoteMethodParameterTypeType.REFERENCE;
                        break;
                }
            }
            return result;
        }

        public static List<Exam> TranslateActiveExams(VistaRadService_v1.GetActiveWorklistResponseContentsType activeExamsResult, string siteNumber)
        {
            List<Exam> result = new List<Exam>();
            foreach (VistaRadService_v1.ActiveWorklistItemType worklistItem in activeExamsResult.worklistItems)
            {
                Exam exam = new Exam(siteNumber, "", worklistItem.examid);
                result.Add(exam);
            }
            return result;
        }

        public static Exam TranslateExam(VistaRadService_v1.ShallowExamType shallowExam)
        {
            Exam result = new Exam(shallowExam.examDetails.sitenumber, shallowExam.examDetails.patienticn, shallowExam.examDetails.examid);

            return result;
        }

        public static Exam TranslateFatExam(VistaRadService_v1.FatExamType exam)
        {
            Exam result = new Exam(exam.examDetails.sitenumber, exam.examDetails.patienticn, exam.examDetails.examid);
            result.RequisitionReport = exam.requisitionreport;
            result.RadiologyReport = exam.radiologyreport;

            if (exam.componentImages.images != null)
            {
                foreach (VistaRadService_v1.ExamImageType image in exam.componentImages.images)
                {
                    result.Images.Add(TranslateExamImage(exam, image));
                }
            }

            return result;
        }

        public static ExamImage TranslateExamImage(VistaRadService_v1.FatExamType exam, VistaRadService_v1.ExamImageType image)
        {
            ExamImage result = new ExamImage(exam.examDetails.sitenumber, exam.examDetails.patienticn, exam.examDetails.examid, image.imageid);
            result.RequestString = image.requestquerystring;
            return result;
        }

        public static List<Study> TranslateStudies(ClinicalDisplayService.ShallowStudyType[] studies)
        {
            List<Study> result = new List<Study>();
            foreach (ClinicalDisplayService.ShallowStudyType studyType in studies)
            {
                if (studyType.studyid != null)
                {
                    Study study = new Study(studyType.studyid);
                    result.Add(study);
                }
            }
            return result;
        }


        public static List<Study> TranslateStudies(ClinicalDisplayService_v2.ShallowStudyType[] studies)
        {
            List<Study> result = new List<Study>();
            foreach (ClinicalDisplayService_v2.ShallowStudyType studyType in studies)
            {
                if (studyType.studyid != null)
                {
                    Study study = new Study(studyType.studyid);
                    result.Add(study);
                }
            }
            return result;
        }

        public static List<Study> TranslateStudies(ClinicalDisplayService_v4.ShallowStudyType[] studies)
        {
            List<Study> result = new List<Study>();
            if (studies != null)
            {
                foreach (ClinicalDisplayService_v4.ShallowStudyType studyType in studies)
                {
                    if (studyType.studyid != null)
                    {
                        Study study = new Study(studyType.studyid);
                        result.Add(study);
                    }
                }
            }
            return result;
        }

        public static List<Study> TranslateStudies(ClinicalDisplayService_v6.ShallowStudyType[] studies)
        {
            List<Study> result = new List<Study>();
            if (studies != null)
            {
                foreach (ClinicalDisplayService_v6.ShallowStudyType studyType in studies)
                {
                    if (studyType.studyid != null)
                    {
                        Study study = new Study(studyType.studyid);
                        result.Add(study);
                    }
                }
            }
            return result;
        }

        public static Series TranslateImagesToSeries(ClinicalDisplayService_v6.FatImageType[] images, string studyId)
        {
            Series series = new Series(studyId);
            foreach (ClinicalDisplayService_v6.FatImageType imageType in images)
            {
                Instance instance = new Instance(imageType.imageid);
                if ((imageType.imagetype != null) && (imageType.imagetype.Length > 0))
                {
                    instance.ImageType = int.Parse(imageType.imagetype);
                }
                series.Instances.Add(instance);
            }
            return series;
        }

        public static Series TranslateImagesToSeries(ClinicalDisplayService_v2.FatImageType[] images, string studyId)
        {
            Series series = new Series(studyId);
            foreach (ClinicalDisplayService_v2.FatImageType imageType in images)
            {
                Instance instance = new Instance(imageType.imageid);
                if ((imageType.imagetype != null) && (imageType.imagetype.Length > 0))
                {
                    instance.ImageType = int.Parse(imageType.imagetype);
                }
                series.Instances.Add(instance);
            }
            return series;
        }

        public static Series TranslateImagesToSeries(ClinicalDisplayService_v4.FatImageType[] images, string studyId)
        {
            Series series = new Series(studyId);
            foreach (ClinicalDisplayService_v4.FatImageType imageType in images)
            {
                Instance instance = new Instance(imageType.imageid);
                if ((imageType.imagetype != null) && (imageType.imagetype.Length > 0))
                {
                    instance.ImageType = int.Parse(imageType.imagetype);
                }
                series.Instances.Add(instance);
            }
            return series;
        }

        public static Series TranslateImagesToSeries(ClinicalDisplayService.FatImageType[] images, string studyId)
        {
            Series series = new Series(studyId);
            foreach (ClinicalDisplayService.FatImageType imageType in images)
            {
                Instance instance = new Instance(imageType.imageid);
                if ((imageType.imagetype != null) && (imageType.imagetype.Length > 0))
                {
                    instance.ImageType = int.Parse(imageType.imagetype);
                }
                series.Instances.Add(instance);
            }
            return series;
        }

        public static List<Study> TranslateStudies(ExchangeService_v2.StudyType[] studies)
        {
            List<Study> result = new List<Study>();
            if (studies == null)
                return result;
            foreach (ExchangeService_v2.StudyType studyType in studies)
            {
                Study study = new Study(studyType.studyid);
                study.StudyDescription = studyType.description;
                if (studyType.componentseries != null)
                {
                    List<Series> series = TranslateSeries(studyType.componentseries);
                    study.Series.AddRange(series);
                }
                result.Add(study);
            }
            return result;
        }


        public static List<Study> TranslateStudies(ExchangeService.StudyType[] studies)
        {
            List<Study> result = new List<Study>();
            foreach (ExchangeService.StudyType studyType in studies)
            {
                Study study = new Study(studyType.studyid);
                study.StudyDescription = studyType.description;
                if (studyType.componentseries != null)
                {
                    List<Series> series = TranslateSeries(studyType.componentseries);
                    study.Series.AddRange(series);
                }
                result.Add(study);
            }
            return result;
        }

        public static Study TranslateGroup(AWIVService_v1.GroupType group)
        {
            Study study = new Study(group.studyUrn);
            Series series = new Series(group.studyUrn);
            study.Series.Add(series);
            foreach (AWIVService_v1.ImageType image in group.images)
            {
                series.Instances.Add(TranslateGroupImage(image));

            }
            return study;
        }

        private static Instance TranslateGroupImage(AWIVService_v1.ImageType image)
        {
            Instance instance = new Instance(image.imageurn);
            instance.ImageType = int.Parse( image.imagetype);
            return instance;
        }

        private static List<Series> TranslateSeries(ExchangeService_v2.SeriesType[] series)
        {
            List<Series> result = new List<Series>();
            foreach (ExchangeService_v2.SeriesType seriesType in series)
            {
                Series ser = new Series(seriesType.seriesid);
                ser.Description = seriesType.description;
                ser.DicomUid = seriesType.dicomuid;
                if (seriesType.componentinstances != null)
                {
                    List<Instance> instances = TranslateImages(seriesType.componentinstances);
                    ser.Instances.AddRange(instances);
                }
                result.Add(ser);

            }
            return result;
        }

        private static List<Series> TranslateSeries(ExchangeService.SeriesType[] series)
        {
            List<Series> result = new List<Series>();
            foreach (ExchangeService.SeriesType seriesType in series)
            {
                Series ser = new Series(seriesType.seriesid);
                ser.Description = seriesType.description;
                ser.DicomUid = seriesType.dicomuid;
                if (seriesType.componentinstances != null)
                {
                    List<Instance> instances = TranslateImages(seriesType.componentinstances);
                    ser.Instances.AddRange(instances);
                }
                result.Add(ser);

            }
            return result;
        }

        private static List<Instance> TranslateImages(ExchangeService.InstanceType[] instances)
        {
            List<Instance> images = new List<Instance>();

            foreach (ExchangeService.InstanceType instance in instances)
            {
                Instance image = new Instance(instance.imageurn);
                image.DicomUid = instance.dicomuid;
                images.Add(image);
            }
            return images;
        }

        private static List<Instance> TranslateImages(ExchangeService_v2.InstanceType[] instances)
        {
            List<Instance> images = new List<Instance>();

            foreach (ExchangeService_v2.InstanceType instance in instances)
            {
                Instance image = new Instance(instance.imageurn);
                image.DicomUid = instance.dicomuid;
                images.Add(image);
            }
            return images;
        }
    }

    public class XcaServiceTranslator
    {
        public static List<Study> createDocumentSetsFromXml(string xml)
        {            
            List<Study> result = new List<Study>();
         
            XmlDocument xmlDoc = new XmlDocument();
            string status = "";
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(xmlDoc.NameTable);
            nsmgr.AddNamespace("soapenv", "http://www.w3.org/2003/05/soap-envelope");
            nsmgr.AddNamespace("urn", "urn:oasis:names:tc:ebxml-regrep:xsd:query:3.0");
            nsmgr.AddNamespace("urn1", "urn:oasis:names:tc:ebxml-regrep:xsd:rim:3.0");
            nsmgr.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
            nsmgr.AddNamespace("wsa", "http://www.w3.org/2005/08/addressing");
            nsmgr.AddNamespace("ns3", "urn:oasis:names:tc:ebxml-regrep:xsd:rs:3.0");
            xmlDoc.LoadXml(xml);


            XmlNode adHocQueryResponseNode = xmlDoc.SelectSingleNode("/soapenv:Envelope/soapenv:Body/urn:AdhocQueryResponse", nsmgr);
            if (adHocQueryResponseNode != null)
            {
                if (adHocQueryResponseNode.Attributes["status"] != null)
                {
                    status = adHocQueryResponseNode.Attributes["status"].Value;
                }
            }

            XmlNode registryObjectListNode = xmlDoc.SelectSingleNode("/soapenv:Envelope/soapenv:Body/urn:AdhocQueryResponse/urn1:RegistryObjectList", nsmgr);
            if (registryObjectListNode != null)
            {
                XmlNodeList extrinsicObjectNodes = registryObjectListNode.SelectNodes("urn1:ExtrinsicObject", nsmgr);
                foreach (XmlNode extrinsicObjectNode in extrinsicObjectNodes)
                {
                    result.Add(parseExtrinsicObject(extrinsicObjectNode, nsmgr));
                }
                XmlNodeList identifiableNodes = registryObjectListNode.SelectNodes("urn1:Identifiable", nsmgr);
                foreach (XmlNode identifiableNode in identifiableNodes)
                {
                    if (identifiableNode.Attributes["xsi:type"] != null)
                    {
                        string type = identifiableNode.Attributes["xsi:type"].Value;
                        if (type.EndsWith("ExtrinsicObjectType"))// only parsing if actually ExtrinsicObject
                        {
                            result.Add(parseExtrinsicObject(identifiableNode, nsmgr));
                        }
                    }

                }
            }
            return result;
        }

        private static DocumentSet parseExtrinsicObject(XmlNode extrinsicObjectNode, XmlNamespaceManager nsmgr)
        {
            string mimeType = extrinsicObjectNode.Attributes["mimeType"].Value;
            string id = extrinsicObjectNode.Attributes["id"].Value;
            string uri = "";
            string patientIcn = "";
            string creationTime = "";
            string homeCommunityId = extrinsicObjectNode.Attributes["home"].Value;
            string repositoryId = "";
            string size = "";
            string hash = "";

            XmlNodeList slotNodes = extrinsicObjectNode.SelectNodes("urn1:Slot", nsmgr);
            foreach (XmlNode slotNode in slotNodes)
            {
                string slotName = slotNode.Attributes["name"].Value;
                if (slotName == "URI")
                {
                    uri = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "sourcePatientId")
                {
                    patientIcn = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "creationTime")
                {
                    creationTime = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "repositoryUniqueId")
                {
                    repositoryId = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "size")
                {
                    size = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "hash")
                {
                    hash = extractValueFromSlot(slotNode, nsmgr);
                }
            }
            XmlNode nameNode = extrinsicObjectNode.SelectSingleNode("urn1:Name/urn1:LocalizedString", nsmgr);
            string name = null;
            if (nameNode != null)
            {
                name = nameNode.Attributes["value"].Value;
            }
            XmlNode descriptionNode = extrinsicObjectNode.SelectSingleNode("urn1:Description/urn1:LocalizedString", nsmgr);
            string description = null;
            if (descriptionNode != null)
            {
                if (descriptionNode.Attributes["value"] != null)
                {
                    description = descriptionNode.Attributes["value"].Value;
                }
            }

            string codingScheme = "";
            string classCode = "";

            XmlNodeList classificationNodes = extrinsicObjectNode.SelectNodes("urn1:Classification", nsmgr);
            foreach (XmlNode classificationNode in classificationNodes)
            {
                string classificationScheme = classificationNode.Attributes["classificationScheme"].Value;
                if (classificationScheme == "urn:uuid:f0306f51-975f-434e-a61c-c59651d33983")
                {
                    classCode = classificationNode.Attributes["nodeRepresentation"].Value;
                    XmlNode classificationSlotNode = classificationNode.SelectSingleNode("urn1:Slot", nsmgr);
                    codingScheme = extractValueFromSlot(classificationSlotNode, nsmgr);

                }
            }
            DocumentSet documentSet = new DocumentSet(homeCommunityId, repositoryId, id);
            //document.mstrCreationTime = crea
            //document.mstrDescription = description;
            //document.mstrName = name;
            //document.mstrSourcePatientId = patientIcn;
            //document.mstrUri = uri;
            //document.mstrMimeType = mimeType;
            //document.mstrCodingScheme = codingScheme;
            //document.mstrClassCode = classCode;
            //document.mstrCreationTime = creationTime;
            //document.mstrId = id;
            //document.mstrHomeCommunityId = homeCommunityId;
            //document.mstrRepositoryId = repositoryId;

            Series seriesHolder = new Series(id);
            string globalId = homeCommunityId + "_" + repositoryId + "_" + id;
            seriesHolder.Instances.Add(new Instance(id, globalId));
            documentSet.Series.Add(seriesHolder);
            return documentSet;
        }

        private static string extractValueFromSlot(XmlNode slotNode, XmlNamespaceManager nsmgr)
        {
            XmlNode valueNode = slotNode.SelectSingleNode("urn1:ValueList/urn1:Value", nsmgr);
            return valueNode.InnerText.Trim();
        }

    }
}
