/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class VistaRadServiceCommon
    {
    }

    public class VistaRadImageRequestInput
    {
        private readonly ExamImage image;
        private readonly string transactionId;

        private string mstrRequestUrl;

        public string RequestUrl
        {
            get { return mstrRequestUrl; }
            set { mstrRequestUrl = value; }
        }

        public ExamImage Image
        {
            get { return image; }
        } 

        public string TransactionId
        {
            get { return transactionId; }
        }

        public VistaRadImageRequestInput(ExamImage image, string transactionId)
        {
            this.image = image;
            this.transactionId = transactionId;
        }

    }

    public class VistaRadSavableDownloadResult : SavableObject
    {
        private string mstrReturnedImageQuality;
        private string mstrReturnedImageFormat;
        private string mstrReturnedImageChecksum;
        private int mintFileSize;

        public string ReturnedImageQuality
        {
            get { return mstrReturnedImageQuality; }
            set { mstrReturnedImageQuality = value; }
        }

        public string ReturnedImageFormat
        {
            get { return mstrReturnedImageFormat; }
            set { mstrReturnedImageFormat = value; }
        }

        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
            set { mstrReturnedImageChecksum = value; }
        }

        public int FileSize
        {
            get { return mintFileSize; }
            set { mintFileSize = value; }
        }

        public VistaRadSavableDownloadResult()
        {
        }

        public VistaRadSavableDownloadResult(VistaRadDownloadResult vistaRadDownloadResult)
        {
            mstrReturnedImageChecksum = vistaRadDownloadResult.ReturnedImageChecksum;
            mstrReturnedImageFormat = vistaRadDownloadResult.ReturnedImageFormat;
            mstrReturnedImageQuality = vistaRadDownloadResult.ReturnedImageQuality;
            mintFileSize = vistaRadDownloadResult.ImageBuffer.Length;
        }

        public override bool Equals(object obj)
        {
            if (obj is VistaRadSavableDownloadResult)
            {
                VistaRadSavableDownloadResult that = (VistaRadSavableDownloadResult)obj;
                return (this.mintFileSize == that.mintFileSize) &&
                    (this.mstrReturnedImageChecksum == that.mstrReturnedImageChecksum) &&
                    (this.mstrReturnedImageFormat == that.mstrReturnedImageFormat) &&
                    (this.mstrReturnedImageQuality == that.mstrReturnedImageQuality);
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class VistaRadSavableDownloadResultList : SavableObject
    {
        private List<VistaRadSavableDownloadResult> results = new List<VistaRadSavableDownloadResult>();

        public List<VistaRadSavableDownloadResult> Results
        {
            get { return results; }
            set { results = value; }
        }

        public VistaRadSavableDownloadResultList()
        {
        }

        public VistaRadSavableDownloadResultList(List<VistaRadSavableDownloadResult> results)
        {
            this.results.AddRange(results);
        }

        public override bool Equals(object obj)
        {
            if (obj is VistaRadSavableDownloadResultList)
            {
                VistaRadSavableDownloadResultList that = (VistaRadSavableDownloadResultList)obj;
                if (this.results.Count != that.results.Count)
                    return false;
                for (int i = 0; i < this.results.Count; i++)
                {
                    VistaRadSavableDownloadResult r = this.results[i];
                    VistaRadSavableDownloadResult t = that.results[i];
                    if (!r.Equals(t))
                        return false;
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class VistaRadDownloadResult : DownloadResult
    {
        /*
        private readonly Dictionary<string, string> mResponseHeaders = 
            new Dictionary<string, string>();

        private readonly string mstrReturnedImageQuality;
        private readonly string mstrReturnedImageFormat;
        private readonly string mstrReturnedImageChecksum;
        private readonly byte[] mImageBuffer;

        public byte[] ImageBuffer
        {
            get { return mImageBuffer; }
        } 

        public string ReturnedImageQuality
        {
            get { return mstrReturnedImageQuality; }
        } 

        public string ReturnedImageFormat
        {
            get { return mstrReturnedImageFormat; }
        } 

        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
        }       

        public Dictionary<string, string> ResponseHeaders
        {
            get { return mResponseHeaders; }
        }

        public string HeaderChecksum
        {
            get
            {
                if (mResponseHeaders.ContainsKey("xxx-checksum"))
                {
                    return mResponseHeaders["xxx-checksum"];
                }
                return "";
            }
        }

        /// <summary>
        /// Return the name of the machine (server) that returned the image (from HTTP response header)
        /// </summary>
        public string HeaderMachineName
        {
            get
            {
                if (mResponseHeaders.ContainsKey("xxx-machineName"))
                {
                    return mResponseHeaders["xxx-machineName"];
                }
                return "";
            }
        }

        public VistaRadDownloadResult(string returnedImageQuality, string returnedImageFormat, string returnedImageChecksum, 
            WebHeaderCollection responseHeaders, byte[] imageBuffer)
        {
            mstrReturnedImageChecksum = returnedImageChecksum;
            mstrReturnedImageFormat = returnedImageFormat;
            mstrReturnedImageQuality = returnedImageQuality;
            mImageBuffer = imageBuffer;
            foreach (string key in responseHeaders.AllKeys)
            {
                mResponseHeaders.Add(key, responseHeaders[key]);
            }
        }*/

        public VistaRadDownloadResult(string returnedImageQuality, string returnedImageFormat, string returnedImageChecksum,
            WebHeaderCollection responseHeaders, byte[] imageBuffer)
            : base(returnedImageQuality, returnedImageFormat, returnedImageChecksum, responseHeaders, imageBuffer)
        {


        }
    }
  
}
