/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using VixAKCommon.gov.va.med.imaging.vix.tester;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class VistaRadServiceImpl : AbstractBaseVixInterfaceService
    {
        private readonly string mstrMetadataUrl;
        private readonly string mstrImageUrl;
        private readonly string mstrTextUrl;
        private readonly string mstrUsername = "";
        private readonly string mstrPassword = "";

        /*
        private string mstrImageFormat = "";
        private string mstrImageQuality = "";

        private string mstrReturnedImageChecksum = "";

        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
        }

        public string ReturnedImageFormat
        {
            get { return mstrImageFormat; }
        }

        public string ReturnedImageQuality
        {
            get { return mstrImageQuality; }
        }*/

        private VistaRadService_v1.ImageMetadataVistaRadService metadataService;

        public VistaRadServiceImpl(string metadataUrl, string imageUrl, string txtUrl, LogMessageDelegate OnLogMessage)
            : base(OnLogMessage)
        {
            mstrMetadataUrl = metadataUrl;
            mstrImageUrl = imageUrl;
            mstrTextUrl = txtUrl;
            metadataService = new VixAKCommon.VistaRadService_v1.ImageMetadataVistaRadService();
            metadataService.Url = mstrMetadataUrl;
            metadataService.Timeout = 500000; // 500 sec     
            metadataService.Credentials = new NetworkCredential(mstrUsername, mstrPassword);
        }

        private VistaRadService_v1.UserCredentials GetUserCredentials()
        {
            VistaRadService_v1.UserCredentials credentials = new VistaRadService_v1.UserCredentials();
            credentials.duz = "";
            credentials.fullname = "";
            credentials.siteName = "";
            credentials.siteNumber = "";
            credentials.ssn = "";
            credentials.securityToken = "";
            return credentials;
        }

        public ExamSite getPatientExams(string patientIcn, string siteNumber)
        {
            VistaRadService_v1.ExamSite examSite = 
                metadataService.getSiteExamList(Guid.NewGuid().ToString(), GetUserCredentials(), patientIcn, siteNumber, false);
            return ServiceTranslator.TranslateExamSite(examSite, patientIcn);
        }

        public List<Exam> getActiveExams(string siteNumber, string userDivision, string listDescriptor)
        {
            VistaRadService_v1.GetActiveWorklistResponseContentsType result =
                metadataService.getActiveWorklist(Guid.NewGuid().ToString(), GetUserCredentials(), siteNumber, userDivision, listDescriptor);
            return ServiceTranslator.TranslateActiveExams(result, siteNumber);
        }

        public Exam getExamDetails(string examId)
        {
            VistaRadService_v1.FatExamType fatExam =
                metadataService.getExamDetails(Guid.NewGuid().ToString(), GetUserCredentials(), examId);

            return ServiceTranslator.TranslateFatExam(fatExam);

        }

        public VistaRadDownloadResult DownloadTextFile(VistaRadImageRequestInput imageRequestInput)
        {
            string url = mstrTextUrl + "?imageUrn=" + imageRequestInput.Image.ImageId;
            imageRequestInput.RequestUrl = url;

            WebRequest request = WebRequest.Create(url);
            request.Timeout = 500000; // 500 seconds
            request.Credentials = new NetworkCredential(mstrUsername, mstrPassword);
            VistaRadService_v1.UserCredentials userCredentials = GetUserCredentials();
            request.Headers.Add("xxx-fullname", userCredentials.fullname);
            request.Headers.Add("xxx-sitename", userCredentials.siteName);
            request.Headers.Add("xxx-sitenumber", userCredentials.siteNumber);
            request.Headers.Add("xxx-ssn", userCredentials.ssn);
            request.Headers.Add("xxx-duz", userCredentials.duz);
            request.Headers.Add("xxx-transaction-id", imageRequestInput.TransactionId);
            /*
            mstrImageFormatSubTypeRequestString = "";
            if (mbIncludeImageFormatsSubTypes)
            {
                mstrImageFormatSubTypeRequestString = GetContentType(instance.ImageType, imageQuality, true);
                request.Headers.Add("xxx-contentTypeWithSubType", mstrImageFormatSubTypeRequestString);
            }*/

            WebResponse response = null;
            try
            {
                response = request.GetResponse();
            }
            catch (System.Net.WebException webEx)
            {
                if (webEx.Status != System.Net.WebExceptionStatus.ProtocolError)
                {
                    LogMsg(webEx.InnerException + "\n\n" + webEx.StackTrace);
                }
                System.Net.HttpWebResponse resp = (System.Net.HttpWebResponse)webEx.Response;
                if (resp.StatusCode == HttpStatusCode.NotFound)
                    throw new vix.exceptions.ImageNotFoundException(webEx.Message);
                else if (resp.StatusCode == HttpStatusCode.InternalServerError)
                    throw new vix.exceptions.InternalServerErrorException(webEx.Message);
                else
                    throw webEx;
            }
            if (response == null)
                throw new Exception("Got null response from server for url '" + url + "' from transaction [" + imageRequestInput.TransactionId + "].");
            Stream stream = response.GetResponseStream();

            byte[] buffer = VixAKCommonUtils.readStreamIntoBuffer(stream);

            return new VistaRadDownloadResult(null, null, null, response.Headers, buffer);
        }

        public VistaRadDownloadResult DownloadImage(VistaRadImageRequestInput imageRequestInput)
        {
            string url = createImageUrl(imageRequestInput.Image);
            imageRequestInput.RequestUrl = url;
            
            WebRequest request = WebRequest.Create(url);
            request.Timeout = 500000; // 500 seconds
            request.Credentials = new NetworkCredential(mstrUsername, mstrPassword);
            VistaRadService_v1.UserCredentials userCredentials = GetUserCredentials();
            request.Headers.Add("xxx-fullname", userCredentials.fullname);
            request.Headers.Add("xxx-sitename", userCredentials.siteName);
            request.Headers.Add("xxx-sitenumber", userCredentials.siteNumber);
            request.Headers.Add("xxx-ssn", userCredentials.ssn);
            request.Headers.Add("xxx-duz", userCredentials.duz);
            request.Headers.Add("xxx-transaction-id", imageRequestInput.TransactionId);
            /*
            mstrImageFormatSubTypeRequestString = "";
            if (mbIncludeImageFormatsSubTypes)
            {
                mstrImageFormatSubTypeRequestString = GetContentType(instance.ImageType, imageQuality, true);
                request.Headers.Add("xxx-contentTypeWithSubType", mstrImageFormatSubTypeRequestString);
            }*/

            WebResponse response = null;
            try
            {
                response = request.GetResponse();
            }
            catch (System.Net.WebException webEx)
            {
                if (webEx.Status != System.Net.WebExceptionStatus.ProtocolError)
                {
                    LogMsg(webEx.InnerException + "\n\n" + webEx.StackTrace);
                }
                System.Net.HttpWebResponse resp = (System.Net.HttpWebResponse)webEx.Response;
                if (resp.StatusCode == HttpStatusCode.NotFound)
                    throw new vix.exceptions.ImageNotFoundException(webEx.Message);
                else if (resp.StatusCode == HttpStatusCode.InternalServerError)
                    throw new vix.exceptions.InternalServerErrorException(webEx.Message);
                else
                    throw webEx;
            }
            if (response == null)
                throw new Exception("Got null response from server for url '" + url + "' from transaction [" + imageRequestInput.TransactionId + "].");
            Stream stream = response.GetResponseStream();

            byte[] buffer = VixAKCommonUtils.readStreamIntoBuffer(stream);

            string calculatedAlder32 = VixAKCommonUtils.CalculateAdler32(buffer);
            string headerAlder32 = response.Headers["xxx-checksum"];

            if (calculatedAlder32 != headerAlder32)
            {
                throw new Exception("Calculated checksum [" + calculatedAlder32 + "] does not match checksum [" + headerAlder32 + "] from ViX");
            }

            /*
            if (!string.IsNullOrEmpty(imageRequestInput.ExpectedChecksum))
            {
                if (imageRequestInput.ExpectedChecksum != calculatedAlder32)
                {
                    throw new Exception("Calculated checksum [" + calculatedAlder32 + "] does not match expected checksum [" + imageRequestInput.ExpectedChecksum + "]");
                }
            }*/

            string format = response.Headers["xxx-image-format"];
            string imageFormatReturned = "";
            if (format != null)
                imageFormatReturned = format;
            string quality = response.Headers["xxx-image-quality"];
            string imageQualityReturned = "";
            if (quality != null)
                imageQualityReturned = quality;

            return new VistaRadDownloadResult(imageQualityReturned, imageFormatReturned, calculatedAlder32, 
                response.Headers, buffer);
        }

        private string createImageUrl(ExamImage image)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(mstrImageUrl);
            sb.Append("?");
            sb.Append(image.RequestString); 
            return sb.ToString();
        }

        private VistaRadService_v1.UserCredentials translateVixUser(VixTestUser user)
        {
            VistaRadService_v1.UserCredentials credentials = new VistaRadService_v1.UserCredentials();
            credentials.duz = user.Duz;
            credentials.fullname = user.Fullname;
            credentials.siteName = user.SiteName;
            credentials.siteNumber = user.SiteNumber;
            credentials.ssn = user.Ssn;
            return credentials;
        }

        public string ExecutePassthroughMethod(string transactionId, VixTestUser testUser, string siteNumber, string methodName,
            List<PassthroughParameter> parameters)
        {
            VistaRadService_v1.UserCredentials credentials = null;// GetUserCredentials();
            if (testUser == null)
            {
                credentials = GetUserCredentials();
            }
            else
            {
                credentials = translateVixUser(testUser);
            }
            VistaRadService_v1.RemoteMethodParameterType [] parameterTypes =
                ServiceTranslator.translateVistaRad(parameters);

            return metadataService.remoteMethodPassthrough(transactionId, credentials,
                siteNumber, methodName, parameterTypes);
        }
    }
}
