/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester;
using System.Net;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public interface VixInterface
    {
        List<Study> GetPatientStudies(string siteNumber, string patientIcn, string transactionId);

        string GetStudyReport(string studyId, string transactionId);

        DownloadResult DownloadImage(ImageRequestInput imageRequestInput);

        //DownloadResult DownloadImage(Study study, Instance instance, ImageQuality imageQuality, string transactionId, ChecksumTestOptions checksumOptions);

        string MetadataUrl
        {
            get;
        }

        string ImageUrl
        {
            get;
        }
        /*
        string ReturnedImageFormat
        {
            get;
        }

        string ReturnedImageQuality
        {
            get;
        }

        string RequestedImageFormat
        {
            get;
        }

        string ReturnedImageChecksum
        {
            get;
        }*/

        event LogMessageDelegate OnLogMessageEvent;
    }

    public class SavableDownloadResultList : SavableObject
    {
        private List<SavableDownloadResult> results = new List<SavableDownloadResult>();

        public List<SavableDownloadResult> Results
        {
            get { return results; }
            set { results = value; }
        }

        public SavableDownloadResultList()
        {
        }

        public SavableDownloadResultList(List<DownloadResult> downloadResults)
        {
            foreach (DownloadResult dr in downloadResults)
            {
                results.Add(new SavableDownloadResult(dr));
            }
        }

        public override bool Equals(object obj)
        {
            if (obj is SavableDownloadResultList)
            {
                SavableDownloadResultList that = (SavableDownloadResultList)obj;
                if (this.results.Count != that.results.Count)
                    return false;
                for (int i = 0; i < results.Count; i++)
                {
                    SavableDownloadResult s = results[i];
                    SavableDownloadResult t = that.results[i];
                    if (!s.Equals(t))
                    {
                        return false;
                    }
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class SavableDownloadResult : SavableObject
    {
        private string mstrReturnedImageQuality;
        private string mstrReturnedImageFormat;
        private string mstrReturnedImageChecksum;
        private int mintFileSize;

        public string ReturnedImageQuality
        {
            get { return mstrReturnedImageQuality; }
            set { mstrReturnedImageQuality = value; }
        }
        
        public string ReturnedImageFormat
        {
            get { return mstrReturnedImageFormat; }
            set { mstrReturnedImageFormat = value; }
        }        

        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
            set { mstrReturnedImageChecksum = value; }
        }        

        public int FileSize
        {
            get { return mintFileSize; }
            set { mintFileSize = value; }
        }

        public SavableDownloadResult()
        {
        }

        public SavableDownloadResult(DownloadResult downloadResult)
        {
            mintFileSize = downloadResult.ImageBuffer.Length;
            mstrReturnedImageChecksum = downloadResult.ReturnedImageChecksum;
            mstrReturnedImageFormat = downloadResult.ReturnedImageFormat;
            mstrReturnedImageQuality = downloadResult.ReturnedImageQuality;
        }

        public override bool Equals(object obj)
        {
            if (obj is SavableDownloadResult)
            {
                SavableDownloadResult that = (SavableDownloadResult)obj;
                return (mstrReturnedImageChecksum == that.mstrReturnedImageChecksum) &&
                    (mstrReturnedImageFormat == that.mstrReturnedImageFormat) &&
                    (mstrReturnedImageQuality == that.mstrReturnedImageQuality) &&
                    (mintFileSize == that.mintFileSize);
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class DownloadResult
    {
        private readonly Dictionary<string, string> mResponseHeaders = 
            new Dictionary<string, string>();

        private readonly string mstrReturnedImageQuality;
        private readonly string mstrReturnedImageFormat;
        private readonly string mstrReturnedImageChecksum;
        private readonly byte[] mImageBuffer;

        public byte[] ImageBuffer
        {
            get { return mImageBuffer; }
        }

        public int ImageBufferSize
        {
            get { return mImageBuffer == null ? -1 : mImageBuffer.Length; }
        }

        public string ReturnedImageQuality
        {
            get { return mstrReturnedImageQuality; }
        } 

        public string ReturnedImageFormat
        {
            get { return mstrReturnedImageFormat; }
        } 

        public string ReturnedImageChecksum
        {
            get { return mstrReturnedImageChecksum; }
        }

        public Dictionary<string, string> ResponseHeaders
        {
            get { return mResponseHeaders; }
        }

        public string HeaderChecksum
        {
            get
            {
                if (mResponseHeaders.ContainsKey("xxx-checksum"))
                {
                    return mResponseHeaders["xxx-checksum"];
                }
                return "";
            }
        }

        /// <summary>
        /// Return the name of the machine (server) that returned the image (from HTTP response header)
        /// </summary>
        public string HeaderMachineName
        {
            get
            {
                if (mResponseHeaders.ContainsKey("xxx-machineName"))
                {
                    return mResponseHeaders["xxx-machineName"];
                }
                return "";
            }
        }

        public DownloadResult(string returnedImageQuality, string returnedImageFormat, string returnedImageChecksum, 
            WebHeaderCollection responseHeaders, byte[] imageBuffer)
        {
            mstrReturnedImageChecksum = returnedImageChecksum;
            mstrReturnedImageFormat = returnedImageFormat;
            mstrReturnedImageQuality = returnedImageQuality;
            mImageBuffer = imageBuffer;
            foreach (string key in responseHeaders.AllKeys)
            {
                mResponseHeaders.Add(key, responseHeaders[key]);
            }
        }

    } // DownloadResult

    public class ImageRequestInput
    {
        private readonly Study study;
        private readonly Instance instance;
        private readonly ImageQuality imageQuality;
        private readonly string transactionId;
        private readonly bool failIfChecksumFromVixDoesNotMatch;       

        private string mstrRequestedImageFormat;
        private string mstrImageFormatSubTypeRequestString;
        private string mstrRequestUrl;

        public string RequestedImageFormat
        {
            get { return mstrRequestedImageFormat; }
            set { mstrRequestedImageFormat = value; }
        }        

        public string ImageFormatSubTypeRequestString
        {
            get { return mstrImageFormatSubTypeRequestString; }
            set { mstrImageFormatSubTypeRequestString = value; }
        }        

        public string RequestUrl
        {
            get { return mstrRequestUrl; }
            set { mstrRequestUrl = value; }
        }

        public bool FailIfChecksumFromVixDoesNotMatch
        {
            get { return failIfChecksumFromVixDoesNotMatch; }
        } 

        public Instance Instance
        {
            get { return instance; }
        } 

        public ImageQuality ImageQuality
        {
            get { return imageQuality; }
        }         

        public string TransactionId
        {
            get { return transactionId; }
        } 

        public Study Study
        {
            get { return study; }
        }

        public ImageRequestInput(Study study, Instance instance, ImageQuality imageQuality,
            string transactionId, bool failIfChecksumFromVixDoesNotMatch)
        {
            this.study = study;
            this.instance = instance;
            this.imageQuality = imageQuality;
            this.transactionId = transactionId;
            this.failIfChecksumFromVixDoesNotMatch = failIfChecksumFromVixDoesNotMatch;

            mstrImageFormatSubTypeRequestString = "";
            mstrRequestedImageFormat = "";
            mstrRequestUrl = "";
        }
    }
}
