/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net;

namespace VixAKCommon.gov.va.med.imaging.vix.services
{
    public class VixTransactionLogImpl
    {

        private readonly string mstrVixUrl;
        private readonly string mstrUsername;
        private readonly string mstrPassword;

        public VixTransactionLogImpl(string vixUrl, string username, string password)
        {
            mstrVixUrl = vixUrl;
            mstrUsername = username;
            mstrPassword = password;
        }

        public void DownloadLog(string outputFilename, DateTime downloadDate)
        {
            StreamReader reader = null;
            StreamWriter writer = null;
            try
            {
                string date = downloadDate.ToString("MM/dd/yyyy");
                date = date.Replace("/", "%2F");
                StringBuilder urlBuilder = new StringBuilder();
                urlBuilder.Append("/Vix/secure/ExcelTransactionLog?startIndex=0&endIndex=100&format=text%2Fcsv&fromDate=");
                urlBuilder.Append(date);
                urlBuilder.Append("&toDate=");
                urlBuilder.Append(date);
                urlBuilder.Append("&imageQuality=unselected&user=&modality=&datasourceProtocol=unselected&errorMessage=&transactionId=&imageUrn=&forwardIteration=False&resultsPerPage=1000");
                //string url = mstrVixUrl + "/Vix/secure/ExcelTransactionLog?startIndex=0&endIndex=100&format=text%2Fcsv&fromDate=12%2F22%2F2008&toDate=12%2F22%2F2008&imageQuality=unselected&user=&modality=&datasourceProtocol=unselected&errorMessage=&transactionId=&imageUrn=&forwardIteration=False&resultsPerPage=100";
                string url = mstrVixUrl + urlBuilder.ToString();
                WebRequest request = WebRequest.Create(url);
                request.Timeout = 500000; // 500 seconds
                request.Credentials = new NetworkCredential(mstrUsername, mstrPassword);                
                WebResponse response = request.GetResponse();
                reader = new StreamReader(response.GetResponseStream());
                writer = new StreamWriter(outputFilename, false);
                string line = reader.ReadLine();
                while (line != null)
                {
                    writer.WriteLine(line);
                    line = reader.ReadLine();
                }
                reader.Close();
                writer.Close();
            }
            finally
            {
                if (reader != null)
                    reader.Close();
                if (writer != null)
                    writer.Close();
            }
        }
    }
}
