/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using VixAKCommon.gov.va.med.imaging;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using System.Xml.XPath;
using System.Xml.Xsl;
using System.IO;

namespace VixAKCommon.gov.va.med.imaging.vix.tester
{
    public class AKActionResults
    {

        private readonly string mstrFilename;
        private readonly List<ActionResult> mActionResults = new List<ActionResult>();
        private readonly string mstrActionScript;
        private readonly DateTime startTime;

        public DateTime StartTime
        {
            get { return startTime; }
        } 


        public List<ActionResult> ActionResults
        {
            get { return mActionResults; }
        } 

        public AKActionResults(string actionScript, string filename, DateTime startTime)
        {
            mstrFilename = filename;
            this.startTime = startTime;
            mstrActionScript = actionScript;
        }

        public bool AllTestsPassed
        {
            get
            {
                foreach (ActionResult result in mActionResults)
                {
                    if (result.ActionCompletedResult != ActionCompletedResult.success)
                        return false;
                }
                return true;
            }
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("Completed '" + mActionResults.Count + "' actions");
            sb.AppendLine((AllTestsPassed ? "All tests passed" : "At least one test failed"));
            foreach (ActionResult ar in mActionResults)
            {
                string description = ar.Action.TestDescription;
                if (description != null && description.Length <= 0)
                    description = null;
                sb.AppendLine(ar.Action.GetActionName() + (description == null ? "" : " [" + description + "]") + ": " + ar.ActionCompletedResult);
            }

            return sb.ToString();
        }

        public string ConvertToXslt(string xsltFilename)
        {
            XPathDocument myXPathDoc = new XPathDocument(mstrFilename);
            XslCompiledTransform myXslTrans = new XslCompiledTransform();
            //XslTransform myXslTrans = new XslTransform();
            //load the Xsl 
            myXslTrans.Load(xsltFilename);

            //create the output stream
            MemoryStream outputStream = new MemoryStream();
            
            
            //XmlTextWriter myWriter = new XmlTextWriter
            //    ("result.html", null);

            //do the actual transform of Xml
            //myXslTrans.Transform(myXPathDoc, null, myWriter);
            myXslTrans.Transform(myXPathDoc, null, outputStream);

            return System.Text.ASCIIEncoding.ASCII.GetString(outputStream.GetBuffer());            
            //myWriter.Close();
        }

        /// <summary>
        /// Save the results in a simple format
        /// </summary>
        /// <param name="filename"></param>
        public void SaveSimpleResults(string filename)
        {
            StreamWriter writer = null;
            try
            {
                writer = new StreamWriter(filename, false);
                writer.WriteLine("Tests started at " + startTime.ToString());
                int successful = 0;
                int failed = 0;
                foreach (ActionResult result in mActionResults)
                {
                    if (result.ActionCompletedResult == ActionCompletedResult.success)
                        successful++;
                    else
                        failed++;
                }
                writer.WriteLine("Executed " + mActionResults.Count + " test actions");
                writer.WriteLine("[" + successful + "] completed successfully");
                writer.WriteLine("[" + failed + "] failed to complete");
                writer.WriteLine("See '" + mstrFilename + "' for test details");
            }      
            finally
            {
                if (writer != null)
                    writer.Close();
            }

        }

        public void Save()
        {
            XmlDocument xmlDoc = new XmlDocument();
            XmlProcessingInstruction instruction =
                xmlDoc.CreateProcessingInstruction("xml-stylesheet", "type='text/xsl' href='display.xsl'");
            xmlDoc.AppendChild(instruction);   

            XmlNode rootNode = xmlDoc.CreateElement("AKActionResults");
            xmlDoc.AppendChild(rootNode);

            XmlAttribute attr = xmlDoc.CreateAttribute("TestDate");
            attr.Value = startTime.ToString();
            rootNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("AllTestsPassed");
            attr.Value = AllTestsPassed.ToString();
            rootNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("ActionScriptFilename");
            attr.Value = mstrActionScript;
            rootNode.Attributes.Append(attr);

            foreach (ActionResult actionResult in mActionResults)
            {
                BaseAction action = actionResult.Action;

                /*
                XmlNode actionResultNode = xmlDoc.CreateElement("ActionResult");
                rootNode.AppendChild(actionResultNode);
                attr = xmlDoc.CreateAttribute("ActionName");
                attr.Value = action.GetActionName();
                actionResultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("ActionCompletedResult");
                attr.Value = actionResult.ActionCompletedResult.ToString();
                actionResultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("ResultsVerified");
                attr.Value = action.ResultsVerified.ToString();
                actionResultNode.Attributes.Append(attr);
                 * */

                XmlNode actionNode = xmlDoc.CreateElement("Action");
                rootNode.AppendChild(actionNode);
                attr = xmlDoc.CreateAttribute("ActionName");
                attr.Value = action.GetActionName();
                actionNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Description");
                attr.Value = (action.TestDescription == null ? "" : action.TestDescription);
                actionNode.Attributes.Append(attr);

                XmlNode actionParametersNode = xmlDoc.CreateElement("Parameters");
                actionNode.AppendChild(actionParametersNode);
                foreach (string key in action.NecessaryProperties.Keys)
                {
                    XmlNode parameterNode = xmlDoc.CreateElement("Parameter");
                    actionParametersNode.AppendChild(parameterNode);
                    attr = xmlDoc.CreateAttribute("Name");
                    attr.Value = key;
                    parameterNode.Attributes.Append(attr);

                    object obj = action.NecessaryProperties[key];
                    attr = xmlDoc.CreateAttribute("Value");
                    attr.Value = (obj == null ? "" : obj.ToString());
                    parameterNode.Attributes.Append(attr);
                }

                XmlNode resultNode = xmlDoc.CreateElement("TestResult");
                actionNode.AppendChild(resultNode);
                attr = xmlDoc.CreateAttribute("ActionCompletedResult");
                attr.Value = actionResult.ActionCompletedResult.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("ResultsVerified");
                attr.Value = action.ResultsVerified.ToString();
                resultNode.Attributes.Append(attr);


            }
            
            xmlDoc.Save(mstrFilename);
        }
    }

    public class ActionResult
    {
        private readonly BaseAction action;
        private readonly ActionCompletedResult actionCompletedResult;   

        public BaseAction Action
        {
            get { return action; }
        } 

        public ActionCompletedResult ActionCompletedResult
        {
            get { return actionCompletedResult; }
        }

        public ActionResult(BaseAction action, ActionCompletedResult actionCompletedResult)
        {
            this.actionCompletedResult = actionCompletedResult;
            this.action = action;
        }
    }
}
