/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Xml.Serialization;

namespace VixAKCommon.gov.va.med.imaging.vix.tester
{
    /// <summary>
    /// THIS IS DEPRECATED AND NO LONGER USED!
    /// </summary>
    public class ExpectedTestResult
    {
        private List<VixPatientTestResult> results = new List<VixPatientTestResult>();
        private readonly string mstrFilename;

        public string Filename
        {
            get { return mstrFilename; }
        } 


        public List<VixPatientTestResult> Results
        {
            get { return results; }
        }

        public ExpectedTestResult(string filename, bool loadExistingResults)
        {
            mstrFilename = filename;
            if (loadExistingResults)
            {
                LoadExpectedResults();
            }
        }

        private void LoadExpectedResults()
        {
            results.Clear();
            if (File.Exists(mstrFilename))
            {                
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(mstrFilename);
                XmlNodeList resultNodes = xmlDoc.SelectNodes("/ExpectedResults/Result");
                foreach (XmlNode resultNode in resultNodes)
                {
                    int error404 = 0;
                    int error500 = 0;
                    int errorOther = 0;
                    int downloaded = 0;
                    int failed = 0;
                    int skipped = 0;
                    int studies = 0;
                    int totalImages = 0;
                    foreach (XmlAttribute attr in resultNode.Attributes)
                    {
                        switch (attr.Name)
                        {
                            case "Error404":
                                error404 = int.Parse(attr.Value);
                                break;
                            case "Error500":
                                error500 = int.Parse(attr.Value);
                                break;
                            case "ErrorOther":
                                errorOther = int.Parse(attr.Value);
                                break;
                            case "Downloaded":
                                downloaded = int.Parse(attr.Value);
                                break;
                            case "Failed":
                                failed = int.Parse(attr.Value);
                                break;
                            case "Skipped":
                                skipped = int.Parse(attr.Value);
                                break;
                            case "Studies":
                                studies = int.Parse(attr.Value);
                                break;
                            case "TotalImages":
                                totalImages = int.Parse(attr.Value);
                                break;

                        }
                    }
                    XmlNode patientNode = resultNode.SelectSingleNode("Patient");
                    if (patientNode == null)
                        throw new Exception("Could not find patient node");
                    string patientName = patientNode.Attributes["Name"].Value;
                    string icn = patientNode.Attributes["ICN"].Value;
                    VixPatientTestResult result = new VixPatientTestResult(new ImagingPatient(icn, patientName));
                    result.Error404 = error404;
                    result.Error500 = error500;
                    result.ErrorOther = errorOther;
                    result.ImagesDownloaded = downloaded;
                    result.SkippedImages = skipped;
                    result.Studies = studies;
                    result.TotalImages = totalImages;
                    result.ImagesFailed = failed;
                    results.Add(result);
                }
            }
        }

        public void Save()
        {
            XmlDocument xmlDoc = new XmlDocument();
            XmlNode rootNode = xmlDoc.CreateElement("ExpectedResults");
            xmlDoc.AppendChild(rootNode);
            foreach (VixPatientTestResult result in results)
            {
                XmlNode resultNode = xmlDoc.CreateElement("Result");
                rootNode.AppendChild(resultNode);
                XmlAttribute attr = xmlDoc.CreateAttribute("Error404");
                attr.Value = result.Error404.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Error500");
                attr.Value = result.Error500.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("ErrorOther");
                attr.Value = result.ErrorOther.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Downloaded");
                attr.Value = result.ImagesDownloaded.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Failed");
                attr.Value = result.ImagesFailed.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Skipped");
                attr.Value = result.SkippedImages.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Studies");
                attr.Value = result.Studies.ToString();
                resultNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("TotalImages");
                attr.Value = result.TotalImages.ToString();
                resultNode.Attributes.Append(attr);
                XmlNode patientNode = xmlDoc.CreateElement("Patient");
                resultNode.AppendChild(patientNode);
                attr = xmlDoc.CreateAttribute("Name");
                attr.Value = result.Patient.PatientName;
                patientNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("ICN");
                attr.Value = result.Patient.PatientIcn;
                patientNode.Attributes.Append(attr);
            }

            xmlDoc.Save(mstrFilename);
        }

        private XmlSerializer getSerializer()
        {
            return new XmlSerializer(typeof(List<VixPatientTestResult>));
        }

        /// <summary>
        /// Compares the testResult with the values in here
        /// </summary>
        /// <param name="testResults"></param>
        /// <returns></returns>
        public bool validateTestResults(VixTestResult testResults, LogInfoMessageDelegate LogMsgEvent, 
            LogErrorMessageDelegate LogErrorMsgEvent)
        {
            bool testPassed = true;
            LogMsgEvent("Verifying test results");
            if (testResults.Threaded)
            {
                // this won't work if we do threading for the active exam test - we don't know how many "patients" (exams) will be in the result when the test starts
                int expectedEntries = testResults.WorkerCount * testResults.PatientCount;
                if (testResults.Patients.Count != expectedEntries)
                {
                    LogErrorMsgEvent("Test results contains '" + testResults.Patients.Count + "' items, should have '" + expectedEntries + "'.");
                    testPassed = false;
                }
            }
            else
            {
                if (testResults.Patients.Count != this.results.Count)
                {
                    LogErrorMsgEvent("Test results contains '" + testResults.Patients.Count + "' items, should have '" + this.results.Count + "' items.");
                    testPassed = false;
                }
            }

            foreach (VixPatientTestResult patientTestResult in testResults.Patients)
            {
                VixPatientTestResult expectedResult = FindPatientExpectedResult(patientTestResult.Patient.PatientIcn);
                if (expectedResult == null)
                {
                    LogErrorMsgEvent("Could not find expected result for patient '" + patientTestResult.Patient.PatientIcn + "'.");
                    testPassed = false;
                }
                else
                {
                    string patientName = patientTestResult.Patient.ToString(true);
                    if (patientTestResult.IsExam)
                    {
                        LogMsgEvent("Comparing exam '" + patientName + "' to expected results");
                    }
                    else
                    {
                        LogMsgEvent("Comparing patient '" + patientName + "' to expected results");
                    }
                    if (expectedResult.Error404 != patientTestResult.Error404)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.Error404 + "' 404 Errors, expected '" + expectedResult.Error404 + "'");
                        testPassed = false;
                    }
                    if (expectedResult.Error500 != patientTestResult.Error500)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.Error500 + "' 500 Errors, expected '" + expectedResult.Error500 + "'");
                        testPassed = false;
                    }
                    if (expectedResult.ErrorOther != patientTestResult.ErrorOther)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.ErrorOther + "' Other Errors, expected '" + expectedResult.ErrorOther + "'");
                        testPassed = false;
                    }
                    if (expectedResult.ImagesDownloaded != patientTestResult.ImagesDownloaded)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.ImagesDownloaded + "' images downloaded, expected '" + expectedResult.ImagesDownloaded + "'");
                        testPassed = false;
                    }
                    if (expectedResult.ImagesFailed != patientTestResult.ImagesFailed)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.ImagesFailed + "' failed images, expected '" + expectedResult.ImagesFailed + "'");
                        testPassed = false;
                    }
                    if (expectedResult.SkippedImages != patientTestResult.SkippedImages)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.SkippedImages + "' skipped images, expected '" + expectedResult.SkippedImages + "'");
                        testPassed = false;
                    }
                    if (expectedResult.Studies != patientTestResult.Studies)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.Studies + "' studies, expected '" + expectedResult.Studies + "'");
                        testPassed = false;
                    }
                    if (expectedResult.TotalImages != patientTestResult.TotalImages)
                    {
                        LogErrorMsgEvent("Patient [" + patientName + "] got '" + patientTestResult.TotalImages + "' total images, expected '" + expectedResult.TotalImages + "'");
                        testPassed = false;
                    }
                    if (patientTestResult.IsExam)
                    {
                        LogMsgEvent("Done comparing exam '" + patientName + "' to expected results.");
                    }
                    else
                    {
                        LogMsgEvent("Done comparing patient '" + patientName + "' to expected results.");
                    }
                    LogMsgEvent("---------------------------------------------------");
                }
            }

            if (testPassed)
            {
                LogMsgEvent("Test results passed.");
            }
            else
            {
                LogErrorMsgEvent("Test results failed!");
            }


            return testPassed;
        }

        private VixPatientTestResult FindPatientExpectedResult(string icn)
        {
            foreach (VixPatientTestResult result in results)
            {
                if (result.Patient.PatientIcn == icn)
                {
                    return result;
                }
            }
            return null;
        }
    }
}
