/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using System.Xml;

namespace VixAKCommon.gov.va.med.imaging.vix.tester
{
    public class SiteServiceActionType : BaseCustomOption
    {
        private readonly string mstrSiteServiceActionName;
        private readonly int mintActionIdentifier;

        public int ActionIdentifier
        {
            get { return mintActionIdentifier; }
        } 

        public string SiteServiceActionName
        {
            get { return mstrSiteServiceActionName; }
        }

        public SiteServiceActionType(string actionName, int actionNumber)
        {
            mstrSiteServiceActionName = actionName;
            mintActionIdentifier = actionNumber;
        }

        public override string ToString()
        {
            return mstrSiteServiceActionName;
        }

        public static string SITE_SERVICE_ACTION_TYPE = "SiteServiceActionType";

        public override XmlNode toXml(XmlDocument xmlDoc)
        {
            XmlNode node = xmlDoc.CreateElement(SITE_SERVICE_ACTION_TYPE);
            XmlAttribute attr = xmlDoc.CreateAttribute("actionName");
            attr.Value = mstrSiteServiceActionName;
            node.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("actionNumber");
            attr.Value = mintActionIdentifier.ToString();
            node.Attributes.Append(attr);
            return node;
        }

        public static List<BaseCustomOption> fromXml(XmlNode customOptionNode)
        {
            List<BaseCustomOption> customOptions = new List<BaseCustomOption>();
            XmlNodeList siteServiceActionNodes = customOptionNode.SelectNodes(SITE_SERVICE_ACTION_TYPE);
            foreach (XmlNode siteServiceActionNode in siteServiceActionNodes)
            {
                string actionName = siteServiceActionNode.Attributes["actionName"].Value;
                int actionNumber = int.Parse(siteServiceActionNode.Attributes["actionNumber"].Value);
                SiteServiceActionType action = new SiteServiceActionType(actionName, actionNumber);
                customOptions.Add(action);
            }
            return customOptions;
        }

        public override bool Equals(object obj)
        {
            if (obj is SiteServiceActionType)
            {
                SiteServiceActionType that = (SiteServiceActionType)obj;
                return this.mintActionIdentifier == that.mintActionIdentifier;
            }
            else
            {
                return base.Equals(obj);
            }
        }

        public override int GetHashCode()
        {
            return base.GetHashCode() ^ mintActionIdentifier;
        }

        public static bool operator ==(SiteServiceActionType a, SiteServiceActionType b)
        {
            // If both are null, or both are same instance, return true.
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            // If one is null, but not both, return false.
            if (((object)a == null) || ((object)b == null))
            {
                return false;
            }
            return a.mintActionIdentifier == b.mintActionIdentifier;
        }

        public static bool operator !=(SiteServiceActionType a, SiteServiceActionType b)
        {
            return !(a == b);
        }

        public static SiteServiceActionType GET_SITE = new SiteServiceActionType("GetSite", 1);
        public static SiteServiceActionType GET_SITES = new SiteServiceActionType("GetSites", 2);
        public static SiteServiceActionType GET_ALL_SITES = new SiteServiceActionType("GetAllSites", 3);
        public static SiteServiceActionType GET_REGION = new SiteServiceActionType("GetRegion", 4);

        public static List<SiteServiceActionType> GetSiteServiceActions()
        {
            List<SiteServiceActionType> actions = new List<SiteServiceActionType>();
            actions.Add(GET_SITE);
            actions.Add(GET_SITES);
            actions.Add(GET_ALL_SITES);
            actions.Add(GET_REGION);
            return actions;
        }
    }
}
