/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using TransactionLog;
using System.Xml.Serialization;

namespace VixAKCommon.gov.va.med.imaging.vix.tester
{
    /// <summary>
    /// Holder that contains the values from the transaction log for a single entry and the relevant keys that are used to determine 
    /// equality
    /// </summary>
    [XmlRoot("TransactionLogEntry")] 
    public class VixAKTransactionLogEntry
    {

        private Dictionary<string, string> values = new Dictionary<string, string>();
        private List<string> relevantKeys = new List<string>();

        [XmlIgnore()] 
        public Dictionary<string, string> Values
        {
            get { return values; }
            set { values = value; }
        }        

        public List<string> RelevantKeys
        {
            get { return relevantKeys; }
            set { relevantKeys = value; }
        }

        [XmlArray("Values")]
        [XmlArrayItem("Value", Type = typeof(DictionaryEntry))]  
        public DictionaryEntry[] _Values
        {
            get
            {
                DictionaryEntry[] entries = new DictionaryEntry[values.Count];
                int i = 0;
                foreach (KeyValuePair<string, string> value in values)
                {
                    entries[i] = new DictionaryEntry(value.Key, value.Value);
                    i++;
                }
                return entries;
            }
            set
            {
                values.Clear();
                for (int i = 0; i < value.Length; i++)
                {
                    values.Add((string)value[i].Key, (string)value[i].Value);
                }
            }
        }

            /// <summary>
            /// necessary for serialization
            /// </summary>
            public VixAKTransactionLogEntry()
        {
        }

        public VixAKTransactionLogEntry(TransactionLog.TransactionLogEntry entry, List<string> relevantKeys)
        {
            this.values = entry.Values;
            this.relevantKeys = relevantKeys;
        }

        public VixAKTransactionLogEntry(Dictionary<string, string> values, List<string> relevantKeys)
        {
            this.values = values;
            this.relevantKeys = relevantKeys;
        }

        public override bool Equals(object obj)
        {
            if (obj is VixAKTransactionLogEntry)
            {
                VixAKTransactionLogEntry that = (VixAKTransactionLogEntry)obj;
                // number of values doesn't matter since one might be newer and have more values, only matters if relevant values are different
                if (this.relevantKeys.Count != that.relevantKeys.Count)
                    return false;
                foreach (string key in relevantKeys)
                {
                    if (!this.values.ContainsKey(key))
                        return false;
                    if (!that.values.ContainsKey(key))
                        return false;
                    string s = this.values[key];
                    string t = that.values[key];
                    if (s != t)
                        return false;
                }
                return true;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class VixAKTransactionLogEntryHolder
    {
        private List<VixAKTransactionLogEntry> entries = new List<VixAKTransactionLogEntry>();

        public List<VixAKTransactionLogEntry> Entries
        {
            get { return entries; }
            set { entries = value; }
        }

        public VixAKTransactionLogEntryHolder()
        {
        }

        public VixAKTransactionLogEntryHolder(List<VixAKTransactionLogEntry> entries)
        {
            this.entries = entries;
        }
    }
}
