/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public abstract class AbstractPassthroughAction : VerifiableTestAction
    {
        private const string PROPERTY_REMOTE_METHOD_NAME = "Remote Method Name";
        private const string PROPERTY_PARAMETERS = "Parameters";
        private const string PROPERTY_USER = "User";
        private const string PROPERTY_SITE_NUMBER = "Site Number";

        public AbstractPassthroughAction()
            : base()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());            
            SetNecessaryPropertyWithDefault(PROPERTY_REMOTE_METHOD_NAME, "MAGG GET TIMEOUT");
            SetNecessaryPropertyWithDefault(PROPERTY_PARAMETERS, "1:literal:DISPLAY");
            SetNecessaryPropertyWithDefault(PROPERTY_SITE_NUMBER, "");

            List<VixTestUser> users = VixTestUser.GetVixTestUsers();
            ActionCustomOption usersOptions = new ActionCustomOption(PROPERTY_USER, false);
            foreach (VixTestUser user in users)
            {
                usersOptions.Options.Add((BaseCustomOption)user);
            }
            this.necessaryProperties.Add(PROPERTY_USER, usersOptions);
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            VixTestUser testUser = GetSelectedUser();

            string methodName = necessaryProperties[PROPERTY_REMOTE_METHOD_NAME].ToString();
            string siteNumber = necessaryProperties[PROPERTY_SITE_NUMBER].ToString();
            string stringParameters = necessaryProperties[PROPERTY_PARAMETERS].ToString();

            //LogInfoMsg("Executing remote method [" + methodName + "] with parameters [" + stringParameters + "] to site [" + siteNumber + "]");
            //LogInfoMsg("Running test as user [" + testUser + "]");
            string result = null;
            List<PassthroughParameter> parameters = translate(stringParameters);
            LogInfoMsg("Executing remote method [" + methodName + "] with '" + parameters.Count + "' parameters");
            foreach (PassthroughParameter param in parameters)
            {
                LogInfoMsg("Parameter [" + param.ParameterIndex + "] of type [" + param.ParameterType + "] with value [" + param.ParameterValue.ValueField + "]");
            }
            LogInfoMsg("Running test as user [" + testUser + "]");
            string transactionId = Guid.NewGuid().ToString();
            try
            {
                result = ExecutePassthrough(transactionId, testUser, siteNumber, methodName, parameters);
            }
            catch (Exception ex)
            {
                throw new Exception("Transaction Id:" + transactionId + " Error:" + ex.Message);

            }
            return new VixAKTransactionLogSavableResultHolder(new StringSavableObject(result));
        }

        protected abstract string ExecutePassthrough(string transactionId, VixTestUser testUser, 
            string siteNumber, string methodName, List<PassthroughParameter> parameters);

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_REMOTE_METHOD_NAME))
                return false;
            if (!validatePropertyIncluded(PROPERTY_SITE_NUMBER))
                return false;
            if (!validatePropertyIncluded(PROPERTY_PARAMETERS))
                return false;
            try
            {
                GetSelectedUser();
            }
            catch (Exception ex)
            {
                LogErrorMsg(ex.Message);
                return false;
            }
            return true;
        }

        private VixTestUser GetSelectedUser()
        {
            return GetSelectedUser(PROPERTY_USER);
        }

        protected List<PassthroughParameter> translate(string stringParameters)
        {
            List<PassthroughParameter> parameters = new List<PassthroughParameter>();
            string[] paramss = stringParameters.Split('|');
            foreach (string param in paramss)
            {
                string[] paramPieces = param.Split(':');
                string num = paramPieces[0];
                PassthroughParameterType paramType = PassthroughParameterType.literal;
                switch (paramPieces[1])
                {
                    case "literal":
                        paramType = PassthroughParameterType.literal;
                        break;
                    case "list":
                        paramType = PassthroughParameterType.list;
                        break;
                    case "reference":
                        paramType = PassthroughParameterType.reference;
                        break;
                }
                string stringValue = param.Substring(paramPieces[0].Length + paramPieces[1].Length + 2);
                // not currently supporting list/multiples
                parameters.Add(new PassthroughParameter(num, paramType, new PassthroughParameterValue(stringValue)));
            }
            return parameters;
        }
    }
}
