/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class ClearTomcatLogsAction : BaseAction
    {
        private const string PROPERTY_TOMCAT_ROOT_DIR = "Tomcat Root Directory";

        public static string CLEAR_TOMCAT_LOGS_ACTION_NAME = "Clear Tomcat Logs";
        private const string PROPERTY_SERVICE_NAME = "Tomcat Service Name";
        private const string PROPERTY_MACHINE_NAME = "Machine Name";
        private const string PROPERTY_RESTART_SERVICE_AT_END = "Restart Tomcat After Clear";

        public ClearTomcatLogsAction()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
            SetNecessaryPropertyWithDefault(PROPERTY_TOMCAT_ROOT_DIR, "c:\\Program Files\\Apache Software Foundation\\Tomcat 6.0");
            SetNecessaryPropertyWithDefault(PROPERTY_SERVICE_NAME, "tomcat6");
            SetNecessaryPropertyWithDefault(PROPERTY_MACHINE_NAME, "");
            SetNecessaryPropertyWithDefault(PROPERTY_RESTART_SERVICE_AT_END, "true");
        }

        public override string GetActionName()
        {
            return CLEAR_TOMCAT_LOGS_ACTION_NAME;
        }

        public override void runAction()
        {
            Logger().Info(LogMsg("Running action [" + GetActionName() + "]"));
            try
            {
                string serviceName = necessaryProperties[PROPERTY_SERVICE_NAME].ToString();
                string machineName = necessaryProperties[PROPERTY_MACHINE_NAME].ToString();
                
                ServiceControlHelper serviceHelper = new ServiceControlHelper(machineName, serviceName);
                serviceHelper.OnDoApplicationEventsEvent += DoApplicationEvents;
                serviceHelper.OnLogMessageEvent += LogMsg;
                Logger().Info(LogMsg("Using service [" + serviceName + "]"));
                if (serviceHelper.IsServiceRunning)
                {
                    if (!serviceHelper.StopService())
                    {
                        ActionComplete(ActionCompletedResult.error);
                        return;
                    }
                }
                string tomcatRootDir = necessaryProperties[PROPERTY_TOMCAT_ROOT_DIR].ToString();

                Logger().Info(LogMsg("Root Tomcat dir [" + tomcatRootDir + "]"));
                string logsDir = tomcatRootDir + "\\logs";
                DirectoryInfo logDirectory = new DirectoryInfo(logsDir);
                if (!logDirectory.Exists)
                {
                    Logger().Info(LogMsg("Directory '" + logDirectory.FullName + "' does not exist"));
                    ActionComplete(ActionCompletedResult.error);
                    return;
                }                
                FileInfo[] files = logDirectory.GetFiles();
                ProgressBarMax(files.Length);
                foreach (FileInfo file in files)
                {
                    Logger().Info(LogMsg("Deleting file [" + file.FullName + "]"));
                    file.Delete();
                    IncrementProgress();
                    if (file.Exists)
                    {
                        Logger().Error(LogMsg("Did not delete file '" + file.FullName + "'"));
                        ActionComplete(ActionCompletedResult.error);
                        return;
                    }
                }
                DoApplicationEvents();
                Logger().Info(LogMsg("Tomcat log dir [" + logsDir + "] cleared"));
                if (GetRestartTomcat())
                {
                    serviceHelper.StartService();
                }
                Logger().Info(LogMsg("Action [" + GetActionName() + "] complete"));
            }
            catch (Exception ex)
            {
                Logger().Error(LogMsg(ex.Message));
                ActionComplete(ActionCompletedResult.error);
                return;
            }
            ActionComplete(mbCanRun ? ActionCompletedResult.success : ActionCompletedResult.error);
        }

        private bool GetRestartTomcat()
        {
            return bool.Parse(necessaryProperties[PROPERTY_RESTART_SERVICE_AT_END].ToString());
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!(validatePropertyIncluded(PROPERTY_TOMCAT_ROOT_DIR)))
                return false;
            if (necessaryProperties[PROPERTY_TOMCAT_ROOT_DIR] == null)
                return false;
            if (!validatePropertyIncluded(PROPERTY_SERVICE_NAME))
                return false;
            if (!validatePropertyIncluded(PROPERTY_MACHINE_NAME))
                return false;
            if (!validatePropertyIncluded(PROPERTY_RESTART_SERVICE_AT_END))
                return false;
            return true;
        }
    }
}
