/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class ClinicalDisplayGetImageInformationAction : VerifiableTestAction
    {
        public static string CLINICAL_DISPLAY_GET_IMAGE_INFORMATION_NAME = "Clinical Display Get Image Information";

        private const string PROPERTY_METADATA_URL = "MetadataUrl";
        private const string PROPERTY_IMAGE_URN = "Image URN";
        private const string PROPERTY_IMAGE_INFO_REPEAT = "Repeat Image Info Request Count";
        private const string PROPERTY_IMAGE_INFO_USER = "Image Info User";
        private const string PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION = "Clinical Display Interface Version";

        public ClinicalDisplayGetImageInformationAction()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URN, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_INFO_REPEAT, "1");
            SetNecessaryPropertyWithDefault(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION, "6");

            List<VixTestUser> users = VixTestUser.GetVixTestUsers();
            ActionCustomOption usersOptions = new ActionCustomOption(PROPERTY_IMAGE_INFO_USER, false);
            foreach (VixTestUser user in users)
            {
                usersOptions.Options.Add((BaseCustomOption)user);
            }
            this.necessaryProperties.Add(PROPERTY_IMAGE_INFO_USER, usersOptions);
        }

        public override string GetActionName()
        {
            return CLINICAL_DISPLAY_GET_IMAGE_INFORMATION_NAME;
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            int repeatCount = GetRepeatCount();

            VixTestUser testUser = GetSelectedUser(PROPERTY_IMAGE_INFO_USER);

            string interfaceVersion = "6";
            if (necessaryProperties.ContainsKey(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
            {
                interfaceVersion = necessaryProperties[PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION].ToString();
            }
            string imageUrn = necessaryProperties[PROPERTY_IMAGE_URN].ToString();

            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "].");
            LogInfoMsg("Retrieving information for image [" + imageUrn + "] using Clinical Display version [" + interfaceVersion + "]");
            LogInfoMsg("Repeating image info action [" + repeatCount + "] times.");
            LogInfoMsg("Running test as user [" + testUser + "]");
            ClinicalDisplayInterface clinicalDisplayInterface = null;
            switch (interfaceVersion)
            {
                case "2":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v2(metadataUrl, "", false, LogMsg);
                    break;
                case "3":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v3(metadataUrl, "", false, LogMsg);
                    break;
                case "4":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v4(metadataUrl, "", false, false, VixTestUser.USER126, LogMsg);
                    break;
                case "6":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v6(metadataUrl, "", false, false, VixTestUser.USER126, LogMsg);
                    break;
                default:
                    LogErrorMsg("Could not create interface for version '" + interfaceVersion + "', cannot continue");
                    break;
            }

            List<string> imageInformationList = new List<string>();

            //Study study = new Study(studyId);
            //Instance instance = new Instance(imageUrn);

            ProgressBarMax(repeatCount);
            for (int i = 0; i < repeatCount; i++)
            {
                string transactionId = Guid.NewGuid().ToString();
                try
                {
                    string imageInfo = clinicalDisplayInterface.GetImageInformation(imageUrn, transactionId, testUser);
                    imageInformationList.Add(imageInfo);
                    LogInfoMsg("Image (" + imageUrn + ") information:\n---------------------\n" + imageInfo + "\n---------------------");
                }
                catch (Exception ex)
                {
                    Series series = new Series("");
                    //String imagePrefix = VixCommon.GetInstanceInfo(study, series, instance);
                    throw new Exception(imageUrn + " Transaction Id:" + transactionId + " Error:" + ex.Message);
                    /*
                    Logger().Error(LogMsg(imagePrefix + " Transaction Id:" + transactionId + " Error:" + ex.Message));
                    //Logger().Error(ex.InnerException + "\n\n" + ex.StackTrace);
                    ActionComplete(ActionCompletedResult.error);
                    return;*/
                }
            }
            return new VixAKTransactionLogSavableResultHolder(new StringSavableObjectList(imageInformationList));
        }

        /*
        public override void runAction()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            int repeatCount = GetRepeatCount();

            ClinicalDisplayServiceImpl_v3 vixInterface = new ClinicalDisplayServiceImpl_v3(metadataUrl, "", false, LogMsg);
            VixTestUser testUser = GetSelectedUser();

            Logger().Info(LogMsg("Running action [" + GetActionName() + "]"));
            Logger().Info(LogMsg("Client workstation [" + VixCommon.GetHostname() + "]"));
            Logger().Info(LogMsg("Retrieving data from metadata url [" + vixInterface.MetadataUrl + "]."));
            Logger().Info(LogMsg("Repeating image info action [" + repeatCount + "] times."));
            LogInfoMsg("Running test as user [" + testUser + "]");

            string imageUrn = necessaryProperties[PROPERTY_IMAGE_URN].ToString();
            string[] urnPieces = imageUrn.Split('-');

            string studyId = urnPieces[2];
            string patientIcn = urnPieces[3];

            Study study = new Study(studyId);
            Instance instance = new Instance(imageUrn);

            

            ProgressBarMax(repeatCount);
            for (int i = 0; i < repeatCount; i++)
            {
                string transactionId = Guid.NewGuid().ToString();
                try
                {
                    string imageInfo = vixInterface.GetImageInformation(imageUrn, transactionId, testUser);
                    LogInfoMsg("Image (" + imageUrn + ") information:\n---------------------\n" + imageInfo + "\n---------------------");
                }
                catch (Exception ex)
                {
                    Series series = new Series("");
                    String imagePrefix = VixCommon.GetInstanceInfo(study, series, instance);
                    Logger().Error(LogMsg(imagePrefix + " Transaction Id:" + transactionId + " Error:" + ex.Message));
                    //Logger().Error(ex.InnerException + "\n\n" + ex.StackTrace);
                    ActionComplete(ActionCompletedResult.error);
                    return;
                }
            }
            ActionComplete(ActionCompletedResult.success);
        }*/

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URN))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_INFO_REPEAT))
                return false;
            if (!validatePropertyIncluded(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
                return false;
            try
            {
                GetRepeatCount();
            }
            catch (Exception ex)
            {
                LogErrorMsg(ex.Message);
                return false;
            }
            try
            {
                GetSelectedUser(PROPERTY_IMAGE_INFO_USER);
            }
            catch (Exception ex)
            {
                LogErrorMsg(ex.Message);
                return false;
            }
            return true;
        }

        private int GetRepeatCount()
        {
            return int.Parse(necessaryProperties[PROPERTY_IMAGE_INFO_REPEAT].ToString());
        }

        
    }
}
