/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class ClinicalDisplayTestAction : BaseVixTestAction
    {
        public static string CLINICAL_DISPLAY_ACTION_NAME = "Clinical Display ViX Test";

        private const string PROPERTY_ALLOW_J2K = "Allow J2K Compression";
        private const string PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION = "Clinical Display Interface Version";
        private const string PROPERTY_INCLUDE_IMAGE_SUB_TYPES = "Include Image Format Sub Types";
        

        private const string CLINICAL_DISPLAY_DEFAULT_VERSION = "6";

        public ClinicalDisplayTestAction()
            : base(CLINICAL_DISPLAY_ACTION_NAME)
        {
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_ALLOW_J2K, "true");
            SetNecessaryPropertyWithDefault(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION, CLINICAL_DISPLAY_DEFAULT_VERSION);
            SetNecessaryPropertyWithDefault(PROPERTY_INCLUDE_IMAGE_SUB_TYPES, "true");
        }

        public override string GetActionName()
        {
            return CLINICAL_DISPLAY_ACTION_NAME;
        }

        public override void runAction()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            bool allowJ2K = getAllowJ2kCompression();

            VixInterface vixInterface = null;
            bool includeSubTypes = getIncludeImageSubTypes();

            string selectedVersion = getClinicalDisplayVersion();
            switch (selectedVersion)
            {
                case "2":
                    vixInterface = new ClinicalDisplayServiceImpl_v2(metadataUrl, imageUrl, allowJ2K, LogMsg);
                    break;
                case "3":
                    vixInterface = new ClinicalDisplayServiceImpl_v3(metadataUrl, imageUrl, allowJ2K, includeSubTypes, LogMsg);
                    break;
                case "6":
                    vixInterface = new ClinicalDisplayServiceImpl_v6(metadataUrl, imageUrl, allowJ2K, includeSubTypes, VixTestUser.USER126, LogMsg);
                    break;
            }

            if (vixInterface == null)
            {
                System.Diagnostics.Debug.Assert(false, "Invalid clinical display version was selected, cannot continue");
            }
            
            
            LogInfoMsg("Allow J2K Comprssion [" + allowJ2K + "]");
            LogInfoMsg("Including image content type sub types [" + includeSubTypes + "]");
            base.runAction(vixInterface);
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!base.ValidateNecessaryProperties())
                return false;
            if (!validatePropertyIncluded(PROPERTY_ALLOW_J2K))
                return false;            
            try
            {
                getAllowJ2kCompression();
            }
            catch (Exception ex)
            {
                Logger().Error(LogMsg(ex.Message));
                return false;
            }
            try
            {
                getClinicalDisplayVersion();
            }
            catch (Exception ex)
            {
                Logger().Error(LogMsg(ex.Message));
                return false;
            }
            try
            {
                getIncludeImageSubTypes();
            }
            catch (Exception ex)
            {
                Logger().Error(LogMsg(ex.Message));
                return false;
            }
            return true;

        }
        private bool getAllowJ2kCompression()
        {
            return bool.Parse(necessaryProperties[PROPERTY_ALLOW_J2K].ToString());
        }

        private string getClinicalDisplayVersion()
        {
            if (necessaryProperties.ContainsKey(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
            {
                return necessaryProperties[PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION].ToString();
            }
            return CLINICAL_DISPLAY_DEFAULT_VERSION;            
        }

        private bool getIncludeImageSubTypes()
        {
            if (necessaryProperties.ContainsKey(PROPERTY_INCLUDE_IMAGE_SUB_TYPES))
            {
                return bool.Parse(necessaryProperties[PROPERTY_INCLUDE_IMAGE_SUB_TYPES].ToString());
            }
            return false;
        }
    }
}
