/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using System.IO;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class CompareVixResultsWithLogAction : BaseAction
    {
        public static string COMPARE_VIX_RESULT_WITH_LOG = "Compare Vix Result with Log";

        private const string PROPERTY_VIX_RESULT_FILENAME = "Vix Test Result Filename";
        private const string PROPERTY_VIX_SERVER = "Vix Server";
        private const string PROPERTY_VIX_TRANSACTION_LOG_USERNAME = "Transaction Log Username";
        private const string PROPERTY_VIX_TRANSACTION_LOG_PASSWORD = "Transaction Log Password";
        private const string PROPERTY_VIX_TRANSACTION_LOG_CSV_FILENAME = "Transaction Log CSV Filename";
        private const string PROPERTY_VIX_RESULTS_COMPARE_OUTPUT_FILENAME = "Vix Test Result Compare Filename";
        private const string PROPERTY_WRITE_COMPARE_RESULTS_FILE = "Write Compare Results File";

        public CompareVixResultsWithLogAction()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_VIX_RESULT_FILENAME, VixAKCommonUtils.getApplicationDir() + "\\testResults.csv");
            SetNecessaryPropertyWithDefault(PROPERTY_VIX_SERVER, "http://server:8080");
            SetNecessaryPropertyWithDefault(PROPERTY_VIX_TRANSACTION_LOG_USERNAME, "");
            SetNecessaryPropertyWithDefault(PROPERTY_VIX_TRANSACTION_LOG_PASSWORD, "");
            SetNecessaryPropertyWithDefault(PROPERTY_VIX_TRANSACTION_LOG_CSV_FILENAME, VixAKCommonUtils.getApplicationDir() + "\\TransactionLog.csv");
            SetNecessaryPropertyWithDefault(PROPERTY_VIX_RESULTS_COMPARE_OUTPUT_FILENAME, VixAKCommonUtils.getApplicationDir() + "\\compare.csv");
            SetNecessaryPropertyWithDefault(PROPERTY_WRITE_COMPARE_RESULTS_FILE, "false");
        }

        private DateTime downloadDate = DateTime.Now;

        public DateTime DownloadDate
        {
            get { return downloadDate; }
            set { downloadDate = value; }
        }

        public string VixServer
        {
            get { return GetVixUrl(); }
            set
            {
                if (necessaryProperties.ContainsKey(PROPERTY_VIX_SERVER))
                {
                    necessaryProperties[PROPERTY_VIX_SERVER] = value;
                }
                else
                {
                    necessaryProperties.Add(PROPERTY_VIX_SERVER, value);
                }
            }
        }

        public string VixTestResultFilename
        {
            get { return GetVixTestResultFilename(); }
            set
            {
                if (necessaryProperties.ContainsKey(PROPERTY_VIX_RESULT_FILENAME))
                {
                    necessaryProperties[PROPERTY_VIX_RESULT_FILENAME] = value;
                }
                else
                {
                    necessaryProperties.Add(PROPERTY_VIX_RESULT_FILENAME, value);
                }
            }
        }

        public string ResultsCompareOutputFilename
        {
            get { return GetResultsCompareOutputFilename(); }
            set
            {
                if (necessaryProperties.ContainsKey(PROPERTY_VIX_RESULTS_COMPARE_OUTPUT_FILENAME))
                {
                    necessaryProperties[PROPERTY_VIX_RESULTS_COMPARE_OUTPUT_FILENAME] = value;
                }
                else
                {
                    necessaryProperties.Add(PROPERTY_VIX_RESULTS_COMPARE_OUTPUT_FILENAME, value);
                }
            }
        }

        private string GetVixTestResultFilename()
        {
            return necessaryProperties[PROPERTY_VIX_RESULT_FILENAME].ToString();
        }

        private string GetResultsCompareOutputFilename()
        {
            return necessaryProperties[PROPERTY_VIX_RESULTS_COMPARE_OUTPUT_FILENAME].ToString();
        }

        public bool GetWriteCompareResultsFile()
        {
            return bool.Parse(necessaryProperties[PROPERTY_WRITE_COMPARE_RESULTS_FILE].ToString());
        }

        private string GetVixUrl()
        {
            return necessaryProperties[PROPERTY_VIX_SERVER].ToString();
        }

        private string GetUsername()
        {
            return necessaryProperties[PROPERTY_VIX_TRANSACTION_LOG_USERNAME].ToString();
        }

        private string GetPassword()
        {
            return necessaryProperties[PROPERTY_VIX_TRANSACTION_LOG_PASSWORD].ToString();
        }

        public override void runAction()
        {
            LogInfoMsg("---------------------------------------------------------");
            LogInfoMsg("Starting action " + GetActionName());
            string vixUrl = GetVixUrl();
            string username = GetUsername();
            string password = GetPassword();
            string transactionLogFilename = VixAKCommonUtils.getApplicationDir() + "\\transactionLogOutput.csv";
            VixTransactionLogImpl transactionLog = new VixTransactionLogImpl(vixUrl, username, password);
            LogInfoMsg("Downloading transaction log from [" + vixUrl + "] on date [" + downloadDate.ToString() + "]");
            transactionLog.DownloadLog(transactionLogFilename, downloadDate);
            CompareOutput(transactionLogFilename);            
        }

        private void CompareOutput(string transactionLogOutputFilename)
        {            
            string vixResultsFilename = GetVixTestResultFilename();
            Dictionary<string, TransactionLogId> transactions = readTransactionsIntoList(transactionLogOutputFilename);
            //LogInfoMsg("Found [" + transactions.Count + "] unique transaction Ids");

            List<VixRequest> vixRequests = readVixRequests(vixResultsFilename);

            string outputFilename = GetResultsCompareOutputFilename();
            StreamWriter writer = null;
            bool writeCompareOutput = GetWriteCompareResultsFile();
            if (writeCompareOutput)
            {
                writer = new StreamWriter(outputFilename);
                createCompareResultsHeader(writer);
            }

            LogInfoMsg("Comparing test results to Vix Transaction Log data");
            LogInfoMsg(vixRequests.Count + " total Vix Image Requests");

            int thumbnailCount = 0;
            int referenceCount = 0;
            int diagnosticCount = 0;
            int totalLogCount = 0;
            int errorCount = 0;
            bool shouldLog = false;
            int shouldLogCount = 0;

            foreach (VixRequest vixRequest in vixRequests)
            {
                shouldLog = false;
                if (writeCompareOutput)
                {
                    writeVixRequestHeader(writer, vixRequest);
                }
                if (vixRequest.ImageQuality == "THUMBNAIL")
                    thumbnailCount++;
                else if (vixRequest.ImageQuality == "REFERENCE")
                {
                    referenceCount++;
                    shouldLog = true;
                }
                else if (vixRequest.ImageQuality == "DIAGNOSTIC")
                {
                    diagnosticCount++;
                    shouldLog = true;
                }
                if (vixRequest.ErrorMessage.Length > 0)
                {
                    errorCount++;
                }
                else if(shouldLog)
                {
                    shouldLogCount++;
                }
                if (transactions.ContainsKey(vixRequest.TransactionId))
                {
                    TransactionLogId logId = transactions[vixRequest.TransactionId];
                    string imageQuality = "";
                    int imgCount = 0;
                    int logCount = 0;
                    foreach (TransactionLogEntry logEntry in logId.LogEntries)
                    {
                        if (writeCompareOutput)
                        {
                            writeTransactionLogEntry(writer, logEntry);
                        }
                        imageQuality = logEntry.ImageQuality;
                        if (logEntry.isImageRequest() == ImageRequestType.IMAGE)
                            imgCount++;
                        else if (logEntry.isImageRequest() == ImageRequestType.LOG_ACCESS)
                        {
                            logCount++;
                            totalLogCount++;
                        }
                    }

                    if ((imageQuality == "REFERENCE") || (imageQuality == "DIAGNOSTIC"))
                    {
                        if (logCount < imgCount)
                            LogInfoMsg("***Did not log image access properly for transaction [" + vixRequest.TransactionId + "]");
                    }

                    //LogInfoMsg("Found [" + logId.LogEntries.Count + "] transactions for id [" + vixRequest.TransactionId + "]");
                }
                else
                {
                    LogErrorMsg("Did not found transaction [" + vixRequest.TransactionId + "] in transaction log");
                }
            }
            if (writeCompareOutput)
            {
                writer.Close();
            }
            LogInfoMsg("Counted [" + thumbnailCount + "] thumbnails, [" + referenceCount + "] reference, [" + diagnosticCount + "] diagnostic images");
            LogInfoMsg("Should Log Count [" + shouldLogCount + "], logged [" + totalLogCount + "], error count [" + errorCount + "]");
            LogInfoMsg("---------------------------------------------------------");
        }

        private const string formatWriterDelimiter = ",";

        private void createCompareResultsHeader(StreamWriter writer)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("Transaction Id");
            sb.Append(formatWriterDelimiter);
            sb.Append("Image Id");
            sb.Append(formatWriterDelimiter);
            sb.Append("Patient ICN");
            writer.WriteLine(sb.ToString());

            StringBuilder sb2 = new StringBuilder();
            sb2.Append(formatWriterDelimiter);
            sb2.Append(formatWriterDelimiter);
            sb2.Append(formatWriterDelimiter);
            sb2.Append("Image Quality");
            sb2.Append(formatWriterDelimiter);
            sb2.Append("In Cache");
            sb2.Append(formatWriterDelimiter);
            sb2.Append("Datasource Protocol");
            sb2.Append(formatWriterDelimiter);
            sb2.Append("Response Code");
            sb2.Append(formatWriterDelimiter);
            sb2.Append("Query Type");
            sb2.Append(formatWriterDelimiter);
            sb2.Append("Guess Request Type");
            sb2.Append(formatWriterDelimiter);
            sb2.Append("Error Message");
            writer.WriteLine(sb2.ToString());
        }

        private void writeVixRequestHeader(StreamWriter writer, VixRequest vixRequest)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(vixRequest.TransactionId);
            sb.Append(formatWriterDelimiter);
            sb.Append(vixRequest.ImageId);
            sb.Append(formatWriterDelimiter);
            sb.Append("");//patient ICN
            writer.WriteLine(sb.ToString());
        }

        private void writeTransactionLogEntry(StreamWriter writer, TransactionLogEntry logEntry)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(formatWriterDelimiter);
            sb.Append(formatWriterDelimiter);
            sb.Append(formatWriterDelimiter);
            sb.Append(logEntry.ImageQuality);
            sb.Append(formatWriterDelimiter);
            sb.Append(logEntry.InCache);
            sb.Append(formatWriterDelimiter);
            sb.Append(logEntry.DatasourceProtocol);
            sb.Append(formatWriterDelimiter);
            sb.Append(logEntry.ResponseCode);
            sb.Append(formatWriterDelimiter);
            sb.Append(logEntry.QueryType);
            sb.Append(formatWriterDelimiter);
            sb.Append(logEntry.isImageRequest());
            sb.Append(formatWriterDelimiter);
            sb.Append(logEntry.ErrorMessage);
            writer.WriteLine(sb.ToString());
        }

        private List<VixRequest> readVixRequests(string vixResultsFilename)
        {
            StreamReader reader = null;
            List<VixRequest> requests = new List<VixRequest>();
            try
            {
                int lineCount = 0;
                reader = new StreamReader(vixResultsFilename);
                string line = reader.ReadLine();
                while (line != null)
                {
                    if (lineCount > 0)
                    {
                        requests.Add(VixRequest.parseLine(line));
                    }
                    line = reader.ReadLine();
                    lineCount++;
                }
            }
            catch (Exception ex)
            {
                LogErrorMsg(ex.Message);
            }
            finally
            {
                if (reader != null)
                    reader.Close();
            }
            return requests;
        }



        private Dictionary<string, TransactionLogId> readTransactionsIntoList(string transactionLogOutputFilename)
        {
            Dictionary<string, TransactionLogId> transactions = new Dictionary<string, TransactionLogId>();
            if (!File.Exists(transactionLogOutputFilename))
            {
                return transactions;
            }
            StreamReader reader = null;

            try
            {
                int lineCount = 0;
                reader = new StreamReader(transactionLogOutputFilename);
                string line = reader.ReadLine();
                while (line != null)
                {
                    if (lineCount > 0)
                    {
                        TransactionLogEntry logId = TransactionLogEntry.parseLine(line);
                        if (transactions.ContainsKey(logId.TransactionId))
                        {
                            TransactionLogId curEntry = transactions[logId.TransactionId];
                            curEntry.LogEntries.Add(logId);
                        }
                        else
                        {
                            TransactionLogId newEntry = new TransactionLogId(logId.TransactionId);
                            newEntry.LogEntries.Add(logId);
                            transactions.Add(newEntry.TransactionId, newEntry);
                        }
                    }
                    lineCount++;
                    line = reader.ReadLine();
                }
            }
            catch (Exception ex)
            {
                LogErrorMsg(ex.Message);
            }
            finally
            {
                if (reader != null)
                    reader.Close();
            }           

            return transactions;
        }

        public override string GetActionName()
        {
            return COMPARE_VIX_RESULT_WITH_LOG;
        }

        public override bool ValidateNecessaryProperties()
        {
            return true;
        }
    }

    class VixRequest
    {
        private readonly string mstrTransactionId;
        private readonly string mstrImageId;
        private readonly string mstrImageQuality;
        private readonly string mstrResponseCode;
        private readonly string mstrErrorMessage;

        public string ErrorMessage
        {
            get { return mstrErrorMessage; }
        } 


        public string TransactionId
        {
            get { return mstrTransactionId; }
        }

        public string ImageId
        {
            get { return mstrImageId; }
        }

        public string ImageQuality
        {
            get { return mstrImageQuality; }
        }

        public string ResponseCode
        {
            get { return mstrResponseCode; }
        }

        public VixRequest(string transactionId, string imageId, string imageQuality, 
            string responseCode, string errorMessage)
        {
            mstrTransactionId = VixAKCommonUtils.TrimValue(transactionId);
            mstrResponseCode = responseCode;
            mstrImageQuality = VixAKCommonUtils.TrimValue(imageQuality);
            mstrImageId = imageId;
            mstrErrorMessage = VixAKCommonUtils.TrimValue(errorMessage);
        }

        public static VixRequest parseLine(string line)
        {
            string[] pieces = line.Split(',');
            return new VixRequest(pieces[1], pieces[0], pieces[4], pieces[6], pieces[7]);
        }
    }

    class TransactionLogId
    {
        private List<TransactionLogEntry> logEntries = new List<TransactionLogEntry>();
        private readonly string mstrTransactionId;

        public string TransactionId
        {
            get { return mstrTransactionId; }
        }

        public List<TransactionLogEntry> LogEntries
        {
            get { return logEntries; }
        }

        public TransactionLogId(string transactionId)
        {
            mstrTransactionId = transactionId;
        }
    }

    class TransactionLogEntry
    {
        private readonly string mstrTransactionLogId;
        private readonly string mstrQueryType;
        private readonly string mstrPatientIcn;
        private readonly string mstrImageUrn;
        private readonly string mstrInCache;
        private readonly string mstrDatasourceProtocol;
        private readonly string mstrResponseCode;
        private readonly string mstrImageQuality;
        private readonly string mstrItemsReturned;
        private readonly string mstrErrorMessage;
        private readonly string mstrBytesTransferred;

        public string BytesTransferred
        {
            get { return mstrBytesTransferred; }
        } 

        public string ErrorMessage
        {
            get { return mstrErrorMessage; }
        }

        public string ItemsReturned
        {
            get { return mstrItemsReturned; }
        }

        public string ImageQuality
        {
            get { return mstrImageQuality; }
        } 

        public string PatientIcn
        {
            get { return mstrPatientIcn; }
        }

        public string ImageUrn
        {
            get { return mstrImageUrn; }
        }        

        public string InCache
        {
            get { return mstrInCache; }
        }

        public string DatasourceProtocol
        {
            get { return mstrDatasourceProtocol; }
        }

        public string ResponseCode
        {
            get { return mstrResponseCode; }
        } 

        public string TransactionId
        {
            get { return mstrTransactionLogId; }
        }

        public string QueryType
        {
            get { return mstrQueryType; }
        }

        public TransactionLogEntry(string transId, string queryType, string imageUrn, string patientIcn, 
            string inCache, string datasourceProtocol, string responseCode, string imageQuality, 
            string itemsReturned, string errorMessage, string bytesTransferred)
        {
            mstrTransactionLogId = VixAKCommonUtils.TrimValue(transId);
            mstrQueryType = queryType;
            mstrDatasourceProtocol = datasourceProtocol;
            mstrImageUrn = imageUrn;
            mstrInCache = inCache;
            mstrPatientIcn = patientIcn;
            mstrResponseCode = responseCode;
            mstrImageQuality = VixAKCommonUtils.TrimValue(imageQuality);
            mstrItemsReturned = itemsReturned;
            mstrErrorMessage = VixAKCommonUtils.TrimValue(errorMessage);
            mstrBytesTransferred = bytesTransferred;
        }        

        public static TransactionLogEntry parseLine(string line)
        {
            string[] pieces = line.Split(',');
            string transId = pieces[10];
            TransactionLogEntry logID = new TransactionLogEntry(transId, pieces[3], pieces[11], 
                pieces[2], pieces[12], pieces[17], pieces[18], pieces[6], pieces[5], 
                pieces[13], pieces[16]);

            return logID;
        }

        public ImageRequestType isImageRequest()
        {
            if ((mstrErrorMessage != null) && (mstrErrorMessage.Length > 0))
                return ImageRequestType.ERROR;

            if (mstrQueryType.Contains("image transfer"))
            {
                if ((mstrItemsReturned == null) || (mstrItemsReturned.Length <= 0))
                {
                    if (ResponseCode.Length <= 0)
                    {
                        return ImageRequestType.LOG_ACCESS;
                    }
                    if ((ResponseCode == "500") || (ResponseCode == "404"))
                    {
                        return ImageRequestType.ERROR;
                    }
                }                
                return ImageRequestType.IMAGE;
            }
            else
            {
                return ImageRequestType.UNKNOWN;
            }
        }
    }

    enum ImageRequestType
    {
        IMAGE, LOG_ACCESS, ERROR, UNKNOWN
    }
}

