/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class ExchangeTestAction : BaseVixTestAction
    {

        private const string PROPERTY_USERNAME = "Username";
        private const string PROPERTY_PASSWORD = "Password";

        public static string EXCHANGE_ACTION_NAME = "Exchange ViX Test";

        private const string PROPERTY_EXCHANGE_INTERFACE_VERSION = "Exchange Interface Version";
        private const string EXCHANGE_DEFAULT_VERSION = "2";

        public ExchangeTestAction() : base(EXCHANGE_ACTION_NAME)
        {
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_EXCHANGE_INTERFACE_VERSION, EXCHANGE_DEFAULT_VERSION);

            this.necessaryProperties.Add(PROPERTY_USERNAME, "");
            this.necessaryProperties.Add(PROPERTY_PASSWORD, "");
        }

        public override void runAction()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string username = necessaryProperties[PROPERTY_USERNAME].ToString();
            string password = necessaryProperties[PROPERTY_PASSWORD].ToString();

            VixInterface vixInterface = null;

            string selectedVersion = getExchangeVersion();
            switch (selectedVersion)
            {
                case "1":
                    vixInterface = new ExchangeServiceImpl(metadataUrl, imageUrl, username, password, LogMsg);
                    break;
                case "2":
                    vixInterface = new ExchangeServiceImpl_v2(metadataUrl, imageUrl, username, password, LogMsg);
                    break;
            }

            if (vixInterface == null)
            {
                System.Diagnostics.Debug.Assert(false, "Invalid Exchange version was selected, cannot continue");
            }
            base.runAction(vixInterface);
        }

        public override string GetActionName()
        {
            return EXCHANGE_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (base.ValidateNecessaryProperties())
            {
                if (!validatePropertyIncluded(PROPERTY_USERNAME))
                    return false;
                if (!validatePropertyIncluded(PROPERTY_PASSWORD))
                    return false;
            }
            else
            {
                return false;
            }
            return true;            
        }

        private string getExchangeVersion()
        {
            if (necessaryProperties.ContainsKey(PROPERTY_EXCHANGE_INTERFACE_VERSION))
            {
                return necessaryProperties[PROPERTY_EXCHANGE_INTERFACE_VERSION].ToString();
            }
            return EXCHANGE_DEFAULT_VERSION;
        }

        
    }
}
