/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class FakeTestAction : BaseAction
    {

        private const string FakeTestActionName = "Fake Test Action";

        private const string PROPERTY_REPEAT_COUNT = "Repeat Count";
        private const string PROPERTY_THREAD_SLEEP_MS = "Thread Sleep ms";
        private const string PROPERTY_THREADED = "Threaded";
        private const string PROPERTY_TEST_BOOLEAN = "Test Boolean";
        private const string PROPERTY_TEST_FAIL = "Test Fail";

        public FakeTestAction()
        {
            necessaryProperties.Add(PROPERTY_REPEAT_COUNT, "10");
            necessaryProperties.Add(PROPERTY_THREAD_SLEEP_MS, "1000");
            necessaryProperties.Add(PROPERTY_THREADED, "false");
            necessaryProperties.Add(PROPERTY_TEST_FAIL, "false");

            /*
             // these are not fully implemented... 
            ActionCustomOption booleanOption = new ActionCustomOption(PROPERTY_TEST_BOOLEAN, false);
            booleanOption.Options.Add(new BooleanOption(true));
            booleanOption.Options.Add(new BooleanOption(false));
            necessaryProperties.Add(PROPERTY_TEST_BOOLEAN, booleanOption);
            */
        }

        public override string GetActionName()
        {
            return FakeTestActionName;
        }

        public override void runAction()
        {
            LogInfoMsg("Running action [" + GetActionName() + "]");
            int repeatCount = int.Parse(necessaryProperties[PROPERTY_REPEAT_COUNT].ToString());
            int threadSleepMs = int.Parse(necessaryProperties[PROPERTY_THREAD_SLEEP_MS].ToString());

            ProgressBarMax(repeatCount);
            for (int i = 0; i < repeatCount; i++)
            {
                LogInfoMsg("Sleeping for [" + threadSleepMs + "] ms");
                System.Threading.Thread.Sleep(threadSleepMs);
                IncrementProgress();
            }

            bool shouldFail = GetFailTest();
            if (shouldFail)
                ActionComplete(ActionCompletedResult.error);
            else
                ActionComplete(mbCanRun ? ActionCompletedResult.success : ActionCompletedResult.error);
        }

        public override string ToString()
        {
            return base.ToString();
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_REPEAT_COUNT))
                return false;
            if (!validatePropertyIncluded(PROPERTY_THREAD_SLEEP_MS))
                return false;
            return true;
        }

        private bool GetFailTest()
        {
            if (necessaryProperties.ContainsKey(PROPERTY_TEST_FAIL))
            {
                try
                {
                    return bool.Parse(necessaryProperties[PROPERTY_TEST_FAIL].ToString());
                }
                catch (Exception ex)
                {
                    LogErrorMsg(ex.Message);
                    return false;
                }
            }
            return false;
        }
    }
}
