/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using System.IO;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class FileCopyAction : BaseCopyAction
    {
        public static string FileCopyActionName = "File Copy";

        private const string PROPERTY_REMOTE_FILENAME = "Remote Filename";

        public FileCopyAction() : base()
        {
            necessaryProperties.Add(PROPERTY_REMOTE_FILENAME, @"c:\temp\200-ref2.dcm");
            necessaryProperties[PROPERTY_CSV_RESULTS_FILE] = @"c:\temp\filecopy.csv";
        }

        public override string ToString()
        {
            return GetActionName();
        }

        protected override string getCopyActionDescription()
        {
            string repeatMsg = "";
            if (getRepeatContinuously())
            {
                repeatMsg = "repeat continuously";
            }
            else
            {
                repeatMsg = "repeat " + getRepeatCount() + " times";
            }
            return "Copying file from [" + getRemotePath() + "] to [" + getTargetDirectory() + "] " + repeatMsg;
        }

        public override ObjectCopyResult ExecuteCopy(string input, string outputDir)
        {
            FileInfo f = new FileInfo(input);
            string targetFilename = outputDir + "\\" + f.Name;
            DateTime startTime = DateTime.Now;
            File.Copy(input, targetFilename, true);
            DateTime endTime = DateTime.Now;
            TimeSpan span = endTime.Subtract(startTime);
            long ms = (long)span.TotalMilliseconds;
            return new FileCopyResult(input, targetFilename, ms, startTime.ToString(), endTime.ToString(), f.Length);
        }

        public override string GetActionName()
        {
            return FileCopyActionName;
        }

        protected override string getRemotePath()
        {
            return necessaryProperties[PROPERTY_REMOTE_FILENAME].ToString();
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!base.ValidateNecessaryProperties())
                return false;
            if (!validatePropertyIncluded(PROPERTY_REMOTE_FILENAME))
                return false;
            string filename = getRemotePath();
            if (!File.Exists(filename))
            {
                LogErrorMsg("Remote File [" + filename + "] does not exist or is not accessible");
                return false;
            }            
            return true;
        }        
    }

    class FileCopyResult : ObjectCopyResult
    {

        public FileCopyResult(string filename, string outputFilename, long ms,
            string startTime, string endTime, long fileSize)
            : base(filename, outputFilename, ms, startTime, endTime, fileSize)
        {
        }

    }
}
