/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using System.IO;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class FileSwapAction : BaseAction
    {
        public static string FileSwapActionName = "File Swap";

        private const string PROPERTY_FILENAME = "Filename";
        private const string PROPERTY_REPLACEMENT_FILENAME = "Replacement Filename";
        private const string PROPERTY_RENAME_OLD_FILE = "Rename Old File";

        private const string renamedExtension = ".renamed";

        public FileSwapAction()
        {
            necessaryProperties.Add(PROPERTY_FILENAME, "c:\\vixconfig\\ViXConfig.xml");
            necessaryProperties.Add(PROPERTY_REPLACEMENT_FILENAME, "c:\\vixconfig\\ViXConfig.noCache.xml");
            necessaryProperties.Add(PROPERTY_RENAME_OLD_FILE, "true");
        }

        public override string GetActionName()
        {
            return FileSwapActionName;
        }

        public override void runAction()
        {
            LogInfoMsg("Running action [" + GetActionName() + "]");

            string filename = GetFilename();
            string replacementFile = GetReplacementFilename();
            bool renameOldFile = GetRenameOldFile();            
            if (renameOldFile)
            {                
                string renamedFilename = filename + renamedExtension;
                LogInfoMsg("Moving old file [" + filename + "] to [" + renamedFilename + "]");
                File.Move(filename, renamedFilename);
            }
            LogInfoMsg("Copying file [" + replacementFile + "] to [" + filename + "]");
            File.Copy(replacementFile, filename, true);

            ActionComplete(ActionCompletedResult.success);
        }

        public override string ToString()
        {
            return GetActionName();
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_FILENAME))
                return false;
            if (!validatePropertyIncluded(PROPERTY_REPLACEMENT_FILENAME))
                return false;

            string filename = GetFilename();
            string replacementFile = GetReplacementFilename();
            if (!File.Exists(filename))
            {
                LogErrorMsg("File [" +filename + "] does not exist");
                return false;
            }
            if (!File.Exists(replacementFile))
            {
                LogErrorMsg("File [" + replacementFile + "] does not exist");
                return false;
            }
            try
            {
                GetRenameOldFile();
            }
            catch (Exception ex)
            {
                LogErrorMsg("Error getting [" + PROPERTY_RENAME_OLD_FILE + "]: " + ex.Message);
                return false;
            }
            return true;
        }

        private string GetFilename()
        {
            return necessaryProperties[PROPERTY_FILENAME].ToString();
        }

        private string GetReplacementFilename()
        {
            return necessaryProperties[PROPERTY_REPLACEMENT_FILENAME].ToString();
        }

        private bool GetRenameOldFile()
        {
            if (necessaryProperties.ContainsKey(PROPERTY_RENAME_OLD_FILE))
            {
                object val = necessaryProperties[PROPERTY_RENAME_OLD_FILE];
                if(val == null)
                    return false;
                if(val.ToString() == "")
                    return false;
                // will throw an exception if val is not something which can be parsed (not true or false)
                return bool.Parse(val.ToString());
            }
            else
            {
                return false;
            }
        }
    }
}
