/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using System.IO;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class FolderCopyAction : BaseCopyAction
    {

        public static string FolderCopyActionName = "Folder Copy";

        private const string PROPERTY_REMOTE_FOLDER = "Remote Folder";        
        private const string PROPERTY_INCLUDE_SUB_DIRS = "Include Sub Dirs";

        public FolderCopyAction() : base()
        {
            necessaryProperties.Add(PROPERTY_REMOTE_FOLDER, @"C:\temp\p72");
            necessaryProperties[PROPERTY_CSV_RESULTS_FILE] = @"c:\temp\foldercopy.csv";
            necessaryProperties.Add(PROPERTY_INCLUDE_SUB_DIRS, "true");
        }

        public override string  ToString()
        {
 	         return GetActionName();
        }

        protected override string getCopyActionDescription()
        {
            StringBuilder sb = new StringBuilder();
            string repeatMsg = "";
            if (getRepeatContinuously())
            {
                repeatMsg = "repeat continuously";
            }
            else
            {
                repeatMsg = "repeat " + getRepeatCount() + " times";
            }            
            sb.AppendLine("Copying folder from [" + getRemotePath() + "] to [" + 
                getTargetDirectory() + "] " + repeatMsg);
            sb.AppendLine("Including Sub Dirs [" + getIncludeSubDirs() + "]");
            return sb.ToString();             
        }

        public override string GetActionName()
        {
            return FolderCopyActionName;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!base.ValidateNecessaryProperties())
                return false;
            if (!validatePropertyIncluded(PROPERTY_REMOTE_FOLDER))
                return false;
            if (!validatePropertyIncluded(PROPERTY_INCLUDE_SUB_DIRS))
                return false;
            string folder = getRemotePath();
            if (!Directory.Exists(folder))
            {
                LogErrorMsg("Remote folder [" + folder + "] does not exist or is not accessible");
                return false;
            }
            return true;
        }

        public override ObjectCopyResult ExecuteCopy(string input, string outputDir)
        {
            DirectoryInfo dir = new DirectoryInfo(input);
            string targetFolder = outputDir + "\\" + dir.Name;
            DateTime startTime = DateTime.Now;

            long size = CopyFolder(dir, targetFolder, getIncludeSubDirs());
            DateTime endTime = DateTime.Now;
            TimeSpan span = endTime.Subtract(startTime);
            long ms = (long)span.TotalMilliseconds;
            return new FolderCopyResult(input, targetFolder, ms, startTime.ToString(), endTime.ToString(), size);
        }

        private long CopyFolder(DirectoryInfo dir, string targetDir, bool includeSubDirs)
        {
            long size = 0;
            FileInfo[] files = dir.GetFiles();
            if (!Directory.Exists(targetDir))
            {
                Directory.CreateDirectory(targetDir);
            }
            foreach (FileInfo file in files)
            {
                string target = targetDir + "\\" + file.Name;
                file.CopyTo(target, true);
                size += file.Length;
            }
            if (includeSubDirs)
            {
                DirectoryInfo[] dirs =  dir.GetDirectories();
                foreach (DirectoryInfo d in dirs)
                {
                    string target = targetDir + "\\" + d.Name;
                    size += CopyFolder(d, target, includeSubDirs);
                }
            }
            return size;
        }

        private bool getIncludeSubDirs()
        {
            return bool.Parse(necessaryProperties[PROPERTY_INCLUDE_SUB_DIRS].ToString());
        }

        protected override string getRemotePath()
        {
            return necessaryProperties[PROPERTY_REMOTE_FOLDER].ToString();
        }
    }

    class FolderCopyResult : ObjectCopyResult
    {

        public FolderCopyResult(string filename, string outputFilename, long ms,
            string startTime, string endTime, long fileSize)
            : base(filename, outputFilename, ms, startTime, endTime, fileSize)
        {
        }

    }
}
