/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix;
using log4net;
using System.ServiceProcess;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class ServiceControlHelper
    {
        private readonly string mstrMachineName;
        private readonly string mstrServiceName;
        private readonly ServiceController controller;

        public event LogMessageDelegate OnLogMessageEvent;
        public event DoApplicationEventsDelegate OnDoApplicationEventsEvent;

        private const int msToWait = 1000;

        public string MachineName
        {
            get { return mstrMachineName; }
        }

        public string ServiceName
        {
            get { return mstrServiceName; }
        }

        public ServiceControlHelper(string machineName, string serviceName)
        {
            mstrMachineName = machineName;
            mstrServiceName = serviceName;
            controller = new ServiceController(mstrServiceName);
            controller.MachineName = mstrMachineName;
        }

        private string LogMsg(string msg)
        {
            if (OnLogMessageEvent != null)
            {
                OnLogMessageEvent(msg);
            }
            return msg;
        }

        protected ILog Logger()
        {
            return LogManager.GetLogger(this.GetType().Name);
        }

        private void LogInfoMsg(string msg)
        {
            Logger().Info(LogMsg(msg));
        }

        private void LogErrorMsg(string msg)
        {
            Logger().Error(LogMsg(msg));
        }

        private void DoApplicationEvents()
        {
            if (OnDoApplicationEventsEvent != null)
            {
                OnDoApplicationEventsEvent();
            }
        }

        public bool StopService()
        {
            if (controller.Status == ServiceControllerStatus.Running)
            {
                if (!controller.CanStop)
                {
                    LogInfoMsg("Cannnot stop service");
                    return false;
                }
                LogInfoMsg("Stopping service '" + mstrServiceName + "'");
                controller.Stop();
                DoApplicationEvents();
                int count = 0;
                while ((count < 8) && (controller.Status != ServiceControllerStatus.Stopped))
                {
                    LogInfoMsg("Sleeping for " + msToWait + " ms");
                    System.Threading.Thread.Sleep(msToWait);
                    count++;
                }
            }
            DoApplicationEvents();
            return true;
        }

        public bool IsServiceRunning
        {
            get { return (controller.Status == ServiceControllerStatus.Running); }
        }

        public bool StartService()
        {
            int longMsToWait = msToWait * 10;
            LogInfoMsg("Starting service '" + mstrServiceName + "'");
            controller.Start();
            for (int i = 0; i < 5; i++)
            {
                LogInfoMsg("Sleeping for " + longMsToWait + " ms");
                System.Threading.Thread.Sleep(longMsToWait);
            }
            DoApplicationEvents();
            return true;
        }
    }
}
