/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    /// <summary>
    /// Test the full capabilities in the interface with a single action
    /// </summary>
    public class VistaRadFullTestAction : BaseVistaRadAction
    {
        public static string VISTARAD_ACTION_NAME = "VistARad Full VIX Test";

        public VistaRadFullTestAction()
            : base(VISTARAD_ACTION_NAME)
        {

        }

        public override void runAction()
        {
            LogInfoMsg("Running action [" + GetActionName() + "]");
            LogInfoMsg("Client workstation [" + GetHostname() + "]");

            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string txtUrl = necessaryProperties[PROPERTY_TEXT_URL].ToString();
            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "], image url [" + imageUrl + "]");

            VistaRadServiceImpl vistaRadService = new VistaRadServiceImpl(metadataUrl, imageUrl, txtUrl, LogMsg);

            List<ImagingPatient> patients = GetSelectedPatients();
            List<ImageQuality> imageQualities = new List<ImageQuality>();
            imageQualities.Add(ImageQuality.DIAGNOSTIC);

            string siteNumber = necessaryProperties[PROPERTY_SITE_NUMBER].ToString();
            LogInfoMsg("Site number [" + siteNumber + "]");
            LogInfoMsg("Selected Patients: ");

            foreach (ImagingPatient patient in patients)
            {
                LogInfoMsg(patient.ToString(true));
            }
            lastTestResult = new VixTestResult(false, 1, imageQualities.Count, false, false, false, patients.Count);
            ProgressBarMax(patients.Count);
            foreach (ImagingPatient patient in patients)
            {
                if (!mbCanRun)
                    break;
                testPatient(vistaRadService, patient, siteNumber, imageQualities);
                IncrementProgress();
            }
            lastTestResult.SetComplete();
            /*
            LogInfoMsg("-----------------------");
            LogInfoMsg(vixTestResult.ToString());
            ActionComplete(true);
             * */
            ProcessingComplete();
        }

        private bool testPatient(VistaRadServiceImpl vistaRadService, ImagingPatient patient, 
            string siteNumber, List<ImageQuality> imageQualities)
        {
            LogInfoMsg("Testing patient '" + patient.ToString(true) + "'");

            VixPatientTestResult patientTestResult = new VixPatientTestResult(patient);
            lastTestResult.Patients.Add(patientTestResult);

            ExamSite examSite = null;
            try
            {
                examSite = vistaRadService.getPatientExams(patient.PatientIcn, siteNumber);
                patientTestResult.Studies = examSite.Exams.Count;
            }
            catch (Exception ex)
            {
                LogErrorMsg("Error getting exam site for patient '" + patient.PatientIcn + "', " + ex.Message);
                patientTestResult.ErrorOther++;
                return false;
            }
            foreach (Exam exam in examSite.Exams)
            {
                if (!testExam(exam, vistaRadService, patientTestResult))
                {
                    return false;
                }
            }
            LogInfoMsg("Patient '" + patient.ToString(true) + "' is complete");
            return true;
        }

        public override string GetActionName()
        {
            return VISTARAD_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_TEXT_URL)) 
                return false;
            return base.ValidateNecessaryProperties();
        }
    }
}
