/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using VixAKCommon.gov.va.med.imaging.vix.exceptions;
using VixAKCommon.gov.va.med.imaging.vix.services;
using log4net;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class VixTestCommon
    {

        public static event LogMessageDelegate OnLogMessageEvent;
        protected static ILog Logger()
        {
            return LogManager.GetLogger(typeof(VixTestCommon).Name);
        }

        protected static string LogMsg(string msg)
        {
            if (OnLogMessageEvent != null)
            {
                OnLogMessageEvent(msg);
            }
            return msg;
        }

        protected static void LogInfoMsg(string msg)
        {
            Logger().Info(LogMsg(msg));
            //LogMsg(msg);
        }

        protected static void LogErrorMsg(string msg)
        {
            Logger().Error(LogMsg(msg));
        }     


        public static void VerifyExpectedChecksum(byte[] buffer, 
            ChecksumTestOptions checksumTestOptions, string calculatedChecksum, string transactionId)
        {
            if (checksumTestOptions != null)
            {
                if (!string.IsNullOrEmpty(checksumTestOptions.ExpectedChecksum))
                {
                    if (checksumTestOptions.ExpectedChecksum != calculatedChecksum)
                    {
                        TryOutputImage(checksumTestOptions, buffer, transactionId);
                        throw new ChecksumFailedException("Calculated checksum [" + calculatedChecksum + "] does not match expected checksum [" + checksumTestOptions.ExpectedChecksum + "]", 
                            checksumTestOptions.ExpectedChecksum, calculatedChecksum);
                    }
                }
            }
        }

        public static void TryOutputImage(ChecksumTestOptions checksumTestOptions, byte[] buffer, string transactionId)
        {
            if (checksumTestOptions != null)
            {
                if ((checksumTestOptions.OutputImageIfChecksumFails) && (!string.IsNullOrEmpty(checksumTestOptions.OutputImageDirectory)))
                {
                    if (!Directory.Exists(checksumTestOptions.OutputImageDirectory))
                    {
                        Directory.CreateDirectory(checksumTestOptions.OutputImageDirectory);
                    }
                    string fname = checksumTestOptions.OutputImageDirectory + "\\" + transactionId;
                    FileStream writer = null;
                    try
                    {
                        writer = new FileStream(fname, FileMode.Create, FileAccess.Write);
                        writer.Write(buffer, 0, buffer.Length);
                        writer.Close();
                    }
                    catch (Exception ex)
                    {
                        LogErrorMsg(ex.Message);
                    }
                    finally
                    {
                        if(writer != null)
                            writer.Close();
                    }
                }
            }
        }

        public static void OutputImage(byte[] buffer, string imageId, ImageQuality imageQuality,
            string transactionId, string outputDirectory)
        {
            if (outputDirectory != null && outputDirectory.Length > 0)
            {
                if (!Directory.Exists(outputDirectory))
                {
                    Directory.CreateDirectory(outputDirectory);
                }
                // this will put the images into a single directory with a real long filename...
                string fname = outputDirectory + "\\" + cleanupFilename(imageId) + "_" + 
                    imageQuality.ToString("d") + "_" + transactionId;
                FileStream writer = null;
                try
                {
                    writer = new FileStream(fname, FileMode.Create, FileAccess.Write);
                    writer.Write(buffer, 0, buffer.Length);
                    writer.Close();
                }
                catch (Exception ex)
                {
                    LogErrorMsg(ex.Message);
                }
                finally
                {
                    if(writer != null)
                        writer.Close();
                }
            }
        }

        private static string cleanupFilename(string filename)
        {
            return filename.Replace(":", "_");
        }

    }
}
