/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions
{
    public class XcaTestAction : BaseVixTestAction
    {
        public static string XCA_ACTION_NAME = "XCA ViX Test";

        private const string PROPERTY_USERNAME = "Username";
        private const string PROPERTY_PASSWORD = "Password";

        public XcaTestAction() : base(XCA_ACTION_NAME)
        {
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");

            this.necessaryProperties.Add(PROPERTY_USERNAME, "");
            this.necessaryProperties.Add(PROPERTY_PASSWORD, "");

            List<ImageQuality> imageQualities = ImageQuality.GetImageQualities();
            ActionCustomOption qualitiesOptions = new ActionCustomOption(PROPERTY_IMAGE_QUALITIES);
            qualitiesOptions.Options.Add((BaseCustomOption)ImageQuality.DIAGNOSTIC);
            if (this.necessaryProperties.ContainsKey(PROPERTY_IMAGE_QUALITIES))
            {
                this.necessaryProperties[PROPERTY_IMAGE_QUALITIES] = qualitiesOptions;
            }
            else
            {
                this.necessaryProperties.Add(PROPERTY_IMAGE_QUALITIES, qualitiesOptions);
            }
        }

        public override string GetActionName()
        {
            return XCA_ACTION_NAME;
        }

        public override void runAction()
        {
            

            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();

            string username = necessaryProperties[PROPERTY_USERNAME].ToString();
            string password = necessaryProperties[PROPERTY_PASSWORD].ToString();
            

            VixInterface vixInterface = new XcaServiceImpl(metadataUrl, imageUrl, 
                VixTestUser.USER126, username, password, LogMsg);

            if (vixInterface == null)
            {
                System.Diagnostics.Debug.Assert(false, "Did not create XCA Interface, cannot continue");
            }

            base.runAction(vixInterface);
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!base.ValidateNecessaryProperties())
                return false;
            return true;

        }
    }
}
