/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.cache
{
    public class GetDocumentCachedAction : VerifiableTestAction
    {
        public static string GET_DOCUMENT_CACHED_NAME = "Get Document Cached";

        private const string PROPERTY_DOCUMENT_CACHED_BASE_URL = "Document Cached Base URL";
        private const string PROPERTY_GAI = "Global Artifact Identifier";

        public GetDocumentCachedAction()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
            SetNecessaryPropertyWithDefault(PROPERTY_DOCUMENT_CACHED_BASE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_GAI, "");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string baseUrl = necessaryProperties[PROPERTY_DOCUMENT_CACHED_BASE_URL].ToString();
            string gai = necessaryProperties[PROPERTY_GAI].ToString();

            LogInfoMsg("Checking if document '" + gai + "' is in cache with URL '" + baseUrl + "'");

            CacheService cacheService = new CacheService(LogMsg, baseUrl);
            bool result = cacheService.IsArtifactCached(gai, null);
            LogInfoMsg("Document is " + (result ? "" : "not ") + "in cache");

            return new VixAKTransactionLogSavableResultHolder(new BooleanSavableObject(result));
        }

        public override string GetActionName()
        {
            return GET_DOCUMENT_CACHED_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_DOCUMENT_CACHED_BASE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_GAI))
                return false;

            return true;
        }
    }
}
