/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.cache
{
    public class GetImageCachedAction : VerifiableTestAction
    {
        public static string GET_IMAGE_CACHED_NAME = "Get Image Cached";

        private const string PROPERTY_IMAGE_CACHED_BASE_URL = "Image Cached Base URL";
        private const string PROPERTY_IMAGE_URN = "Image URN";
        private const string PROPERTY_ACCEPT = "Accept";

        public GetImageCachedAction()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_CACHED_BASE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URN, "");
            SetNecessaryPropertyWithDefault(PROPERTY_ACCEPT, "application/dicom;q=.70,application/dicom/image/j2k;q=.70");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string baseUrl = necessaryProperties[PROPERTY_IMAGE_CACHED_BASE_URL].ToString();
            string imageUrn = necessaryProperties[PROPERTY_IMAGE_URN].ToString();
            string accept = necessaryProperties[PROPERTY_ACCEPT].ToString();

            LogInfoMsg("Checking if image '" + imageUrn + "' with accept '" + accept + "' is in cache with URL '" + baseUrl + "'");

            CacheService cacheService = new CacheService(LogMsg, baseUrl);
            bool result = cacheService.IsArtifactCached(imageUrn, accept);
            LogInfoMsg("Image is " + (result ? "" : "not ") + "in cache");

            return new VixAKTransactionLogSavableResultHolder(new BooleanSavableObject(result));
        }

        public override string GetActionName()
        {
            return GET_IMAGE_CACHED_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_IMAGE_CACHED_BASE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URN))
                return false;
            if (!validatePropertyIncluded(PROPERTY_ACCEPT))
                return false;

            return true;
        }
    }
}
