/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.clinicaldisplay
{
    public abstract class AbstractClinicalDisplayGetImageAction : VerifiableTestAction
    {
        protected const string PROPERTY_IMAGE_URL = "ImageUrl";
        protected const string PROPERTY_IMAGE_URN = "Image URN";
        protected const string PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION = "Clinical Display Interface Version";

        public AbstractClinicalDisplayGetImageAction()
            : base()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URN, "");
            SetNecessaryPropertyWithDefault(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION, "6");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string interfaceVersion = "6";
            if (necessaryProperties.ContainsKey(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
            {
                interfaceVersion = necessaryProperties[PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION].ToString();
            }
            string imageUrn = necessaryProperties[PROPERTY_IMAGE_URN].ToString();

            ClinicalDisplayInterface clinicalDisplayInterface = null;
            LogInfoMsg("Retrieving data from image url [" + imageUrl + "] for URN [" + imageUrn + "]");
            LogInfoMsg("Using Clinical Display interface version [" + interfaceVersion + "]");
            switch (interfaceVersion)
            {
                case "2":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v2("", imageUrl, false, LogMsg);
                    break;
                case "3":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v3("", imageUrl, false, LogMsg);
                    break;
                case "4":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v4("", imageUrl, false, false, 
                        VixTestUser.USER126, LogMsg);
                    break;
                case "6":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v6("", imageUrl, false, false, 
                        VixTestUser.USER126, LogMsg);
                    break;
                default:
                    LogInfoMsg("Could not create interface for version '" + interfaceVersion + "', cannot continue");
                    break;
            }
            return GetImageResults(clinicalDisplayInterface, imageUrl, imageUrn);
        }

        protected abstract VixAKTransactionLogSavableResultHolder GetImageResults(ClinicalDisplayInterface clinicalDisplayInterface,
            string imageUrl, string imageUrn);
        

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URN))
                return false;
            
            return true;
        }
    }
}
