/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.clinicaldisplay
{
    /// <summary>
    /// This is an abstract base class for getting clinical display image info.  
    /// This includes dev fields and global nodes (not currently image info).
    /// </summary>
    public abstract class AbstractClinicalDisplayGetImageInfoAction : VerifiableTestAction
    {
        protected const string PROPERTY_METADATA_URL = "MetadataUrl";
        protected const string PROPERTY_IMAGE_URN = "Image URN";
        protected const string PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION = "Clinical Display Interface Version";
        protected const string PROPERTY_USER = "User";
        protected const string PROPERTY_SKIP_LINES = "Skip Lines";

        public AbstractClinicalDisplayGetImageInfoAction()
            : base()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URN, "");
            SetNecessaryPropertyWithDefault(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION, "6");
            SetNecessaryPropertyWithDefault(PROPERTY_SKIP_LINES, ""); 

            List<VixTestUser> users = VixTestUser.GetVixTestUsers();
            ActionCustomOption usersOptions = new ActionCustomOption(PROPERTY_USER, false);
            foreach (VixTestUser user in users)
            {
                usersOptions.Options.Add((BaseCustomOption)user);
            }
            this.necessaryProperties.Add(PROPERTY_USER, usersOptions);
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();

            VixTestUser testUser = GetSelectedUser(PROPERTY_USER);

            string interfaceVersion = "6";
            if (necessaryProperties.ContainsKey(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
            {
                interfaceVersion = necessaryProperties[PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION].ToString();
            }
            string imageUrn = necessaryProperties[PROPERTY_IMAGE_URN].ToString();
            string skipLine = necessaryProperties[PROPERTY_SKIP_LINES].ToString();
            string[] skipLines = skipLine.Split('|');

            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "].");
            LogInfoMsg("Retrieving information for image [" + imageUrn + "]skipping lines that start with [" + skipLine + "] using Clinical Display version [" + interfaceVersion + "]");
            LogInfoMsg("Running test as user [" + testUser + "]");
            ClinicalDisplayInterface clinicalDisplayInterface = null;
            switch (interfaceVersion)
            {
                case "2":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v2(metadataUrl, "", false, LogMsg);
                    break;
                case "3":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v3(metadataUrl, "", false, LogMsg);
                    break;
                case "4":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v4(metadataUrl, "", false, false, testUser, LogMsg);
                    break;
                case "6":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v6(metadataUrl, "", false, false, testUser, LogMsg);
                    break;
                default:
                    LogErrorMsg("Could not create interface for version '" + interfaceVersion + "', cannot continue");
                    break;
            }
            string transactionId = Guid.NewGuid().ToString();
            //string result = clinicalDisplayInterface.GetImageDevFields(transactionId, testUser, imageUrn, flags);
            string result = GetImageInformation(clinicalDisplayInterface, transactionId, testUser, imageUrn);

            return new VixAKTransactionLogSavableResultHolder(new StringSkipLineSavableObject(result, skipLines));
        }

        protected abstract string GetImageInformation(ClinicalDisplayInterface clinicalDisplayInterface,
            string transactionId, VixTestUser testUser, string imageUrn);

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URN))
                return false;
            if (!validatePropertyIncluded(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
                return false;
            try
            {
                GetSelectedUser(PROPERTY_USER);
            }
            catch (Exception ex)
            {
                LogErrorMsg(ex.Message);
                return false;
            }
            return true;
        }
    }
}
