/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.clinicaldisplay
{
    public class ClinicalDisplayPassthroughAction : AbstractPassthroughAction
    {
        public static string CLINICAL_DISPLAY_PASSTHROUGH_ACTION_NAME = "Clinical Display Passthrough Test";

        private const string PROPERTY_METADATA_URL = "MetadataUrl";
        private const string PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION = "Clinical Display Interface Version";        

        public ClinicalDisplayPassthroughAction()
            : base()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
                  
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION, "6");            
        }

        /*
        protected override VixAKTransactionLogSavableResultHolder  GetActionResult()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();

            VixTestUser testUser = GetSelectedUser();

            string interfaceVersion = "6";
            if (necessaryProperties.ContainsKey(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
            {
                interfaceVersion = necessaryProperties[PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION].ToString();
            }
            
            string methodName = necessaryProperties[PROPERTY_REMOTE_METHOD_NAME].ToString();
            string siteNumber = necessaryProperties[PROPERTY_SITE_NUMBER].ToString();
            string stringParameters = necessaryProperties[PROPERTY_PARAMETERS].ToString();

            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "].");
            LogInfoMsg("Executing remote method [" + methodName + "] with parameters [" + stringParameters + "] to site [" + siteNumber + "] using Clinical Display version [" + interfaceVersion + "]");
            LogInfoMsg("Running test as user [" + testUser + "]");
            ClinicalDisplayInterface clinicalDisplayInterface = null;
            switch (interfaceVersion)
            {
                case "2":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v2(metadataUrl, "", false, LogMsg);
                    break;
                case "3":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v3(metadataUrl, "", false, LogMsg);
                    break;
                case "4":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v4(metadataUrl, "", false, false, VixTestUser.USER126, LogMsg);
                    break;
                case "6":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v6(metadataUrl, "", false, false, VixTestUser.USER126, LogMsg);
                    break;
                default:
                    LogErrorMsg("Could not create interface for version '" + interfaceVersion + "', cannot continue");
                    break;
            }

            string result = null;
            List<PassthroughParameter> parameters = translate(stringParameters);
            string transactionId = Guid.NewGuid().ToString();
            try
            {
                // get parameters from request and execute method!
                result = clinicalDisplayInterface.ExecutePassthroughMethod(transactionId, testUser, siteNumber, methodName, parameters);
                
            }
            catch (Exception ex)
            {
                throw new Exception("Transaction Id:" + transactionId + " Error:" + ex.Message);
       
            }
            return new VixAKTransactionLogSavableResultHolder(new  StringSavableObject(result));
            
        }*/

        public override string  GetActionName()
        {
            return CLINICAL_DISPLAY_PASSTHROUGH_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!base.ValidateNecessaryProperties())
                return false;
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
                return false;
            return true;
        }

        protected override string ExecutePassthrough(string transactionId, 
            VixTestUser testUser, string siteNumber, string methodName, 
            List<PassthroughParameter> parameters)
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string interfaceVersion = "6";
            if (necessaryProperties.ContainsKey(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
            {
                interfaceVersion = necessaryProperties[PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION].ToString();
            }
            ClinicalDisplayInterface clinicalDisplayInterface = null;
            switch (interfaceVersion)
            {
                case "2":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v2(metadataUrl, "", false, LogMsg);
                    break;
                case "3":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v3(metadataUrl, "", false, LogMsg);
                    break;
                case "4":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v4(metadataUrl, "", false, false, VixTestUser.USER126, LogMsg);
                    break;
                case "6":
                    clinicalDisplayInterface = new ClinicalDisplayServiceImpl_v6(metadataUrl, "", false, false, VixTestUser.USER126, LogMsg);
                    break;
                default:
                    LogErrorMsg("Could not create interface for version '" + interfaceVersion + "', cannot continue");
                    break;
            }
            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "] using Clinical Display version [" + interfaceVersion + "]");

            return clinicalDisplayInterface.ExecutePassthroughMethod(transactionId, testUser, siteNumber, methodName, parameters);
        }
    }
}
