/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.services;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions.options;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.clinicaldisplay
{
    public class ClinicalDisplayPatientStudiesAction : VerifiableTestAction
    {
        public static string CLINICAL_DISPLAY_PATIENT_STUDIES_ACTION_NAME = "Clinical Display Patient Studies Test";

        private const string PROPERTY_METADATA_URL = "MetadataUrl";
        private const string PROPERTY_IMAGE_URL = "ImageUrl";
        private const string PROPERTY_SITE_NUMBER = "Site Number";

        private const string PROPERTY_PATIENT_ICN = "Patient ICN";
        private const string PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION = "Clinical Display Interface Version";
        private const string PROPERTY_AUTHORIZED_SENSITIVE_LEVEL = "Authorized Sensitive Level";
        private const string PROPERTY_INCLUDE_ARTIFACTS = "Include Artifacts";
        private const string PROPERTY_FROM_DATE = "From Date";
        private const string PROPERTY_TO_DATE = "To Date";

        public ClinicalDisplayPatientStudiesAction()
            : base()
        {
            this.LoadDefaultValues(VixAKCommonUtils.getDefaultValuesFile());
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_PATIENT_ICN, "");
            SetNecessaryPropertyWithDefault(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION, "6");
            SetNecessaryPropertyWithDefault(PROPERTY_SITE_NUMBER, "");
            SetNecessaryPropertyWithDefault(PROPERTY_AUTHORIZED_SENSITIVE_LEVEL, "2");
            SetNecessaryPropertyWithDefault(PROPERTY_INCLUDE_ARTIFACTS, "true");
            SetNecessaryPropertyWithDefault(PROPERTY_FROM_DATE, "");
            SetNecessaryPropertyWithDefault(PROPERTY_TO_DATE, "");

        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string siteNumber = necessaryProperties[PROPERTY_SITE_NUMBER].ToString();
            string patientIcn = necessaryProperties[PROPERTY_PATIENT_ICN].ToString();
            string authorizedSensitiveLevel = necessaryProperties[PROPERTY_AUTHORIZED_SENSITIVE_LEVEL].ToString();
            bool includeArtifacts = bool.Parse(necessaryProperties[PROPERTY_INCLUDE_ARTIFACTS].ToString());
            string fromDate = necessaryProperties[PROPERTY_FROM_DATE].ToString();
            string toDate = necessaryProperties[PROPERTY_TO_DATE].ToString();
            string interfaceVersion = necessaryProperties[PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION].ToString();
            ClinicalDisplayStudiesRequest request = new ClinicalDisplayStudiesRequest(siteNumber, patientIcn, fromDate, toDate, includeArtifacts, authorizedSensitiveLevel);
            String transactionId = Guid.NewGuid().ToString();
            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "], image url [" + imageUrl + "]");
            LogInfoMsg("Using Clinical Display interface version [" + interfaceVersion + "]");
            LogInfoMsg("Retrieving studies: " + request.ToString() + ", with transactionId [" + transactionId + "]");
            ClinicalDisplayInterface cdInterface = null;
            switch (interfaceVersion)
            {
                case "2":
                    cdInterface = new ClinicalDisplayServiceImpl_v2(metadataUrl, imageUrl, false, LogMsg);
                    break;
                case "3":
                    cdInterface = new ClinicalDisplayServiceImpl_v3(metadataUrl, imageUrl, false, LogMsg);
                    break;
                case "4":
                    cdInterface = new ClinicalDisplayServiceImpl_v4(metadataUrl, imageUrl, false, false, VixTestUser.USER126, LogMsg);
                    break;
                case "6":
                    cdInterface = new ClinicalDisplayServiceImpl_v6(metadataUrl, imageUrl, false, false, VixTestUser.USER126, LogMsg);
                    break;
                default:
                    LogInfoMsg("Could not create interface for version '" + interfaceVersion + "', cannot continue");
                    break;
            }

            if (cdInterface == null)
            {
                throw new Exception("Cannot find Clinical Display interface for version '" + interfaceVersion + "'");
            }
            
            return new VixAKTransactionLogSavableResultHolder( cdInterface.GetPatientStudies(request, transactionId));
        }

        public override string GetActionName()
        {
            return CLINICAL_DISPLAY_PATIENT_STUDIES_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_CLINICAL_DISPLAY_INTERFACE_VERSION))
                return false;
            if (!validatePropertyIncluded(PROPERTY_PATIENT_ICN))
                return false;
            if (!validatePropertyIncluded(PROPERTY_SITE_NUMBER))
                return false;
            if (!validatePropertyIncluded(PROPERTY_AUTHORIZED_SENSITIVE_LEVEL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_INCLUDE_ARTIFACTS))
                return false;
            if (!validatePropertyIncluded(PROPERTY_FROM_DATE))
                return false;
            if (!validatePropertyIncluded(PROPERTY_TO_DATE))
                return false;

            return true;
        }
    }
}
