/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.exchange
{
    public class ExchangeGetPatientStudies : VerifiableTestAction
    {
        public static string EXCHANGE_GET_PATIENT_STUDIES = "Exchange Get Patient Studies";

        private const string PROPERTY_METADATA_URL = "MetadataUrl";
        private const string PROPERTY_IMAGE_URL = "ImageUrl";
        private const string PROPERTY_INTERFACE_VERSION = "Interface Version";
        private const string PROPERTY_USERNAME = "Username";
        private const string PROPERTY_PASSWORD = "Password";
        private const string PROPERTY_SITE_NUMBER = "Site Number";

        private const string PROPERTY_PATIENT_ICN = "Patient ICN";

        public ExchangeGetPatientStudies()
            : base()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_PATIENT_ICN, "");
            SetNecessaryPropertyWithDefault(PROPERTY_INTERFACE_VERSION, "2");

            SetNecessaryPropertyWithDefault(PROPERTY_USERNAME, "");
            SetNecessaryPropertyWithDefault(PROPERTY_PASSWORD, "");
            SetNecessaryPropertyWithDefault(PROPERTY_SITE_NUMBER, "*");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {

            VixInterface vixInterface = null;
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string interfaceVersion = necessaryProperties[PROPERTY_INTERFACE_VERSION].ToString();
            string username = necessaryProperties[PROPERTY_USERNAME].ToString();
            string password = necessaryProperties[PROPERTY_PASSWORD].ToString();
            string patientIcn = necessaryProperties[PROPERTY_PATIENT_ICN].ToString();
            string siteNumber = necessaryProperties[PROPERTY_SITE_NUMBER].ToString();
            String transactionId = Guid.NewGuid().ToString();
            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "], image url [" + imageUrl + "]");
            LogInfoMsg("Retrieving from site [" + siteNumber + "], for patient [" + patientIcn + "], using interface version [" + interfaceVersion + "] with transactionId [" + transactionId + "]");

            switch (interfaceVersion)
            {
                case "1":
                    vixInterface = new ExchangeServiceImpl(metadataUrl, imageUrl, username, password, LogMsg);
                    break;
                case "2":
                    vixInterface = new ExchangeServiceImpl_v2(metadataUrl, imageUrl, username, password, LogMsg);
                    break;
            }
            if (vixInterface == null)
            {
                throw new Exception("Unable to create Exchange interface for version '" + interfaceVersion + "'");
            }

            List<Study> studies = vixInterface.GetPatientStudies(siteNumber, patientIcn, transactionId);

            return new VixAKTransactionLogSavableResultHolder(new StudySavableList(studies));
        }

        public override string GetActionName()
        {
            return EXCHANGE_GET_PATIENT_STUDIES;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_PATIENT_ICN))
                return false;
            if (!validatePropertyIncluded(PROPERTY_SITE_NUMBER))
                return false;
            if (!validatePropertyIncluded(PROPERTY_INTERFACE_VERSION))
                return false;
            if (!validatePropertyIncluded(PROPERTY_USERNAME))
                return false;
            if (!validatePropertyIncluded(PROPERTY_PASSWORD))
                return false;

            return true;
        }
    }
}
