/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.vistarad
{
    public class VistaRadGetActiveExams : VerifiableTestAction
    {
        public static string VISTARAD_ACTIVE_EXAMS_ACTION_NAME = "VistARad Get Active Exams";

        protected const string PROPERTY_METADATA_URL = "MetadataUrl";
        protected const string PROPERTY_IMAGE_URL = "ImageUrl";
        protected const string PROPERTY_TEXT_URL = "TextUrl";

        protected const string PROPERTY_LIST_DESCRIPTOR = "ListDescriptor";
        protected const string PROPERTY_USER_DIVISION = "UserDivision";
        protected const string PROPERTY_SITE_NUMBER = "Site Number";

        public VistaRadGetActiveExams()
            : base()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_TEXT_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_LIST_DESCRIPTOR, "9991");
            SetNecessaryPropertyWithDefault(PROPERTY_USER_DIVISION, "");
            SetNecessaryPropertyWithDefault(PROPERTY_SITE_NUMBER, "");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string textUrl = necessaryProperties[PROPERTY_TEXT_URL].ToString();
            string siteNumber = necessaryProperties[PROPERTY_SITE_NUMBER].ToString();
            string listDescriptor = necessaryProperties[PROPERTY_LIST_DESCRIPTOR].ToString();
            string userDivision = necessaryProperties[PROPERTY_USER_DIVISION].ToString();

            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "], image url [" + imageUrl + "], text url [" + textUrl + "]");
            LogInfoMsg("Retrieving from site [" + siteNumber + "], user Division [" + userDivision + "], with list descriptor [" + listDescriptor + "]");

            VistaRadServiceImpl vistaRadService = new VistaRadServiceImpl(metadataUrl, imageUrl, textUrl, LogMsg);
            List<Exam> exams = vistaRadService.getActiveExams(siteNumber, userDivision, listDescriptor);
            return new VixAKTransactionLogSavableResultHolder(new ExamSavableList(exams));
        }

        public override string GetActionName()
        {
            return VISTARAD_ACTIVE_EXAMS_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_LIST_DESCRIPTOR))
                return false;
            if (!validatePropertyIncluded(PROPERTY_SITE_NUMBER))
                return false;
            if (!validatePropertyIncluded(PROPERTY_TEXT_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_USER_DIVISION))
                return false;

            return true;
        }

        
    }
}
