/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.vistarad
{
    public class VistaRadGetExamImage : VerifiableTestAction
    {
        public static string VISTARAD_GET_EXAM_IMAGE_ACTION_NAME = "VistARad Get Exam Image";

        protected const string PROPERTY_METADATA_URL = "MetadataUrl";
        protected const string PROPERTY_IMAGE_URL = "ImageUrl";
        protected const string PROPERTY_TEXT_URL = "TextUrl";

        protected const string PROPERTY_IMAGE_ID = "Image ID";
        protected const string PROPERTY_GET_TEXT_FILE = "Get Text File";

        public VistaRadGetExamImage()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_TEXT_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_ID, "");
            SetNecessaryPropertyWithDefault(PROPERTY_GET_TEXT_FILE, "true");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string textUrl = necessaryProperties[PROPERTY_TEXT_URL].ToString();
            string imageId = necessaryProperties[PROPERTY_IMAGE_ID].ToString();
            bool getTextFile = bool.Parse(necessaryProperties[PROPERTY_GET_TEXT_FILE].ToString());

            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "], image url [" + imageUrl + "], text url [" + textUrl + "]");
            LogInfoMsg("Retrieving exam image with Id [" + imageId + "] and " + (getTextFile ? "getting" : "not getting") + " text file.");
            VistaRadServiceImpl vistaRadService = new VistaRadServiceImpl(metadataUrl, imageUrl, textUrl, LogMsg);
            ExamImage examImage = ExamImage.CreateFromImageId(imageId);
            string transactionId = Guid.NewGuid().ToString();
            VistaRadImageRequestInput input = new VistaRadImageRequestInput(examImage, transactionId);

            List<VistaRadSavableDownloadResult> results = new List<VistaRadSavableDownloadResult>();

            VistaRadDownloadResult downloadResult = vistaRadService.DownloadImage(input);
            results.Add(new VistaRadSavableDownloadResult(downloadResult));

            if (getTextFile)
            {
                LogInfoMsg("Retrieving exam image text file for Id [" + imageId + "]");
                transactionId = Guid.NewGuid().ToString();
                input = new VistaRadImageRequestInput(examImage, transactionId);
                VistaRadDownloadResult textFileDownloadResult = vistaRadService.DownloadTextFile(input);
                results.Add(new VistaRadSavableDownloadResult(textFileDownloadResult));
            }

            return new VixAKTransactionLogSavableResultHolder(new VistaRadSavableDownloadResultList(results));
        }

        public override string GetActionName()
        {
            return VISTARAD_GET_EXAM_IMAGE_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_TEXT_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_ID))
                return false;
            if (!validatePropertyIncluded(PROPERTY_GET_TEXT_FILE)) 
                return false;

            return true;
        }
    }
}
