/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.vistarad
{
    public class VistaRadGetExamImageTextFile : VerifiableTestAction
    {

        public static string VISTARAD_GET_EXAM_IMAGE_TEXT_FILE_ACTION_NAME = "VistARad Get Exam Image Text File";

        protected const string PROPERTY_TEXT_URL = "TextUrl";
        protected const string PROPERTY_IMAGE_ID = "Image ID";

        public VistaRadGetExamImageTextFile()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_TEXT_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_ID, "");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string textUrl = necessaryProperties[PROPERTY_TEXT_URL].ToString();
            string imageId = necessaryProperties[PROPERTY_IMAGE_ID].ToString();

            LogInfoMsg("Retrieving data from text url [" + textUrl + "]");
            LogInfoMsg("Retrieving exam image text file with Id [" + imageId + "]");
            VistaRadServiceImpl vistaRadService = new VistaRadServiceImpl("", "", textUrl, LogMsg);
            ExamImage examImage = ExamImage.CreateFromImageId(imageId);
            string transactionId = Guid.NewGuid().ToString();
            VistaRadImageRequestInput input = new VistaRadImageRequestInput(examImage, transactionId);

            List<VistaRadSavableDownloadResult> results = new List<VistaRadSavableDownloadResult>();

            VistaRadDownloadResult textFileDownloadResult = vistaRadService.DownloadTextFile(input);
            results.Add(new VistaRadSavableDownloadResult(textFileDownloadResult));

            return new VixAKTransactionLogSavableResultHolder(new VistaRadSavableDownloadResultList(results));
        }

        public override string GetActionName()
        {
            return VISTARAD_GET_EXAM_IMAGE_TEXT_FILE_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_TEXT_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_ID))
                return false;

            return true;
        }
    }
}
