/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.vistarad
{
    public class VistaRadGetExamImages : VerifiableTestAction
    {
        public static string VISTARAD_GET_EXAM_IMAGES_ACTION_NAME = "VistARad Get Exam Images";

        protected const string PROPERTY_METADATA_URL = "MetadataUrl";
        protected const string PROPERTY_IMAGE_URL = "ImageUrl";
        protected const string PROPERTY_TEXT_URL = "TextUrl";
        protected const string PROPERTY_EXAM_ID = "Exam ID";

        public VistaRadGetExamImages()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_METADATA_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_IMAGE_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_TEXT_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_EXAM_ID, "");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string metadataUrl = necessaryProperties[PROPERTY_METADATA_URL].ToString();
            string imageUrl = necessaryProperties[PROPERTY_IMAGE_URL].ToString();
            string textUrl = necessaryProperties[PROPERTY_TEXT_URL].ToString();
            string examId = necessaryProperties[PROPERTY_EXAM_ID].ToString();

            LogInfoMsg("Retrieving data from metadata url [" + metadataUrl + "], image url [" + imageUrl + "], text url [" + textUrl + "]");
            LogInfoMsg("Retrieving exam with Id [" + examId + "]");

            VistaRadServiceImpl vistaRadService = new VistaRadServiceImpl(metadataUrl, imageUrl, textUrl, LogMsg);
            return new VixAKTransactionLogSavableResultHolder(vistaRadService.getExamDetails(examId));

        }

        public override string GetActionName()
        {
            return VISTARAD_GET_EXAM_IMAGES_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_METADATA_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_IMAGE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_TEXT_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_EXAM_ID))
                return false;

            return true;
        }
    }
}
