/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.xca
{
    public class XcaGetDocument : VerifiableTestAction
    {
        public static string XCA_GET_DOCUMENT_ACTION_NAME = "XCA Get Document";

        //private const string PROPERTY_QUERY_URL = "MetadataUrl";
        private const string PROPERTY_RETRIEVE_URL = "RetrieveUrl";

        private const string PROPERTY_USERNAME = "Username";
        private const string PROPERTY_PASSWORD = "Password";
        private const string PROPERTY_HOME_COMMUNITY_ID = "Home Community ID";
        private const string PROPERTY_REPOSITORY_ID = "Repository ID";
        private const string PROPERTY_DOCUMENT_ID = "Document ID";

        public XcaGetDocument()
            : base()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_RETRIEVE_URL, "https://server:443/XCARespondingGateway/services/RespondingGateway_Service");
            SetNecessaryPropertyWithDefault(PROPERTY_USERNAME, "");
            SetNecessaryPropertyWithDefault(PROPERTY_PASSWORD, "");
            SetNecessaryPropertyWithDefault(PROPERTY_DOCUMENT_ID, "");
            SetNecessaryPropertyWithDefault(PROPERTY_HOME_COMMUNITY_ID, "");
            SetNecessaryPropertyWithDefault(PROPERTY_REPOSITORY_ID, "");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string retrieveUrl = necessaryProperties[PROPERTY_RETRIEVE_URL].ToString();
            string username = necessaryProperties[PROPERTY_USERNAME].ToString();
            string password = necessaryProperties[PROPERTY_PASSWORD].ToString();
            string homeCommunityId= necessaryProperties[PROPERTY_HOME_COMMUNITY_ID].ToString();
            string repositoryId = necessaryProperties[PROPERTY_REPOSITORY_ID].ToString();
            string documentId = necessaryProperties[PROPERTY_DOCUMENT_ID].ToString();
            XcaServiceImpl xcaService = new XcaServiceImpl("", retrieveUrl, VixTestUser.DOD_USER, username, password, LogMsg);
            string transactionId = Guid.NewGuid().ToString();
            LogInfoMsg("Retrieving data from retrieve url [" + retrieveUrl + "]");
            LogInfoMsg("Retrieving document with home community Id [" + homeCommunityId + "] repository Id [" + repositoryId + "], document Id [" + documentId + "], transaction Id [" + transactionId + "]");

            
            DocumentSet documentSet = new DocumentSet(homeCommunityId, repositoryId, documentId);

            ImageRequestInput input = new ImageRequestInput(documentSet, null, ImageQuality.DIAGNOSTIC, transactionId, false);

            DownloadResult downloadResult = xcaService.DownloadImage(input);


            return new VixAKTransactionLogSavableResultHolder(new SavableDownloadResult(downloadResult));
        }

        public override string GetActionName()
        {
            return XCA_GET_DOCUMENT_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_RETRIEVE_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_USERNAME))
                return false;
            if (!validatePropertyIncluded(PROPERTY_PASSWORD))
                return false;
            if (!validatePropertyIncluded(PROPERTY_DOCUMENT_ID))
                return false;
            if (!validatePropertyIncluded(PROPERTY_HOME_COMMUNITY_ID))
                return false;
            if (!validatePropertyIncluded(PROPERTY_REPOSITORY_ID))
                return false;

            return true;
        }
    }
}
