/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixAKCommon.gov.va.med.imaging.vix.tester.actions;
using VixAKCommon.gov.va.med.imaging.vix.services;

namespace VixAKCommon.gov.va.med.imaging.vix.tester.actions.xca
{
    public class XcaGetDocuments : VerifiableTestAction
    {
        public static string XCA_GET_DOCUMENTS_ACTION_NAME = "XCA Get Documents";

        private const string PROPERTY_QUERY_URL = "QueryUrl";
        private const string PROPERTY_USERNAME = "Username";
        private const string PROPERTY_PASSWORD = "Password";
        private const string PROPERTY_PATIENT_ICN = "Patient ICN";

        public XcaGetDocuments()
            : base()
        {
            SetNecessaryPropertyWithDefault(PROPERTY_QUERY_URL, "");
            SetNecessaryPropertyWithDefault(PROPERTY_USERNAME, "");
            SetNecessaryPropertyWithDefault(PROPERTY_PASSWORD, "");
            SetNecessaryPropertyWithDefault(PROPERTY_PATIENT_ICN, "");
        }

        protected override VixAKTransactionLogSavableResultHolder GetActionResult()
        {
            string queryUrl = necessaryProperties[PROPERTY_QUERY_URL].ToString();
            string username = necessaryProperties[PROPERTY_USERNAME].ToString();
            string password = necessaryProperties[PROPERTY_PASSWORD].ToString();
            string patientIcn = necessaryProperties[PROPERTY_PATIENT_ICN].ToString();
            XcaServiceImpl xcaService = new XcaServiceImpl(queryUrl, null, VixTestUser.DOD_USER, username, password, LogMsg);
            String transactionId = Guid.NewGuid().ToString();
            LogInfoMsg("Retrieving data from query url [" + queryUrl + "]");
            LogInfoMsg("Retrieving documents for patient [" + patientIcn + "] with transactionId [" + transactionId + "]");            
            List<Study> studies = xcaService.GetPatientStudies(null, patientIcn, transactionId);
            studies.Sort(new DocumentSetComparer()); // document sets need to be sorted so they are consistent

            return new VixAKTransactionLogSavableResultHolder( new StudySavableList(studies));
        }

        public override string GetActionName()
        {
            return XCA_GET_DOCUMENTS_ACTION_NAME;
        }

        public override bool ValidateNecessaryProperties()
        {
            if (!validatePropertyIncluded(PROPERTY_QUERY_URL))
                return false;
            if (!validatePropertyIncluded(PROPERTY_USERNAME))
                return false;
            if (!validatePropertyIncluded(PROPERTY_PASSWORD))
                return false;
            if (!validatePropertyIncluded(PROPERTY_PATIENT_ICN))
                return false;

            return true;
        }
    }
}
