/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net;

namespace VixCertificateTool
{
    public delegate void DoEventsDelegate();

    public class CertificateServerHelper
    {
        private static string WELCOME_URL = "/console/portal/Welcome";
        private static string SECURITY_CHECK_URL = "/console/portal/j_security_check";
        private static string SUBMIT_CERTIFICATE_URL = "/console/portal//Security/Certificate%20Authority/__ac0x3console-base0x2CertificationAuthority!1242194507|0/__pm0x3console-base0x2CertificationAuthority!1242194507|0_view?mode=processCSR-after&pkcs10certreq=";
        private static string CONFIRM_CERTIFICATE_URL = "/console/portal//Security/Certificate%20Authority/__ac0x3console-base0x2CertificationAuthority!1242194507|0/__pm0x3console-base0x2CertificationAuthority!1242194507|0_view?mode=confirmClientCert-after&pkcs10certreq=";

        public static LogMessageDelegate VerboseMessageLogger = null;

        private static void LogVerboseMessage(string msg)
        {
            if (VerboseMessageLogger != null)
            {
                VerboseMessageLogger(msg);
            }
        }

        public static bool LoginToCertificateServer(string serverUrl, string username, string password, 
            LogMessageDelegate logMsg, DoEventsDelegate doEvents, ref string sessionId)
        {
            // get session Id
            string url = serverUrl + WELCOME_URL;
            doEvents();
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.Method = "GET";

            HttpWebResponse response = (HttpWebResponse)request.GetResponse();


            sessionId = response.Headers["Set-Cookie"];
            response.Close();

            if ((sessionId == null) || (sessionId.Length <= 0))
            {
                logMsg("No session Id found from server");
                return false;
            }

            if (sessionId.Contains(";"))
            {
                string[] pieces = sessionId.Split(';');
                sessionId = pieces[0];
            }
            doEvents();

            // authenticate user
            url = serverUrl + SECURITY_CHECK_URL;

            request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.Method = "POST";
            request.Headers.Add("Cookie", sessionId);
            request.Referer = serverUrl + WELCOME_URL;
            request.ContentType = "application/x-www-form-urlencoded";



            StreamWriter writer = new StreamWriter(request.GetRequestStream());

            writer.Write("j_username=" + username + "&j_password=" + password + "&submit=Login");
            writer.Close();

            response = (HttpWebResponse)request.GetResponse();

            LogResponse("Login to Certificate Server to URL [" + url + "]", response);

            response.Close();

            if (response.ResponseUri.LocalPath == "/console/portal/j_security_check")
            {
                logMsg("Authentication failed");
                return false;
            }
            else if (response.ResponseUri.LocalPath == "/console/portal/Welcome")
            {
                logMsg("User authenticated");
                return true;
            }

            logMsg("Unknown response Uri [" + response.ResponseUri.ToString() + "]");
            doEvents();
            return false;
        }

        private static void LogResponse(string action, HttpWebResponse response)
        {
            if (response != null)
            {
                if (VerboseMessageLogger != null)
                {
                    StringBuilder sb = new StringBuilder();
                    sb.AppendLine(action);
                    StreamReader reader = new StreamReader(response.GetResponseStream());
                    string line = reader.ReadLine();
                    while (line != null)
                    {
                        sb.AppendLine(line);
                        line = reader.ReadLine();
                    }
                    LogVerboseMessage(sb.ToString());
                }
            }
        }

        public static bool SubmitCSRRequest(string serverUrl, string sessionId, string csrFilename, string csrResponseFilename,
            KeystoreParameters keystoreParameters, string validFrom, string validTo, LogMessageDelegate logMsg, 
            DoEventsDelegate doEvents, ref string certificateSerialNumber)
        {
            doEvents();
            string csrDetails = GetCsrContents(csrFilename);
            string url = serverUrl + SUBMIT_CERTIFICATE_URL;

            csrDetails = csrDetails.Replace("-----BEGIN NEW CERTIFICATE REQUEST-----", "-----BEGIN+NEW+CERTIFICATE+REQUEST-----");
            csrDetails = csrDetails.Replace("-----END NEW CERTIFICATE REQUEST-----", "-----END+NEW+CERTIFICATE+REQUEST-----");

            csrDetails = csrDetails.Replace("+", "%2B");
            csrDetails = csrDetails.Replace(" ", "%20");
            url += csrDetails;

            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.Method = "GET";
            //request.AllowAutoRedirect = true;
            request.Headers.Add("Cookie", sessionId);

            HttpWebResponse response = (HttpWebResponse)request.GetResponse();
            StreamReader reader = new StreamReader(response.GetResponseStream());
            string line = reader.ReadLine();

            string modulus = "";
            string publicExponent = "";
            certificateSerialNumber = "";
            string certificateSerialNumberSearchstring = "<input type=\"text\" name=\"sNo\" size=\"20\" maxlength=\"200\" value=\"";
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("Submit Certificate Signing Request to URL [" + url + "]");
            while (line != null)
            {
                sb.AppendLine(line);
                // need to get the modulus and the public exponent
                if (line.Contains("modulus: "))
                {
                    int loc = line.IndexOf("modulus: ");
                    if (loc >= 0)
                    {
                        modulus = line.Substring(loc + 9);
                        modulus = modulus.Trim();
                    }
                }
                if (line.Contains("public exponent: "))
                {
                    if (line.EndsWith("</pre>"))
                    {
                        int loc = line.IndexOf("public exponent: ");
                        if (loc >= 0)
                        {
                            publicExponent = line.Substring(loc + 17);
                            publicExponent = publicExponent.Replace("</pre>", "");
                            publicExponent = publicExponent.Trim();
                        }
                    }
                }
                if (line.Contains(certificateSerialNumberSearchstring))
                {
                    int loc = line.IndexOf(certificateSerialNumberSearchstring);
                    if (loc >= 0)
                    {
                        certificateSerialNumber = line.Substring(loc + certificateSerialNumberSearchstring.Length);
                        certificateSerialNumber = certificateSerialNumber.Replace("\" READONLY/>", "");
                        certificateSerialNumber = certificateSerialNumber.Trim();
                    }
                }
                line = reader.ReadLine();
            }
            response.Close();
            LogVerboseMessage(sb.ToString());
            doEvents();

            if (modulus.Length <= 0)
            {
                logMsg("Modulus not found");
                return false;
            }
            if (publicExponent.Length <= 0)
            {
                logMsg("Public Exponent not found");
                return false;
            }
            if (certificateSerialNumber.Length <= 0)
            {
                logMsg("Certificate Serial Number not found");
                return false;
            }

            

            url = serverUrl + CONFIRM_CERTIFICATE_URL;
            url += GetCsrContents(csrFilename);
            url += "requestId=&subject=C%3D" + keystoreParameters.Country +
                "%2CST%3D" + keystoreParameters.State +
                "%2CL%3D" + keystoreParameters.City +
                "%2CO%3D" + keystoreParameters.Organization +
                "%2COU%3D" + keystoreParameters.OrganizationalUnit +
                "%2CCN%3D" + keystoreParameters.ServerName +
                "&publickey=Sun+RSA+public+key%2C+1024+bits%0D%0A++modulus%3A+" + modulus + "%0D%0A++public+exponent%3A+" + publicExponent +
                "&sNo=" + certificateSerialNumber + "&validFrom=" + formatDate(validFrom) + "&validTo=" + formatDate(validTo)+ "&algorithm=SHA1withRSA";

            // submit the request
            request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.Method = "GET";
            //request.AllowAutoRedirect = true;
            request.Headers.Add("Cookie", sessionId);

            response = (HttpWebResponse)request.GetResponse();
            reader = new StreamReader(response.GetResponseStream());
            line = reader.ReadLine();

            string findCertificateResponseString = "<form><textarea rows=\"15\" cols=\"80\" READONLY>";
            bool foundCert = false;
            // extract the certificate response
            StringBuilder certificateResponse = new StringBuilder();
            StringBuilder serverResponse = new StringBuilder();
            serverResponse.AppendLine("Confirm Certificate Request to URL [" + url + "]");
            while (line != null)
            {
                serverResponse.AppendLine(line);
                if (line.Contains(findCertificateResponseString))
                {
                    int loc = line.IndexOf(findCertificateResponseString);
                    if (loc >= 0)
                    {
                        foundCert = true;
                        string l = line.Substring(loc + findCertificateResponseString.Length);
                        certificateResponse.AppendLine(l);
                    }
                }
                else
                {
                    if (foundCert)
                    {
                        certificateResponse.AppendLine(line);
                        if (line == "-----END CERTIFICATE-----")
                            foundCert = false;
                    }
                }
                line = reader.ReadLine();
            }
            response.Close();
            LogVerboseMessage(serverResponse.ToString());
            if (certificateResponse.Length <= 0)
            {
                logMsg("Did not receive certificate response");
                return false;
            }
            StreamWriter writer = null;
            try
            {
                writer = new StreamWriter(csrResponseFilename);
                writer.Write(certificateResponse.ToString());
            }
            finally
            {
                if (writer != null)
                    writer.Close();
            }
            logMsg("Certificate response saved to file [" + csrResponseFilename + "]");
            return true;


        }

        private static string formatDate(string date)
        {
            return date.Replace("/", "%2F");
        }

        private static string GetCsrContents(string csrFilename)
        {
            StreamReader csrreader = null;
            try
            {
                StringBuilder csr = new StringBuilder();
                csrreader = new StreamReader(csrFilename);
                string line = csrreader.ReadLine();
                while (line != null)
                {
                    csr.Append(line);
                    csr.Append("%0D%0A");
                    line = csrreader.ReadLine();
                }
                csrreader.Close();
                string csrString = csr.ToString();

                csrString = csrString.Replace("-----BEGIN NEW CERTIFICATE REQUEST-----", "-----BEGIN+NEW+CERTIFICATE+REQUEST-----");
                csrString = csrString.Replace("-----END NEW CERTIFICATE REQUEST-----", "-----END+NEW+CERTIFICATE+REQUEST-----");

                csrString = csrString.Replace("+", "%2B");
                csrString = csrString.Replace(" ", "%20");
                return csrString;
            }
            finally
            {
                if (csrreader != null)
                    csrreader.Close();
            }
        }
    }
}
