/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixCertificateTool
{
    public class KeystoreParameters
    {
        public static string KEYSTORE_DEFAULT_ORGANIZATIONAL_UNIT = "";
        public static string KEYSTORE_DEFAULT_ORGANIZATION = "";
        public static string KEYSTORE_DEFAULT_CITY = "";
        public static string KEYSTORE_DEFAULT_STATE = "";
        public static string KEYSTORE_DEFAULT_COUNTRY_CODE = "";

        public static string KEYSTORE_DEFAULT_ALIAS = "";
        public static string KEYSTORE_DEFAULT_PASSWORD = "";

        private readonly string mstrServerName;
        private readonly string mstrOrganizationalUnit;
        private readonly string mstrOrganization;
        private readonly string mstrCity;
        private readonly string mstrState;
        private readonly string mstrCountry;
        private readonly string mstrPassword;
        private readonly string mstrAlias;

        public string Password
        {
            get { return mstrPassword; }
        }

        public string Alias
        {
            get { return mstrAlias; }
        }

        public string ServerName
        {
            get { return mstrServerName; }
        }

        public string OrganizationalUnit
        {
            get { return mstrOrganizationalUnit; }
        }

        public string Organization
        {
            get { return mstrOrganization; }
        }

        public string City
        {
            get { return mstrCity; }
        }

        public string State
        {
            get { return mstrState; }
        }

        public string Country
        {
            get { return mstrCountry; }
        }

        private KeystoreParameters(string alias, string password, string serverName, string organizationalUnit, string organization, string city, string state, string country)
        {
            this.mstrAlias = alias;
            this.mstrPassword = password;
            this.mstrCity = city;
            this.mstrCountry = country;
            this.mstrOrganization = organization;
            this.mstrOrganizationalUnit = organizationalUnit;
            this.mstrServerName = serverName;
            this.mstrState = state;
        }

        public static KeystoreParameters CreateKeystore(string alias, string password, string serverName, string organizationalUnit, string organization, string city, string state, string country)
        {
            return new KeystoreParameters(alias, password,
                serverName, organizationalUnit, organization,
                city, state, country);
        }

        public static KeystoreParameters CreateDefaultKeystore(string serverName)
        {
            return new KeystoreParameters(KEYSTORE_DEFAULT_ALIAS, KEYSTORE_DEFAULT_PASSWORD,
                serverName, KEYSTORE_DEFAULT_ORGANIZATIONAL_UNIT, KEYSTORE_DEFAULT_ORGANIZATION,
                KEYSTORE_DEFAULT_CITY, KEYSTORE_DEFAULT_STATE, KEYSTORE_DEFAULT_COUNTRY_CODE);
        }

        public static KeystoreParameters CreateDefaultKeystore(string serverName, string password)
        {
            return new KeystoreParameters(KEYSTORE_DEFAULT_ALIAS, password,
                serverName, KEYSTORE_DEFAULT_ORGANIZATIONAL_UNIT, KEYSTORE_DEFAULT_ORGANIZATION,
                KEYSTORE_DEFAULT_CITY, KEYSTORE_DEFAULT_STATE, KEYSTORE_DEFAULT_COUNTRY_CODE);
        }

        public override string ToString()
        {
            return mstrServerName;
        }

        public string convertToGenkeyArguments(string keystoreFilename)
        {
            StringBuilder arguments = new StringBuilder();
            arguments.Append("-genkey ");
            arguments.Append("-alias ");
            arguments.Append(mstrAlias);
            arguments.Append(" -keyalg RSA ");
            arguments.Append(" -keystore \"");
            arguments.Append(keystoreFilename);
            arguments.Append("\" ");
            arguments.Append("-storepass ");
            arguments.Append(mstrPassword);
            arguments.Append(" -keypass ");
            arguments.Append(mstrPassword);
            arguments.Append(" -dname \"");
            arguments.Append("cn=");
            arguments.Append(mstrServerName);
            arguments.Append(", ou=");
            arguments.Append(mstrOrganizationalUnit);
            arguments.Append(", o=");
            arguments.Append(mstrOrganization);
            arguments.Append(", l=");
            arguments.Append(mstrCity);
            arguments.Append(", s=");
            arguments.Append(mstrState);
            arguments.Append(", c=");
            arguments.Append(mstrCountry);
            arguments.Append("\"");
            return arguments.ToString();
        }
    }
}
