/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using log4net;
using VixCertificateTool.Properties;


namespace VixCertificateTool
{
    public partial class frmMain : Form
    {
        private string outputDir = null;

        protected ILog Logger()
        {
            return LogManager.GetLogger(this.GetType().Name);
        }

        public frmMain()
        {
            InitializeComponent();
        }

        private void VerboseMsg(string msg)
        {
            Logger().Debug(msg);
        }

        private void LogMsg(string msg)
        {
            txtLog.AppendText(msg + Environment.NewLine);
            Logger().Info(msg);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            
        }

        private void btnBrowseKeystore_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Title = "Select Keystore Filename";
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtKeystoreFilename.Text = saveFileDialog1.FileName;
            }
        }

        private KeystoreParameters keystoreParameters = null;

        private void btnGenerateKeystore_Click(object sender, EventArgs e)
        {
            CreateKeystore(false);
        }

        private bool CreateKeystore(bool silent)
        {
            Utils.ClearDirectory(outputDir);
            keystoreParameters = KeystoreParameters.CreateDefaultKeystore(txtKeystoreServerName.Text, 
                txtKeystorePassword.Text);
            string filename = txtKeystoreFilename.Text;
            if(CertificateHelper.CreateKeystore(filename, keystoreParameters, LogMsg))
            {
                LogMsg("Keystore [" + filename + "] created");
                txtCSRKeystoreFile.Text = filename;
                txtRootKeystoreFilename.Text = filename;
                txtCSRKeystorePassword.Text = txtKeystorePassword.Text;
                txtRootKeystorePassword.Text = txtKeystorePassword.Text;
                txtResponseKeystoreFilename.Text = filename;
                txtResponseKeystorePassword.Text = txtKeystorePassword.Text;
                if (!silent)
                {
                    MessageBox.Show("Keystore [" + filename + "] created", "Keystore Created",
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                txtCSRFilename.Focus();
                chkRootCertEnabled.Checked = true;
                btnImportRootCertificate.Focus();
                return true;
            }
            else
            {
                MessageBox.Show("Error creating keystore, see log", "Error", MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
                return false;
            }
        }

        private void btnBrowseCSRKeystoreFile_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = "Select Keystore Filename";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtCSRKeystoreFile.Text = openFileDialog1.FileName;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Title = "Select CSR Filename";
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtCSRFilename.Text = saveFileDialog1.FileName;
            }
        }

        private void btnCreateCSR_Click(object sender, EventArgs e)
        {
            CreateCSR(false);
        }

        private bool CreateCSR(bool silent)
        {
            if (CertificateHelper.CreateCSR(txtCSRKeystoreFile.Text,
                   txtCSRFilename.Text, KeystoreParameters.KEYSTORE_DEFAULT_ALIAS, txtCSRKeystorePassword.Text, LogMsg))
            {
                LogMsg("CSR Request File [" + txtCSRFilename.Text + "] created");
                //MessageBox.Show("CSR Request Created" + Environment.NewLine +
                //"Take the CSR to the Certificate Authority to get the Certificate Response", "CSR Created", MessageBoxButtons.OK, MessageBoxIcon.Information);
                if (!silent)
                {
                    MessageBox.Show("CSR Request Created", "CSR Created", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                txtCARequest.Text = txtCSRFilename.Text;
                chkCAEnabled.Checked = true;
                btnSubmitCSR.Focus();
                return true;
            }
            else
            {
                MessageBox.Show("Error creating CSR, see log", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private void btnRootBrowseKeystore_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = "Select Keystore Filename";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtRootKeystoreFilename.Text = openFileDialog1.FileName;
            }
        }

        private void btnRootCertificateBrowse_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = "Select Root Certificate Filename";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtRootCertificate.Text = openFileDialog1.FileName;
            }
        }

        private void btnResponseKeystoreBrowse_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = "Select Keystore Filename";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtResponseKeystoreFilename.Text = openFileDialog1.FileName;
            }
        }

        private void btnResponseBrowseCertificateResponse_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = "Select Certificate Response Filename";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtResponseCertificateFilename.Text = openFileDialog1.FileName;
            }
        }

        private void btnImportCertificateResponse_Click(object sender, EventArgs e)
        {
            ImportCSResponse(false);
        }

        private bool ImportCSResponse(bool silent)
        {
            if (CertificateHelper.ImportCertificateResponseIntoKeystore(txtResponseKeystoreFilename.Text,
                   txtResponseCertificateFilename.Text, txtResponseKeystorePassword.Text,
                   KeystoreParameters.KEYSTORE_DEFAULT_ALIAS, LogMsg))
            {
                LogMsg("Certificate Response Imported into keystore");
                if (!silent)
                {
                    MessageBox.Show("Server Keystore is complete");
                }
                return true;
            }
            else
            {
                MessageBox.Show("Error importing certificate response, see log", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private void btnImportRootCertificate_Click(object sender, EventArgs e)
        {
            ImportRootCertificate(false);
        }

        private bool ImportRootCertificate(bool silent)
        {
            if (CertificateHelper.ImportRootCertIntoKeystore(txtRootKeystoreFilename.Text, 
                txtRootCertificate.Text, txtRootKeystorePassword.Text, LogMsg))
            {
                LogMsg("Root certificate imported into keystore");
                chkCSREnabled.Checked = true;
                if (!silent)
                {
                    MessageBox.Show("Root certificate imported into keystore", "Root Certificate Imported",
                       MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                btnCreateCSR.Focus();
                return true;
            }
            else
            {
                MessageBox.Show("Error importing root certificate into keystore, see log", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private void btnLaunchCA_Click(object sender, EventArgs e)
        {
            frmNavigator navigator = new frmNavigator();

            navigator.ShowNavigator(Properties.Settings.Default.CertificateAuthorityServer);
        }

        private void DoEvents()
        {
            Application.DoEvents();
        }

        private void button3_Click(object sender, EventArgs e)
        {

            MessageBox.Show(dtValidFrom.Value.ToString("MM/dd/yyyy"));
        }

        private void btnCARequestBrowse_Click(object sender, EventArgs e)
        {

        }

        private void btnSubmitCSR_Click(object sender, EventArgs e)
        {         
            SubmitCsr(false, chkLogVerboseResponse.Checked);
        }

        private bool SubmitCsr(bool silent, bool verboseMessage)
        {
            string sessionId = "";
            string certificateSerialNumber = "";
            if (verboseMessage)
            {
                CertificateServerHelper.VerboseMessageLogger = VerboseMsg;
            }
            else
            {
                CertificateServerHelper.VerboseMessageLogger = null;
            }
            if (CertificateServerHelper.LoginToCertificateServer(Settings.Default.CertificateAuthorityServer,
                Settings.Default.CertificateAuthorityUsername, Settings.Default.CertificateAuthorityPassword,
                LogMsg, DoEvents, ref sessionId))
            {
                //KeystoreParameters keystoreParameters = KeystoreParameters.CreateDefaultKeystore(keystoreParameters.ServerName);
                if (CertificateServerHelper.SubmitCSRRequest(Settings.Default.CertificateAuthorityServer, sessionId,
                    txtCARequest.Text, txtCAResponse.Text,
                    keystoreParameters, dtValidFrom.Value.ToString("MM/dd/yyyy"),
                    dtValidTo.Value.ToString("MM/dd/yyyy"), LogMsg, DoEvents, ref certificateSerialNumber))
                {
                    txtResponseCertificateFilename.Text = txtCAResponse.Text;
                    if (!silent)
                    {
                        MessageBox.Show("Certificate Response receieved, certificate Serial Number: " + certificateSerialNumber,
                            "Received Response from CA", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    chkResponseEnabled.Checked = true;
                    btnImportCertificateResponse.Focus();
                    return true;
                }
                else
                {
                    MessageBox.Show("Error getting CSR response from CA, see log", "Error",
                       MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
            }
            else
            {
                MessageBox.Show("Error logging into Certificate Authority", "Authentication Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private void Reset()
        {
            txtKeystoreFilename.Text = outputDir + "\\federation.keystore";
            txtKeystorePassword.Text = Properties.Settings.Default.DefaultKeystorePassword;
            txtCSRFilename.Text = outputDir + "\\certreq.csr";
            txtCAResponse.Text = outputDir + "\\certreq.resp";
            dtValidFrom.Value = DateTime.Now;
            dtValidTo.Value = new DateTime(2025, 12, 31);

            txtRootKeystoreFilename.Text = "";
            txtRootKeystorePassword.Text = "";
            txtCSRKeystoreFile.Text = "";
            txtCSRKeystorePassword.Text = "";
            txtCARequest.Text = "";

            txtResponseCertificateFilename.Text = "";
            txtResponseKeystoreFilename.Text = "";
            txtResponseKeystorePassword.Text = "";

            EnabledRootCert();
            EnabledCSR();
            EnabledCA();
            EnabledResponse();

            //txtKeystoreFilename.Focus();
            //btnGenerateKeystore.Focus();
            txtKeystoreServerName.Focus();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            string appDir = Utils.getAppDir(Application.ExecutablePath);

            txtRootCertificate.Text = appDir + "\\Data\\root.cert";
            outputDir = appDir + "\\Output";

            string tempDir = appDir + "\\temp";

            if (!Directory.Exists(outputDir))
            {
                Directory.CreateDirectory(outputDir);
            }
            if (!Directory.Exists(tempDir))
            {
                Directory.CreateDirectory(tempDir);
            }
            //txtKeystoreServerName.Text = System.Environment.MachineName;
            txtBatchOutputDir.Text = outputDir;
            txtBatchTempDirectory.Text = tempDir;
            txtBatchRootCertificateFilename.Text = txtRootCertificate.Text;
            txtBatchTruststoreFilename.Text = appDir + "\\Data\\federation.truststore";
            txtBatchKeystorePassword.Text = Properties.Settings.Default.DefaultKeystorePassword;

            dtBatchValidFrom.Value = DateTime.Now;
            dtBatchValidTo.Value = new DateTime(2025, 12, 31);

            pnlBatch.Visible = false;

            Reset();
        }

        private void btnGenerateAll_Click(object sender, EventArgs e)
        {
            if (CreateKeystore(true))
            {
                if (ImportRootCertificate(true))
                {
                    if (CreateCSR(true))
                    {
                        if (SubmitCsr(true, false))
                        {
                            if (ImportCSResponse(true))
                            {
                                MessageBox.Show("Keystore generated and configured properly", "Keystore Created", 
                                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                                Reset();
                            }
                        }
                    }
                }
            }
        }

        private void btnRollupResults_Click(object sender, EventArgs e)
        {
            string zipDir = outputDir + "\\zip";
            if (!Directory.Exists(zipDir))
                Directory.CreateDirectory(zipDir);
            string zipFilename = zipDir + "\\" + txtKeystoreServerName.Text + "_federationCerts.zip";
            RollupResults(false, txtKeystoreFilename.Text, 
                Utils.getAppDir(Application.ExecutablePath) + "\\Data\\federation.truststore",
                zipFilename);
            DialogResult dialogResult =
                MessageBox.Show("Certificates rolled up into zip file, open directory?", "Open ZIP Directory", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dialogResult == System.Windows.Forms.DialogResult.Yes)
            {
                System.Diagnostics.Process.Start(zipDir);
            }
        }

        private bool RollupResults(bool silent, string keystoreFilename, string truststoreFilename, string zipFilename)
        {
            List<string> files = new List<string>();
            files.Add(keystoreFilename);
            //files.Add(txtRootCertificate.Text);
            files.Add(truststoreFilename);
            
            bool response = ZipHelper.ZipFiles(files, zipFilename);
            if (!silent)
            {
                if (response)
                {
                    MessageBox.Show("Federation Certificate Files zipped to file [" + zipFilename + "]",
                        "Files Zipped", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Error Zipping files", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            return response;
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            Reset();
        }

        private void chkRootCertEnabled_CheckedChanged(object sender, EventArgs e)
        {
            EnabledRootCert();
        }

        private void EnabledRootCert()
        {
            txtRootKeystorePassword.Enabled = chkRootCertEnabled.Checked;
            txtRootKeystoreFilename.Enabled = chkRootCertEnabled.Checked;
            txtRootCertificate.Enabled = chkRootCertEnabled.Checked;
            btnImportRootCertificate.Enabled = chkRootCertEnabled.Checked;
        }

        private void chkCSREnabled_CheckedChanged(object sender, EventArgs e)
        {
            EnabledCSR();
        }

        private void EnabledCSR()
        {
            txtCSRFilename.Enabled = chkCSREnabled.Checked;
            txtCSRKeystoreFile.Enabled = chkCSREnabled.Checked;
            txtCSRKeystorePassword.Enabled = chkCSREnabled.Checked;
            btnCreateCSR.Enabled = chkCSREnabled.Checked;
        }

        private void chkCAEnabled_CheckedChanged(object sender, EventArgs e)
        {
            EnabledCA();
        }

        private void EnabledCA()
        {
            txtCARequest.Enabled = chkCAEnabled.Checked;
            txtCAResponse.Enabled = chkCAEnabled.Checked;
            dtValidFrom.Enabled = chkCAEnabled.Checked;
            dtValidTo.Enabled = chkCAEnabled.Checked;
            btnSubmitCSR.Enabled = chkCAEnabled.Checked;
        }

        private void chkResponseEnabled_CheckedChanged(object sender, EventArgs e)
        {
            EnabledResponse();
        }

        private void EnabledResponse()
        {
            txtResponseCertificateFilename.Enabled = chkResponseEnabled.Checked;
            txtResponseKeystoreFilename.Enabled = chkResponseEnabled.Checked;
            txtResponseKeystorePassword.Enabled = chkResponseEnabled.Checked;
            btnImportCertificateResponse.Enabled = chkResponseEnabled.Checked;
        }

        private void btnBatchBrowseOutputDir_Click(object sender, EventArgs e)
        {
            string dir = Utils.SelectDirectory("Select Batch Output Directory");
            if (dir != "")
            {
                txtBatchOutputDir.Text = dir;
            }
        }

        private void btnBatchBrowseTempDirectory_Click(object sender, EventArgs e)
        {
            string dir = Utils.SelectDirectory("Select Batch Temp Directory");
            if (dir != "")
            {
                txtBatchTempDirectory.Text = dir;
            }
        }

        private void btnBatchBrowseRootCertFilename_Click(object sender, EventArgs e)
        {
            string file = Utils.SelectExistingFile("Select Root Certificate Filename");
            if (file != "")
            {
                txtBatchRootCertificateFilename.Text = file;    
            }
        }

        private void btnBatchBrowseTruststoreFilename_Click(object sender, EventArgs e)
        {
            string file = Utils.SelectExistingFile("Select Truststore Filename");
            if (file != "")
            {
                txtBatchTruststoreFilename.Text = file;
            }
        }

        private void btnBatchCreateCertificates_Click(object sender, EventArgs e)
        {
            foreach (string line in txtBatchServerList.Lines)
            {
                CreateBatchCertificate(line);
            }
            txtBatchServerList.Clear();
            MessageBox.Show("Batch process complete");
        }

        private void CreateBatchCertificate(string servername)
        {
            LogMsg("Creating keystore for server [" + servername + "]");
            Utils.ClearDirectory(txtBatchTempDirectory.Text);
            string keystoreFilename = txtBatchTempDirectory.Text + "\\federation.keystore";
            KeystoreParameters keystoreParam = KeystoreParameters.CreateDefaultKeystore(servername, txtBatchKeystorePassword.Text);
            if (CertificateHelper.CreateKeystore(keystoreFilename, keystoreParam, LogMsg))
            {
                LogMsg("Keystore for server [" + servername + "] generated");
                if (CertificateHelper.ImportRootCertIntoKeystore(keystoreFilename, txtBatchRootCertificateFilename.Text, txtBatchKeystorePassword.Text, LogMsg))
                {
                    LogMsg("Root certificate for server [" + servername + "] imported");
                    string csrFilename = txtBatchTempDirectory.Text + "\\certreq.csr";
                    string csrResponseFilename = txtBatchTempDirectory.Text + "\\certreq.resp";

                    string sessionId = "";
                    if (CertificateServerHelper.LoginToCertificateServer(
                        Properties.Settings.Default.CertificateAuthorityServer, 
                        Properties.Settings.Default.CertificateAuthorityUsername, 
                        Properties.Settings.Default.CertificateAuthorityPassword, 
                        LogMsg, DoEvents, ref sessionId))
                    {
                        LogMsg("Login to CA for server [" + servername + "] completed");
                        string certificateSerialNumber = "";
                        if (CertificateServerHelper.SubmitCSRRequest(Properties.Settings.Default.CertificateAuthorityServer,
                            sessionId, csrFilename, csrResponseFilename, keystoreParam, 
                            dtBatchValidFrom.Value.ToString("MM/dd/yyyy"), dtBatchValidTo.Value.ToString("MM/dd/yyyy"), 
                            LogMsg, DoEvents, ref certificateSerialNumber))
                        {
                            LogMsg("CSR Submitted successfully for server [" + servername + "].");
                            if (CertificateHelper.ImportCertificateResponseIntoKeystore(keystoreFilename, 
                                csrResponseFilename, txtBatchKeystorePassword.Text, 
                                KeystoreParameters.KEYSTORE_DEFAULT_ALIAS, LogMsg))
                            {
                                LogMsg("Certificate Response for server [" + servername + "] imported into keystore");

                                string zipFilename = txtBatchOutputDir.Text + "\\" + servername + "_federationCerts.zip";

                                if (RollupResults(true, keystoreFilename, txtBatchTruststoreFilename.Text, zipFilename))
                                {
                                    LogMsg("Certificate rolled up for server [" + servername + "] to file [" + zipFilename + "]");
                                }
                                else
                                {
                                    LogMsg("Error rolling up certificates for server [" + servername + "] to file [" + zipFilename + "]");
                                }
                            }
                        }
                    }
                }
            }
        }

        private void btnBatchAddServer_Click(object sender, EventArgs e)
        {
            if (txtBatchAddServer.Text.Length > 0)
            {
                txtBatchServerList.AppendText(txtBatchAddServer.Text + Environment.NewLine);
                txtBatchAddServer.Text = "";
                txtBatchAddServer.Focus();
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("VIX Certificate Tool\nVersion: " + Application.ProductVersion, 
                "About", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
      
    }
}
