/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace VixClientV3
{
    public class MethodFactoryConfiguration
    {
        private readonly string mstrFactoryType;
        private readonly string mstrAssemblyFilename;
        private readonly string mstrName;

        public string Name
        {
            get { return mstrName; }
        }

        public string FactoryType
        {
            get { return mstrFactoryType; }
        } 

        public string AssemblyFilename
        {
            get { return mstrAssemblyFilename; }
        } 

        public MethodFactoryConfiguration(string name, string factoryType, string assemblyFilename)
        {
            mstrFactoryType = factoryType;
            mstrAssemblyFilename = assemblyFilename;
            mstrName = name;
        }

        public override string ToString()
        {
            return mstrName;
        }
    }

    public class MethodFactoriesConfiguration
    {
        private readonly string mstrFilename;
        private readonly List<MethodFactoryConfiguration> configurations = new List<MethodFactoryConfiguration>();

        public List<MethodFactoryConfiguration> Configurations
        {
            get { return configurations; }
        } 


        public MethodFactoriesConfiguration(string filename)
        {
            mstrFilename = filename;
            LoadConfiguration();
        }

        private void LoadConfiguration()
        {
            if (System.IO.File.Exists(mstrFilename))
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(mstrFilename);
                XmlNodeList methodFactoryNodes = xmlDoc.SelectNodes("/MethodFactories/MethodFactory");
                foreach (XmlNode methodFactoryNode in methodFactoryNodes)
                {
                    string factoryType = methodFactoryNode.Attributes["FactoryType"].Value;
                    string assemblyFilename = methodFactoryNode.Attributes["AssemblyFilename"].Value;
                    string name = methodFactoryNode.Attributes["Name"].Value;
                    configurations.Add(new MethodFactoryConfiguration(name, factoryType, assemblyFilename));
                }
            }
        }

        public void Save()
        {
            XmlDocument xmlDoc = new XmlDocument();

            XmlNode rootNode = xmlDoc.CreateElement("MethodFactories");
            xmlDoc.AppendChild(rootNode);

            foreach (MethodFactoryConfiguration configuration in configurations)
            {
                XmlNode methodFactoryNode = xmlDoc.CreateElement("MethodFactory");
                rootNode.AppendChild(methodFactoryNode);
                XmlAttribute attr = xmlDoc.CreateAttribute("FactoryType");
                attr.Value = configuration.FactoryType;
                methodFactoryNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("AssemblyFilename");
                attr.Value = configuration.AssemblyFilename;
                methodFactoryNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Name");
                attr.Value = configuration.Name;
                methodFactoryNode.Attributes.Append(attr);
            }

            xmlDoc.Save(mstrFilename);
        }
    }
}
