/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using VixCommon;
using VixClientV3Common;
using System.Xml;

namespace VixClientV3
{
    public delegate void SetParametersDelegate(IMethod method, Dictionary<string, MethodParameter> parameters);

    public partial class MethodResultDisplay : UserControl
    {
        public MethodResultDisplay()
        {
            InitializeComponent();
        }

        private VixCommon.ObjectDisplayerControl objectDisplayer = null;
        public event TypeSelectedHandlerDelegate OnTypeSelectedHandlerEvent = null;
        private DateTime requestDate;

        private IMethod method;
        Dictionary<string, MethodParameter> parameters = null;
        public event SetParametersDelegate OnSetParametersEvent = null;

        private void MethodResultDisplay_Load(object sender, EventArgs e)
        {
            
        }

        public void DisplayResults(Dictionary<string, MethodParameter> parameters, IMethod method, object result, DateTime requestDate)
        {
            this.requestDate = requestDate;
            objectDisplayer = new VixCommon.ObjectDisplayerControl();

            objectDisplayer.Parent = panel1;// splitContainer1.Panel2;
            objectDisplayer.Dock = DockStyle.Fill;

            this.parameters = parameters;
            this.method = method;
            foreach (string parameterName in parameters.Keys)
            {
                ListViewItem item = new ListViewItem();
                item.Text = parameterName;
                MethodParameter methodParam = parameters[parameterName];
                item.SubItems.Add(methodParam.ParameterValue);
                item.Tag = methodParam;
                lstParameters.Items.Add(item);
            }

            List<AssociatedMethod> associatedMethods = method.getAssociatedMethods();
            if (associatedMethods != null)
            {
                objectDisplayer.TypeSelectedHandlers.AddRange(convertAssociatedMethodsToTypeSelectionHandler(associatedMethods));
            }

            splitContainer1.SplitterDistance = (splitContainer1.Height / 4);

            objectDisplayer.DisplayItem(result);
        }

        public void ShowSearch()
        {
            if (!objectDisplayer.ShowSearch())
            {
                MessageBox.Show("No results found");
            }
        }

        private List<TypeSelectedHandler> convertAssociatedMethodsToTypeSelectionHandler(List<AssociatedMethod> associatedMethods)
        {
            List<TypeSelectedHandler> handlers = new List<TypeSelectedHandler>();
            foreach (AssociatedMethod associatedMethod in associatedMethods)
            {
                handlers.Add(new TypeSelectedHandler(associatedMethod.MatchingType,
                    associatedMethod.MethodToCall.Name, associatedMethod.MethodToCall, OnTypeSelectedHandlerEvent));
            }
            return handlers;
        }

        private void saveRequestAndOutputToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = "";
            saveFileDialog1.Filter = "XML Files (*.xml)|*.xml";
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                SaveRequestAndOutput(saveFileDialog1.FileName);
            }
        }

        private void SaveRequestAndOutput(string filename)
        {
            try
            {
                XmlDocument xmlDoc = new XmlDocument();

                XmlDocument objectXmlDoc = new XmlDocument();
                objectXmlDoc.LoadXml(objectDisplayer.OutputToXml());

                XmlNode rootNode = xmlDoc.CreateElement("VIXClientRequest");
                xmlDoc.AppendChild(rootNode);

                XmlNode requestNode = xmlDoc.CreateElement("Request");

                XmlAttribute requestAttr = xmlDoc.CreateAttribute("Date");
                requestAttr.Value = requestDate.ToString();
                requestNode.Attributes.Append(requestAttr);
                rootNode.AppendChild(requestNode);
                XmlNode responseNode = xmlDoc.CreateElement("Response");
                rootNode.AppendChild(responseNode);

                foreach (string key in parameters.Keys)
                {
                    XmlNode parameterNode = xmlDoc.CreateElement("Parameter");
                    requestNode.AppendChild(parameterNode);
                    MethodParameter param = parameters[key];

                    XmlAttribute attr = xmlDoc.CreateAttribute("name");
                    attr.Value = param.ParameterName;
                    parameterNode.Attributes.Append(attr);
                    attr = xmlDoc.CreateAttribute("value");
                    attr.Value = param.ParameterValue;
                    parameterNode.Attributes.Append(attr);
                }

                XmlNode importedNode = xmlDoc.ImportNode(objectXmlDoc.DocumentElement, true);
                responseNode.AppendChild(importedNode);
                

                xmlDoc.Save(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error saving output, " + ex.Message);
            }
        }

        private void copyValueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (lstParameters.SelectedItems.Count > 0)
            {
                Clipboard.SetText(lstParameters.SelectedItems[0].SubItems[1].Text);
            }
        }

        private void reuseParametersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (OnSetParametersEvent != null)
            {
                OnSetParametersEvent(method, parameters);
            }
        }
    }
}
