/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using VixClientV3Common;
using System.Diagnostics;

namespace VixClientV3
{
    public partial class frmConfig : Form
    {
        public frmConfig()
        {
            InitializeComponent();
        }

        private VixConfiguration configuration = null;
        private UserConfigurations userConfigurations = null;
        private UserCredentials user = null;
        private MethodFactoriesConfiguration methodFactoriesConfiguration = null;
        private MethodFactoryConfiguration methodFactoryConfiguration = null;

        public MethodFactoryConfiguration MethodFactoryConfiguration
        {
            get { return methodFactoryConfiguration; }
        }

        public VixConfiguration Configuration
        {
            get { return configuration; }
        }

        public UserCredentials User
        {
            get { return user; }
        }

        public UserConfigurations UserConfigurations
        {
            get { return userConfigurations; }
        }

        public DialogResult ShowConfiguration(MethodFactoriesConfiguration methodFactoriesConfiguration, 
            UserConfigurations userConfigurations)
        {
            this.methodFactoriesConfiguration = methodFactoriesConfiguration;
            this.userConfigurations = userConfigurations;
            cboInterfaces.Items.AddRange(methodFactoriesConfiguration.Configurations.ToArray());

            cboUsers.Items.AddRange(userConfigurations.UserCredentials.ToArray());
            txtDICOMViewer.Text = userConfigurations.DicomViewerPath;
            chkExceptionsAsOutput.Checked = userConfigurations.ShowErrorsAsOutput;

            // configurations come from two places, old vix client config files and from the UserConfigurations object
            cboConfigurations.Items.AddRange(userConfigurations.VixServerConfigurations.ToArray());

            if (userConfigurations.LastVixServerConfiguration != null)
            {
                txtServer.Text = userConfigurations.LastVixServerConfiguration.Server;
                txtPort.Text = userConfigurations.LastVixServerConfiguration.Port + "";
            }
            if (userConfigurations.LastUserCredentials != null)
            {
                DisplayUserCredentials(userConfigurations.LastUserCredentials);
            }
            if (userConfigurations.LastVixInterfaceName != null)
            {
                foreach (object obj in cboInterfaces.Items)
                {
                    if (obj.ToString() == userConfigurations.LastVixInterfaceName)
                    {
                        cboInterfaces.SelectedItem = obj;
                        break;
                    }
                }
            }

            this.ShowDialog();
            return this.DialogResult;
        }

        private void frmConfig_Load(object sender, EventArgs e)
        {            
            //txtServer.Text = "localhost";
            //txtPort.Text = "8080";

            txtClientVersion.Text = Application.ProductVersion;

            VixServerConfiguration[] configurations = getVixConfigurations();
            if (configurations != null)
            {
                cboConfigurations.Items.AddRange(configurations);
            }
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (cboInterfaces.SelectedItem == null)
            {
                MessageBox.Show("You must select a VIX Interface", "Interface Required", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                this.DialogResult = DialogResult.None;                
            }
            else if ((txtDuz.Text.Length <= 0) && (txtFullname.Text.Length <= 0))
            {
                MessageBox.Show("You must enter user credentials for accessing the VIX", "User Information Required", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                this.DialogResult = DialogResult.None;                
            }
            else
            {
                user = new UserCredentials(txtDuz.Text, txtFullname.Text, txtSiteName.Text, txtSiteNumber.Text, txtSsn.Text, txtCacheLocationId.Text, txtBseToken.Text);
                configuration = new VixConfiguration(txtServer.Text, int.Parse(txtPort.Text));
                configuration.DicomViewerPath = txtDICOMViewer.Text;
                configuration.ImageDirectory = userConfigurations.ImageDirectory;
                configuration.ClientVersion = txtClientVersion.Text;
                methodFactoryConfiguration = (MethodFactoryConfiguration)cboInterfaces.SelectedItem;
                userConfigurations.LastVixServerConfiguration = new VixServerConfiguration(configuration.Server, configuration.Port);
                userConfigurations.LastUserCredentials = new UserCredentials(user.Duz, user.Name, user.SiteName, user.SiteNumber, user.Ssn, user.CacheLocationId, user.BseToken);
                userConfigurations.LastVixInterfaceName = methodFactoryConfiguration.Name;
                userConfigurations.DicomViewerPath = txtDICOMViewer.Text;
                userConfigurations.ShowErrorsAsOutput = chkExceptionsAsOutput.Checked;
                userConfigurations.Save();
                this.DialogResult = DialogResult.OK;
            }
        }

        private VixServerConfiguration[] getVixConfigurations()
        {
            List<VixServerConfiguration> clientConfigs = null;
            string vixClientPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "vix-client");
            if (Directory.Exists(vixClientPath))
            {
                string[] files = Directory.GetFiles(vixClientPath, "*.vix-client");
                clientConfigs = new List<VixServerConfiguration>();
                for (int i = 0; i < files.Length; i++)
                {
                    clientConfigs.Add(readClientConfig(files[i]));
                }
            }
            if (clientConfigs != null)
            {
                return clientConfigs.ToArray();
            }
            else
            {
                return null;
            }
        }

        private VixServerConfiguration readClientConfig(string filename)
        {
            FileInfo configFile = new FileInfo(filename);
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(filename);
            XmlNode metadataNode = xmlDoc.SelectSingleNode("java/object/void[@property=\"xchangeMetadataParameters\"]");


            XmlNode hostNode = metadataNode.SelectSingleNode("object/void[@property=\"host\"]/string");
            XmlNode usernameNode = metadataNode.SelectSingleNode("object/void[@property=\"uid\"]/string");
            XmlNode passwordNode = metadataNode.SelectSingleNode("object/void[@property=\"credentials\"]/string");
            XmlNode applicationPathNode = metadataNode.SelectSingleNode("object/void[@property=\"application\"]/string");
            XmlNode pathNode = metadataNode.SelectSingleNode("object/void[@property=\"path\"]/string");
            XmlNode protocolNode = metadataNode.SelectSingleNode("object/void[@property=\"protocol\"]/string");
            XmlNode portNode = metadataNode.SelectSingleNode("object/void[@property=\"port\"]/int");

            //string metadataPath = "http://" + hostNode.InnerText + ":" + portNode.InnerText + "/VistaRadWebApp/vistarad-ws/ImageMetadataVistarad.V1";
            //string username = usernameNode.InnerText;
            //string password = passwordNode.InnerText;

            XmlNode imageNode = xmlDoc.SelectSingleNode("java/object/void[@property=\"xchangeImageParameters\"]");
            hostNode = imageNode.SelectSingleNode("object/void[@property=\"host\"]/string");
            applicationPathNode = imageNode.SelectSingleNode("object/void[@property=\"application\"]/string");
            pathNode = imageNode.SelectSingleNode("object/void[@property=\"path\"]/string");
            protocolNode = imageNode.SelectSingleNode("object/void[@property=\"protocol\"]/string");
            portNode = imageNode.SelectSingleNode("object/void[@property=\"port\"]/int");

            

            //string imageUrl = "http://" + hostNode.InnerText + ":" + portNode.InnerText + "/VistaRadWebApp/vistarad/image";
            VixServerConfiguration config = new VixServerConfiguration(hostNode.InnerText, int.Parse(portNode.InnerText));
            config.Name = configFile.Name;
            return config;
        }

        private void cboConfigurations_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboConfigurations.SelectedItem != null)
            {
                VixServerConfiguration config = (VixServerConfiguration)cboConfigurations.SelectedItem;
                txtServer.Text = config.Server;
                txtPort.Text = config.Port + "";
            }
        }

        private void btnAddInterface_Click(object sender, EventArgs e)
        {
            frmAddInterfaceConfiguration addInterface = new frmAddInterfaceConfiguration();
            if (addInterface.ShowAddInterface() == DialogResult.OK)
            {
                MethodFactoryConfiguration newConfig = addInterface.getMethodFactoryConfiguration();
                methodFactoriesConfiguration.Configurations.Add(newConfig);
                methodFactoriesConfiguration.Save();
                cboInterfaces.Items.Add(newConfig);
            }
        }

        private void btnAddUser_Click(object sender, EventArgs e)
        {
            frmAddUser addUser = new frmAddUser();
            if (addUser.ShowAddUser() == DialogResult.OK)
            {
                UserCredentials newUser = addUser.GetUserCredentials();
                userConfigurations.UserCredentials.Add(newUser);
                userConfigurations.Save();
                cboUsers.Items.Add(newUser);
            }
        }

        private void cboUsers_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboUsers.SelectedItem != null)
            {
                UserCredentials userCredentials = (UserCredentials)cboUsers.SelectedItem;
                DisplayUserCredentials(userCredentials);
            }
        }

        private void DisplayUserCredentials(UserCredentials userCredentials)
        {
            txtCacheLocationId.Text = userCredentials.CacheLocationId;
            txtDuz.Text = userCredentials.Duz;
            txtFullname.Text = userCredentials.Name;
            txtSiteName.Text = userCredentials.SiteName;
            txtSiteNumber.Text = userCredentials.SiteNumber;
            txtSsn.Text = userCredentials.Ssn;
        }

        private void cboInterfaces_SelectedIndexChanged(object sender, EventArgs e)
        {
            
        }

        private void btnGenerateToken_Click(object sender, EventArgs e)
        {
            generateToken();
        }

        private void generateToken()
        {
            FileInfo appPath = new FileInfo(Application.ExecutablePath);
            string bseTokenGeneratorFilename = appPath.Directory.FullName + @"\BseTokenGenerator.exe";
            if (!File.Exists(bseTokenGeneratorFilename))
            {
                MessageBox.Show("Cannot find BSE Token Generator '" + bseTokenGeneratorFilename + "', required to generate BSE token", 
                    "BSE Token Generator Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                try
                {
                    string outputFile = appPath.Directory.FullName + "\\bse.txt";
                    if (File.Exists(outputFile))
                    {
                        File.Delete(outputFile);
                    }

                    Process proc = new Process();
                    proc.StartInfo.FileName = bseTokenGeneratorFilename;
                    proc.StartInfo.Arguments = "\"" + outputFile + "\"";

                    proc.Start();
                    proc.WaitForExit(1000 * 60 * 5); // wait 5 minutes

                    if (File.Exists(outputFile))
                    {
                        StreamReader reader = new StreamReader(outputFile);
                        string bseToken = reader.ReadLine();
                        txtBseToken.Text = bseToken;
                        reader.Close();
                        File.Delete(outputFile);
                    }
                    else
                    {
                        MessageBox.Show("BSE token not generated properly, output file does not exist");
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error generating BSE token, " + ex.Message);
                }
            }
        }

        private void btnAddServerConfiguration_Click(object sender, EventArgs e)
        {
            frmAddVixServerConfiguration addVixConfig = new frmAddVixServerConfiguration();
            if (addVixConfig.ShowAddVixServerConfiguration() == DialogResult.OK)
            {
                userConfigurations.VixServerConfigurations.Add(addVixConfig.VixServerConfiguration);
                userConfigurations.Save();
                cboConfigurations.Items.Add(addVixConfig.VixServerConfiguration);
            }
        }
    }
}
