/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using VixClientV3Common.configuration;
using VixClientV3Common;

namespace VixClientV3
{
    public partial class frmSaveMethodParameters : Form
    {
        public frmSaveMethodParameters()
        {
            InitializeComponent();
        }

        private IMethod method;

        private StoredMethodParameters storedMethodParameter = null;

        public StoredMethodParameters StoredMethodParameter
        {
            get { return storedMethodParameter; }
        }

        public DialogResult ShowSaveMethodParameters(IMethod method, 
            Dictionary<string, MethodParameter> methodParameters)
        {
            this.method = method;
            txtMethodName.Text = method.ToString();

            foreach (string key in methodParameters.Keys)
            {
                MethodParameter parameter = methodParameters[key];
                if (parameter.Saveable)
                {
                    ListViewItem item = new ListViewItem();
                    item.Text = parameter.ParameterName;
                    item.SubItems.Add(parameter.ParameterValue);
                    item.Tag = parameter;
                    item.Checked = true;
                    lstParameters.Items.Add(item);
                }  
            }
            txtDescription.Focus();
            this.ShowDialog();

            return this.DialogResult;
        }

        private void btnSaveChecked_Click(object sender, EventArgs e)
        {
            if (txtDescription.Text.Length <= 0)
            {
                MessageBox.Show("Must enter a description for these parameters");
            }
            else
            {
                storedMethodParameter = new StoredMethodParameters(method.ToString(), txtDescription.Text);
                foreach (ListViewItem item in lstParameters.Items)
                {
                    if (item.Checked)
                    {
                        MethodParameter param = (MethodParameter)item.Tag;

                        StoredMethodParameterValue parameterValue = new StoredMethodParameterValue(param.ParameterName, param.ParameterValue);
                        storedMethodParameter.StoredParameters.Add(parameterValue);
                    }
                }
            }
        }

        private void frmSaveMethodParameters_Load(object sender, EventArgs e)
        {

        }
    }
}
