/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using VixClientV3Common;

namespace VixClientV3
{
    public partial class frmVixInterfaceConfiguration : Form
    {
        public frmVixInterfaceConfiguration()
        {
            InitializeComponent();
        }

        private List<MethodFactoryParameter> methodFactoryParameters = null;
        private IMethodFactory methodFactory = null;

        public List<MethodFactoryParameter> MethodFactoryParameters
        {
            get { return methodFactoryParameters; }
        }


        public DialogResult ShowMethodFactoryConfiguration(IMethodFactory methodFactory)
        {
            this.methodFactory = methodFactory;
            List<MethodFactoryParameter> methodFactoryParameters = methodFactory.getMethodFactoryParameters();
            this.Text = "VIX Interface Configuration - " + methodFactory.getName();

            DisplayParameters(methodFactoryParameters);

            List<StoredMethodFactoryParameters> storedMethodFactoryParameters = methodFactory.getStoredMethodFactoryParameters();

            cboStoredMethodFactoryParameters.Items.AddRange(storedMethodFactoryParameters.ToArray());


            this.ShowDialog();

            return this.DialogResult;
        }

        private void DisplayParameters(List<MethodFactoryParameter> methodFactoryParameters)
        {
            foreach (MethodFactoryParameter parameter in methodFactoryParameters)
            {
                ListViewItem item = new ListViewItem();
                item.Tag = parameter;
                item.Text = parameter.ParameterName;
                item.SubItems.Add(parameter.ParameterValue);
                item.SubItems.Add(parameter.Required.ToString());
                lstParameters.Items.Add(item);
            }
        }

        private void lstParameters_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (lstParameters.SelectedItems.Count > 0)
            {
                MethodFactoryParameter param = (MethodFactoryParameter)lstParameters.SelectedItems[0].Tag;
                frmInputDialog input = new frmInputDialog();
                if (input.ShowInputDialog("Set Value For " + param.ParameterName, 
                    "Set value for parameter '" + param.ParameterName + "'.", param.ParameterValue) == DialogResult.OK)
                {
                    param.ParameterValue = input.Value;
                    lstParameters.SelectedItems[0].SubItems[1].Text = input.Value;
                }
            }
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            methodFactoryParameters = new List<MethodFactoryParameter>();
            foreach (ListViewItem item in lstParameters.Items)
            {
                MethodFactoryParameter param = (MethodFactoryParameter)item.Tag;
                methodFactoryParameters.Add(param);
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            frmInputDialog input = new frmInputDialog();
            if (input.ShowInputDialog("Description", "Enter description of stored parameters", "") == DialogResult.OK)
            {
                StoredMethodFactoryParameters storedMethodFactoryParameters = new StoredMethodFactoryParameters(input.Value);
                foreach (ListViewItem item in lstParameters.Items)
                {
                    MethodFactoryParameter parameter = (MethodFactoryParameter)item.Tag;
                    if (parameter.Saveable)
                    {
                        storedMethodFactoryParameters.MethodFactoryParameters.Add(parameter);
                    }
                }
                methodFactory.addStoredMethodFactoryParameter(storedMethodFactoryParameters);
            }
        }

        private void cboStoredMethodFactoryParameters_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboStoredMethodFactoryParameters.SelectedItem != null)
            {
                StoredMethodFactoryParameters storedMethodFactoryParameters = 
                    (StoredMethodFactoryParameters)cboStoredMethodFactoryParameters.SelectedItem;

                foreach (StoredMethodFactoryParameter storedParameter in storedMethodFactoryParameters.MethodFactoryParameters)
                {
                    foreach (ListViewItem item in lstParameters.Items)
                    {
                        MethodFactoryParameter parameter = (MethodFactoryParameter)item.Tag;
                        if (parameter.ParameterName == storedParameter.ParameterName)
                        {
                            parameter.ParameterValue = storedParameter.ParameterValue;
                            item.SubItems[1].Text = parameter.ParameterValue;
                            break;
                        }
                    }
                }

            }
        }
    }
}
