/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common.configuration;

namespace VixClientV3Common
{
    public abstract class AbstractBaseMethodFactory //: IMethodFactory
    {
        private MethodParameterConfiguration methodParameterConfiguration = null;
        protected event LogMsgDelegate OnLogMsgEvent = null;

        public AbstractBaseMethodFactory()
        {
            methodParameterConfiguration = new MethodParameterConfiguration(this.getName());
        }

        public List<StoredMethodParameters> getStoredMethodParameters(string methodName)
        {
            List<StoredMethodParameters> parameters = new List<StoredMethodParameters>();

            foreach (StoredMethodParameters methodParameters in methodParameterConfiguration.StoredMethods)
            {
                if (methodParameters.MethodName == methodName)
                {
                    parameters.Add(methodParameters);
                }
            }
            return parameters;
        }

        public List<StoredMethodFactoryParameters> getStoredMethodFactoryParameters()
        {
            return methodParameterConfiguration.StoredMethodFactoryParameters;
        }

        public void addStoredMethodParameter(StoredMethodParameters methodParameters)
        {
            methodParameterConfiguration.StoredMethods.Add(methodParameters);
            methodParameterConfiguration.Save();
        }

        public void addStoredMethodFactoryParameter(StoredMethodFactoryParameters storedMethodFactoryParameters)
        {
            methodParameterConfiguration.StoredMethodFactoryParameters.Add(storedMethodFactoryParameters);
            methodParameterConfiguration.Save();
        }

        /*
        public List<MethodFactoryParameter> getMethodFactoryParameters()
        {
            return null;
        }

        public abstract List<IMethod> getMethods(List<MethodFactoryParameter> userMethodFactoryParameters, 
            UserCredentials userCredentials, VixConfiguration vixConfiguration);
         * */

        public abstract string getName();

        public override string ToString()
        {
            return this.getName();
        }

        public void setLogMsg(LogMsgDelegate logMsgDelegate)
        {
            OnLogMsgEvent = logMsgDelegate;
        }

        protected void LogMsg(string msg, bool isError)
        {
            if (OnLogMsgEvent != null)
            {
                OnLogMsgEvent(msg, isError);
            }
        }

        protected void LogMsg(string msg)
        {
            if (OnLogMsgEvent != null)
            {
                OnLogMsgEvent(msg, false);
            }
        }
        
    }
}
