/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixClientV3Common
{
    public delegate void LogMsgDelegate(string msg, bool error);

    public interface IMethod
    {
        /// <summary>
        /// Execute this method with the given input parameters
        /// </summary>
        /// <param name="values"></param>
        /// <returns></returns>
        object execute(Dictionary<string, MethodParameter> values);

        Dictionary<string, MethodParameter> getParameters();

        bool validateParameters(Dictionary<string, MethodParameter> values);

        /// <summary>
        /// Get the list of methods that are associated with this method.  This list is the list of methods that are associated
        /// with a specific type that will be returned by this method
        /// </summary>
        /// <returns></returns>
        List<AssociatedMethod> getAssociatedMethods();

        /// <summary>
        /// Given an object of any type, get updated parameters based on that object. Its up to the implemeting IMethod
        /// to determine what properties it can get from the obj and new properties. This method is not required to return
        /// all parameters for this IMethod, only the updated ones
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj);

        void setLogMsg(LogMsgDelegate logMsgDelegate);
    }
}
