/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixClientV3Common
{
    public delegate void ParameterSetDelegate(string value);

    public class MethodParameter : StoredMethodParameterValue
    {        
        private readonly bool mbRequired;
        private readonly MethodParameterType mParameterType;
        private readonly bool mbSaveable;
        private readonly string mstrParameterDescription;

        public string ParameterDescription
        {
            get { return mstrParameterDescription; }
        } 

        public bool Saveable
        {
            get { return mbSaveable; }
        }

        public MethodParameterType ParameterType
        {
            get { return mParameterType; }
        }

        public bool Required
        {
            get { return mbRequired; }
        }

        public MethodParameter(string parameterName, bool required, string parameterValue, MethodParameterType parameterType, bool saveable,
            string parameterDescription)
            : base(parameterName, parameterValue)
        {
            mbRequired = required;
            mParameterType = parameterType;
            mbSaveable = saveable;
            mstrParameterDescription = parameterDescription;
        }

        public MethodParameter(string parameterName, bool required, string parameterValue, MethodParameterType parameterType, bool saveable)
            : this(parameterName, required, parameterValue, parameterType, saveable, "")
        {
            
        }

        public MethodParameter(string parameterName, bool required, string parameterValue, MethodParameterType parameterType)
            : this(parameterName, required, parameterValue, parameterType, false)
        {

        }

        public MethodParameter(string parameterName, bool required, string parameterValue)
            : this(parameterName, required, parameterValue, MethodParameterType.unknown)
        {
        }

        public MethodParameter(string parameterName, bool required, string parameterValue, bool saveable)
            : this(parameterName, required, parameterValue, MethodParameterType.unknown, saveable)
        {
        }

        public MethodParameter(string parameterName, bool required, string parameterValue, bool saveable, string parameterDescription)
            : this(parameterName, required, parameterValue, MethodParameterType.unknown, saveable, parameterDescription)
        {
        }

        public MethodParameter(string parameterName, bool required)
            : this(parameterName, required, "", MethodParameterType.unknown)
        {
        }

        
    }

    public enum MethodParameterType
    {
        unknown, stringType, intType, saveFileType, openFileType, dateType
    }

    public class StoredMethodParameterValue
    {
        private readonly string mstrParameterName;
        protected string mstrParameterValue;

        public event ParameterSetDelegate OnParameterSetEvent = null;

        public string ParameterValue
        {
            get { return mstrParameterValue; }
            set
            {
                mstrParameterValue = value;
                if (OnParameterSetEvent != null)
                {
                    OnParameterSetEvent(mstrParameterValue);
                }
            }
        }

        public string ParameterName
        {
            get { return mstrParameterName; }
        }

        public StoredMethodParameterValue(string parameterName, string parameterValue)
        {
            this.mstrParameterName = parameterName;
            this.mstrParameterValue = parameterValue;
        }
    }

    public delegate string AutoGenerateNewValueDelegate();

    public class AutogeneratingMethodParameter : MethodParameter
    {
        private readonly AutoGenerateNewValueDelegate OnAutoGenerateNewValueEvent;

        public AutogeneratingMethodParameter(string parameterName, bool required, 
            string parameterValue, MethodParameterType parameterType, AutoGenerateNewValueDelegate OnAutoGenerateNewValueEvent)
            : base(parameterName, required, parameterValue, parameterType)
        {
            this.OnAutoGenerateNewValueEvent += OnAutoGenerateNewValueEvent;
        }

        public void updateValue()
        {
            if (OnAutoGenerateNewValueEvent != null)
            {
                this.mstrParameterValue = OnAutoGenerateNewValueEvent();
            }
        }
    }

    /// <summary>
    /// Type that would allow multiple enumerated options for possible values, not fully implemented yet
    /// </summary>
    public class EnumerableMethodParameter : MethodParameter
    {
        private object [] possibleValues;

        public EnumerableMethodParameter(string parameterName, bool required, 
            string parameterValue, MethodParameterType parameterType, object [] possibleValues)
            : base(parameterName, required, parameterValue, parameterType)
        {
            this.possibleValues = possibleValues;
        }
    }
}
