/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Reflection;

namespace VixClientV3Common.configuration
{
    public class MethodParameterConfiguration
    {
        private readonly string mstrFilename;
        private readonly List<StoredMethodParameters> storedMethods = 
            new List<StoredMethodParameters>();
        private readonly List<StoredMethodFactoryParameters> storedMethodFactoryParameters =
            new List<StoredMethodFactoryParameters>();

        public List<StoredMethodFactoryParameters> StoredMethodFactoryParameters
        {
            get { return storedMethodFactoryParameters; }
        } 

        public List<StoredMethodParameters> StoredMethods
        {
            get { return storedMethods; }
        } 

        public MethodParameterConfiguration(string factoryName)
        {
            //mstrFilename=

            FileInfo file = new FileInfo(Assembly.GetExecutingAssembly().Location);

            mstrFilename = file.Directory + "\\" + factoryName + ".config";
            LoadData();
        }

        private void LoadData()
        {
            if (File.Exists(mstrFilename))
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(mstrFilename);
                storedMethods.Clear();
                storedMethodFactoryParameters.Clear();

                XmlNodeList methodsList = xmlDoc.SelectNodes("/StoredParameters/Methods/Method");
                foreach (XmlNode methodNode in methodsList)
                {
                    string name = methodNode.Attributes["Name"].Value;
                    string description = methodNode.Attributes["Description"].Value;
                    StoredMethodParameters storedMethod = new StoredMethodParameters(name, description);
                    storedMethods.Add(storedMethod);

                    XmlNodeList parametersNodeList = methodNode.SelectNodes("Parameters/Parameter");
                    foreach (XmlNode parameterNode in parametersNodeList)
                    {
                        string parameterName = parameterNode.Attributes["Name"].Value;
                        string parameterValue = parameterNode.Attributes["Value"].Value;
                        storedMethod.StoredParameters.Add(new StoredMethodParameterValue(parameterName, parameterValue));
                    }
                }

                XmlNodeList storedMethodFactoryParameterNodes = 
                    xmlDoc.SelectNodes("/StoredParameters/FactoryConfiguration/StoredMethodFactoryParameters/StoredMethodFactoryParameter");
                foreach (XmlNode storedMethodFactoryParameterNode in storedMethodFactoryParameterNodes)
                {
                    string description = storedMethodFactoryParameterNode.Attributes["Description"].Value;
                    StoredMethodFactoryParameters newParameters = new StoredMethodFactoryParameters(description);
                    XmlNodeList methodFactoryParameterNodes = storedMethodFactoryParameterNode.SelectNodes("MethodFactoryParameters/MethodFactoryParameter");
                    foreach (XmlNode methodFactoryParameterNode in methodFactoryParameterNodes)
                    {
                        string name = methodFactoryParameterNode.Attributes["Name"].Value;
                        string value = methodFactoryParameterNode.Attributes["Value"].Value;
                        newParameters.MethodFactoryParameters.Add(new StoredMethodFactoryParameter(name, value));
                    }
                    storedMethodFactoryParameters.Add(newParameters);
                }                
            }
        }

        public void Save()
        {
            XmlDocument xmlDoc = new XmlDocument();

            XmlNode rootNode = xmlDoc.CreateElement("StoredParameters");
            xmlDoc.AppendChild(rootNode);

            XmlNode methodsNode = xmlDoc.CreateElement("Methods");
            rootNode.AppendChild(methodsNode);

            foreach (StoredMethodParameters methods in storedMethods)
            {
                XmlNode methodNode = xmlDoc.CreateElement("Method");
                methodsNode.AppendChild(methodNode);

                XmlAttribute attr = xmlDoc.CreateAttribute("Name");
                attr.Value = methods.MethodName;
                methodNode.Attributes.Append(attr);
                attr = xmlDoc.CreateAttribute("Description");
                attr.Value = methods.ParameterDescription;
                methodNode.Attributes.Append(attr);

                XmlNode parametersNode = xmlDoc.CreateElement("Parameters");
                methodNode.AppendChild(parametersNode);

                foreach (StoredMethodParameterValue parameterValues in methods.StoredParameters)
                {
                    XmlNode parameterNode = xmlDoc.CreateElement("Parameter");
                    parametersNode.AppendChild(parameterNode);
                    attr = xmlDoc.CreateAttribute("Name");
                    attr.Value = parameterValues.ParameterName;
                    parameterNode.Attributes.Append(attr);
                    attr = xmlDoc.CreateAttribute("Value");
                    attr.Value = parameterValues.ParameterValue;
                    parameterNode.Attributes.Append(attr);
                }
            }

            XmlNode factoryConfiguration = xmlDoc.CreateElement("FactoryConfiguration");
            rootNode.AppendChild(factoryConfiguration);
            XmlNode methodFactoryParameters = xmlDoc.CreateElement("StoredMethodFactoryParameters");
            factoryConfiguration.AppendChild(methodFactoryParameters);
            foreach (StoredMethodFactoryParameters storedMethodFactoryParameter in storedMethodFactoryParameters)
            {
                XmlNode storedMethodFactoryParameterNode = xmlDoc.CreateElement("StoredMethodFactoryParameter");
                methodFactoryParameters.AppendChild(storedMethodFactoryParameterNode);

                XmlAttribute attr = xmlDoc.CreateAttribute("Description");
                attr.Value = storedMethodFactoryParameter.Description;
                storedMethodFactoryParameterNode.Attributes.Append(attr);

                XmlNode methodFactoryParameterNodes = xmlDoc.CreateElement("MethodFactoryParameters");
                storedMethodFactoryParameterNode.AppendChild(methodFactoryParameterNodes);

                foreach (StoredMethodFactoryParameter methodFactoryParameter in storedMethodFactoryParameter.MethodFactoryParameters)
                {
                    XmlNode methodFactoryParameterNode = xmlDoc.CreateElement("MethodFactoryParameter");
                    methodFactoryParameterNodes.AppendChild(methodFactoryParameterNode);
                    attr = xmlDoc.CreateAttribute("Name");
                    attr.Value = methodFactoryParameter.ParameterName;
                    methodFactoryParameterNode.Attributes.Append(attr);
                    attr = xmlDoc.CreateAttribute("Value");
                    attr.Value = methodFactoryParameter.ParameterValue;
                    methodFactoryParameterNode.Attributes.Append(attr);
                }
            }




            xmlDoc.Save(mstrFilename);
        }

    }

    public class StoredMethodParameters
    {
        private readonly string mstrMethodName;
        private readonly string mstrParameterDescription;       

        private readonly List<StoredMethodParameterValue> storedParameters =
            new List<StoredMethodParameterValue>();

        public string MethodName
        {
            get { return mstrMethodName; }
        } 

        public List<StoredMethodParameterValue> StoredParameters
        {
            get { return storedParameters; }
        }

        public Dictionary<string, StoredMethodParameterValue> StoredParametersDictionary
        {
            get
            {
                Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
                foreach (StoredMethodParameterValue param in storedParameters)
                {
                    updatedParameters.Add(param.ParameterName, new StoredMethodParameterValue(param.ParameterName, param.ParameterValue));
                }
                return updatedParameters;
            }
        }

        public string ParameterDescription
        {
            get { return mstrParameterDescription; }
        } 

        public StoredMethodParameters(string methodName, string parameterDescription)
        {
            mstrMethodName = methodName;
            mstrParameterDescription = parameterDescription;
        }

        public override string ToString()
        {
            return mstrParameterDescription;
        }
    }
}
