/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_AWIV.methods
{
    public class GetImage : AbstractBaseGetImageMethod
    {
        public GetImage(UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(true, userCredentials, vixConfiguration)
        {
            
        }

        protected override string getDefaultOpenFileExecutable()
        {
            return vixConfiguration.DicomViewerPath;
        }

        protected override Dictionary<string, string> getExtraRequestHeaders(Dictionary<string, MethodParameter> values)
        {
            return null;
        }

        protected override void addOrUpdateParameters(Dictionary<string, MethodParameter> values)
        {
            values[parameterQuality].ParameterValue = "70";
            values[parameterContentType].ParameterValue = "*/*";
        }

        protected override System.Net.ICredentials getNetworkCredentials(Dictionary<string, MethodParameter> values)
        {
            return new System.Net.NetworkCredential("", "");
        }

        public override string ToString()
        {
            return this.GetType().Name;
        }

        protected override string getImageRequestUrl(Dictionary<string, MethodParameter> values)
        {
            string imageUrn = values[parameterImageUrn].ParameterValue;
            string quality = values[parameterQuality].ParameterValue;
            string contentType = values[parameterContentType].ParameterValue;
            StringBuilder url = new StringBuilder();
            url.Append("http://");
            url.Append(vixConfiguration.Server);
            url.Append(":");
            url.Append(vixConfiguration.Port);
            url.Append("/AWIVWebApp/awiv/image");
            url.Append("?imageURN=");
            url.Append(imageUrn);
            url.Append("&imageQuality=");
            url.Append(quality);
            url.Append("&contentType=");
            url.Append(contentType);

            return url.ToString();
        }

        public override Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is awivWebReference.ImageType)
            {
                awivWebReference.ImageType imageType = (awivWebReference.ImageType)obj;
                updatedParameters.Add(parameterImageUrn, new StoredMethodParameterValue(parameterImageUrn, imageType.imageurn));
                string imageUri = imageType.fullimageURI;
                if (imageUri != null)
                {
                    string imageQuality = extractPiece(imageUri, "imageQuality=", "&");
                    if (imageQuality != null)
                    {
                        updatedParameters.Add(parameterQuality, new StoredMethodParameterValue(parameterQuality, imageQuality));
                    }
                    string contentType = extractPiece(imageUri, "contentType=", null);
                    if (contentType != null)
                    {
                        updatedParameters.Add(parameterContentType, new StoredMethodParameterValue(parameterContentType, contentType));
                    }
                }
            }
            return updatedParameters;
        }
    }
}
