/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_AWIVV2.methods
{
    public class GetPatientPhotoImage : AbstractBaseGetImageMethod
    {
        private readonly string parameterSiteNumber = "SiteNumber";
        private readonly string parameterPatientIcn = "PatientIcn";

        public GetPatientPhotoImage(UserCredentials userCredentials, VixConfiguration vixConfiguration, 
            LogMsgDelegate logMsgDelegate)
            : base(true, userCredentials, vixConfiguration, logMsgDelegate)
        {
            
        }

        protected override Dictionary<string, string> getExtraRequestHeaders(Dictionary<string, MethodParameter> values)
        {
            return null;
        }

        protected override string getDefaultOpenFileExecutable()
        {
            return vixConfiguration.DicomViewerPath;
        }

        protected override void addOrUpdateParameters(Dictionary<string, MethodParameter> values)
        {
            values.Remove(parameterQuality);
            values.Remove(parameterContentType);
            values.Remove(parameterImageUrn);
            values.Add(parameterSiteNumber, new MethodParameter(parameterSiteNumber, true, "", true));
            values.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true, "", true));
            
        }

        protected override System.Net.ICredentials getNetworkCredentials(Dictionary<string, MethodParameter> values)
        {
            return new System.Net.NetworkCredential("", "");
        }

        public override string ToString()
        {
            return this.GetType().Name;
        }

        protected override string getImageRequestUrl(Dictionary<string, MethodParameter> values)
        {
            string siteNumber = values[parameterSiteNumber].ParameterValue;
            string patientIcn = values[parameterPatientIcn].ParameterValue;
            
            StringBuilder url = new StringBuilder();
            url.Append("http://");
            url.Append(vixConfiguration.Server);
            url.Append(":");
            url.Append(vixConfiguration.Port);
            url.Append("/AWIVWebApp/awiv/photo");
            url.Append("?siteNumber=");
            url.Append(siteNumber);
            url.Append("&patientIcn=");
            url.Append(patientIcn);

            return url.ToString();
        }
    }
}
