/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_AWIVV2.methods
{
    public class GetStudyReport : AbstractBaseAWIVMethod, IMethod
    {
        private readonly string parameterStudyUrn = "studyUrn";

        public GetStudyReport(awivV2WebReference.ImageMetadataAWIVService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate) 
            : base(service, userCredentials, vixConfiguration, logMsgDelegate)
        {
            
        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            string studyUrn = values[parameterStudyUrn].ParameterValue;

            awivV2WebReference.UserCredentialsType userCredentialsType = GetServiceUserCredentialsFromParameters(values);

            return service.getStudyReport(transactionId, userCredentialsType, studyUrn);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterStudyUrn, new MethodParameter(parameterStudyUrn, true, "urn:vastudy:660-1752-1006184063V088473", true));
            
            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterStudyUrn))
                return false;
            return true;
        }

        public override string ToString()
        {
            return this.GetType().Name;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is awivV2WebReference.GroupType)
            {
                awivV2WebReference.GroupType groupType = (awivV2WebReference.GroupType)obj;
                updatedParameters.Add(parameterStudyUrn, new StoredMethodParameterValue(parameterStudyUrn, groupType.studyUrn));
            }
            return updatedParameters;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }
    }
}
