/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_AWIVV2.methods
{
    public class PingServer : AbstractBaseAWIVMethod, IMethod
    {

        private readonly string parameterWorkstation = "workstationName";
        private readonly string parameterRequestingSiteNumber = "requestingSiteNumber";

        public PingServer(awivV2WebReference.ImageMetadataAWIVService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
            : base(service, userCredentials, vixConfiguration, logMsgDelegate)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            string workstation = values[parameterWorkstation].ParameterValue;
            string requestingSiteNumber = values[parameterRequestingSiteNumber].ParameterValue;
            awivV2WebReference.UserCredentialsType userCredentialsType = GetServiceUserCredentialsFromParameters(values);

            return service.pingServerEvent(transactionId, userCredentialsType, workstation, requestingSiteNumber);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());




            parameters.Add(parameterWorkstation, new MethodParameter(parameterWorkstation, true, "workstation-name", true));
            parameters.Add(parameterRequestingSiteNumber, new MethodParameter(parameterRequestingSiteNumber, true, "", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterWorkstation))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterRequestingSiteNumber))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> parameters = new Dictionary<string, StoredMethodParameterValue>();

            return parameters;
        }
    }
}
