/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_AWIVV2.methods
{
    public class PostImageAccessEvent : AbstractBaseAWIVMethod, IMethod
    {
        private readonly string parameterEventType = "EventType";
        private readonly string parameterImageId = "ImageId";
        private readonly string parameterPatientIcn = "PatientIcn";
        private readonly string parameterReason = "Reason";

        public PostImageAccessEvent(awivV2WebReference.ImageMetadataAWIVService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration,
            LogMsgDelegate logMsgDelegate)
            : base(service, userCredentials, vixConfiguration, logMsgDelegate)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            awivV2WebReference.ImageAccessLogEventType eventType = getImageAccessLogEventTypeFromParameters(values);
            string transactionId = values[parameterTransactionId].ParameterValue;
            return service.postImageAccessEvent(transactionId, eventType);
        }

        private awivV2WebReference.ImageAccessLogEventType getImageAccessLogEventTypeFromParameters(
            Dictionary<string, MethodParameter> values)
        {
            awivV2WebReference.ImageAccessLogEventType logEvent =
                new awivV2WebReference.ImageAccessLogEventType();

            logEvent.credentials = GetServiceUserCredentialsFromParameters(values);
            logEvent.eventType = getLogEventType(values);
            logEvent.id = values[parameterImageId].ParameterValue;
            logEvent.patientIcn = values[parameterPatientIcn].ParameterValue;
            logEvent.reason = values[parameterReason].ParameterValue;

            return logEvent;
        }

        private awivV2WebReference.ImageAccessLogEventTypeEventType getLogEventType(
            Dictionary<string, MethodParameter> values)
        {
            string value = values[parameterEventType].ParameterValue;
            switch (value)
            {
                case "IMAGE_ACCESS":
                    return awivV2WebReference.ImageAccessLogEventTypeEventType.IMAGE_ACCESS;
                case "IMAGE_COPY":
                    return awivV2WebReference.ImageAccessLogEventTypeEventType.IMAGE_COPY;
                case "IMAGE_PRINT":
                    return awivV2WebReference.ImageAccessLogEventTypeEventType.IMAGE_PRINT;
                case "PATIENT_ID_MISMATCH":
                    return awivV2WebReference.ImageAccessLogEventTypeEventType.LOG_RESTRICTED_ACCESS;
                case "LOG_RESTRICTED_ACCESS":
                    return awivV2WebReference.ImageAccessLogEventTypeEventType.PATIENT_ID_MISMATCH;
            }
            return awivV2WebReference.ImageAccessLogEventTypeEventType.IMAGE_ACCESS;
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterImageId, new MethodParameter(parameterImageId, true, "urn:vaimage:660-GM3Q-GM2A-1006184063V088473-", true));
            parameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true, "1006184063V088473", true));
            parameters.Add(parameterReason, new MethodParameter(parameterReason, false, "1", true));
            parameters.Add(parameterEventType, new MethodParameter(parameterEventType, true,
                awivV2WebReference.ImageAccessLogEventTypeEventType.IMAGE_ACCESS.ToString(), true,
                "IMAGE_ACCESS, IMAGE_COPY, IMAGE_PRINT, PATIENT_ID_MISMATCH, LOG_RESTRICTED_ACCESS"));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> parameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is awivV2WebReference.ImageType)
            {
                awivV2WebReference.ImageType imageType = (awivV2WebReference.ImageType)obj;
                parameters.Add(parameterImageId, new StoredMethodParameterValue(parameterImageId, imageType.imageurn));
            }
            return parameters;
        }
    }
}
