/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_AWIVV2.methods
{
    public class PostThinClientAccessEvent : AbstractBaseAWIVMethod, IMethod
    {
        private readonly string parameterStudyId = "StudyId";
        private readonly string parameterPatientIcn = "PatientIcn";
        private readonly string parameterScreenResolution = "ScreenResolution";
        private readonly string parameterIsGroup = "IsGroup";

        public PostThinClientAccessEvent(awivV2WebReference.ImageMetadataAWIVService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration,
            LogMsgDelegate logMsgDelegate)
            : base(service, userCredentials, vixConfiguration, logMsgDelegate)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            awivV2WebReference.ThinClientAccessLogEventType eventType = getThinClientAccessEventTypeFromParameters(values);
            string transactionId = values[parameterTransactionId].ParameterValue;
            return service.thinClientAccessEvent(transactionId, eventType);
        }

        private awivV2WebReference.ThinClientAccessLogEventType getThinClientAccessEventTypeFromParameters(
            Dictionary<string, MethodParameter> values)
        {
            awivV2WebReference.ThinClientAccessLogEventType logEvent =
                new awivV2WebReference.ThinClientAccessLogEventType();

            logEvent.credentials = GetServiceUserCredentialsFromParameters(values);
            logEvent.id = values[parameterStudyId].ParameterValue;
            logEvent.isgroup = bool.Parse(values[parameterIsGroup].ParameterValue);
            logEvent.patientIcn = values[parameterPatientIcn].ParameterValue;
            logEvent.screenresolution = values[parameterScreenResolution].ParameterValue;

            return logEvent;
        }


        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterStudyId, new MethodParameter(parameterStudyId, true, "urn:vastudy:660-19391-1006184063V088473", true));
            parameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true, "1006184063V088473", true));
            parameters.Add(parameterIsGroup, new MethodParameter(parameterIsGroup, true, "true", true));
            parameters.Add(parameterScreenResolution, new MethodParameter(parameterScreenResolution, true, "1024x768", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> parameters = new Dictionary<string, StoredMethodParameterValue>();
            /*
            if (obj is awivV2WebReference.ImageType)
            {
                awivV2WebReference.ImageType imageType = (awivV2WebReference.ImageType)obj;
                parameters.Add(parameterImageId, new StoredMethodParameterValue(parameterImageId, imageType.imageurn));
            }*/
            return parameters;
        }
    }
}
