/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_ClinicalDisplayV6.methods
{
    public class GetPatientSensitivity : AbstractBaseClinicalDisplayMethod, IMethod
    {
        private readonly string parameterPatientIcn = "PatientIcn";
        private readonly string parameterSiteNumber = "SiteNumber";

        public GetPatientSensitivity(clinicalDisplayV6WebReference.ImageMetadataClinicalDisplayService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string patientIcn = values[parameterPatientIcn].ParameterValue;
            string transactionId = values[parameterTransactionId].ParameterValue;
            string siteNumber = values[parameterSiteNumber].ParameterValue;
            clinicalDisplayV6WebReference.UserCredentialsType userCredentialsType = GetServiceUserCredentialsFromParameters(values);
            return service.getPatientSensitivityLevel(transactionId, siteNumber, patientIcn, userCredentialsType);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterSiteNumber, new MethodParameter(parameterSiteNumber, true, "", true));
            parameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true, "", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterSiteNumber))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterPatientIcn))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            return new Dictionary<string, StoredMethodParameterValue>();
        }

     
    }
}
