/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_ClinicalDisplayV6.methods
{
    public class GetStudyReport : AbstractBaseClinicalDisplayMethod, IMethod
    {
        private readonly string parameterStudyId = "StudyId";

        public GetStudyReport(clinicalDisplayV6WebReference.ImageMetadataClinicalDisplayService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            string studyId = values[parameterStudyId].ParameterValue;
            clinicalDisplayV6WebReference.UserCredentialsType userCredentialsType = GetServiceUserCredentialsFromParameters(values);

            return service.getStudyReport(transactionId, userCredentialsType, studyId);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterStudyId, new MethodParameter(parameterStudyId, true, "urn:vastudy:660-GM2A-1006184063V088473", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterStudyId))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> parameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is clinicalDisplayV6WebReference.ShallowStudyType)
            {
                clinicalDisplayV6WebReference.ShallowStudyType studyType = (clinicalDisplayV6WebReference.ShallowStudyType)obj;
                parameters.Add(parameterStudyId, new StoredMethodParameterValue(parameterStudyId, studyType.studyid));
            } 
            else if (obj is clinicalDisplayV6WebReference.FatImageType)
            {
                clinicalDisplayV6WebReference.FatImageType imageType = (clinicalDisplayV6WebReference.FatImageType)obj;
                parameters.Add(parameterStudyId, new StoredMethodParameterValue(parameterStudyId, imageType.imageid));
            }
            return parameters;
        }
    }
}
