/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using System.Net;

namespace VixClientV3_ClinicalDisplayV7.methods
{
    public class ClinicalDisplayMethodFactory : AbstractBaseMethodFactory, IMethodFactory
    {
        private const string parameterUsername = "Username";
        private const string parameterPassword = "Password";
        private const string parameterApplicationPath = "Application Path";

        private readonly string defaultUsername = "";
        private readonly string defaultPassword = "";
        private readonly string defaultApplicationPath = "/ClinicalDisplayWebApp/webservices/ImageMetadataClinicalDisplay.V7";

        public List<IMethod> getMethods(List<MethodFactoryParameter> userMethodFactoryParameters, 
            UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
        {
            List<IMethod> methods = new List<IMethod>();

            clinicalDisplayV7WebReference.ImageMetadataClinicalDisplayService service =
                new VixClientV3_ClinicalDisplayV7.clinicalDisplayV7WebReference.ImageMetadataClinicalDisplayService();

            string username = defaultUsername;
            string password = defaultPassword;
            string applicationPath = defaultApplicationPath;

            if (userMethodFactoryParameters != null)
            {
                foreach (MethodFactoryParameter param in userMethodFactoryParameters)
                {
                    switch (param.ParameterName)
                    {
                        case parameterUsername:
                            username = param.ParameterValue;
                            break;
                        case parameterPassword:
                            password = param.ParameterValue;
                            break;
                        case parameterApplicationPath:
                            applicationPath = param.ParameterValue;
                            break;
                    }
                }
            }

            service.Url = "http://" + vixConfiguration.Server + ":" + vixConfiguration.Port + applicationPath;
            service.Credentials = new NetworkCredential(username, password);
            
            methods.Add(new GetPatientShallowStudyList(service, userCredentials, vixConfiguration));
            methods.Add(new GetStudyImageList(service, userCredentials, vixConfiguration));
            methods.Add(new GetStudyReport(service, userCredentials, vixConfiguration));
            methods.Add(new GetImage(userCredentials, vixConfiguration));
            methods.Add(new PostImageAccessEvent(service, userCredentials, vixConfiguration));
            methods.Add(new GetPatientSensitivity(service, userCredentials, vixConfiguration));
            methods.Add(new GetImageDevFields(service, userCredentials, vixConfiguration));
            methods.Add(new PingServer(service, userCredentials, vixConfiguration));
            methods.Add(new GetImageInformation(service, userCredentials, vixConfiguration));
            methods.Add(new GetImageSystemGlobalNode(service, userCredentials, vixConfiguration));
            methods.Add(new GetImageAnnotations(service, userCredentials, vixConfiguration));
            methods.Add(new GetImageAnnotationDetails(service, userCredentials, vixConfiguration));
            methods.Add(new PostImageAnnotationDetails(service, userCredentials, vixConfiguration));
            methods.Add(new GetUserDetails(service, userCredentials, vixConfiguration));
            methods.Add(new GetImageAnnotationsSupported(service, userCredentials, vixConfiguration));             

            return methods;
        }

        public override string getName()
        {
            return "Clinical Display V7";
        }

        public new List<MethodFactoryParameter> getMethodFactoryParameters()
        {
            List<MethodFactoryParameter> parameters = new List<MethodFactoryParameter>();
            parameters.Add(new MethodFactoryParameter(parameterUsername, true, defaultUsername, true));
            parameters.Add(new MethodFactoryParameter(parameterPassword, true, defaultPassword, true));
            parameters.Add(new MethodFactoryParameter(parameterApplicationPath, true,
                defaultApplicationPath, true));
            return parameters;
        }
    }
}
