/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_ClinicalDisplayV7.methods
{
    public class GetImageAnnotationsSupported : AbstractBaseClinicalDisplayMethod, IMethod
    {
        private readonly string parameterSiteId = "siteId";
        

        public GetImageAnnotationsSupported(clinicalDisplayV7WebReference.ImageMetadataClinicalDisplayService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            string siteId = values[parameterSiteId].ParameterValue;            
            clinicalDisplayV7WebReference.UserCredentialsType userCredentialsType = GetServiceUserCredentialsFromParameters(values);

            return service.isAnnotationsSupported(transactionId, userCredentialsType, siteId);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterSiteId, new MethodParameter(parameterSiteId, true, "", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterSiteId))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> parameters = new Dictionary<string, StoredMethodParameterValue>();
            /*
            if (obj is clinicalDisplayV7WebReference.AnnotationType)
            {
                clinicalDisplayV7WebReference.AnnotationType annotationType = (clinicalDisplayV7WebReference.AnnotationType)obj;
                parameters.Add(parameterAnnotationId, new StoredMethodParameterValue(parameterAnnotationId, annotationType.annotationid));
                parameters.Add(parameterImageId, new StoredMethodParameterValue(parameterImageId, annotationType.imageid));
            }*/
            return parameters;
        }
    }
}
