/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_ClinicalDisplayV7.methods
{
    public class GetPatientShallowStudyList : AbstractBaseClinicalDisplayMethod, IMethod
    {
        private readonly string parameterSiteNumber = "SiteNumber";
        private readonly string parameterPatientIcn = "PatientIcn";
        private readonly string parameterAuthorizedSensitivityLevel = "authorizedSensitivityLevel";
        private readonly string parameterIncludeArtifacts = "IncludeArtifacts";
        private readonly string parameterIncludeDeletedImages = "IncludeDeletedImages";
        private readonly string parameterFilterFromDate = "Filter_FromDate";
        private readonly string parameterFilterToDate = "Filter_ToDate";

        public GetPatientShallowStudyList(clinicalDisplayV7WebReference.ImageMetadataClinicalDisplayService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            string authorizedSensitivityLevel = values[parameterAuthorizedSensitivityLevel].ParameterValue;
            string patientIcn = values[parameterPatientIcn].ParameterValue;
            string siteNumber = values[parameterSiteNumber].ParameterValue;
            bool includeArtifacts = bool.Parse(values[parameterIncludeArtifacts].ParameterValue);
            clinicalDisplayV7WebReference.FilterType filter = getFilterFromParameters(values);
            clinicalDisplayV7WebReference.UserCredentialsType userCredentialsType = GetServiceUserCredentialsFromParameters(values);

            return service.getPatientShallowStudyList(transactionId, siteNumber, patientIcn,
                filter, userCredentialsType, authorizedSensitivityLevel, includeArtifacts);
        }

        private clinicalDisplayV7WebReference.FilterType getFilterFromParameters(Dictionary<string, MethodParameter> values)
        {
            clinicalDisplayV7WebReference.FilterType filter = new clinicalDisplayV7WebReference.FilterType();

            bool includeDeletedImages = bool.Parse(values[parameterIncludeDeletedImages].ParameterValue);
            string fromDate = values[parameterFilterFromDate].ParameterValue;
            string toDate = values[parameterFilterToDate].ParameterValue;
            filter.includedeleted = includeDeletedImages;

            if (!string.IsNullOrEmpty(fromDate))
            {
                filter.fromdate = fromDate;
            }
            if (!string.IsNullOrEmpty(toDate))
            {
                filter.todate = toDate;
            }

            return filter;
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterSiteNumber, new MethodParameter(parameterSiteNumber, true, "", true));
            parameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true, "", true));
            parameters.Add(parameterAuthorizedSensitivityLevel, new MethodParameter(parameterAuthorizedSensitivityLevel, true, "2", true));
            parameters.Add(parameterIncludeArtifacts, new MethodParameter(parameterIncludeArtifacts, true, "true", true));
            parameters.Add(parameterIncludeDeletedImages, new MethodParameter(parameterIncludeDeletedImages, false, "false", true));
            parameters.Add(parameterFilterFromDate, new MethodParameter(parameterFilterFromDate, false, "", MethodParameterType.dateType, true));
            parameters.Add(parameterFilterToDate, new MethodParameter(parameterFilterToDate, false, "", MethodParameterType.dateType, true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterSiteNumber))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterAuthorizedSensitivityLevel))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterPatientIcn))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            List<AssociatedMethod> methods = new List<AssociatedMethod>();
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.ShallowStudyType), typeof(GetStudyImageList)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.ShallowStudyType), typeof(GetStudyReport)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.FatImageType), typeof(GetImage)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.FatImageType), typeof(PostImageAccessEvent)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.ShallowStudyType), typeof(GetImageInformation)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.FatImageType), typeof(GetImageInformation)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.ShallowStudyType), typeof(GetImageSystemGlobalNode)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.FatImageType), typeof(GetImageSystemGlobalNode)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.ShallowStudyType), typeof(GetImageDevFields)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV7WebReference.FatImageType), typeof(GetImageDevFields)));
            /*
            
            
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV5WebReference.FatImageType), typeof(GetImage)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV5WebReference.ShallowStudyType), typeof(GetImageInformation)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV5WebReference.FatImageType), typeof(GetImageInformation)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV5WebReference.ShallowStudyType), typeof(GetImageSystemGlobalNode)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV5WebReference.FatImageType), typeof(GetImageSystemGlobalNode)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV5WebReference.ShallowStudyType), typeof(GetImageDevFields)));
            methods.Add(new AssociatedMethod(typeof(clinicalDisplayV5WebReference.FatImageType), typeof(GetImageDevFields)));
             * */
            return methods;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> parameters = new Dictionary<string, StoredMethodParameterValue>();

            return parameters;
        }
    }
}
