/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using System.Net;

namespace VixClientV3_Exchange.methods
{
    public class GetPatientStudyGraph : AbstractBaseExchangeMethod, IMethod
    {
        private readonly string parameterPatientIcn = "patientIcn";
        private readonly string parameterDataSource = "dataSource";

        public GetPatientStudyGraph(exchangeWebReference.ImageMetadataService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {
            
        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            service.Credentials = getNetworkCredentials(values);

            exchangeWebReference.FilterType filter = GetFilterFromParameters(values);
            exchangeWebReference.RequestorType requestor = GetRequestorFromParameters(values);

            string patientIcn = values[parameterPatientIcn].ParameterValue;
            string transactionId = values[parameterTransactionId].ParameterValue;
            string datasource = values[parameterDataSource].ParameterValue;            

            return service.getPatientStudyList(datasource, requestor, filter, patientIcn, transactionId);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = getCredentialsParameters();
            addAVCredentialsParameters(parameters);
            //addRequestorParameters(parameters);
            addFilterParameters(parameters);
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true,
                "", true));
            parameters.Add(parameterDataSource, new MethodParameter(parameterDataSource, true,
                "", true));
            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterPatientIcn))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterDataSource))
                return false;

            // should check for credentials, requestor and filter params, but I'm lazy

            return true;
        }

        public override string ToString()
        {
            return this.GetType().Name;
        }

        private Dictionary<string, MethodParameter> addAVCredentialsParameters(Dictionary<string, MethodParameter> parameters)
        {
            parameters.Add("credentials_accessCode", new MethodParameter("credentials_accessCode", true, ""));
            parameters.Add("credentials_verifyCode", new MethodParameter("credentials_verifyCode", true, ""));
            return parameters;
        }

        private Dictionary<string, MethodParameter> addFilterParameters(Dictionary<string, MethodParameter> parameters)
        {
            parameters.Add("filter_fromDate", new MethodParameter("filter_fromDate", false, ""));
            parameters.Add("filter_toDate", new MethodParameter("filter_toDate", false, ""));
            parameters.Add("filter_studyId", new MethodParameter("filter_studyId", false, ""));
            return parameters;
        }

        private NetworkCredential getNetworkCredentials(Dictionary<string, MethodParameter> parameters)
        {
            string accessCode = parameters["credentials_accessCode"].ParameterValue;
            string verifyCode = parameters["credentials_verifyCode"].ParameterValue;

            return new NetworkCredential(accessCode, verifyCode);
        }

        private exchangeWebReference.RequestorType GetRequestorFromParameters(Dictionary<string, MethodParameter> values)
        {
            UserCredentials userCreds = GetUserCredentialsFromParameters(values);
            exchangeWebReference.RequestorType requestor =
                new exchangeWebReference.RequestorType();
            requestor.facilityid = userCreds.SiteNumber;
            requestor.facilityname = userCreds.SiteName;
            requestor.purposeofuse = VixClientV3_Exchange.exchangeWebReference.RequestorTypePurposeofuse.Routinehealthcare;
            requestor.ssn = userCreds.Ssn;
            requestor.username = userCreds.Name;
            /*
            requestor.facilityid = values["requestor_facilityId"].ParameterValue;
            requestor.facilityname = values["requestor_facilityName"].ParameterValue;
            requestor.purposeofuse = VixClientV3_Exchange.exchangeWebReference.RequestorTypePurposeofuse.Routinehealthcare;
            requestor.ssn = values["requestor_ssn"].ParameterValue;
            requestor.username = values["requestor_username"].ParameterValue;
             * */
            return requestor;
        }

        private exchangeWebReference.FilterType GetFilterFromParameters(Dictionary<string, MethodParameter> values)
        {
            exchangeWebReference.FilterType filter = new exchangeWebReference.FilterType();
            filter.fromdate = values["filter_fromDate"].ParameterValue;
            filter.todate = values["filter_toDate"].ParameterValue;
            filter.studyid = values["filter_studyId"].ParameterValue;
            if (filter.fromdate == "")
                filter.fromdate = null;
            if (filter.todate == "")
                filter.todate = null;
            if (filter.studyid == "")
                filter.studyid = null;
            return filter;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            List<AssociatedMethod> types = new List<AssociatedMethod>();
            types.Add(new AssociatedMethod(typeof(exchangeWebReference.InstanceType), typeof(GetImage)));
            return types;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            return new Dictionary<string, StoredMethodParameterValue>();
            
        }
    }
}
