/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using System.IO;
using System.Xml;

namespace VixClientV3_Exchange.methods
{
    class TestStudyGraph : AbstractBaseExchangeMethod, IMethod
    {
        public TestStudyGraph(exchangeWebReference.ImageMetadataService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {
            
        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            /*

            exchangeWebReference.FilterType filter = GetFilterFromParameters(values);
            exchangeWebReference.RequestorType requestor = GetRequestorFromParameters(values);

            string patientIcn = values[parameterPatientIcn].ParameterValue;
            string transactionId = values[parameterTransactionId].ParameterValue;
            string datasource = values[parameterDataSource].ParameterValue;            

            return service.getPatientStudyList(datasource, requestor, filter, patientIcn, transactionId);
             * */

            List<exchangeWebReference.StudyType> studies = new List<VixClientV3_Exchange.exchangeWebReference.StudyType>();

            XmlDocument xmlDoc = new XmlDocument();
            string filename = @"C:\eclipse\workspace.trunk\TestProject\main\src\java\gov\va\med\imaging\test\soap.xml";
            filename = @"C:\eclipse\workspace.trunk\TestProject\main\src\java\gov\va\med\imaging\test\soap2.xml";
            xmlDoc.Load(filename);

            XmlNode envelopeNode = xmlDoc.ChildNodes[1];
            XmlNode bodyNode = envelopeNode.ChildNodes[0];
            XmlNode patientStudyListResponseNode = bodyNode.ChildNodes[0];
            foreach (XmlNode studyNody in patientStudyListResponseNode.ChildNodes)
            {
                if (studyNody.Name == "study")
                {
                    exchangeWebReference.StudyType study = new VixClientV3_Exchange.exchangeWebReference.StudyType();
                    studies.Add(study);
                    foreach (XmlNode childNode in studyNody.ChildNodes)
                    {
                        switch (childNode.Name)
                        {
                            case "study-id":
                                study.studyid = getNodeValue(childNode);
                                break;
                            case "dicom-uid":
                                study.dicomuid = getNodeValue(childNode);
                                break;
                            case "description":
                                study.description = getNodeValue(childNode);
                                break;
                            case "radiology-report":
                                study.radiologyreport = getNodeValue(childNode);
                                break;
                            case "component-series":
                                study.componentseries = parseComponentSeries(childNode);
                                break;
                        }
                    }
                }
            }

            return studies.ToArray();
        }

        private exchangeWebReference.SeriesType[] parseComponentSeries(XmlNode componentSeriesNode)
        {
            List<exchangeWebReference.SeriesType> serieses = new List<VixClientV3_Exchange.exchangeWebReference.SeriesType>();
            foreach (XmlNode seriesNode in componentSeriesNode.ChildNodes)
            {
                exchangeWebReference.SeriesType series = new VixClientV3_Exchange.exchangeWebReference.SeriesType();
                serieses.Add(series);
                foreach (XmlNode childNode in seriesNode.ChildNodes)
                {
                    switch (childNode.Name)
                    {
                        case "series-id":
                            series.seriesid = getNodeValue(childNode);
                            break;
                        case "component-instances":
                            series.componentinstances = parseComponentInstances(childNode);
                            break;
                    }
                }
                
            }
            return serieses.ToArray();
        }

        private exchangeWebReference.InstanceType[] parseComponentInstances(XmlNode componentInstancesNode)
        {
            List<exchangeWebReference.InstanceType> instances = new List<VixClientV3_Exchange.exchangeWebReference.InstanceType>();
            foreach (XmlNode instanceNode in componentInstancesNode.ChildNodes)
            {
                exchangeWebReference.InstanceType instance = new VixClientV3_Exchange.exchangeWebReference.InstanceType();
                instances.Add(instance);
                foreach (XmlNode childNode in instanceNode.ChildNodes)
                {
                    switch (childNode.Name)
                    {
                        case "image-urn":
                            instance.imageurn = getNodeValue(childNode);
                            break;
                        case "dicom-uid":
                            instance.dicomuid = getNodeValue(childNode);
                            break;
                        case "dicom-instance-number":
                            int res = -1;
                            if (int.TryParse(getNodeValue(childNode), out res))
                            {
                                instance.dicominstancenumber = res;
                            }                            
                            break;
                    }
                }
                
            }

            return instances.ToArray();
        }

        private string getNodeValue(XmlNode node)
        {
            return node.InnerText;
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = getCredentialsParameters();
            
            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {            
            return true;
        }

        public override string ToString()
        {
            return this.GetType().Name;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            List<AssociatedMethod> types = new List<AssociatedMethod>();
            return types;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            return new Dictionary<string, StoredMethodParameterValue>();
            
        }
    }
}
