/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using System.Net;

namespace VixClientV3_ExchangeV2.methods
{
    public class AbstractBaseExchangeMethod : AbstractBaseMethod
    {
        protected exchangeV2WebReference.ImageMetadataService service = null;

        private readonly string parameterAccessCode = "credentials_accessCode";
        private readonly string parameterVerifyCode = "credentials_verifyCode";

        public AbstractBaseExchangeMethod(exchangeV2WebReference.ImageMetadataService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(userCredentials, vixConfiguration)
        {
            this.service = service;
        }

        protected Dictionary<string, MethodParameter> addAVCredentialsParameters(Dictionary<string, MethodParameter> parameters)
        {
            parameters.Add(parameterAccessCode, new MethodParameter(parameterAccessCode, true, "", true));
            parameters.Add(parameterVerifyCode, new MethodParameter(parameterVerifyCode, true, "", true));
            return parameters;
        }

        protected NetworkCredential getNetworkCredentials(Dictionary<string, MethodParameter> parameters)
        {
            string accessCode = parameters[parameterAccessCode].ParameterValue;
            string verifyCode = parameters[parameterVerifyCode].ParameterValue;

            return new NetworkCredential(accessCode, verifyCode);
        }

        protected exchangeV2WebReference.RequestorType GetRequestorFromParameters(Dictionary<string, MethodParameter> values)
        {
            UserCredentials userCreds = GetUserCredentialsFromParameters(values);
            exchangeV2WebReference.RequestorType requestor =
                new exchangeV2WebReference.RequestorType();
            requestor.facilityid = userCreds.SiteNumber;
            requestor.facilityname = userCreds.SiteName;
            requestor.purposeofuse = exchangeV2WebReference.RequestorTypePurposeofuse.Routinehealthcare;
            requestor.ssn = userCreds.Ssn;
            requestor.username = userCreds.Name;
            /*
            requestor.facilityid = values["requestor_facilityId"].ParameterValue;
            requestor.facilityname = values["requestor_facilityName"].ParameterValue;
            requestor.purposeofuse = VixClientV3_Exchange.exchangeWebReference.RequestorTypePurposeofuse.Routinehealthcare;
            requestor.ssn = values["requestor_ssn"].ParameterValue;
            requestor.username = values["requestor_username"].ParameterValue;
             * */
            return requestor;
        }
    }
}
