/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_ExchangeV2.methods
{
    public class ExchangeV2MethodFactory : AbstractBaseMethodFactory, IMethodFactory
    {
        private const string parameterMetadataPath = "Metadata Path";
        private const string parameterImagePath = "Image Path";

        private const string defaultMetadataPath = "/ImagingExchangeWebApp/xchange-ws/ImageMetadata.V2";
        private const string defaultImagePath = "/ImagingExchangeWebApp/xchange/xchange";

        public List<IMethod> getMethods(List<MethodFactoryParameter> userMethodFactoryParameters,
            UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
        {
            List<IMethod> methods = new List<IMethod>();

            string metadataPath = defaultMetadataPath;
            string imagePath = defaultImagePath;
            if (userMethodFactoryParameters != null)
            {
                foreach (MethodFactoryParameter parameter in userMethodFactoryParameters)
                {
                    switch (parameter.ParameterName)
                    {
                        case parameterMetadataPath:
                            metadataPath = parameter.ParameterValue;
                            break;
                        case parameterImagePath:
                            imagePath = parameter.ParameterValue;
                            break;
                    }
                }
            }

            exchangeV2WebReference.ImageMetadataService service = new VixClientV3_ExchangeV2.exchangeV2WebReference.ImageMetadataService();
            service.Url = "http://" + vixConfiguration.Server + ":" + vixConfiguration.Port + metadataPath;// "/ImagingExchangeWebApp/xchange-ws/ImageMetadata.V2";
            LogMsg("Setting ExchangeV2 service URL to '" + service.Url + "'.");
            methods.Add(new GetPatientStudyGraph(service, userCredentials, vixConfiguration));
            methods.Add(new GetStudyReport(service, userCredentials, vixConfiguration));
            methods.Add(new GetImage(userCredentials, vixConfiguration, imagePath));
            //methods.Add(new GetImage(userCredentials, vixConfiguration));
            //methods.Add(new TestStudyGraph(service, userCredentials, vixConfiguration));
            return methods;
        }

        public override string getName()
        {
            return "ExchangeV2";
        }

        public override string ToString()
        {
            return getName();
        }

        public List<MethodFactoryParameter> getMethodFactoryParameters()
        {
            List<MethodFactoryParameter> parameters = new List<MethodFactoryParameter>();
            parameters.Add(new MethodFactoryParameter(parameterMetadataPath, true, defaultMetadataPath, true));
            parameters.Add(new MethodFactoryParameter(parameterImagePath, true, defaultImagePath, true));
            return parameters;
        }
    }
}
