/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_ExchangeV2.methods
{
    public class GetImage : AbstractBaseGetImageMethod
    {
        private readonly string mstrImagePath;
        public readonly string parameterAllowFederationCompression = "allowFederationCompression";

        public GetImage(UserCredentials userCredentials, VixConfiguration vixConfiguration, string imagePath)
            : base(false, userCredentials, vixConfiguration)
        {
            mstrImagePath = imagePath;
        }

        protected override string getDefaultOpenFileExecutable()
        {
            return vixConfiguration.DicomViewerPath;
        }

        protected override Dictionary<string, string> getExtraRequestHeaders(Dictionary<string, MethodParameter> values)
        {
            if (values != null)
            {
                if (values.ContainsKey(parameterAllowFederationCompression))
                {
                    Dictionary<string, string> result = new Dictionary<string, string>();
                    result.Add("xxx-AllowAddFederationCompression", values[parameterAllowFederationCompression].ParameterValue);
                    return result;
                }
            }
            return null;
        }

        protected override void addOrUpdateParameters(Dictionary<string, MethodParameter> values)
        {
            values.Add("credentials_accessCode", new MethodParameter("credentials_accessCode", true, ""));
            values.Add("credentials_verifyCode", new MethodParameter("credentials_verifyCode", true, ""));
            values.Add(parameterAllowFederationCompression, new MethodParameter(parameterAllowFederationCompression, false, "true", true));
            values[parameterContentType].ParameterValue = "application/dicom";
        }

        protected override System.Net.ICredentials getNetworkCredentials(Dictionary<string, MethodParameter> values)
        {

            string accessCode = values["credentials_accessCode"].ParameterValue;
            string verifyCode = values["credentials_verifyCode"].ParameterValue;

            return new System.Net.NetworkCredential(accessCode, verifyCode);           
        }

        public override string ToString()
        {
            return this.GetType().Name;
        }

        protected override string getImageRequestUrl(Dictionary<string, MethodParameter> values)
        {
            string imageUrn = values[parameterImageUrn].ParameterValue;
            string quality = values[parameterQuality].ParameterValue;
            string contentType = values[parameterContentType].ParameterValue;
            StringBuilder url = new StringBuilder();

            url.Append("http://");
            url.Append(vixConfiguration.Server);
            url.Append(":");
            url.Append(vixConfiguration.Port);
            //url.Append("/ImagingExchangeWebApp/xchange/xchange");
            url.Append(mstrImagePath);
            url.Append("?imageURN=");
            url.Append(imageUrn);
            url.Append("&imageQuality=");
            url.Append(quality);
            url.Append("&contentType=");
            url.Append(contentType);

            return url.ToString();
        }

        public override Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is exchangeV2WebReference.InstanceType)
            {
                exchangeV2WebReference.InstanceType imageType = (exchangeV2WebReference.InstanceType)obj;
                updatedParameters.Add(parameterImageUrn, new MethodParameter(parameterImageUrn, true, imageType.imageurn));
            }
            return updatedParameters;
        }
    }
}
