/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_ExchangeV2.methods
{
    public class GetStudyReport : AbstractBaseExchangeMethod, IMethod
    {
        private readonly string parameterDataSource = "dataSource";
        private readonly string parameterStudyId = "studyId";
        private readonly string parameterPatientIcn = "patientIcn";

        public GetStudyReport(exchangeV2WebReference.ImageMetadataService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {

            service.Credentials = getNetworkCredentials(values);
            
            exchangeV2WebReference.RequestorType requestor = GetRequestorFromParameters(values);

            string studyId = values[parameterStudyId].ParameterValue;
            string transactionId = values[parameterTransactionId].ParameterValue;
            string datasource = values[parameterDataSource].ParameterValue;
            string patientIcn = values[parameterPatientIcn].ParameterValue;

            return service.getPatientReport(datasource, requestor, patientIcn, transactionId, studyId);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = getCredentialsParameters();
            addAVCredentialsParameters(parameters);
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true,
                "", true));
            parameters.Add(parameterStudyId, new MethodParameter(parameterStudyId, true,
                "study-id", true));
            parameters.Add(parameterDataSource, new MethodParameter(parameterDataSource, true,
                "", true));
            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterPatientIcn))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterDataSource))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterStudyId))
                return false;

            // should check for credentials, requestor and filter params, but I'm lazy

            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            List<AssociatedMethod> types = new List<AssociatedMethod>();
            return types;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is exchangeV2WebReference.StudyType)
            {
                exchangeV2WebReference.StudyType studyType = (exchangeV2WebReference.StudyType)obj;
                updatedParameters.Add(parameterStudyId, new MethodParameter(parameterStudyId, true, studyType.studyid));
                updatedParameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, true, studyType.patientid));
            }
            return updatedParameters;
        }

        
    }
}
