/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using RestSharp;
using VixClientV3_Passthrough.types;

namespace VixClientV3_Passthrough.methods
{
    public class ExecuteMethod : AbstractBaseMethod, IMethod
    {
        private PassthroughService passthroughService;

        private readonly string parameterContext = "context";
        private readonly string parameterSiteId = "siteId";
        private readonly string parameterMethodParameters = "methodParameters";

        public ExecuteMethod(PassthroughService passthroughService, UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate) 
            : base(userCredentials, vixConfiguration, logMsgDelegate)
        {
            this.passthroughService = passthroughService;
        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            string context = values[parameterContext].ParameterValue;
            string siteId = values[parameterSiteId].ParameterValue;
            context = "MAGTP_WORKLIST_MGR";
            context = "MAG_WINDOWS";
            string methodUrl = "passthrough/" + siteId + "/" + context;

            string xmlbody = "<passthroughMethodType><methodName>MAG4 GET IMAGE INFO</methodName><parameters><index>0</index><parameterType>literal</parameterType><value>1785</value></parameters></passthroughMethodType>";
            xmlbody = "<passthroughMethodType><methodName>MAGTP GET ACTIVE</methodName><parameters><index>0</index><parameterType>literal</parameterType><value>0</value></parameters></passthroughMethodType>";
            xmlbody = "<passthroughMethodType><methodName>MAGG PAT INFO</methodName><parameters><parameter><index>0</index><parameterType>literal</parameterType><value>711^^0</value></parameter></parameters></passthroughMethodType>";

            string result = passthroughService.Execute(methodUrl, xmlbody, transactionId);
            return result;
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterContext, new MethodParameter(parameterContext, true, "MAG WINDOWS", true));
            parameters.Add(parameterSiteId, new MethodParameter(parameterSiteId, true, "", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterContext))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterSiteId))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            List<AssociatedMethod> types = new List<AssociatedMethod>();
            /*
            types.Add(new AssociatedMethod(typeof(ROIRequestType), typeof(GetQueueRequestStatus)));
            types.Add(new AssociatedMethod(typeof(ROIRequestType), typeof(GetROIDisclosure)));
             * */
            return types;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            /*
            if (obj is ROIRequestType)
            {
                ROIRequestType requestType = (ROIRequestType)obj;
                updatedParameters.Add(parameterGuid, new StoredMethodParameterValue(parameterGuid, requestType.Guid));
            }*/
            return updatedParameters;
        }
    }
}
