/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using RestSharp;
using System.Net;
using System.IO;

namespace VixClientV3_Passthrough.methods
{
    public class PassthroughService
    {
        private readonly string username;
        private readonly string password;
        private readonly string baseUrl;

        public PassthroughService(string baseUrl, string username, string password)
        {
            this.username = username;
            this.password = password;
            this.baseUrl = baseUrl;
            ServicePointManager.Expect100Continue = false;
        }

        /*
        public string Execute(RestRequest request) 
        {
            var client = new RestClient();
            client.BaseUrl = baseUrl;
            client.Authenticator = new HttpBasicAuthenticator(username, password);
            RestResponse response = client.Execute(request);
            return response.Content;
        }*/

        public string Execute(string methodUrl, string body, string transactionId)
        {
            string url = baseUrl + methodUrl;
            HttpWebRequest request = (HttpWebRequest) HttpWebRequest.Create(url);
            request.PreAuthenticate = true;

            request.Method = "POST";
            request.Accept = "text/plain,application/xml";
            request.Headers.Add("xxx-transaction-id", transactionId);
            request.ContentType = "application/xml; charset=UTF-8;";
            
            request.Credentials = new NetworkCredential(username, password);

            byte[] byteArray = Encoding.UTF8.GetBytes(body);

            Stream requestStream = request.GetRequestStream();
            requestStream.Write(byteArray, 0, byteArray.Length);
            requestStream.Close();

            WebResponse response = request.GetResponse();
            Stream responseStream = response.GetResponseStream();
            StreamReader reader = new StreamReader(responseStream);
            StringBuilder result = new StringBuilder();
            string line = reader.ReadLine();
            while (line != null)
            {
                result.AppendLine(line);
                line = reader.ReadLine();
            }

            return result.ToString();
        }
    }
}
