/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using VixClientV3_ROI.types;

namespace VixClientV3_ROI.methods
{
    public class GetROIDisclosure : AbstractBaseGetImageMethod, IMethod
    {
        private readonly string parameterGuid = "guid";
        private readonly string parameterStudyId = "studyId";

        public GetROIDisclosure(UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate) 
            :base(true, userCredentials, vixConfiguration, logMsgDelegate, "GET")
        {
            
        }

        protected override void addOrUpdateParameters(Dictionary<string, MethodParameter> values)
        {
            values.Remove(parameterQuality);
            values.Remove(parameterContentType);
            values.Remove(parameterImageUrn);

            values.Add(parameterGuid, new MethodParameter(parameterGuid, true, "", true));
            values.Add(parameterStudyId, new MethodParameter(parameterStudyId, true, "", true));
            MethodParameter filenameParam = GetFilenameParameter(parameterFilename);
            filenameParam.ParameterValue = filenameParam.ParameterValue + ".zip";
            values[parameterFilename] = filenameParam;
        }

        protected override Dictionary<string, string> getExtraRequestHeaders(Dictionary<string, MethodParameter> values)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();

            return result;
        }

        protected override string getDefaultOpenFileExecutable()
        {
            return "Explorer.exe";
        }

        protected override string getImageRequestUrl(Dictionary<string, MethodParameter> values)
        {
            string studyId = values[parameterStudyId].ParameterValue;
            string guid = values[parameterGuid].ParameterValue;
            StringBuilder url = new StringBuilder();
            url.Append("http://");
            url.Append(vixConfiguration.Server);
            url.Append(":");
            url.Append(vixConfiguration.Port);
            url.Append("/ROIWebApp/disclosure");
            url.Append("?studyId=");
            url.Append(studyId);
            url.Append("&guid=");
            url.Append(guid);

            return url.ToString();
        }

        protected override System.Net.ICredentials getNetworkCredentials(Dictionary<string, MethodParameter> values)
        {
            return new System.Net.NetworkCredential("", "");
        }

        public override Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is roiRequestType)
            {
                roiRequestType requestType = (roiRequestType)obj;
                updatedParameters.Add(parameterGuid, new StoredMethodParameterValue(parameterGuid, requestType.Guid));
                updatedParameters.Add(parameterStudyId, new StoredMethodParameterValue(parameterStudyId, requestType.StudyId));
            }
            return updatedParameters;
        }
        
    }
}
