/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using RestSharp;

namespace VixClientV3_Study.methods
{
    public class GetFilteredSitePatientStudies : GetSitePatientStudies
    {
        private readonly string parameterFilterFromDate = "Filter_FromDate";
        private readonly string parameterFilterToDate = "Filter_ToDate";
        private readonly string parameterFilterResultType = "Filter_ResultType";

        public GetFilteredSitePatientStudies(StudyService studyService, UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate) 
            : base(studyService, userCredentials, vixConfiguration, logMsgDelegate)
        {
            
        }

        protected override RestSharp.RestRequest CreateRestRequest(Dictionary<string, MethodParameter> values)
        {
            string patientIcn = values[parameterPatientIcn].ParameterValue;
            string fromDate = "";
            string toDate = "";
            if (values.ContainsKey(parameterFilterFromDate))
            {
                fromDate = values[parameterFilterFromDate].ParameterValue;
            }
            if (values.ContainsKey(parameterFilterToDate))
            {
                toDate = values[parameterFilterToDate].ParameterValue;
            }


            /**
            <studyFilterType>
                <event>event</event>
                <fromDate>2013-03-15T12:05:52.736-04:00</fromDate>
                <origin>origin</origin>
                <specialty>specialty</specialty>
                <toDate>2013-03-15T12:05:52.736-04:00</toDate>
                <types>types</types>
                <_class>class</_class>
                <_package>package</_package>
            </studyFilterType>
             * 
             * 
             * 
             <studyFilter>
                <fromDate>2010-01-01</fromDate>
                <toDate>2012-12-31</toDate>
                <resultType>all</resultType>
            </studyFilter>

                * 
                */


            StringBuilder body = new StringBuilder();
            body.AppendLine("<studyFilter>");

            string filterResultType = values[parameterFilterResultType].ParameterValue;


            body.AppendLine("<resultType>" + filterResultType + "</resultType>");


            string siteId = values[parameterSiteId].ParameterValue;
            RestRequest request = new RestRequest("studies/site/icn/" + siteId + "/" + patientIcn, Method.POST);


            if (!String.IsNullOrEmpty(fromDate))
            {
                body.AppendLine("<fromDate>" + fromDate + "</fromDate>");
            }
            else
            {
                body.AppendLine("<fromDate></fromDate>");
            }
            if (!string.IsNullOrEmpty(toDate))
            {
                body.AppendLine("<toDate>" + toDate + "</toDate>");
            }
            else
            {
                body.AppendLine("<toDate></toDate>");
            }

            body.AppendLine("</studyFilter>");

            request.AddParameter("application/xml; charset=utf-8",
                body.ToString(), ParameterType.RequestBody);

            return request;
        }

        public override Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = base.getParameters();

            parameters.Add(parameterFilterFromDate, new MethodParameter(parameterFilterFromDate, false, "2001-01-01", true));
            parameters.Add(parameterFilterToDate, new MethodParameter(parameterFilterToDate, false, "2005-01-01", true));
            parameters.Add(parameterFilterResultType, new MethodParameter(parameterFilterResultType, false, "all", true));

            return parameters;
        }
    }
}
