/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using VixClientV3_Study.types;

namespace VixClientV3_Study.methods
{
    public class GetPatientPhotoIDImage : AbstractBaseGetImageMethod
    {
        protected static string parameterPatientId = "patientId";
        protected static string parameterSiteId = "siteId";

        public GetPatientPhotoIDImage(UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
            : base(true, userCredentials, vixConfiguration, logMsgDelegate)
        {

        }

        protected override string getDefaultOpenFileExecutable()
        {
            return vixConfiguration.DicomViewerPath;
        }

        protected override Dictionary<string, string> getExtraRequestHeaders(Dictionary<string, MethodParameter> values)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();

            return result;
        }

        protected override string getImageRequestUrl(Dictionary<string, MethodParameter> values)
        {
            string patientId = values[parameterPatientId].ParameterValue;
            string siteId = values[parameterSiteId].ParameterValue;
            StringBuilder url = new StringBuilder();
            url.Append("http://");
            url.Append(vixConfiguration.Server);
            url.Append(":");
            url.Append(vixConfiguration.Port);
            url.Append("/ImageWebApp/photo");
            url.Append("/id/");
            url.Append(siteId);
            url.Append("/");
            url.Append(patientId);

            return url.ToString();
        }

        protected override void addOrUpdateParameters(Dictionary<string, MethodParameter> values)
        {
            values.Remove(parameterQuality);
            values.Remove(parameterImageUrn);
            values.Remove(parameterQuality);
            values.Remove(parameterContentType);
            
            values.Add(parameterPatientId, new MethodParameter(parameterPatientId, true, "", true, "Patient Identifier"));
            values.Add(parameterSiteId, new MethodParameter(parameterSiteId, true, "", true, "Site Number"));
        }

        protected override System.Net.ICredentials getNetworkCredentials(Dictionary<string, MethodParameter> values)
        {
            return new System.Net.NetworkCredential("", "");
        }

        public override Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is StudyType )
            {
                StudyType studyType = (StudyType)obj;
                updatedParameters.Add(parameterPatientId, new StoredMethodParameterValue(parameterPatientId, studyType.PatientId));
                updatedParameters.Add(parameterSiteId, new StoredMethodParameterValue(parameterSiteId, studyType.SiteNumber));
            }
            return updatedParameters;
        }
    }

}
