/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using RestSharp;
using VixClientV3_Study.types;

namespace VixClientV3_Study.methods
{
    public class GetStudyReport: AbstractBaseMethod, IMethod
    {
        private StudyService studyService;

        private readonly string parameterStudyId = "StudyId";

        public GetStudyReport(StudyService studyService, UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate) 
            : base(userCredentials, vixConfiguration, logMsgDelegate)
        {
            this.studyService = studyService;
        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            if (!values.ContainsKey(parameterTransactionId))
            {
                throw new Exception("Missing transaction ID key");
            }
            if (!values.ContainsKey(parameterStudyId))
            {
                throw new Exception("Missing Study ID key");
            }

            string transactionId = values[parameterTransactionId].ParameterValue;
            string studyId = values[parameterStudyId].ParameterValue;
            string uri = "study/report/" + studyId;

            RestRequest request = new RestRequest(uri, Method.GET);
            request.AddHeader("xxx-transaction-id", transactionId);

            request.AddHeader("xxx-fullname", userCredentials.Name);
            request.AddHeader("xxx-sitename", userCredentials.SiteName);
            request.AddHeader("xxx-sitenumber", userCredentials.SiteNumber);
            request.AddHeader("xxx-ssn", userCredentials.Ssn);
            request.AddHeader("xxx-duz", userCredentials.Duz);

            request.AddHeader("xxx-cacheLocationId", userCredentials.CacheLocationId);
            request.AddHeader("xxx-securityToken", userCredentials.BseToken);
            request.AddHeader("xxx-client-version", vixConfiguration.ClientVersion);

            LogMsg("Getting study '" + studyId + "' information");
            LogMsg("Request URL: " + request.Resource);
            RestStringType study = studyService.ExecuteAndDeserializeRequest<RestStringType>(request);
            return study;

        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterStudyId, new MethodParameter(parameterStudyId, true, "urn:vastudy:660-1783-1006184063V088473", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterStudyId))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            List<AssociatedMethod> types = new List<AssociatedMethod>();
            types.Add(new AssociatedMethod(typeof(StudyImageType), typeof(GetThumbnailImage)));
            types.Add(new AssociatedMethod(typeof(StudyImageType), typeof(GetReferenceImage)));
            types.Add(new AssociatedMethod(typeof(StudyImageType), typeof(GetDiagnosticImage)));
            types.Add(new AssociatedMethod(typeof(StudyImageType), typeof(GetPatientPhotoIDImage)));
            types.Add(new AssociatedMethod(typeof(StudyType), typeof(GetStudyImages)));
            return types;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            return new Dictionary<string, StoredMethodParameterValue>();
        }
    }
}
