/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_Study.methods
{
    public class StudyMethodFactory : AbstractBaseMethodFactory, IMethodFactory
    {
        public List<IMethod> getMethods(List<MethodFactoryParameter> userMethodFactoryParameters,
            UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
        {
            string baseUrl = "http://" + vixConfiguration.Server + ":" + vixConfiguration.Port + "/Study/restservices/";
            LogMsg("Creating StudyService with baseURL: " + baseUrl);
            
            
            StudyService studyService = new StudyService(baseUrl, "", "");
                        
            List<IMethod> methods = new List<IMethod>();
            methods.Add(new GetSitePatientStudies(studyService, userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetThumbnailImage (userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetReferenceImage(userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetDiagnosticImage(userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetPatientPhotoIDImage(userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetStudyImages(studyService, userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetStudyReport(studyService, userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetFilteredSitePatientStudies(studyService, userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetVaPatientStudies(studyService, userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetFilteredVaPatientStudies(studyService, userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetLocalSitePatientStudies(studyService, userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetFilteredLocalSitePatientStudies(studyService, userCredentials, vixConfiguration, logMsgDelegate));
          
            return methods;
        }

        public override string getName()
        {
            return "Study";
        }

        public override string ToString()
        {
            return getName();
        }

        public List<MethodFactoryParameter> getMethodFactoryParameters()
        {
            return null;
        }    
    }
}
