/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using VixClientV3Common;

namespace VixClientV3_VRad.methods
{
    public abstract class AbstractGetExamImage : AbstractBaseGetImageMethod, IMethod
    {

        public AbstractGetExamImage(UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(true, userCredentials, vixConfiguration)
        {

        }

        public AbstractGetExamImage(UserCredentials userCredentials, VixConfiguration vixConfiguration, string httpRequestMethod)
            : base(true, userCredentials, vixConfiguration, null, httpRequestMethod)
        {
            
        }

        public AbstractGetExamImage(UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
            : base(true, userCredentials, vixConfiguration, logMsgDelegate)
        {

        }

        protected abstract string getImageServlet();
        protected abstract string getExtraUrlParameters();

        protected override void addOrUpdateParameters(Dictionary<string, MethodParameter> values)
        {
            values[parameterQuality].ParameterValue = "90";
            values[parameterContentType].ParameterValue = "application/dicom,image/x-targa";
        }

        protected override ICredentials getNetworkCredentials(Dictionary<string, MethodParameter> values)
        {
            return new NetworkCredential("", "");
        }

        protected override string getDefaultOpenFileExecutable()
        {
            return vixConfiguration.DicomViewerPath;
        }

        protected override Dictionary<string, string> getExtraRequestHeaders(Dictionary<string, MethodParameter> values)
        {
            return null;
        }

        protected override string getImageRequestUrl(Dictionary<string, MethodParameter> values)
        {
            string imageUrn = values[parameterImageUrn].ParameterValue;
            string quality = values[parameterQuality].ParameterValue;
            string contentType = values[parameterContentType].ParameterValue;

            StringBuilder url = new StringBuilder();
            url.Append("http://");
            url.Append(vixConfiguration.Server);
            url.Append(":" );
            url.Append(vixConfiguration.Port);
            url.Append("/VistaRadWebApp/");
            url.Append(getImageServlet());
            url.Append("?imageURN=");
            url.Append(imageUrn);
            url.Append("&imageQuality=");
            url.Append(quality);
            url.Append("&contentType=");
            url.Append(contentType);
            url.Append(getExtraUrlParameters());
            return url.ToString();
        }

        public override Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is vistaRadWebReference.ExamImageType)
            {
                vistaRadWebReference.ExamImageType imageType = (vistaRadWebReference.ExamImageType)obj;
                updatedParameters.Add(parameterImageUrn, new StoredMethodParameterValue(parameterImageUrn, imageType.imageid));
            }

            return updatedParameters;
        }
    }
}
