/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_VRad.methods
{
    public class PrefetchExams : AbstractBaseVistaRadMethod, IMethod
    {
        private readonly string parameterExamId = "examId";

        public PrefetchExams(vistaRadWebReference.ImageMetadataVistaRadService service,
            UserCredentials userCredentials, VixConfiguration vixConfiguration)
            : base(service, userCredentials, vixConfiguration)
        {
            
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!this.ValidateCredentials(values))
                return false;

            if (!CheckParameterNotEmptyOrNull(values, parameterTransactionId))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterExamId))
                return false;

            return true;
        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            vistaRadWebReference.UserCredentials credentials = GetServiceUserCredentialsFromParameters(values);
            return service.prefetchExam(values[parameterTransactionId].ParameterValue, credentials,
                values[parameterExamId].ParameterValue);            
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;
            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterExamId, new MethodParameter(parameterExamId, true,
                "urn:vastudy:660-GEZDAN26GY4TMOBYG42S4OBVGY3V4MK6GEZDO-9299999994V735878", true));
            return parameters;
        }

        public override string ToString()
        {
            return "PrefetchExams";
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is vistaRadWebReference.ExamTypeDetails)
            {
                vistaRadWebReference.ExamTypeDetails examDetails = (vistaRadWebReference.ExamTypeDetails)obj;
                updatedParameters.Add(parameterExamId, new StoredMethodParameterValue(parameterExamId, examDetails.examid));
            }

            return updatedParameters;
        }
    }
}
