/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using System.Net;


namespace VixClientV3_VRad.methods
{
    public class VistaRadMethodFactory : AbstractBaseMethodFactory, IMethodFactory
    {

        public List<IMethod> getMethods(List<MethodFactoryParameter> userMethodFactoryParameters,
            UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
        {
            List<IMethod> methods = new List<IMethod>();
            vistaRadWebReference.ImageMetadataVistaRadService service = new VixClientV3_VRad.vistaRadWebReference.ImageMetadataVistaRadService();
            
            service.Url = "http://" + vixConfiguration.Server + ":" + vixConfiguration.Port + "/VistaRadWebApp/vistarad-ws/ImageMetadataVistarad.V1";
            service.Credentials = new NetworkCredential("", "");
            // create new VRad service and put it in here
            methods.Add(new GetSiteExamList(service, userCredentials, vixConfiguration));
            methods.Add(new GetExamImage(userCredentials, vixConfiguration));
            methods.Add(new GetExamImageFromCache(userCredentials, vixConfiguration));
            methods.Add(new GetExamImageNotFromCache(userCredentials, vixConfiguration));
            methods.Add(new GetExamImageTextFile(userCredentials, vixConfiguration));
            methods.Add(new GetExamImageTextFileFromCache(userCredentials, vixConfiguration));
            methods.Add(new GetExamDetails(service, userCredentials, vixConfiguration));
            methods.Add(new GetActiveWorklist(service, userCredentials, vixConfiguration));
            methods.Add(new GetExamRequisitionReport(service, userCredentials, vixConfiguration));
            methods.Add(new GetExamReport(service, userCredentials, vixConfiguration));
            methods.Add(new PostImageAccessEvent(service, userCredentials, vixConfiguration));
            methods.Add(new PrefetchExams(service, userCredentials, vixConfiguration));
            methods.Add(new GetExamSiteMetadataCacheStatus(service, userCredentials, vixConfiguration));
            methods.Add(new PingServerEvent(service, userCredentials, vixConfiguration));
            methods.Add(new GetExamImageFormat(userCredentials, vixConfiguration));
            return methods;
        }

        public override string getName()
        {
            return "VistARad";
        }

        public override string ToString()
        {
            return getName();
        }

        public List<MethodFactoryParameter> getMethodFactoryParameters()
        {
            return null;
        }
    }
}
