/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_XCA.methods
{
    public class GetDocument : AbstractBaseXcaMethod, IMethod
    {
        private readonly string parameterRetrieveUri = "Retrieve URI";
        private readonly string parameterHomeCommunityId = "Home Community Id";
        private readonly string parameterRepositoryId = "Repository Id";
        private readonly string parameterDocumentId = "Document Id";
        protected readonly string parameterFilename = "filename";
        protected readonly string parameterOpenImageEXE = "openImageEXE";
        private readonly string parameterHostname = "Hostname";
        private readonly string parameterPort = "Port";
        private readonly string parameterProtocol = "Protocol";
        private readonly string parameterUsername = "Username";
        private readonly string parameterPassword = "Password";
        private readonly string parameterDebugMode = "Debug Mode";

        public static string factoryParameterHAIMSRepositoryId = "HAIMS Repository ID";
        public static string factoryParameterNCATRepositoryId = "NCAT Repository ID";
        public static string factoryParameterVAHomeCommunityId = "VA Home Community ID";

        public static string factoryParameterHAIMSHostname = "HAIMS Hostname";
        public static string factoryParameterHAIMSPort = "HAIMS Port";
        public static string factoryParameterHAIMSUri = "HAIMS Uri";
        public static string factoryParameterHAIMSProtocol = "HAIMS Protocol";

        public static string factoryParameterNCATHostname= "NCAT Hostname";
        public static string factoryParameterNCATPort = "NCAT Port";
        public static string factoryParameterNCATUri = "NCAT Uri";
        public static string factoryParameterNCATProtocol = "NCAT Protocol";
        public static string factoryParameterNCATUsername = "NCAT Username";
        public static string factoryParameterNCATPassword = "NCAT Password";

        public static string factoryParameterVAProtocol = "VA Protocol";
        public static string factoryParameterVAUsername = "VA Username";
        public static string factoryParameterVAPassword = "VA Password";
        public static string factoryParameterVAPort = "VA Port";

        private Dictionary<string, MethodFactoryParameter> methodFactoryParameters;

        public GetDocument(UserCredentials userCredentials, VixConfiguration vixConfiguration,
            List<MethodFactoryParameter> userMethodFactoryParameters, LogMsgDelegate logMsgDelegate)
            : base(userCredentials, vixConfiguration, logMsgDelegate)
        {
            //this.userMethodFactoryParameters = userMethodFactoryParameters;
            methodFactoryParameters = new Dictionary<string, MethodFactoryParameter>();
            if (userMethodFactoryParameters != null)
            {
                foreach (MethodFactoryParameter parameter in userMethodFactoryParameters)
                {
                    methodFactoryParameters.Add(parameter.ParameterName, parameter);
                }
            }
        }

        /*
        protected override string getDefaultOpenFileExecutable()
        {
            return vixConfiguration.DicomViewerPath;
        }*/

        public override string ToString()
        {
            return this.GetType().Name;
        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            UserCredentials userCred = GetUserCredentialsFromParameters(values);
            string retrieveUri = values[parameterRetrieveUri].ParameterValue;
            string homeCommunityId = values[parameterHomeCommunityId].ParameterValue;
            string repositoryId = values[parameterRepositoryId].ParameterValue;
            string id = values[parameterDocumentId].ParameterValue;
            string filename = values[parameterFilename].ParameterValue;

            string host = values[parameterHostname].ParameterValue;
            int port = int.Parse(values[parameterPort].ParameterValue);
            string protocol = values[parameterProtocol].ParameterValue;

            string openImageExe = null;
            if (values.ContainsKey(parameterOpenImageEXE))
            {
                openImageExe = values[parameterOpenImageEXE].ParameterValue;
            }
            string username = null;
            if (values.ContainsKey(parameterUsername))
            {
                username = values[parameterUsername].ParameterValue;
            }
            string password = null;
            if (values.ContainsKey(parameterPassword))
            {
                password = values[parameterPassword].ParameterValue;
            }
            bool debug = false;
            if (values.ContainsKey(parameterDebugMode))
            {
                debug = bool.Parse(values[parameterDebugMode].ParameterValue);
            }


            return mXcaService.ExecuteRetrieve(retrieveUri, homeCommunityId, repositoryId, 
                id, transactionId, userCred, filename, openImageExe, host, port, protocol,
                username, password, debug);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterRetrieveUri, new MethodParameter(parameterRetrieveUri, 
                true, "XCARespondingGateway/services/RespondingGateway_Service", true));
            parameters.Add(parameterHomeCommunityId, new MethodParameter(parameterHomeCommunityId, 
                true, "2.16.840.1.113883.3.166", true));
            parameters.Add(parameterRepositoryId, new MethodParameter(parameterRepositoryId, 
                true, "", true));
            parameters.Add(parameterDocumentId, new MethodParameter(parameterDocumentId,
                true, "", true));
            //parameters.Add(parameterFilename, new MethodParameter(parameterFilename, true, 
                //vixConfiguration.ImageDirectory + "\\" + DateTime.Now.Ticks, MethodParameterType.saveFileType));
            parameters.Add(parameterFilename, GetFilenameParameter(parameterFilename));
            parameters.Add(parameterOpenImageEXE, new MethodParameter(parameterOpenImageEXE,
                false, vixConfiguration.DicomViewerPath, MethodParameterType.openFileType));

            parameters.Add(parameterHostname, new MethodParameter(parameterHostname, true, vixConfiguration.Server, true));
            parameters.Add(parameterPort, new MethodParameter(parameterPort, true, vixConfiguration.Port + "", true));
            parameters.Add(parameterProtocol, new MethodParameter(parameterProtocol, true, "http", true));
            parameters.Add(parameterUsername, new MethodParameter(parameterUsername, false, "", true));
            parameters.Add(parameterPassword, new MethodParameter(parameterPassword, false, "", true));
            parameters.Add(parameterDebugMode, new MethodParameter(parameterDebugMode, false, "false", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterHostname))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterPort))
                return false;
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            return null;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            Dictionary<string, StoredMethodParameterValue> updatedParameters = new Dictionary<string, StoredMethodParameterValue>();
            if (obj is VixClientV3_XCA.types.Document)
            {
                VixClientV3_XCA.types.Document document = (VixClientV3_XCA.types.Document)obj;
                updatedParameters.Add(parameterDocumentId, 
                    new StoredMethodParameterValue(parameterDocumentId, document.Id));
                updatedParameters.Add(parameterHomeCommunityId, 
                    new StoredMethodParameterValue(parameterHomeCommunityId, document.HomeCommunityId));
                updatedParameters.Add(parameterRepositoryId, 
                    new StoredMethodParameterValue(parameterRepositoryId, document.RepositoryId));

                string repositoryId = document.RepositoryId;
                string homeCommunityId = document.HomeCommunityId;

                string haimsRepositoryId = methodFactoryParameters[factoryParameterHAIMSRepositoryId].ParameterValue;
                string ncatRepositoryId = methodFactoryParameters[factoryParameterNCATRepositoryId].ParameterValue;
                string vaHomeCommunityId = methodFactoryParameters[factoryParameterVAHomeCommunityId].ParameterValue;

                string haimsHostname = methodFactoryParameters[factoryParameterHAIMSHostname].ParameterValue;
                string haimsPort = methodFactoryParameters[factoryParameterHAIMSPort].ParameterValue;
                string haimsUri = methodFactoryParameters[factoryParameterHAIMSUri].ParameterValue;
                string haimsProtocol = methodFactoryParameters[factoryParameterHAIMSProtocol].ParameterValue;

                string ncatHostname = methodFactoryParameters[factoryParameterNCATHostname].ParameterValue;
                string ncatPort = methodFactoryParameters[factoryParameterNCATPort].ParameterValue;
                string ncatUri = methodFactoryParameters[factoryParameterNCATUri].ParameterValue;
                string ncatProtocol = methodFactoryParameters[factoryParameterNCATProtocol].ParameterValue;
                string ncatUsername = methodFactoryParameters[factoryParameterNCATUsername].ParameterValue;
                string ncatPassword = methodFactoryParameters[factoryParameterNCATPassword].ParameterValue;

                string vaPort = methodFactoryParameters[factoryParameterVAPort].ParameterValue;
                string vaProtocol = methodFactoryParameters[factoryParameterVAProtocol].ParameterValue;
                string vaUsername = methodFactoryParameters[factoryParameterVAUsername].ParameterValue;
                string vaPassword = methodFactoryParameters[factoryParameterVAPassword].ParameterValue;

                if (repositoryId == haimsRepositoryId)
                {
                    updatedParameters.Add(parameterHostname, new StoredMethodParameterValue(parameterHostname, haimsHostname));
                    updatedParameters.Add(parameterPort, new StoredMethodParameterValue(parameterPort, haimsPort));
                    updatedParameters.Add(parameterRetrieveUri, new StoredMethodParameterValue(parameterRetrieveUri, haimsUri));
                    updatedParameters.Add(parameterProtocol, new StoredMethodParameterValue(parameterProtocol, haimsProtocol));
                }
                else if (repositoryId == ncatRepositoryId)
                {
                    updatedParameters.Add(parameterHostname, new StoredMethodParameterValue(parameterHostname, ncatHostname));
                    updatedParameters.Add(parameterPort, new StoredMethodParameterValue(parameterPort, ncatPort));
                    updatedParameters.Add(parameterRetrieveUri, new StoredMethodParameterValue(parameterRetrieveUri, ncatUri));
                    updatedParameters.Add(parameterProtocol, new StoredMethodParameterValue(parameterProtocol, ncatProtocol));
                    updatedParameters.Add(parameterUsername, new StoredMethodParameterValue(parameterUsername, ncatUsername));
                    updatedParameters.Add(parameterPassword, new StoredMethodParameterValue(parameterPassword, ncatPassword));
                }
                else if (homeCommunityId == vaHomeCommunityId)
                {
                    updatedParameters.Add(parameterPassword, new StoredMethodParameterValue(parameterPassword, vaPassword));
                    updatedParameters.Add(parameterPort, new StoredMethodParameterValue(parameterPort, vaPort));
                    updatedParameters.Add(parameterProtocol, new StoredMethodParameterValue(parameterProtocol, vaProtocol));
                    updatedParameters.Add(parameterUsername, new StoredMethodParameterValue(parameterUsername, vaUsername));
                }

            }
            return updatedParameters;
        }

        
    }
}
