/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;

namespace VixClientV3_XCA.methods
{
    public class GetDocuments : AbstractBaseXcaMethod, IMethod
    {
        private readonly string parameterQueryUri = "Query URI";
        private readonly string parameterPatientIcn = "Patient ICN";
        private readonly string parameterDateFrom = "Date From";
        private readonly string parameterDateTo = "Date To";
        private readonly string parameterClassCode = "Class Code";
        private readonly string parameterClassCodeScheme = "Class Code Scheme";
        private readonly string parameterProtocol = "Protocol";
        private readonly string parameterUsername = "Username";
        private readonly string parameterPassword = "Password";
        private readonly string parameterDebugMode = "Debug Mode";

        public GetDocuments(UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
            : base(userCredentials, vixConfiguration, logMsgDelegate)
        {

        }

        public object execute(Dictionary<string, MethodParameter> values)
        {
            string transactionId = values[parameterTransactionId].ParameterValue;
            UserCredentials userCred = GetUserCredentialsFromParameters(values);
            string queryUri = values[parameterQueryUri].ParameterValue;
            string patientIcn = values[parameterPatientIcn].ParameterValue;
            string fromDate = values[parameterDateFrom].ParameterValue;
            string toDate = values[parameterDateTo].ParameterValue;
            string classCode = values[parameterClassCode].ParameterValue;
            string classCodeScheme = values[parameterClassCodeScheme].ParameterValue;
            string protocol = values[parameterProtocol].ParameterValue;

            string username = null;
            if (values.ContainsKey(parameterUsername))
            {
                username = values[parameterUsername].ParameterValue;
            }
            string password = null;
            if (values.ContainsKey(parameterPassword))
            {
                password = values[parameterPassword].ParameterValue;
            }
            bool debug = false;
            if (values.ContainsKey(parameterDebugMode))
            {
                debug = bool.Parse(values[parameterDebugMode].ParameterValue);
            }

            return mXcaService.ExecuteQuery(queryUri, patientIcn, transactionId, 
                userCred, fromDate, toDate, classCode, classCodeScheme, 
                protocol, username, password, debug);
        }

        public Dictionary<string, MethodParameter> getParameters()
        {
            Dictionary<string, MethodParameter> parameters = null;

            parameters = getCredentialsParameters();
            parameters.Add(parameterTransactionId, GetTransactionIdParameter());
            parameters.Add(parameterQueryUri, new MethodParameter(parameterQueryUri, 
                true, "XCARespondingGateway/services/RespondingGateway_Service", true));
            parameters.Add(parameterPatientIcn, new MethodParameter(parameterPatientIcn, 
                true, "", true));
            parameters.Add(parameterDateFrom, new MethodParameter(parameterDateFrom, false, "", MethodParameterType.dateType, true));
            parameters.Add(parameterDateTo, new MethodParameter(parameterDateTo, false, "", MethodParameterType.dateType, true));
            parameters.Add(parameterClassCode, new MethodParameter(parameterClassCode, false, "", true));
            parameters.Add(parameterClassCodeScheme, new MethodParameter(parameterClassCodeScheme, false, "", true));
            parameters.Add(parameterProtocol, new MethodParameter(parameterProtocol, true, "https", true));
            parameters.Add(parameterUsername, new MethodParameter(parameterUsername, false, "", true));
            parameters.Add(parameterPassword, new MethodParameter(parameterPassword, false, "", true));
            parameters.Add(parameterDebugMode, new MethodParameter(parameterDebugMode, false, "false", true));

            return parameters;
        }

        public bool validateParameters(Dictionary<string, MethodParameter> values)
        {
            if (values == null)
                return false;
            if (!ValidateRequiredParameters(values))
                return false;
            if (!CheckParameterNotEmptyOrNull(values, parameterProtocol))
                return false;
            if (!validateDateParameterIfExists(values, parameterDateFrom))
                return false;
            if (!validateDateParameterIfExists(values, parameterDateTo))
                return false;
            return true;
        }

        private bool validateDateParameterIfExists(Dictionary<string, MethodParameter> values, string dateParameterName)
        {
            if (values.ContainsKey(dateParameterName))
            {
                string dateFrom = values[dateParameterName].ParameterValue;
                if (!string.IsNullOrEmpty(dateFrom))
                {
                    DateTime dt;
                    if (!DateTime.TryParse(dateFrom, out dt))
                        // could not parse the date, throw an exception
                        return false;
                }
            }
            return true;
        }

        public List<AssociatedMethod> getAssociatedMethods()
        {
            List<AssociatedMethod> types = new List<AssociatedMethod>();
            types.Add(new AssociatedMethod(typeof(VixClientV3_XCA.types.Document), typeof(GetDocument)));
            types.Add(new AssociatedMethod(typeof(VixClientV3_XCA.types.Document[]), typeof(GetDocument)));
            return types;
        }

        public Dictionary<string, StoredMethodParameterValue> getParametersFromObject(object obj)
        {
            return new Dictionary<string, StoredMethodParameterValue>();
        }

        public override string ToString()
        {
            return this.GetType().Name;
        }

        
    }
}
