/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixClientV3Common;
using System.Net;

namespace VixClientV3_XCA.methods
{
    public class XCAMethodFactory : AbstractBaseMethodFactory, IMethodFactory
    {
        public List<IMethod> getMethods(List<MethodFactoryParameter> userMethodFactoryParameters,
            UserCredentials userCredentials, VixConfiguration vixConfiguration, LogMsgDelegate logMsgDelegate)
        {
            List<IMethod> methods = new List<IMethod>();
            methods.Add(new GetDocuments(userCredentials, vixConfiguration, logMsgDelegate));
            methods.Add(new GetDocument(userCredentials, vixConfiguration, 
                userMethodFactoryParameters, logMsgDelegate));   
            return methods;
        }

        public override string getName()
        {
            return "XCA";
        }

        public override string ToString()
        {
            return getName();
        }

        public List<MethodFactoryParameter> getMethodFactoryParameters()
        {
            List<MethodFactoryParameter> parameters = new List<MethodFactoryParameter>();

            // HAIMS on HTTP
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterHAIMSHostname, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterHAIMSPort, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterHAIMSRepositoryId, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterHAIMSUri, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterHAIMSProtocol, true, "", true));

            //NCAT on HTTP
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterNCATHostname, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterNCATPort, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterNCATRepositoryId, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterNCATUri, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterNCATProtocol, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterNCATUsername, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterNCATPassword, true, "", true));

            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterVAHomeCommunityId, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterVAUsername, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterVAPassword, true, "", true));            
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterVAProtocol, true, "", true));
            parameters.Add(new MethodFactoryParameter(GetDocument.factoryParameterVAPort, true, "", true));

            return parameters;
        }
    }
}
