/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace VixClientV3_XCA.types
{
    public class Document
    {
        private string mstrUri;
        private string mstrCreationTime;
        private string mstrSourcePatientId;
        private string mstrName;
        private string mstrDescription;
        private string mstrMimeType;
        private string mstrCodingScheme;
        private string mstrClassCode;
        private string mstrId;
        private string mstrHomeCommunityId;
        private string mstrRepositoryId;
        private string mstrHash;
        private string mstrSize;

        public string Hash
        {
            get { return mstrHash; }
            set { mstrHash = value; }
        }        

        public string Size
        {
            get { return mstrSize; }
            set { mstrSize = value; }
        }

        public string Id
        {
            get { return mstrId; }
            set { mstrId = value; }
        }        

        public string HomeCommunityId
        {
            get { return mstrHomeCommunityId; }
            set { mstrHomeCommunityId = value; }
        }        

        public string RepositoryId
        {
            get { return mstrRepositoryId; }
            set { mstrRepositoryId = value; }
        }

        public string MimeType
        {
            get { return mstrMimeType; }
            set { mstrMimeType = value; }
        }        

        public string CodingScheme
        {
            get { return mstrCodingScheme; }
            set { mstrCodingScheme = value; }
        }        

        public string ClassCode
        {
            get { return mstrClassCode; }
            set { mstrClassCode = value; }
        }

        public string Uri
        {
            get { return mstrUri; }
            set { mstrUri = value; }
        }        

        public string CreationTime
        {
            get { return mstrCreationTime; }
            set { mstrCreationTime = value; }
        }        

        public string SourcePatientId
        {
            get { return mstrSourcePatientId; }
            set { mstrSourcePatientId = value; }
        }        

        public string Name
        {
            get { return mstrName; }
            set { mstrName = value; }
        }        

        public string Description
        {
            get { return mstrDescription; }
            set { mstrDescription = value; }
        }

        public Document()
        {

        }

        public static DocumentQueryResponse createDocumentResponseFromXml(string xml)
        {
            List<Document> documents = new List<Document>();
            List<DocumentResponseError> errors = new List<DocumentResponseError>();
            List<DocumentResponseFault> faults = new List<DocumentResponseFault>();
            XmlDocument xmlDoc = new XmlDocument();
            string status = "";
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(xmlDoc.NameTable);
            nsmgr.AddNamespace("soapenv", "http://www.w3.org/2003/05/soap-envelope");
            nsmgr.AddNamespace("urn", "urn:oasis:names:tc:ebxml-regrep:xsd:query:3.0");
            nsmgr.AddNamespace("urn1", "urn:oasis:names:tc:ebxml-regrep:xsd:rim:3.0");
            nsmgr.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
            nsmgr.AddNamespace("wsa", "http://www.w3.org/2005/08/addressing");
            nsmgr.AddNamespace("ns3", "urn:oasis:names:tc:ebxml-regrep:xsd:rs:3.0");
            xmlDoc.LoadXml(xml);

            XmlNode faultNode = xmlDoc.SelectSingleNode("/soapenv:Envelope/soapenv:Body/soapenv:Fault", nsmgr);
            if (faultNode != null)
            {
                XmlNode codeNode = faultNode.SelectSingleNode("soapenv:Code", nsmgr);
                XmlNode valueNode = codeNode.SelectSingleNode("soapenv:Value", nsmgr);
                String code = valueNode.InnerText;

                XmlNode reasonNode = faultNode.SelectSingleNode("soapenv:Reason", nsmgr);
                XmlNode textNode = reasonNode.SelectSingleNode("soapenv:Text", nsmgr);
                String reason = textNode.InnerText;

                faults.Add(new DocumentResponseFault(code, reason));
            }

            XmlNode adHocQueryResponseNode = xmlDoc.SelectSingleNode("/soapenv:Envelope/soapenv:Body/urn:AdhocQueryResponse", nsmgr);
            if (adHocQueryResponseNode != null)
            {
                if (adHocQueryResponseNode.Attributes["status"] != null)
                {
                    status = adHocQueryResponseNode.Attributes["status"].Value;
                }

                XmlNodeList registryErrorNodeList = adHocQueryResponseNode.SelectNodes("ns3:RegistryErrorList/ns3:RegistryError", nsmgr);
                if (registryErrorNodeList != null)
                {
                    foreach (XmlNode registerErrorNode in registryErrorNodeList)
                    {

                        string codeContext = "";
                        if (registerErrorNode.Attributes["codeContext"] != null)
                            codeContext = registerErrorNode.Attributes["codeContext"].Value;

                        string errorCode = "";
                        if (registerErrorNode.Attributes["errorCode"] != null)
                            errorCode = registerErrorNode.Attributes["errorCode"].Value;

                        string location = "";
                        if (registerErrorNode.Attributes["location"] != null)
                            location = registerErrorNode.Attributes["location"].Value;

                        string severity = "";
                        if (registerErrorNode.Attributes["severity"] != null)
                            severity = registerErrorNode.Attributes["severity"].Value;

                        errors.Add(new DocumentResponseError(codeContext, errorCode, location, severity));
                    }
                }
            }

            XmlNode registryObjectListNode = xmlDoc.SelectSingleNode("/soapenv:Envelope/soapenv:Body/urn:AdhocQueryResponse/urn1:RegistryObjectList", nsmgr);
            if (registryObjectListNode != null)
            {
                XmlNodeList extrinsicObjectNodes = registryObjectListNode.SelectNodes("urn1:ExtrinsicObject", nsmgr);
                foreach (XmlNode extrinsicObjectNode in extrinsicObjectNodes)
                {
                    documents.Add(parseExtrinsicObject(extrinsicObjectNode, nsmgr));
                }
                XmlNodeList identifiableNodes = registryObjectListNode.SelectNodes("urn1:Identifiable", nsmgr);
                foreach (XmlNode identifiableNode in identifiableNodes)
                {
                    if (identifiableNode.Attributes["xsi:type"] != null)
                    {
                        string type = identifiableNode.Attributes["xsi:type"].Value;
                        if (type.EndsWith("ExtrinsicObjectType"))// only parsing if actually ExtrinsicObject
                        {
                            documents.Add(parseExtrinsicObject(identifiableNode, nsmgr));
                        }
                    }

                }

                /*
                XmlNodeList identifiableNodes = registryObjectListNode.ChildNodes;

                XmlNamespaceManager identifiableNsmgr = new XmlNamespaceManager(xmlDoc.NameTable);
                identifiableNsmgr.AddNamespace("urn", "urn:oasis:names:tc:ebxml-regrep:xsd:rim:3.0");
                identifiableNsmgr.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");

                foreach (XmlNode identifiableNode in identifiableNodes)
                {
                    string mimeType = identifiableNode.Attributes["mimeType"].Value;
                    string id = identifiableNode.Attributes["id"].Value;
                    string uri = "";
                    string patientIcn = "";
                    string creationTime = "";
                    string homeCommunityId = identifiableNode.Attributes["home"].Value;
                    string repositoryId = "";
                    string size = "";
                    string hash = "";

                    XmlNodeList slotNodes = identifiableNode.SelectNodes("urn:Slot", identifiableNsmgr);
                    foreach (XmlNode slotNode in slotNodes)
                    {
                        string slotName = slotNode.Attributes["name"].Value;
                        if (slotName == "URI")
                        {
                            uri = extractValueFromSlot(slotNode, identifiableNsmgr);
                        }
                        else if (slotName == "sourcePatientId")
                        {
                            patientIcn = extractValueFromSlot(slotNode, identifiableNsmgr);
                        }
                        else if (slotName == "creationTime")
                        {
                            creationTime = extractValueFromSlot(slotNode, identifiableNsmgr);
                        }
                        else if (slotName == "repositoryUniqueId")
                        {
                            repositoryId = extractValueFromSlot(slotNode, identifiableNsmgr);
                        }
                        else if (slotName == "size")
                        {
                            size = extractValueFromSlot(slotNode, identifiableNsmgr);
                        }
                        else if (slotName == "hash")
                        {
                            hash = extractValueFromSlot(slotNode, identifiableNsmgr);
                        }
                    }
                    XmlNode nameNode = identifiableNode.SelectSingleNode("urn:Name/urn:LocalizedString", identifiableNsmgr);
                    string name = nameNode.Attributes["value"].Value;
                    XmlNode descriptionNode = identifiableNode.SelectSingleNode("urn:Description/urn:LocalizedString", identifiableNsmgr);
                    string description = null;
                    if (descriptionNode != null)
                    {
                        if (descriptionNode.Attributes["value"] != null)
                        {
                            description = descriptionNode.Attributes["value"].Value;
                        }
                    }


                    string codingScheme = "";
                    string classCode = "";


                    XmlNodeList classificationNodes = identifiableNode.SelectNodes("urn:Classification", identifiableNsmgr);
                    foreach (XmlNode classificationNode in classificationNodes)
                    {
                        string classificationScheme = classificationNode.Attributes["classificationScheme"].Value;
                        if (classificationScheme == "urn:uuid:f0306f51-975f-434e-a61c-c59651d33983")
                        {
                            classCode = classificationNode.Attributes["nodeRepresentation"].Value;
                            XmlNode classificationSlotNode = classificationNode.SelectSingleNode("urn:Slot", identifiableNsmgr);
                            codingScheme = extractValueFromSlot(classificationSlotNode, identifiableNsmgr);

                        }
                    }

                    Document document = new Document();
                    //document.mstrCreationTime = crea
                    document.mstrDescription = description;
                    document.mstrName = name;
                    document.mstrSourcePatientId = patientIcn;
                    document.mstrUri = uri;
                    document.mstrMimeType = mimeType;
                    document.mstrCodingScheme = codingScheme;
                    document.mstrClassCode = classCode;
                    document.mstrCreationTime = creationTime;
                    document.mstrId = id;
                    document.mstrHomeCommunityId = homeCommunityId;
                    document.mstrRepositoryId = repositoryId;
                    document.mstrSize = size;
                    document.Hash = hash;
                    documents.Add(document);
                }*/

            }

            return new DocumentQueryResponse(status, documents, errors, faults);
        }

        private static Document parseExtrinsicObject(XmlNode extrinsicObjectNode, XmlNamespaceManager nsmgr)
        {
            string mimeType = extrinsicObjectNode.Attributes["mimeType"].Value;
            string id = extrinsicObjectNode.Attributes["id"].Value;
            string uri = "";
            string patientIcn = "";
            string creationTime = "";
            string homeCommunityId = extrinsicObjectNode.Attributes["home"].Value;
            string repositoryId = "";
            string size = "";
            string hash = "";

            XmlNodeList slotNodes = extrinsicObjectNode.SelectNodes("urn1:Slot", nsmgr);
            foreach (XmlNode slotNode in slotNodes)
            {
                string slotName = slotNode.Attributes["name"].Value;
                if (slotName == "URI")
                {
                    uri = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "sourcePatientId")
                {
                    patientIcn = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "creationTime")
                {
                    creationTime = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "repositoryUniqueId")
                {
                    repositoryId = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "size")
                {
                    size = extractValueFromSlot(slotNode, nsmgr);
                }
                else if (slotName == "hash")
                {
                    hash = extractValueFromSlot(slotNode, nsmgr);
                }
            }
            XmlNode nameNode = extrinsicObjectNode.SelectSingleNode("urn1:Name/urn1:LocalizedString", nsmgr);
            string name = null;
            if (nameNode != null)
            {
                name = nameNode.Attributes["value"].Value;
            }
            XmlNode descriptionNode = extrinsicObjectNode.SelectSingleNode("urn1:Description/urn1:LocalizedString", nsmgr);
            string description = null;
            if (descriptionNode != null)
            {
                if (descriptionNode.Attributes["value"] != null)
                {
                    description = descriptionNode.Attributes["value"].Value;
                }
            }


            string codingScheme = "";
            string classCode = "";


            XmlNodeList classificationNodes = extrinsicObjectNode.SelectNodes("urn1:Classification", nsmgr);
            foreach (XmlNode classificationNode in classificationNodes)
            {
                string classificationScheme = classificationNode.Attributes["classificationScheme"].Value;
                if (classificationScheme == "urn:uuid:f0306f51-975f-434e-a61c-c59651d33983")
                {
                    classCode = classificationNode.Attributes["nodeRepresentation"].Value;
                    XmlNode classificationSlotNode = classificationNode.SelectSingleNode("urn1:Slot", nsmgr);
                    codingScheme = extractValueFromSlot(classificationSlotNode, nsmgr);

                }
            }
            Document document = new Document();
            //document.mstrCreationTime = crea
            document.mstrDescription = description;
            document.mstrName = name;
            document.mstrSourcePatientId = patientIcn;
            document.mstrUri = uri;
            document.mstrMimeType = mimeType;
            document.mstrCodingScheme = codingScheme;
            document.mstrClassCode = classCode;
            document.mstrCreationTime = creationTime;
            document.mstrId = id;
            document.mstrHomeCommunityId = homeCommunityId;
            document.mstrRepositoryId = repositoryId;
            document.mstrSize = size;
            document.Hash = hash;
            return document;
        }

        private static string extractValueFromSlot(XmlNode slotNode, XmlNamespaceManager nsmgr)
        {
            XmlNode valueNode = slotNode.SelectSingleNode("urn1:ValueList/urn1:Value", nsmgr);
            return valueNode.InnerText.Trim();
        }
    }
}
