/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixCommon;

namespace SiteService
{
    public class SiteServiceHelper
    {
        private static exchangeSiteService.ImagingExchangeSiteService service = 
            new SiteService.exchangeSiteService.ImagingExchangeSiteService();

        public static List<VaSite> GetVixSites(string siteServiceUrl)
        {
            return GetSites(siteServiceUrl, true);
        }

        public static List<VaSite> GetAllSites(string siteServiceUrl)
        {
            return GetSites(siteServiceUrl, false);
        }

        private static List<VaSite> GetSites(string siteServiceUrl, bool onlyVixSites)
        {
            service.Url = siteServiceUrl;
            exchangeSiteService.ImagingExchangeSiteTO[] exchangeSites = service.getImagingExchangeSites();
            return translateExchangeSites(exchangeSites, onlyVixSites);
        }

        private static List<VaSite> translateExchangeSites(exchangeSiteService.ImagingExchangeSiteTO[] exchangeSites, bool onlyVixSites)
        {
            List<VaSite> sites = new List<VaSite>();
            foreach (exchangeSiteService.ImagingExchangeSiteTO exchangeSite in exchangeSites)
            {
                VaSite site = translate(exchangeSite);
                if (onlyVixSites)
                {
                    if (site.HasVix)
                    {
                        if (site.SiteNumber != "200")
                        {
                            sites.Add(site);
                        }
                    }
                }
                else
                {
                    sites.Add(site);
                }
            }
            return sites;
        }

        private static VaSite translate(exchangeSiteService.ImagingExchangeSiteTO site)
        {
            return new VaSite(site.siteNumber, site.siteName, site.regionID, site.vistaServer,
                site.vistaPort, site.acceleratorServer, site.acceleratorPort, site.siteAbbr);
        }
    }
}
