/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using VixCommon;
using System.IO;

namespace TransactionLog
{
    public class TransactionLogDownloader
    {
        public static List<TransactionLogEntry> DownloadTransactions(string server, DateTime startDate,
            DateTime endDate, string transactionId, string username, string password)
        {

            if (transactionId.StartsWith("{"))
            {
                transactionId = "\\" + transactionId;
            }
            if (transactionId.EndsWith("}"))
            {
                transactionId = transactionId.Substring(0, transactionId.Length - 1) + "\\}";
            }

            string dateFormat = "";//12/20/2010

            StringBuilder url = new StringBuilder();

            url.Append("http://");
            url.Append(server);
            url.Append("/Vix/secure/ExcelTransactionLog?");
            url.Append("fromDate=");
            url.Append(startDate.ToString(dateFormat));
            url.Append("&toDate=");
            url.Append(endDate.ToString(dateFormat));
            url.Append("&transactionId=");
            url.Append(transactionId);
            url.Append("&format=text%2Ftab-separated-values");

            WebRequest request = WebRequest.Create(url.ToString());
            request.Credentials = new NetworkCredential(username, password);
            WebResponse response = request.GetResponse();
            Stream responseStream = response.GetResponseStream();


            List<TransactionLogEntry> entries =
                TransactionLogParser.ParseLogStream(responseStream, "log");
            return entries;
        }

        public static List<TransactionLogEntry> DownloadTransactions(string server, DateTime startDate,
            DateTime endDate, string transactionId)
        {

            return DownloadTransactions(server, startDate, endDate, transactionId, "", "");
        }

        public static void DownloadLog(VaSite site, DateTime startDate, DateTime endDate, string logDirectory)
        {
            string vixUrl = getSiteVixUrl(site);
            if (vixUrl == null)
                return;
            DateTime curDate = startDate;
            vixUrl = "http://" + vixUrl;
            string logD = logDirectory + "\\" + site.SiteNumber;
            if (!Directory.Exists(logD))
            {
                Directory.CreateDirectory(logD);
            }

            while (!VixCommon.VixCommonUtils.isDateAfter(curDate, endDate))
            {
                string filename = TransactionLogUtils.getLogFilename(logDirectory, site.SiteNumber, curDate);
                DownloadLog(vixUrl, curDate, filename);
                curDate = curDate.AddDays(1);
            }

            //vixUrl = "http://" + vixUrl + "/Vix/secure/ExcelTransactionLog?startIndex=&endIndex=&format=text%2Fcsv&fromDate=01%2F27%2F2009&toDate=01%2F29%2F2009&imageQuality=unselected&user=&modality=&datasourceProtocol=unselected&errorMessage=&transactionId=&imageUrn=&forwardIteration=False&resultsPerPage=100";          

        }



        private static void DownloadLog(string vixBaseUrl, DateTime date, string outputFilename)
        {
            if (File.Exists(outputFilename))
                return;
            WebClient client = new WebClient();
            client.Credentials = getVixLogCredentials();
            string dateString = date.ToString("MM") + "%2F" + date.ToString("dd") + "%2F" + date.ToString("yyyy");
            //string url = vixBaseUrl + "/Vix/secure/ExcelTransactionLog?startIndex=&endIndex=&format=text%2Fcsv&fromDate=" + dateString + "&toDate=" + dateString + "&imageQuality=unselected&user=&modality=&datasourceProtocol=unselected&errorMessage=&transactionId=&imageUrn=&forwardIteration=False&resultsPerPage=100";
            string url = vixBaseUrl + "/Vix/secure/ExcelTransactionLog?startIndex=&endIndex=&format=text%2Ftab-separated-values&fromDate=" + dateString + "&toDate=" + dateString + "&imageQuality=unselected&user=&modality=&datasourceProtocol=unselected&errorMessage=&transactionId=&imageUrn=&forwardIteration=False&resultsPerPage=100";
            client.DownloadFile(url, outputFilename);
        }

        private static NetworkCredential getVixLogCredentials()
        {
            return new NetworkCredential("vixlog", "tachik0ma");
        }

        private static string getSiteVixUrl(VaSite site)
        {
            return site.VixServer + ":" + site.VixPort;
        }
    }
}
