/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using VixCommon;

namespace TransactionLog
{
    public class TransactionLogParser
    {
        public static List<TransactionLogEntry> ParseLogs(string logDir, string siteNumber, DateTime startDate, DateTime endDate)
        {
            List<TransactionLogEntry> entries = new List<TransactionLogEntry>();
            DateTime curDate = startDate;
            while (!VixCommonUtils.isDateAfter(curDate, endDate))
            {
                string filename = TransactionLogUtils.getLogFilename(logDir, siteNumber, curDate);
                entries.AddRange(ParseFile(filename, siteNumber));
                curDate = curDate.AddDays(1);
            }
            return entries;
        }

        public static List<TransactionLogEntry> ParseLogStream(Stream stream, string siteNumber)
        {
            List<TransactionLogEntry> entries = new List<TransactionLogEntry>();
            StreamReader reader = new StreamReader(stream);
            try
            {
                string headerLine = reader.ReadLine();
                string line = reader.ReadLine();
                while (line != null)
                {
                    TransactionLogEntry entry = TransactionLogEntry.parseTSVLine(siteNumber, headerLine, line);
                    if (entry != null)
                        entries.Add(entry);
                    line = reader.ReadLine();
                }
            }
            finally
            {
                reader.Close();
            }

            return entries;
        }

        private static List<TransactionLogEntry> ParseFile(string filename, string siteNumber)
        {
            List<TransactionLogEntry> entries = new List<TransactionLogEntry>();

            FileStream fStream = new FileStream(filename, FileMode.Open);
            try
            {
                entries = ParseLogStream(fStream, siteNumber);
            }
            catch (Exception)
            {
                // catch and do nothing
            }
            return entries;         
        }
    }
}
