/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Xml;

namespace VixCommon
{
    

    public partial class ObjectDisplayerControl : UserControl
    {
        public ObjectDisplayerControl()
        {
            InitializeComponent();
        }

        //private readonly List<Type> mHighlightedTypes = new List<Type>();
        private readonly List<TypeSelectedHandler> mTypeSelectedHandlers = new List<TypeSelectedHandler>();

        public List<TypeSelectedHandler> TypeSelectedHandlers
        {
            get { return mTypeSelectedHandlers; }
        } 

        /*
        public List<Type> HighlightedTypes
        {
            get { return mHighlightedTypes; }
        } */

        public void DisplayItems(object[] items)
        {
            treDisplay.Nodes.Clear();
            treDisplay.BeginUpdate();
            foreach (object item in items)
            {              
                DisplayItem(item, null, 0);
            }
            treDisplay.EndUpdate();
            treDisplay.ExpandAll(); // this might be causing the performance issue - not sure
        }

        public void DisplayItem(object item)
        {
            if (false)
            {
                writer = new StreamWriter("C:\\temp\\objectDisplayer.txt");
            }
            treDisplay.Nodes.Clear();
            try
            {
                treDisplay.BeginUpdate();
                if (item == null)
                    addItemToTree("<null value>", null, null);
                else
                    DisplayItem(item, null, 0);
                treDisplay.ExpandAll(); // this might be causing the performance issue - not sure
                if (false)
                {
                    if (writer != null)
                        writer.Close();
                }
            }
            finally
            {
                treDisplay.EndUpdate();
            }
        }

        private bool isPrimitive(Type t)
        {
            if (t.IsPrimitive)
                return true;
            if(t == typeof(string))
                return true;
            return false;
        }

        private StreamWriter writer = null;

        private void LogOutput(string msg)
        {
            if (writer != null)
            {
                writer.WriteLine(msg);
                writer.Flush();
            }
        }

        private int maxLevel = 5;//10;

        private void DisplayItem(object item, TreeNode parentNode, int level)
        {
            LogOutput("DisplayItem - " + (item == null ? "null" : (item.GetType() + ": " + item.ToString())) + ", level (" + level + ")");
            if (level > maxLevel)//10)
            {
                LogOutput("Level " + level + " > " + maxLevel + ", quitting");
                return;
            }
            Type type = item.GetType();
            parentNode = addItemToTree(type.ToString(), null, parentNode, item);

            if (isEnum(item))
            {
                addItemToTree("value", item.ToString(), parentNode, item);
                return;
            }

            if(isPrimitive(type))
            //if (type == typeof(string))
            {
                addItemToTree("value", item.ToString(), parentNode, item);
                return;
            }

            if (item is Exception)
            {
                Exception ex = (Exception)item;

                TreeNode node = addItemToTree(type.Name, ex.Message, parentNode, item);
                if (ex.StackTrace != null)
                    addItemToTree("StackTrace", ex.StackTrace, node);
                return;
            }

            PropertyInfo[] properties = item.GetType().GetProperties();
            foreach (PropertyInfo property in properties)
            {
                if (property.CanRead)
                {
                    try
                    {
                        Object propertyValue = property.GetValue(item, null);
                        if (propertyValue == null)
                        {
                            addItemToTree(property.Name, "<null>", parentNode);
                        }
                        else
                        {
                            Type propertyValueType = propertyValue.GetType();
                            if ((propertyValueType == typeof(String))
                                || (propertyValueType == typeof(int))
                                || (propertyValueType == typeof(Int64))
                                || (propertyValueType == typeof(bool))
                                || (propertyValueType == typeof(DateTime))
                                )
                            {
                                addItemToTree(property.Name, propertyValue.ToString(), parentNode, propertyValue);
                            }
                            else if (propertyValueType == typeof(String[]))
                            {
                                string[] values = (string[])propertyValue;
                                string value = "";
                                for (int i = 0; i < values.Length; i++)
                                {
                                    if (value.Length > 0)
                                    {
                                        value += ", " + values[i];
                                    }
                                    else
                                    {
                                        value = values[i];
                                    }
                                }
                                addItemToTree(property.Name, value, parentNode, propertyValue);
                            }
                            else if (isEnum(propertyValue))
                            {
                                TreeNode node = addItemToTree(propertyValueType.Name, propertyValue.ToString(), parentNode, propertyValue);
                            }
                            else if (propertyValueType.IsArray)
                            {
                                // JMW 1/18/2011 - this is the old way, was not showing length of the array
                                //TreeNode node = addItemToTree(propertyValueType.ToString(), null, parentNode);   
                                System.Array array = (Array)propertyValue; // cast to an array to get its length
                                // JMW 1/18/2011 - put the length of the array into the output (hopefully it won't cause an exception)
                                TreeNode node = addItemToTree(propertyValueType.ToString(), array.Length + "", parentNode);
                                object[] objects = (object[])propertyValue;
                                foreach (object obj in objects)
                                {
                                    DisplayItem(obj, node, level + 1);
                                }
                            }
                            else
                            {
                                DisplayItem(propertyValue, parentNode, level + 1);
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        addItemToTree(property.Name, "<cannot read property>", parentNode);
                        // would like to log, but not sure what to do with
                    }
                }
            }
        }

        public TreeNode GetSelectedNode()
        {
            return treDisplay.SelectedNode;
        }

        private bool isEnum(object property)
        {
            try
            {
                return Enum.IsDefined(property.GetType(), property);
            }
            catch (Exception)
            {
                return false;
            }
        }

        private TreeNode addItemToTree(string name, string value, TreeNode parentNode)
        {
            return addItemToTree(name, value, parentNode, null);
        }

        private TreeNode addItemToTree(string name, string value, TreeNode parentNode, object item)
        {
            TreeNode node = new TreeNode();
            node.Tag = item;
            if (parentNode == null)
            {
                treDisplay.Nodes.Add(node);

            }
            else
            {
                parentNode.Nodes.Add(node);

            }
            if (value == null)
            {
                node.Text = name;
            }
            else
            {
                node.Text = name + " = " + value;
            }

            if (item != null)
            {
                if (checkHighlightItem(item.GetType()))
                {
                    node.ForeColor = Color.Green;

                    Font f = new Font(treDisplay.Font, FontStyle.Bold);                    
                    node.NodeFont = f;                    
                }
                //checkTypeHandled(item);
            }

            LogOutput("Added TreeNode: " + node.Text);

            return node;
        }

        private bool checkHighlightItem(Type type)
        {
            if (type == null)
                return false;
            foreach (TypeSelectedHandler handler in mTypeSelectedHandlers)
            {
                if (handler.TypeToFind == type)
                {
                    return true;
                }
            }
            return false;
        }

        private void treDisplay_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                treDisplay.SelectedNode = treDisplay.GetNodeAt(e.X, e.Y);
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (treDisplay.SelectedNode != null)
            {
                string t = treDisplay.SelectedNode.Text;
                int loc = t.IndexOf("=");
                if (loc >= 0)
                {
                    t = t.Substring(loc + 1).Trim();
                }
                //Clipboard.SetText(treDisplay.SelectedNode.Text);
                Clipboard.SetText(t);
            }
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            ClearHandlerMenus();
            if (treDisplay.SelectedNode != null)
            {
                if (treDisplay.SelectedNode.Tag != null)
                {
                    AddHandlersForTag(treDisplay.SelectedNode.Tag);
                }
            }
        }

        private List<ToolStripItem> getAlwaysShowMenuItems()
        {
            List<ToolStripItem> items = new List<ToolStripItem>();
            items.Add(copyToolStripMenuItem);
            items.Add(toolStripSeparator1);
            items.Add(saveOutputToolStripMenuItem);
            return items;
        }


        private void ClearHandlerMenus()
        {
            List<ToolStripItem> itemsToRemove = new List<ToolStripItem>();
            foreach (ToolStripItem item in contextMenuStrip1.Items)
            {
                List<ToolStripItem> doNotRemoveItems = getAlwaysShowMenuItems();
                bool remove = true;
                foreach (ToolStripItem doNotRemoveItem in doNotRemoveItems)
                {
                    if (item == doNotRemoveItem)
                    {
                        remove = false;
                        break;
                    }
                }
                if (remove)
                {
                    itemsToRemove.Add(item);
                }
            }
            foreach (ToolStripItem item in itemsToRemove)
            {
                contextMenuStrip1.Items.Remove(item);
            }
        }

        private void AddHandlersForTag(object tag)
        {
            

            foreach (TypeSelectedHandler handler in mTypeSelectedHandlers)
            {
                if (handler.TypeToFind == tag.GetType())
                {
                    ToolStripItem item = new ToolStripMenuItem(handler.MethodToCallName);
                                        
                    HandlerMenuClicked hmc = new HandlerMenuClicked(handler, tag);
                    item.Click += hmc.MenuClicked;
                    contextMenuStrip1.Items.Add(item);
                    //handler.OnTypeSelectedHandlerEvent(item, handler.MethodToCallType);
                }
            }
        }

        private void saveOutputToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Filter = "XML Files (*.xml)|*.xml";
            saveFileDialog1.FileName = "";
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                SaveOutput(saveFileDialog1.FileName);
            }
        }

        public string OutputToXml()
        {
            XmlDocument xmlDoc = OutputToXmlDocument();
            return xmlDoc.OuterXml;
        }

        private XmlDocument OutputToXmlDocument()
        {
            XmlDocument xmlDoc = new XmlDocument();
            XmlNode rootNode = xmlDoc.CreateElement("ObjectOutput");
            xmlDoc.AppendChild(rootNode);
            OutputNodes(treDisplay.Nodes, rootNode);
            return xmlDoc;
        }

        private void SaveOutput(string filename)
        {            
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                XmlDocument xmlDoc = OutputToXmlDocument();
                xmlDoc.Save(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error saving output, " + ex.Message);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void OutputNodes(TreeNodeCollection treeNodes, XmlNode parentNode)
        {
            foreach (TreeNode node in treeNodes)
            {
                string text = node.Text;
                string name = "";
                string value = "";
                int loc = text.IndexOf("=");
                if (loc >= 0)
                {
                    name = text.Substring(0, loc);
                    value = text.Substring(loc + 1);
                }
                else
                {
                    name = text;
                }

                XmlNode xmlNode = parentNode.OwnerDocument.CreateElement("Object");

                XmlAttribute nameAttr = parentNode.OwnerDocument.CreateAttribute("name");
                xmlNode.Attributes.Append(nameAttr);
                nameAttr.Value = name;

                parentNode.AppendChild(xmlNode);
                if (!string.IsNullOrEmpty(value))
                {
                    XmlText textNode = parentNode.OwnerDocument.CreateTextNode(value.Trim());
                    xmlNode.AppendChild(textNode);
                }

                if (node.Nodes.Count > 0)
                {
                    OutputNodes(node.Nodes, xmlNode);
                }
            }
        }

        /// <summary>
        /// Show the search dialog
        /// </summary>
        /// <returns></returns>
        public bool ShowSearch()
        {
            frmSearch s = new frmSearch();
            if (s.ShowSearchDialog() == DialogResult.OK)
            {
                return SearchForText(s.SearchResult);
            }
            return true;
        }

        /// <summary>
        /// Search for the first instance of a specific string
        /// </summary>
        /// <param name="search"></param>
        /// <returns></returns>
        public bool SearchForText(string search)
        {
            return SearchText(search, treDisplay.Nodes);
        }

        private bool SearchText(string search, TreeNodeCollection nodes)
        {
            foreach (TreeNode node in nodes)
            {
                if (node.Text.IndexOf(search, StringComparison.CurrentCultureIgnoreCase) >= 0)
                {
                    treDisplay.SelectedNode = node;
                    return true;
                }
                if (node.Nodes.Count > 0)
                {
                    if (SearchText(search, node.Nodes))
                        return true;
                }
            }
            return false;
        }
    }

    public class HandlerMenuClicked
    {
        private TypeSelectedHandler handler;
        private object item;

        public HandlerMenuClicked(TypeSelectedHandler handler, object item)
        {
            this.handler = handler;
            this.item = item;
        }

        public void MenuClicked(object sender, EventArgs e)
        {
            handler.OnTypeSelectedHandlerEvent(item, handler.MethodToCallType);
        }
    }
}
