/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixCommon
{
    public class VaSite : IComparable<VaSite>
    {
        private string mstrSiteNumber;
        private string mstrSiteName;
        private string mstrVixServer;
        private int mintVixPort;
        private string mstrVistaServer;
        private string mstrRegionNumber;
        private int mintVistaPort;
        private string mstrSiteAbbr;

        public string SiteAbbr
        {
            get { return mstrSiteAbbr; }
            set { mstrSiteAbbr = value; }
        }

        public string VistaServer
        {
            get { return mstrVistaServer; }
            set { mstrVistaServer = value; }
        }

        public int VistaPort
        {
            get { return mintVistaPort; }
            set { mintVistaPort = value; }
        }        

        public string RegionNumber
        {
            get { return mstrRegionNumber; }
            set { mstrRegionNumber = value; }
        }

        public string SiteNumber
        {
            get { return mstrSiteNumber; }
            set { mstrSiteNumber = value; }
        }        

        public string SiteName
        {
            get { return mstrSiteName; }
            set { mstrSiteName = value; }
        }
        

        public string VixServer
        {
            get { return mstrVixServer; }
            set { mstrVixServer = value; }
        }        

        public int VixPort
        {
            get { return mintVixPort; }
            set { mintVixPort = value; }
        }

        public VaSite(string siteNumber, string siteName, string regionNumber, string vistaServer, int vistaPort,
            string vixServer, int vixPort)
        {
            mstrSiteName = siteName;
            mstrSiteNumber = siteNumber;
            mstrVixServer = vixServer;
            mintVixPort = vixPort;
            mstrVistaServer = VistaServer;
            mintVistaPort = vistaPort;
            mstrRegionNumber = regionNumber;
        }

        public VaSite(string siteNumber, string siteName, string regionNumber, string vistaServer, int vistaPort,
            string vixServer, int vixPort, string siteAbbr)
        {
            mstrSiteName = siteName;
            mstrSiteNumber = siteNumber;
            mstrVixServer = vixServer;
            mintVixPort = vixPort;
            mstrVistaServer = VistaServer;
            mintVistaPort = vistaPort;
            mstrRegionNumber = regionNumber;
            mstrSiteAbbr = siteAbbr;
        }

        public override string ToString()
        {
            return mstrSiteName + "(" + mstrSiteNumber + ")";
        }

        public bool HasVix
        {
            get
            {
                if ((mstrVixServer == null) && (mstrVixServer.Length <= 0))
                {
                    return false;
                }
                if (mintVixPort <= 0)
                    return false;
                return true;
            }
        }

        public int CompareTo(VaSite other)
        {
            return this.mstrSiteName.CompareTo(other.mstrSiteName);
        }
    }

    public class VaSiteVisnComparer : IComparer<VaSite>
    {

        public int Compare(VaSite x, VaSite y)
        {
            int regionX = int.Parse(x.RegionNumber);
            int regionY = int.Parse(y.RegionNumber);

            int i = regionX.CompareTo(regionY);
            if (i != 0)
                return i;
            return x.SiteName.CompareTo(y.SiteName);
        }

        
    }
}
