/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace VixCommon
{
    public class VixCommonUtils
    {
        public static string getApplicationDir(string executablePath)
        {
            FileInfo file = new FileInfo(executablePath);
            return file.Directory.FullName;
        }

        public static string getApplicationDir()
        {
            string path = Path.GetDirectoryName(System.Reflection.Assembly.GetAssembly(typeof(VixCommonUtils)).CodeBase);
            return path;
        }

        public static bool isDateAfter(DateTime dt1, DateTime dt2)
        {
            int val = dt1.CompareTo(dt2);
            if (val > 0)
                return true;
            return false;
        }

        public static string formatTimeSpan(TimeSpan ts, bool showMilliseconds)
        {
            StringBuilder sb = new StringBuilder();
            string prefix = "";
            if (ts.Days > 0)
            {
                sb.Append(ts.Days);
                sb.Append(" days");
                prefix = ", ";
            }
            if (ts.Hours > 0)
            {
                sb.Append(prefix);
                sb.Append(ts.Hours);
                sb.Append(" hours");
                prefix = ", ";
            }
            if (ts.Minutes > 0)
            {
                sb.Append(prefix);
                sb.Append(ts.Minutes);
                sb.Append(" minutes");
                prefix = ", ";
            }
            if (ts.Seconds > 0)
            {
                sb.Append(prefix);
                sb.Append(ts.Seconds);
                sb.Append(" seconds");
                prefix = ", ";
            }
            if (showMilliseconds)
            {
                if (ts.Milliseconds > 0)
                {
                    sb.Append(prefix);
                    sb.Append(ts.Milliseconds);
                    sb.Append(" ms");
                    prefix = ", ";
                }
            }

            return sb.ToString();
        }

        public static string formatBytes(long bytes)
        {
            if (bytes < 1024)
            {
                return bytes + " bytes";
            }
            double kb = (double)bytes / 1024.0f;
            double mb = kb / 1024.0f;
            double gb = mb / 1024.0f;
            double tb = gb / 1024.0f;

            if (tb > 1.0)
            {
                return tb.ToString("N2") + " TB";
            }
            else if (gb > 1.0)
            {
                return gb.ToString("N2") + " GB";
            }
            else if (mb > 1.0)
            {
                return mb.ToString("N2") + " MB";
            }
            else
            {
                return kb.ToString("N2") + " KB";
            }
        }

        public static string GetHostname()
        {
            return System.Environment.MachineName;
        }
    }
}
