/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Mail;

namespace VixCommon.email
{
    public class EmailSender
    {
        public static void EmailMessage(EmailServerConfiguration serverConfiguration, string[] addresses, string emailFrom,
            string subject, string messageBody)
        {
            EmailMessage(serverConfiguration, addresses, emailFrom, subject, messageBody, MailPriority.Normal);
        }

        public static void EmailMessage(EmailServerConfiguration serverConfiguration, string[] addresses, string emailFrom,
            string subject, string messageBody, System.Net.Mail.MailPriority priority)
        {
            SmtpClient client = null;

            if(serverConfiguration.UseIIS)
            {
                client = new SmtpClient();
                client.DeliveryMethod = SmtpDeliveryMethod.PickupDirectoryFromIis;
            }
            else
            {
                client = new SmtpClient(serverConfiguration.SmtpServer);
                client.DeliveryMethod = SmtpDeliveryMethod.Network;
            }

            MailAddress msgFrom = new MailAddress(emailFrom);
            MailMessage message = new MailMessage();
            message.From = msgFrom;
            foreach (string to in addresses)
            {
                message.To.Add(to);
            }
            message.Subject = subject;
            message.Priority = priority;
            message.IsBodyHtml = true;
            message.Body = messageBody;
            client.Send(message);
        }
    }

    public class EmailServerConfiguration
    {
        private readonly string mstrSmtpServer;
        private readonly bool mbUseIIS;

        public string SmtpServer
        {
            get { return mstrSmtpServer; }
        } 

        public bool UseIIS
        {
            get { return mbUseIIS; }
        } 


        private EmailServerConfiguration(string smtpServer, bool useIIS)
        {
            this.mstrSmtpServer = smtpServer;
            this.mbUseIIS = useIIS;
        }

        public static EmailServerConfiguration createSmtpConfiguration(string smtpServer)
        {
            return new EmailServerConfiguration(smtpServer, false);
        }

        public static EmailServerConfiguration createIISConfiguration()
        {
            return new EmailServerConfiguration(null, true);
        }
    }
}
