/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using VixLogAnalyzerEngine;
using System.Windows.Threading;
using System.Collections.ObjectModel;

namespace VixLogAnalyzer
{
    

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private DispatcherTimer timer = null;
        private string logsDirectory = null;
        private SiteServiceHelper siteServiceHelper = new SiteServiceHelper();
        private LogAnalyzerCommon logAnalyzerCommon = null;

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            logsDirectory = GetApplicationDirectory() + "\\logs";
            if (!System.IO.Directory.Exists(logsDirectory))
            {
                System.IO.Directory.CreateDirectory(logsDirectory);
            }

            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += timer1_Tick;
            timer.Start();

            dtFromDate.SelectedDate = DateTime.Now.AddDays(-1);
            dtToDate.SelectedDate = DateTime.Now.AddDays(-1);
            cbTimeOnVixRange.Items.Add(RangeOption.greaterThan);
            cbTimeOnVixRange.Items.Add(RangeOption.lessThan);
            logAnalyzerCommon = new LogAnalyzerCommon(StatusMsg);
            txtSearchDirectory.Text = @"G:\VIXLogs";
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            timer.Stop();
            try
            {
                this.Cursor = Cursors.Wait;
                StatusMsg("Downloading sites");

                List<VASite> vixSites = siteServiceHelper.GetVASites(true);
                vixSites.Add(VASite.AllSites);
                vixSites.Sort();
                foreach (VASite site in vixSites)
                {
                    cboSites.Items.Add(site);
                }
                StatusMsg("Ready");
            }
            finally
            {
                this.Cursor = Cursors.Arrow;
            }
        }

        private void StatusMsg(string msg)
        {
            txtStatus.Text = msg;
        }

        private void btnDownload_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                
                if (cboSites.SelectedItem == null)
                {
                    MessageBox.Show("You must select a site", "Select Site", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                VASite site = (VASite)cboSites.SelectedItem;
                if (site == VASite.AllSites)
                {
                    StringBuilder errors = new StringBuilder();

                    List<VASite> vixSites = siteServiceHelper.GetVASites(true);
                    foreach (VASite s in vixSites)
                    {
                        try
                        {
                            DateTime currentDate = dtFromDate.SelectedDate.Value;
                            DateTime endDate = dtToDate.SelectedDate.Value;
                            while (currentDate <= endDate)
                            {
                                IEnumerable<TransactionLogEntry> results = logAnalyzerCommon.Search(logsDirectory, s,
                                    currentDate, txtQueryType.Text, txtCommandClassName.Text,
                                txtTransactionId.Text, txtErrorMessage.Text, txtExceptionClass.Text, txtPatientICN.Text, txtUrn.Text,
                                (cbTimeOnVixRange.SelectedItem == null ? null : new SearchRange((RangeOption)cbTimeOnVixRange.SelectedItem, slTimeOnVix.Value)),
                                txtRespondingSite.Text,
                                (chkMoreThan0ItemsReturned.IsChecked == true ? true : false), txtDataSource.Text);
                                if (results != null)
                                {
                                    DisplayResults(results, s, currentDate);
                                }
                                currentDate =  currentDate.AddDays(1);
                            }
                        }
                        catch (Exception ex)
                        {
                            errors.AppendLine("Error from site '" + s.SiteNumber + ", " + ex.Message);
                        }
                    }
                    if (errors.Length > 0)
                    {
                        throw new Exception(errors.ToString());
                    }
                }
                else
                {
                    DateTime currentDate = dtFromDate.SelectedDate.Value;
                    DateTime endDate = dtToDate.SelectedDate.Value;
                    while (currentDate <= endDate)
                    {
                        IEnumerable<TransactionLogEntry> results = logAnalyzerCommon.Search(logsDirectory, site,
                            currentDate, txtQueryType.Text, txtCommandClassName.Text,
                            txtTransactionId.Text, txtErrorMessage.Text, txtExceptionClass.Text, txtPatientICN.Text, txtUrn.Text,
                            (cbTimeOnVixRange.SelectedItem == null ? null : new SearchRange((RangeOption)cbTimeOnVixRange.SelectedItem, slTimeOnVix.Value)),
                            txtRespondingSite.Text,
                            (chkMoreThan0ItemsReturned.IsChecked == true ? true : false), txtDataSource.Text);
                        DisplayResults(results, site, currentDate);
                        currentDate = currentDate.AddDays(1);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                this.Cursor = Cursors.Arrow;
            }
        }

        private IEnumerable<TransactionLogEntry> Search(VASite site, DateTime date,
            string queryType, string commandClassName, string transactionId,
            string errorMessage, string exceptionClass, string patientIcn, string urn, SearchRange timeOnVix,
            string respondingSite, bool itemsReturnedMoreThan0, string datasource)
        {
            try
            {
                this.Cursor = Cursors.Wait;
                return logAnalyzerCommon.Search(logsDirectory, site, date, queryType, commandClassName,
                    transactionId, errorMessage, exceptionClass, patientIcn, urn, timeOnVix,
                    respondingSite, itemsReturnedMoreThan0, datasource);
            }
            catch (Exception ex)
            {
                StatusMsg(ex.Message);
                return null;
            }
            finally
            {
                this.Cursor = Cursors.Arrow;
            }
        }

        

        private void DisplayResults(IEnumerable<TransactionLogEntry> results, VASite site, DateTime date)
        {
            if (results.Count<TransactionLogEntry>() > 0)
            {
                TabItem ti = new TabItem();
                ViewTransactionsControl vtc = new ViewTransactionsControl();
                vtc.OnStatusMessageEvent += StatusMsg;
                vtc.OnSearchByIdEvent += SearchById;
                vtc.OnSearchAssociatedSitesEvent += SearchAssociatedSites;
                ti.Header = site.ToString() + " " + date.ToString("MM/dd/yyyy");
                ti.Content = vtc;
                tcTransactionEntries.Items.Add(ti);
                tcTransactionEntries.SelectedItem = ti;
                vtc.DisplayTransactions(results, site);
            }
            else
            {
                StatusMsg("No results to display");
            }
        }

        private void SearchAssociatedSites(VASite site, TransactionLogEntry entry)
        {
            List<TransactionLogEntry> entries = new List<TransactionLogEntry>();

            // this will only work if the transaction selected is the data source transaction
            // should change this so this loads the associated transactions from the same site the user initially queried, find
            // the data source command(s) and then get associated commands from that

            VASite requestingSite = siteServiceHelper.GetVixSiteSite(entry.RequestingSite, entry.DataSourceResponseServer);
            if (requestingSite != null)
            {
                IEnumerable<TransactionLogEntry> results = Search(requestingSite, entry.Date, null, null, entry.TransactionId, 
                    null, null, null, null, null, null, false, null);
                entries.AddRange(results);
            }
            VASite respondingSite = siteServiceHelper.GetVixSiteSite(entry.RespondingSite, entry.DataSourceResponseServer);
            if (respondingSite != null)
            {
                IEnumerable<TransactionLogEntry> results = Search(respondingSite, entry.Date, null, null, entry.TransactionId, 
                    null, null, null, null, null, null, false, null);
                entries.AddRange(results);
            }
            DisplayResults(entries, site, entry.Date);
        }

        

        private void SearchById(VASite site, DateTime date, string transactionId)
        {
            IEnumerable<TransactionLogEntry> results = Search(site, date, null, null, 
                transactionId, null, null, null, null, null, null, false, null);
            DisplayResults(results, site, date);
        }

        /*
        private void dgTransactions_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (dgTransactions.SelectedItem != null)
            {
                TransactionLogEntry logEntry = (TransactionLogEntry)dgTransactions.SelectedItem;
                TransactionWindow tw = new TransactionWindow();
                tw.ShowTransactions(logEntry);
            }
            
        }*/

        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            if (sender is MenuItem)
            {
                MenuItem mi = (MenuItem)sender;
                /*
                if (mi == mnuFileExit)
                {
                    Application.Current.Shutdown();
                }*/
                /*
                if (mi == mnuTransactionsRemoveResult)
                {
                    RemoveSelectedItems();
                }
                else if (mi == mnuTransactionsCopyIdToClipboard)
                {
                    if (dgTransactions.SelectedItem != null)
                    {
                        TransactionLogEntry entry = (TransactionLogEntry)dgTransactions.SelectedItem;
                        Clipboard.SetText(entry.TransactionId);
                        StatusMsg("Copied transaction '" + entry.TransactionId + "' to clipboard");
                    }
                }*/
            }
        }

        /*
        private void RemoveSelectedItems()
        {
            List<TransactionLogEntry> items = new List<TransactionLogEntry>();
            foreach (object item in dgTransactions.SelectedItems)
            {
                items.Add((TransactionLogEntry)item);
            }
            foreach (TransactionLogEntry item in items)
            {
                if (displayedList != null)
                {
                    displayedList.Remove(item);
                    //dgTransactions.Items.Remove(item);
                }
            }
        }

        private void TransactionLog_AutoGenerateColumn(object sender, DataGridAutoGeneratingColumnEventArgs e)
        {
            DataGrid grid = sender as DataGrid;
            if (grid == null) { return; }

            if (e.PropertyName == "Values")
                e.Cancel = true;

            return;
        }*/

        private void slTimeOnVix_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            lblTimeOnVix.Content = ((int)e.NewValue).ToString();
        }

        public static string GetApplicationDirectory()
        {
            return AppDomain.CurrentDomain.BaseDirectory;
            //System.IO.Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName)
        }

        private void grpSites_Expanded(object sender, RoutedEventArgs e)
        {
            if (tcTransactionEntries != null)
            {
                tcTransactionEntries.Margin = new Thickness(tcTransactionEntries.Margin.Left, 198, tcTransactionEntries.Margin.Right, tcTransactionEntries.Margin.Bottom);
            }
        }

        private void grpSites_Collapsed(object sender, RoutedEventArgs e)
        {
            if (tcTransactionEntries != null)
            {
                tcTransactionEntries.Margin = new Thickness(tcTransactionEntries.Margin.Left, 39, tcTransactionEntries.Margin.Right, tcTransactionEntries.Margin.Bottom);
            }
        }

        private void TabMenuItem_Click(object sender, RoutedEventArgs e)
        {
            if (sender is MenuItem)
            {
                MenuItem mi = (MenuItem)sender;
                if (mi == mnuTabCloseTab)
                {
                    object selectedItem = tcTransactionEntries.SelectedItem;
                    if (selectedItem != null)
                        tcTransactionEntries.Items.Remove(selectedItem);
                }
                else if (mi == mnuTabCloseTabs)
                {
                    List<object> items = new List<object>();
                    foreach (object item in tcTransactionEntries.Items)
                    {
                        items.Add(item);
                    }
                    foreach (object item in items)
                    {
                        tcTransactionEntries.Items.Remove(item);
                    }
                }
            }
        }

        private void btnSearchPatients_Click(object sender, RoutedEventArgs e)
        {
            PatientWindow pw = new PatientWindow();
            pw.LogsDirectory = logsDirectory;
            pw.ShowDialog();
        }

        private void btnsearchDirectory_Click(object sender, RoutedEventArgs e)
        {
            try
            {

                if (cboSites.SelectedItem == null)
                {
                    MessageBox.Show("You must select a site", "Select Site", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                if (string.IsNullOrEmpty(txtSearchDirectory.Text))
                {
                    MessageBox.Show("You must specify the directory for the logs", "Specify Logs Directory", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                VASite site = (VASite)cboSites.SelectedItem;
                if (site == VASite.AllSites)
                {
                    StringBuilder errors = new StringBuilder();

                    List<VASite> vixSites = siteServiceHelper.GetVASites(true);
                    foreach (VASite s in vixSites)
                    {
                        try
                        {
                            IEnumerable<TransactionLogEntry> results = logAnalyzerCommon.SearchDirectory(txtSearchDirectory.Text, s,
                                dtFromDate.SelectedDate.Value, txtQueryType.Text, txtCommandClassName.Text,
                            txtTransactionId.Text, txtErrorMessage.Text, txtExceptionClass.Text, txtPatientICN.Text, txtUrn.Text,
                            (cbTimeOnVixRange.SelectedItem == null ? null : new SearchRange((RangeOption)cbTimeOnVixRange.SelectedItem, slTimeOnVix.Value)),
                            txtRespondingSite.Text,
                            (chkMoreThan0ItemsReturned.IsChecked == true ? true : false), txtDataSource.Text);
                            if (results != null)
                            {
                                DisplayResults(results, s, dtFromDate.SelectedDate.Value);
                            }
                        }
                        catch (Exception ex)
                        {
                            errors.AppendLine("Error from site '" + s.SiteNumber + ", " + ex.Message);
                        }
                    }
                    if (errors.Length > 0)
                    {
                        throw new Exception(errors.ToString());
                    }
                }
                else
                {

                    IEnumerable<TransactionLogEntry> results = logAnalyzerCommon.SearchDirectory(txtSearchDirectory.Text, site,
                        dtFromDate.SelectedDate.Value, txtQueryType.Text, txtCommandClassName.Text,
                        txtTransactionId.Text, txtErrorMessage.Text, txtExceptionClass.Text, txtPatientICN.Text, txtUrn.Text,
                        (cbTimeOnVixRange.SelectedItem == null ? null : new SearchRange((RangeOption)cbTimeOnVixRange.SelectedItem, slTimeOnVix.Value)),
                        txtRespondingSite.Text,
                        (chkMoreThan0ItemsReturned.IsChecked == true ? true : false), txtDataSource.Text);
                    DisplayResults(results, site, dtFromDate.SelectedDate.Value);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                this.Cursor = Cursors.Arrow;
            }
        }
    }
}
