/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using VixLogAnalyzerEngine;

namespace VixLogAnalyzer
{
    /// <summary>
    /// Interaction logic for PatientWindow.xaml
    /// </summary>
    public partial class PatientWindow : Window
    {
        public PatientWindow()
        {
            InitializeComponent();
        }

        private LogAnalyzerCommon logAnalyzerCommon = null;
        private SiteServiceHelper siteService = new SiteServiceHelper();
        private List<VASite> sites = null;
        private Dictionary<string, PatientSites> patients = new Dictionary<string, PatientSites>();

        public string LogsDirectory { get; set; }

        private void btnSearchPatients_Click(object sender, RoutedEventArgs e)
        {
            DateTime date = dtDate.SelectedDate.Value;
            sites = siteService.GetVASites(true);
            string [] lines = txtPatients.Text.Split('\n');
            string[] icns = new string[lines.Length];
            Dictionary<string, string> allIcns = new Dictionary<string, string>();
            for (int i = 0; i < lines.Length; i++)
            {
                icns[i] = lines[i].Trim();
                allIcns.Add(icns[i], null);
            }



            patients.Clear();
            //List<VASite> oneSite = new List<VASite>();
            //oneSite.Add(sites[0]);
            foreach (VASite site in sites)
            {
                /*
                if (site.SiteNumber == "537")
                {
                    TestSite(site, icns, date);
                }*/
                TestSite(site, icns, date);
            }

            foreach (PatientSites ps in patients.Values)
            {
                StringBuilder result = new StringBuilder();
                result.Append(ps.Icn + ",");
                string prefix = "";
                foreach (string s in ps.Sites.Keys)
                {
                    result.Append(prefix);
                    result.Append(s);
                    prefix = "^";
                }
                result.Append("\n");
                txtResults.Text = txtResults.Text + result.ToString();
                if (allIcns.ContainsKey(ps.Icn))
                {
                    allIcns.Remove(ps.Icn);
                }
            }
            if (allIcns.Count > 0)
            {
                txtResults.AppendText("------------ NOT FOUND ICNs -----------\n");
                foreach (string icn in allIcns.Keys)
                {
                    txtResults.AppendText(icn + "\n");
                }
            }

            /*
            foreach (string line in lines)
            {
                TestPatient(line.Trim(), date);
            }*/

        }

        private void TestSite(VASite site, string[] icns, DateTime date)
        {
            try
            {
                Dictionary<string, string> sitePatients = new Dictionary<string, string>();
                IEnumerable<TransactionLogEntry> entries =
                            logAnalyzerCommon.Search(LogsDirectory, site, date, icns);
                if (entries != null)
                {
                    foreach (TransactionLogEntry entry in entries)
                    {
                        if (entry.QueryType.StartsWith("Clinical Display") || entry.QueryType.StartsWith("VistARad WebApp"))
                        {
                            if (!sitePatients.ContainsKey(entry.PatientIcn))
                            {
                                sitePatients.Add(entry.PatientIcn, null);
                            }
                        }
                    }

                }
                foreach (string icn in sitePatients.Keys)
                {
                    PatientSites ps = null;
                    if (patients.ContainsKey(icn))
                    {
                        ps = patients[icn];
                    }
                    else
                    {
                        ps = new PatientSites(icn);
                        patients.Add(icn, ps);
                    }
                    if (!ps.Sites.ContainsKey(site.SiteNumber))
                    {
                        ps.Sites.Add(site.SiteNumber, null);
                    }
                }
            }
            catch (Exception)
            {
            }
            
        }

        private void TestPatient(string icn, DateTime date)
        {
            Dictionary<string, string> patientSites = new Dictionary<string, string>();
            foreach (VASite site in sites)
            {
                try
                {
                    IEnumerable<TransactionLogEntry> entries =
                        logAnalyzerCommon.Search(LogsDirectory, site, date, null, null, null, null, null, icn, null, null, null, false, null);
                    if (entries != null)
                    {
                        foreach (TransactionLogEntry entry in entries)
                        {
                            if (!patientSites.ContainsKey(entry.SiteNumber))
                            {
                                patientSites.Add(entry.SiteNumber, null);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message + "\n\n"+ ex.StackTrace);
                    // do nothing right now
                }

            }
            StringBuilder result = new StringBuilder();
            result.Append(icn + ",");
            string prefix = "";
            foreach (string s in patientSites.Keys)
            {
                result.Append(prefix);
                result.Append(s);                
                prefix = "^";
            }
            result.Append("\n");
            txtResults.Text = txtResults.Text + result.ToString();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            logAnalyzerCommon = new LogAnalyzerCommon(StatusMsg);
            dtDate.SelectedDate = new DateTime(2011, 9, 27);
        }

        private void StatusMsg(string msg)
        {

        }


    }

    public class PatientSites
    {
        public string Icn { get; private set; }
        public Dictionary<string, string> Sites { get; private set; }

        public PatientSites(string icn)
        {
            this.Icn = icn;
            Sites = new Dictionary<string, string>();
        }
    }
}
