/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using VixLogAnalyzerEngine;
using System.Collections.ObjectModel;

namespace VixLogAnalyzer
{
    /// <summary>
    /// Interaction logic for ViewTransactionsControl.xaml
    /// </summary>
    public partial class ViewTransactionsControl : UserControl
    {
        public ViewTransactionsControl()
        {
            InitializeComponent();
        }

        public event StatusMessageDelegate OnStatusMessageEvent = null;
        public event SearchByIdDelegate OnSearchByIdEvent = null;
        public event SearchAssociatedSitesDelegate OnSearchAssociatedSitesEvent = null;

        private ObservableCollection<TransactionLogEntry> displayedList = null;
        private VASite site = null;

        public void DisplayTransactions(IEnumerable<TransactionLogEntry> entries, VASite site)
        {
            displayedList = new ObservableCollection<TransactionLogEntry>(entries);
            this.site = site;

            dgTransactions.DataContext = null;
            dgTransactions.DataContext = displayedList;

            StatusMsg("Displayed '" + displayedList.Count + "' log entries");
        }

        private void dgTransactions_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (dgTransactions.SelectedItem != null)
            {
                /*
                TransactionLogEntry logEntry = (TransactionLogEntry)dgTransactions.SelectedItem;
                TransactionWindow tw = new TransactionWindow();
                tw.ShowTransactions(logEntry);
                 * */
            }

        }

        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            if (sender is MenuItem)
            {
                MenuItem mi = (MenuItem)sender;
                /*
                if (mi == mnuFileExit)
                {
                    Application.Current.Shutdown();
                }*/
                if (mi == mnuTransactionsRemoveResult)
                {
                    RemoveSelectedItems();
                }
                else if (mi == mnuTransactionsCopyIdToClipboard)
                {
                    if (dgTransactions.SelectedItem != null)
                    {
                        TransactionLogEntry entry = (TransactionLogEntry)dgTransactions.SelectedItem;
                        Clipboard.SetText(entry.TransactionId);
                        StatusMsg("Copied transaction '" + entry.TransactionId + "' to clipboard");
                    }
                }
                else if (mi == mnuTransactionsSearchById)
                {
                    if (dgTransactions.SelectedItem != null)
                    {
                        TransactionLogEntry entry = (TransactionLogEntry)dgTransactions.SelectedItem;
                        SearchById(entry.Date, entry.TransactionId);
                    }
                }
                else if (mi == mnuSearchAssociatedSites)
                {
                    if (dgTransactions.SelectedItem != null)
                    {
                        TransactionLogEntry entry = (TransactionLogEntry)dgTransactions.SelectedItem;
                        SearchAssociatedSites(entry);
                    }
                }
                else if (mi == mnuTransactionsCopyValues)
                {
                    if (dgTransactions.SelectedItem != null)
                    {
                        TransactionLogEntry entry = (TransactionLogEntry)dgTransactions.SelectedItem;
                        CopyEntryToClipboard(entry);
                    }
                }
            }
        }

        private void CopyEntryToClipboard(TransactionLogEntry entry)
        {
            StringBuilder sb = new StringBuilder();
            foreach (string key in entry.Values.Keys)
            {
                sb.Append(key);
                sb.Append(":");
                sb.Append(entry.Values[key]);
                sb.Append(Environment.NewLine);
            }
            Clipboard.SetText(sb.ToString());
        }

        private void SearchById(DateTime date, string transactionId)
        {
            if (OnSearchByIdEvent != null)
                OnSearchByIdEvent(this.site, date, transactionId);
        }

        private void StatusMsg(string msg)
        {
            if (OnStatusMessageEvent != null)
                OnStatusMessageEvent(msg);
        }

        private void SearchAssociatedSites(TransactionLogEntry entry)
        {
            if (OnSearchAssociatedSitesEvent != null)
            {
                OnSearchAssociatedSitesEvent(this.site, entry);
            }
        }

        private void RemoveSelectedItems()
        {
            List<TransactionLogEntry> items = new List<TransactionLogEntry>();
            foreach (object item in dgTransactions.SelectedItems)
            {
                items.Add((TransactionLogEntry)item);
            }
            foreach (TransactionLogEntry item in items)
            {
                if (displayedList != null)
                {
                    displayedList.Remove(item);
                }
            }
        }

        private void TransactionLog_AutoGenerateColumn(object sender, DataGridAutoGeneratingColumnEventArgs e)
        {
            DataGrid grid = sender as DataGrid;
            if (grid == null) { return; }

            if (e.PropertyName == "Values")
                e.Cancel = true;

            return;
        }
    }
}
