/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VixLogAnalyzerEngine
{
    public class SiteServiceHelper
    {
        private readonly string vaRadiologyOid = "1.3.6.1.4.1.3768";
        private readonly string bhieRadiologyOid = "2.16.840.1.113883.3.42.10012.100001.207";

        private Dictionary<string, VASite> mSites = null;
        public VASite GetVixSiteSite(string siteNumber, string serverName = null)
        {
            if (mSites == null)
            {
                GetVASites(false);
            }

            string sNumber = siteNumber;
            if (siteNumber.StartsWith(vaRadiologyOid))
            {
                // VA Radiology
                sNumber = siteNumber.Substring(vaRadiologyOid.Length + 1);
                
            }
            else if (siteNumber.StartsWith(bhieRadiologyOid))
            {
                // BHIE radiology
                //sNumber = siteNumber.Substring(bhieRadiologyOid.Length + 1);
                // need to get data from CVIX
                sNumber = "2001";
            }

            
            if (serverName != null)
            {
                // switch to using linq
                foreach (string key in mSites.Keys)
                {
                    VASite site = mSites[key];
                    if (site.HasVix && site.VixServer.ToLower().StartsWith(serverName.ToLower()))
                        return site;
                }
            }

            return mSites[sNumber];
        }

        public List<VASite> GetVASites(bool onlyIncludeVixSites)
        {
            List<VASite> result = new List<VASite>();
            ExchangeSiteService.ImagingExchangeSiteServiceSoapClient client =
                new ExchangeSiteService.ImagingExchangeSiteServiceSoapClient();
            IEnumerable<ExchangeSiteService.ImagingExchangeSiteTO> sites = null;
            sites = from site in client.getImagingExchangeSites()
                    select site;
            mSites = new Dictionary<string, VASite>();
            foreach (ExchangeSiteService.ImagingExchangeSiteTO site in sites)
            {
                VASite s = translate(site);
                if (s.SiteNumber != "2001" && s.SiteNumber != "2002" && s.SiteNumber != "2003" && s.SiteNumber != "2004")
                {
                    if (!onlyIncludeVixSites || s.HasVix)
                    {
                        result.Add(s);
                    }
                    mSites.Add(s.SiteNumber, s);
                }
            }
            /*

            if (onlyIncludeVixSites)
            {
                sites = from site in client.getImagingExchangeSites()
                           where !String.IsNullOrWhiteSpace(site.acceleratorServer)
                           select site;
            }
            else
            {
                sites = from site in client.getImagingExchangeSites()
                           select site;
            }
            foreach (ExchangeSiteService.ImagingExchangeSiteTO site in sites)
            {
                result.Add(translate(site));
            }
             * */


            return result;
        }

        private VASite translate(ExchangeSiteService.ImagingExchangeSiteTO site)
        {
            return new VASite(site.siteNumber, site.siteName, site.siteAbbr, site.regionID, 
                site.vistaServer, site.vistaPort, site.acceleratorServer, site.acceleratorPort);
        }
    }
}
