/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using VixLogCollector.Services;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using VixLogCollector.SiteService;
using System.IO;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using VixLogCollector;
using System.Xml;
using System.Xml.Serialization;
using VixLogCollector.Model;

namespace VixLogCollector.Core.Test
{
    /// <summary>
    ///This is a test class for VixLogCollectorServiceTest and is intended
    ///to contain all VixLogCollectorServiceTest Unit Tests
    ///</summary>
    [TestClass()]
    public class VixLogCollectorServiceTest
    {
        private DirectoryInfo logDir = new DirectoryInfo(ConfigurationManager.AppSettings["logFolder"]);
        private string vixId = ConfigurationManager.AppSettings["siteNumber"];

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        [TestInitialize]
        public void Setup()
        {
            logDir.Delete(true);
            logDir.Create();
        }

        /// <summary>
        ///A test for CollectLogForSite
        ///</summary>
        [TestMethod()]
        [DeploymentItem("VixLogCollector.Core.dll")]
        public void CollectLogForSiteTodayTest()
        {
            VixLogCollectorService target = new VixLogCollectorService();
            DateTime date = DateTime.Today;
            target.CollectMissingLogs(vixId);
            VixInfo vix = new VixInfo(vixId);
            VixLogInfo log = vix.GetLog(date);
            Assert.IsTrue(log.LogFileInfo.Exists);
            Assert.IsTrue(log.LogFileInfo.Length > 0);
        }

        /// <summary>
        ///A test for CollectLogForSite
        ///</summary>
        [TestMethod()]
        [DeploymentItem("VixLogCollector.Core.dll")]
        public void CollectLogForSiteSinceLastTest()
        {
            DateTime startDate = DateTime.Today.AddMonths(-2);
            DateTime endDate = DateTime.Today.AddDays(-5);
            CreateDummyLogFiles(vixId, startDate, endDate);
            VixLogCollectorService target = new VixLogCollectorService();
            target.CollectMissingLogs(vixId);
            VixInfo vix = new VixInfo(vixId);
            for (DateTime i = endDate; i <= DateTime.Today.AddDays(-2); i = i.AddDays(1))
            {
                VixLogInfo log = vix.GetLog(i);
                Assert.IsTrue(log.LogFileInfo.Exists);
                Assert.IsTrue(log.LogFileInfo.Length > 0);
            }
        }

        /// <summary>
        ///A test for GetLogList
        ///</summary>
        [TestMethod()]
        public void GetLogListTest()
        {
            CreateDummyLogFiles(vixId, DateTime.Parse("12/30/2009"), DateTime.Parse("1/2/2011"));
            VixLogCollectorService target = new VixLogCollectorService();
            DateTime startDate = DateTime.Parse("12/20/2009");
            DateTime endDate = DateTime.Parse("12/20/2010");
            IEnumerable<VixLogInfo> actual;
            actual = target.GetLogList(startDate, endDate, vixId);
            Assert.IsTrue(actual.Count() == 76);
        }

        [TestMethod()]
        public void GetVixesTest()
        {
            CreateDummyLogFiles(vixId, DateTime.Parse("1/1/2010"), DateTime.Parse("1/1/2011"));
            VixLogCollectorService target = new VixLogCollectorService();
            VixInfo[] expected = new VixInfo[1] { new VixInfo(vixId) { StartLogDate = DateTime.Parse("12/30/2009") } };
            VixInfo[] actual = target.GetVixes();
            Assert.AreEqual(expected.Count(), actual.Count());
        }

        /// <summary>
        ///A test for UpdateVixes
        ///</summary>
        [TestMethod()]
        public void UpdateVixesTest()
        {
            VixLogCollectorService target = new VixLogCollectorService();
            IEnumerable<VixInfo> vixes = target.UpdateVixes();
            Assert.IsTrue(vixes.Count() > 10);
        }

        //[TestMethod()]
        public void CreateTestData()
        {
            DateTime startDate = DateTime.Parse("12/20/2009");
            DateTime endDate = DateTime.Parse("1/20/2011");
            VixLogCollectorService target = new VixLogCollectorService();
            IEnumerable<VixInfo> vixes = target.UpdateVixes();
            foreach (VixInfo vix in vixes)
            {
                CreateDummyLogFiles(vix.Id, startDate, endDate);
            }
        }

        private void CreateDummyLogFiles(String vixId, DateTime startDate, DateTime endDate)
        {
            VixInfo vix = VixInfo.Load(vixId);
            if (vix == null)
            {
                vix = new VixInfo(vixId)
                {
                    StartLogDate = startDate,
                    Name = "Test Vix",
                };
                vix.Save();
            }
            VixLogInfo log;

            for (DateTime i = startDate; i <= endDate; i = i.AddDays(1))
            {
                log = vix.GetLog(i);
                if (!log.LogFileInfo.Directory.Exists)
                    log.LogFileInfo.Directory.Create();
                using (StreamWriter writer = log.LogFileInfo.CreateText())
                {
                    writer.WriteLine("Test log");
                    writer.Flush();
                    writer.Close();
                }
            }
        }
    }
}
