/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Quartz;
using VixLogCollector.Services;
using VixLogCollector.Model;
using System.Net.Mail;
using System.Configuration;
using System.Diagnostics;

namespace VixLogCollector.Jobs
{
    public class DailyErrorEmailJob : IJob
    {
        public void Execute(IJobExecutionContext context)
        {
            try
            {
                Dictionary<VixInfo, VixLogInfo[]> problemVixes = GetProblems();
                if (problemVixes == null || problemVixes.Count == 0)
                {
                    SendSuccessEmail();
                }
                else
                {
                    SendErrorEmail(problemVixes);
                }
            }
            catch (Exception e)
            {
                EventLog appLog = new EventLog("Application")
                {
                    Source = "VIX Log Collector"
                };
                appLog.WriteEntry(e.ToString(), EventLogEntryType.Error);
            }
        }

        private void SendSuccessEmail()
        {
            MailMessage message = new MailMessage();
            message.From = new MailAddress("email8");
            message.Subject = "Log Collection Success";
            message.IsBodyHtml = false;
            message.Body = "All logs successfully collected on " + Environment.MachineName;
            SetEmailAddresses(message);
            SmtpClient smtp = new SmtpClient(ConfigurationManager.AppSettings["smtpServer"]);
            smtp.Send(message);
        }

        private void SendErrorEmail(Dictionary<VixInfo, VixLogInfo[]> problemVixes)
        {
            MailMessage message = new MailMessage();
            message.From = new MailAddress("email8");
            message.Subject = "Log Collection Errors";
            message.IsBodyHtml = true;
            message.Body = BuildErrorMessage(problemVixes);
            SetEmailAddresses(message);
            SmtpClient smtp = new SmtpClient(ConfigurationManager.AppSettings["smtpServer"]);
            smtp.Send(message);
        }

        private Dictionary<VixInfo, VixLogInfo[]> GetProblems()
        {
            IVixLogCollectorService collectorService = new VixLogCollectorService();
            DateTime startDate = DateTime.Now.AddMonths(-2).Date;
            DateTime endDate = DateTime.Now.AddDays(-1).Date;
            return collectorService.GetProblemVixes(startDate, endDate);
        }

        private string BuildErrorMessage(Dictionary<VixInfo, VixLogInfo[]> problemVixes)
        {
            VixLogInfo[] problemLogs;
            VixInfo vix;
            StringBuilder messageBuilder = new StringBuilder();
            messageBuilder.AppendLine("One or more errors were detected in log collection.<br /><br />");
            messageBuilder.Append("Collector: ");
            messageBuilder.Append(Environment.MachineName);
            messageBuilder.AppendLine("<br />");
            messageBuilder.AppendLine("<br />");
            messageBuilder.AppendLine("<table border='1'>");
            foreach (var problem in problemVixes)
            {
                vix = problem.Key;
                problemLogs = problemVixes[problem.Key];
                messageBuilder.AppendLine("<tr><td style='valign:top'>");
                messageBuilder.Append(vix.Name);
                messageBuilder.Append(" (");
                messageBuilder.Append(vix.Id);
                messageBuilder.Append(")");
                messageBuilder.AppendLine("</td><td>");
                foreach (VixLogInfo log in problemLogs)
                {
                    messageBuilder.Append("<a href='");
                    messageBuilder.Append(log.Vix.GetLogUri(log.LogDate));
                    messageBuilder.Append("'>");
                    messageBuilder.Append(log.LogDate.ToString("MM/dd/yyyy"));
                    messageBuilder.Append("</a> ");
                }
                messageBuilder.AppendLine("</td></tr>");
            }
            messageBuilder.AppendLine("</table>");
            return messageBuilder.ToString();
        }

        private void SetEmailAddresses(MailMessage message)
        {
            String emailAddressString = ConfigurationManager.AppSettings["emailAddress"];
            foreach (string emailAddress in emailAddressString.Split(','))
                message.To.Add(emailAddress);

        }
    }
}
