/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Quartz;
using VixLogCollector.SiteService;
using System.Net;
using System.Configuration;
using System.IO;
using VixLogCollector.Model;
using System.Diagnostics;
using Quartz.Impl.Triggers;

namespace VixLogCollector.Jobs
{
    public class LogCollectorJob : IJob
    {
        private const string fileExtension = ".tsv";
        private static readonly char pathSeperator = Path.DirectorySeparatorChar;

        private WebClient Client = new WebClient()
        {
            Credentials = new NetworkCredential()
            {
                UserName = ConfigurationManager.AppSettings["username"],
                Password = ConfigurationManager.AppSettings["password"]
            }
        };

        public void Execute(IJobExecutionContext context)
        {
            VixInfo vix = (VixInfo)context.Trigger.JobDataMap["vix"];
            try
            {
                Services.IVixLogCollectorService service = new Services.VixLogCollectorService();
                service.CollectMissingLogs(vix.Id);
            }
            catch(Exception e)
            {
                EventLog appLog = new EventLog("Application")
                {
                    Source = "VIX Log Collector"
                };
                appLog.WriteEntry(e.ToString(), EventLogEntryType.Error);

                // if it failed, retry it in 10 minutes
                int retryDelaySeconds = 1200;
                int.TryParse(ConfigurationManager.AppSettings["retryDelaySeconds"], out retryDelaySeconds);
                ISimpleTrigger trigger = new SimpleTriggerImpl(vix.Id, "LogCollector", DateTime.Now.AddSeconds(retryDelaySeconds), null, 0, TimeSpan.Zero);
                trigger.JobDataMap["vix"] = vix;
                context.Scheduler.ScheduleJob(trigger);
            }
        }
    }
}
