/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Quartz;
using VixLogCollector.SiteService;
using VixLogCollector.Model;
using VixLogCollector.Services;
using System.Diagnostics;
using Quartz.Impl.Triggers;

namespace VixLogCollector.Jobs
{
    public class ScheduleLogCollectionJob : IJob
    {
        public void Execute(IJobExecutionContext context)
        {
            try
            {
                int? delaySeconds = (int?)context.Trigger.JobDataMap["delaySeconds"];
                if (delaySeconds == null)
                    delaySeconds = 1;
                IVixLogCollectorService service = new VixLogCollectorService();
                IEnumerable<VixInfo> vixes = service.GetVixes();
                IScheduler scheduler = context.Scheduler;
                ISimpleTrigger trigger;
                foreach (VixInfo vix in vixes)
                {
                    trigger = new SimpleTriggerImpl(vix.Id, "VixLog", DateTime.Now.AddSeconds(delaySeconds.Value))
                    {
                        JobName = "LogCollector",
                        JobGroup = "VixLog",
                        MisfireInstruction = MisfireInstruction.SmartPolicy
                    };
                    trigger.JobDataMap.Add("vix", vix);
                    scheduler.ScheduleJob(trigger);
                }
            }
            catch (Exception e)
            {
                EventLog appLog = new EventLog("Application")
                {
                    Source = "VIX Log Collector"
                };
                appLog.WriteEntry(e.ToString(), EventLogEntryType.Error);
            }
        }
    }
}
