/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration.Install;
using System.Linq;
using System.IO;
using System.Configuration;
using System.Diagnostics;
using System.Security.AccessControl;
using System.ServiceProcess;
using System.ServiceModel.Configuration;
using Microsoft.Win32;
using VixLogCollector.Model;
using System.Security.Principal;
using System.Xml.Linq;
using System.Xml.XPath;


namespace VixLogCollector.WindowsService
{
    [RunInstaller(true)]
    public partial class ProjectInstaller : System.Configuration.Install.Installer
    {
        private NTAccount networkServiceAccount = new NTAccount(@"NT AUTHORITY\NETWORK SERVICE");

        public ProjectInstaller()
        {
            InitializeComponent();
            this.Committed += new InstallEventHandler(ProjectInstaller_Committed);
        }

        private void ProjectInstaller_Committed(object sender, InstallEventArgs e)
        {
//#if DEBUG
//            Debugger.Launch();
//#endif
            if (this.Context.Parameters.ContainsKey("excludeVixes"))
            {
                String[] excludeVixes = this.Context.Parameters["excludeVixes"].Split(',');
                UpdateConfigFile();
                SetupLogDirectory();
                ExcludeVixes(excludeVixes);
            }
            StartService();
        }

        private void UpdateConfigFile()
        {
            String logFolder = this.Context.Parameters["logFolder"];
            String emailAddress = this.Context.Parameters["emailAddress"];
            String siteService = this.Context.Parameters["siteService"];
            String targetDir = this.Context.Parameters["targetDir"];
            String exePath = string.Format("{0}VixLogCollector.WindowsService.exe", targetDir);
            String configPath = string.Format("{0}.config", exePath);

            XDocument configXml = XDocument.Load(configPath);
            if (!String.IsNullOrWhiteSpace(logFolder))
            {
                configXml.XPathSelectElement("/configuration/appSettings/add[@key='logFolder']")
                    .SetAttributeValue(XName.Get("value"), logFolder);
                configXml.XPathSelectElement("/configuration/log4net/appender[@name='RollingFileAppender']/file")
                    .SetAttributeValue(XName.Get("value"), string.Format(@"{0}\AppLogs\VixLogCollector.log", logFolder));
            }
            if (!String.IsNullOrWhiteSpace(emailAddress))
                configXml.XPathSelectElement("/configuration/appSettings/add[@key='emailAddress']")
                    .SetAttributeValue(XName.Get("value"), emailAddress);
            if (!String.IsNullOrWhiteSpace(siteService))
                configXml.XPathSelectElement("/configuration/system.serviceModel/client/endpoint[@name='ImagingExchangeSiteServiceSoap']")
                    .SetAttributeValue(XName.Get("address"), siteService);
            configXml.Save(configPath);
        }

        private void SetupLogDirectory()
        {
            DirectoryInfo dir = new DirectoryInfo(this.Context.Parameters["logFolder"]);
            if (!dir.Exists)
            {
                DirectorySecurity dSecurity = dir.Parent.GetAccessControl();
                dSecurity.AddAccessRule(new FileSystemAccessRule(networkServiceAccount,
                                                                FileSystemRights.FullControl,
                                                                InheritanceFlags.ContainerInherit | InheritanceFlags.ObjectInherit,
                                                                PropagationFlags.InheritOnly,
                                                                AccessControlType.Allow));
                dir.Create(dSecurity);
            }
        }

        private void StartService()
        {
            if (!IsServerClusterNode())
            {
                using (var serviceController = new ServiceController(this.serviceInstaller1.ServiceName))
                    serviceController.Start();
            }
        }

        private void ExcludeVixes(String[] excludeVixes)
        {
            if (excludeVixes == null)
                return;
            VixInfo.LogDirectory = new DirectoryInfo(this.Context.Parameters["logFolder"]);
            foreach (String vixId in excludeVixes)
            {
                VixInfo vix = VixInfo.Load(vixId);
                if (vix == null)
                {
                    vix = new VixInfo(vixId);
                }
                vix.IsActive = false;
                vix.Save();
            }
        }

        #region ClusterNode Methods

        private static bool IsServerClusterNode()
        {
            bool isCluster = false;
            if (IsWindowsServer2003() || IsWindowsServer2008())
            {
                string key = @"Cluster";
                using (RegistryKey cluster = Registry.LocalMachine.OpenSubKey(key))
                {
                    if (cluster != null)
                    {
                        isCluster = true;
                    }
                }
            }
            return isCluster;
        }

        private static bool IsWindowsServer2003()
        {
            bool is2003 = false;
            if (Environment.OSVersion.Platform == PlatformID.Win32NT)
            {
                if (Environment.OSVersion.Version.Major == 5 && Environment.OSVersion.Version.Minor == 2)
                {
                    is2003 = true;
                }
            }
            return is2003;
        }

        private static bool IsWindowsServer2008()
        {
            bool is2008 = false;
            if (Environment.OSVersion.Platform == PlatformID.Win32NT)
            {
                if (Environment.OSVersion.Version.Major == 6 && Environment.OSVersion.Version.Minor == 1)
                {
                    is2008 = true;
                }
            }
            return is2008;
        }

        #endregion
    }
}
