/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using VixLogErrorCollectorUtility;

namespace VixLogErrorCollector
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private string mstrLogDirectory = null;
        private List<TransactionLogEntry> transactionLogEntries = null;
        private List<VistaSite> sites = new List<VistaSite>();

        private void frmMain_Load(object sender, EventArgs e)
        {
            dtStartDate.Value = dtStartDate.Value.AddDays(-1);
            dtEndDate.Value= dtEndDate.Value.AddDays(-1);
            sites.AddRange(SiteServiceHelper.GetVixSites());
            foreach (VistaSite site in sites)
            {
                int idx = chkSites.Items.Add(site);
                chkSites.SetItemChecked(idx, true);
                
            }
            mstrLogDirectory = Utils.getApplicationDir(Application.ExecutablePath) + "\\logs";
            if (!Directory.Exists(mstrLogDirectory))
            {
                Directory.CreateDirectory(mstrLogDirectory);
            }
            lstReport.ListViewItemSorter = new ReportListViewSorter();
            rbGroupDate.Checked = true;
        }

        private void LoadSelectedTransactionLogEntries()
        {
            transactionLogEntries = new List<TransactionLogEntry>();
            DateTime startDate = clearDateTime(dtStartDate.Value);
            DateTime endDate = clearDateTime(dtEndDate.Value);
            foreach (object obj in chkSites.CheckedItems)
            {
                VistaSite site = (VistaSite)obj;
                TransactionLogDownloader.DownloadLog(site, startDate, endDate, mstrLogDirectory);
                transactionLogEntries.AddRange(TransactionLogParser.ParseLogs(mstrLogDirectory, site.SiteNumber, startDate, endDate));

            }
        }

        private void btnGenerateReport_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                LoadSelectedTransactionLogEntries();
                //int errors = ReportGenerator.CountErrors(entries);
                //MessageBox.Show("Got [" + entries.Count + "] entries, [" + errors + "] errors");
                //ListErrors(entries);
                ListExceptions(transactionLogEntries);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Exception, " + ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }

            // count stuff up
        }

        private DateTime clearDateTime(DateTime dt)
        {
            DateTime d = new DateTime(dt.Year, dt.Month, dt.Day);
            return d;
        }

        private void ListExceptions(List<TransactionLogEntry> entries)
        {
            lstReport.Items.Clear();
            lstErrorDetails.Items.Clear();
            foreach (TransactionLogEntry entry in entries)
            {
                if (entry.IsError())
                    //AddEntryToList(entry);
                    AddExceptionToList(entry);
            }
        }

        private void ListErrors(List<TransactionLogEntry> entries)
        {
            lstReport.Items.Clear();
            lstErrorDetails.Items.Clear();
            foreach (TransactionLogEntry entry in entries)
            {
                if (entry.IsError())
                    AddEntryToList(entry);
            }
        }

        private void AddEntryToList(TransactionLogEntry entry)
        {
            ListViewItem item = new ListViewItem();
            item.Text = entry.SiteNumber;
            item.SubItems.Add(entry.TransactionDate.ToString(Utils.getDateTimeFormat()));
            item.SubItems.Add(entry.TimeOnVix.ToString());
            item.SubItems.Add(entry.QueryType);
            
            item.SubItems.Add(entry.TransactionId);
            item.SubItems.Add(entry.URN);
            item.SubItems.Add(entry.Quality);
            item.SubItems.Add(entry.ResponseCode);
            item.SubItems.Add(entry.ExceptionClassName);
            item.SubItems.Add(entry.ErrorMessage);
            item.Tag = entry;
            //lstReport.Items.Add(item);

            ListViewGroup group = getItemGroup(entry);
            item.Group = group;
            //group.Items.Add(item);
            lstReport.Items.Add(item);
            



            //AddExceptionToList(entry);
        }

        private ListViewGroup getItemGroup(TransactionLogEntry entry)
        {
            if (rbGroupSite.Checked)
            {
                ListViewGroup group = lstReport.Groups[entry.SiteNumber];
                if (group == null)
                {
                    group = new ListViewGroup(entry.SiteNumber, getSiteName(entry.SiteNumber));
                    lstReport.Groups.Add(group);
                }
                return group;
            }
            else
            {
                string dateString = entry.TransactionDate.ToString("yyyy-MM-dd");

                ListViewGroup group = lstReport.Groups[dateString];
                if (group == null)
                {
                    group = new ListViewGroup(dateString, entry.TransactionDate.ToString("MMM dd, yyyy"));
                    lstReport.Groups.Add(group);
                }
                return group;
            }
        }

        private VistaSite getSite(string siteNumber)
        {
            foreach (VistaSite site in sites)
            {
                if (site.SiteNumber == siteNumber)
                    return site;
            }
            return null;
        }

        private string getSiteName(string siteNumber)
        {
            VistaSite site = getSite(siteNumber);
            if (site == null)
                return "";
            return site.SiteName;
        }

        private void AddExceptionToList(TransactionLogEntry entry)
        {
            bool found = false;
            foreach (ListViewItem item in lstErrorDetails.Items)
            {
                TransactionLogEntry curEntry = (TransactionLogEntry)item.Tag;
                if (curEntry.ExceptionClassName == entry.ExceptionClassName)
                {
                    int count = int.Parse(item.SubItems[1].Text);
                    count++;
                    item.SubItems[1].Text = count.ToString();
                    found = true;
                }
            }
            if (!found)
            {
                ListViewItem item = new ListViewItem();
                item.Text = entry.ExceptionClassName;
                item.SubItems.Add("1");
                item.Tag = entry;
                lstErrorDetails.Items.Add(item);
            }
        }

        private void cxErrorReport_Opening(object sender, CancelEventArgs e)
        {
            if (lstReport.SelectedItems.Count > 0)
            {
                copyTransactionIdToolStripMenuItem.Enabled = true;
            }
            else
            {
                copyTransactionIdToolStripMenuItem.Enabled = false;
            }
        }

        private void copyTransactionIdToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (lstReport.SelectedItems.Count > 0)
            {
                TransactionLogEntry entry = (TransactionLogEntry)lstReport.SelectedItems[0].Tag;
                Clipboard.SetText(entry.TransactionId);
            }
        }

        private int sortColumn = 0;

        private void lstReport_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (e.Column == sortColumn)
            {
                if (lstReport.Sorting == SortOrder.Ascending)
                {
                    lstReport.Sorting = SortOrder.Descending;
                }
                else
                {
                    lstReport.Sorting = SortOrder.Ascending;
                }
            }
            sortColumn = e.Column;
            lstReport.Sort();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void findTransactionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmFindTransaction findTransaction = new frmFindTransaction();
            if (findTransaction.ShowFindTransaction() == DialogResult.OK)
            {
                selectTransaction(findTransaction.TransactionId);
            }
        }

        private void selectTransaction(string transactionId)
        {
            foreach (ListViewItem item in lstReport.Items)
            {
                item.BackColor = Color.White;
            }

            foreach (ListViewItem item in lstReport.Items)
            {
                TransactionLogEntry entry = (TransactionLogEntry)item.Tag;
                if (entry.TransactionId == transactionId)
                {
                    lstReport.EnsureVisible(item.Index);
                    item.BackColor = Color.Yellow;                    
                    break;
                }
            }
        }

        private void selectExceptionClassName(string exceptionClassName)
        {
            foreach (ListViewItem item in lstReport.Items)
            {
                TransactionLogEntry entry = (TransactionLogEntry)item.Tag;
                if (entry.ExceptionClassName == exceptionClassName)
                {
                    item.ForeColor = Color.Red;
                }
                else
                {
                    item.ForeColor = Color.Black;
                }
            }
        }

        private void lstErrorDetails_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (lstErrorDetails.SelectedItems.Count > 0)
            {
                TransactionLogEntry entry = (TransactionLogEntry)lstErrorDetails.SelectedItems[0].Tag;
                //selectExceptionClassName(entry.ExceptionClassName);
                DisplayEntriesForExceptionClass(entry.ExceptionClassName);
            }
        }

        private void DisplayEntriesForExceptionClass(string exceptionClass)
        {
            lstReport.Items.Clear();
            foreach (TransactionLogEntry entry in transactionLogEntries)
            {
                if ((entry.IsError()) && (entry.ExceptionClassName == exceptionClass))
                    AddEntryToList(entry);
            }
        }

        private void lstReport_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (lstReport.SelectedItems.Count > 0)
            {
                TransactionLogEntry entry = (TransactionLogEntry)lstReport.SelectedItems[0].Tag;
                DisplayTransactionLog(entry);
            }
        }

        private void DisplayTransactionLog(TransactionLogEntry entry)
        {
            frmViewLogEntry viewLog = new frmViewLogEntry();
            viewLog.ShowTransactionLogEntry(entry);
        }

        private void tabPage1_Click(object sender, EventArgs e)
        {

        }

        private void btnOutputXml_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                LoadSelectedTransactionLogEntries();

                string xmlOutput = Utils.getApplicationDir(Application.ExecutablePath) + "\\xml";
                if (!Directory.Exists(xmlOutput))
                    Directory.CreateDirectory(xmlOutput);

                TransactionLogXmlOutput.OutputToXml(this.transactionLogEntries, xmlOutput);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Exception, " + ex.Message + "\n\n" + ex.StackTrace);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void btnLogParserTest_Click(object sender, EventArgs e)
        {
            DirectoryInfo logsDir = new DirectoryInfo(@"C:\work\VIX\Transaction Log");


            DateTime st = new DateTime(2009, 12, 10);
            DateTime et = new DateTime(2009, 12, 31);
            List<TransactionLogEntry> entries =
                VixLogErrorCollectorUtility.TransactionLogParser.ParseLogs(logsDir.FullName,
                "688", st, et);
            MessageBox.Show("Got '" + entries.Count + "' entries");
        }
    }
}
