/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using VixLogErrorCollectorUtility;

namespace VixLogErrorCollector
{
    public partial class frmViewLogEntry : Form
    {
        public frmViewLogEntry()
        {
            InitializeComponent();
        }

        private TransactionLogEntry mEntry = null;

        public void ShowTransactionLogEntry(TransactionLogEntry entry)
        {
            mEntry = entry;
            this.Text = "Transaction Log Entry - " + entry.TransactionId;
            DisplayEntryDetails(mEntry);
            txtDetails.Text = mEntry.ErrorMessage;
            this.Show();
        }

        private void frmViewLogEntry_Load(object sender, EventArgs e)
        {

        }

        private void DisplayEntryDetails(TransactionLogEntry item)
        {

            lstEntry.Items.Clear();

            PropertyInfo[] properties = item.GetType().GetProperties();
            foreach (PropertyInfo property in properties)
            {
                if (property.CanRead)
                {
                    Object propertyValue = property.GetValue(item, null);
                    if (propertyValue == null)
                    {
                        addItemToList(property.Name, propertyValue + "");
                    }
                    else
                    {
                        Type propertyType = propertyValue.GetType();
                        if (propertyType == typeof(String))
                        {
                            addItemToList(property.Name, propertyValue.ToString());
                        }
                        else if (propertyType == typeof(int))
                        {
                            addItemToList(property.Name, propertyValue.ToString());
                        }
                        else if (propertyType == typeof(String[]))
                        {
                            string[] values = (string[])propertyValue;
                            string value = "";
                            for (int i = 0; i < values.Length; i++)
                            {
                                if (value.Length > 0)
                                {
                                    value += ", " + values[i];
                                }
                                else
                                {
                                    value = values[i];
                                }
                            }
                            addItemToList(property.Name, value);
                        }
                    }
                }
            }



        }

        private void addItemToList(string itemName, string itemValue)
        {
            ListViewItem item = new ListViewItem(itemName);
            item.SubItems.Add(itemValue);
            lstEntry.Items.Add(item);
        }

        private void lstEntry_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstEntry.SelectedItems.Count > 0)
            {
                txtDetails.Text = lstEntry.SelectedItems[0].SubItems[1].Text;
            }
            else
            {
                txtDetails.Text = "";
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
