/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Mail;

namespace VixLogErrorCollectorUtility
{
    public class TransactionLogEmailer
    {

        private static string donotreplyEmail = "<donotreply@domain>";

        public static void SendTestEmail(string[] addresses, bool useIIS, string smtpServer)
        {
            string emailFrom = "VIX Error Monitor Test" + donotreplyEmail;
            string message = "This is a test message from the email collector " + DateTime.Now.ToString();
            string subject = "VIX Error monitor test";
            EmailMessage(addresses, useIIS, emailFrom, smtpServer, subject, message);
        }

        public static void EmailTransactions(List<TransactionLogEntry> entries, string[] addresses, DateTime processDate,
            bool useIIS, string smtpServer, string webServerPath)
        {
            string emailFrom = "VIX Error Monitor " + donotreplyEmail;
            string message = createMessageBody(entries, processDate, webServerPath);
            string subject = "VIX Error Monitor found '" + entries.Count + "' critical errors on " + processDate.ToString("MMM dd, yyyy");
            EmailMessage(addresses, useIIS, emailFrom, smtpServer, subject, message);

            /*
            SmtpClient client = null;

            if (useIIS)
            {
                client = new SmtpClient();
                client.DeliveryMethod = SmtpDeliveryMethod.PickupDirectoryFromIis;
            }
            else
            {
                client = new SmtpClient(smtpServer);
                client.DeliveryMethod = SmtpDeliveryMethod.Network;
            }

            MailAddress msgFrom = new MailAddress("VIX Error Monitor <donotreply@domain>");
            MailMessage message = new MailMessage();
            message.From = msgFrom;
            foreach (string to in addresses)
            {
                message.To.Add(to);
            }
            message.Subject = "VIX Error Monitor found '" + entries.Count + "' critical errors on " + processDate.ToString("MMM dd, yyyy");
            message.Priority = MailPriority.High;
            message.IsBodyHtml = true;
            message.Body = createMessageBody(entries, processDate, webServerPath);
            client.Send(message);
             * */
        }

        public static void EmailProcessingErrorMessage(string[] addresses, Exception ex, DateTime errorDate,
            bool useIIS, string smtpServer)
        {
            string emailFrom = "VIX Error Monitor " + donotreplyEmail;
            StringBuilder message = new StringBuilder();
            message.AppendLine("Error processing logs on " + errorDate.ToString() + "<br>");
            message.AppendLine(ex.Message + "<br>");
            message.AppendLine(ex.StackTrace + "<br>");
            string subject = "VIX Error Monitor Processing Error on " + errorDate.ToString("MMM dd, yyyy");
            EmailMessage(addresses, useIIS, emailFrom, smtpServer, subject, message.ToString());
        }

        private static void EmailMessage(string[] addresses, bool useIIS, string emailFrom,
            string smtpServer, string subject, string messageBody)
        {
            SmtpClient client = null;

            if (useIIS)
            {
                client = new SmtpClient();
                client.DeliveryMethod = SmtpDeliveryMethod.PickupDirectoryFromIis;
            }
            else
            {
                client = new SmtpClient(smtpServer);
                client.DeliveryMethod = SmtpDeliveryMethod.Network;
            }

            MailAddress msgFrom = new MailAddress(emailFrom);
            MailMessage message = new MailMessage();
            message.From = msgFrom;
            foreach (string to in addresses)
            {
                message.To.Add(to);
            }
            message.Subject = subject;
            message.Priority = MailPriority.High;
            message.IsBodyHtml = true;
            message.Body = messageBody;
            client.Send(message);
        }

        private static string createMessageBody(List<TransactionLogEntry> entries, DateTime processDate, string webServerPath)
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">");
            sb.AppendLine("<html>");
            sb.AppendLine("<HTML><HEAD><META http-equiv=Content-Type content=\"text/html; charset=iso-8859-1\"></HEAD>");
            sb.AppendLine("<body>");

            sb.AppendLine("<h2>Error Report for " + processDate.ToString("MMM dd, yyyy") + "</h2>");
            sb.AppendLine("<table border='1'>");
            sb.AppendLine("<tr><th>Transaction Id</th><th>Date</th><th>Site</th><th>Time on VIX</th><th>Exception</th><th>Query Type</th><th>Error Message</th></tr>");
            foreach (TransactionLogEntry entry in entries)
            {
                sb.Append("<tr>");
                sb.Append("<td>");
                sb.Append("<a href=\"" + webServerPath + "\\" + processDate.ToString(Utils.getDateTimeFormat()) + ".xml\">");
                sb.Append(entry.TransactionId);
                sb.Append("</a>");
                sb.Append("</td>");
                sb.Append("<td>" + entry.TransactionDate.ToString() + "</td>");
                sb.Append("<td>" + entry.SiteNumber + "</td>");
                sb.Append("<td>" + entry.TimeOnVix + "</td>");
                sb.Append("<td>" + entry.ExceptionClassName + "</td>");
                sb.Append("<td>" + entry.QueryType + "</td>");
                sb.Append("<td>" + entry.ErrorMessage + "</td>");
                sb.Append("</tr>");
                sb.Append(Environment.NewLine);
            }
            sb.AppendLine("</table>");

            List<VistaSite> sites = SiteServiceHelper.GetVixSites();

            sb.AppendLine("<h2>Site Key</h2>");
            sb.AppendLine("<table border='1'>");
            sb.AppendLine("<tr><th>Site Number</th><th>Site Name</th></tr>");
            foreach (VistaSite site in sites)
            {
                sb.Append("<tr>");
                sb.Append("<td>" + site.SiteNumber + "</td>");
                sb.Append("<td>" + site.SiteName + "</td>");
                sb.Append("</tr>");
                sb.Append(Environment.NewLine);
            }
            sb.AppendLine("</table>");

            sb.AppendLine("</body>");
            sb.AppendLine("</html>");
            return sb.ToString();
        }
    }
}
