/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;

namespace VixLogErrorCollectorUtility
{
    public class TransactionLogEntry
    {
        private Dictionary<string, string> mValues = new Dictionary<string, string>();

        public Dictionary<string, string> Values
        {
            get { return mValues; }
        }

        public static int maxOkTimeOnVix = 1000 * 60 * 15; // 15 minutes

        private string mstrMachineName;
        private string mstrQueryFilter;
        private string mstrFacadeBytesReturned;
        private string mstrFadeBytesReceived;
        private string mstrDataSourceBytesReturned;
        private string mstrDataSourceBytesReceived;
        private string mstrRequestingSite;
        private string mstrPurposeOfUse;
        private string mstrRealmSiteNumber;
        private int mintTimeToFirstByte = -1;
        private string mstrVixSoftwareVersion;
        private string mstrDataSourceItemsReceived;
        private string mstrCommandClassName;
        private bool mbAsynchronous = false;
        private string mstrRespondingSite;

        public string QueryFilter
        {
            get { return mstrQueryFilter; }
            set { mstrQueryFilter = value; }
        }        

        public string FacadeBytesReturned
        {
            get { return mstrFacadeBytesReturned; }
            set { mstrFacadeBytesReturned = value; }
        }        

        public string FadeBytesReceived
        {
            get { return mstrFadeBytesReceived; }
            set { mstrFadeBytesReceived = value; }
        }        

        public string DataSourceBytesReturned
        {
            get { return mstrDataSourceBytesReturned; }
            set { mstrDataSourceBytesReturned = value; }
        }        

        public string DataSourceBytesReceived
        {
            get { return mstrDataSourceBytesReceived; }
            set { mstrDataSourceBytesReceived = value; }
        }

        public string MachineName
        {
            get { return mstrMachineName; }
            set { mstrMachineName = value; }
        }        

        public string RequestingSite
        {
            get { return mstrRequestingSite; }
            set { mstrRequestingSite = value; }
        }
        
        public string PurposeOfUse
        {
            get { return mstrPurposeOfUse; }
            set { mstrPurposeOfUse = value; }
        }
        

        public string RealmSiteNumber
        {
            get { return mstrRealmSiteNumber; }
            set { mstrRealmSiteNumber = value; }
        }        

        public int TimeToFirstByte
        {
            get { return mintTimeToFirstByte; }
            set { mintTimeToFirstByte = value; }
        }        

        public string VixSoftwareVersion
        {
            get { return mstrVixSoftwareVersion; }
            set { mstrVixSoftwareVersion = value; }
        }
        
        public string DataSourceItemsReceived
        {
            get { return mstrDataSourceItemsReceived; }
            set { mstrDataSourceItemsReceived = value; }
        }        

        public bool Asynchronous
        {
          get { return mbAsynchronous; }
          set { mbAsynchronous = value; }
        }                

        public string RespondingSite
        {
            get { return mstrRespondingSite; }
            set { mstrRespondingSite = value; }
        }


        public string CommandClassName
        {
            get { return mstrCommandClassName; }
            set { mstrCommandClassName = value; }
        }

        private string mstrModality;

        public string Modality
        {
            get { return mstrModality; }
            set { mstrModality = value; }
        }

        private int mintTimeOnVix;

        public int TimeOnVix
        {
            get { return mintTimeOnVix; }
            set { mintTimeOnVix = value; }
        }

        private string mstrTransactionId;

        public string TransactionId
        {
            get { return mstrTransactionId; }
            set { mstrTransactionId = value; }
        }
        private DateTime mdtTransactionDate;

        public DateTime TransactionDate
        {
            get { return mdtTransactionDate; }
            set { mdtTransactionDate = value; }
        }
        private readonly string mstrSiteNumber;

        public string SiteNumber
        {
            get { return mstrSiteNumber; }            
        }
        private string mstrPatientIcn;

        public string PatientIcn
        {
            get { return mstrPatientIcn; }
            set { mstrPatientIcn = value; }
        }
        private string mstrQueryType;

        public string QueryType
        {
            get { return mstrQueryType; }
            set { mstrQueryType = value; }
        }
        private string mstrItemsReturned;

        public string ItemsReturned
        {
            get { return mstrItemsReturned; }
            set { mstrItemsReturned = value; }
        }
        private string mstrQuality;

        public string Quality
        {
            get { return mstrQuality; }
            set { mstrQuality = value; }
        }
        private string mstrUserIPAddress;

        public string UserIPAddress
        {
            get { return mstrUserIPAddress; }
            set { mstrUserIPAddress = value; }
        }
        private string mstrUser;

        public string User
        {
            get { return mstrUser; }
            set { mstrUser = value; }
        }
        private string mstrURN;

        public string URN
        {
            get { return mstrURN; }
            set { mstrURN = value; }
        }
        private bool mbCached;

        public bool Cached
        {
            get { return mbCached; }
            set { mbCached = value; }
        }
        private string mstrErrorMessage;

        public string ErrorMessage
        {
            get { return mstrErrorMessage; }
            set { mstrErrorMessage = value; }
        }
        private string mstrResponseCode;

        public string ResponseCode
        {
            get { return mstrResponseCode; }
            set { mstrResponseCode = value; }
        }
        private int mintBytesTransfered;

        public int BytesTransfered
        {
            get { return mintBytesTransfered; }
            set { mintBytesTransfered = value; }
        }
        private string mstrDatasource;

        public string Datasource
        {
            get { return mstrDatasource; }
            set { mstrDatasource = value; }
        }
        private string mstrExceptionClassName;

        public string ExceptionClassName
        {
            get { return mstrExceptionClassName; }
            set { mstrExceptionClassName = value; }
        }

        public TransactionLogEntry(string siteNumber)
        {
            mstrSiteNumber = siteNumber;
        }

        public bool IsLongVixTransaction()
        {
            if (mintTimeOnVix >= maxOkTimeOnVix)
                return true;
            return false;
        }

        public bool IsError()
        {
            if (mstrErrorMessage.Length > 0)
                return true;
            if (mstrExceptionClassName.Length > 0)
                return true;
            if ((mstrResponseCode.Length > 0) && (mstrResponseCode != "200"))
                return true;
            // if the query takes too long, then its an error (might get false positives for images)
            if (IsLongVixTransaction())
                return true;
            return false;
        }

        private static char stringDelimiter = '\t';

        public static TransactionLogEntry parseTSVLine(string siteNumber, string headerLine, string line)
        {            
            string[] headerPieces = headerLine.Split(stringDelimiter);
            string[] entryPieces = line.Split(stringDelimiter);
            if (headerPieces.Length > entryPieces.Length)
                return null;
            TransactionLogEntry logEntry = new TransactionLogEntry(siteNumber);
            for (int i = 0; i < headerPieces.Length; i++)
            {
                SetLogEntryField(logEntry, headerPieces[i], extractQuotes( entryPieces[i]));
            }
            return logEntry;
        }

        private static void SetLogEntryField(TransactionLogEntry logEntry, string headerName, string value)
        {
            logEntry.mValues.Add(headerName, value);
            switch (headerName)
            {
                case "Date and Time":
                    logEntry.TransactionDate = parseDateTime( value);
                    break;
                case "Time on ViX (msec)":
                    logEntry.TimeOnVix = parseInt( value);
                    break;
                case "Patient ICN":
                    logEntry.PatientIcn = value;
                    break;
                case "Query Type":
                    logEntry.QueryType = value;
                    break;
                case "Items (Size) Returned":
                    logEntry.ItemsReturned = value;
                    break;
                case "Quality":
                    logEntry.Quality = value;
                    break;
                case "Originating IP Address":
                    logEntry.UserIPAddress = value;
                    break;
                case "User":
                    logEntry.User = value;
                    break;
                case "Transaction Number":
                    logEntry.TransactionId = value;
                    break;
                case "URN":
                    logEntry.URN = value;
                    break;
                case "Item in cache?":
                    logEntry.Cached = parseBool(value);
                    break;
                case "Error Message":
                    logEntry.ErrorMessage = value;
                    break;
                case "Bytes Transferred":
                case "Bytes Returned":
                    logEntry.BytesTransfered = parseInt( value);
                    break;
                case "Datasource Protocol":
                    logEntry.Datasource = value;
                    break;
                case "Response Code":
                    logEntry.ResponseCode = value;
                    break;
                case "Exception Class Name":
                    logEntry.ExceptionClassName = value;
                    break;
                case "Modality":
                    logEntry.Modality = value;
                    break;
                case "Command Class Name":
                    logEntry.CommandClassName = value;
                    break;
                case "Responding Site":
                    logEntry.RespondingSite = value;
                    break;
                case "Machine Name":
                    logEntry.mstrMachineName = value;
                    break;
                case "Requesting Site":
                    logEntry.mstrRequestingSite = value;
                    break;
                case "Purpose of Use":
                    logEntry.mstrPurposeOfUse = value;
                    break;
                case "Realm Site Number":
                    logEntry.mstrRealmSiteNumber = value;
                    break;
                case "Time To First Byte":
                    if (!string.IsNullOrEmpty(value))
                    {
                        logEntry.mintTimeToFirstByte = int.Parse(value);
                    }                    
                    break;
                case "Vix Software Version":
                    logEntry.mstrVixSoftwareVersion = value;
                    break;
                case "Data Source Items Received":
                    logEntry.mstrDataSourceItemsReceived = value;
                    break;
                case "Asynchronous?":
                    if (!string.IsNullOrEmpty(value))
                    {
                        logEntry.mbAsynchronous = bool.Parse(value);
                    }
                    break;
                case "Query Filter":
                    logEntry.mstrQueryFilter = value;
                    break;
                case "Facade Bytes Returned":
                    logEntry.mstrFacadeBytesReturned = value;
                    break;
                case "Facade Bytes Received":
                    logEntry.mstrFadeBytesReceived = value;
                    break;
                case "DataSource Bytes Returned":
                    logEntry.mstrDataSourceBytesReturned = value;
                    break;
                case "DataSource Bytes Received":
                    logEntry.mstrDataSourceBytesReceived = value;
                    break;
            }
        }

        private static DateTime parseDateTime(string value)
        {            
            return DateTime.Parse(value);
        }

        private static int parseInt(string value)
        {
            int result = -1;
            if ((value == null) || (value.Length <= 0))
                return result;
            if (int.TryParse(value, out result))
                return result;
            return -1;
        }

        private static bool parseBool(string value)
        {
            if ((value == null) || (value.Length <= 0))
                return false;
            bool result=false;
            if (bool.TryParse(value, out result))
                return result;
            return false;
        }

        private static string extractQuotes(string value)
        {
            if ((value == null) || (value.Length <= 0))
                return value;
            if (value.StartsWith("\""))
            {
                value = value.Substring(1);
            }
            if (value.EndsWith("\""))
            {
                value = value.Substring(0, value.Length - 1);
            }
            return value;
        }

    }

    public class TransactionLogEntrySorter : Comparer<TransactionLogEntry>
    {
        public override int Compare(TransactionLogEntry x, TransactionLogEntry y)
        {
            int val = x.TransactionDate.CompareTo(y.TransactionDate);
            if (val != 0)
                return val;
            return x.SiteNumber.CompareTo(y.SiteNumber);
        }
    }
}
