/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;

namespace VixLogErrorCollectorUtility
{
    public class TransactionLogXmlOutput
    {

        public static List<TransactionLogEntry> ReadFromXml(Stream input)
        {
            List<TransactionLogEntry> entries = new List<TransactionLogEntry>();

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(input);

            XmlNodeList entryNodes = xmlDoc.SelectNodes("/Entries/Entry");
            foreach (XmlNode entryNode in entryNodes)
            {
                string siteNumber = entryNode.Attributes["SiteNumber"].Value;
                TransactionLogEntry entry = new TransactionLogEntry(siteNumber);
                entries.Add(entry);
                string TransactionId = entryNode.Attributes["TransactionId"].Value;
                string TransactionDate = entryNode.Attributes["TransactionDate"].Value;
                string URN = entryNode.Attributes["URN"].Value;
                string User = entryNode.Attributes["User"].Value;
                string UserIPAddress = entryNode.Attributes["UserIPAddress"].Value;
                string TimeOnVix = entryNode.Attributes["TimeOnVix"].Value;
                string SiteNumber = entryNode.Attributes["SiteNumber"].Value;
                string ResponseCode = entryNode.Attributes["ResponseCode"].Value;
                string QueryType = entryNode.Attributes["QueryType"].Value;
                string Quality = entryNode.Attributes["Quality"].Value;
                string PatientICN = entryNode.Attributes["PatientICN"].Value;
                string ItemsReturned = entryNode.Attributes["ItemsReturned"].Value;
                string ExceptionClassName = entryNode.Attributes["ExceptionClassName"].Value;
                string DataSource = entryNode.Attributes["DataSource"].Value;
                string BytesTransfered = entryNode.Attributes["BytesTransfered"].Value;
                string Cached = entryNode.Attributes["Cached"].Value;
                string CommandClassName = entryNode.Attributes["CommandClassName"].Value;
                string errorMessage = entryNode.InnerText;
                entry.BytesTransfered = int.Parse( BytesTransfered);
                entry.Cached = bool.Parse( Cached);
                entry.CommandClassName = CommandClassName;
                entry.Datasource = DataSource;
                entry.ErrorMessage = errorMessage;
                entry.ExceptionClassName = ExceptionClassName;
                entry.ItemsReturned = ItemsReturned;
                //entry.Modality = moda
                entry.PatientIcn = PatientICN;
                entry.Quality = Quality;
                entry.QueryType = QueryType;
                entry.ResponseCode = ResponseCode;
                entry.TimeOnVix = int.Parse(TimeOnVix);
                entry.TransactionDate = DateTime.Parse(TransactionDate);
                entry.TransactionId = TransactionId;
                entry.URN = URN;
                entry.User = User;
                entry.UserIPAddress = UserIPAddress;

            }


            return entries;
        }

        public static void OutputToXml(List<TransactionLogEntry> entries, string outputDirectory)
        {
            entries.Sort(new TransactionLogEntrySorter());
            Dictionary<string, XmlDocument> xmlDocs = new Dictionary<string, XmlDocument>();

            foreach (TransactionLogEntry entry in entries)
            {
                if (entry.IsError())
                {
                    XmlDocument doc = getXmlDocument(xmlDocs, entry);
                    WriteXmlEntry(entry, doc);
                }
            }

            foreach (string key in xmlDocs.Keys)
            {
                SaveXmlDoc(xmlDocs[key], outputDirectory, key);
            }
        }

        private static void SaveXmlDoc(XmlDocument xmlDoc, string outputDir, string key)
        {
            string filename = outputDir + "\\" + key + ".xml";
            if (!File.Exists(filename))
            {
                xmlDoc.Save(filename);
            }
            // don't overwrite - kludgy since already did all the work
        }

        private static XmlDocument getXmlDocument(Dictionary<string, XmlDocument> xmlDocs, TransactionLogEntry entry)
        {
            string key = entry.TransactionDate.ToString("yyyy-MM-dd");
            if (xmlDocs.ContainsKey(key))
                return xmlDocs[key];
            XmlDocument newDoc = createXmlDoc(entry.TransactionDate);
            xmlDocs.Add(key, newDoc);
            return newDoc;
        }

        private static void WriteXmlEntry(TransactionLogEntry entry, XmlDocument xmlDoc)
        {
            XmlNode rootNode = xmlDoc.SelectSingleNode("Entries");
            XmlNode newNode = xmlDoc.CreateElement("Entry");
            rootNode.AppendChild(newNode);

            XmlAttribute attr = xmlDoc.CreateAttribute("TransactionId");
            attr.Value = entry.TransactionId;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("TransactionDate");
            attr.Value = entry.TransactionDate.ToString();
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("URN");
            attr.Value = entry.URN;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("User");
            attr.Value = entry.User;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("UserIPAddress");
            attr.Value = entry.UserIPAddress;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("TimeOnVix");
            attr.Value = entry.TimeOnVix.ToString();
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("SiteNumber");
            attr.Value = entry.SiteNumber;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("ResponseCode");
            attr.Value = entry.ResponseCode;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("QueryType");
            attr.Value = entry.QueryType;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("Quality");
            attr.Value = entry.Quality;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("PatientICN");
            attr.Value = entry.PatientIcn;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("ItemsReturned");
            attr.Value = entry.ItemsReturned;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("ExceptionClassName");
            attr.Value = entry.ExceptionClassName;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("DataSource");
            attr.Value = entry.Datasource;
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("BytesTransfered");            
            attr.Value = entry.BytesTransfered.ToString();
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("Cached");
            attr.Value = entry.Cached.ToString();
            newNode.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("CommandClassName");
            attr.Value = entry.CommandClassName;
            newNode.Attributes.Append(attr);

            XmlText text = xmlDoc.CreateTextNode(entry.ErrorMessage);
            newNode.AppendChild(text);
        }

        private static XmlDocument createXmlDoc(DateTime entryDate)
        {
            XmlDocument xmlDoc = new XmlDocument();
            XmlProcessingInstruction instruction =
                xmlDoc.CreateProcessingInstruction("xml-stylesheet", "type='text/xsl' href='display.xsl'");
            xmlDoc.AppendChild(instruction);            
            XmlNode rootNode = xmlDoc.CreateElement("Entries");
            XmlAttribute attr = xmlDoc.CreateAttribute("Date");
            attr.Value = entryDate.ToString("MMM dd, yyyy");
            rootNode.Attributes.Append(attr);
            xmlDoc.AppendChild(rootNode);
            return xmlDoc;
        }
    }
}
