/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using VixLogErrorCollectorUtility;
using System.IO;
using System.Reflection;
using System.Net;

namespace VixLogErrorViewer
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("The purpose of this application is to display the XML output generated frmo the VIX Log Error Console.\n" +
                "The web viewer doesn't always display properly, so this can be used for a cleaner view of the data.");
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private Stream getFileStream()
        {
            if (txtErrorLogXmlFile.Text.StartsWith("http"))
            {
                WebClient client = new WebClient();
                return client.OpenRead(txtErrorLogXmlFile.Text);
            }
            else
            {
                return new StreamReader(txtErrorLogXmlFile.Text).BaseStream;
            }
        }

        private List<TransactionLogEntry> entries = null;

        private void LoadEntries()
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                toolStripStatusLabel1.Text = "Loading entries";
                entries = TransactionLogXmlOutput.ReadFromXml(getFileStream());
                PopulateFilters();
                toolStripStatusLabel1.Text = "Loaded '" + entries.Count + "' entries.";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void PopulateFilters()
        {
            cboExceptionClassName.Items.Clear();
            if (entries == null)
            {
                return;
            }

            Dictionary<string, string> exceptionClassNames = new Dictionary<string, string>();
            Dictionary<string, string> commandClassNames = new Dictionary<string, string>();
            Dictionary<string, string> siteNumbers = new Dictionary<string, string>();
            Dictionary<string, string> responseCodes = new Dictionary<string, string>();
            Dictionary<string, string> datasources = new Dictionary<string, string>();

            foreach (TransactionLogEntry entry in entries)
            {
                string exceptionClassName = entry.ExceptionClassName;
                string commandClassName = entry.CommandClassName;
                string siteNumber = entry.SiteNumber;
                string responseCode = entry.ResponseCode;
                string datasource = entry.Datasource;
                if (!exceptionClassNames.ContainsKey(exceptionClassName))
                {
                    exceptionClassNames.Add(exceptionClassName, "");
                }
                if (!commandClassNames.ContainsKey(commandClassName))
                {
                    commandClassNames.Add(commandClassName, "");
                }
                if (!siteNumbers.ContainsKey(siteNumber))
                {
                    siteNumbers.Add(siteNumber, "");
                }
                if (!responseCodes.ContainsKey(responseCode))
                {
                    responseCodes.Add(responseCode, "");
                }
                if (!datasources.ContainsKey(datasource))
                {
                    datasources.Add(datasource, "");
                }
            }

            cboExceptionClassName.Items.Clear();
            cboCommandClassName.Items.Clear();
            cboDataSource.Items.Clear();
            cboResponseCodes.Items.Clear();
            cboSiteNumber.Items.Clear();

            cboExceptionClassName.Items.AddRange(DictionaryKeysToArray(exceptionClassNames));
            cboCommandClassName.Items.AddRange(DictionaryKeysToArray(commandClassNames));
            cboSiteNumber.Items.AddRange(DictionaryKeysToArray(siteNumbers));
            cboResponseCodes.Items.AddRange(DictionaryKeysToArray(responseCodes));
            cboDataSource.Items.AddRange(DictionaryKeysToArray(datasources));
        }

        private string[] DictionaryKeysToArray(Dictionary<string, string> items)
        {
            List<string> keys = new List<string>();
            foreach(string key in items.Keys)
            {
                keys.Add(key);
            }
            return keys.ToArray();
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            LoadEntries();
                   
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            BuildColumns();
            txtErrorLogXmlFile.Text = "http://zzzzzzimmixdb3/errors/2010-01-02.xml";
        }

        int errorMessageColumn = -1;

        private void BuildColumns()
        {
            Type t = typeof(TransactionLogEntry);
            PropertyInfo [] properties = t.GetProperties();
            foreach (PropertyInfo property in properties)
            {
                ColumnHeader header = new ColumnHeader();
                header.Text = property.Name;
                if (property.Name == "ErrorMessage")
                    errorMessageColumn = lstErrors.Columns.Count;
                lstErrors.Columns.Add(header);
            }
        }

        private void lstErrors_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstErrors.SelectedItems.Count > 0)
            {
                TransactionLogEntry entry = (TransactionLogEntry)lstErrors.SelectedItems[0].Tag;
                txtErrorMessage.Text = entry.ErrorMessage;
            }
        }

        private void btnDisplay_Click(object sender, EventArgs e)
        {
            if (entries == null)
            {
                MessageBox.Show("Transaction Log entries must be loaded before filtering");
                return;
            }

            DisplayFilteredResults();
        }

        private void DisplayFilteredResults()
        {
            lstErrors.Items.Clear();
            List<TransactionLogEntry> filteredEntries = GetFilteredResults();

            Type t = typeof(TransactionLogEntry);
            PropertyInfo[] properties = t.GetProperties();
            foreach (TransactionLogEntry entry in filteredEntries)
            {
                ListViewItem item = new ListViewItem();
                lstErrors.Items.Add(item);
                item.Tag = entry;
                int idx = 0;
                foreach (PropertyInfo property in properties)
                {
                    object value = property.GetValue(entry, null);
                    string strValue = "";
                    if (value != null)
                    {
                        strValue = value.ToString();
                    }
                    if (item.SubItems.Count <= idx)
                        item.SubItems.Add("");
                    //item.SubItems.Add(strValue);
                    item.SubItems[idx].Text = strValue;
                    idx++;
                }
            }
            for (int i = 0; i < lstErrors.Columns.Count; i++)
            {
                if (i != errorMessageColumn)
                {
                    lstErrors.Columns[i].AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
                }
            }
            toolStripStatusLabel1.Text = "Displayed '" + filteredEntries.Count + "' filtered results";
        }

        private List<TransactionLogEntry> GetFilteredResults()
        {
            List<TransactionLogEntry> filteredEntries = new List<TransactionLogEntry>();
            foreach (TransactionLogEntry entry in entries)
            {
                if (includeEntryInFilter(entry))
                {
                    filteredEntries.Add(entry);
                }
            }
            return filteredEntries;
        }

        private bool includeEntryInFilter(TransactionLogEntry entry)
        {
            if (cboCommandClassName.Text.Length > 0)
            {
                if (!entry.CommandClassName.Contains(cboCommandClassName.Text))
                {
                    return false;
                }
            }
            if (cboExceptionClassName.Text.Length > 0)
            {
                if (!entry.ExceptionClassName.Contains(cboExceptionClassName.Text))
                {
                    return false;
                }
            }
            if (cboSiteNumber.Text.Length > 0)
            {
                if (!entry.SiteNumber.Contains(cboSiteNumber.Text))
                {
                    return false;
                }
            }
            if (cboResponseCodes.Text.Length > 0)
            {
                if (!entry.ResponseCode.Contains(cboResponseCodes.Text))
                {
                    return false;
                }
            }
            if (cboDataSource.Text.Length > 0)
            {
                if (!entry.Datasource.Contains(cboDataSource.Text))
                {
                    return false;
                }
            }

            return true;
        }

        private void copyTransactionIdToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (lstErrors.SelectedItems.Count > 0)
            {
                TransactionLogEntry entry = (TransactionLogEntry)(lstErrors.SelectedItems[0].Tag);
                Clipboard.SetText(entry.TransactionId);
            }
        }
    }
}
