/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace VixMonitorBusiness.Vix.Monitor.Business
{
    public class VixMonitorLogger
    {
        private string mstrFilename;
        private TextWriter writer = null;

        private static readonly int maxLogFileSize = 1024 * 1024 * 5; // 5 MB

        public VixMonitorLogger(string filename)
        {
            mstrFilename = filename;

            if (File.Exists(mstrFilename))
            {
                FileInfo fi = new FileInfo(mstrFilename);
                if (fi.Length > maxLogFileSize)
                {
                    string oldFilename = mstrFilename + ".old";
                    if (File.Exists(oldFilename))
                    {
                        File.Delete(oldFilename);
                    }
                    fi.MoveTo(oldFilename);
                }
            }

            writer = new StreamWriter(filename, true);
        }

        public void CloseLog()
        {
            if (writer != null)
            {
                writer.Flush();
                writer.Close();
            }
        }


        public void LogMsg(string msg, bool isError)
        {
            string error = "INFO";
            if (isError)
            {
                error = "ERROR";
            }

            writer.WriteLine(DateTime.Now.ToString() + " [" + error + "] " + msg);
            writer.Flush();
        }

        public void LogMsg(string msg)
        {
            LogMsg(msg, false);
        }
    }
}
