/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

namespace VixMonitorBusiness.Vix.Monitor.Business
{
    public class VixServer
    {
        private readonly string mstrVixHost;
        private readonly int mintVixPort;
        private readonly string mstrVistaHost;
        private readonly int mintVistaPort;
        private readonly string mstrSiteNumber;
        private readonly string mstrSiteName;
        private readonly string mstrSiteAbbreviation;
        private readonly List<VixTestResultGroup> serverTests = new List<VixTestResultGroup>();

        public string VistaHost
        {
            get { return mstrVistaHost; }
        } 

        public int VistaPort
        {
            get { return mintVistaPort; }
        } 

        public List<VixTestResultGroup> ServerTests
        {
            get { return serverTests; }
        }

        public string VixHost
        {
            get { return mstrVixHost; }
        }        

        public int VixPort
        {
            get { return mintVixPort; }
        }        

        public string SiteNumber
        {
            get { return mstrSiteNumber; }
        }        

        public string SiteName
        {
            get { return mstrSiteName; }
        }        

        public string SiteAbbreviation
        {
            get { return mstrSiteAbbreviation; }
        }

        public VixServer(string siteName, string siteNumber, string siteAbbreviation, string vistaHost, int vistaPort, string vixHost, int vixPort)
        {
            mstrSiteAbbreviation = siteAbbreviation;
            mstrSiteName = siteName;
            mstrSiteNumber = siteNumber;
            mstrVixHost = vixHost;
            mintVixPort = vixPort;
            mintVistaPort = vistaPort;
            mstrVistaHost = vistaHost;
        }

        public bool HasVix()
        {
            if ((mstrVixHost.Length > 0) && (mintVixPort > 0))
            {
                return true;
            }
            return false;
        }

        public bool IsDOD()
        {
            if (SiteNumber == "200")
            {
                return true;
            }
            return false;
        }

        public void AddTestResults(VixTestResultGroup testResult)
        {
            this.serverTests.Add(testResult);
        }

        public void AddTestResults(List<VixTestResultGroup> testResults)
        {
            this.serverTests.AddRange(testResults);
        }

        public VixTestResultGroup GetMostRecentTest()
        {
            if (this.serverTests.Count <= 0)
            {
                return null;
            }
            this.serverTests.Sort(new VixTestResultGroupSorter());


            return this.serverTests[this.serverTests.Count - 1];
        }

        public override string ToString()
        {
            return this.mstrSiteName + " [" + mstrSiteNumber + "]";
        }
        
    }

    public class VixServerSorter : Comparer<VixServer>
    {
        public override int Compare(VixServer x, VixServer y)
        {
            return x.SiteName.CompareTo(y.SiteName);
        }
    }
}
