/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixHealthMonitorCommon;

namespace VixMonitorBusiness.Vix.Monitor.Business
{
    public class VixTestResultGroup
    {
        private readonly DateTime mdtTestTime;
        private List<VixTestResult> tests = new List<VixTestResult>();
        private VixServerHealth vixHealth = null;

        public VixServerHealth VixHealth
        {
            get { return vixHealth; }
            set { vixHealth = value; }
        }

        public List<VixTestResult> VixTests
        {
            get { return tests; }
        }          

        public DateTime TestTime
        {
            get { return mdtTestTime; }
        }

        public VixTestResultGroup(DateTime testTime)
        {
            mdtTestTime = testTime;
        }

        public bool TestsPassed
        {
            get
            {
                for (int i = 0; i < tests.Count; i++)
                {
                    if (!tests[i].TestResult)
                        return false;
                }
                return true;
            }
        }

        public bool RepeatTests
        {
            get
            {
                for (int i = 0; i < tests.Count; i++)
                {
                    if ((!tests[i].TestResult) && (tests[i].RepeatFailedTest))
                        return true;
                }
                return false;
            }
        }
    }

    public class VixTestResultGroupSorter : IComparer<VixTestResultGroup>
    {
        public int Compare(VixTestResultGroup x, VixTestResultGroup y)
        {
            return x.TestTime.CompareTo(y.TestTime);
        }
    }

    public class VixTestResult
    {
        protected readonly VixTestType testType;
        protected bool mbTestResult;
        protected string mstrTestResultMessage;
        protected readonly bool mbRepeatFailedTest;

        public bool RepeatFailedTest
        {
            get { return mbRepeatFailedTest; }
        } 

        public VixTestType TestType
        {
            get { return testType; }
        }

        public string TestTypeString
        {
            get
            {
                switch (testType)
                {
                    case VixTestType.PING:
                        return "Ping";
                    case VixTestType.FAILED_QUERY:
                        return "Failed Query";
                    case VixTestType.TOMCAT:
                        return "Web Server Online";
                    case VixTestType.IDS_QUERY:
                        return "IDS Query";
                    case VixTestType.VIX_HEALTH:
                        return "VIX Server Health";
                    case VixTestType.VIX_DETAILS:
                        return "VIX Details";
                }
                return "Unknown";
            }
        }

        public bool TestResult
        {
            get { return mbTestResult; }
            set { mbTestResult = value; }
        }

        public string TestResultMessage
        {
            get { return mstrTestResultMessage; }
            set { mstrTestResultMessage = value; }
        } 

        public VixTestResult(VixTestType testType, bool testResult, string testResultMsg, bool repeatFailedTest)
        {
            this.testType = testType;
            mstrTestResultMessage = testResultMsg;
            mbTestResult = testResult;
            mbRepeatFailedTest = repeatFailedTest;
        }

        public VixTestResult(VixTestType testType, bool testResult, bool repeatFailedTest) : this(testType, testResult, "", repeatFailedTest)
        {
            
        }

        public VixTestResult(VixTestType testType, bool repeatFailedTest)
            : this(testType, false, "", repeatFailedTest)
        {

        }
    }

    public class VixHealthTestResult : VixTestResult
    {
        private VixServerHealth vixHealth = null;

        public VixServerHealth VixHealth
        {
            get { return vixHealth; }
            set { vixHealth = value; }
        }

        public VixHealthTestResult(VixTestType testType, bool testResult, string testResultMsg, bool repeatFailedTest, VixServerHealth vixHealth)
            : base(testType, testResult, testResultMsg, repeatFailedTest)
        {
            this.vixHealth = vixHealth;
        }

        public VixHealthTestResult(VixTestType testType, bool repeatFailedTest)
            : this(testType, false, "", repeatFailedTest, null)
        {

        }
    }

    public enum VixTestType
    {
        PING,TOMCAT,FAILED_QUERY,IDS_QUERY,VIX_HEALTH, VIX_DETAILS
    }
}
