/**
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created:
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */
using System;
using System.Collections.Generic;
using System.Text;
using VixMonitorBusiness.Vix.Monitor.Business;
using System.Net.Mail;
using System.Configuration;

using System.Windows.Forms;

namespace VixMonitorConsole
{
    class Program
    {
        private static VixMonitorLogger logger = null;

        static void Main(string[] args)
        {
            try
            {
                string appPath = getApplicationDir(Application.ExecutablePath);
                logger = new VixMonitorLogger(appPath + "\\log.txt");
                VixTesterHelper.logger = logger;

                List<VixServer> vixServers = VixMonitorConsoleHelper.LoadSitesFromSiteService();
                LogMsg("Found [" + vixServers.Count + "] ViX sites in Site Service");
                //VixMonitorConsoleHelper.MergeVixServersFromExportFile(vixServers);

                bool sendMessage = false;
                bool sendFailedMessage = bool.Parse(ConfigurationManager.AppSettings["EmailOnError"]);
                bool sendSuccessMessage = bool.Parse(ConfigurationManager.AppSettings["EmailOnSuccess"]);
                bool sendChangeMessage = bool.Parse(ConfigurationManager.AppSettings["EmailOnChange"]);
                bool onlyIncludeFailed = bool.Parse(ConfigurationManager.AppSettings["OnlyIncludeFailed"]);
                bool includeVixDetails = false;
                if (ConfigurationManager.AppSettings["IncludeVixDetails"] != null)
                {
                    includeVixDetails = bool.Parse(ConfigurationManager.AppSettings["IncludeVixDetails"]);
                }

                string excludeSites = ConfigurationManager.AppSettings["ExcludeSites"];
                string skipPingSites = ConfigurationManager.AppSettings["SkipPingTestSiteNumbers"];
                int vixHealthTimeout = 90;
                if (ConfigurationManager.AppSettings["VixHealthTimeout"] != null)
                {
                    vixHealthTimeout = int.Parse(ConfigurationManager.AppSettings["VixHealthTimeout"]);
                }
                bool testVixHealth = true;
                if (ConfigurationManager.AppSettings["TestVixHealth"] != null)
                {
                    testVixHealth = bool.Parse(ConfigurationManager.AppSettings["TestVixHealth"]);
                }
                LogMsg("Excluding sites list [" + excludeSites + "]");
                LogMsg("Skipping ping at sites [" + skipPingSites + "]");
                if (!testVixHealth)
                    includeVixDetails = false;
                LogMsg("Testing VIX Health [" + testVixHealth + "], including VIX Details [" + includeVixDetails + "]");

                string[] excludedSites = null;
                if ((excludeSites != null) && (excludeSites.Length > 0))
                {
                    excludedSites = excludeSites.Split(',');
                }
                string[] skippedPingSites = null;
                if ((skipPingSites != null) && (skipPingSites.Length > 0))
                {
                    skippedPingSites = skipPingSites.Split(',');
                }

                string sysLogServer = ConfigurationManager.AppSettings["SysLogServer"];
                SysLogClient.Client sysLogClient = new SysLogClient.Client(sysLogServer);

                bool testFailed = false;
                List<VixServer> testedVixServers = new List<VixServer>();
                for (int i = 0; i < vixServers.Count; i++)
                {
                    if (isSiteIncluded(vixServers[i].SiteNumber, excludedSites))
                    {
                        testedVixServers.Add(vixServers[i]);
                        LogMsg("Testing site [" + vixServers[i].SiteName + "]");
                        if (vixServers[i].SiteNumber != "")
                        {
                            
                            VixTestResultGroup result = VixTesterHelper.TestVix(vixServers[i], 
                                !isSiteSkipPing(vixServers[i].SiteNumber, skippedPingSites), 
                                vixHealthTimeout, testVixHealth);
                            vixServers[i].ServerTests.Add(result);
                            if (!result.TestsPassed)
                            {
                                LogMsg("Testing failed for site [" + vixServers[i].SiteName + "]");
                                testFailed = true;
                            }
                            SendSysLogSiteResult(result, sysLogClient, vixServers[i]);
                        }
                        else
                        {
                            VixTestResultGroup result = new VixTestResultGroup(DateTime.Now);
                            result.VixTests.Add(new VixTestResult(VixTestType.PING, true));
                            vixServers[i].ServerTests.Add(result);
                        }
                    }
                    else
                    {
                        LogMsg("Excluding site [" + vixServers[i].SiteNumber + "] from testing");
                    }

                }

                StringBuilder sb = new StringBuilder();
                List<VixServer> serversToSend = null;
                ReportType reportType = ReportType.PASSED;
                if (onlyIncludeFailed)
                {
                    LogMsg("Only including failed sites");
                    serversToSend = VixMonitorConsoleHelper.RemoveSuccessfulVixServers(testedVixServers);
                }
                else if (sendChangeMessage)
                {
                    LogMsg("Only sending for changed sites");
                    serversToSend = VixMonitorConsoleHelper.FindStatusChangedVix(testedVixServers);
                }
                else
                {
                    serversToSend = vixServers;
                }

                VixMonitorConsoleHelper.UpdateVixServersFromExportFile(testedVixServers);
                VixMonitorConsoleHelper.PruneXmlExportIfNeeded();
                LogMsg("Pruned server list to send to [" + serversToSend.Count + "] sites");
                if (serversToSend.Count > 0)
                {
                    StringBuilder plainTextOutput = new StringBuilder();
                    TestResultsOutputBuilder.SaveResultsAsHtml(sb, serversToSend, sendChangeMessage, includeVixDetails);
                    TestResultsOutputBuilder.SaveResultsAsPlainText(plainTextOutput, serversToSend, sendChangeMessage);

                    if (sendChangeMessage)
                    {
                        sendMessage = true;
                        reportType = ReportType.STATUS_CHANGED;
                    }
                    else
                    {
                        if (testFailed && sendFailedMessage)
                        {
                            sendMessage = true;
                            reportType = ReportType.FAILED;
                        }
                        if (!testFailed && sendSuccessMessage)
                        {
                            sendMessage = true;
                            reportType = ReportType.PASSED;
                        }
                    }
                    if (sendMessage)
                    {
                        SendEmail(reportType, sb.ToString(), plainTextOutput.ToString());
                        LogMsg("Done checking servers, email should have been sent");
                    }
                    else
                    {
                        LogMsg("Done checking servers, not sending email");
                    }
                }
                else
                {
                    LogMsg("No Vix Servers selected to send, done");
                }

            }
            catch (Exception ex)
            {
                LogMsg(ex.Message + "\n\n" + ex.StackTrace, true);
            }
            finally
            {
                if (logger != null)
                {
                    logger.CloseLog();
                }
            }
        }

        private static void SendSysLogSiteResult(VixTestResultGroup testResultGroup,
            SysLogClient.Client sysLogClient, VixServer vixServer)
        {
            try
            {
                int facility = (int)SysLogClient.SysLogFacility.User;
                int level = (int)SysLogClient.SysLogLevel.Information;
                if (!testResultGroup.TestsPassed)
                {
                    level = (int)SysLogClient.SysLogLevel.Error;
                }
                string msg = CreateSysLogXmlMessage(testResultGroup, vixServer);
                sysLogClient.Send(new SysLogClient.SysLogMessage(facility, level, msg.ToString()));
            }
            catch (Exception ex)
            {
                LogMsg(ex.Message, true);
            }
        }

        private static string CreateSysLogXmlMessage(VixTestResultGroup testResultGroup, VixServer vixServer)
        {
            StringBuilder xml = new StringBuilder();
            xml.Append("<msg type=\"VIXMonitorResult\">");
            xml.Append("<msgDetails>");
            xml.Append("<site name=\"" + vixServer.SiteName + "\" number=\"" + vixServer.SiteNumber + "\" />");
            xml.Append("<tests passed=\"" + testResultGroup.TestsPassed.ToString() + "\">");
            foreach (VixTestResult testResult in testResultGroup.VixTests)
            {
                xml.Append("<test name=\"" + testResult.TestTypeString + "\" result=\"" + testResult.TestResult + "\" msg=\"" + testResult.TestResultMessage + "\" />");
            }
            xml.Append("</tests>");
            xml.Append("</msgDetails>");
            xml.Append("</msg>");
            return xml.ToString();
        }

        private static bool isSiteSkipPing(string siteNumber, string[] skippedPingSites)
        {
            if (skippedPingSites == null)
                return false;
            foreach (string sNumber in skippedPingSites)
            {
                if (sNumber == siteNumber)
                {
                    return true;
                }
            }
            return false;
        }

        private static bool isSiteIncluded(string siteNumber, string[] excludedSites)
        {
            if (excludedSites == null)
                return true;
            foreach (string sNumber in excludedSites)
            {
                if (sNumber == siteNumber)
                {
                    return false;
                }
            }
            return true;
        }

        private static void SendEmail(ReportType reportType, string htmlMessage)
        {
            VixMonitorConsoleHelper helper = new VixMonitorConsoleHelper(logger);
            helper.SendEmail(reportType, htmlMessage, "");
        }

        private static void SendEmail(ReportType reportType, string htmlMessage, string plainTextMsg)
        {
            VixMonitorConsoleHelper helper = new VixMonitorConsoleHelper(logger);
            helper.SendEmail(reportType, htmlMessage, plainTextMsg);
        }


        private static string getApplicationDir(string executablePath)
        {
            int loc = executablePath.LastIndexOf("\\");
            return executablePath.Substring(0, loc);
        }

        private static void LogMsg(string msg, bool isError)
        {
            logger.LogMsg(msg, isError);
            Console.WriteLine(msg);
        }

        private static void LogMsg(string msg)
        {
            LogMsg(msg, false);
        }
    }
}
